/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "translation.h"
#include "misc.h"
#include "xor.h"
#include "dia_comm.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_attribute.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "obj_rectangle.h"
#include "obj_polygon.h"
#include "obj_spline.h"
#include "obj_elarc.h"
#include "obj_insert.h"
#include "dim_horiz.h"
#include "dim_vert.h"
#include "dim_aligned.h"
#include "dim_radius.h"
#include "dim_diameter.h"
#include "dim_center.h"
#include "dim_continue.h"
#include "dim_baseline.h"
#include "dim_leader.h"
#include "dim_angular.h"
#include "tool_zoom.h"
#include "tool_copy.h"
#include "tool_erase.h"
#include "tool_offset.h"
#include "tool_array.h"
#include "tool_move.h"
#include "tool_mirror.h"
#include "tool_rotate.h"
#include "tool_trim.h"
#include "tool_idpoint.h"
#include "tool_dist.h"
#include "tool_fillet.h"
#include "tool_break.h"
#include "tool_chamfer.h"
#include "tool_stretch.h"
#include "tool_scale.h"
#include "tool_block.h"
#include "edit_text.h"
#include "select.h"
#include "calc.h"
#include "osnap.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
static void dragmousebutton(DOCUMENT *doc, gint button, gdouble mx, gdouble my)
{
  if(button==1)
    print_coord(mx,my);
  
  /* Is object snap active? */
  if(gl_osnap!=OSNAP_NONE)
    {
      if(!osnap_button(doc,button, &mx,&my))
	return;
    }
  
  switch(gl_tool)
    {
    case TOOL_MOVE:
      if(gl_stage==1)
	tool_button_move_drag(work_document, doc, button, mx, my);	  
      break;
    case TOOL_COPY:
      if(gl_stage==1 || gl_stage==3)
	tool_button_copy_drag(work_document, doc, button, mx, my);	  
      break;
    }
}

void mousebutton(DOCUMENT *doc, gint button, gdouble mx, gdouble my)
{
  /* If button 2 select panning */
  if(button==2)
    {
      if(!(gl_tool==TOOL_ZOOM && gl_stage==3))
	{
	  tool_set_tool_zoom(NULL, GINT_TO_POINTER(2));
	  print_coord(mx,my);
	  tool_button_zoom(doc,1, mx, my);
	  set_cursor(doc);
	  return;
	}
      else
	button=1;
    }

  /* No tool selected */
  if(gl_tool==TOOL_IDLE)
    return;

  if(work_document==NULL)
    work_document=doc;

  if(( (gl_tool==TOOL_MOVE && gl_stage==1) || 
       (gl_tool==TOOL_COPY && (gl_stage==1 || gl_stage==3))) &&
     doc!=work_document)
    {
      dragmousebutton(doc, button, mx, my);  
      return;
    }
  else if(doc!=work_document)   /* Switch drawing? */
    {
      deselect_all_objects(work_document);
      print(_("*Switching drawing*"));
      gl_stage=0;
      work_document=doc;
      set_prompt();
    }

  if(button==1)
    print_coord(mx,my);

  /* Is object snap active? */
  if(gl_osnap!=OSNAP_NONE)
    {
      if(!osnap_button(doc,button, &mx,&my))
	return;
    }

  switch(gl_tool)
    {
    case TOOL_LINE:
      obj_button_line(doc, button, mx,my);
      break;
    case TOOL_SPLINE:
      obj_button_spline(doc, button, mx,my);
      break;
    case TOOL_CIRCLE:
      obj_button_circle(doc, button, mx,my);
      break;
    case TOOL_ARC:
      obj_button_arc(doc, button, mx,my);
      break;
    case TOOL_POLYLINE:
      obj_button_polyline(doc, button, mx,my);
      break;
    case TOOL_POINT:
      obj_button_point(doc, button, mx,my);
      break;
    case TOOL_RLINE:
      obj_button_rline(doc, button, mx,my);
      break;
    case TOOL_CLINE:
      obj_button_cline(doc, button, mx,my);
      break;
    case TOOL_ELLIPSE:
      obj_button_ellipse(doc, button, mx,my);
      break;
    case TOOL_ELARC:
      obj_button_elarc(doc, button, mx,my);
      break;
    case TOOL_TEXT:
      obj_button_text(doc, button, mx,my);
      break;
    case TOOL_ATTRIBUTE:
      obj_button_attribute(doc, button, mx,my);
      break;
    case TOOL_RECTANGLE:
      obj_button_rectangle(doc, button, mx,my);
      break;
    case TOOL_POLYGON:
      obj_button_polygon(doc, button, mx,my);
      break;
    case TOOL_INSERT:
      obj_button_insert(doc, button, mx,my);
      break;
    case TOOL_ZOOM:
      tool_button_zoom(doc, button, mx,my);
      break;
    case TOOL_COPY:
      tool_button_copy(doc, button, mx,my);
      break;
    case TOOL_OFFSET:
      tool_button_offset(doc, button, mx,my);
      break;
    case TOOL_ARRAY:
      tool_button_array(doc, button, mx,my);
      break;
    case TOOL_MOVE:
      tool_button_move(doc, button, mx,my);
      break;
    case TOOL_MIRROR:
      tool_button_mirror(doc, button, mx,my);
      break;
    case TOOL_ROTATE:
      tool_button_rotate(doc, button, mx,my);
      break;
    case TOOL_TRIM:
      tool_button_trim(doc, button, mx,my);
      break;
    case TOOL_IDPOINT:
      tool_button_idpoint(doc, button, mx,my);
      break;
    case TOOL_DIST:
      tool_button_dist(doc, button, mx,my);
      break;
    case TOOL_FILLET:
      tool_button_fillet(doc, button, mx,my);
      break;
    case TOOL_BREAK:
      tool_button_break(doc, button, mx,my);
      break;
    case TOOL_CHAMFER:
      tool_button_chamfer(doc, button, mx,my);
      break;
    case TOOL_STRETCH:
      tool_button_stretch(doc, button, mx,my);
      break;
    case TOOL_SCALE:
      tool_button_scale(doc, button, mx,my);
      break;
    case TOOL_BLOCK:
      tool_button_block(doc, button, mx,my);
      break;
    case TOOL_HORIZ:
      dim_button_horiz(doc, button, mx,my);
      break;
    case TOOL_VERT:
      dim_button_vert(doc, button, mx,my);
      break;
    case TOOL_ALIGNED:
      dim_button_aligned(doc, button, mx,my);
      break;
    case TOOL_RADIUS:
      dim_button_radius(doc, button, mx,my);
      break;
    case TOOL_DIAMETER:
      dim_button_diameter(doc, button, mx,my);
      break;
    case TOOL_CENTER:
      dim_button_center(doc, button, mx,my);
      break;
    case TOOL_CONTINUE:
      dim_button_continue(doc, button, mx,my);
      break;
    case TOOL_BASELINE:
      dim_button_baseline(doc, button, mx,my);
      break;
    case TOOL_LEADER:
      dim_button_leader(doc, button, mx,my);
      break;
    case TOOL_ANGULAR:
      dim_button_angular(doc, button, mx,my);
      break;
    case TOOL_ETEXT:
      tool_button_etext(doc, button, mx,my);
      break;
    case TOOL_SELECT:
      spec_button_select(doc, button, mx,my);
      break;
    }

  set_cursor(doc);
}







