/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "misc.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
static void calc_angle_for_arc_3p(gdouble *sa, gdouble ma, gdouble *ea)
{
  gdouble ta;

  if(*ea>*sa)
    {
      if(ma<*sa)
	{
	  ta=*ea;
	  *ea=*sa;
	  *sa=ta;
	}
      else if(ma>*ea)
	{
	  ta=*ea;
	  *ea=*sa;
	  *sa=ta;
	}
    }
  else if(*ea<*sa)
      {
	if(ma<*sa && ma>*ea)
	  {
	    ta=*ea;
	    *ea=*sa;
	    *sa=ta;
	  }
      }
}

gint calc_arc_3p(gdouble startpoint_x, gdouble startpoint_y, gdouble midpoint_x, gdouble midpoint_y,
		 gdouble endpoint_x, gdouble endpoint_y, OBJECT_ARC *output)
{
  gdouble A,B,C,D,E,F,G;
  gdouble r,cen_x,cen_y;
  gdouble sangle, eangle;

  A=midpoint_x-startpoint_x;
  B=midpoint_y-startpoint_y;
  C=endpoint_x-startpoint_x;
  D=endpoint_y-startpoint_y;
  
  E=A*(startpoint_x+midpoint_x)+B*(startpoint_y+midpoint_y);
  F=C*(startpoint_x+endpoint_x)+D*(startpoint_y+endpoint_y);
  
  G=2.0*(A*(endpoint_y-midpoint_y)-B*(endpoint_x-midpoint_x));

  if(G!=0){
    cen_x=(D*E-B*F)/G;
    cen_y=(A*F-C*E)/G;

    r=dist(startpoint_x, startpoint_y, cen_x,cen_y);

    sangle=get_angle(cen_x,cen_y,startpoint_x,startpoint_y);
    eangle=get_angle(cen_x,cen_y,endpoint_x,endpoint_y);

    calc_angle_for_arc_3p(&sangle, get_angle(cen_x,cen_y,midpoint_x,midpoint_y), &eangle);

    output->centrum.x=cen_x;
    output->centrum.y=cen_y;
    output->radius=r;
    output->start_angle=sangle;
    output->end_angle=eangle;

    return TRUE;
  }

  return FALSE;
}

gint calc_arc_CSE(gdouble cen_x, gdouble cen_y, gdouble startpoint_x, gdouble startpoint_y,
		  gdouble endpoint_x, gdouble endpoint_y, OBJECT_ARC *output)
{
  gdouble sangle,eangle,r;
  
  r=dist(startpoint_x,startpoint_y,cen_x,cen_y);
  
  sangle=get_angle(cen_x,cen_y,startpoint_x,startpoint_y);
  eangle=get_angle(cen_x,cen_y,endpoint_x,endpoint_y);

  output->centrum.x=cen_x;
  output->centrum.y=cen_y;
  output->radius=r;
  output->start_angle=sangle;
  output->end_angle=eangle;

  return TRUE;
}

gint calc_arc_CSA(gdouble cen_x, gdouble cen_y, gdouble startpoint_x, gdouble startpoint_y,
		  gdouble angle, OBJECT_ARC *output)
{
  gdouble sangle,r;
  
  r=dist(startpoint_x,startpoint_y,cen_x,cen_y);
  
  sangle=get_angle(cen_x,cen_y,startpoint_x,startpoint_y);

  output->centrum.x=cen_x;
  output->centrum.y=cen_y;
  output->radius=r;
  output->start_angle=sangle;
  output->end_angle=sangle+angle;

  return TRUE;
}

gint calc_arc_CSL(gdouble cen_x, gdouble cen_y, gdouble startpoint_x, gdouble startpoint_y,
		  gdouble value, OBJECT_ARC *output)
{
  gdouble sangle,r;
  gdouble eangle;

  r=dist(startpoint_x,startpoint_y,cen_x,cen_y);
  
  if(value>2*r)
    value=2*r;
  if(value<-2*r)
    value=-2*r;

  sangle=get_angle(cen_x,cen_y,startpoint_x,startpoint_y);

  eangle=acos( (2*r*r-value*value)/(2*r*r) )*180.0/M_PI;

  output->centrum.x=cen_x;
  output->centrum.y=cen_y;
  output->radius=r;
  if(value>=0.0)
    {
      output->start_angle=sangle;
      output->end_angle=sangle+eangle;
    }
  else
    {
      output->start_angle=sangle;
      output->end_angle=sangle-eangle;
    }
  return TRUE;
}

gint calc_arc_SEA(gdouble startpoint_x, gdouble startpoint_y, gdouble endpoint_x, gdouble endpoint_y, 
		  gdouble angle, OBJECT_ARC *output)
{
  gdouble r, sangle, eangle;
  gdouble d,a;
  gdouble p2_x, p2_y;
  gdouble p3_x, p3_y, h;

  d=dist(startpoint_x,startpoint_y,endpoint_x,endpoint_y);

  r=sqrt( (d*d)/ (2-2*cos(angle*M_PI/180)));

  if(d>2*r)
    r=d/2;

  a=d/2;

  p2_x=startpoint_x+a*(endpoint_x-startpoint_x)/d;
  p2_y=startpoint_y+a*(endpoint_y-startpoint_y)/d;

  h=sqrt(r*r-a*a);

  if(angle>180 || (angle>-180.0 && angle<0) )
    {
      p3_x=p2_x+h*(endpoint_y-startpoint_y)/d;
      p3_y=p2_y-h*(endpoint_x-startpoint_x)/d;
    }
  else
    {
      p3_x=p2_x-h*(endpoint_y-startpoint_y)/d;
      p3_y=p2_y+h*(endpoint_x-startpoint_x)/d;
    }

  eangle=get_angle(p3_x,p3_y,endpoint_x,endpoint_y);      
  sangle=get_angle(p3_x,p3_y,startpoint_x,startpoint_y);

  output->centrum.x=p3_x;
  output->centrum.y=p3_y;
  output->radius=r;

  if(angle<0)
    {
      output->start_angle=eangle;
      output->end_angle=sangle;
    }
  else
    {
      output->start_angle=sangle;
      output->end_angle=eangle;
    }
  return TRUE;
}

gint calc_arc_SER(gdouble startpoint_x, gdouble startpoint_y, gdouble endpoint_x, gdouble endpoint_y, 
		  gint side, gdouble radius, OBJECT_ARC *output)
{
  gdouble sangle,eangle;
  gdouble d, a;
  gdouble p2_x, p2_y;
  gdouble p3_x, p3_y, h;

  d=dist(startpoint_x,startpoint_y,endpoint_x,endpoint_y);

  if(d>2*radius)
    radius=d/2;

  a=d/2;

  p2_x=startpoint_x+a*(endpoint_x-startpoint_x)/d;
  p2_y=startpoint_y+a*(endpoint_y-startpoint_y)/d;

  h=sqrt(radius*radius-a*a);

  if(side)
    {
     p3_x=p2_x-h*(endpoint_y-startpoint_y)/d;
     p3_y=p2_y+h*(endpoint_x-startpoint_x)/d;

     eangle=get_angle(p3_x,p3_y,startpoint_x,startpoint_y);
     sangle=get_angle(p3_x,p3_y,endpoint_x,endpoint_y);  
    }
  else
    {
      p3_x=p2_x+h*(endpoint_y-startpoint_y)/d;
      p3_y=p2_y-h*(endpoint_x-startpoint_x)/d;     

      sangle=get_angle(p3_x,p3_y,startpoint_x,startpoint_y);
      eangle=get_angle(p3_x,p3_y,endpoint_x,endpoint_y);  

    }

  output->centrum.x=p3_x;
  output->centrum.y=p3_y;
  output->radius=radius;
  output->start_angle=sangle;
  output->end_angle=eangle;

  return TRUE;
}

gint calc_arc_SED(gdouble sx, gdouble sy, gdouble ex, gdouble ey,
		  gdouble ta, gint side, OBJECT_ARC *output)
{
  gdouble r;
  gdouble tx,ty;
  gdouble alfa;
  gdouble cenx, ceny;
  gdouble sangle,eangle;

  if(side)
    {
      tx=100*cos(ta+M_PI/2.0)+sx;
      ty=-100*sin(ta+M_PI/2.0)+sy; 

      alfa=acos(  ((tx-sx)*(ex-sx)+(ty-sy)*(ey-sy) )/(dist(tx,ty,sx,sy)*dist(ex,ey,sx,sy)));

      r=(2*dist(sx,sy,ex,ey)*sin(alfa))/sin(M_PI-2*alfa);
      
      tx=r*cos(ta+M_PI/2.0)+sx;
      ty=-r*sin(ta+M_PI/2.0)+sy; 

      cenx=(sx+tx)/2;
      ceny=(sy+ty)/2;
      
      sangle=get_angle(cenx,ceny,ex,ey);
      eangle=get_angle(cenx,ceny,sx,sy);
    }
  else
    {
      tx=100*cos(ta-M_PI/2.0)+sx;
      ty=-100*sin(ta-M_PI/2.0)+sy; 

      alfa=acos(  ((tx-sx)*(ex-sx)+(ty-sy)*(ey-sy) )/(dist(tx,ty,sx,sy)*dist(ex,ey,sx,sy)));

      r=(2*dist(sx,sy,ex,ey)*sin(alfa))/sin(M_PI-2*alfa);
      
      tx=r*cos(ta-M_PI/2.0)+sx;
      ty=-r*sin(ta-M_PI/2.0)+sy; 

      cenx=(sx+tx)/2;
      ceny=(sy+ty)/2;
      
      sangle=get_angle(cenx,ceny,sx,sy);
      eangle=get_angle(cenx,ceny,ex,ey);
    }
  
  output->centrum.x=cenx;
  output->centrum.y=ceny;
  output->radius=r/2;
  output->start_angle=sangle;
  output->end_angle=eangle;

  return TRUE;
}

gint calc_arc_cont(DOCUMENT *doc, gdouble sx, gdouble sy, gdouble ex, gdouble ey)
{
  /*  gdouble tx,ty;
  gdouble cenx,ceny;
  gdouble ta;
  gdouble r;
  gdouble d;
  gdouble alfa;
  gdouble sangle,eangle;
  OBJECT *object;
  */
  /* ta=doc->continue_arc->data.arc.end_angle*(gdouble)M_PI/180.0; */
  /*
  ta=doc->arc_tangent;

  tx=100*cos(ta+M_PI/2)+sx;
  ty=-100*sin(ta+M_PI/2)+sy;   

  d=dist(tx,ty,sx,sy);

  if((((ty-ey)*(sx-tx)-(tx-ex)*(sy-ty))/(d*d))>0)
    {
      tx=100*cos(ta)+sx;
      ty=-100*sin(ta)+sy;   

      alfa=acos(  ((tx-sx)*(ex-sx)+(ty-sy)*(ey-sy) )/(dist(tx,ty,sx,sy)*dist(ex,ey,sx,sy)));
      
      r=(2*dist(ex,ey,sx,sy)*sin(alfa))/sin(M_PI-2*alfa);
      
      tx=r*cos(ta)+sx;
      ty=-r*sin(ta)+sy; 
      
      cenx=(sx+tx)/2;
      ceny=(sy+ty)/2;
      
      eangle=get_angle(cenx,ceny,sx,sy);
      sangle=get_angle(cenx,ceny,ex,ey);
    }
  else
    {
      tx=100*cos(ta+M_PI)+sx;
      ty=-100*sin(ta+M_PI)+sy;   

      
      alfa=acos(  ((tx-sx)*(ex-sx)+(ty-sy)*(ey-sy) )/(dist(tx,ty,sx,sy)*dist(ex,ey,sx,sy)));
      
      r=(2*dist(ex,ey,sx,sy)*sin(alfa))/sin(M_PI-2*alfa);
      
      tx=r*cos(ta+M_PI)+sx;
      ty=-r*sin(ta+M_PI)+sy; 

      cenx=(sx+tx)/2;
      ceny=(sy+ty)/2;
      
      sangle=get_angle(cenx,ceny,sx,sy);
      eangle=get_angle(cenx,ceny,ex,ey);
    }

  if(doc->arc_side)
    {

      object=obj_add_arc(doc,doc->current_layer,doc->type_colour, doc->current_colour,cenx,ceny,r/2,sangle,eangle); 
      make_continue_arc(doc,TRUE,object);
    }
  else
    {
      object=obj_add_arc(doc,doc->current_layer,doc->type_colour, doc->current_colour,cenx,ceny,r/2,eangle,sangle);
      make_continue_arc(doc,FALSE,object);
    }

  obj_draw_arc(doc, doc->current_layer,object);

  tool_idle(doc);
  */
  return TRUE;
}

gint calc_arc_2p(gdouble ta, gdouble sx, gdouble sy, gdouble ex, gdouble ey, gdouble *mx, gdouble *my,
		 OBJECT_ARC *output)
{
  gdouble tx,ty;
  gdouble r;
  gdouble d;
  gdouble alfa, sangle, eangle;

  tx=100*cos(ta+M_PI/2)+sx;
  ty=-100*sin(ta+M_PI/2)+sy;   

  d=dist(tx,ty,sx,sy);

  if((((ty-ey)*(sx-tx)-(tx-ex)*(sy-ty))/(d*d))>0)
    {
      tx=100*cos(ta)+sx;
      ty=-100*sin(ta)+sy;   

      alfa=acos(  ((tx-sx)*(ex-sx)+(ty-sy)*(ey-sy) )/
		  (dist(tx,ty,sx,sy)*dist(ex,ey,sx,sy)));
      
      output->radius=r=(2*dist(ex,ey,sx,sy)*sin(alfa))/sin(M_PI-2*alfa);
      
      tx=r*cos(ta)+sx;
      ty=-r*sin(ta)+sy; 
      
      output->centrum.x=(sx+tx)/(gdouble)2;
      output->centrum.y=(sy+ty)/(gdouble)2;
      
      eangle=output->end_angle=get_angle(output->centrum.x,output->centrum.y,sx,sy);
      sangle=output->start_angle=get_angle(output->centrum.x,output->centrum.y,ex,ey);

    }
  else
    {
      tx=100*cos(ta+M_PI)+sx;
      ty=-100*sin(ta+M_PI)+sy;   
      
      alfa=acos(((tx-sx)*(ex-sx)+(ty-sy)*(ey-sy) )/
		(dist(tx,ty,sx,sy)*dist(ex,ey,sx,sy)));
      
      output->radius=r=(2*dist(ex,ey,sx,sy)*sin(alfa))/sin(M_PI-2*alfa);
      
      tx=r*cos(ta+M_PI)+sx;
      ty=-r*sin(ta+M_PI)+sy; 

      output->centrum.x=(sx+tx)/(gdouble)2;
      output->centrum.y=(sy+ty)/(gdouble)2;
      
      sangle=output->start_angle=get_angle(output->centrum.x,output->centrum.y,sx,sy);
      eangle=output->end_angle=get_angle(output->centrum.x,output->centrum.y,ex,ey);

    }

  if(sangle<eangle)
    sangle=(eangle-sangle)/2+sangle;
  else
    {
      sangle=(360-sangle+eangle)/2+sangle;
      if(sangle>360)
	sangle-=360;
    }

  if(mx!=NULL)
    (*mx)=output->centrum.x+output->radius/2*cos(sangle*M_PI/180);
  if(my!=NULL)
    (*my)=output->centrum.y-output->radius/2*sin(sangle*M_PI/180); 

  return TRUE;
}









