/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "global.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
void allocate_gc(DOCUMENT *doc)
{
  GdkGCValues values;
  GdkGCValuesMask values_mask;

  values.foreground = doc->black;   /* change back grey90? */ 
  values.background = doc->black; 
  
  values.function = GDK_XOR;
  values_mask = GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_FUNCTION;
  doc->xor_gc = gdk_gc_new_with_values(doc->widgets.draw->window, &values, values_mask);

  if (doc->xor_gc == NULL) {
    g_error(_("Couldn't allocate xor_gc\n"));
    exit(-1);
  }

  values.foreground = doc->black; 
  values.background = doc->black; 

  values.function = GDK_COPY;
  values_mask = GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_FUNCTION;
  doc->replace_gc = gdk_gc_new_with_values(doc->widgets.draw->window, &values, values_mask);

  if (doc->replace_gc == NULL) {
    g_error(_("Couldn't allocate replace_gc\n"));
    exit(-1);
  }

  values.foreground = doc->green; 
  values.background = doc->black; 

  values.function = GDK_COPY;
  values_mask = GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_FUNCTION;
  doc->select_gc = gdk_gc_new_with_values(doc->widgets.draw->window, &values, values_mask);

  if (doc->select_gc == NULL) {
    g_error(_("Couldn't allocate select_gc\n"));
    exit(-1);
  }

  values.foreground = doc->white; 
  values.background = doc->black; 

  values.function = GDK_COPY;
  values_mask = GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_FUNCTION;
  doc->erase_gc = gdk_gc_new_with_values(doc->widgets.draw->window, &values, values_mask);

  if (doc->erase_gc == NULL) {
    g_error(_("Couldn't allocate erase_gc\n"));
    exit(-1);
  }

}

void allocate_color(DOCUMENT *doc)
{
  int ret;
  GdkColormap *colourmap;

  colourmap=gdk_colormap_get_system();

  gdk_color_parse("black", &(doc->black));
  ret = gdk_color_alloc(colourmap, &(doc->black));
  if (ret == 0) {
    g_error(_("Could not allocate the colour black!\n"));
    exit(-1);
  }

  gdk_color_parse("green", &(doc->green));
  ret = gdk_color_alloc(colourmap, &(doc->green));
  if (ret == 0) {
    g_error(_("Could not allocate the color green!\n"));
    exit(-1);
  }
  
  gdk_color_parse("white", &(doc->white));
  ret = gdk_color_alloc(colourmap, &(doc->white));
  if (ret == 0) {
    g_error(_("Could not allocate the color white!\n"));
    exit(-1);
  }
}

COLOUR *make_new_userdefined_colour(DOCUMENT *doc, gdouble r, gdouble g, gdouble b)
{
  GdkColormap *colourmap;
  COLOUR *colour;

  colour=(COLOUR *)g_malloc(sizeof(COLOUR));
  
  colour->type=USERDEFINED;

  colour->predefined_number=0;

  colour->c[0]=r;
  colour->c[1]=g;
  colour->c[2]=b;
  
  colour->doc=doc;

  /* Get drawingarea colormap */  
  colourmap = gdk_window_get_colormap (doc->widgets.draw->window);

  /* Fit to a unsigned 16 bit integer (0..65535) and insert into the GdkColor structure */
  colour->colour.red = (guint16)(r*65535.0);
  colour->colour.green = (guint16)(g*65535.0);
  colour->colour.blue = (guint16)(b*65535.0);

  /* Allocate color */
  gdk_color_alloc (colourmap, &colour->colour);

  if(r+b+g<GFLOAT_SENSITIVE)
    {
      colour->xor_colour.red = (guint16)(0*65535.0);
      colour->xor_colour.green = (guint16)((1.0)*65535.0);
      colour->xor_colour.blue = (guint16)(0*65535.0);
    }
  else
    {
      colour->xor_colour.red = (guint16)((1.0-r)*65535.0);
      colour->xor_colour.green = (guint16)((1.0-g)*65535.0);
      colour->xor_colour.blue = (guint16)((1.0-b)*65535.0);
    }

  gdk_color_alloc (colourmap, &colour->xor_colour);

  return colour;
}

void make_new_colour(DOCUMENT *doc, gint nr, COLOUR *colour, gdouble r, gdouble g, gdouble b)
{
  GdkColormap *colourmap;

  colour->type=PREDEFINED;

  colour->predefined_number=nr;

  colour->c[0]=r;
  colour->c[1]=g;
  colour->c[2]=b;
  
  colour->doc=doc;

  /* Get drawingarea colormap */  
  colourmap = gdk_window_get_colormap (doc->widgets.draw->window);

  /* Fit to a unsigned 16 bit integer (0..65535) and insert into the GdkColor structure */
  colour->colour.red = (guint16)(r*65535.0);
  colour->colour.green = (guint16)(g*65535.0);
  colour->colour.blue = (guint16)(b*65535.0);

  /* Allocate color */
  gdk_color_alloc (colourmap, &colour->colour);

  /* Small fix for black color */
  if(r+b+g<GFLOAT_SENSITIVE)
    {
      colour->xor_colour.red = (guint16)(0*65535.0);
      colour->xor_colour.green = (guint16)((1.0)*65535.0);
      colour->xor_colour.blue = (guint16)(0*65535.0);
    }
  else
    {
      colour->xor_colour.red = (guint16)((1.0-r)*65535.0);
      colour->xor_colour.green = (guint16)((1.0-g)*65535.0);
      colour->xor_colour.blue = (guint16)((1.0-b)*65535.0);
    }
  gdk_color_alloc (colourmap, &colour->xor_colour);

}


void allocate_predefined_colours(DOCUMENT *doc)
{
  gint i;

  for(i=0;i<MAX_NO_PREDEFINED_COLOURS;i++)
    make_new_colour(doc, i,&doc->predefined_colours[i], 0,0,0);  /* Black */    

  make_new_colour(doc, 1,&doc->predefined_colours[1], 0,0,0);  /* White */
  make_new_colour(doc, 2,&doc->predefined_colours[2], 1,0,0);  /* White */
  make_new_colour(doc, 3,&doc->predefined_colours[3], 0,1,0);  /* White */
  make_new_colour(doc, 4,&doc->predefined_colours[4], 0,0,1);  /* White */
  make_new_colour(doc, 5,&doc->predefined_colours[5], 1,1,0);  /* White */
  make_new_colour(doc, 6,&doc->predefined_colours[6], 1,0,1);  /* White */
  make_new_colour(doc, 7,&doc->predefined_colours[7], 0,1,1);  /* White */ 

  doc->no_predefined_colours=8;
}

void make_color(DOCUMENT *doc)
{
  allocate_color(doc);
  allocate_gc(doc);

  allocate_predefined_colours(doc);
}






