/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <sys/stat.h>
#include <dirent.h>
#include <strings.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "misc.h"
#include "dia_misc.h"
#include "redraw.h"
#include "dia_message.h"
#include "translation.h"

/* --  L O C A L   V A R I A B L E S  -- */
#define FILE_DIRECTORY 0
#define FILE_RESCAN    1  /* "." */
#define FILE_PREVIOUS  2  /* ".." */
#define FILE_BLOCK     3

typedef struct{
  /* Type of file */
  gint type;

  /* Simple filename */
  gchar *filename;

  /* Full filename */
  gchar *full_filename;

} _DIRENTRY;

static gchar *ocadis_block_libraries; /* Where are all blocks? */

typedef struct {
  GtkWidget *window;
  GtkWidget *vbox9;
  GtkWidget *vpaned1;
  GtkWidget *frame8;
  GtkWidget *scrolledwindow6;
  GtkWidget *libraries_clist;
  GtkWidget *label30;
  GtkWidget *frame9;
  GtkWidget *scrolledwindow7;
  GtkWidget *block_clist;
  GtkWidget *label31;
  GtkWidget *hseparator3;
  GtkWidget *vbox10;
  GtkWidget *frame10;
  GtkWidget *vbox11;
  GtkWidget *select_entry;
  GtkWidget *frame11;
  GtkWidget *vbox13;
  GtkWidget *table9;
  GtkWidget *scale_entry;
  GtkWidget *rotation_entry;
  GtkWidget *label32;
  GtkWidget *label33;
  GtkWidget *checkbutton_ignore;
  GtkWidget *checkbutton_repeated;
  GtkWidget *hbuttonbox9;
  GtkWidget *insert_button;  
} _DIA_BLOCK;

_DIA_BLOCK *dia_block=NULL;

/* A Glist of directories and block names */
static GList *directory_list=NULL;

/* -- F U N C T I O N S -- */
static gchar *get_previous_path(gchar *path)
{
  gint i;
  gint length;

  if(path==NULL)
    return g_strdup(G_DIR_SEPARATOR_S);

  length=strlen(path);

  if(length==1)
    return g_strdup(G_DIR_SEPARATOR_S);
  
  /* If end of this path contains includes a separator remove it */
  if(path[length-1]==G_DIR_SEPARATOR)  
    length--;

  /* Scanning from right to left to find a separator */
  for(i=length;i>0;i--)
    {
      if(path[i]==G_DIR_SEPARATOR)
	{
	  return g_strndup(path, i);
	}
    }

  /* No separator found */
  return g_strdup(G_DIR_SEPARATOR_S);  
}

static void release_direntry(_DIRENTRY *direntry)
{
  /* Release memory */
  /*  if(direntry->filename!=NULL)
    g_free(direntry->filename);

  if(direntry->full_filename!=NULL)
    g_free(direntry->full_filename);

    g_free(direntry); */
}

static void g(gpointer data, gpointer user_data)
{
  release_direntry((_DIRENTRY *)data);  
}

static void remove_old_list(void)
{
  g_list_foreach(directory_list, (GFunc) g, NULL);
  
  g_list_free(directory_list);

  directory_list=NULL;
}

gboolean open_dir(gchar *path)
{
  _DIRENTRY *direntry;
  gint row;
  gchar *data[1];
  DIR *directory;
  struct stat ent_sbuf;
  struct dirent *dirent_ptr;

  if(directory_list!=NULL)
    remove_old_list();

  /* Trying to open dir */
  directory=opendir(path);

  /* Cannot open directory */
  if(directory==NULL)
    {
      message(ICON_WARNING, _("Unable to locate block libraries.Please set\nenviroment variable \"OCADIS_BLOCK_LIBRARIES\"
so it points to the correct path."));
      return FALSE;
    }

  /* Create directory list */
  gtk_clist_clear(GTK_CLIST(dia_block->libraries_clist));
  gtk_clist_freeze(GTK_CLIST(dia_block->libraries_clist));

  gtk_clist_clear(GTK_CLIST(dia_block->block_clist));
  gtk_clist_freeze(GTK_CLIST(dia_block->block_clist));
 
  while((dirent_ptr=readdir(directory))!=NULL)
    {    
      /* Create new directory */
      direntry=g_new(_DIRENTRY, 1);

      /* Check type */
      if(!strcmp(dirent_ptr->d_name, "."))
	{
	  direntry->type=FILE_RESCAN;

	  /* Copy name */
	  direntry->filename=g_strdup(_("<Rescan directory>"));

	  /* No full name */
	  direntry->full_filename=g_strdup(path);
	}
      else if(!strcmp(dirent_ptr->d_name, ".."))
	{
	  direntry->type=FILE_PREVIOUS;

	  /* Copy name */
	  direntry->filename=g_strdup(_("<Previous directory>"));

	  /* No full name */
	  direntry->full_filename=get_previous_path(path);
	}
      else
	{	 
	  /* Copy name */
	  direntry->filename=g_strdup(dirent_ptr->d_name);

	  /* Check for ending slash and then create full filename */
	  if(path[strlen(path)-1]==G_DIR_SEPARATOR)
	    direntry->full_filename=g_strconcat(path, dirent_ptr->d_name, NULL);
	  else
	    direntry->full_filename=g_strconcat(path, G_DIR_SEPARATOR_S, dirent_ptr->d_name, NULL);

	  if(stat(direntry->full_filename, &ent_sbuf) >= 0 && S_ISDIR(ent_sbuf.st_mode))
	    direntry->type=FILE_DIRECTORY;
	  else
	    /* stat may fail, and we don't mind, since it could be a
	     * dangling symlink. */
	    direntry->type=FILE_BLOCK;	    
	}

      /* Add this entry to clist */
      data[0]=direntry->filename;
      if(direntry->type==FILE_BLOCK)
	{
	  row=gtk_clist_append(GTK_CLIST(dia_block->block_clist), data);
	  gtk_clist_set_row_data(GTK_CLIST(dia_block->block_clist), row, direntry);
	}
     else
       {
	 row=gtk_clist_append(GTK_CLIST(dia_block->libraries_clist), data);
	 gtk_clist_set_row_data(GTK_CLIST(dia_block->libraries_clist), row, direntry);
       }
      
      directory_list=g_list_append(directory_list, (gpointer)direntry);                               
    }

  closedir(directory);

  /* Free path */
  g_free(path);

  /* Unfreeze list */
  gtk_clist_thaw(GTK_CLIST(dia_block->libraries_clist));
  gtk_clist_thaw(GTK_CLIST(dia_block->block_clist));

  return TRUE;
}

static void select_row_callback_dir(GtkWidget *widget,gint row,gint column,
				    GdkEventButton *event, gpointer data)
{
  _DIRENTRY *direntry;

  direntry=gtk_clist_get_row_data(GTK_CLIST(dia_block->libraries_clist), row);

  if(direntry==NULL)
    return;

  if(event->type==GDK_2BUTTON_PRESS)
    {
      open_dir(g_strdup(direntry->full_filename));
      gtk_entry_set_text(GTK_ENTRY(dia_block->select_entry), "");
    }
}

static void select_row_callback_block(GtkWidget *widget,gint row,gint column,
				      GdkEventButton *event, gpointer data)
{
  _DIRENTRY *direntry;

  direntry=gtk_clist_get_row_data(GTK_CLIST(dia_block->block_clist), row);

  if(direntry==NULL)
    return;

  gtk_entry_set_text(GTK_ENTRY(dia_block->select_entry), direntry->filename);
}

/* Callback for button insert */
static gint button_insert(GtkWidget *widget, gpointer data)
{
  _DIRENTRY *direntry;
  GList *list;

  /* See if the select filename exists */
  list=g_list_first(directory_list);

  while(list!=NULL)
    {
      direntry=(_DIRENTRY *)list->data;

      if(!strcmp(direntry->filename, gtk_entry_get_text(GTK_ENTRY(dia_block->select_entry))))
	{
	  if(direntry->type==FILE_DIRECTORY)
	    {
	      open_dir(g_strdup(direntry->full_filename));	      
	      gtk_entry_set_text(GTK_ENTRY(dia_block->select_entry), "");
	    }
	  else if(direntry->type==FILE_BLOCK)
	    {
	    }

	  return TRUE;
	}

      list=g_list_next(list);
    }
  
  message(ICON_WARNING, _("The selected block was not found.\nPlease check your spelling."));
  
  return TRUE;
}

/* Callback for window close */
static gint block_close(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_block->window);
  return TRUE;
}

/* Function to create the actually dialog */
static void make_dialog_block(void)
{
  /* Create struct */
  dia_block=g_new(_DIA_BLOCK,1);

  /* Copy & Paste from Glade */
  dia_block->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (dia_block->window), _("oCADis: Block libraries"));

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_block->window),"delete_event",
                     GTK_SIGNAL_FUNC(block_close),NULL);

  dia_block->vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_block->vbox9);
  gtk_container_add (GTK_CONTAINER (dia_block->window), dia_block->vbox9);

  dia_block->vpaned1 = gtk_vpaned_new ();
  gtk_widget_show (dia_block->vpaned1);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox9), dia_block->vpaned1, TRUE, TRUE, 0);
  gtk_paned_set_position (GTK_PANED (dia_block->vpaned1), 90);

  dia_block->frame8 = gtk_frame_new (_("Libraries"));
  gtk_widget_show (dia_block->frame8);
  gtk_paned_pack1 (GTK_PANED (dia_block->vpaned1), dia_block->frame8, FALSE, TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->frame8), 3);

  dia_block->scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (dia_block->scrolledwindow6);
  gtk_container_add (GTK_CONTAINER (dia_block->frame8), dia_block->scrolledwindow6);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dia_block->scrolledwindow6), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  dia_block->libraries_clist = gtk_clist_new (1);
  gtk_widget_show (dia_block->libraries_clist);
  gtk_container_add (GTK_CONTAINER (dia_block->scrolledwindow6), dia_block->libraries_clist);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->libraries_clist), 3);
  gtk_clist_set_column_width (GTK_CLIST (dia_block->libraries_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (dia_block->libraries_clist));

  /* Add signals */
  gtk_signal_connect(GTK_OBJECT(dia_block->libraries_clist), "select_row", GTK_SIGNAL_FUNC(select_row_callback_dir), NULL);

  dia_block->label30 = gtk_label_new (_("label30"));
  gtk_widget_show (dia_block->label30);
  gtk_clist_set_column_widget (GTK_CLIST (dia_block->libraries_clist), 0, dia_block->label30);

  dia_block->frame9 = gtk_frame_new (_("Block"));
  gtk_widget_show (dia_block->frame9);
  gtk_paned_pack2 (GTK_PANED (dia_block->vpaned1), dia_block->frame9, TRUE, TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->frame9), 3);

  dia_block->scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (dia_block->scrolledwindow7);
  gtk_container_add (GTK_CONTAINER (dia_block->frame9), dia_block->scrolledwindow7);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dia_block->scrolledwindow7), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  dia_block->block_clist = gtk_clist_new (1);
  gtk_widget_show (dia_block->block_clist);
  gtk_container_add (GTK_CONTAINER (dia_block->scrolledwindow7), dia_block->block_clist);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->block_clist), 3);
  gtk_clist_set_column_width (GTK_CLIST (dia_block->block_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (dia_block->block_clist));

  /* Add signals */
  gtk_signal_connect(GTK_OBJECT(dia_block->block_clist), "select_row", GTK_SIGNAL_FUNC(select_row_callback_block), NULL);

  dia_block->label31 = gtk_label_new (_("label31"));
  gtk_widget_show (dia_block->label31);
  gtk_clist_set_column_widget (GTK_CLIST (dia_block->block_clist), 0, dia_block->label31);

  dia_block->hseparator3 = gtk_hseparator_new ();
  gtk_widget_show (dia_block->hseparator3);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox9), dia_block->hseparator3, FALSE, FALSE, 3);

  dia_block->vbox10 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_block->vbox10);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox9), dia_block->vbox10, FALSE, FALSE, 0);

  dia_block->frame10 = gtk_frame_new (_("Selection"));
  gtk_widget_show (dia_block->frame10);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox10), dia_block->frame10, FALSE, FALSE, 3);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->frame10), 3);

  dia_block->vbox11 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_block->vbox11);
  gtk_container_add (GTK_CONTAINER (dia_block->frame10), dia_block->vbox11);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->vbox11), 3);

  dia_block->select_entry = gtk_entry_new ();
  gtk_widget_show (dia_block->select_entry);
  /* Attach signals to entry */
  gtk_signal_connect(GTK_OBJECT(dia_block->select_entry),"activate",
                     GTK_SIGNAL_FUNC(button_insert),NULL);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox11), dia_block->select_entry, FALSE, FALSE, 0);

  dia_block->frame11 = gtk_frame_new (_("Settings"));
  gtk_widget_show (dia_block->frame11);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox10), dia_block->frame11, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->frame11), 3);

  dia_block->vbox13 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_block->vbox13);
  gtk_container_add (GTK_CONTAINER (dia_block->frame11), dia_block->vbox13);

  dia_block->table9 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (dia_block->table9);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox13), dia_block->table9, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->table9), 3);
  gtk_table_set_row_spacings (GTK_TABLE (dia_block->table9), 3);

  dia_block->scale_entry = gtk_entry_new ();
  gtk_entry_set_text(GTK_ENTRY(dia_block->scale_entry), "1.0");
  gtk_widget_show (dia_block->scale_entry);
  gtk_table_attach (GTK_TABLE (dia_block->table9), dia_block->scale_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_block->rotation_entry = gtk_entry_new ();
  gtk_entry_set_text(GTK_ENTRY(dia_block->rotation_entry), "1.0");
  gtk_widget_show (dia_block->rotation_entry);
  gtk_table_attach (GTK_TABLE (dia_block->table9), dia_block->rotation_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_block->label32 = gtk_label_new (_("Scale: "));
  gtk_widget_show (dia_block->label32);
  gtk_table_attach (GTK_TABLE (dia_block->table9), dia_block->label32, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (dia_block->label32), GTK_JUSTIFY_RIGHT);

  dia_block->label33 = gtk_label_new (_("Rotation: "));
  gtk_widget_show (dia_block->label33);
  gtk_table_attach (GTK_TABLE (dia_block->table9), dia_block->label33, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (dia_block->label33), GTK_JUSTIFY_RIGHT);

  dia_block->checkbutton_ignore = gtk_check_button_new_with_label (_("Ignore attributes"));
  gtk_widget_show (dia_block->checkbutton_ignore);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox13), dia_block->checkbutton_ignore, FALSE, FALSE, 0);

  dia_block->checkbutton_repeated = gtk_check_button_new_with_label (_("Repeated insert"));
  gtk_widget_show (dia_block->checkbutton_repeated);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox13), dia_block->checkbutton_repeated, FALSE, FALSE, 0);

  dia_block->hbuttonbox9 = gtk_hbutton_box_new ();
  gtk_widget_show (dia_block->hbuttonbox9);
  gtk_box_pack_start (GTK_BOX (dia_block->vbox9), dia_block->hbuttonbox9, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (dia_block->hbuttonbox9), 3);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dia_block->hbuttonbox9), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (dia_block->hbuttonbox9), 114, -1);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(dia_block->hbuttonbox9), GTK_BUTTONBOX_SPREAD);
  dia_block->insert_button = gtk_button_new_with_label (_("Insert"));
  /* Attach signals to button */
  gtk_signal_connect(GTK_OBJECT(dia_block->insert_button),"clicked",
                     GTK_SIGNAL_FUNC(button_insert),NULL);
  gtk_widget_show (dia_block->insert_button);
  gtk_container_add (GTK_CONTAINER (dia_block->hbuttonbox9), dia_block->insert_button);
  GTK_WIDGET_SET_FLAGS (dia_block->insert_button, GTK_CAN_DEFAULT);
}

void dia_show_block(void)
{
  DOCUMENT *doc;

  doc=misc_object_get_document();

  if(doc==NULL)
    return;
  
  /* Check for where blocks are located */
  ocadis_block_libraries=g_getenv("OCADIS_BLOCK_LIBRARIES");

  if(dia_block==NULL)
    make_dialog_block();

  /* Block library not found! */
  if(ocadis_block_libraries!=NULL)
    {
      /* Init libraries */
      if(open_dir(g_strdup(ocadis_block_libraries)))
	/* View window */
	gtk_widget_show(dia_block->window);
    }
  else
    message(ICON_WARNING, _("Unable to locate block libraries.Please set\nenviroment variable \"OCADIS_BLOCK_LIBRARIES\"
so it points to the correct path."));
}



















