/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include "struct.h"
#include "initmenu.h"
#include "misc.h"
#include "color.h"
#include "dia_message.h"
#include "dia_misc.h"
#include "pop_colour.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static char *colouricon[]={
"18 18 3 1",
"      c None",
".     c #000000000000",
"X     c #111111111111",
"                  ",
" ................ ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" ................ ",
"                  "};


static GtkWidget *dia_window;
static GtkWidget *dia_table;

static GtkAdjustment *dia_adj;
static GtkWidget *dia_spin;

static GtkWidget *dia_text;

static GtkWidget *dia_scrwin;
static GtkWidget *dia_list;

static GtkWidget *dia_close_button;

static DOCUMENT *grab_document;

static gchar *titles[2]={"Number","Colour"};
static gint column_width[2]={50,50};

/* --  P R O T O T Y P E S  -- */
static void generate_row(DOCUMENT *doc, gint i)
{
  gint curr_row;
  gchar *text[2]={NULL,NULL};
  GdkPixmap *color_pixmap;
  gchar colour_string[30];

  text[0]=g_strdup_printf("%d", i+1);

  curr_row=gtk_clist_append(GTK_CLIST(dia_list), text);

  /* Create pixmap for the colour */
  g_snprintf(colour_string,29,"X     c #%04x%04x%04x",doc->predefined_colours[i].colour.red,
	     doc->predefined_colours[i].colour.green,
	     doc->predefined_colours[i].colour.blue);
  
  colouricon[3]=colour_string;

  color_pixmap=gdk_pixmap_create_from_xpm_d(dia_window->window,
					    NULL,
					    &(dia_window->style->bg[GTK_STATE_NORMAL]),
					    colouricon);    
  gtk_clist_set_pixmap(GTK_CLIST(dia_list),curr_row,1, color_pixmap,NULL);

  gdk_pixmap_unref(color_pixmap);
}

static void generate_all_rows(DOCUMENT *doc)
{
  gint i;

  gtk_clist_clear(GTK_CLIST(dia_list));
  gtk_clist_freeze(GTK_CLIST(dia_list));

  gtk_widget_realize(dia_window);

  for(i=0;i<doc->no_predefined_colours;i++)
    generate_row(doc,i);      

  gtk_clist_thaw(GTK_CLIST(dia_list));
}

static void select_row_callback(GtkWidget *widget,gint row,gint column,
				GdkEventButton *event, gpointer data)
{
  GdkPixmap *color_pixmap;
  gchar colour_string[40];
  gdouble colour[3];

  if(column==1 && event->type==GDK_2BUTTON_PRESS)  /* Colour */
    {
      if(dia_colour_selection(colour))
	{

	  make_new_colour(grab_document, row,&grab_document->predefined_colours[row], colour[0], colour[1], colour[2]); 
	  
	  /* Create pixmap for the colour */
	  g_snprintf(colour_string,29,"X     c #%04x%04x%04x",grab_document->predefined_colours[row].colour.red,
	  grab_document->predefined_colours[row].colour.green,
	  grab_document->predefined_colours[row].colour.blue);
	  
	  colouricon[3]=colour_string;
	  
	  color_pixmap=gdk_pixmap_create_from_xpm_d(dia_window->window,
						    NULL,
						    &(dia_window->style->bg[GTK_STATE_NORMAL]),
						    colouricon);    
	  gtk_clist_set_pixmap(GTK_CLIST(dia_list),row,1, color_pixmap,NULL);
	  
	  gdk_pixmap_unref(color_pixmap);
	}
    }
}

static void change_spin( GtkWidget *widget, gpointer data)
{
  grab_document->no_predefined_colours=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(dia_spin));
  generate_all_rows(grab_document);
}

static void colour_set_dialog(DOCUMENT *doc)
{
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(dia_spin), doc->no_predefined_colours);
  generate_all_rows(doc);
}

static void colour_get_dialog(DOCUMENT *doc)
{
  doc->no_predefined_colours=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dia_spin));
  make_colour_popmenu(doc);
  gtk_option_menu_set_history(GTK_OPTION_MENU(doc->widgets.toolbar_colour),0);
  doc->type_colour=BYLAYER;
  doc->current_colour=doc->current_layer->colour;
}

void dia_show_colour(GtkWidget *widget, gpointer data)
{
  grab_document=get_document();
  if(grab_document!=NULL)
    {
      menus_set_sensitive("/Assist/Format/Colour...", FALSE);
      colour_set_dialog(grab_document);
      gtk_widget_show(dia_window);
    }
  else
    message(0, _("No active drawing found.\nPlease make a new drawing or open an existing drawing."));

}

static gint colour_close(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_window);
  colour_get_dialog(grab_document);
  menus_set_sensitive("/Assist/Format/Colour...", TRUE);
  return TRUE;
}


void make_dialog_colour(void)
{
  gint i;

  /* Create main window */
  dia_window=gtk_dialog_new();

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window),_("oCADis: Colour"));

  gtk_window_set_modal(GTK_WINDOW(dia_window),TRUE);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(colour_close),NULL);

  /* Make table */
  dia_table=gtk_table_new(2,2,FALSE);
  gtk_widget_show(dia_table);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dia_window)->vbox),dia_table,TRUE,TRUE,0);

  /* Make text */
  dia_text=gtk_label_new(_("Number of predefined colours: "));
  gtk_table_attach(GTK_TABLE(dia_table), dia_text,0,1,0,1,
		  GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL,0,0);
  gtk_misc_set_alignment(GTK_MISC(dia_text),1.0,0.5);    
  gtk_widget_show(dia_text);

  /* Make spin */
  dia_adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0, MAX_NO_PREDEFINED_COLOURS, 1.0,
						  5.0, 0.0);
  dia_spin = gtk_spin_button_new (dia_adj, 0, 0);

  gtk_signal_connect(GTK_OBJECT (dia_adj), "value_changed",
		     GTK_SIGNAL_FUNC (change_spin), NULL);  
  gtk_table_attach(GTK_TABLE(dia_table), dia_spin,1,2,0,1,
		  GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL,0,0);

  gtk_widget_show(dia_spin);

  /* Create scrolled window */
  dia_scrwin=gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dia_scrwin),GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_table_attach(GTK_TABLE(dia_table), dia_scrwin,0,2,1,2,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_scrwin);

  /* Make list */
  dia_list=gtk_clist_new_with_titles(2,titles);
  gtk_clist_column_titles_passive(GTK_CLIST(dia_list));
  gtk_clist_column_titles_show(GTK_CLIST(dia_list));
  gtk_clist_set_row_height(GTK_CLIST(dia_list),18);
  gtk_clist_set_selection_mode(GTK_CLIST(dia_list), GTK_SELECTION_BROWSE);

  /* Set column width */
  for(i=0;i<2;i++)
    gtk_clist_set_column_width(GTK_CLIST(dia_list), i, column_width[i]);

  gtk_signal_connect(GTK_OBJECT(dia_list),"select_row",
		     GTK_SIGNAL_FUNC(select_row_callback),
		     NULL);
  
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),0,GTK_JUSTIFY_RIGHT);
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),1,GTK_JUSTIFY_CENTER);

  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(dia_scrwin),dia_list);
  gtk_widget_show(dia_list);

  /* Make buttons */
  dia_close_button=gtk_button_new_with_label(_("Close"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_close_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_close_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_close_button);
  gtk_signal_connect(GTK_OBJECT(dia_close_button),"clicked",
		     GTK_SIGNAL_FUNC(colour_close), NULL); 
  gtk_widget_show (dia_close_button);
}



