/* oCADis
 * Copyright (C) 2000 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include "gtk_mtext.h"
#include "struct.h"
#include "global.h"
#include "translation.h"
#include "struct/grdraw.h"
#include "parse.h"
#include "misc.h"
#include "redraw.h"
#include "xor.h"
#include "osnap.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_attribute.h"
#include "obj_polyline.h"
#include "obj_polygon.h"
#include "obj_point.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_rectangle.h"
#include "obj_polyline.h"
#include "obj_spline.h"
#include "obj_insert.h"
#include "dim_horiz.h"
#include "dim_vert.h"
#include "dim_aligned.h"
#include "dim_radius.h"
#include "dim_diameter.h"
#include "dim_center.h"
#include "dim_continue.h"
#include "dim_baseline.h"
#include "dim_leader.h"
#include "dim_angular.h"
#include "tool_zoom.h"
#include "tool_copy.h"
#include "tool_offset.h"
#include "tool_array.h"
#include "tool_move.h"
#include "tool_mirror.h"
#include "tool_rotate.h"
#include "tool_trim.h"
#include "tool_fillet.h"
#include "tool_chamfer.h"
#include "tool_stretch.h"
#include "tool_scale.h"
#include "tool_break.h"
#include "tool_dist.h"
#include "tool_idpoint.h"
#include "tool_block.h"
#include "edit_text.h"
#include "dia_coords.h"
#include "select.h"
#include "initmenu.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;
static GtkWidget *dia_table;
GtkWidget *dia_mtext;
static GtkWidget *dia_vscrollbar;

static gint visible;
static gint items;

/* -- P R O T O T Y P E S -- */
void set_prompt(void);

/* --  F U N C T I O N S  -- */
void print(gchar *string, ...)
{
  gchar buffer[1025];
  va_list args;

  va_start (args, string);
  vsprintf (buffer, string, args);
  va_end (args);

  gtk_mtext_set_point(GTK_MTEXT(dia_mtext),gtk_mtext_get_length(GTK_MTEXT(dia_mtext)));
  gtk_mtext_insert(GTK_MTEXT(dia_mtext),NULL,NULL,NULL,(gchar *)buffer,strlen((gchar*)buffer));
  gtk_mtext_scroll_bottom(GTK_MTEXT(dia_mtext));
}

void print_coord(gdouble x, gdouble y){
  gchar *string=g_strdup_printf("<%f,%f>",x,y);
  print(string);
  g_free(string);
}

void comm_keyboard(DOCUMENT *doc, GdkEventKey *event)
{
  /* If pressed ESCAPE then make cancel */
  if(event->keyval==65307)
    {
      remove_old();
      if(work_document!=NULL)
	deselect_all_objects(work_document);
      print(_("*Cancel*"));
      gl_osnap=OSNAP_NONE;
      tool_idle(doc);
    }
  /* If pressed TAB then try and make autocompletion */
  if(event->keyval==65289)
    {
      gchar *buffer;
      gchar *n_prefix;
      gint no;

      buffer=gtk_editable_get_chars(GTK_EDITABLE(dia_mtext), GTK_MTEXT(dia_mtext)->prompt_index,-1);
      if (!buffer) return;

      no=parse_autocompletion(buffer, &n_prefix);

      /* Many prefix. Just repeat the prompt */
      if(no==0)
	{
	  set_prompt();

	  gtk_mtext_set_point(GTK_MTEXT(dia_mtext), gtk_mtext_get_length(GTK_MTEXT(dia_mtext)));
	  gtk_mtext_insert(GTK_MTEXT(dia_mtext), NULL, NULL, NULL, buffer, strlen(buffer));
	  gtk_mtext_scroll_bottom(GTK_MTEXT(dia_mtext));
	}
      /* One prefix */
      else if(no==1)
	{
	  gtk_mtext_set_point(GTK_MTEXT(dia_mtext), gtk_mtext_get_length(GTK_MTEXT(dia_mtext)));
	  gtk_mtext_insert(GTK_MTEXT(dia_mtext), NULL, NULL, NULL, &n_prefix[strlen(buffer)], strlen(n_prefix)-strlen(buffer));
	  gtk_mtext_scroll_bottom(GTK_MTEXT(dia_mtext));

	  g_free(n_prefix);
	}
      /* No prefix */
      else if(no==-1)
	gdk_beep();
    }
  else
    gtk_mtext_key_press(dia_mtext,event);
}

void prompt(void)
{
  gtk_mtext_thaw(GTK_MTEXT(dia_mtext));
  gtk_mtext_insert(GTK_MTEXT(dia_mtext),NULL,NULL,NULL,
		   GTK_MTEXT(dia_mtext)->prompt,
		   GTK_MTEXT(dia_mtext)->prompt_len);
  GTK_MTEXT(dia_mtext)->prompt_index=gtk_mtext_get_point(GTK_MTEXT(dia_mtext));
  GTK_MTEXT(dia_mtext)->cursor_mark.index=GTK_MTEXT(dia_mtext)->prompt_index;
  GTK_EDITABLE (dia_mtext)->current_pos = GTK_MTEXT(dia_mtext)->cursor_mark.index;
  gtk_mtext_scroll_bottom(GTK_MTEXT(dia_mtext));
}

void set_prompt_text(gchar *string, ...)
{ 
  gchar buffer[1025];
  va_list args;

  va_start (args, string);
  vsprintf (buffer, string, args);
  va_end (args);

  print("\n");
  gtk_mtext_set_prompt(GTK_MTEXT(dia_mtext),buffer,strlen(buffer));
  prompt();
}

void set_prompt(void)
{
  UpdateCoordsLabel();

  switch(gl_tool)
    {
    case TOOL_IDLE:
      set_prompt_text( _("> "));
      break;
    case TOOL_LINE:
      obj_set_prompt_line();
      break;
    case TOOL_RLINE:
      obj_set_prompt_rline();
      break;
    case TOOL_CLINE:
      obj_set_prompt_cline();
      break;
    case TOOL_CIRCLE:
      obj_set_prompt_circle();
      break;
    case TOOL_ARC:
      obj_set_prompt_arc();
      break;
    case TOOL_POLYLINE:
      obj_set_prompt_polyline();
      break;
    case TOOL_SPLINE:
      obj_set_prompt_spline();
      break;
    case TOOL_POINT:
      obj_set_prompt_point();
      break;
    case TOOL_ELLIPSE:
      obj_set_prompt_ellipse();
      break;
    case TOOL_ELARC:
      obj_set_prompt_elarc();
      break;
    case TOOL_TEXT:
      obj_set_prompt_text();
      break;
    case TOOL_ATTRIBUTE:
      obj_set_prompt_attribute();
      break;
    case TOOL_RECTANGLE:
      obj_set_prompt_rectangle();
      break;
    case TOOL_POLYGON:
      obj_set_prompt_polygon();
      break;
    case TOOL_INSERT:
      obj_set_prompt_insert();
      break;
    case TOOL_ZOOM:
      tool_set_prompt_zoom();
      break;
    case TOOL_COPY:
      tool_set_prompt_copy();
      break;
    case TOOL_OFFSET:
      tool_set_prompt_offset();
      break;
    case TOOL_ARRAY:
      tool_set_prompt_array();
      break;
    case TOOL_MOVE:
      tool_set_prompt_move();
      break;
    case TOOL_MIRROR:
      tool_set_prompt_mirror();
      break;
    case TOOL_ROTATE:
      tool_set_prompt_rotate();
      break;
    case TOOL_TRIM:
      tool_set_prompt_trim();
      break;
    case TOOL_IDPOINT:
      tool_set_prompt_idpoint();
      break;
    case TOOL_DIST:
      tool_set_prompt_dist();
      break;
    case TOOL_FILLET:
      tool_set_prompt_fillet();
      break;
    case TOOL_BREAK:
      tool_set_prompt_break();
      break;
    case TOOL_CHAMFER:
      tool_set_prompt_chamfer();
      break;
    case TOOL_STRETCH:
      tool_set_prompt_stretch();
      break;
    case TOOL_SCALE:
      tool_set_prompt_scale();
      break;
    case TOOL_BLOCK:
      tool_set_prompt_block();
      break;
    case TOOL_HORIZ:
      dim_set_prompt_horiz();
      break;
    case TOOL_VERT:
      dim_set_prompt_vert();
      break;
    case TOOL_ALIGNED:
      dim_set_prompt_aligned();
      break;
    case TOOL_RADIUS:
      dim_set_prompt_radius();
      break;
    case TOOL_DIAMETER:
      dim_set_prompt_diameter();
      break;
    case TOOL_CENTER:
      dim_set_prompt_center();
      break;
    case TOOL_CONTINUE:
      dim_set_prompt_continue();
      break;
    case TOOL_BASELINE:
      dim_set_prompt_baseline();
      break;
    case TOOL_LEADER:
      dim_set_prompt_leader();
      break;
    case TOOL_ANGULAR:
      dim_set_prompt_angular();
      break;
    case TOOL_ETEXT:
      tool_set_prompt_etext();
      break;
    case TOOL_SELECT:
      spec_set_prompt_select();
      break;
    }

  if(gl_osnap!=OSNAP_NONE)
    set_snap_text(gl_osnap);

}

void add_prompt_text(gchar *string)
{
  gtk_mtext_set_prompt(GTK_MTEXT(dia_mtext),string,strlen(string));
  prompt();
}


void get_enter(GtkWidget *widget, GtkMText *mtext)
{ 
  gchar *buffer;
  GtkMTextHistItem *hist_item;
  gint len,n,create=FALSE;
  GList *list;
  gpointer pt;
  
  items++;
  buffer=gtk_editable_get_chars(GTK_EDITABLE(mtext), mtext->prompt_index,-1);
  if (!buffer) return;

  if(gl_tool==TOOL_IDLE)
    {
      n=g_list_length(mtext->history)-1;
      len=strlen(buffer);
      if (len>0){
	hist_item=NULL;
	if (n>0){ 
	  pt=(g_list_nth (mtext->history, n))->data;
	  hist_item=(GtkMTextHistItem *)pt;
	}
	if (hist_item==NULL || n==0 || hist_item->len>0){ 
	  hist_item=(GtkMTextHistItem *) g_malloc(sizeof(GtkMTextHistItem));
	  create=TRUE;
	}
	if (hist_item){ 
	  hist_item->entry=buffer;
	  hist_item->len=strlen(buffer);
	  if (create==TRUE)
	    list=g_list_append(mtext->history,(gpointer *) hist_item);
	  len=g_list_length(mtext->history);
	  mtext->hist_mark=len-1;
	}
      }
      
      items=n;
      if (items>10)
	gtk_mtext_remove_hist_n(GTK_MTEXT(mtext), 1);
    }

  parse(NULL, buffer);
}


gint dia_show_command(GtkWidget *widget, gpointer data)
{
  GtkWidget *menuitem;

  menuitem=menus_get_widget( _("/View/Show Command"));      

  if(visible)
    {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(menuitem)->child), _("Show Command"));
      gtk_widget_hide(dia_window);
    }
  else
    {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(menuitem)->child), _("Hide Command"));
      gtk_widget_show(dia_window);
    }

  visible=!visible;
  
  return TRUE;
}

static gint window_keyboard(GtkWidget *widget, GdkEventKey *event)
{
  comm_keyboard(get_document(), event);
  return TRUE;
}

void make_dialog_command(void)
{
  /* Create main window */
  dia_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_container_border_width(GTK_CONTAINER(dia_window),2);
  gtk_window_set_policy(GTK_WINDOW(dia_window), TRUE, TRUE, TRUE);
  gtk_window_set_default_size(GTK_WINDOW(dia_window), 600, 100);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window), _("oCADis: Command Window"));

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
		     GTK_SIGNAL_FUNC(dia_show_command),NULL);
  gtk_signal_connect(GTK_OBJECT(dia_window),"key_press_event",
		     (GtkSignalFunc)window_keyboard, NULL);

  /* Create table */
  dia_table = gtk_table_new (2,1,FALSE);
  gtk_widget_show(dia_table);

  /* Create mtext */
  dia_mtext=gtk_mtext_new(NULL,NULL);
  gtk_widget_show(dia_mtext);

  gtk_signal_connect(GTK_OBJECT(dia_mtext), "activate",
		     GTK_SIGNAL_FUNC(get_enter),GTK_MTEXT(dia_mtext));
  gtk_mtext_set_editable(GTK_MTEXT(dia_mtext), TRUE);
  gtk_mtext_set_prompt(GTK_MTEXT(dia_mtext), _("> "),1);


  dia_vscrollbar = gtk_vscrollbar_new (GTK_MTEXT(dia_mtext)->vadj);
  gtk_widget_show(dia_vscrollbar);

  /* Attach to table */
  gtk_table_attach (GTK_TABLE(dia_table), dia_mtext, 0, 1, 0, 1,
                    GTK_FILL|GTK_EXPAND|GTK_SHRINK,
                    GTK_FILL|GTK_EXPAND|GTK_SHRINK,
                    0,0);
  gtk_table_attach (GTK_TABLE(dia_table), dia_vscrollbar, 1, 2, 0, 1,
		    GTK_FILL,GTK_FILL,0,0);

  /* Attach table to window */
  gtk_container_add(GTK_CONTAINER(dia_window),dia_table);

  /* Window is hidden when init */
  visible=FALSE;

  /* Set up gtk_mtext */
  gtk_widget_realize(dia_mtext);

  /* Make nice welcome text */
  print(_("%s version %s, Copyright (C) 2000 Jonas Jarvoll\n"), PACKAGE, VERSION);
  print(_("%s comes with ABSOLUTELY NO WARRANTY.\n"), PACKAGE);
  print(_("This is free software, and you are welcome to redistribute it\n"));
  print(_("under certain conditions;"));
  print(_(" For details, select menuitem Help/GPL."));
  set_prompt_text( _("> "));  
}





