/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include "struct.h"
#include "global.h"
#include "gtk_mtext.h"
#include "misc.h"
#include "initmenu.h"
#include "dia_toolbox.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* Struct for holding a coordinate window */
typedef struct s_coords
{
  GtkWidget *window;

  GtkWidget *table;
  GtkWidget *tool_entry;
  GtkWidget *tool_label;
  GtkWidget *text[4];
  GtkWidget *entry[4];

  GtkWidget *vbox;
} _COORDS;

static _COORDS *dia_coords=NULL;

static gint visible;


/* -- P R O T O T Y P E S -- */
gint dia_show_coords(GtkWidget *widget, gpointer data);
static void make_dialog_coords(void);

/* --  F U N C T I O N S  -- */
static void view(gint a, gint b, gint c, gint d)
{  
  if(dia_coords==NULL)
    make_dialog_coords();

  if(a)
    {
      gtk_widget_show(dia_coords->text[0]);
      gtk_widget_show(dia_coords->entry[0]);
    }
  else
    {
      gtk_widget_hide(dia_coords->text[0]);
      gtk_widget_hide(dia_coords->entry[0]);
    }
  if(b)
    {
      gtk_widget_show(dia_coords->text[1]);
      gtk_widget_show(dia_coords->entry[1]);
    }
  else
    {
      gtk_widget_hide(dia_coords->text[1]);
      gtk_widget_hide(dia_coords->entry[1]);
    }
  if(c)
    {
      gtk_widget_show(dia_coords->text[2]);
      gtk_widget_show(dia_coords->entry[2]);
    }
  else
    {
      gtk_widget_hide(dia_coords->text[2]);
      gtk_widget_hide(dia_coords->entry[2]);
    }
  if(d)
    {
      gtk_widget_show(dia_coords->text[3]);
      gtk_widget_show(dia_coords->entry[3]);
    }
  else
    {
      gtk_widget_hide(dia_coords->text[3]);
      gtk_widget_hide(dia_coords->entry[3]);
    }
}

void UpdateCoordsLabelTool(gchar *text)
{
  if(dia_coords==NULL)
    make_dialog_coords();

  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry),text);
}

void coord_update_label_tool(gchar *text)
{
  if(dia_coords==NULL)
    make_dialog_coords();

  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry),text);
}

void coord_update_label_text(gint i, gchar *text)
{
  if(dia_coords==NULL)
    make_dialog_coords();

  gtk_entry_set_text(GTK_ENTRY(dia_coords->text[i]),text);
}

void coord_update_label_entry(gint i, gchar *text)
{
  if(dia_coords==NULL)
    make_dialog_coords();

  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[i]),text);
}

void coord_view_label(gboolean a, gboolean b, gboolean c, gboolean d)
{
  if(dia_coords==NULL)
    make_dialog_coords();

  view(a, b, c, d);
}

void coord_update_all(void)
{
  if(dia_coords==NULL)
    make_dialog_coords();

  switch(gl_tool)
    {

/** I N G U I R Y **/
    case TOOL_IDPOINT:
      break;
    case TOOL_DIST:
      break;
    case TOOL_AREA:
      break;
    case TOOL_LIST:
      break;

/** O B J E C T S **/
    case TOOL_RLINE:
      break;
    case TOOL_CLINE:
      break;
    case TOOL_LINE:
      break;
    case TOOL_SPLINE:
      break;
    case TOOL_POLYLINE:
      break;
    case TOOL_RECTANGLE:
      break;
    case TOOL_CIRCLE:
      break;
    case TOOL_ARC:
      break;
    case TOOL_ELLIPSE:
      break;
    case TOOL_ELARC:
      break;
    case TOOL_POINT:
      break;
    case TOOL_TEXT:
      break;
    case TOOL_POLYGON:
      break;
    }
}



void UpdateCoordsLabel(void)
{
  if(dia_coords==NULL)
    make_dialog_coords();
  
  switch(gl_tool)
    {

/** I N G U I R Y **/

    case TOOL_IDPOINT:
      toolbox_set_button("Meas Point", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Id point"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
      view(TRUE,TRUE,FALSE,FALSE);
      break;                      
    case TOOL_DIST:
      toolbox_set_button("Meas Dist", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Distance"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Distance: "));
      view(TRUE,TRUE,TRUE,FALSE);
      break;
  case TOOL_AREA:
      toolbox_set_button("Meas Area", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Area"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
  case TOOL_LIST:
      toolbox_set_button("Meas List", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("List"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;

/** O B J E C T S **/

    case TOOL_RLINE:
    case TOOL_CLINE:
    case TOOL_LINE:
    case TOOL_SPLINE:
    case TOOL_POLYLINE:
      if(gl_tool==TOOL_LINE)
	{
	  toolbox_set_button("Line", 0);	
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Line"));
	}
      else if(gl_tool==TOOL_CLINE)
	{
	  toolbox_set_button("Guideline" ,1);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Construction Line"));
	}
      else if(gl_tool==TOOL_RLINE)
	{
	  toolbox_set_button("Guideline", 0);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Ray"));
	}
      else if(gl_tool==TOOL_POLYLINE)
	{
	  toolbox_set_button("pline", 0);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Polyline"));
	}
      else if(gl_tool==TOOL_SPLINE)
	{
	  toolbox_set_button("Spline", 0);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Spline"));
	}

      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Start X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Start Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("End X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("End Y: "));    
      view(TRUE,TRUE,TRUE,TRUE);
      break;
    case TOOL_RECTANGLE:
      toolbox_set_button("Rectangle", 0);	
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Rectangle"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("1st corner X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("1st corner Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("2nd corner X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("2nd corner Y: "));    
      view(TRUE,TRUE,TRUE,TRUE);
      break;
    case TOOL_ARC:
      toolbox_set_button("Arc", 0);	
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Arc"));
      switch(gl_stage)
	{
	case 0:
	case 4:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Start X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Start Y: "));
	  view(TRUE,TRUE,FALSE,FALSE);
	  break;
	case 1:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Second X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Second Y: "));
	  view(TRUE,TRUE,FALSE,FALSE);
	  break;
	case 2:
	case 13:
	case 5:
	case 8:
	case 15:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("End X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("End Y: "));
	  view(TRUE,TRUE,FALSE,FALSE);
	  break;
	case 3:
	case 12:
	case 9:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  view(TRUE,TRUE,FALSE,FALSE);
	  break;
	case 6:
	case 14:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Included angle: "));
	  view(TRUE,FALSE,FALSE,FALSE);
	  break;
	case 7:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Length: "));
	  view(TRUE,FALSE,FALSE,FALSE);
	  break;
	case 10:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Direction: "));
	  view(TRUE,FALSE,FALSE,FALSE);
	  break;
	case 11:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Radius: "));
	  view(TRUE,FALSE,FALSE,FALSE);
	  break;
	}
      break;
    case TOOL_CIRCLE:
      switch(gl_stage)
	{
	case 0: /* Radius */
	case 1: /* Radius */
	  toolbox_set_button("Circle", 0);
	  break;
	case 2: /* Diameter */
	case 3: /* Diameter */
	  toolbox_set_button("Circle", 1);
	  break;
	case 8: /* 2 P */
	  toolbox_set_button("Circle", 2);
	  break;
	case 4: /* 3 P */
	  toolbox_set_button("Circle", 3);
	  break;
	case 13: /* TTR */
	  toolbox_set_button("Circle", 4);
	  break;
	case 17: /* TTT */
	  toolbox_set_button("Circle", 5);
	  break;
	case 10: /* Donut */
	  toolbox_set_button("Circle", 6);
	  break;
	}

      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Circle"));

      switch(gl_stage)
	{
	case 0:
	case 1:
	case 2:
	case 3:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X center: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y center: "));
	  if(!( (gl_stage==2) || (gl_stage==3)))
	    gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Radius: "));
	  else
	    gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Diameter: "));
	  view(TRUE,TRUE,TRUE,FALSE);
	  break;
	case 4:
	case 8:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("First X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("First Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Radius: "));
	  view(TRUE,TRUE,TRUE,FALSE);
	  break;
	case 5:
	case 9:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Second X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Second Y: "));
	  view(TRUE,TRUE,TRUE,FALSE);
	  break;
	case 6:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Third X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Third Y: "));
	  view(TRUE,TRUE,TRUE,FALSE);
	  break;
	case 10:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Inner Diameter: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Outer Diameter: "));
	  view(TRUE,TRUE,TRUE,TRUE);
	  break;
	}
      break;
    case TOOL_ELLIPSE:
    case TOOL_ELARC:
      if(gl_tool==TOOL_ELLIPSE)
	{
	  switch(gl_stage)
	    {
	    case 0:
	      toolbox_set_button("Ellipse", 1);
	      break;
	    case 3:
	      toolbox_set_button("Ellipse", 0);
	      break;
	    }

	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Ellipse"));
	}
      else
	{
	  toolbox_set_button("Ellipse", 2);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Ellipical Arc"));
	}
      switch(gl_stage)
	{
	case 0:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Axis End X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Axis End Y: "));    
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Center Y: "));
	  break;
	case 3:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Axis End X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Axis End Y: "));    
	  break;
	case 2:
	case 5:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Major axis: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Minor axis: "));
	  break;
	case 6:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Major axis: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Rotation: "));
	  break;
	case 7:
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Start angle: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("End angle: "));
	  break;
	case 9:
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Include angle: "));
	  break;
	}
      view(TRUE,TRUE,TRUE,TRUE);
      break;
    case TOOL_POINT:
      toolbox_set_button("Point", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Point"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
      view(TRUE,TRUE,FALSE,FALSE);
      break;
    case TOOL_TEXT:
      toolbox_set_button("Text", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Text"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Insert X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Insert Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Height: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Rotation: "));    
      view(TRUE,TRUE,TRUE,TRUE);
      break;
    case TOOL_POLYGON:
      toolbox_set_button("Polygon", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Polygon"));
      if(gl_stage>=3)
	{
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X1: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y1: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("X2: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Y2: "));
	  view(TRUE,TRUE,TRUE,TRUE);
	}
      else
	{
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Radius: "));
	  view(TRUE,TRUE,TRUE,FALSE);
	}
      break;      

/** D I M E N S I O N S **/

   case TOOL_HORIZ:
      toolbox_set_button("Linear1", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Horizontal"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_VERT:
      toolbox_set_button("Linear2", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Vertical"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_CONTINUE:
      toolbox_set_button("Continue", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Continue"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
    case TOOL_BASELINE:
      toolbox_set_button("Baseline", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Baseline"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_RADIUS:
      toolbox_set_button("Radial", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Radius"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_DIAMETER:
      toolbox_set_button("Radial", 1);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Diameter"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_CENTER:
      toolbox_set_button("Center", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Center"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_LEADER:
      toolbox_set_button("Leader", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Leader"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_ALIGNED:
      toolbox_set_button("Linear3", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Aligned"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_ANGULAR:
      toolbox_set_button("Angular", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Angular"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;

/** C O N S T R U C T **/                            

    case TOOL_COPY:
      toolbox_set_button("Copy", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Copy"));

      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Base X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Base Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Y: "));
      view(TRUE,TRUE,TRUE,TRUE);
      break;
    case TOOL_OFFSET:
      toolbox_set_button("Offset", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Offset"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Distance: "));
      view(TRUE,TRUE,TRUE,FALSE);
      break;
    case TOOL_MIRROR:
      toolbox_set_button("Mirror", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Mirror"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X1: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y1: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("X2: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Y2: "));
      view(TRUE,TRUE,TRUE,TRUE);
      break;                      
    case TOOL_ARRAY:
      if(gl_stage==0)
	{
	  toolbox_set_button("Array", 0);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Rectangular Array"));
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("dX: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("dY: "));
	  view(TRUE,TRUE,TRUE,TRUE);
	}
      else if(gl_stage==5)
	{
	  toolbox_set_button("Array", 1);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Polar Array"));
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Center X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Center Y: "));
	  view(TRUE,TRUE,FALSE,FALSE);
	}
      else if(gl_stage==8)
	{
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Path Array"));
	  gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
	  gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
	  view(TRUE,TRUE,FALSE,FALSE);
	}
      break;
    case TOOL_CHAMFER:
      toolbox_set_button("Chamfer", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Chamfer"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Distance 1: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Distance 2: "));
      view(TRUE,TRUE,TRUE,TRUE);
      break;                      
    case TOOL_FILLET:
      toolbox_set_button("Fillet", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Fillet"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;

/** M O D I F Y **/

    case TOOL_MOVE:
      toolbox_set_button("Move", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Move"));
      
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Base X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Base Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Y: "));
      view(TRUE,TRUE,TRUE,TRUE);
      break;
    case TOOL_ROTATE:
      toolbox_set_button("Rotate", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Rotate"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Base X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Base Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Angle: "));
      view(TRUE,TRUE,TRUE,FALSE);
      break;                
    case TOOL_STRETCH:
      toolbox_set_button("Stretch", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Stretch"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
    case TOOL_SCALE:
      toolbox_set_button("Scale", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Scale"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("Base X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Base Y: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("Reference: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Scale: "));
      view(TRUE,TRUE,TRUE,TRUE);
      break;            
    case TOOL_LENGTHEN:
      toolbox_set_button("Lengthen", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Lengthen"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
    case TOOL_TRIM:
      toolbox_set_button("Trim", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Trim"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
    case TOOL_EXTEND:
      toolbox_set_button("Extend", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Extend"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
    case TOOL_BREAK:
      toolbox_set_button("Break", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Break"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X1: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y1: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[2]), _("X2: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[3]), _("Y2: "));
      view(TRUE,TRUE,TRUE,TRUE);
      break;                
   case TOOL_EPLINE:
      toolbox_set_button("Edit", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Edit Polyline"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_ESPLINE:
      toolbox_set_button("Edit", 1);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Edit Spline"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_ETEXT:
      toolbox_set_button("Edit", 2);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Edit Text"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_EPROP:
      toolbox_set_button("Edit", 3);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Edit Properties"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;
   case TOOL_EXPLODE:
      toolbox_set_button("Explode", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Explode"));
      view(FALSE, FALSE, FALSE, FALSE);
      break;          
    case TOOL_ERASE:
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Erase"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
      toolbox_set_button("Erase", 0);
      view(TRUE,TRUE,FALSE,FALSE);
      break;

/** M I S C **/

    case TOOL_IDLE:
      toolbox_set_button("Idle", 0);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Idle"));
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));
      view(TRUE,TRUE,FALSE,FALSE);
      break;
    case TOOL_SELECT:
      gtk_label_set(GTK_LABEL(dia_coords->text[0]), _("X: "));
      gtk_label_set(GTK_LABEL(dia_coords->text[1]), _("Y: "));

      if(gl_stage==0) /* Single point */
	toolbox_set_button("Select Sing", 0);
      else if(gl_stage==1 || gl_stage==2) /* Window */
	toolbox_set_button("Select Win", 0);
      else if(gl_stage==3 || gl_stage==4) /* Crossing */
	toolbox_set_button("Select Cross", 0);
    
      view(TRUE,TRUE,FALSE,FALSE);
      break;                      
   case TOOL_ZOOM:
     if(gl_stage==0 || gl_stage==1)
       {
	 toolbox_set_button("Zoom Win", 0);
	 gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Zoom"));
	 view(FALSE, FALSE, FALSE, FALSE);
       }
     else if(gl_stage==2 || gl_stage==3)
       {
	 toolbox_set_button("Zoom Pan", 0);
	 gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry), _("Panning"));
	 view(FALSE, FALSE, FALSE, FALSE);
       }
      break;
    }
}

void UpdateSingelCoord(gint which, gdouble value)
{
  gchar *t;

  if(dia_coords==NULL)
    make_dialog_coords();

  if(which<0)
    gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[ABS(which)]),"");  
  else
    {
      t=g_strdup_printf("%f", (gdouble)value);
      gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[which]), t);     
      g_free(t);
    }
}

void UpdateCoords(DOCUMENT *doc, gdouble x, gdouble y)
{
  if(dia_coords==NULL)
     make_dialog_coords();

  switch(gl_tool)
    {
    case TOOL_IDLE:
    case TOOL_IDPOINT:
    case TOOL_POINT:
      UpdateSingelCoord(0, x);
      UpdateSingelCoord(1, y);
      break;
    case TOOL_COPY:
    case TOOL_MOVE:
      if(gl_stage==0 || gl_stage==2)
	{
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
	}
      else
	{
	  UpdateSingelCoord(2, x);
	  UpdateSingelCoord(3, y);
	}
      break;
    case TOOL_OFFSET:
      if(gl_stage==0)
	{
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
	}
      else{}  /* This is in the tool_offset.c */
      break;
    case TOOL_ARRAY:
      if(gl_stage==0 || gl_stage==2 || gl_stage==5 || gl_stage==8 || gl_stage==9 )
	{
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
	}
      else if(gl_stage==4)
	{
	  UpdateSingelCoord(2, x-gl_oldmx);
	  UpdateSingelCoord(3, y-gl_oldmy);
	}
      break;
    case TOOL_MIRROR:
    case TOOL_CLINE:
    case TOOL_RLINE:
    case TOOL_LINE:
    case TOOL_POLYLINE:
    case TOOL_SPLINE:
    case TOOL_RECTANGLE:
      if(gl_stage==0){
	UpdateSingelCoord(0, x);
	UpdateSingelCoord(1, y);

        gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
        gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
      }else if(gl_stage==1 || (gl_tool==TOOL_SPLINE && gl_stage==2)){
	UpdateSingelCoord(2, x);
	UpdateSingelCoord(3, y);
      }
      break;
    case TOOL_ELLIPSE:
    case TOOL_ELARC:
      if(gl_stage==0 || gl_stage==3){
	UpdateSingelCoord(0, x);
	UpdateSingelCoord(1, y);

        gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
        gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
      }else if(gl_stage==1 || gl_stage==4){
	  UpdateSingelCoord(2, x);
	  UpdateSingelCoord(3, y);
	}
      break;
    case TOOL_POLYGON:
      if(gl_stage==1)
	{
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
	}
      else if(gl_stage==3)
	{
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
	}
      else if(gl_stage==4)
	{
	  UpdateSingelCoord(2, x);
	  UpdateSingelCoord(3, y);       
	}    
      break;
    case TOOL_TEXT:
      if(gl_stage==0){
	UpdateSingelCoord(0, x);
	UpdateSingelCoord(1, y);
      }
      break;
    case TOOL_CIRCLE:
      switch(gl_stage)
	{
	case 0:
	case 2:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),"");                 
	  break;
	case 1:
	  UpdateSingelCoord(2, (gdouble)dist(x,y,gl_oldmx,gl_oldmy));
	  break;
	case 3:
	  UpdateSingelCoord(2, (gdouble)2*dist(x,y,gl_oldmx,gl_oldmy));
	  break;
	case 9:
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  UpdateSingelCoord(2, (gdouble)dist(x,y,gl_oldmx,gl_oldmy)/2);        
	  break;
	case 10:
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  break;
	}

      break;
    case TOOL_ARC:
      /* For the other stages the update is placed in resp xor-function */
      switch(gl_stage)
      {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 8:
      case 9:
      case 12:
      case 13:
      case 15:
	UpdateSingelCoord(0, x);
	UpdateSingelCoord(1, y);
	break;
      }
      break;
    case TOOL_ROTATE:
      if(gl_stage==0){
	UpdateSingelCoord(0, x);
	UpdateSingelCoord(1, y);
        gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
      }else if(gl_stage==1){}	/* Update in XOR-function */
      break;
    case TOOL_DIST:
      if(gl_stage==0){
	UpdateSingelCoord(0, x);
	UpdateSingelCoord(1, y);
        gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
	gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
      }else if(gl_stage==1){
	UpdateSingelCoord(2, dist(gl_oldmx,gl_oldmy,x,y));
      }
      break;
    case TOOL_BREAK:
      if(gl_stage==0 || gl_stage==2){
	UpdateSingelCoord(0, x);
	UpdateSingelCoord(1, y);
        gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[2]),""); 
	gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
      }else if(gl_stage==1 || gl_stage==3){
	UpdateSingelCoord(2, x);
	UpdateSingelCoord(3, y);
      }
      break;
    case TOOL_CHAMFER:
      UpdateSingelCoord(0, x);
      UpdateSingelCoord(1, y);
      break;
    case TOOL_SCALE:
      if(gl_stage==0)
	{
	  UpdateSingelCoord(0, x);
	  UpdateSingelCoord(1, y);
	  gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[3]),""); 
	}
      else if(gl_stage==3)
	UpdateSingelCoord(2, dist(x, y, gl_oldmx, gl_oldmy));
      break;
    case TOOL_SELECT:
      UpdateSingelCoord(0, x);
      UpdateSingelCoord(1, y);
      break;
    }
}

static gint window_keyboard( GtkWidget *widget, GdkEventKey *event)
{
  gtk_mtext_key_press(dia_mtext,event);
  return TRUE;
}


static void make_dialog_coords(void)
{
  gint i;

  /* Create struct */
  dia_coords=g_new(_COORDS,1);

  /* Create main window */
  dia_coords->window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_container_border_width(GTK_CONTAINER(dia_coords->window),2);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_coords->window), _("oCADis: Coordinates"));

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_coords->window),"delete_event",
                     GTK_SIGNAL_FUNC(dia_show_coords),NULL);
  gtk_signal_connect(GTK_OBJECT(dia_coords->window),"key_press_event",
                     (GtkSignalFunc)window_keyboard, NULL);

  /* Make packing boxes */
  dia_coords->vbox=gtk_vbox_new(FALSE,0);
  gtk_widget_show(dia_coords->vbox);
  
  /* Make table */
  dia_coords->table=gtk_table_new(2,5,FALSE);
  gtk_widget_show(dia_coords->table);

  /* Make tool entry */
  dia_coords->tool_label=gtk_label_new( _("Tool: "));
  gtk_misc_set_alignment(GTK_MISC(dia_coords->tool_label),0.0,0.5);
  gtk_widget_show(dia_coords->tool_label);

  dia_coords->tool_entry=gtk_entry_new();
  gtk_widget_show(dia_coords->tool_entry);
  gtk_entry_set_editable(GTK_ENTRY(dia_coords->tool_entry),FALSE);
  gtk_entry_set_text(GTK_ENTRY(dia_coords->tool_entry),"");

  gtk_table_attach(GTK_TABLE(dia_coords->table), dia_coords->tool_label,0,1,0,1,
		   GTK_FILL, GTK_FILL|GTK_EXPAND,0,0);
  gtk_table_attach(GTK_TABLE(dia_coords->table), dia_coords->tool_entry,1,2,0,1,
		   GTK_FILL|GTK_EXPAND, GTK_FILL,0,0);

  /* Make labels */
  for(i=0;i<4;i++){
    dia_coords->text[i]=gtk_label_new("");
    gtk_widget_show(dia_coords->text[i]);
    gtk_misc_set_alignment(GTK_MISC(dia_coords->text[i]),0.0,0.5);
     
    dia_coords->entry[i]=gtk_entry_new();
    gtk_widget_show(dia_coords->entry[i]);
    gtk_entry_set_editable(GTK_ENTRY(dia_coords->entry[i]),FALSE);
    gtk_entry_set_text(GTK_ENTRY(dia_coords->entry[i]),"");
 
    gtk_table_attach(GTK_TABLE(dia_coords->table), dia_coords->text[i],0,1,i+1,i+2,
                     GTK_FILL, GTK_FILL|GTK_EXPAND,0,0);
    gtk_table_attach(GTK_TABLE(dia_coords->table), dia_coords->entry[i],1,2,i+1,i+2,
                     GTK_FILL|GTK_EXPAND, GTK_FILL,0,0);
  }

  gtk_box_pack_start(GTK_BOX(dia_coords->vbox),dia_coords->table,TRUE,TRUE,0);  
  gtk_container_add(GTK_CONTAINER(dia_coords->window),dia_coords->vbox);
  
  /* Window is hidden when init */
  visible=FALSE;

  UpdateCoordsLabel();
}

gint dia_show_coords(GtkWidget *widget, gpointer data)
{
  GtkWidget *menuitem;

  if(dia_coords==NULL)
    make_dialog_coords();

  menuitem=menus_get_widget( _("/View/Show Coordinates"));      

  if(visible)
    {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(menuitem)->child), _("Show Coordinates"));
      gtk_widget_hide(dia_coords->window);
    }
  else
    {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(menuitem)->child), _("Hide Coordinates"));
      gtk_widget_show(dia_coords->window);
    }

  visible=!visible;

  return TRUE;
}

