/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "layer.h"
#include "color.h"
#include "misc.h"
#include "dia_misc.h"
#include "redraw.h"
#include "dia_message.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;

static GtkWidget *dia_scrwin;
static GtkWidget *dia_list;

static GtkWidget *dia_close_button;

static DOCUMENT *grab_document;

static gchar *titles[3]={"Variable", "Description", "Value"};
static gint column_width[3]={90,300,70};

/*

  This is the complete list of variables 
  These should eventually be implemtened.

#define NO_DIM_VAR 42

static gchar dimvar_name[NO_DIM_VAR*2][50]={ {"DIMALT"}, {"Alternate units"},
					     {"DIMALTD"}, {"Alternate units decimal places"},
					     {"DIMALTF"}, {"Alternate units scale factor"},
					     {"DIMAPOST"}, {"Alternate units text suffix"},
					     {"DIMASO"}, {"Associative dimensioning"},
					     {"DIMASZ"}, {"Arrow size"},
					     {"DIMBLK"}, {"Arrow block"},
					     {"DIMBLK1"}, {"Separate arrow block 1"},
					     {"DIMBLK2"}, {"Separate arrow block 2"},
					     {"DIMCEN"}, {"Center mark size"},
					     {"DIMCLRD"}, {"Dimension line colour"},
					     {"DIMCLRE"}, {"Extension line colour"},
					     {"DIMCLRT"}, {"Dimesion text colour"},
					     {"DIMDLE"}, {"Dimesion text extension"},
					     {"DIMDLI"}, {"Dimesion line increment"},
					     {"DIMEXE"}, {"Extension line extension"},
					     {"DIMEXO"}, {"Extension line offset"},
					     {"DIMGAP"}, {"Dimension line gap"},
					     {"DIMLFAC"}, {"Length factor"},
					     {"DIMLIM"}, {"Limits dimensioning"},
					     {"DIMPOST"}, {"Dimension text suffix"},
					     {"DIMRND"}, {"Dimension rounding value"},
					     {"DIMSAH"}, {"Separate arrow blocks"},
					     {"DIMSCALE"}, {"Global scale factor"},
					     {"DIMSE1"}, {"Suppress extension line 1"},
					     {"DIMSE2"}, {"Suppress extension line 2"},
					     {"DIMSHO"}, {"Show dragged dimension"},
					     {"DIMSOXD"}, {"Suppress outside dimension lines"},
					     {"DIMSTYLE"}, {"Dimension style"},
					     {"DIMTAD"}, {"Text above dimension line"},
					     {"DIMTFAC"}, {"Tolerance text scale factor"},
					     {"DIMTIH"}, {"Text inside horizontal"},
					     {"DIMTIX"}, {"Text inside extension lines"},
					     {"DIMTM"}, {"Minus tolerance value"},
					     {"DIMTP"}, {"Plus tolerance value"},
					     {"DIMTOFL"}, {"Text outside, force line inside"},
					     {"DIMTOH"}, {"Text outside horizontal"},
					     {"DIMTOL"}, {"Tolerance dimensioning"},
					     {"DIMTSZ"}, {"Tick size"},
					     {"DIMTVP"}, {"Text vertical position"},
					     {"DIMTXT"}, {"Text size"},
					     {"DIMZIN"}, {"Zero suppression"}};
*/

#define NO_DIM_VAR 10

static gchar dimvar_name[NO_DIM_VAR*2][70]={ 
  {"dimexe"}, {"Distance extension line from dimension line"},
  {"dimexo"}, {"Distance between defpoint and start of line"},
  {"dimgap"}, {"Distance between dimension text and dimension line"},
  {"dimtxt"}, {"Size of dimension text"},
  {"dimdec"}, {"Number of unit decimal places"},
  {"dimdli"}, {"Distance between 2 dimension lines in baseline"},
  {"dimatp"}, {"Type of arrow"},
  {"dimasz"}, {"Size of arrow"},
  {"dimmtp"}, {"Type of centermark"},
  {"dimmsz"}, {"Size of centermark"}
};

/* --  P R O T O T Y P E S  -- */
static void generate_all_rows(DOCUMENT *doc)
{
  gint i;
  gchar *text[3]={NULL,NULL,NULL};

  gtk_clist_clear(GTK_CLIST(dia_list));
  gtk_clist_freeze(GTK_CLIST(dia_list));

  for(i=0;i<NO_DIM_VAR;i++)
    {
      text[0]=dimvar_name[i*2];
      text[1]=dimvar_name[i*2+1];

      switch(i)
	{
	case 0:	  /* dimexe */
	  text[2]=g_strdup_printf("%f", doc->current_dimensionstyle->dimexe);
	  break;
	case 1:	  /* dimexo */
	  text[2]=g_strdup_printf("%f", doc->current_dimensionstyle->dimexo);
	  break;
	case 2:	  /* dimgap */
	  text[2]=g_strdup_printf("%f", doc->current_dimensionstyle->dimgap);
	  break;
	case 3:	  /* dimtxt */
	  text[2]=g_strdup_printf("%f", doc->current_dimensionstyle->dimtxt);
	  break;
	case 4:	  /* dimdec */
	  text[2]=g_strdup_printf("%d", doc->current_dimensionstyle->dimdec);
	  break;
	case 5:	  /* dimdli */
	  text[2]=g_strdup_printf("%f", doc->current_dimensionstyle->dimdli);
	  break;
	case 6:	  /* dimatp */
	  text[2]=g_strdup_printf("%d", doc->current_dimensionstyle->dimatp);
	  break;
	case 7:	  /* dimasz */
	  text[2]=g_strdup_printf("%f", doc->current_dimensionstyle->dimasz);
	  break;
	case 8:	  /* dimmtp */
	  text[2]=g_strdup_printf("%d", doc->current_dimensionstyle->dimmtp);
	  break;
	case 9:	  /* dimmsz */
	  text[2]=g_strdup_printf("%f", doc->current_dimensionstyle->dimmsz);
	  break;
	}

      gtk_clist_append(GTK_CLIST(dia_list), text);
    }

  gtk_clist_thaw(GTK_CLIST(dia_list));
}

static void select_row_callback(GtkWidget *widget,gint row,gint column,
				GdkEventButton *event, gpointer data)
{
  gchar *new_value;
  gchar *old_value;

  if(column==2 && event->type==GDK_2BUTTON_PRESS)  /* Layer name */
  {
    gtk_clist_get_text(GTK_CLIST(dia_list), row, column, &old_value);
    new_value=dia_enter_text(_("Change value"), _("New value: "), old_value);
    
    if(new_value!=NULL)
      {
	switch(row)
	  {
	  case 0:	  /* dimexe */
	    grab_document->current_dimensionstyle->dimexe=(gdouble)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%f", grab_document->current_dimensionstyle->dimexe));
	    break;
	  case 1:	  /* dimexo */
	    grab_document->current_dimensionstyle->dimexo=(gdouble)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%f", grab_document->current_dimensionstyle->dimexo));
	    break;
	  case 2:	  /* dimgap */
	    grab_document->current_dimensionstyle->dimgap=(gdouble)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%f", grab_document->current_dimensionstyle->dimgap));
	    break;
	  case 3:	  /* dimtxt */
	    grab_document->current_dimensionstyle->dimtxt=(gdouble)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%f", grab_document->current_dimensionstyle->dimtxt));
	    break;
	  case 4:	  /* dimdec */
	    grab_document->current_dimensionstyle->dimdec=(gint)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%d", grab_document->current_dimensionstyle->dimdec));
	    break;
	  case 5:	  /* dimdli */
	    grab_document->current_dimensionstyle->dimdli=(gdouble)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%f", grab_document->current_dimensionstyle->dimdli));
	    break;
	  case 6:	  /* dimatp */
	    grab_document->current_dimensionstyle->dimatp=(gint)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%d", grab_document->current_dimensionstyle->dimatp));
	    break;
	  case 7:	  /* dimasz */
	    grab_document->current_dimensionstyle->dimasz=(gdouble)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%f", grab_document->current_dimensionstyle->dimasz));
	    break;
	  case 8:	  /* dimmtp */
	    grab_document->current_dimensionstyle->dimmtp=(gint)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%d", grab_document->current_dimensionstyle->dimmtp));
	    break;
	  case 9:	  /* dimmsz */
	    grab_document->current_dimensionstyle->dimmsz=(gdouble)g_strtod(new_value,NULL);
	    gtk_clist_set_text(GTK_CLIST(dia_list),row,2, 
			       g_strdup_printf("%f", grab_document->current_dimensionstyle->dimmsz));
	    break;
	  }	 
      }    
  }
}

static gint dimsetup_close(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_window);
  menus_set_sensitive("/Assist/Format/Dimension Setup...", TRUE);
  return TRUE;
}

void dia_show_dimsetup(GtkWidget *widget, gpointer data)
{
  grab_document=get_document();
  if(grab_document!=NULL)
    {
      menus_set_sensitive("/Assist/Format/Dimension Setup...", FALSE);
      generate_all_rows(grab_document);
      gtk_widget_show(dia_window);
    }
  else
    message(0, _("No active drawing found.\nPlease make a new drawing or open an existing drawing."));

}

void make_dialog_dimsetup(void)
{
  gint i;

  /* Create main window */
  dia_window=gtk_dialog_new();
  gtk_widget_realize(dia_window);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window), _("oCADis: Dimension Setup"));
  gtk_widget_set_usize(GTK_WIDGET( dia_window),600,400);

  gtk_window_set_modal(GTK_WINDOW(dia_window),TRUE);
  
  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(dimsetup_close),NULL);

  /* Create scrolled window */
  dia_scrwin=gtk_scrolled_window_new(NULL,NULL);
  gtk_widget_show(dia_scrwin);

  /* Create list */
  dia_list=gtk_clist_new_with_titles(3, titles);
  gtk_clist_set_row_height(GTK_CLIST(dia_list),18);
  gtk_clist_column_titles_passive(GTK_CLIST(dia_list));
  gtk_clist_column_titles_show(GTK_CLIST(dia_list));
  gtk_clist_set_selection_mode(GTK_CLIST(dia_list), GTK_SELECTION_BROWSE);

  gtk_signal_connect(GTK_OBJECT(dia_list),"select_row",
		     GTK_SIGNAL_FUNC(select_row_callback),
		     NULL);

  gtk_clist_set_column_justification(GTK_CLIST(dia_list),0,GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),1,GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),2,GTK_JUSTIFY_RIGHT);

  /* Set column width */
  for(i=0;i<3;i++)
    gtk_clist_set_column_width(GTK_CLIST(dia_list), i, column_width[i]);

  gtk_widget_show(dia_list);

  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(dia_scrwin),dia_list);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->vbox),
		      dia_scrwin, TRUE, TRUE, 0);


  /* Make buttons */
  dia_close_button=gtk_button_new_with_label( _("Close"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_close_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_close_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_close_button);
  gtk_signal_connect(GTK_OBJECT(dia_close_button),"clicked",
		     GTK_SIGNAL_FUNC(dimsetup_close), NULL); 
  gtk_widget_show (dia_close_button);
}






