/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "gtk_mtext.h"
#include "initmenu.h"
#include "misc.h"
#include "grid.h"
#include "dia_message.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;
static GtkWidget *dia_table;

static GtkWidget *dia_ok_button;
static GtkWidget *dia_cancel_button;

static GtkWidget *dia_frame1;
static GtkWidget *dia_frame1_vbox;
static GtkWidget *dia_frame2;
static GtkWidget *dia_frame2_vbox;
static GtkWidget *dia_frame3;
static GtkWidget *dia_frame3_vbox;
static GtkWidget *dia_frame4;
static GtkWidget *dia_frame4_vbox;

static GtkWidget *dia_frame1_ortho;
static GtkWidget *dia_frame1_solidfill;
static GtkWidget *dia_frame1_quicktext;
static GtkWidget *dia_frame1_blip;
static GtkWidget *dia_frame1_highlights;
static GtkWidget *dia_frame1_groups;
static GtkWidget *dia_frame1_hatch;

static GtkWidget *dia_frame2_table;
static GtkWidget *dia_frame2_snap;
static GtkWidget *dia_frame2_text[5];
static GtkWidget *dia_frame2_entry[5];

static GtkWidget *dia_frame3_table;
static GtkWidget *dia_frame3_grid;
static GtkWidget *dia_frame3_text[2];
static GtkAdjustment *dia_frame3_adj[2];
static GtkWidget *dia_frame3_spin[2];

static GtkWidget *dia_frame4_table;
static GtkWidget *dia_frame4_iso;
static GSList *dia_frame4_radiogroup;
static GtkWidget *dia_frame4_radio[3];

static DOCUMENT *grab_document;

/* --  F U N C T I O N S  -- */
static void drawaids_set_dialog(DOCUMENT *doc)
{
  /* Set toggle buttons */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_frame1_ortho),(gint)doc->gl_ortho);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_frame2_snap),(gint)doc->gl_snap);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_frame3_grid),(gint)doc->gl_grid);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_frame4_iso),(gint)doc->gl_iso_view);
  //  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dia_frame1_blip),(gint)gl_blip);
  
  /* Set text entry data */
  
  /* Set snap data */
  /* This could be a memory leak! */
  gtk_entry_set_text(GTK_ENTRY(dia_frame2_entry[0]),
		     g_strdup_printf("%f", doc->gl_snap_width));
  gtk_entry_set_text(GTK_ENTRY(dia_frame2_entry[1]),
		     g_strdup_printf("%f", doc->gl_snap_height));
  gtk_entry_set_text(GTK_ENTRY(dia_frame2_entry[2]),
		     g_strdup_printf("%f", (gfloat)(doc->gl_snap_angle*180/M_PI)));
  gtk_entry_set_text(GTK_ENTRY(dia_frame2_entry[3]),
		     g_strdup_printf("%f", doc->gl_snap_basex));
  gtk_entry_set_text(GTK_ENTRY(dia_frame2_entry[4]),
		     g_strdup_printf("%f", doc->gl_snap_basey));

  /* Set grid data */
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(dia_frame3_spin[0]), doc->gl_grid_x);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(dia_frame3_spin[1]), doc->gl_grid_y);

  /* Set iso data */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_frame4_radio[0]), (doc->gl_iso_type==ISOMETRIC_LEFT) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_frame4_radio[1]), (doc->gl_iso_type==ISOMETRIC_TOP) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_frame4_radio[2]), (doc->gl_iso_type==ISOMETRIC_RIGHT) );

}

static gint drawaids_get_dialog(DOCUMENT *doc)
{
  gdouble a;
  /* Check range of the snap angle */
  a=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_frame2_entry[2])),NULL);

  if( a<-90 || a>90)
    {
      message(0, _("Snap angle is out of range.\nThe angle may range from 0 to 90 or from 0 to -90."));
      return FALSE;
    }

  /* Set toggle buttons */
  doc->gl_ortho=(gint)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_frame1_ortho));
  doc->gl_snap=(gint)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_frame2_snap));
  doc->gl_grid=(gint)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_frame3_grid));
  doc->gl_iso_view=(gint)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_frame4_iso));

  /* Get text entry data */
  
  /* Get snap data */
  doc->gl_snap_width=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_frame2_entry[0])),NULL);
  doc->gl_snap_height=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_frame2_entry[1])),NULL);
  doc->gl_snap_angle=M_PI/180*a;
  doc->gl_snap_basex=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_frame2_entry[3])),NULL);
  doc->gl_snap_basey=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_frame2_entry[4])),NULL);

  /* Get grid data */
  doc->gl_grid_x=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dia_frame3_spin[0]));
  doc->gl_grid_y=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dia_frame3_spin[1]));

  /* Get iso data */
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_frame4_radio[0])))
     doc->gl_iso_type=ISOMETRIC_LEFT;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_frame4_radio[1])))
     doc->gl_iso_type=ISOMETRIC_TOP;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_frame4_radio[2])))
     doc->gl_iso_type=ISOMETRIC_RIGHT;

  /* Redraw grid */
  draw_grid(doc);

  /* Set togglebuttons in toolbar */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(doc->widgets.toolbar_ortho), doc->gl_ortho);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(doc->widgets.toolbar_snap), doc->gl_snap);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(doc->widgets.toolbar_grid), doc->gl_grid);  

  return TRUE;
}

static gint drawaids_cancel(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_window);
  menus_set_sensitive("/Assist/Drawing Aids...", TRUE);
  return TRUE;
}

static gint drawaids_ok(GtkWidget *widget, gpointer data)
{
  if(drawaids_get_dialog(grab_document))
     {
       gtk_widget_hide(dia_window);
       menus_set_sensitive("/Assist/Drawing Aids...", TRUE);
     }
  return TRUE;
}

void dia_show_drawaids(GtkWidget *widget, gpointer data)
{
  grab_document=get_document();
  if(grab_document!=NULL)
    {
      menus_set_sensitive("/Assist/Drawing Aids...", FALSE);
      drawaids_set_dialog(grab_document);
      gtk_widget_show(dia_window);
    }
  else
    message(0, _("No active drawing found.\nPlease make a new drawing or open an existing drawing."));

}

void make_dialog_drawaids(void)
{
  gint i;

  /* Create main window */
  dia_window=gtk_dialog_new();

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window), _("oCADis: Drawing Aids"));

  gtk_window_set_modal(GTK_WINDOW(dia_window),TRUE);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(drawaids_cancel),NULL);

  /* Make packing boxes */
  dia_frame1_vbox=gtk_vbox_new(FALSE,0);
  gtk_widget_show(dia_frame1_vbox);
  dia_frame2_vbox=gtk_vbox_new(FALSE,0);
  gtk_widget_show(dia_frame2_vbox);
  dia_frame3_vbox=gtk_vbox_new(FALSE,0);
  gtk_widget_show(dia_frame3_vbox);
  dia_frame4_vbox=gtk_vbox_new(FALSE,0);
  gtk_widget_show(dia_frame4_vbox);

  /* Frame 1 -- Modes */
  dia_frame1_ortho=gtk_check_button_new_with_label( _("Ortho"));
  gtk_widget_show(dia_frame1_ortho);

  dia_frame1_solidfill=gtk_check_button_new_with_label( _("Solid Fill"));
  gtk_widget_show(dia_frame1_solidfill);

  dia_frame1_quicktext=gtk_check_button_new_with_label( _("Quick Text"));
  gtk_widget_show(dia_frame1_quicktext);

  dia_frame1_blip=gtk_check_button_new_with_label( _("Blip"));
  gtk_widget_show(dia_frame1_blip);

  dia_frame1_highlights=gtk_check_button_new_with_label( _("Highlights"));
  gtk_widget_show(dia_frame1_highlights);

  dia_frame1_groups=gtk_check_button_new_with_label( _("Groups"));
  gtk_widget_show(dia_frame1_groups);

  dia_frame1_hatch=gtk_check_button_new_with_label( _("Hatch"));
  gtk_widget_show(dia_frame1_hatch);

  /* Frame 2 -- Snap */
  dia_frame2_table=gtk_table_new(2,6,FALSE);
  gtk_widget_show(dia_frame2_table);
  dia_frame2_snap=gtk_check_button_new_with_label( _("Snap"));
  gtk_widget_show(dia_frame2_snap);

  dia_frame2_text[0]=gtk_label_new( _("X Spacing: "));
  dia_frame2_text[1]=gtk_label_new( _("Y Spacing: "));
  dia_frame2_text[2]=gtk_label_new( _("Snap Angle: "));
  dia_frame2_text[3]=gtk_label_new( _("X Base: "));
  dia_frame2_text[4]=gtk_label_new( _("Y Base: "));

  gtk_table_attach(GTK_TABLE(dia_frame2_table), dia_frame2_snap,0,2,0,1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);

  for(i=0;i<5;i++)
    {
      gtk_misc_set_alignment(GTK_MISC(dia_frame2_text[i]),1.0,0.5);

      dia_frame2_entry[i]=gtk_entry_new();
      gtk_widget_show(dia_frame2_text[i]);
      gtk_widget_show(dia_frame2_entry[i]);

      gtk_table_attach(GTK_TABLE(dia_frame2_table), dia_frame2_text[i],0,1,i+1,i+2,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
      gtk_table_attach(GTK_TABLE(dia_frame2_table), dia_frame2_entry[i],1,2,i+1,i+2,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
    }

  /* Frame 3 -- Grid */
  dia_frame3_table=gtk_table_new(2,3,FALSE);
  gtk_widget_show(dia_frame3_table);
  dia_frame3_grid=gtk_check_button_new_with_label( _("Grid"));
  gtk_widget_show(dia_frame3_grid);

  dia_frame3_text[0]=gtk_label_new( _("X Spacing: "));
  dia_frame3_text[1]=gtk_label_new( _("Y Spacing: "));

  gtk_table_attach(GTK_TABLE(dia_frame3_table), dia_frame3_grid,0,2,0,1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);

  for(i=0;i<2;i++)
    {
      gtk_misc_set_alignment(GTK_MISC(dia_frame3_text[i]),1.0,0.5);

      dia_frame3_adj[i] = (GtkAdjustment *) gtk_adjustment_new (1.0, 1, 655535.0, 1.0,
							      5.0, 0.0);
      dia_frame3_spin[i] = gtk_spin_button_new (dia_frame3_adj[i], 0, 0);

      gtk_widget_show(dia_frame3_text[i]);
      gtk_widget_show(dia_frame3_spin[i]);

      gtk_table_attach(GTK_TABLE(dia_frame3_table), dia_frame3_text[i],0,1,i+1,i+2,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
      gtk_table_attach(GTK_TABLE(dia_frame3_table), dia_frame3_spin[i],1,2,i+1,i+2,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
    }
  
  /* Frame 4 -- Isometric */
  dia_frame4_table=gtk_table_new(3,2,FALSE);
  gtk_widget_show(dia_frame4_table);
  dia_frame4_iso=gtk_check_button_new_with_label( _("On"));
  gtk_widget_show(dia_frame4_iso);

  gtk_table_attach(GTK_TABLE(dia_frame4_table), dia_frame4_iso,0,3,0,1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);

  dia_frame4_radio[0]=gtk_radio_button_new_with_label(NULL, _("Left"));
  dia_frame4_radiogroup=gtk_radio_button_group(GTK_RADIO_BUTTON(dia_frame4_radio[0]));
  dia_frame4_radio[1]=gtk_radio_button_new_with_label(dia_frame4_radiogroup, _("Top"));
  dia_frame4_radio[2]=gtk_radio_button_new_with_label(dia_frame4_radiogroup, _("Right"));

  for(i=0;i<3;i++)
    {
      gtk_widget_show(dia_frame4_radio[i]);
      gtk_table_attach(GTK_TABLE(dia_frame4_table), dia_frame4_radio[i],i,i+1,1,2,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
    }

  /* Make table */
  dia_table=gtk_table_new(3,2,FALSE);
  gtk_widget_show(dia_table);

  /* Make frames */
  dia_frame1=gtk_frame_new( _("Modes"));
  gtk_widget_show(dia_frame1);
  dia_frame2=gtk_frame_new( _("Snap"));
  gtk_widget_show(dia_frame2);
  dia_frame3=gtk_frame_new( _("Grid"));
  gtk_widget_show(dia_frame3);
  dia_frame4=gtk_frame_new( _("Isometric Snap/Grid"));
  gtk_widget_show(dia_frame4);

  /* Attach */ 
  gtk_table_attach(GTK_TABLE(dia_table), dia_frame1,0,1,0,2,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,5,5);
  gtk_table_attach(GTK_TABLE(dia_table), dia_frame2,1,2,0,2,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,5,5);
  gtk_table_attach(GTK_TABLE(dia_table), dia_frame3,2,3,0,1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,5,5);
  gtk_table_attach(GTK_TABLE(dia_table), dia_frame4,2,3,1,2,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,5,5);
  
  gtk_box_pack_start(GTK_BOX(dia_frame1_vbox),dia_frame1_ortho,FALSE,FALSE,0);    
  gtk_box_pack_start(GTK_BOX(dia_frame1_vbox),dia_frame1_solidfill,FALSE,FALSE,0);  
  gtk_box_pack_start(GTK_BOX(dia_frame1_vbox),dia_frame1_quicktext,FALSE,FALSE,0);  
  gtk_box_pack_start(GTK_BOX(dia_frame1_vbox),dia_frame1_blip,FALSE,FALSE,0);  
  gtk_box_pack_start(GTK_BOX(dia_frame1_vbox),dia_frame1_highlights,FALSE,FALSE,0);  
  gtk_box_pack_start(GTK_BOX(dia_frame1_vbox),dia_frame1_groups,FALSE,FALSE,0);  
  gtk_box_pack_start(GTK_BOX(dia_frame1_vbox),dia_frame1_hatch,FALSE,FALSE,0);  

  gtk_box_pack_start(GTK_BOX(dia_frame2_vbox),dia_frame2_table,FALSE,FALSE,0);  
  gtk_box_pack_start(GTK_BOX(dia_frame3_vbox),dia_frame3_table,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(dia_frame4_vbox),dia_frame4_table,FALSE,FALSE,0);

  gtk_container_add(GTK_CONTAINER(dia_frame1),dia_frame1_vbox);
  gtk_container_add(GTK_CONTAINER(dia_frame2),dia_frame2_vbox);
  gtk_container_add(GTK_CONTAINER(dia_frame3),dia_frame3_vbox);
  gtk_container_add(GTK_CONTAINER(dia_frame4),dia_frame4_vbox);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dia_window)->vbox),dia_table,TRUE,TRUE,0);

  /* Make buttons */
  dia_ok_button=gtk_button_new_with_label( _("OK"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_ok_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_ok_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_ok_button);
  gtk_signal_connect(GTK_OBJECT(dia_ok_button),"clicked",
		     GTK_SIGNAL_FUNC(drawaids_ok), NULL); 
  gtk_widget_show (dia_ok_button);

  dia_cancel_button=gtk_button_new_with_label( _("Cancel"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_cancel_button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dia_cancel_button),"clicked",
		     GTK_SIGNAL_FUNC(drawaids_cancel), NULL); 
  gtk_widget_show (dia_cancel_button);
}







