/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include "initmenu.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;
static GtkWidget *dia_box1;
static GtkWidget *dia_box2;
static GtkWidget *dia_hbox;
static GtkWidget *dia_button;
static GtkWidget *dia_separator;
static GtkWidget *dia_table;
static GtkWidget *dia_vscrollbar;
static GtkWidget *dia_text;
static GdkFont *dia_fixed_font;
static gint visible;

/* --  F U N C T I O N S  -- */
void dia_show_gpl(GtkWidget *widget, gpointer data)
{
  if(visible)
    gtk_widget_hide(dia_window);
  else
    gtk_widget_show(dia_window);

  visible=!visible;

  menus_set_sensitive("/Help/GPL", !visible);
}

void make_dialog_gpl(void)
{
  FILE *infile;

  dia_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize (dia_window, 760, 550);
  gtk_window_set_policy (GTK_WINDOW(dia_window), TRUE, TRUE, FALSE);  
  gtk_signal_connect (GTK_OBJECT (dia_window), "delete_event",
                      GTK_SIGNAL_FUNC(dia_show_gpl),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (dia_window), _("oCADis: GNU General Public License"));
  gtk_container_set_border_width (GTK_CONTAINER (dia_window), 0);
  
  
  dia_box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (dia_window), dia_box1);
  gtk_widget_show (dia_box1);
  
  
  dia_box2 = gtk_vbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (dia_box2), 10);
  gtk_box_pack_start (GTK_BOX (dia_box1), dia_box2, TRUE, TRUE, 0);
  gtk_widget_show (dia_box2);
  
  dia_table = gtk_table_new (2, 1, FALSE);
  gtk_box_pack_start (GTK_BOX (dia_box2), dia_table, TRUE, TRUE, 0);
  gtk_widget_show (dia_table);
  
  /* Create the GtkText widget */
  dia_text = gtk_text_new (NULL, NULL);
  gtk_text_set_editable (GTK_TEXT (dia_text), FALSE);
  gtk_table_attach (GTK_TABLE (dia_table), dia_text, 0, 1, 0, 1,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show (dia_text);

  /* Add a vertical scrollbar to the GtkText widget */
  dia_vscrollbar = gtk_vscrollbar_new (GTK_TEXT (dia_text)->vadj);
  gtk_table_attach (GTK_TABLE (dia_table), dia_vscrollbar, 1, 2, 0, 1,
                    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show (dia_vscrollbar);

  /* Load a fixed font */
  dia_fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");

  /* Realizing a widget creates a window for it, ready for us to insert some text */
  gtk_widget_realize (dia_text);

  /* Freeze the text widget, ready for multiple updates */
  gtk_text_freeze (GTK_TEXT (dia_text));
  
  infile = fopen("COPYING", "r");
  
  if (infile) {
    char buffer[1024];
    int nchars;
    
    while (1)
      {
        nchars = fread(buffer, 1, 1024, infile);
        gtk_text_insert (GTK_TEXT (dia_text), dia_fixed_font, NULL,
                         NULL, buffer, nchars);
        
        if (nchars < 1024)
          break;
      }
    
    fclose (infile);
  }
  else
     gtk_text_insert (GTK_TEXT (dia_text), dia_fixed_font, NULL,
                         NULL, _("The file COPYING is not found."), -1);

  /* Thaw the text widget, allowing the updates to become visible */  
  gtk_text_thaw (GTK_TEXT (dia_text));
  
  dia_hbox = gtk_hbutton_box_new ();
  gtk_box_pack_start (GTK_BOX (dia_box2), dia_hbox, FALSE, FALSE, 0);
  gtk_widget_show (dia_hbox);

  dia_separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (dia_box1), dia_separator, FALSE, TRUE, 0);
  gtk_widget_show (dia_separator);

  dia_box2 = gtk_vbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (dia_box2), 10);
  gtk_box_pack_start (GTK_BOX (dia_box1), dia_box2, FALSE, TRUE, 0);
  gtk_widget_show (dia_box2);
  
  dia_button = gtk_button_new_with_label ( _("Close"));
  gtk_signal_connect (GTK_OBJECT (dia_button), "clicked",
                      GTK_SIGNAL_FUNC(dia_show_gpl),
                      NULL);

  gtk_box_pack_start (GTK_BOX (dia_box2), dia_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_button);
  gtk_widget_show (dia_button);

  /* Window is hidden when init */
  visible=FALSE;


}




