/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "layer.h"
#include "color.h"
#include "misc.h"
#include "dia_misc.h"
#include "redraw.h"
#include "dia_message.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static char *colouricon[]={
"18 18 3 1",
"      c None",
".     c #000000000000",
"X     c #111111111111",
"                  ",
" ................ ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" .XXXXXXXXXXXXXX. ",
" ................ ",
"                  "};


static GtkWidget *dia_window;

static GtkWidget *dia_scrwin;
static GtkWidget *dia_list;

static GtkWidget *dia_close_button;
static GtkWidget *dia_add_button;
static GtkWidget *dia_remove_button;
static GtkWidget *dia_copy_button;

static DOCUMENT *grab_document;

static gint focus_row;

static gchar *titles[7]={"Current","Lock","Visible","Name","Colour","Linewidth","Linestyle"};
static gint column_width[7]={50,30,50,200,50,70,50};

/* --  P R O T O T Y P E S  -- */
static void generate_row(DOCUMENT *doc, LAYER *layer, gint row)
{
  gint curr_row;
  gchar *text[7]={NULL,NULL,NULL,NULL,NULL,NULL,NULL};
  GdkPixmap *color_pixmap;
  gchar colour_string[30];

  text[3]=layer->name;
  text[5]=g_strdup_printf("%f", layer->linewidth);
  if(row==-1)
    curr_row=gtk_clist_append(GTK_CLIST(dia_list), text);
  else
    curr_row=gtk_clist_insert(GTK_CLIST(dia_list), row, text);

  if(layer==doc->current_layer)
    gtk_clist_set_pixmap(GTK_CLIST(dia_list),curr_row,0, icon_list[6].pixmap, icon_list[6].mask);
  
  if(!layer->editable)
    gtk_clist_set_pixmap(GTK_CLIST(dia_list),curr_row,1, icon_list[5].pixmap, icon_list[5].mask);
  
  if(layer->visible)
    gtk_clist_set_pixmap(GTK_CLIST(dia_list),curr_row,2, icon_list[4].pixmap, icon_list[4].mask);

  /* Create pixmap for the colour */
  g_snprintf(colour_string,29,"X     c #%04x%04x%04x",layer->colour->colour.red,
	     layer->colour->colour.green,
	     layer->colour->colour.blue);

  colouricon[3]=colour_string;

  color_pixmap=gdk_pixmap_create_from_xpm_d(dia_window->window,
					    NULL,
					    &(dia_window->style->bg[GTK_STATE_NORMAL]),
					    colouricon);    
  gtk_clist_set_pixmap(GTK_CLIST(dia_list),curr_row,4, color_pixmap,NULL);

  gdk_pixmap_unref(color_pixmap);

  gtk_clist_set_pixmap(GTK_CLIST(dia_list),curr_row,6, 
		       layer->linestyle->pic.pixmap, layer->linestyle->pic.mask);

  gtk_clist_set_row_data(GTK_CLIST(dia_list), curr_row, (gpointer)layer);

}

static void generate_all_rows(DOCUMENT *doc)
{
  GList *list;

  gtk_clist_clear(GTK_CLIST(dia_list));
  gtk_clist_freeze(GTK_CLIST(dia_list));

  list=doc->layer_list;

  while(list!=NULL)
    {
      generate_row(doc,((LAYER *)list->data), -1);      
      list=g_list_next(list);
    }

  gtk_clist_thaw(GTK_CLIST(dia_list));
}

static void select_row_callback(GtkWidget *widget,gint row,gint column,
				GdkEventButton *event, gpointer data)
{
  GdkPixmap *color_pixmap;
  gchar colour_string[40];
  LAYER *click_layer;
  gdouble colour[3];
  gchar *tmp;

  focus_row=row;

  click_layer=(LAYER *)gtk_clist_get_row_data(GTK_CLIST(dia_list),row);

  if(column==0)  /* Current layer */
    {
      /* Deselect previous current layer */
      gtk_clist_set_text(GTK_CLIST(dia_list), 
			 gtk_clist_find_row_from_data(GTK_CLIST(dia_list),
						      (gpointer)click_layer->doc->current_layer),
			 0, "");

      /* Select current layer */
      gtk_clist_set_pixmap(GTK_CLIST(dia_list),row,0, icon_list[6].pixmap, icon_list[6].mask);
      click_layer->doc->current_layer=click_layer;
      if(click_layer->doc->type_colour==BYLAYER)
	click_layer->doc->current_colour=click_layer->colour;

      if(click_layer->doc->type_linestyle==BYLAYER)
	click_layer->doc->current_linestyle=click_layer->linestyle;

      /* Set current layer on toolbar */
      gtk_option_menu_set_history(GTK_OPTION_MENU(click_layer->doc->widgets.toolbar_option),row);       
    }
  
  if(column==1)  /* Editable */
    {
      click_layer->editable=!click_layer->editable;

      if(!click_layer->editable)
	gtk_clist_set_pixmap(GTK_CLIST(dia_list),row,1, icon_list[5].pixmap, icon_list[5].mask);
      else
	gtk_clist_set_text(GTK_CLIST(dia_list), row,1,"");
    }

  if(column==2)  /* Visible */
    {
      click_layer->visible=!click_layer->visible;

      if(click_layer->visible)
	gtk_clist_set_pixmap(GTK_CLIST(dia_list),row,2, icon_list[4].pixmap, icon_list[4].mask);
      else
	gtk_clist_set_text(GTK_CLIST(dia_list), row,2,"");
    }

  if(column==3 && event->type==GDK_2BUTTON_PRESS)  /* Layer name */
    {
      tmp=dia_enter_text( _("Change layername"), _("Name: "), click_layer->name);
      
      if(tmp!=NULL)
	{
	  g_free(click_layer->name);

	  click_layer->name=(gchar *)g_malloc(sizeof(gchar)*(strlen(tmp)+1));
	  strcpy(click_layer->name, tmp);

	  gtk_clist_set_text(GTK_CLIST(dia_list),row,3, tmp);

	  make_layer_popmenu(click_layer->doc);
	  gtk_option_menu_set_history(GTK_OPTION_MENU(click_layer->doc->widgets.toolbar_option),
			      gtk_clist_find_row_from_data(GTK_CLIST(dia_list),click_layer->doc->current_layer));

	}
    }

  if(column==4 && event->type==GDK_2BUTTON_PRESS)  /* Colour */
    {
      if(dia_colour_selection(colour))
	{
	  GList *list;

	  g_free(click_layer->colour);
	  
	  click_layer->colour=make_new_userdefined_colour(click_layer->doc,colour[0], colour[1], colour[2]);

	  if(click_layer->doc->type_colour==BYLAYER && click_layer->doc->current_layer==click_layer)
	    click_layer->doc->current_colour=click_layer->colour;
	  
	  /* Create pixmap for the colour */
	  g_snprintf(colour_string,39,"X     c #%04x%04x%04x",click_layer->colour->colour.red,
		     click_layer->colour->colour.green,
		     click_layer->colour->colour.blue);

	  colouricon[3]=colour_string;

	  color_pixmap=gdk_pixmap_create_from_xpm_d(dia_window->window,
	  					    NULL,
	  					    &(dia_window->style->bg[GTK_STATE_NORMAL]),
	  					    colouricon);    
	  gtk_clist_set_pixmap(GTK_CLIST(dia_list),row,4, color_pixmap,NULL);	  
	  gdk_pixmap_unref(color_pixmap);	 


	  /* Change colour of each object on that layer */
	  list=click_layer->object_list;

	  while(list!=NULL)
	    {
	      if( ((OBJECT *)list->data)->type_colour==BYLAYER || 
		  ((OBJECT *)list->data)->type_colour==BYBLOCK)
		{
		  ((OBJECT *)list->data)->colour=click_layer->colour;
		  redraw_object(click_layer->doc, (OBJECT *)list->data);
		}
	      list=g_list_next(list);
	    }
	}
    }

  if(column==5  && event->type==GDK_2BUTTON_PRESS)
    {
       tmp=dia_enter_text( _("Change linewidth of layer"), _("Linewidth: "), 
			   g_strdup_printf("%f", click_layer->linewidth));

      if(tmp!=NULL)
	{
	  GList *list;

	  click_layer->linewidth=g_strtod(tmp, NULL);
	  gtk_clist_set_text(GTK_CLIST(dia_list),row,5, tmp);

	  /* Change linewidth of each object on that layer */
	  list=click_layer->object_list;

	  while(list!=NULL)
	    {
	      if( ((OBJECT *)list->data)->type_linewidth==BYLAYER ||
		  ((OBJECT *)list->data)->type_linewidth==BYBLOCK)
		{
		  ((OBJECT *)list->data)->selected=OBJECT_ERASE;
		  redraw_object(click_layer->doc, (OBJECT *)list->data);
		  ((OBJECT *)list->data)->linewidth=click_layer->linewidth;
		  ((OBJECT *)list->data)->selected=OBJECT_NOT_SEL;
		  redraw_object(click_layer->doc, (OBJECT *)list->data);
		}
	      list=g_list_next(list);
	    }

	  if(click_layer==click_layer->doc->current_layer && click_layer->doc->type_linewidth==BYLAYER)
	    click_layer->doc->current_linewidth=click_layer->linewidth;
	    
	}
    }

  if(column==6  && event->type==GDK_2BUTTON_PRESS)  /* Change linestyle */
    {
      LINESTYLE *style;

      style=dia_get_linestyle(click_layer->doc->linestyle_list);

      if(style!=NULL)
	{
	  GList *list;

	  click_layer->linestyle=style;

	  gtk_clist_set_pixmap(GTK_CLIST(dia_list),row,6, 
			       click_layer->linestyle->pic.pixmap, click_layer->linestyle->pic.mask);

	  /* Change linestyle of each object on that layer */
	  list=click_layer->object_list;

	  while(list!=NULL)
	    {
	      if( ((OBJECT *)list->data)->type_linestyle==BYLAYER ||
		  ((OBJECT *)list->data)->type_linestyle==BYBLOCK)
		{
		  ((OBJECT *)list->data)->selected=OBJECT_ERASE;
		  redraw_object(click_layer->doc, (OBJECT *)list->data);
		  ((OBJECT *)list->data)->linestyle=click_layer->linestyle;
		  ((OBJECT *)list->data)->selected=OBJECT_NOT_SEL;
		  redraw_object(click_layer->doc, (OBJECT *)list->data);
		}
	      list=g_list_next(list);
	    }

	  if(click_layer==click_layer->doc->current_layer && click_layer->doc->type_linestyle==BYLAYER)
	    click_layer->doc->current_linestyle=click_layer->linestyle;

	}
    }

}

static gint layer_close(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_window);
  menus_set_sensitive("/Assist/Format/Layer...", TRUE);
  menus_set_sensitive("/Assist/Layers...", TRUE);
  return TRUE;
}

static gint layer_add(GtkWidget *widget, gpointer data)
{
  generate_row(grab_document, layer_new(grab_document,focus_row+1, grab_document->linestyle_list->data,
					0,0,0,0, _("Layer"), TRUE, TRUE), focus_row+1);

  make_layer_popmenu(grab_document);
  gtk_option_menu_set_history(GTK_OPTION_MENU(grab_document->widgets.toolbar_option),
			      gtk_clist_find_row_from_data(GTK_CLIST(dia_list),grab_document->current_layer));
  
  return TRUE;
}

static gint layer_copy(GtkWidget *widget, gpointer data)
{
  make_layer_popmenu(grab_document);
  return TRUE;
}

static gint layer_remove(GtkWidget *widget, gpointer data)
{
  LAYER *layer;
  GList *list;

  layer=gtk_clist_get_row_data(GTK_CLIST(dia_list),focus_row);
  list=g_list_find(layer->doc->layer_list, (gpointer)layer);

  /* Check if this is the only layer */
  if(g_list_next(list)==NULL && g_list_previous(list)==NULL)
    message(ICON_WARNING, _("This is the last layer\nand can therefor not be removed."));
  else 
    {

      if(layer->object_list!=NULL)
	{
	  if(alert(ICON_QUESTION, _("Erase objects in layer"), _("Yes"), _("No"),NULL,
		   _("The layer \"%s\" contains objects.The objects in it\nwill be erased.\nAre you sure you want to remove this layer?"), layer->name)!=1)
	    return TRUE;
	}

      /* Is this the current layer? */
      if(layer->doc->current_layer==layer)
	{
	  gint trow;

	  if(g_list_previous(list)!=NULL)
	    layer->doc->current_layer=(LAYER *)g_list_previous(list)->data;
	  else
	    layer->doc->current_layer=(LAYER *)g_list_next(list)->data;

	  trow= gtk_clist_find_row_from_data(GTK_CLIST(dia_list),layer->doc->current_layer);
	  gtk_clist_set_pixmap(GTK_CLIST(dia_list), trow ,0, 
			       icon_list[6].pixmap, icon_list[6].mask);	  

	      
	}

      /* Remove it from the list and layer_list */
      gtk_clist_remove(GTK_CLIST(dia_list), focus_row);
      layer->doc->layer_list=g_list_remove(layer->doc->layer_list, (gpointer)layer);

      make_layer_popmenu(grab_document);
      gtk_option_menu_set_history(GTK_OPTION_MENU(grab_document->widgets.toolbar_option),
				  gtk_clist_find_row_from_data(GTK_CLIST(dia_list),grab_document->current_layer));
    }

  focus_row=GTK_CLIST(dia_list)->focus_row;  
  return TRUE;
}

void dia_show_layer(GtkWidget *widget, gpointer data)
{
  grab_document=get_document();
  if(grab_document!=NULL)
    {
      menus_set_sensitive("/Assist/Format/Layer...", FALSE);
      menus_set_sensitive("/Assist/Layers...", FALSE);
      focus_row=0;
      generate_all_rows(grab_document);
      gtk_widget_show(dia_window);
    }
  else
    message(0, _("No active drawing found.\nPlease make a new drawing or open an existing drawing."));

}

void make_dialog_layer(void)
{
  gint i;

  /* Create main window */
  dia_window=gtk_dialog_new();
  gtk_widget_realize(dia_window);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window), _("oCADis: Layers"));
  gtk_widget_set_usize(GTK_WIDGET( dia_window),600,400);

  gtk_window_set_modal(GTK_WINDOW(dia_window),TRUE);
  
  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(layer_close),NULL);

  /* Create scrolled window */
  dia_scrwin=gtk_scrolled_window_new(NULL,NULL);
  gtk_widget_show(dia_scrwin);

  /* Create list */
  dia_list=gtk_clist_new_with_titles(7, titles);
  gtk_clist_set_row_height(GTK_CLIST(dia_list),18);
  gtk_clist_column_titles_passive(GTK_CLIST(dia_list));
  gtk_clist_column_titles_show(GTK_CLIST(dia_list));
  gtk_clist_set_selection_mode(GTK_CLIST(dia_list), GTK_SELECTION_BROWSE);

  gtk_signal_connect(GTK_OBJECT(dia_list),"select_row",
		     GTK_SIGNAL_FUNC(select_row_callback),
		     NULL);

  gtk_clist_set_column_justification(GTK_CLIST(dia_list),0,GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),1,GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),2,GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),4,GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(dia_list),5,GTK_JUSTIFY_RIGHT);

  /* Set column width */
  for(i=0;i<7;i++)
    gtk_clist_set_column_width(GTK_CLIST(dia_list), i, column_width[i]);

  gtk_widget_show(dia_list);

  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(dia_scrwin),dia_list);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->vbox),
		      dia_scrwin, TRUE, TRUE, 0);


  /* Make buttons */
  dia_add_button=gtk_button_new_with_label( _("Add"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_add_button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dia_add_button),"clicked",
		     GTK_SIGNAL_FUNC(layer_add), NULL); 
  gtk_widget_show(dia_add_button);
 
  dia_copy_button=gtk_button_new_with_label( _("Copy"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_copy_button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dia_copy_button),"clicked",
		     GTK_SIGNAL_FUNC(layer_copy), NULL); 
  gtk_widget_show(dia_copy_button);

  dia_remove_button=gtk_button_new_with_label( _("Remove"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_remove_button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dia_remove_button),"clicked",
		     GTK_SIGNAL_FUNC(layer_remove), NULL); 
  gtk_widget_show(dia_remove_button);

  dia_close_button=gtk_button_new_with_label( _("Close"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_close_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_close_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_close_button);
  gtk_signal_connect(GTK_OBJECT(dia_close_button),"clicked",
		     GTK_SIGNAL_FUNC(layer_close), NULL); 
  gtk_widget_show (dia_close_button);



}






