/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <strings.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "misc.h"
#include "dia_misc.h"
#include "redraw.h"
#include "dia_message.h"
#include "linestyle.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

typedef struct {
  GtkWidget *list_vbox;
  GtkWidget *scrolledwindow;
  GtkWidget *clist;
  GtkWidget *name_label;
  GtkWidget *scale_label;
  GtkWidget *desc_label;
  GtkWidget *picture_label;
  GtkWidget *frame;
  GtkWidget *vbox5;
  GtkWidget *hbox1;
  GtkWidget *table4;
  GtkWidget *label17;
  GtkWidget *name_entry;
  GtkWidget *table5;
  GtkWidget *label18;
  GtkWidget *scale_entry;
  GtkWidget *table3;
  GtkWidget *label16;
  GtkWidget *desc_entry;
  GtkWidget *hbox2;
  GtkWidget *hbuttonbox4;
  GtkWidget *update_button;
  GtkWidget *hbuttonbox2;
  GtkWidget *modify_button;
  GtkWidget *add_button;
  GtkWidget *remove_button;
} _LINESTYLE;

_LINESTYLE *dia_linestyle=NULL;

/* --  P R O T O T Y P E S  -- */
static LINESTYLE *new_style;

static void update_new(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  if(object->linestyle==new_style)
    {
      object->selected=OBJECT_NOT_SEL;
      redraw_object(doc, object);
    }
}

static void update_objects_new(DOCUMENT *doc, LINESTYLE *style)
{
  new_style=style;
  foreach_object(doc, update_new);
}

static void update_old(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  if(object->linestyle==new_style)
    {
      object->selected=OBJECT_ERASE;
      redraw_object(doc, object);
    }
}

static void update_objects_old(DOCUMENT *doc, LINESTYLE *style)
{
  new_style=style;
  foreach_object(doc, update_old);
}


static gchar *make_nice_description(gint no, gdouble *pattern)
{
  gint i;
  gchar *text;

  text=g_strdup_printf("%.3f", pattern[0]);

  for(i=1;i<no;i++)
    {	  
      gchar *tmp;
      tmp=text;
      text=g_strdup_printf("%s,%.3f",text, pattern[i]);
      g_free(tmp);
    }

  return text;
}

static void select_row_callback(GtkWidget *widget,gint row,gint column,
				GdkEventButton *event, gpointer data)
{
  DOCUMENT *doc;
  LINESTYLE *linestyle;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_linestyle->clist));

  if(doc==NULL)
    return;

  linestyle=gtk_clist_get_row_data(GTK_CLIST(dia_linestyle->clist), row);

  if(linestyle==NULL)
    return;

  /* Update entries */
  gtk_entry_set_text(GTK_ENTRY(dia_linestyle->name_entry), linestyle->name); 
  gtk_entry_set_text(GTK_ENTRY(dia_linestyle->scale_entry), g_strdup_printf("%.2f", linestyle->dash_length)
); 
  if(linestyle->type==LINE_NOT_SOLID)
    gtk_entry_set_text(GTK_ENTRY(dia_linestyle->desc_entry), make_nice_description(linestyle->dash_no, linestyle->dash_dlist));
  else
    gtk_entry_set_text(GTK_ENTRY(dia_linestyle->desc_entry), "");

}

static void linestyle_update(DOCUMENT *doc)
{
  GList *list;
  LINESTYLE *linestyle;
  gchar *text[4]={NULL,"\0","\0",NULL};
  gint row;

  if(doc==NULL)
    return;
  if(dia_linestyle==NULL)
    return;
  if(dia_linestyle->clist==NULL)
    return;

  /* Create list for linestyle */
  gtk_clist_clear(GTK_CLIST(dia_linestyle->clist));
  gtk_clist_freeze(GTK_CLIST(dia_linestyle->clist));
  
  list=g_list_first(doc->linestyle_list);

  while(list!=NULL)
    {
      linestyle=(LINESTYLE *)list->data;

      text[0]=linestyle->name;
      text[1]=g_strdup_printf("%.2f", linestyle->dash_length);

      if(linestyle->type==LINE_NOT_SOLID)
	text[2]=make_nice_description(linestyle->dash_no, linestyle->dash_dlist);
      else
	text[2]=g_strdup("\0");

      row=gtk_clist_append(GTK_CLIST(dia_linestyle->clist), text);
    
      gtk_clist_set_pixmap(GTK_CLIST(dia_linestyle->clist), row, 3, 
			   linestyle->pic.pixmap, linestyle->pic.mask);

      gtk_clist_set_row_data(GTK_CLIST(dia_linestyle->clist), row, (gpointer)linestyle);

      list=g_list_next(list);
    }

  /* Set user data */
  gtk_object_set_user_data(GTK_OBJECT(dia_linestyle->clist), (gpointer)doc);

  /* Mark first row */
  gtk_clist_select_row(GTK_CLIST(dia_linestyle->clist), 0, 0);

  /* Unfreeze list */
  gtk_clist_thaw(GTK_CLIST(dia_linestyle->clist));
}

static gint linestyle_update_button(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_linestyle->clist));
  
  if(doc!=NULL)
    linestyle_update(doc);

  return TRUE;
}


static gint linestyle_modify_button(GtkWidget *widget, gpointer data)
{
  gint i;
  gchar **desc;
  GList   *items;
  DOCUMENT *doc;
  LINESTYLE *linestyle;
  
  doc=gtk_object_get_user_data(GTK_OBJECT(dia_linestyle->clist));
  
  if(doc==NULL)
    return TRUE;

  /* Get current selected linestyle */
  items=g_list_first(GTK_CLIST(dia_linestyle->clist)->selection);

  if(items!=NULL)
    {
      linestyle=(LINESTYLE *)gtk_clist_get_row_data(GTK_CLIST(dia_linestyle->clist), GPOINTER_TO_INT(items->data));

      /* Erase old objects that uses this soon to be modified linestyle */
      update_objects_old(doc, linestyle);

      /* New linstyle name */
      g_free(linestyle->name);
      linestyle->name=g_strdup(gtk_entry_get_text(GTK_ENTRY(dia_linestyle->name_entry)));

      /* Scale */
      linestyle->dash_length=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_linestyle->scale_entry)),NULL);

      /* Description */
      if(linestyle->type==LINE_NOT_SOLID && linestyle->dash_dlist!=NULL)
	g_free(linestyle->dash_dlist);

      desc=g_strsplit(gtk_entry_get_text(GTK_ENTRY(dia_linestyle->desc_entry)),",",30);

      /* How many */
      for(i=0;desc[i]!=NULL;i++);

      if(i<=1) /* This is a solid line */
	{
	  linestyle->type=LINE_SOLID;       
	  linestyle->dash_list=NULL;
	}
      else
	{
	  /* Make a dash list for that many */
	  linestyle->dash_dlist=(gdouble *)g_malloc(sizeof(gdouble)*i);
	  linestyle->dash_list=(gchar *)g_malloc(sizeof(gchar)*i);

	  linestyle->type=LINE_NOT_SOLID;           

	  /* Translate user input to gchar */
	  for(i=0;desc[i]!=NULL;i++)
	    linestyle->dash_dlist[i]=g_strtod(desc[i],NULL);
	  
	  linestyle->dash_no=i;
	}
      g_strfreev(desc);

      make_picture_of_line(linestyle->doc, linestyle);

      /* Redraw objects that uses this modified linestyle */
      update_objects_new(doc, linestyle);

      /* Update */
      linestyle_update(doc);

      /* Mark previous row */
      gtk_clist_select_row(GTK_CLIST(dia_linestyle->clist), GPOINTER_TO_INT(items->data), 0);

      /* Update linestyle popmenu */
      make_linestyle_popmenu(doc);
    }

  return TRUE;
}

static gint linestyle_add_button(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;
  gchar **desc;
  gint no=0,i, type=LINE_SOLID;
  gdouble *pattern=NULL;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_linestyle->clist));
  
  if(doc==NULL)
    return TRUE;

  if(linestyle_find(doc, gtk_entry_get_text(GTK_ENTRY(dia_linestyle->name_entry)))==NULL)
    {
      desc=g_strsplit(gtk_entry_get_text(GTK_ENTRY(dia_linestyle->desc_entry)),",",30);
      
      /* How many */
      for(i=0;desc[i]!=NULL;i++);
      
      if(i>1)
	{           
	  type=LINE_NOT_SOLID;
	  pattern=g_new(gdouble, i);
	  
	  /* Translate user input to gdouble */
	  for(i=0;desc[i]!=NULL;i++)
	    pattern[i]=g_strtod(desc[i],NULL);
	  
	  no=i;          
	}
      g_strfreev(desc);
      
      linestyle_new(doc, -1, gtk_entry_get_text(GTK_ENTRY(dia_linestyle->name_entry)), type, 
		    g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_linestyle->scale_entry)),NULL), no, pattern);
      
      linestyle_update(doc);

      /* Update linestyle popmenu */
      make_linestyle_popmenu(doc);
    }
  else
    message(ICON_WARNING, _("This linestyle does already exist.\nPlease select another name."));
  
  return TRUE;
}


static gint linestyle_remove_button(GtkWidget *widget, gpointer data)
{
  GList *items;
  DOCUMENT *doc;
  LINESTYLE *linestyle;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_linestyle->clist));
  
  if(doc==NULL)
    return TRUE;

  /* Get current selected linestyle */
  items=g_list_first(GTK_CLIST(dia_linestyle->clist)->selection);

  if(items!=NULL)
    {
      linestyle=(LINESTYLE *)gtk_clist_get_row_data(GTK_CLIST(dia_linestyle->clist), GPOINTER_TO_INT(items->data));

      g_free(linestyle->name);

      if(linestyle->type==LINE_NOT_SOLID && linestyle->dash_dlist!=NULL)
	g_free(linestyle->dash_dlist);      

      /* Remove linestyle from list in document */
      doc->linestyle_list=g_list_remove(doc->linestyle_list, (gpointer)linestyle);

      /* Update */
      linestyle_update(doc);

      /* Mark previous row */
      gtk_clist_select_row(GTK_CLIST(dia_linestyle->clist), MIN(0, GPOINTER_TO_INT(items->data)-1), 0);

      /* Update linestyle popmenu */
      make_linestyle_popmenu(doc);
    }

  return TRUE;
}

/* Callback for document list */
static void doc_select_row_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
  DOCUMENT *doc;

  /* Find selected document */
  doc=(DOCUMENT *)gtk_clist_get_row_data(GTK_CLIST(data), row);

  /* Update the dialog */
  if(doc!=NULL)
    linestyle_update(doc);
}

GtkWidget *linestyle_create_notebook(GtkWidget *list)
{
  /* Create struct */
  dia_linestyle=g_new(_LINESTYLE,1);

  dia_linestyle->list_vbox=gtk_vbox_new (FALSE, 0);
  gtk_widget_show(dia_linestyle->list_vbox);
  gtk_container_set_border_width(GTK_CONTAINER(dia_linestyle->list_vbox), 4);

  dia_linestyle->scrolledwindow=gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(dia_linestyle->scrolledwindow);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->list_vbox), dia_linestyle->scrolledwindow, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dia_linestyle->scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  /* Create list for linestyles */
  dia_linestyle->clist=gtk_clist_new(4);
  gtk_widget_show(dia_linestyle->clist);
  gtk_container_add(GTK_CONTAINER(dia_linestyle->scrolledwindow), dia_linestyle->clist);
  gtk_clist_set_column_width(GTK_CLIST(dia_linestyle->clist), 0, 204);
  gtk_clist_set_column_width(GTK_CLIST(dia_linestyle->clist), 1, 61);
  gtk_clist_set_column_width(GTK_CLIST(dia_linestyle->clist), 2, 115);
  gtk_clist_set_column_width(GTK_CLIST(dia_linestyle->clist), 3, 80);
  gtk_clist_set_selection_mode(GTK_CLIST(dia_linestyle->clist), GTK_SELECTION_BROWSE);
  gtk_clist_column_titles_show(GTK_CLIST(dia_linestyle->clist));

  /* Add signals */
  gtk_signal_connect(GTK_OBJECT(dia_linestyle->clist), "select_row", GTK_SIGNAL_FUNC(select_row_callback), dia_linestyle->clist);

  dia_linestyle->name_label=gtk_label_new (_("Name"));
  gtk_widget_show(dia_linestyle->name_label);
  gtk_clist_set_column_widget(GTK_CLIST(dia_linestyle->clist), 0, dia_linestyle->name_label);
  gtk_label_set_justify(GTK_LABEL(dia_linestyle->name_label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(dia_linestyle->name_label), 0.0, 0.5);

  dia_linestyle->scale_label=gtk_label_new (_("Scale"));
  gtk_widget_show(dia_linestyle->scale_label);
  gtk_clist_set_column_widget(GTK_CLIST(dia_linestyle->clist), 1, dia_linestyle->scale_label);
  gtk_misc_set_alignment(GTK_MISC(dia_linestyle->scale_label), 1, 0.5);

  dia_linestyle->desc_label=gtk_label_new (_("Description"));
  gtk_widget_show(dia_linestyle->desc_label);
  gtk_clist_set_column_widget(GTK_CLIST(dia_linestyle->clist), 2, dia_linestyle->desc_label);
  gtk_label_set_justify(GTK_LABEL(dia_linestyle->desc_label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(dia_linestyle->desc_label), 0.0, 0.5);

  dia_linestyle->picture_label=gtk_label_new (_("Picture"));
  gtk_widget_show(dia_linestyle->picture_label);
  gtk_clist_set_column_widget(GTK_CLIST(dia_linestyle->clist), 3, dia_linestyle->picture_label);

  /* Deactivates column titles */
  gtk_clist_column_titles_passive(GTK_CLIST (dia_linestyle->clist));

  dia_linestyle->frame=gtk_frame_new(_("Properties"));
  gtk_widget_show(dia_linestyle->frame);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->list_vbox), dia_linestyle->frame, FALSE, FALSE, 3);

  dia_linestyle->vbox5=gtk_vbox_new (FALSE, 0);
  gtk_widget_show(dia_linestyle->vbox5);
  gtk_container_add(GTK_CONTAINER(dia_linestyle->frame), dia_linestyle->vbox5);
  gtk_container_set_border_width(GTK_CONTAINER(dia_linestyle->vbox5), 3);

  dia_linestyle->hbox1=gtk_hbox_new(FALSE, 17);
  gtk_widget_show(dia_linestyle->hbox1);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->vbox5), dia_linestyle->hbox1, FALSE, FALSE, 3);

  dia_linestyle->table4=gtk_table_new (1, 2, FALSE);
  gtk_widget_show(dia_linestyle->table4);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->hbox1), dia_linestyle->table4, TRUE, TRUE, 0);

  dia_linestyle->label17=gtk_label_new (_("Name: "));
  gtk_widget_show(dia_linestyle->label17);
  gtk_table_attach(GTK_TABLE(dia_linestyle->table4), dia_linestyle->label17, 0, 1, 0, 1,
		   (GtkAttachOptions) (0),
		   (GtkAttachOptions) (0), 0, 0);

  dia_linestyle->name_entry=gtk_entry_new();
  gtk_widget_show(dia_linestyle->name_entry);
  gtk_table_attach(GTK_TABLE(dia_linestyle->table4), dia_linestyle->name_entry, 1, 2, 0, 1,
		   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		   (GtkAttachOptions) (0), 0, 0);
  
  dia_linestyle->table5=gtk_table_new(1, 2, FALSE);
  gtk_widget_show(dia_linestyle->table5);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->hbox1), dia_linestyle->table5, TRUE, TRUE, 0);

  dia_linestyle->label18=gtk_label_new (_("Scale: "));
  gtk_widget_show(dia_linestyle->label18);
  gtk_table_attach(GTK_TABLE(dia_linestyle->table5), dia_linestyle->label18, 0, 1, 0, 1,
                   (GtkAttachOptions) (0),
                   (GtkAttachOptions) (0), 0, 0);

  dia_linestyle->scale_entry=gtk_entry_new();
  gtk_widget_show(dia_linestyle->scale_entry);
  gtk_table_attach(GTK_TABLE(dia_linestyle->table5), dia_linestyle->scale_entry, 1, 2, 0, 1,
		   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		   (GtkAttachOptions) (0), 0, 0);

  dia_linestyle->table3=gtk_table_new (1, 2, FALSE);
  gtk_widget_show(dia_linestyle->table3);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->vbox5), dia_linestyle->table3, TRUE, TRUE, 0);

  dia_linestyle->label16=gtk_label_new (_("Description: "));
  gtk_widget_show(dia_linestyle->label16);
  gtk_table_attach(GTK_TABLE(dia_linestyle->table3), dia_linestyle->label16, 0, 1, 0, 1,
                   (GtkAttachOptions) (0),
                   (GtkAttachOptions) (0), 0, 0);

  dia_linestyle->desc_entry=gtk_entry_new();
  gtk_widget_show(dia_linestyle->desc_entry);
  gtk_table_attach(GTK_TABLE(dia_linestyle->table3), dia_linestyle->desc_entry, 1, 2, 0, 1,
		   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		   (GtkAttachOptions) (0), 0, 0);

  dia_linestyle->hbox2=gtk_hbox_new (FALSE, 0);
  gtk_widget_show(dia_linestyle->hbox2);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->list_vbox), dia_linestyle->hbox2, FALSE, FALSE, 0);

  dia_linestyle->hbuttonbox4=gtk_hbutton_box_new ();
  gtk_widget_show(dia_linestyle->hbuttonbox4);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->hbox2), dia_linestyle->hbuttonbox4, TRUE, TRUE, 0);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(dia_linestyle->hbuttonbox4), GTK_BUTTONBOX_START);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(dia_linestyle->hbuttonbox4), 0);
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(dia_linestyle->hbuttonbox4), 114, -1);

  dia_linestyle->update_button=gtk_button_new_with_label(_("Update"));
  gtk_signal_connect(GTK_OBJECT(dia_linestyle->update_button),"clicked",
		     GTK_SIGNAL_FUNC(linestyle_update_button), NULL); 
  gtk_widget_show(dia_linestyle->update_button);
  gtk_container_add(GTK_CONTAINER(dia_linestyle->hbuttonbox4), dia_linestyle->update_button);
  GTK_WIDGET_SET_FLAGS(dia_linestyle->update_button, GTK_CAN_DEFAULT);
  

  dia_linestyle->hbuttonbox2=gtk_hbutton_box_new();
  gtk_widget_show(dia_linestyle->hbuttonbox2);
  gtk_box_pack_start(GTK_BOX(dia_linestyle->hbox2), dia_linestyle->hbuttonbox2, TRUE, TRUE, 0);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(dia_linestyle->hbuttonbox2), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(dia_linestyle->hbuttonbox2), 0);
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(dia_linestyle->hbuttonbox2), 114, 27);

  dia_linestyle->modify_button=gtk_button_new_with_label(_("Modify"));
  gtk_widget_show(dia_linestyle->modify_button);
  gtk_container_add(GTK_CONTAINER(dia_linestyle->hbuttonbox2), dia_linestyle->modify_button);
  GTK_WIDGET_SET_FLAGS(dia_linestyle->modify_button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(dia_linestyle->modify_button),"clicked",
		     GTK_SIGNAL_FUNC(linestyle_modify_button), NULL); 

  dia_linestyle->add_button=gtk_button_new_with_label(_("Add"));
  gtk_widget_show(dia_linestyle->add_button);
  gtk_container_add(GTK_CONTAINER(dia_linestyle->hbuttonbox2), dia_linestyle->add_button);
  GTK_WIDGET_SET_FLAGS(dia_linestyle->add_button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(dia_linestyle->add_button),"clicked",
		     GTK_SIGNAL_FUNC(linestyle_add_button), NULL); 

  dia_linestyle->remove_button=gtk_button_new_with_label (_("Remove"));
  gtk_widget_show(dia_linestyle->remove_button);
  gtk_container_add(GTK_CONTAINER(dia_linestyle->hbuttonbox2), dia_linestyle->remove_button);
  GTK_WIDGET_SET_FLAGS(dia_linestyle-> remove_button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(dia_linestyle->remove_button),"clicked",
		     GTK_SIGNAL_FUNC(linestyle_remove_button), NULL); 

  /* Attach signal to list so we know when user selects another document */
  gtk_signal_connect(GTK_OBJECT(list), "select_row", GTK_SIGNAL_FUNC(doc_select_row_callback), list);
  
  return dia_linestyle->list_vbox;
}




