/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "initmenu.h"
#include "layer.h"
#include "misc.h"
#include "dia_message.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;
static GtkWidget *dia_table;
static GtkWidget *dia_scrwin;
static GtkWidget *dia_tree;
static GtkWidget *dia_vbox;
static GtkWidget *dia_frame[3];

static GtkWidget *dia_obj_vbox;
static GtkWidget *dia_obj_table;
static GtkWidget *dia_obj_label[9];
static GtkWidget *dia_obj_entry[9];
static GtkWidget *dia_objspec_vbox[4];

static GtkWidget *dia_obj_line_table;
static GtkWidget *dia_obj_line_label[8];
static GtkWidget *dia_obj_line_entry[8];

static GtkWidget *dia_obj_circle_table;
static GtkWidget *dia_obj_circle_label[5];
static GtkWidget *dia_obj_circle_entry[5];

static GtkWidget *dia_close_button;

/* --  P R O T O T Y P E S  -- */
static void list_tree_deselect(GtkWidget *item, gint frame_no)
{
  gtk_widget_hide(dia_frame[frame_no]);
}

static void list_tree_document(GtkWidget *item, DOCUMENT *doc)
{
  //  g_warning("Treesel document: %s", doc->drawing);

  gtk_widget_show(dia_frame[0]);
}


static void list_tree_layer(GtkWidget *item, LAYER *layer)
{
  g_warning("Treesel layer: %s", layer->name);

  gtk_widget_show(dia_frame[1]);
}


static void list_tree_object(GtkWidget *item, OBJECT *object)
{
  /* Fix the none specific data */
  //  gtk_entry_set_text(GTK_ENTRY(dia_obj_entry[0]), object->layer->doc->drawing);
  gtk_entry_set_text(GTK_ENTRY(dia_obj_entry[1]), object->layer->name);
  
  /* Hide old object specific vboxes */
  gtk_widget_hide(dia_objspec_vbox[0]);
  gtk_widget_hide(dia_objspec_vbox[1]);
  gtk_widget_hide(dia_objspec_vbox[2]);
  gtk_widget_hide(dia_objspec_vbox[3]);

  switch(object->type)
    {
    case TYPE_LINE:
      gtk_entry_set_text(GTK_ENTRY(dia_obj_entry[3]), "Line");
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[0]),
			 g_strdup_printf("%f", object->data.line.p1.x));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[1]),
			 g_strdup_printf("%f", object->data.line.p1.y));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[2]),
			 g_strdup_printf("%f", object->data.line.p2.x));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[3]),
			 g_strdup_printf("%f", object->data.line.p2.y));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[4]),
			 g_strdup_printf("%f", dist(object->data.line.p1.x, object->data.line.p1.y,
						    object->data.line.p2.x, object->data.line.p2.y)));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[5]),
			 g_strdup_printf("%f", get_angle(object->data.line.p1.x, object->data.line.p1.y,
							 object->data.line.p2.x, object->data.line.p2.y)));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[6]),
			 g_strdup_printf("%f", object->data.line.p2.x-object->data.line.p1.x));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_line_entry[7]),
			 g_strdup_printf("%f", object->data.line.p2.y-object->data.line.p1.y));
      gtk_widget_show(dia_objspec_vbox[0]);
      break;
    case TYPE_POLYLINE:
      gtk_entry_set_text(GTK_ENTRY(dia_obj_entry[3]), "Polyline");
      break;
    case TYPE_CIRCLE:
      gtk_entry_set_text(GTK_ENTRY(dia_obj_entry[3]), "Circle");
      gtk_entry_set_text(GTK_ENTRY(dia_obj_circle_entry[0]),
			 g_strdup_printf("%f", object->data.circle.centrum.x));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_circle_entry[1]),
			 g_strdup_printf("%f", object->data.circle.centrum.y));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_circle_entry[2]),
			 g_strdup_printf("%f", object->data.circle.radius));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_circle_entry[3]),
			 g_strdup_printf("%f", (gdouble)(object->data.circle.radius*2*M_PI)));
      gtk_entry_set_text(GTK_ENTRY(dia_obj_circle_entry[4]),
			 g_strdup_printf("%f", (gdouble)(object->data.circle.radius*object->data.circle.radius*M_PI)));
      gtk_widget_show(dia_objspec_vbox[2]);
      break;
    case TYPE_ARC:
      gtk_entry_set_text(GTK_ENTRY(dia_obj_entry[3]), "Arc");
      break;
    }
  gtk_widget_show(dia_frame[2]);
}

static void make_tree_nodes(DOCUMENT *doc)
{
  OBJECT *object;
  GtkWidget *layersubtree, *item=NULL;
  GtkWidget *docsubtree;
  GList *layer_list, *object_list;

  /* Remove old trees */
  gtk_tree_clear_items(GTK_TREE(dia_tree),0,-1);

  /* Create rootnode in tree (document's name) */
  //  item=gtk_tree_item_new_with_label(doc->drawing);
  gtk_signal_connect (GTK_OBJECT(item), "select",
		      GTK_SIGNAL_FUNC(list_tree_document), (gpointer)doc);
  gtk_signal_connect (GTK_OBJECT(item), "deselect",
		      GTK_SIGNAL_FUNC(list_tree_deselect), (gpointer)0);

  gtk_tree_append(GTK_TREE(dia_tree),item);
  gtk_widget_show(item); 

  docsubtree=gtk_tree_new();
  gtk_tree_item_set_subtree(GTK_TREE_ITEM(item),docsubtree);

  /* Make layer data and object data within layer */
  layer_list=doc->layer_list;

  while(layer_list!=NULL)
    {
      object_list=((LAYER *)layer_list->data)->object_list;

      /* Make item for this layer */
      item=gtk_tree_item_new_with_label(((LAYER *)layer_list->data)->name);
      gtk_signal_connect (GTK_OBJECT(item), "select",
			  GTK_SIGNAL_FUNC(list_tree_layer), (gpointer)layer_list->data);
      gtk_signal_connect (GTK_OBJECT(item), "deselect",
			  GTK_SIGNAL_FUNC(list_tree_deselect), (gpointer)1);
      gtk_tree_append(GTK_TREE(docsubtree),item);
      gtk_widget_show(item); 

      if(object_list!=NULL)
	{
	  layersubtree=gtk_tree_new();
	  gtk_tree_item_set_subtree(GTK_TREE_ITEM(item),layersubtree);

	  while(object_list!=NULL)
	    {

	      object=(OBJECT *)object_list->data;

	      if(object->selected==OBJECT_SEL)
		{
		  switch(object->type)
		    {
		    case TYPE_LINE:
		      item=gtk_tree_item_new_with_label("Line");
		      break;
		    case TYPE_POLYLINE:
		      item=gtk_tree_item_new_with_label("Polyline");
		      break;
		    case TYPE_CIRCLE:
		      item=gtk_tree_item_new_with_label("Circle");
		      break;
		    case TYPE_ARC:
		      item=gtk_tree_item_new_with_label("Arc");
		      break;
		    }

		  gtk_signal_connect (GTK_OBJECT(item), "select",
				      GTK_SIGNAL_FUNC(list_tree_object), (gpointer)object_list->data);
		  gtk_signal_connect (GTK_OBJECT(item), "deselect",
				      GTK_SIGNAL_FUNC(list_tree_deselect), (gpointer)2);
		  gtk_tree_append(GTK_TREE(layersubtree),item);
		  gtk_widget_show(item); 
		}

	      object_list=g_list_next(object_list);
	    }
	}
      layer_list=g_list_next(layer_list);
    }


}

static gint list_close(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_window);
  menus_set_sensitive("/Assist/Inguiry/List", TRUE);
  return TRUE;
}

void dia_show_list(DOCUMENT *doc)
{
  /* Make sure frames are hidden */
  gtk_widget_hide(dia_frame[0]);
  gtk_widget_hide(dia_frame[1]);
  gtk_widget_hide(dia_frame[2]);

  /* Hide old object specific vboxes */
  gtk_widget_hide(dia_objspec_vbox[0]);
  gtk_widget_hide(dia_objspec_vbox[1]);
  gtk_widget_hide(dia_objspec_vbox[2]);
  gtk_widget_hide(dia_objspec_vbox[3]);

  menus_set_sensitive("/Assist/Inguiry/List", FALSE);
  make_tree_nodes(doc);
  gtk_widget_show(dia_window);
}

void make_dialog_list(void)
{
  gint i;

  /* Create main window */
  dia_window=gtk_dialog_new();

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window),"oCADis: List");

  gtk_window_set_modal(GTK_WINDOW(dia_window),TRUE);

  gtk_widget_set_usize(GTK_WIDGET( dia_window),600,600);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(list_close),NULL);

  /* Make table */
  dia_table=gtk_table_new(2,1,FALSE);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dia_window)->vbox),dia_table,TRUE,TRUE,0);
  gtk_widget_show(dia_table);

  /* Make scrolled window */
  dia_scrwin=gtk_scrolled_window_new(NULL,NULL);
  gtk_table_attach(GTK_TABLE(dia_table), dia_scrwin,0,1,0,1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);

  gtk_widget_show(dia_scrwin);

  /* Make vbox */
  dia_vbox=gtk_vbox_new(TRUE,0);
  gtk_table_attach(GTK_TABLE(dia_table), dia_vbox,1,2,0,1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_vbox);

  /* Make frames */
  dia_frame[0]=gtk_frame_new("Document");
  gtk_box_pack_start(GTK_BOX(dia_vbox),dia_frame[0],TRUE,TRUE,0);

  dia_frame[1]=gtk_frame_new("Layer");
  gtk_box_pack_start(GTK_BOX(dia_vbox),dia_frame[1],TRUE,TRUE,0);

  dia_frame[2]=gtk_frame_new("Object");
  gtk_box_pack_start(GTK_BOX(dia_vbox),dia_frame[2],TRUE,TRUE,0);

  /* Make Tree */
  dia_tree=gtk_tree_new();
  gtk_widget_show(dia_tree);

  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(dia_scrwin),dia_tree);

  /* Make buttons */
  dia_close_button=gtk_button_new_with_label("Close");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_close_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_close_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_close_button);
  gtk_signal_connect(GTK_OBJECT(dia_close_button),"clicked",
		     GTK_SIGNAL_FUNC(list_close), NULL); 
  gtk_widget_show (dia_close_button);

  /* -- Make widgets for object dialog */
  dia_obj_vbox=gtk_vbox_new(TRUE,0);
  gtk_container_add(GTK_CONTAINER(dia_frame[2]),dia_obj_vbox);
  gtk_widget_show(dia_obj_vbox);

  dia_obj_table=gtk_table_new(2,7, FALSE);
  gtk_box_pack_start(GTK_BOX(dia_obj_vbox), dia_obj_table, TRUE, TRUE, 0);
  gtk_widget_show(dia_obj_table);

  dia_obj_label[0]=gtk_label_new("Document: ");
  dia_obj_label[1]=gtk_label_new("Layer: ");
  dia_obj_label[2]=gtk_label_new("Name: ");
  dia_obj_label[3]=gtk_label_new("Type: ");
  dia_obj_label[4]=gtk_label_new("Bounding box: ");
  dia_obj_label[5]=gtk_label_new("Colourtype: ");
  dia_obj_label[6]=gtk_label_new("Colour: ");
  dia_obj_label[7]=gtk_label_new("Linewidthtype: ");
  dia_obj_label[8]=gtk_label_new("Linewidth: ");

  for(i=0;i<9;i++)
    {
      gtk_misc_set_alignment(GTK_MISC(dia_obj_label[i]),1.0,0.5);
      gtk_table_attach(GTK_TABLE(dia_obj_table), dia_obj_label[i],0,1,i,i+1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
      gtk_widget_show(dia_obj_label[i]);
    }

  /* Entry for document name */
  dia_obj_entry[0]=gtk_entry_new();
  gtk_table_attach(GTK_TABLE(dia_obj_table), dia_obj_entry[0],1,2,0,1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_obj_entry[0]);

  /* Entry for layer name */
  dia_obj_entry[1]=gtk_entry_new();
  gtk_table_attach(GTK_TABLE(dia_obj_table), dia_obj_entry[1],1,2,1,2,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_obj_entry[1]);

  /* Entry for object name */
  dia_obj_entry[2]=gtk_entry_new();
  gtk_table_attach(GTK_TABLE(dia_obj_table), dia_obj_entry[2],1,2,2,3,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_obj_entry[2]);

  /* Entry for object type */
  dia_obj_entry[3]=gtk_entry_new();
  gtk_table_attach(GTK_TABLE(dia_obj_table), dia_obj_entry[3],1,2,3,4,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_obj_entry[3]);

  /* Vbox for line */
  dia_objspec_vbox[0]=gtk_vbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(dia_obj_vbox), dia_objspec_vbox[0], TRUE, TRUE, 0);
  gtk_widget_show(dia_objspec_vbox[0]);

  dia_obj_line_table=gtk_table_new(2,8,FALSE);
  gtk_box_pack_start(GTK_BOX(dia_objspec_vbox[0]),dia_obj_line_table,TRUE,TRUE,0);
  gtk_widget_show(dia_obj_line_table);
  
  dia_obj_line_label[0]=gtk_label_new("X1: ");
  dia_obj_line_label[1]=gtk_label_new("Y1: ");
  dia_obj_line_label[2]=gtk_label_new("X2: ");
  dia_obj_line_label[3]=gtk_label_new("Y2: ");
  dia_obj_line_label[4]=gtk_label_new("Length: ");
  dia_obj_line_label[5]=gtk_label_new("Angle: ");
  dia_obj_line_label[6]=gtk_label_new("Delta X: ");
  dia_obj_line_label[7]=gtk_label_new("Delta Y: ");

  for(i=0;i<8;i++)
    {
      gtk_misc_set_alignment(GTK_MISC(dia_obj_line_label[i]),1.0,0.5);
      gtk_table_attach(GTK_TABLE(dia_obj_line_table), dia_obj_line_label[i],0,1,i,i+1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
      gtk_widget_show(dia_obj_line_label[i]);

      dia_obj_line_entry[i]=gtk_entry_new();
      gtk_table_attach(GTK_TABLE(dia_obj_line_table), dia_obj_line_entry[i],1,2,i,i+1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
      gtk_widget_show(dia_obj_line_entry[i]);
    }

  /* Vbox for polyline */
  dia_objspec_vbox[1]=gtk_vbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(dia_obj_vbox), dia_objspec_vbox[1], TRUE, TRUE, 0);
  gtk_widget_show(dia_objspec_vbox[1]);

  /* Vbox for circle */
  dia_objspec_vbox[2]=gtk_vbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(dia_obj_vbox), dia_objspec_vbox[2], TRUE, TRUE, 0);
  gtk_widget_show(dia_objspec_vbox[2]);

  dia_obj_circle_table=gtk_table_new(2,5,FALSE);
  gtk_box_pack_start(GTK_BOX(dia_objspec_vbox[2]),dia_obj_circle_table,TRUE,TRUE,0);
  gtk_widget_show(dia_obj_circle_table);
  
  dia_obj_circle_label[0]=gtk_label_new("X centrum: ");
  dia_obj_circle_label[1]=gtk_label_new("Y centrum: ");
  dia_obj_circle_label[2]=gtk_label_new("Radius: ");
  dia_obj_circle_label[3]=gtk_label_new("Circumference: ");
  dia_obj_circle_label[4]=gtk_label_new("Area: ");

  for(i=0;i<5;i++)
    {
      gtk_misc_set_alignment(GTK_MISC(dia_obj_circle_label[i]),1.0,0.5);
      gtk_table_attach(GTK_TABLE(dia_obj_circle_table), dia_obj_circle_label[i],0,1,i,i+1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
      gtk_widget_show(dia_obj_circle_label[i]);

      dia_obj_circle_entry[i]=gtk_entry_new();
      gtk_table_attach(GTK_TABLE(dia_obj_circle_table), dia_obj_circle_entry[i],1,2,i,i+1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
      gtk_widget_show(dia_obj_circle_entry[i]);
    }

  /* Vbox for Arc */
  dia_objspec_vbox[3]=gtk_vbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(dia_obj_vbox), dia_objspec_vbox[3], TRUE, TRUE, 0);
  gtk_widget_show(dia_objspec_vbox[3]);
}










