/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <strings.h>
#include "struct.h"
#include "global.h"
#include "icons.h"
#include "graphics/eye.xpm"
#include "graphics/lock.xpm"
#include "graphics/curr.xpm"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */
extern GtkWidget *main_window;

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;
static GtkWidget *dia_hbox;
static GtkWidget *dia_button_1;
static GtkWidget *dia_button_2;
static GtkWidget *dia_button_3;

static gint but_selected;

/* --  F U N C T I O N S  -- */
void make_all_icons(void)
{
  gint i;

  icon_list[0].pixmap = gdk_pixmap_create_from_xpm_d (main_window->window,
						      &icon_list[0].mask,
						      NULL, info_xpm);
  icon_list[1].pixmap = gdk_pixmap_create_from_xpm_d (main_window->window,
						      &icon_list[1].mask,
						      NULL, warning_xpm);
  icon_list[2].pixmap = gdk_pixmap_create_from_xpm_d (main_window->window,
						      &icon_list[2].mask,
						      NULL, question_xpm);
  icon_list[3].pixmap = gdk_pixmap_create_from_xpm_d (main_window->window,
						      &icon_list[3].mask,
						      NULL, kill_xpm);
  icon_list[4].pixmap = gdk_pixmap_create_from_xpm_d (main_window->window,
						      &icon_list[4].mask,
						      NULL, eye);
  icon_list[5].pixmap = gdk_pixmap_create_from_xpm_d (main_window->window,
						      &icon_list[5].mask,
						      NULL, lock);
  icon_list[6].pixmap = gdk_pixmap_create_from_xpm_d (main_window->window,
						      &icon_list[6].mask,
						      NULL, curr);
 
  for(i=0;i<NO_ICONS;i++)
    {
      gdk_pixmap_ref (icon_list[i].pixmap);
      gdk_bitmap_ref (icon_list[i].mask);
    }
}


static gint alert_button(GtkWidget *widget, gint data)
{
  but_selected=data;

  gtk_main_quit();

  return TRUE;
}

static gint delete_alert_window(GtkWidget *widget, GdkEvent *event, gint data)
{
  but_selected=data;

  gtk_main_quit();

  return TRUE;
}

gint alert(gint pic, gchar *title, gchar *but1, gchar *but2, gchar *but3, const char *text, ... )
{
  gchar buffer[1025];
  va_list args;

  va_start (args, text);
  vsprintf (buffer, text, args);
  va_end (args);

  dia_window=gtk_dialog_new();

  g_snprintf(buffer,1024,"oCADis: %s",title);
  gtk_window_set_title(GTK_WINDOW(dia_window),buffer);

  gtk_window_set_position(GTK_WINDOW(dia_window), GTK_WIN_POS_MOUSE);
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event", 
                     GTK_SIGNAL_FUNC(delete_alert_window),(gpointer)0);

  gtk_widget_realize (dia_window);

  dia_hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->vbox), dia_hbox,
		      FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (dia_hbox), 10);
  
  gtk_box_pack_start (GTK_BOX (dia_hbox),
		      gtk_pixmap_new (icon_list[pic].pixmap,
				      icon_list[pic].mask),
		      TRUE, FALSE, 5);

  {
    char *tmp = g_strdup_vprintf (text, args);
    
    gtk_box_pack_start (GTK_BOX (dia_hbox), gtk_label_new (tmp), TRUE, TRUE, 5);
    g_free (tmp);
  }

  if(but1)
    {
      dia_button_1 = gtk_button_new_with_label (but1);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area), dia_button_1,
			  TRUE, TRUE, 10);
      GTK_WIDGET_SET_FLAGS (dia_button_1, GTK_CAN_DEFAULT);
      gtk_widget_grab_default (dia_button_1);

      gtk_signal_connect (GTK_OBJECT (dia_button_1), "clicked",
                          GTK_SIGNAL_FUNC (alert_button), (gpointer)1);

    }

  if(but2)
    {
      dia_button_2 = gtk_button_new_with_label (but2);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area), dia_button_2,
			  TRUE, TRUE, 10);

      gtk_signal_connect (GTK_OBJECT (dia_button_2), "clicked",
                          GTK_SIGNAL_FUNC (alert_button), (gpointer)2);
    }

  if(but3)
    {
      dia_button_3 = gtk_button_new_with_label (but3);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area), dia_button_3,
			  TRUE, TRUE, 10);

      gtk_signal_connect (GTK_OBJECT (dia_button_3), "clicked",
                          GTK_SIGNAL_FUNC (alert_button), (gpointer)3);
    }

  gtk_widget_show_all (dia_window);

  gtk_grab_add(dia_window);
  gtk_main();
  gtk_grab_remove(dia_window);

  gtk_widget_destroy(dia_window);

  return but_selected;
}

gint message(gint picture, gchar *text, va_list list)
{
  alert(picture, _("Message"), _("Close"), NULL, NULL, text, list); 

  return TRUE;
}


