/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "initmenu.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gint OK=FALSE;

/* --  F U N C T I O N S  -- */
gint option_menu_get_current(GtkOptionMenu *opt)
{
  GtkWidget *menu, *active;
  menu=gtk_option_menu_get_menu(GTK_OPTION_MENU(opt));
  active=gtk_menu_get_active(GTK_MENU(menu));
  return (gint)gtk_object_get_data(GTK_OBJECT(active),"current");
}

/* -- D i a l o g  f o r  f i l e s e l e c t i o n -- */ 
static GtkWidget *dia_filesel=NULL;
static gchar *filepath=NULL;

static gint dia_filesel_ok(GtkWidget *widget, DOCUMENT *doc)
{
  OK=TRUE;

  /* Get filepath and remember it */
  if(filepath!=NULL)
    g_free(filepath);

  filepath=g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (dia_filesel)));  

  gtk_main_quit();
  return TRUE;
}

static gint dia_filesel_hide(GtkWidget *widget, DOCUMENT *doc)
{
  OK=FALSE;
  gtk_main_quit();
  return TRUE;
}

gchar *dia_fileselector(gchar *title, gchar *extension)
     /* Views a fileselector box with the title title.
	Returns a filepath or NULL if user has canceled. 
	Parameter def has no use.
	Modal!
     */
{
  menus_set_sensitive("/File/Open Drawing...", FALSE);
  menus_set_sensitive("/File/Open Script...", FALSE);
  menus_set_sensitive("/File/Save", FALSE);
  menus_set_sensitive("/File/Save As...", FALSE);

  if(dia_filesel!=NULL)
    gtk_widget_destroy(dia_filesel);
  
  dia_filesel=gtk_file_selection_new(g_strdup_printf("oCADis: %s",title));

  if(filepath!=NULL)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(dia_filesel),filepath);
  else
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(dia_filesel),"");

  /* Set extension */
  if(extension!=NULL)
    gtk_file_selection_complete(GTK_FILE_SELECTION(dia_filesel), extension);
  else
    gtk_file_selection_complete(GTK_FILE_SELECTION(dia_filesel), "");

  gtk_signal_connect (GTK_OBJECT (dia_filesel), "delete_event",
  		      (GtkSignalFunc)dia_filesel_hide, NULL);
  
  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (dia_filesel)->ok_button),
		      "clicked", (GtkSignalFunc) dia_filesel_ok, NULL );
        
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (dia_filesel)->cancel_button),
			     "clicked", (GtkSignalFunc) dia_filesel_hide, NULL);

  gtk_widget_show(dia_filesel);
  gtk_grab_add(dia_filesel);

  gtk_main();

  gtk_grab_remove(dia_filesel);
  gtk_widget_hide(dia_filesel);

  menus_set_sensitive("/File/Open Drawing...", TRUE);
  menus_set_sensitive("/File/Open Script...", TRUE);
  menus_set_sensitive("/File/Save", TRUE);
  menus_set_sensitive("/File/Save As...", TRUE);

  if(OK)
    return gtk_file_selection_get_filename (GTK_FILE_SELECTION (dia_filesel));     
  
  return NULL;


}

/* -- D i a l o g   f o r   C o l o u r   S e l e c t i o n -- */
static GtkWidget *dia_selcolour=NULL;

static gint dia_selcolour_ok(GtkWidget *widget, DOCUMENT *doc)
{
  OK=TRUE;
  gtk_main_quit();
  return TRUE;
}

static gint dia_selcolour_hide(GtkWidget *widget, DOCUMENT *doc)
{
  OK=FALSE;
  gtk_main_quit();
  return TRUE;
}

gint dia_colour_selection(gdouble *colour)
{
  if(dia_selcolour==NULL)
    dia_selcolour=gtk_color_selection_dialog_new( _("Select colour"));

  gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(dia_selcolour)->ok_button),"clicked",
                     GTK_SIGNAL_FUNC(dia_selcolour_ok),NULL);
  gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(dia_selcolour)->cancel_button),"clicked",
                     GTK_SIGNAL_FUNC(dia_selcolour_hide),NULL);
  gtk_signal_connect(GTK_OBJECT(dia_selcolour),"delete_event",
                     GTK_SIGNAL_FUNC(dia_selcolour_hide),NULL);


  gtk_widget_show(dia_selcolour);
  gtk_grab_add(dia_selcolour);

  gtk_main();

  gtk_grab_remove(dia_selcolour);
  gtk_widget_hide(dia_selcolour);

  if(OK)
    {
      gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(dia_selcolour)->colorsel),colour);
      return TRUE;
    }
  
  return FALSE;
}

/* -- D i a l o g  f o r  t e x t  e n t r y -- */
struct s_dia_entry
{
  GtkWidget *window;
  GtkWidget *label;
  GtkWidget *entry;
  GtkWidget *table;
  GtkWidget *ok;
  GtkWidget *cancel;
};

static struct s_dia_entry *dia_entry=NULL;

static gint dia_enter_text_ok(GtkWidget *widget, gpointer data)
{
  OK=TRUE;
  gtk_main_quit();
  return TRUE;
}

static gint dia_enter_text_hide(GtkWidget *widget, gpointer data)
{
  OK=FALSE;
  gtk_main_quit();
  return TRUE;
}

gchar *dia_enter_text(gchar *title, gchar *label, gchar *def)
{
  if(dia_entry!=NULL)
    g_free(dia_entry);

  dia_entry=(struct s_dia_entry *)g_malloc(sizeof(struct s_dia_entry));

  dia_entry->window=gtk_dialog_new();

  gtk_window_set_title(GTK_WINDOW(dia_entry->window),g_strdup_printf("oCADis: %s",title));

  gtk_signal_connect(GTK_OBJECT(dia_entry->window),"delete_event",
                     GTK_SIGNAL_FUNC(dia_enter_text_hide),NULL);

  /* Make table */
  dia_entry->table=gtk_table_new(2,1,FALSE);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_entry->window)->vbox), dia_entry->table,
		      TRUE, TRUE, 10);
  gtk_widget_show(dia_entry->table);

  /* Make label and entry */
  dia_entry->label=gtk_label_new(label);
  gtk_misc_set_alignment(GTK_MISC(dia_entry->label),1.0,0.5);
  gtk_table_attach(GTK_TABLE(dia_entry->table), dia_entry->label,0,1,0,1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_entry->label);

  dia_entry->entry=gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(dia_entry->entry),def);
  gtk_table_attach(GTK_TABLE(dia_entry->table), dia_entry->entry,1,2,0,1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show(dia_entry->entry);

  /* Make buttons and connect functions */
  dia_entry->ok=gtk_button_new_with_label( _("OK"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_entry->window)->action_area), dia_entry->ok,
		      TRUE, TRUE, 10);
  GTK_WIDGET_SET_FLAGS (dia_entry->ok, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_entry->ok);
  gtk_signal_connect (GTK_OBJECT (dia_entry->ok), "clicked",
                          GTK_SIGNAL_FUNC (dia_enter_text_ok),NULL);
  gtk_widget_show(dia_entry->ok);

  dia_entry->cancel=gtk_button_new_with_label( _("Cancel"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_entry->window)->action_area), dia_entry->cancel,
		      TRUE, TRUE, 10);
  gtk_signal_connect (GTK_OBJECT (dia_entry->cancel), "clicked",
                          GTK_SIGNAL_FUNC (dia_enter_text_hide),NULL);
  gtk_widget_show(dia_entry->cancel);

  /* Show it all */
  gtk_widget_show(dia_entry->window);
  gtk_grab_add(dia_entry->window);

  gtk_main();
  
  gtk_grab_remove(dia_entry->window);
  gtk_widget_hide(dia_entry->window);
  
  if(OK)
      return(gtk_entry_get_text(GTK_ENTRY(dia_entry->entry)));
  
  return NULL;
}

/* -- D i a l o g  f o r  l i n e s t y l e  e n t r y -- */
struct s_dia_linestyle
{
  GtkWidget *window;
  GtkWidget *scrwin;
  GtkWidget *list;
  GtkWidget *cancel;
  GtkWidget *ok;
};

static struct s_dia_linestyle *dia_linestyle=NULL;
static gchar *titles[2]={"Name","Picture"};
static LINESTYLE *style;

static gint select_row_callback(GtkWidget *widget,gint row,gint column,
				GdkEventButton *event, gpointer data)
{
  OK=TRUE;
  style=(LINESTYLE *)gtk_clist_get_row_data(GTK_CLIST(dia_linestyle->list),row);
  return TRUE;
}

static gint dia_get_linestyle_hide(GtkWidget *widget, gpointer data)
{
  OK=FALSE;
  gtk_main_quit();
  return TRUE;
}

static gint dia_get_linestyle_ok(GtkWidget *widget, gpointer data)
{
  OK=TRUE;
  gtk_main_quit();
  return TRUE;
}

LINESTYLE *dia_get_linestyle(GList *list)
{
  gchar *text[2]={NULL,"\0"};
  gint curr_row;

  if(dia_linestyle!=NULL)
    g_free(dia_linestyle);

  dia_linestyle=(struct s_dia_linestyle *)g_malloc(sizeof(struct s_dia_linestyle));

  dia_linestyle->window=gtk_dialog_new();

  gtk_window_set_title(GTK_WINDOW(dia_linestyle->window), _("oCADis: Select linestyle"));
  gtk_widget_set_usize(GTK_WIDGET( dia_linestyle->window),250,250);

  gtk_signal_connect(GTK_OBJECT(dia_linestyle->window),"delete_event",
		     GTK_SIGNAL_FUNC(dia_get_linestyle_hide),NULL);

  /* Make scroll window */
  dia_linestyle->scrwin=gtk_scrolled_window_new(NULL,NULL);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_linestyle->window)->vbox),
		      dia_linestyle->scrwin, TRUE, TRUE, 0);
  gtk_widget_show(dia_linestyle->scrwin);

  /* Make clist */
  dia_linestyle->list=gtk_clist_new_with_titles(2,titles);
  gtk_clist_set_row_height(GTK_CLIST(dia_linestyle->list),18);
  gtk_clist_column_titles_passive(GTK_CLIST(dia_linestyle->list));
  gtk_clist_column_titles_show(GTK_CLIST(dia_linestyle->list));
  gtk_clist_set_selection_mode(GTK_CLIST(dia_linestyle->list), GTK_SELECTION_BROWSE);

  gtk_signal_connect(GTK_OBJECT(dia_linestyle->list),"select_row",
		     GTK_SIGNAL_FUNC(select_row_callback),
		     NULL);

  gtk_clist_set_column_justification(GTK_CLIST(dia_linestyle->list),0,GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(dia_linestyle->list),1,GTK_JUSTIFY_CENTER);

  /* Set column width */
  gtk_clist_set_column_width(GTK_CLIST(dia_linestyle->list),0,110);
  gtk_clist_set_column_width(GTK_CLIST(dia_linestyle->list),1,100);

  /* Create data for list */
  gtk_clist_clear(GTK_CLIST(dia_linestyle->list));
  gtk_clist_freeze(GTK_CLIST(dia_linestyle->list));

  while(list!=NULL)
    {
      text[0]=((LINESTYLE *)list->data)->name;

      curr_row=gtk_clist_append(GTK_CLIST(dia_linestyle->list), text); 

      gtk_clist_set_pixmap(GTK_CLIST(dia_linestyle->list),curr_row,1, 
			   ((LINESTYLE *)list->data)->pic.pixmap, ((LINESTYLE *)list->data)->pic.mask);

      gtk_clist_set_row_data(GTK_CLIST(dia_linestyle->list), curr_row, (gpointer)list->data);

      list=g_list_next(list);
    }

  gtk_clist_thaw(GTK_CLIST(dia_linestyle->list));

  /* Force a selection */
  gtk_clist_select_row(GTK_CLIST(dia_linestyle->list),0,0);

  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(dia_linestyle->scrwin),dia_linestyle->list);

  gtk_widget_show(dia_linestyle->list);

  /* Make buttons and connect functions */
  dia_linestyle->ok=gtk_button_new_with_label( _("Ok"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_linestyle->window)->action_area), dia_linestyle->ok,
		      TRUE, TRUE, 10);
  gtk_signal_connect (GTK_OBJECT (dia_linestyle->ok), "clicked",
		      GTK_SIGNAL_FUNC (dia_get_linestyle_ok),NULL);
  gtk_widget_show(dia_linestyle->ok);

  dia_linestyle->cancel=gtk_button_new_with_label( _("Cancel"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_linestyle->window)->action_area), dia_linestyle->cancel,
		      TRUE, TRUE, 10);
  gtk_signal_connect (GTK_OBJECT (dia_linestyle->cancel), "clicked",
		      GTK_SIGNAL_FUNC (dia_get_linestyle_hide),NULL);
  gtk_widget_show(dia_linestyle->cancel);

  /* Show it all */
  gtk_widget_show(dia_linestyle->window);
  gtk_grab_add(dia_linestyle->window);

  gtk_main();
  
  gtk_grab_remove(dia_linestyle->window);
  gtk_widget_hide(dia_linestyle->window);
  
  if(OK)
    return style;
  
  return NULL;
}
