/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "layer.h"
#include "misc.h"
#include "dia_message.h"
#include "dia_toolbox.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;
static GtkWidget *dia_table;

static GtkWidget *dia_ok_button;
static GtkWidget *dia_cancel_button;
static GtkWidget *dia_clear_button;

static GtkWidget *dia_option;
static GtkWidget *dia_menu;

static GtkWidget *dia_label_1;
static GtkWidget *dia_label_2;
static GtkWidget *dia_label_3;
static GtkWidget *dia_label_4;

static GtkWidget *dia_osnap_check[10];

static GSList *dia_is_radiogroup;
static GtkWidget *dia_is_radio[2];

static GtkObject *dia_adj;
static GtkWidget *dia_scale;

static gint sel_option_menu;

/* --  P R O T O T Y P E S  -- */
static void osnap_set_dialog(void)
{
  sel_option_menu=gl_osnap_apply_to;

  gtk_option_menu_set_history(GTK_OPTION_MENU(dia_option),gl_osnap_apply_to);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_is_radio[0]), (gl_osnap_is==OSNAP_ALWAYS) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_is_radio[1]), (gl_osnap_is==OSNAP_ONCE) );

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[0]), (gl_osnap & OSNAP_CEN) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[1]), (gl_osnap & OSNAP_END) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[2]), (gl_osnap & OSNAP_INS) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[3]), (gl_osnap & OSNAP_INT) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[4]), (gl_osnap & OSNAP_MID) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[5]), (gl_osnap & OSNAP_NEA) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[6]), (gl_osnap & OSNAP_NOD) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[7]), (gl_osnap & OSNAP_PER) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[8]), (gl_osnap & OSNAP_QUA) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[9]), (gl_osnap & OSNAP_TAN) );

  gtk_adjustment_set_value(GTK_ADJUSTMENT(dia_adj), gl_osnap_sensitivity);
}

static void osnap_get_dialog(void)
{ 
  gl_osnap_apply_to=sel_option_menu; 
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_is_radio[0])))
    gl_osnap_is=OSNAP_ALWAYS; 
  else
    gl_osnap_is=OSNAP_ONCE;

  gl_osnap=OSNAP_NONE;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[0])))
    gl_osnap= gl_osnap | OSNAP_CEN;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[1])))
    gl_osnap= gl_osnap | OSNAP_END;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[2])))
    gl_osnap= gl_osnap | OSNAP_INS;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[3])))
    gl_osnap= gl_osnap | OSNAP_INT;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[4])))
    gl_osnap= gl_osnap | OSNAP_MID;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[5])))
    gl_osnap= gl_osnap | OSNAP_NEA;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[6])))
    gl_osnap= gl_osnap | OSNAP_NOD;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[7])))
    gl_osnap= gl_osnap | OSNAP_PER;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[8])))
    gl_osnap= gl_osnap | OSNAP_QUA;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_osnap_check[9])))
    gl_osnap= gl_osnap | OSNAP_TAN;

  toolbox_set_osnap(gl_osnap);

  gl_osnap_sensitivity=GTK_ADJUSTMENT(dia_adj)->value; 
}

static gint change_option(GtkWidget *widget, gint data)
{
  sel_option_menu=data;
  return TRUE;
}

static gint osnap_clear(GtkWidget *widget, gpointer data)
{
  gint i;

  for(i=0;i<10;i++)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_osnap_check[i]), FALSE );
  return TRUE;
}

static gint osnap_ok(GtkWidget *widget, gpointer data)
{
  osnap_get_dialog();
  gtk_widget_hide(dia_window);
  menus_set_sensitive("/Assist/Object Snap Settings...", TRUE);
  return TRUE;
}

static gint osnap_cancel(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_window);
  menus_set_sensitive("/Assist/Object Snap Settings...", TRUE);
  return TRUE;
}

void dia_show_osnap(GtkWidget *widget, gpointer data)
{
  menus_set_sensitive("/Assist/Object Snap Settings...", FALSE);
  osnap_set_dialog();
  gtk_widget_show(dia_window);
}

void make_dialog_osnap(void)
{
  GtkWidget *menu;
  gint i;

  /* Create main window */
  dia_window=gtk_dialog_new();

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window), _("oCADis: Object Snap Settings"));

  gtk_window_set_modal(GTK_WINDOW(dia_window),TRUE);

  gtk_widget_set_usize(GTK_WIDGET(dia_window),250,390);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(osnap_cancel),NULL);

  /* Make table */
  dia_table=gtk_table_new(3,14,FALSE);
  gtk_widget_show(dia_table);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dia_window)->vbox),dia_table,TRUE,TRUE,0);

  /* Make options */
  dia_option=gtk_option_menu_new();
  gtk_widget_show(dia_option);
  dia_menu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(dia_option),dia_menu);

  menu=gtk_menu_item_new_with_label( _("Current Layer"));
  gtk_signal_connect(GTK_OBJECT(menu),"activate",
		     GTK_SIGNAL_FUNC (change_option),
		     (gpointer)APPLY_CURRENT_LAYER);
  gtk_widget_show(menu);  
  gtk_menu_append(GTK_MENU(dia_menu), menu);
  menu=gtk_menu_item_new_with_label( _("All Layers"));
  gtk_signal_connect(GTK_OBJECT(menu),"activate",
		     GTK_SIGNAL_FUNC (change_option),
		     (gpointer)APPLY_ALL_LAYERS);
  gtk_widget_show(menu);  
  gtk_menu_append(GTK_MENU(dia_menu), menu);
  menu=gtk_menu_item_new_with_label( _("All Layers but Current"));
  gtk_signal_connect(GTK_OBJECT(menu),"activate",
		     GTK_SIGNAL_FUNC (change_option),
		     (gpointer)APPLY_ALL_LAYERS_BUT_CURRENT);
  gtk_widget_show(menu);  
  gtk_menu_append(GTK_MENU(dia_menu), menu);


  gtk_table_attach(GTK_TABLE(dia_table), dia_option,1,3,0,1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);

  /* Make label */
  dia_label_1=gtk_label_new( _("Apply to: "));
  gtk_table_attach(GTK_TABLE(dia_table), dia_label_1,0,1,0,1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  dia_label_2=gtk_label_new( _("Object snap is: "));
  gtk_table_attach(GTK_TABLE(dia_table), dia_label_2,0,1,1,2,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  dia_label_3=gtk_label_new( _("Object snap: "));
  gtk_table_attach(GTK_TABLE(dia_table), dia_label_3,0,1,3,4,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL,0,0);
  dia_label_4=gtk_label_new( _("Sensitivity: \n(in pixels) "));
  gtk_table_attach(GTK_TABLE(dia_table), dia_label_4,0,1,13,14,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);

  gtk_misc_set_alignment(GTK_MISC(dia_label_1),1.0,0.5);
  gtk_misc_set_alignment(GTK_MISC(dia_label_2),1.0,0.5);
  gtk_misc_set_alignment(GTK_MISC(dia_label_3),1.0,0.5);
  gtk_misc_set_alignment(GTK_MISC(dia_label_4),1.0,0.5);

  gtk_widget_show(dia_label_1);
  gtk_widget_show(dia_label_2);
  gtk_widget_show(dia_label_3);
  gtk_widget_show(dia_label_4);

  /* Make radio buttons */
  dia_osnap_check[0]=gtk_check_button_new_with_label( _("Centerpoint"));
  dia_osnap_check[1]=gtk_check_button_new_with_label( _("Endpoint"));
  dia_osnap_check[2]=gtk_check_button_new_with_label( _("Insert"));
  dia_osnap_check[3]=gtk_check_button_new_with_label( _("Intersection"));
  dia_osnap_check[4]=gtk_check_button_new_with_label( _("Midpoint"));
  dia_osnap_check[5]=gtk_check_button_new_with_label( _("Nearest"));
  dia_osnap_check[6]=gtk_check_button_new_with_label( _("Node"));
  dia_osnap_check[7]=gtk_check_button_new_with_label( _("Perpendicluar"));
  dia_osnap_check[8]=gtk_check_button_new_with_label( _("Quadrant"));
  dia_osnap_check[9]=gtk_check_button_new_with_label( _("Tangent"));

  for(i=0;i<9;i++)
    {
      gtk_widget_show(dia_osnap_check[i]);

      gtk_table_attach(GTK_TABLE(dia_table), dia_osnap_check[i],1,3,i+3,i+1+3,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL,0,0);     
    }

  dia_is_radio[0]=gtk_radio_button_new_with_label(NULL, _("Always On"));
  dia_is_radiogroup=gtk_radio_button_group(GTK_RADIO_BUTTON(dia_is_radio[0]));
  dia_is_radio[1]=gtk_radio_button_new_with_label(dia_is_radiogroup, _("On"));
  gtk_widget_show(dia_is_radio[0]);
  gtk_table_attach(GTK_TABLE(dia_table), dia_is_radio[0],1,2,1,2,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL,0,0);
  gtk_widget_show(dia_is_radio[1]);
  gtk_table_attach(GTK_TABLE(dia_table), dia_is_radio[1],2,3,1,2,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL,0,0);

  /* Make ranger */
  dia_adj = gtk_adjustment_new (1.0, 1.0, 10.0, 1.0, 1.0, 0.0);
  /*  gtk_signal_connect (GTK_OBJECT (dia_adj), "value_changed",
      GTK_SIGNAL_FUNC (cb_digits_scale), NULL); */
  dia_scale = gtk_hscale_new (GTK_ADJUSTMENT (dia_adj));
  gtk_scale_set_digits (GTK_SCALE (dia_scale), 0);
  gtk_table_attach(GTK_TABLE(dia_table), dia_scale,1,3,13,14,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL|GTK_EXPAND|GTK_SHRINK,0,0);
  gtk_widget_show (dia_scale);


  /* Make buttons */
  dia_clear_button=gtk_button_new_with_label( _("Clear"));
  gtk_table_attach(GTK_TABLE(dia_table), dia_clear_button,1,3,9+3,9+3+1,
		       GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL,0,0);
  gtk_signal_connect(GTK_OBJECT(dia_clear_button),"clicked",
      GTK_SIGNAL_FUNC(osnap_clear), NULL);
  gtk_widget_show(dia_clear_button);

  dia_ok_button=gtk_button_new_with_label( _("OK"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_ok_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_ok_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_ok_button);
  gtk_signal_connect(GTK_OBJECT(dia_ok_button),"clicked",
      GTK_SIGNAL_FUNC(osnap_ok), NULL);
  gtk_widget_show (dia_ok_button);

  dia_cancel_button=gtk_button_new_with_label( _("Cancel"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_cancel_button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dia_cancel_button),"clicked",
		     GTK_SIGNAL_FUNC(osnap_cancel), NULL); 
  gtk_widget_show (dia_cancel_button);
}








