/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "initmenu.h"
#include "misc.h"
#include "dia_message.h"
#include "dia_misc.h"
#include "struct/PAGESETUP.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */
PAGESETUP pagesetup={4, 297,210, 10, 10, 10, 10, TRUE, TRUE, 1, 1};

/* --  L O C A L   V A R I A B L E S  -- */
typedef struct {
  gchar *name;
  gdouble width, height;
} PAPERSIZE;

static PAPERSIZE papersize[]={ 
  {"ISO A0", 1189,841},
  {"ISO A1", 841,594},
  {"ISO A2", 594,420},
  {"ISO A3", 420,297},
  {"ISO A4", 297,210},
  {"ISO A5", 210,148},
  {"Letter", 279,216},
  {"Legal", 356,216},
  {"Executive", 254,190},
  {"Tabloid", 432,279},
  {"A", 279,216},
  {"B", 432,279},
  {"C", 559,432},
  {"D", 864,559},
  {"E", 1118,864}
};

static gint nopaper=sizeof(papersize)/sizeof(papersize[0]);

static GtkWidget *dia_window;
static GtkWidget *dia_notebook;

static GtkWidget *dia_framesize;
static GtkWidget *dia_framemargin;

static GtkWidget *dia_ok_button;
static GtkWidget *dia_cancel_button;

/* Margins-widgets */
static GtkWidget *dia_marginlabel[8];
static GtkWidget *dia_marginentry[4];

/* Size-widgets */
static GtkWidget *dia_papersize;
static GtkWidget *dia_papersizelabel;
static GtkWidget *dia_option;
static GtkWidget *dia_menu;

static GtkWidget *dia_setting;
static GSList *dia_radiogroup1;
static GtkWidget *dia_fittopage;
static GtkWidget *dia_scale;

static GtkWidget *dia_orient;
static GSList *dia_radiogroup2;
static GtkWidget *dia_landscape;
static GtkWidget *dia_portrait;

static GtkWidget *dia_scaleframe;
static GtkWidget *dia_scalehbox;
static GtkWidget *dia_scalelabel[2];
static GtkWidget *dia_scaleentry[2];

/* --  F U N C T I O N S  -- */
static void pagesetup_set_dialog(void)
{
  gchar *junk;

  /* Set all values */
  junk=g_strdup_printf("%.1f", pagesetup.margin_top);
  gtk_entry_set_text(GTK_ENTRY(dia_marginentry[0]), junk);
  g_free(junk);
  
  junk=g_strdup_printf("%.1f", pagesetup.margin_bottom);
  gtk_entry_set_text(GTK_ENTRY(dia_marginentry[1]), junk);
  g_free(junk);
  
  junk=g_strdup_printf("%.1f", pagesetup.margin_left);
  gtk_entry_set_text(GTK_ENTRY(dia_marginentry[2]), junk);
  g_free(junk);
  
  junk=g_strdup_printf("%.1f", pagesetup.margin_right);
  gtk_entry_set_text(GTK_ENTRY(dia_marginentry[3]), junk);
  g_free(junk);
  
  junk=g_strdup_printf("%.2f", pagesetup.scale_unit);
  gtk_entry_set_text(GTK_ENTRY(dia_scaleentry[0]), junk);
  g_free(junk);
  
  junk=g_strdup_printf("%.2f", pagesetup.scale_mm);
  gtk_entry_set_text(GTK_ENTRY(dia_scaleentry[1]), junk);
  g_free(junk);
  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_fittopage), pagesetup.fittopage);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_scale), !pagesetup.fittopage);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_landscape), pagesetup.landscape);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_portrait), !pagesetup.landscape);
  
  gtk_option_menu_set_history(GTK_OPTION_MENU(dia_option),pagesetup.paper_type);
}

static void pagesetup_get_dialog(void)
{
  /* Get Paper type */
  pagesetup.paper_type=option_menu_get_current(GTK_OPTION_MENU(dia_option));

  /* Set toggle buttons */
  pagesetup.landscape=(gint)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_landscape));
  pagesetup.fittopage=(gint)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_fittopage));

  /* Get entry text */
  pagesetup.scale_unit=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_scaleentry[0])),NULL);
  pagesetup.scale_mm=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_scaleentry[1])),NULL);

  pagesetup.margin_top=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_marginentry[0])),NULL);
  pagesetup.margin_bottom=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_marginentry[1])),NULL);
  pagesetup.margin_left=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_marginentry[2])),NULL);
  pagesetup.margin_right=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_marginentry[3])),NULL);

  /* Set current page width and height */
  if(pagesetup.landscape)
    {
      pagesetup.width=papersize[pagesetup.paper_type].width;
      pagesetup.height=papersize[pagesetup.paper_type].height;
    }
  else
    {
      pagesetup.width=papersize[pagesetup.paper_type].height;
      pagesetup.height=papersize[pagesetup.paper_type].width;
    }
}

static gint pagesetup_close(GtkWidget *widget, gpointer data)
{
  menus_set_sensitive("/File/Page Setup...", TRUE);
  gtk_widget_hide(dia_window);
  return TRUE;
}

static gint pagesetup_ok(GtkWidget *widget, gpointer data)
{
  pagesetup_get_dialog();
  return pagesetup_close(widget, data);
}

void dia_show_pagesetup(GtkWidget *widget, gpointer data)
{
  menus_set_sensitive("/File/Page Setup...", FALSE);
  pagesetup_set_dialog(); 
  gtk_widget_show(dia_window);
}

void make_dialog_pagesetup(void)
{
  int i;

  /* Create main window */
  dia_window=gtk_dialog_new();
  gtk_widget_set_usize(dia_window, 270, 220);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_window), _("oCADis: Page Setup"));

  gtk_window_set_modal(GTK_WINDOW(dia_window),TRUE);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(pagesetup_close),NULL);

  /* Create a new notebook, place the position of the tabs */
  dia_notebook = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (dia_notebook), GTK_POS_TOP);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dia_window)->vbox),dia_notebook,TRUE,TRUE,0);
  gtk_widget_show(dia_notebook);

  /* Create notebook pages */

  /* Size */
  dia_framesize=gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER (dia_framesize), 10);
  gtk_widget_show(dia_framesize);

  dia_papersize=gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(dia_framesize), dia_papersize, FALSE, FALSE, 0);
  gtk_widget_show(dia_papersize);

  dia_papersizelabel=gtk_label_new( _("Paper size: "));
  gtk_box_pack_start(GTK_BOX(dia_papersize), dia_papersizelabel, FALSE, FALSE, 0);
  gtk_widget_show(dia_papersizelabel);

  /* Make paper size widgets */
  dia_option=gtk_option_menu_new();
  gtk_widget_set_usize(dia_option, 120, 24);
  gtk_widget_show(dia_option);
  dia_menu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(dia_option),dia_menu);

  for(i=0;i<nopaper;i++)
    {
      GtkWidget *menu;
      
      menu=gtk_menu_item_new_with_label(papersize[i].name);
	  
      gtk_object_set_data(GTK_OBJECT(menu),"current",(gpointer)i);
	  
      gtk_widget_show(menu);  
      gtk_menu_append(GTK_MENU(dia_menu), menu);	
    }

  gtk_box_pack_start(GTK_BOX(dia_papersize), dia_option, FALSE, FALSE, 0);

  /* Make orientation */
  dia_orient=gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(dia_framesize), dia_orient, FALSE, FALSE, 0);
  gtk_widget_show(dia_orient);

  /* Make setting */
  dia_setting=gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(dia_framesize), dia_setting, FALSE, FALSE, 0);
  gtk_widget_show(dia_setting);

  /* Radio buttons */

  /* Fit to page or Scale */
  dia_fittopage=gtk_radio_button_new_with_label (NULL, _("Fit to page"));
  gtk_box_pack_start(GTK_BOX (dia_setting), dia_fittopage, TRUE, TRUE, 0);
  gtk_widget_show(dia_fittopage);

  dia_radiogroup1=gtk_radio_button_group(GTK_RADIO_BUTTON(dia_fittopage));

  dia_scale=gtk_radio_button_new_with_label (dia_radiogroup1, _("Scale"));
  gtk_box_pack_start(GTK_BOX (dia_setting), dia_scale, TRUE, TRUE, 0);
  gtk_widget_show(dia_scale);

  /* Landscape or portrait */
  dia_landscape=gtk_radio_button_new_with_label (NULL, _("Landscape"));
  gtk_box_pack_start(GTK_BOX (dia_orient), dia_landscape, TRUE, TRUE, 0);
  gtk_widget_show(dia_landscape);

  dia_radiogroup2=gtk_radio_button_group(GTK_RADIO_BUTTON(dia_landscape));

  dia_portrait=gtk_radio_button_new_with_label (dia_radiogroup2, _("Portrait"));
  gtk_box_pack_start(GTK_BOX (dia_orient), dia_portrait, TRUE, TRUE, 0);
  gtk_widget_show(dia_portrait);


  /* Scale frame */
  dia_scaleframe=gtk_frame_new( _("Scale"));
  gtk_box_pack_start(GTK_BOX (dia_framesize), dia_scaleframe, TRUE, TRUE, 0);
  gtk_widget_show(dia_scaleframe);
  
  dia_scalehbox=gtk_hbox_new(TRUE,0);
  gtk_container_add(GTK_CONTAINER (dia_scaleframe), dia_scalehbox);
  gtk_widget_show(dia_scalehbox);

  dia_scaleentry[0]=gtk_entry_new();
  gtk_box_pack_start(GTK_BOX (dia_scalehbox), dia_scaleentry[0], TRUE, TRUE, 0);
  gtk_widget_show(dia_scaleentry[0]);

  dia_scalelabel[0]=gtk_label_new( _(" unit <=> "));
  gtk_box_pack_start(GTK_BOX (dia_scalehbox), dia_scalelabel[0], TRUE, TRUE, 0);
  gtk_widget_show(dia_scalelabel[0]);

  dia_scaleentry[1]=gtk_entry_new();
  gtk_box_pack_start(GTK_BOX (dia_scalehbox), dia_scaleentry[1], TRUE, TRUE, 0);
  gtk_widget_show(dia_scaleentry[1]);

  dia_scalelabel[1]=gtk_label_new( _(" mm"));
  gtk_box_pack_start(GTK_BOX (dia_scalehbox), dia_scalelabel[1], TRUE, TRUE, 0);
  gtk_widget_show(dia_scalelabel[1]);

  gtk_notebook_append_page (GTK_NOTEBOOK (dia_notebook), dia_framesize, gtk_label_new ( _("Size")));

  
  /* Margins */
  dia_framemargin=gtk_table_new(3,4,TRUE);
  gtk_widget_show(dia_framemargin);

  dia_marginlabel[0]=gtk_label_new( _("Top: "));
  dia_marginlabel[1]=gtk_label_new( _("Bottom: "));
  dia_marginlabel[2]=gtk_label_new( _("Left: "));
  dia_marginlabel[3]=gtk_label_new( _("Right: "));
  dia_marginlabel[4]=gtk_label_new( _(" mm"));
  dia_marginlabel[5]=gtk_label_new( _(" mm"));
  dia_marginlabel[6]=gtk_label_new( _(" mm"));
  dia_marginlabel[7]=gtk_label_new( _(" mm"));

  for(i=0;i<4;i++)
    {
      gtk_misc_set_alignment(GTK_MISC(dia_marginlabel[i+4]),0.0,0.5);
      gtk_table_attach_defaults(GTK_TABLE(dia_framemargin),dia_marginlabel[i+4],2,3,i,i+1);
      gtk_widget_show(dia_marginlabel[i+4]);

      gtk_misc_set_alignment(GTK_MISC(dia_marginlabel[i]),1.0,0.5);
      gtk_table_attach_defaults(GTK_TABLE(dia_framemargin),dia_marginlabel[i],0,1,i,i+1);
      gtk_widget_show(dia_marginlabel[i]);

      dia_marginentry[i]=gtk_entry_new();
      gtk_table_attach_defaults(GTK_TABLE(dia_framemargin),dia_marginentry[i],1,2,i,i+1);
      gtk_widget_show(dia_marginentry[i]);
    }


  gtk_notebook_append_page (GTK_NOTEBOOK (dia_notebook), dia_framemargin, gtk_label_new ("Margins"));

  /* Make buttons */
  dia_ok_button=gtk_button_new_with_label( _("OK"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_ok_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (dia_ok_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (dia_ok_button);
  gtk_signal_connect(GTK_OBJECT(dia_ok_button),"clicked",
		     GTK_SIGNAL_FUNC(pagesetup_ok), NULL); 
  gtk_widget_show (dia_ok_button);

  dia_cancel_button=gtk_button_new_with_label( _("Cancel"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dia_window)->action_area),
		      dia_cancel_button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dia_cancel_button),"clicked",
		     GTK_SIGNAL_FUNC(pagesetup_close), NULL); 
  gtk_widget_show (dia_cancel_button);
}



