/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "initmenu.h"
#include "redraw.h"
#include "draw.h"
#include "layer.h"
#include "misc.h"
#include "dia_message.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
typedef struct s_pointstyle{
  GtkWidget *vbox6;
  GtkWidget *frame3;
  GtkWidget *hbox3;
  GtkWidget *table6;
  GtkWidget *checkbutton_line;
  GtkWidget *checkbutton_box;
  GtkWidget *checkbutton_circle;
  GtkWidget *checkbutton_dot;
  GtkWidget *checkbutton_mark;
  GtkWidget *checkbutton_xmark;
  GtkWidget *frame5;
  GtkWidget *drawingarea;
  GtkWidget *frame4;
  GtkWidget *vbox7;
  GtkWidget *table7;
  GSList *size_group;
  GtkWidget *radiobutton_relative;
  GtkWidget *radiobutton_absolute;
  GtkWidget *table8;
  GtkWidget *label20;
  GtkWidget *size_entry;
  GtkWidget *unit_label;
  GtkWidget *hbox4;
  GtkWidget *hbuttonbox5;
  GtkWidget *update_button;
  GtkWidget *hbuttonbox6;
  GtkWidget *apply_all_button;
  GtkWidget *apply_new_button;
  GtkWidget *label19;
} _POINTSTYLE;

_POINTSTYLE *dia_pointstyle=NULL;

#define DRAWINGAREA_SIZE 64

/* -- F U N C T I O N S -- */
static gint pointstyle_expose_event (GtkWidget *widget, GdkEventExpose *event)
{
  gint point_style=0;

  if(dia_pointstyle==NULL)
    return TRUE;

  gdk_draw_rectangle(dia_pointstyle->drawingarea->window,
		     dia_pointstyle->drawingarea->style->white_gc,
		     TRUE,
		     0, 0,
		     dia_pointstyle->drawingarea->allocation.width,
		     dia_pointstyle->drawingarea->allocation.height);

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_dot)))
    point_style = point_style | POINT_DOT;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_line)))
    point_style = point_style | POINT_LINE;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_circle)))
    point_style = point_style | POINT_CIRCLE;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_box)))
    point_style = point_style | POINT_BOX;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_mark)))
    point_style = point_style | POINT_MARK;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_xmark)))
    point_style = point_style | POINT_XMARK;

  draw_point(dia_pointstyle->drawingarea->window,dia_pointstyle->drawingarea->style->black_gc,point_style, 
	     DRAWINGAREA_SIZE/2-5, DRAWINGAREA_SIZE/2, DRAWINGAREA_SIZE/2);

  return TRUE;
}

static gint pointstyle_toggle(GtkWidget *widget, gpointer data)
{
  if(dia_pointstyle!=NULL)
    gtk_widget_queue_draw(GTK_WIDGET(dia_pointstyle->drawingarea));

  return TRUE;
}

static gint pointstyle_sizetype(GtkWidget *widget, gpointer data)
{
  if(dia_pointstyle==NULL)
    return TRUE;

  if(GPOINTER_TO_INT(data)==POINT_ABSOLUTE)
    gtk_label_set(GTK_LABEL(dia_pointstyle->unit_label), " pixels");
  else
    gtk_label_set(GTK_LABEL(dia_pointstyle->unit_label), " units");

  return TRUE;
}

void pointstyle_update(DOCUMENT *doc)
{
  if(doc==NULL)
    return;
  if(dia_pointstyle==NULL)
    return;

  /* Remember current document */
  gtk_object_set_data(GTK_OBJECT(dia_pointstyle->vbox6), "doc", (gpointer)doc);

  /* Set toggle buttons */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->checkbutton_xmark), 
			       (gboolean)(doc->point_style & POINT_XMARK) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->checkbutton_mark), 
			       (gboolean)(doc->point_style & POINT_MARK) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->checkbutton_box), 
			       (gboolean)(doc->point_style & POINT_BOX) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->checkbutton_circle), 
			       (gboolean)(doc->point_style & POINT_CIRCLE) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->checkbutton_line), 
			       (gboolean)(doc->point_style & POINT_LINE) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->checkbutton_dot), 
			       (gboolean)(doc->point_style & POINT_DOT) );

  /* Set type of size */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->radiobutton_absolute), doc->point_type==POINT_ABSOLUTE );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dia_pointstyle->radiobutton_relative), doc->point_type==POINT_RELATIVE );

  /* Set size value */
  gtk_entry_set_text(GTK_ENTRY(dia_pointstyle->size_entry), g_strdup_printf("%.2f", doc->point_size)); 

  /* Update drawing area */
  gtk_widget_queue_draw(GTK_WIDGET(dia_pointstyle->drawingarea));
}

static gint pointstyle_apply_to_new(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  if(dia_pointstyle==NULL)
    return TRUE;
  
  /* Get current document */
  doc=(DOCUMENT *)gtk_object_get_data(GTK_OBJECT(dia_pointstyle->vbox6),"doc");
  if(doc==NULL)
    return TRUE;

  /* Get style */
  doc->point_style=0;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_dot)))
    doc->point_style=doc->point_style|POINT_DOT;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_line)))
    doc->point_style=doc->point_style|POINT_LINE;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_circle)))
    doc->point_style=doc->point_style|POINT_CIRCLE;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_box)))
    doc->point_style=doc->point_style|POINT_BOX;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_mark)))
    doc->point_style=doc->point_style|POINT_MARK;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->checkbutton_xmark)))
    doc->point_style=doc->point_style|POINT_XMARK;

  /* Get type of size */
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_pointstyle->radiobutton_absolute)))
    doc->point_type=POINT_ABSOLUTE;
  else
    doc->point_type=POINT_RELATIVE;

  /* Get size of point */
  doc->point_size=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_pointstyle->size_entry)),NULL);

  return TRUE;
}

static void change_all_points(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  if(object->type==TYPE_POINT)
    {
      object->selected=OBJECT_ERASE;
      redraw_object(doc,object);
      object->data.point.point_size=doc->point_size;
      object->data.point.point_type=doc->point_type;
      object->data.point.point_style=doc->point_style;
      object->selected=OBJECT_NOT_SEL;
      redraw_object(doc,object);
    }
}

static gint pointstyle_apply_to_all(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  pointstyle_apply_to_new(widget, data);

  /* Get current document */
  doc=(DOCUMENT *)gtk_object_get_data(GTK_OBJECT(dia_pointstyle->vbox6),"doc");
  if(doc==NULL)
    return TRUE;

  foreach_object(doc, change_all_points);

  return TRUE;
}

static gint pointstyle_update_button(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  /* Get current document */
  doc=(DOCUMENT *)gtk_object_get_data(GTK_OBJECT(dia_pointstyle->vbox6),"doc");
  if(doc==NULL)
    return TRUE;
  
  pointstyle_update(doc);

  return TRUE;
}

/* Callback for list */
static void select_row_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
  DOCUMENT *doc;

  /* Find selected document */
  doc=(DOCUMENT *)gtk_clist_get_row_data(GTK_CLIST(data), row);

  /* Update the dialog */
  if(doc!=NULL)
    pointstyle_update(doc);
}

GtkWidget *pointstyle_create_notebook(GtkWidget *list)
{
  /* Create struct */
  dia_pointstyle=g_new(_POINTSTYLE,1);
  dia_pointstyle->size_group=NULL;

  dia_pointstyle->vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_pointstyle->vbox6);
  
  gtk_container_set_border_width (GTK_CONTAINER (dia_pointstyle->vbox6), 3);

  dia_pointstyle->frame3=gtk_frame_new (_("Type"));
  gtk_widget_show(dia_pointstyle->frame3);
  gtk_box_pack_start(GTK_BOX (dia_pointstyle->vbox6), dia_pointstyle->frame3, FALSE, FALSE, 0);

  dia_pointstyle->hbox3=gtk_hbox_new (FALSE, 0);
  gtk_widget_show(dia_pointstyle->hbox3);
  gtk_container_add(GTK_CONTAINER (dia_pointstyle->frame3), dia_pointstyle->hbox3);
  gtk_container_set_border_width(GTK_CONTAINER (dia_pointstyle->hbox3), 3);

  dia_pointstyle->table6 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (dia_pointstyle->table6);
  gtk_box_pack_start (GTK_BOX (dia_pointstyle->hbox3), dia_pointstyle->table6, TRUE, TRUE, 0);

  /* Create checkbuttons for different pointstyle */
  dia_pointstyle->checkbutton_dot = gtk_check_button_new_with_label (_("Dot"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->checkbutton_dot),"toggled",
		     GTK_SIGNAL_FUNC(pointstyle_toggle), NULL); 
  gtk_widget_show (dia_pointstyle->checkbutton_dot);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table6), dia_pointstyle->checkbutton_dot, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_pointstyle->checkbutton_box = gtk_check_button_new_with_label (_("Box"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->checkbutton_box),"toggled",
		     GTK_SIGNAL_FUNC(pointstyle_toggle), NULL); 
  gtk_widget_show (dia_pointstyle->checkbutton_box);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table6), dia_pointstyle->checkbutton_box, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_pointstyle->checkbutton_line = gtk_check_button_new_with_label (_("Line"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->checkbutton_line),"toggled",
		     GTK_SIGNAL_FUNC(pointstyle_toggle), NULL); 
  gtk_widget_show (dia_pointstyle->checkbutton_line);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table6), dia_pointstyle->checkbutton_line, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_pointstyle->checkbutton_circle = gtk_check_button_new_with_label (_("Circle"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->checkbutton_circle),"toggled",
		     GTK_SIGNAL_FUNC(pointstyle_toggle), NULL); 
  gtk_widget_show (dia_pointstyle->checkbutton_circle);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table6), dia_pointstyle->checkbutton_circle, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_pointstyle->checkbutton_mark = gtk_check_button_new_with_label (_("Mark"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->checkbutton_mark),"toggled",
		     GTK_SIGNAL_FUNC(pointstyle_toggle), NULL); 
  gtk_widget_show (dia_pointstyle->checkbutton_mark);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table6), dia_pointstyle->checkbutton_mark, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_pointstyle->checkbutton_xmark = gtk_check_button_new_with_label (_("X-Mark"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->checkbutton_xmark),"toggled",
		   GTK_SIGNAL_FUNC(pointstyle_toggle), NULL); 
  gtk_widget_show (dia_pointstyle->checkbutton_xmark);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table6), dia_pointstyle->checkbutton_xmark, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  /* Create frame for drawing area */
  dia_pointstyle->frame5=gtk_frame_new(_("Picture"));
  gtk_widget_show(dia_pointstyle->frame5);
  gtk_box_pack_start(GTK_BOX(dia_pointstyle->hbox3), dia_pointstyle->frame5, TRUE, FALSE, 0);

  /* Create drawing area */
  dia_pointstyle->drawingarea = gtk_drawing_area_new ();
  gtk_widget_show(dia_pointstyle->drawingarea);
  gtk_widget_set_events(dia_pointstyle->drawingarea, GDK_EXPOSURE_MASK);
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->drawingarea), "expose_event",
		     (GtkSignalFunc) pointstyle_expose_event, NULL);
  gtk_container_add(GTK_CONTAINER(dia_pointstyle->frame5), dia_pointstyle->drawingarea);
  gtk_widget_set_usize(dia_pointstyle->drawingarea, DRAWINGAREA_SIZE, DRAWINGAREA_SIZE);

  dia_pointstyle->frame4 = gtk_frame_new (_("Size"));
  gtk_widget_show (dia_pointstyle->frame4);
  gtk_box_pack_start (GTK_BOX (dia_pointstyle->vbox6), dia_pointstyle->frame4, FALSE, FALSE, 3);

  dia_pointstyle->vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_pointstyle->vbox7);
  gtk_container_add (GTK_CONTAINER (dia_pointstyle->frame4), dia_pointstyle->vbox7);
  gtk_container_set_border_width (GTK_CONTAINER (dia_pointstyle->vbox7), 3);

  dia_pointstyle->table7 = gtk_table_new (1, 2, TRUE);
  gtk_widget_show (dia_pointstyle->table7);
  gtk_box_pack_start (GTK_BOX (dia_pointstyle->vbox7), dia_pointstyle->table7, FALSE, FALSE, 0);

  /* Create radio buttons for size relativa and absolute */
  dia_pointstyle->radiobutton_relative = gtk_radio_button_new_with_label (dia_pointstyle->size_group, _("Relative"));
  dia_pointstyle->size_group = gtk_radio_button_group (GTK_RADIO_BUTTON (dia_pointstyle->radiobutton_relative));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->radiobutton_relative),"toggled",
		     GTK_SIGNAL_FUNC(pointstyle_sizetype), GINT_TO_POINTER(POINT_RELATIVE)); 
  gtk_widget_show (dia_pointstyle->radiobutton_relative);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table7), dia_pointstyle->radiobutton_relative, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  dia_pointstyle->radiobutton_absolute = gtk_radio_button_new_with_label (dia_pointstyle->size_group, _("Absolute"));
  dia_pointstyle->size_group = gtk_radio_button_group (GTK_RADIO_BUTTON (dia_pointstyle->radiobutton_absolute));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->radiobutton_absolute),"toggled",
		     GTK_SIGNAL_FUNC(pointstyle_sizetype), GINT_TO_POINTER(POINT_ABSOLUTE)); 
  gtk_widget_show (dia_pointstyle->radiobutton_absolute);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table7), dia_pointstyle->radiobutton_absolute, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  dia_pointstyle->table8 = gtk_table_new (1, 3, FALSE);
  gtk_widget_show (dia_pointstyle->table8);
  gtk_box_pack_start (GTK_BOX (dia_pointstyle->vbox7), dia_pointstyle->table8, FALSE, FALSE, 0);

  dia_pointstyle->label20=gtk_label_new (_("Size: "));
  gtk_widget_show(dia_pointstyle->label20);
  gtk_table_attach(GTK_TABLE(dia_pointstyle->table8), dia_pointstyle->label20, 0, 1, 0, 1,
                   (GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
                   (GtkAttachOptions)(0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(dia_pointstyle->label20),1.0,0.5);
  
  /* Create entry for size of point */
  dia_pointstyle->size_entry = gtk_entry_new ();
  gtk_widget_show (dia_pointstyle->size_entry);
  gtk_table_attach (GTK_TABLE (dia_pointstyle->table8), dia_pointstyle->size_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  /* Create unit-label for size */
  dia_pointstyle->unit_label=gtk_label_new (_(" units"));
  gtk_widget_show(dia_pointstyle->unit_label);
  gtk_table_attach(GTK_TABLE(dia_pointstyle->table8), dia_pointstyle->unit_label, 2, 3, 0, 1,
		   (GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
		   (GtkAttachOptions)(0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(dia_pointstyle->unit_label),0.0,0.5);

  dia_pointstyle->hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_pointstyle->hbox4);
  gtk_box_pack_end (GTK_BOX (dia_pointstyle->vbox6), dia_pointstyle->hbox4, FALSE, FALSE, 0);

  dia_pointstyle->hbuttonbox5 = gtk_hbutton_box_new ();
  gtk_widget_show (dia_pointstyle->hbuttonbox5);
  gtk_box_pack_start (GTK_BOX (dia_pointstyle->hbox4), dia_pointstyle->hbuttonbox5, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dia_pointstyle->hbuttonbox5), GTK_BUTTONBOX_START);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dia_pointstyle->hbuttonbox5), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (dia_pointstyle->hbuttonbox5), 114, 27);

  dia_pointstyle->update_button=gtk_button_new_with_label(_("Update"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->update_button),"clicked",
		     GTK_SIGNAL_FUNC(pointstyle_update_button), NULL); 
  gtk_widget_show(dia_pointstyle->update_button);
  gtk_container_add(GTK_CONTAINER (dia_pointstyle->hbuttonbox5), dia_pointstyle->update_button);
  GTK_WIDGET_SET_FLAGS(dia_pointstyle->update_button, GTK_CAN_DEFAULT);

  dia_pointstyle->hbuttonbox6=gtk_hbutton_box_new ();
  gtk_widget_show(dia_pointstyle->hbuttonbox6);
  gtk_box_pack_start(GTK_BOX (dia_pointstyle->hbox4), dia_pointstyle->hbuttonbox6, TRUE, TRUE, 0);
  gtk_button_box_set_layout(GTK_BUTTON_BOX (dia_pointstyle->hbuttonbox6), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX (dia_pointstyle->hbuttonbox6), 0);
  gtk_button_box_set_child_size(GTK_BUTTON_BOX (dia_pointstyle->hbuttonbox6), 114, -1);

  dia_pointstyle->apply_all_button=gtk_button_new_with_label (_("Apply to all"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->apply_all_button),"clicked",
		     GTK_SIGNAL_FUNC(pointstyle_apply_to_all), NULL); 
  gtk_widget_show(dia_pointstyle->apply_all_button);
  gtk_container_add(GTK_CONTAINER (dia_pointstyle->hbuttonbox6),dia_pointstyle->apply_all_button);
  GTK_WIDGET_SET_FLAGS(dia_pointstyle->apply_all_button, GTK_CAN_DEFAULT);

  dia_pointstyle->apply_new_button=gtk_button_new_with_label (_("Apply to new"));
  gtk_signal_connect(GTK_OBJECT(dia_pointstyle->apply_new_button),"clicked",
		     GTK_SIGNAL_FUNC(pointstyle_apply_to_new), NULL); 
  gtk_widget_show(dia_pointstyle->apply_new_button);
  gtk_container_add(GTK_CONTAINER (dia_pointstyle->hbuttonbox6), dia_pointstyle->apply_new_button);
  GTK_WIDGET_SET_FLAGS(dia_pointstyle->apply_new_button, GTK_CAN_DEFAULT);

  /* Attach signal to list so we know when user selects another document */
  gtk_signal_connect(GTK_OBJECT(list), "select_row", GTK_SIGNAL_FUNC(select_row_callback), list);

  return dia_pointstyle->vbox6;
}






