/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "initmenu.h"
#include "misc.h"
#include "dia_message.h"
#include "struct/config.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */
extern CONFIG configuration;  /* Defined in main.c */

/* --  L O C A L   V A R I A B L E S  -- */
static GtkWidget *dia_window;
static GtkWidget *vbox1;
static GtkWidget *notebook1;
static GtkWidget *vbox3;
static GtkWidget *vbox4;
static GtkWidget *dia_tablet_table;
static GtkWidget *dia_tablet_label;
static GtkWidget *dia_tablet_entry;
static GtkWidget *dia_tablet_load;
static GtkWidget *dia_device_table;
static GtkWidget *dia_device_label;
static GtkWidget *dia_device_entry;
static GtkWidget *dia_device_load;
static GtkWidget *dia_path_table;
static GtkWidget *dia_path_symbol_label;
static GtkWidget *dia_path_symbol_entry;
static GtkWidget *dia_path_symbol_load;
static GtkWidget *dia_digitizer;
static GtkWidget *dia_path;
static GtkWidget *vbox5;
static GtkWidget *dia_background;
static GtkWidget *dia_general;
static GtkWidget *hseparator1;
static GtkWidget *hbuttonbox1;
static GtkWidget *dia_ok;
static GtkWidget *dia_cancel;

/* --  P R O T O T Y P E S  -- */
static void preferences_set_dialog()
{
  /* Set general page */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dia_background),(gint)configuration.use_background);

  /* Set digitizer page */
  gtk_entry_set_text(GTK_ENTRY(dia_tablet_entry), (configuration.path_to_tablet!=NULL?configuration.path_to_tablet:""));
  gtk_entry_set_text(GTK_ENTRY(dia_device_entry), (configuration.path_to_device!=NULL?configuration.path_to_device:""));

  /* Set path page */
  gtk_entry_set_text(GTK_ENTRY(dia_path_symbol_entry), (configuration.path_to_symbols!=NULL?configuration.path_to_symbols:""));
}

static gint preferences_get_dialog()
{
  /* Get general page */
  configuration.use_background=(gint)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dia_background));

  /* Get digitizer page */
  if(configuration.path_to_tablet==NULL)
    g_free(configuration.path_to_tablet);
  configuration.path_to_tablet=g_strdup(gtk_entry_get_text(GTK_ENTRY(dia_tablet_entry)));

  if(configuration.path_to_device==NULL)
    g_free(configuration.path_to_device);
  configuration.path_to_device=g_strdup(gtk_entry_get_text(GTK_ENTRY(dia_device_entry)));

  /* Get path page */
  if(configuration.path_to_symbols==NULL)
    g_free(configuration.path_to_symbols);
  configuration.path_to_symbols=g_strdup(gtk_entry_get_text(GTK_ENTRY(dia_path_symbol_entry)));

  return TRUE;
}

static gint preferences_cancel(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_window);
  menus_set_sensitive("/File/Preferences...", TRUE);
  return TRUE;
}

static gint preferences_ok(GtkWidget *widget, gpointer data)
{
  preferences_get_dialog();
  menus_set_sensitive("/File/Preferences...", TRUE);
  gtk_widget_hide(dia_window);
  return TRUE;
}

void dia_show_preferences(GtkWidget *widget, gpointer data)
{
  menus_set_sensitive("/File/Preferences...", FALSE);
  preferences_set_dialog();
  gtk_widget_show(dia_window);
}

void make_dialog_preferences(void)
{
  dia_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (dia_window), "dia_window", dia_window);
  gtk_window_set_title (GTK_WINDOW (dia_window), _("oCADis: Preferences"));
  gtk_window_set_modal (GTK_WINDOW (dia_window), TRUE);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_window),"delete_event",
                     GTK_SIGNAL_FUNC(preferences_cancel),NULL);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dia_window), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (dia_window), vbox1);

  /* Make notebooks */
  notebook1 = gtk_notebook_new ();

  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (dia_window), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook1, TRUE, TRUE, 0);


  /* Create General pages */
  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox5);
  gtk_object_set_data_full (GTK_OBJECT (dia_window), "vbox5", vbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox5);
  gtk_container_set_border_width (GTK_CONTAINER (vbox5), 4);

  dia_background = gtk_check_button_new_with_label ( _("Enable background drawing"));
  gtk_widget_ref (dia_background);
  gtk_object_set_data_full (GTK_OBJECT (dia_window), "dia_background", dia_background,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dia_background);
  gtk_box_pack_start (GTK_BOX (vbox5), dia_background, FALSE, FALSE, 0);

  dia_general = gtk_label_new ( _("General"));
  gtk_widget_ref (dia_general);
  gtk_object_set_data_full (GTK_OBJECT (dia_window), "dia_general", dia_general,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dia_general);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), dia_general);


  /* -- Create the Digitizer page -- */
  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox3);
  gtk_container_set_border_width (GTK_CONTAINER (vbox3), 4);

  /* Create tablet */
  dia_tablet_table = gtk_table_new (1, 3, FALSE);
  gtk_widget_show (dia_tablet_table);
  gtk_box_pack_start (GTK_BOX (vbox3), dia_tablet_table, FALSE, TRUE, 0);

  dia_tablet_label = gtk_label_new ( _("Tablet: "));
  gtk_widget_show (dia_tablet_label);
  gtk_table_attach (GTK_TABLE (dia_tablet_table), dia_tablet_label, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  dia_tablet_entry = gtk_entry_new ();
  gtk_widget_show (dia_tablet_entry);
  gtk_table_attach (GTK_TABLE (dia_tablet_table), dia_tablet_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_tablet_load = gtk_button_new_with_label ("...");
  gtk_widget_show (dia_tablet_load);
  gtk_table_attach (GTK_TABLE (dia_tablet_table), dia_tablet_load, 2, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  /* Create device */
  dia_device_table = gtk_table_new (1, 3, FALSE);
  gtk_widget_show (dia_device_table);
  gtk_box_pack_start (GTK_BOX (vbox3), dia_device_table, FALSE, TRUE, 0);

  dia_device_label = gtk_label_new ( _("Device: "));
  gtk_widget_show (dia_device_label);
  gtk_table_attach (GTK_TABLE (dia_device_table), dia_device_label, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  dia_device_entry = gtk_entry_new ();
  gtk_widget_show (dia_device_entry);
  gtk_table_attach (GTK_TABLE (dia_device_table), dia_device_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  dia_device_load = gtk_button_new_with_label ("...");
  gtk_widget_show (dia_device_load);
  gtk_table_attach (GTK_TABLE (dia_device_table), dia_device_load, 2, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  /* Create page label */
  dia_digitizer = gtk_label_new ( _("Digitizer"));
  gtk_object_set_data_full (GTK_OBJECT (dia_window), "dia_digitizer", dia_digitizer,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dia_digitizer);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), dia_digitizer);

  /* Create the path page */
  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox4);
  gtk_container_set_border_width (GTK_CONTAINER (vbox4), 4);

  dia_path_table = gtk_table_new (1, 3, FALSE);
  gtk_widget_show (dia_path_table);
  gtk_box_pack_start (GTK_BOX (vbox4), dia_path_table, FALSE, TRUE, 0);
  
  dia_path_symbol_label = gtk_label_new ( _("Symbols: "));
  gtk_widget_show (dia_path_symbol_label);
  gtk_table_attach (GTK_TABLE (dia_path_table), dia_path_symbol_label, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  
  dia_path_symbol_entry = gtk_entry_new ();
  gtk_widget_show (dia_path_symbol_entry);
  gtk_table_attach (GTK_TABLE (dia_path_table), dia_path_symbol_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  
  dia_path_symbol_load = gtk_button_new_with_label ("...");
  gtk_widget_show (dia_path_symbol_load);
  gtk_table_attach (GTK_TABLE (dia_path_table), dia_path_symbol_load, 2, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  /* Create path label */
  dia_path = gtk_label_new ( _("Path"));
  gtk_widget_show (dia_path);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), dia_path);

  /* -- Create Separator -- */
  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, FALSE, FALSE, 0);


  /* -- Create box for buttons -- */
  gtk_hbutton_box_set_spacing_default (4);
  gtk_hbutton_box_set_layout_default (GTK_BUTTONBOX_END);
  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_container_set_border_width (GTK_CONTAINER (hbuttonbox1), 4);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, FALSE, 0);

  /* Create OK button */
  dia_ok = gtk_button_new_with_label ( _("OK"));
  gtk_widget_show (dia_ok);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), dia_ok);
  gtk_signal_connect(GTK_OBJECT(dia_ok),"clicked",
		     GTK_SIGNAL_FUNC(preferences_ok), NULL); 

  /* Create close button */
  dia_cancel = gtk_button_new_with_label ( _("Cancel"));
  gtk_widget_show (dia_cancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), dia_cancel);
  gtk_signal_connect(GTK_OBJECT(dia_cancel),"clicked",
		     GTK_SIGNAL_FUNC(preferences_cancel), NULL); 
}



