/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <strings.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "misc.h"
#include "dia_misc.h"
#include "redraw.h"
#include "dia_message.h"
#include "translation.h"
#include "dia_point.h"
#include "dia_linestyle.h"
#include "dia_textstyle.h"
#include "struct/dialog.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
typedef struct {
  GtkWidget *window;
  GtkWidget *main_vbox;

  GtkWidget *hpaned;
  GtkWidget *frame;
  GtkWidget *scrolledwindow;
  GtkWidget *viewport;
  GtkWidget *list;

  GtkWidget *hseparator;

  GtkWidget *hbuttonbox;
  GtkWidget *button;

  GtkWidget *notebook;
  
} _STYLE;

_STYLE *dia_style=NULL;

static void make_dialog_style(void);

/* -- F U N C T I O N S -- */

void style_create_document_list(DOCUMENT *current)
{
  gint row;
  gchar *data[1];
  GList *list;
  DOCUMENT *doc;

  if(dia_style==NULL)
    make_dialog_style();

  gtk_clist_freeze(GTK_CLIST(dia_style->list));  

  /* Remove old data */
  gtk_clist_clear(GTK_CLIST(dia_style->list));

  list=g_list_first(document_list);

  while(list!=NULL)
    {
      doc=(DOCUMENT *)list->data;
   
      /* Make nice title */
      if(doc->filename!=NULL)     
	data[0]=doc->filename;
      else
	data[0]=g_strdup(_("[no name]"));

      row=gtk_clist_append(GTK_CLIST(dia_style->list), data);
      gtk_clist_set_row_data(GTK_CLIST(dia_style->list), row, list->data);

      /* Make the current document selected */
      if(doc==current)
	gtk_clist_select_row(GTK_CLIST(dia_style->list), row, 0);

      list=g_list_next(list);
    }

  gtk_clist_thaw(GTK_CLIST(dia_style->list));
}


/* -- Callback functions -- */

/* Callback for close-button and window close */
static gint style_close(GtkWidget *widget, gpointer data)
{
  gtk_widget_hide(dia_style->window);
  return TRUE;
}

/* Function to create the actually dialog */
static void make_dialog_style(void)
{
  GtkWidget *label;

  /* Create struct */
  dia_style=g_new(_STYLE,1);

  /* Create main window */
  dia_style->window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize(dia_style->window);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW (dia_style->window), _("oCADis: Style"));

  /* Set window to modal */
  gtk_window_set_modal(GTK_WINDOW(dia_style->window),TRUE);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_style->window),"delete_event",
                     GTK_SIGNAL_FUNC(style_close),NULL);

  /* Create main vbox */
  dia_style->main_vbox=gtk_vbox_new(FALSE, 0);
  gtk_widget_show(dia_style->main_vbox);
  gtk_container_add(GTK_CONTAINER(dia_style->window), dia_style->main_vbox);

  /* Create panned window */
  dia_style->hpaned=gtk_hpaned_new();
  gtk_widget_show(dia_style->hpaned);
  gtk_box_pack_start(GTK_BOX(dia_style->main_vbox), dia_style->hpaned, TRUE, TRUE, 0);
  gtk_paned_set_position(GTK_PANED(dia_style->hpaned), 90);

  /* Create select drawing part of window */
  dia_style->frame = gtk_frame_new (_("Select drawing"));
  gtk_widget_show (dia_style->frame);
  gtk_paned_pack1 (GTK_PANED (dia_style->hpaned), dia_style->frame, FALSE, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (dia_style->frame), 3);

  dia_style->scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (dia_style->scrolledwindow);
  gtk_container_add (GTK_CONTAINER (dia_style->frame), dia_style->scrolledwindow);
  gtk_container_set_border_width (GTK_CONTAINER (dia_style->scrolledwindow), 3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dia_style->scrolledwindow), 
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  dia_style->viewport = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (dia_style->viewport);
  gtk_container_add (GTK_CONTAINER (dia_style->scrolledwindow), dia_style->viewport);

  /* Create the list that holds every document */
  dia_style->list = gtk_clist_new (1);
  gtk_clist_set_selection_mode(GTK_CLIST(dia_style->list), GTK_SELECTION_BROWSE);
  gtk_clist_column_titles_hide(GTK_CLIST(dia_style->list));
  gtk_widget_show (dia_style->list);

  /* Add list to viewport */
  gtk_container_add (GTK_CONTAINER (dia_style->viewport), dia_style->list);

  /* Fix nice separator */
  dia_style->hseparator=gtk_hseparator_new ();
  gtk_widget_show(dia_style->hseparator);
  gtk_box_pack_start(GTK_BOX (dia_style->main_vbox), dia_style->hseparator, FALSE, TRUE, 0);

  /* Fix Close button */
  dia_style->hbuttonbox=gtk_hbutton_box_new ();
  gtk_widget_show(dia_style->hbuttonbox);
  gtk_box_pack_start(GTK_BOX(dia_style->main_vbox), dia_style->hbuttonbox, FALSE, TRUE, 0);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(dia_style->hbuttonbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(dia_style->hbuttonbox), 0);
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(dia_style->hbuttonbox), 114, 27);

  dia_style->button=gtk_button_new_with_label (_("Close"));
  gtk_widget_show(dia_style->button);
  gtk_container_add(GTK_CONTAINER(dia_style->hbuttonbox), dia_style->button);
  gtk_container_set_border_width(GTK_CONTAINER(dia_style->button), 2);
  GTK_WIDGET_SET_FLAGS(dia_style->button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(dia_style->button),"clicked",
		     GTK_SIGNAL_FUNC(style_close), NULL); 

  /* Create notebook */
  dia_style->notebook=gtk_notebook_new ();
  gtk_widget_show(dia_style->notebook);
  gtk_paned_pack2(GTK_PANED(dia_style->hpaned), dia_style->notebook, FALSE, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (dia_style->notebook), 3);

  /* Create notebook for Linestyle */
  label=gtk_label_new(_("Linestyle"));
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(dia_style->notebook), linestyle_create_notebook(dia_style->list), label);

  /* Create notebook for Pointstyle */
  label=gtk_label_new(_("Pointstyle"));
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(dia_style->notebook), pointstyle_create_notebook(dia_style->list), label);

  /* Create notebook for Textstyle */
  label=gtk_label_new(_("Textstyle"));
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(dia_style->notebook), textstyle_create_notebook(dia_style->list), label);

  /* Create notebook for Dimensionstyle */
  label=gtk_label_new(_("Dimensionstyle"));
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(dia_style->notebook), pointstyle_create_notebook(dia_style->list), label);
}


void dia_show_style(gint dialog)
{
  DOCUMENT *doc;
  gint page_num=0;

  doc=misc_object_get_document();

  if(doc==NULL)
    return;
  
  if(dia_style==NULL)
    make_dialog_style();

  style_create_document_list(doc);
  
  /* Activate correct notepage */
  switch(dialog)
    {
    case DIALOG_LINESTYLE:
      page_num=0;
      break;
    case DIALOG_POINTSTYLE:
      page_num=1;
      break;
    case DIALOG_TEXTSTYLE:
      page_num=2;
      break;
    case DIALOG_DIMSTYLE:
      page_num=3;
      break;
    }
  gtk_notebook_set_page(GTK_NOTEBOOK(dia_style->notebook), page_num);
  
  /* View window */
  gtk_widget_show(dia_style->window);
}


