/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <string.h>
#include "struct.h"
#include "initmenu.h"
#include "textstyle.h"
#include "misc.h"
#include "dia_message.h"
#include "dia_misc.h"
#include "translation.h"
#include "redraw.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
typedef struct
{
  GtkWidget *vbox8;
  GtkWidget *scrolledwindow4;
  GtkWidget *clist;
  GtkWidget *label23;
  GtkWidget *label24;
  GtkWidget *label25;
  GtkWidget *label26;
  GtkWidget *label27;
  GtkWidget *label28;
  GtkWidget *frame6;
  GtkWidget *table9;
  GtkWidget *hbox6;
  GtkWidget *label29;
  GtkWidget *name_entry;
  GtkWidget *hbox7;
  GtkWidget *label30;
  GtkWidget *type_optionmenu;
  GtkWidget *type_optionmenu_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *hbox8;
  GtkWidget *label31;
  GtkWidget *height_entry;
  GtkWidget *hbox9;
  GtkWidget *label32;
  GtkWidget *width_entry;
  GtkWidget *hbox10;
  GtkWidget *label33;
  GtkWidget *oblique_entry;
  GtkWidget *hbox11;
  GtkWidget *label34;
  GtkWidget *flags_entry;
  GtkWidget *hbox5;
  GtkWidget *hbuttonbox7;
  GtkWidget *update_button;
  GtkWidget *hbuttonbox8;
  GtkWidget *modify_button;
  GtkWidget *add_button;
  GtkWidget *remove_button;
  GtkWidget *label22;
  GtkWidget *hseparator1;
  GtkWidget *hbuttonbox1;
  GtkWidget *button1;
} _TEXTSTYLE;

_TEXTSTYLE *dia_textstyle=NULL;

/* --  P R O T O T Y P E S  -- */

/* --  F U N C T I O N S  -- */
static TEXTSTYLE *new_style;

static void update_new(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  switch(object->type)
    {
    case TYPE_TEXT:
      if(object->data.text.style==new_style)
	{
	  object->selected=OBJECT_NOT_SEL;
	  redraw_object(doc, object);
	}
      break;
    case TYPE_ATTRIBUTE:
      if(object->data.attribute.text.style==new_style)
	{
	  object->selected=OBJECT_NOT_SEL;
	  redraw_object(doc, object);
	}
      break;
    }
}

static void update_objects_new(DOCUMENT *doc, TEXTSTYLE *style)
{
  new_style=style;
  foreach_object(doc, update_new);
}

static void update_old(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  switch(object->type)
    {
    case TYPE_TEXT:
      if(object->data.text.style==new_style)
	{
	  object->selected=OBJECT_ERASE;
	  redraw_object(doc, object);
	}
      break;
    case TYPE_ATTRIBUTE:
      if(object->data.attribute.text.style==new_style)
	{
	  object->selected=OBJECT_ERASE;
	  redraw_object(doc, object);
	}
      break;
    }

}

static void update_objects_old(DOCUMENT *doc, TEXTSTYLE *style)
{
  new_style=style;
  foreach_object(doc, update_old);
}


static void select_row_callback(GtkWidget *widget,gint row,gint column,
				GdkEventButton *event, gpointer data)
{
  DOCUMENT *doc;
  TEXTSTYLE *textstyle;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_textstyle->clist));

  if(doc==NULL)
    return;

  textstyle=gtk_clist_get_row_data(GTK_CLIST(dia_textstyle->clist), row);

  if(textstyle==NULL)
    return;

  /* Update entries */
  gtk_entry_set_text(GTK_ENTRY(dia_textstyle->name_entry), textstyle->name); 
  gtk_entry_set_text(GTK_ENTRY(dia_textstyle->height_entry), g_strdup_printf("%.2f", textstyle->height)); 
  gtk_entry_set_text(GTK_ENTRY(dia_textstyle->width_entry), g_strdup_printf("%.2f", textstyle->width)); 
  gtk_entry_set_text(GTK_ENTRY(dia_textstyle->oblique_entry), g_strdup_printf("%.2f", textstyle->oblique)); 
  gtk_entry_set_text(GTK_ENTRY(dia_textstyle->flags_entry), g_strdup_printf("%d", textstyle->flags)); 
}


static void textstyle_update(DOCUMENT *doc)
{
  GList *list;
  TEXTSTYLE *textstyle;
  gchar *text[6]={NULL, "Standard", NULL, NULL, NULL, NULL};
  gint row;

  if(doc==NULL)
    return;
  if(dia_textstyle==NULL)
    return;
  if(dia_textstyle->clist==NULL)
    return;

  /* Create list for linestyle */
  gtk_clist_clear(GTK_CLIST(dia_textstyle->clist));
  gtk_clist_freeze(GTK_CLIST(dia_textstyle->clist));
  
  list=g_list_first(doc->textstyle_list);

  while(list!=NULL)
    {
      textstyle=(TEXTSTYLE *)list->data;

      text[0]=textstyle->name;

      if(textstyle->height<GFLOAT_SENSITIVE)
	text[2]=g_strdup_printf("0.00 (Ask for)");
      else
	text[2]=g_strdup_printf("%.2f", textstyle->height);

      text[3]=g_strdup_printf("%.2f", textstyle->width);
      text[4]=g_strdup_printf("%.2f", textstyle->oblique);

      text[5]=g_strdup_printf("%d", textstyle->flags);

      row=gtk_clist_append(GTK_CLIST(dia_textstyle->clist), text);
    
      gtk_clist_set_row_data(GTK_CLIST(dia_textstyle->clist), row, (gpointer)textstyle);

      list=g_list_next(list);
    }

  /* Set user data */
  gtk_object_set_user_data(GTK_OBJECT(dia_textstyle->clist), (gpointer)doc);

  /* Mark first row */
  gtk_clist_select_row(GTK_CLIST(dia_textstyle->clist), 0, 0);

  /* Unfreeze list */
  gtk_clist_thaw(GTK_CLIST(dia_textstyle->clist));
}

/* Buttons */
static gint textstyle_update_button(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_textstyle->clist));
  
  if(doc!=NULL)
    textstyle_update(doc);

  return TRUE;
}

static gint textstyle_modify_button(GtkWidget *widget, gpointer data)
{
  GList   *items;
  DOCUMENT *doc;
  TEXTSTYLE *textstyle;
  
  doc=gtk_object_get_user_data(GTK_OBJECT(dia_textstyle->clist));
  
  if(doc==NULL)
    return TRUE;

  /* Get current selected linestyle */
  items=g_list_first(GTK_CLIST(dia_textstyle->clist)->selection);

  if(items!=NULL)
    {
      textstyle=(TEXTSTYLE *)gtk_clist_get_row_data(GTK_CLIST(dia_textstyle->clist), GPOINTER_TO_INT(items->data));

      /* Erase old objects that uses this soon to be modified textstyle */
      update_objects_old(doc, textstyle);

      /* New textstyle name */
      g_free(textstyle->name);
      textstyle->name=g_strdup(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->name_entry)));

      /* Height */
      textstyle->height=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->height_entry)),NULL);

      /* Width */
      textstyle->width=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->width_entry)),NULL);

      /* Oblique */
      textstyle->oblique=g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->oblique_entry)),NULL);

      /* Flags */
      textstyle->flags=(gint) g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->flags_entry)),NULL);

      /* Redraw objects that uses this modified textstyle */
      update_objects_new(doc, textstyle);

      /* Update */
      textstyle_update(doc);

      /* Mark previous row */
      gtk_clist_select_row(GTK_CLIST(dia_textstyle->clist), GPOINTER_TO_INT(items->data), 0);

    }
  return TRUE;
}

static gint textstyle_add_button(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_textstyle->clist));
  
  if(doc==NULL)
    return TRUE;

  if(textstyle_find(doc, gtk_entry_get_text(GTK_ENTRY(dia_textstyle->name_entry)))==NULL)
    {
      textstyle_new(doc, -1, 
		    gtk_entry_get_text(GTK_ENTRY(dia_textstyle->name_entry)), 
		    TEXTSTYLE_OCADIS, 
		    (gint)g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->flags_entry)),NULL),
		    g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->height_entry)),NULL),
		    g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->oblique_entry)),NULL),
		    g_strtod(gtk_entry_get_text(GTK_ENTRY(dia_textstyle->width_entry)),NULL));
      textstyle_update(doc);
    }
  else
    message(ICON_WARNING, _("This textstyle does already exist.\nPlease select another name."));

  return TRUE;
}

static gint textstyle_remove_button(GtkWidget *widget, gpointer data)
{
  GList *items;
  DOCUMENT *doc;
  TEXTSTYLE *textstyle;

  doc=gtk_object_get_user_data(GTK_OBJECT(dia_textstyle->clist));
  
  if(doc==NULL)
    return TRUE;

  /* Get current selected textstyle */
  items=g_list_first(GTK_CLIST(dia_textstyle->clist)->selection);

  if(items!=NULL)
    {
      textstyle=(TEXTSTYLE *)gtk_clist_get_row_data(GTK_CLIST(dia_textstyle->clist), GPOINTER_TO_INT(items->data));

      g_free(textstyle->name);

      /* Remove textstyle from list in document */
      doc->textstyle_list=g_list_remove(doc->textstyle_list, (gpointer)textstyle);

      /* Update */
      textstyle_update(doc);

      /* Mark previous row */
      gtk_clist_select_row(GTK_CLIST(dia_textstyle->clist), MIN(0, GPOINTER_TO_INT(items->data)-1), 0);
    }

  return TRUE;
}

/* Callback for document list */
static void doc_select_row_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
  DOCUMENT *doc;

  /* Find selected document */
  doc=(DOCUMENT *)gtk_clist_get_row_data(GTK_CLIST(data), row);

  /* Update the dialog */
  if(doc!=NULL)
    textstyle_update(doc);
}

GtkWidget *textstyle_create_notebook(GtkWidget *list)
{
  /* Create struct */
  dia_textstyle=g_new(_TEXTSTYLE,1);

  /* Copy & Paste code from glade */
  dia_textstyle->vbox8 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->vbox8);
  gtk_container_set_border_width (GTK_CONTAINER (dia_textstyle->vbox8), 3);

  dia_textstyle->scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (dia_textstyle->scrolledwindow4);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->vbox8), dia_textstyle->scrolledwindow4, TRUE, TRUE, 0);

  dia_textstyle->clist = gtk_clist_new (6);
  gtk_widget_show (dia_textstyle->clist);
  gtk_container_add (GTK_CONTAINER (dia_textstyle->scrolledwindow4), dia_textstyle->clist);
  gtk_clist_set_column_width (GTK_CLIST (dia_textstyle->clist), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (dia_textstyle->clist), 1, 80);
  gtk_clist_set_column_width (GTK_CLIST (dia_textstyle->clist), 2, 80);
  gtk_clist_set_column_width (GTK_CLIST (dia_textstyle->clist), 3, 80);
  gtk_clist_set_column_width (GTK_CLIST (dia_textstyle->clist), 4, 80);
  gtk_clist_set_column_width (GTK_CLIST (dia_textstyle->clist), 5, 80);
  gtk_clist_set_selection_mode(GTK_CLIST(dia_textstyle->clist), GTK_SELECTION_BROWSE);
  gtk_clist_column_titles_show (GTK_CLIST (dia_textstyle->clist));

  /* Add signals */
  gtk_signal_connect(GTK_OBJECT(dia_textstyle->clist), "select_row", GTK_SIGNAL_FUNC(select_row_callback), dia_textstyle->clist);

  dia_textstyle->label23 = gtk_label_new ("Name");
  gtk_widget_show (dia_textstyle->label23);
  gtk_clist_set_column_widget (GTK_CLIST (dia_textstyle->clist), 0, dia_textstyle->label23);

  dia_textstyle->label24 = gtk_label_new ("Type");
  gtk_widget_show (dia_textstyle->label24);
  gtk_clist_set_column_widget (GTK_CLIST (dia_textstyle->clist), 1, dia_textstyle->label24);

  dia_textstyle->label25 = gtk_label_new ("Height");
  gtk_widget_show (dia_textstyle->label25);
  gtk_clist_set_column_widget (GTK_CLIST (dia_textstyle->clist), 2, dia_textstyle->label25);

  dia_textstyle->label26 = gtk_label_new ("Width");
  gtk_widget_show (dia_textstyle->label26);
  gtk_clist_set_column_widget (GTK_CLIST (dia_textstyle->clist), 3, dia_textstyle->label26);

  dia_textstyle->label27 = gtk_label_new ("Oblique");
  gtk_widget_show (dia_textstyle->label27);
  gtk_clist_set_column_widget (GTK_CLIST (dia_textstyle->clist), 4, dia_textstyle->label27);

  dia_textstyle->label28 = gtk_label_new ("Flags");
  gtk_widget_show (dia_textstyle->label28);
  gtk_clist_set_column_widget (GTK_CLIST (dia_textstyle->clist), 5, dia_textstyle->label28);

  dia_textstyle->frame6 = gtk_frame_new ("Properties");
  gtk_widget_show (dia_textstyle->frame6);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->vbox8), dia_textstyle->frame6, FALSE, FALSE, 3);
  gtk_container_set_border_width (GTK_CONTAINER (dia_textstyle->frame6), 3);

  /* Deactivates column titles */
  gtk_clist_column_titles_passive(GTK_CLIST (dia_textstyle->clist));

  dia_textstyle->table9 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (dia_textstyle->table9);
  gtk_container_add (GTK_CONTAINER (dia_textstyle->frame6), dia_textstyle->table9);
  gtk_container_set_border_width (GTK_CONTAINER (dia_textstyle->table9), 3);
  gtk_table_set_col_spacings (GTK_TABLE (dia_textstyle->table9), 10);

  dia_textstyle->hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->hbox6);
  gtk_table_attach (GTK_TABLE (dia_textstyle->table9), dia_textstyle->hbox6, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  dia_textstyle->label29 = gtk_label_new ("Name: ");
  gtk_widget_show (dia_textstyle->label29);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox6), dia_textstyle->label29, FALSE, FALSE, 0);

  dia_textstyle->name_entry = gtk_entry_new ();
  gtk_widget_show (dia_textstyle->name_entry);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox6), dia_textstyle->name_entry, TRUE, TRUE, 0);

  dia_textstyle->hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->hbox7);
  gtk_table_attach (GTK_TABLE (dia_textstyle->table9), dia_textstyle->hbox7, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  dia_textstyle->label30 = gtk_label_new ("Type: ");
  gtk_widget_show (dia_textstyle->label30);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox7), dia_textstyle->label30, FALSE, FALSE, 0);

  dia_textstyle->type_optionmenu = gtk_option_menu_new ();
  gtk_widget_show (dia_textstyle->type_optionmenu);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox7), dia_textstyle->type_optionmenu, FALSE, FALSE, 0);
  dia_textstyle->type_optionmenu_menu = gtk_menu_new ();
  dia_textstyle->glade_menuitem = gtk_menu_item_new_with_label ("Standard");
  gtk_widget_show (dia_textstyle->glade_menuitem);
  gtk_menu_append (GTK_MENU (dia_textstyle->type_optionmenu_menu), dia_textstyle->glade_menuitem);
  dia_textstyle->glade_menuitem = gtk_menu_item_new_with_label ("Cursive");
  gtk_widget_show (dia_textstyle->glade_menuitem);
  gtk_menu_append (GTK_MENU (dia_textstyle->type_optionmenu_menu), dia_textstyle->glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (dia_textstyle->type_optionmenu), dia_textstyle->type_optionmenu_menu);

  dia_textstyle->hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->hbox8);
  gtk_table_attach (GTK_TABLE (dia_textstyle->table9), dia_textstyle->hbox8, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  dia_textstyle->label31 = gtk_label_new ("Height: ");
  gtk_widget_show (dia_textstyle->label31);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox8), dia_textstyle->label31, FALSE, FALSE, 0);

  dia_textstyle->height_entry = gtk_entry_new ();
  gtk_widget_show (dia_textstyle->height_entry);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox8), dia_textstyle->height_entry, TRUE, TRUE, 0);

  dia_textstyle->hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->hbox9);
  gtk_table_attach (GTK_TABLE (dia_textstyle->table9), dia_textstyle->hbox9, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  dia_textstyle->label32 = gtk_label_new ("Width: ");
  gtk_widget_show (dia_textstyle->label32);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox9), dia_textstyle->label32, FALSE, FALSE, 0);

  dia_textstyle->width_entry = gtk_entry_new ();
  gtk_widget_show (dia_textstyle->width_entry);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox9), dia_textstyle->width_entry, TRUE, TRUE, 0);

  dia_textstyle->hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->hbox10);
  gtk_table_attach (GTK_TABLE (dia_textstyle->table9), dia_textstyle->hbox10, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  dia_textstyle->label33 = gtk_label_new ("Oblique: ");
  gtk_widget_show (dia_textstyle->label33);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox10), dia_textstyle->label33, FALSE, FALSE, 0);

  dia_textstyle->oblique_entry = gtk_entry_new ();
  gtk_widget_show (dia_textstyle->oblique_entry);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox10), dia_textstyle->oblique_entry, TRUE, TRUE, 0);

  dia_textstyle->hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->hbox11);
  gtk_table_attach (GTK_TABLE (dia_textstyle->table9), dia_textstyle->hbox11, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  dia_textstyle->label34 = gtk_label_new ("Flags: ");
  gtk_widget_show (dia_textstyle->label34);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox11), dia_textstyle->label34, FALSE, FALSE, 0);

  dia_textstyle->flags_entry = gtk_entry_new ();
  gtk_widget_show (dia_textstyle->flags_entry);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox11), dia_textstyle->flags_entry, TRUE, TRUE, 0);

  dia_textstyle->hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (dia_textstyle->hbox5);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->vbox8), dia_textstyle->hbox5, FALSE, FALSE, 0);

  dia_textstyle->hbuttonbox7 = gtk_hbutton_box_new ();
  gtk_widget_show (dia_textstyle->hbuttonbox7);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox5), dia_textstyle->hbuttonbox7, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dia_textstyle->hbuttonbox7), GTK_BUTTONBOX_START);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dia_textstyle->hbuttonbox7), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (dia_textstyle->hbuttonbox7), 114, -1);

  dia_textstyle->update_button = gtk_button_new_with_label ("Update");
  gtk_signal_connect(GTK_OBJECT(dia_textstyle->update_button),"clicked",
		     GTK_SIGNAL_FUNC(textstyle_update_button), NULL); 
  gtk_widget_show (dia_textstyle->update_button);
  gtk_container_add (GTK_CONTAINER (dia_textstyle->hbuttonbox7), dia_textstyle->update_button);
  GTK_WIDGET_SET_FLAGS (dia_textstyle->update_button, GTK_CAN_DEFAULT);

  dia_textstyle->hbuttonbox8 = gtk_hbutton_box_new ();
  gtk_widget_show (dia_textstyle->hbuttonbox8);
  gtk_box_pack_start (GTK_BOX (dia_textstyle->hbox5), dia_textstyle->hbuttonbox8, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dia_textstyle->hbuttonbox8), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dia_textstyle->hbuttonbox8), 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (dia_textstyle->hbuttonbox8), 114, -1);

  dia_textstyle->modify_button = gtk_button_new_with_label ("Modify");
  gtk_signal_connect(GTK_OBJECT(dia_textstyle->modify_button),"clicked",
		     GTK_SIGNAL_FUNC(textstyle_modify_button), NULL); 
  gtk_widget_show (dia_textstyle->modify_button);
  gtk_container_add (GTK_CONTAINER (dia_textstyle->hbuttonbox8), dia_textstyle->modify_button);
  GTK_WIDGET_SET_FLAGS (dia_textstyle->modify_button, GTK_CAN_DEFAULT);

  dia_textstyle->add_button = gtk_button_new_with_label ("Add");
  gtk_signal_connect(GTK_OBJECT(dia_textstyle->add_button),"clicked",
		     GTK_SIGNAL_FUNC(textstyle_add_button), NULL); 
  gtk_widget_show (dia_textstyle->add_button);
  gtk_container_add (GTK_CONTAINER (dia_textstyle->hbuttonbox8), dia_textstyle->add_button);
  GTK_WIDGET_SET_FLAGS (dia_textstyle->add_button, GTK_CAN_DEFAULT);

  dia_textstyle->remove_button = gtk_button_new_with_label ("Remove");
  gtk_signal_connect(GTK_OBJECT(dia_textstyle->remove_button),"clicked",
		     GTK_SIGNAL_FUNC(textstyle_remove_button), NULL); 
  gtk_widget_show (dia_textstyle->remove_button);
  gtk_container_add (GTK_CONTAINER (dia_textstyle->hbuttonbox8), dia_textstyle->remove_button);
  GTK_WIDGET_SET_FLAGS (dia_textstyle->remove_button, GTK_CAN_DEFAULT);

  /* Attach signal to list so we know when user selects another document */
  gtk_signal_connect(GTK_OBJECT(list), "select_row", GTK_SIGNAL_FUNC(doc_select_row_callback), list);
  
  return dia_textstyle->vbox8;

}

