/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <strings.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "misc.h"
#include "dia_coords.h"
#include "translation.h"

#include "graphics/icons/arrow.xpm"

/* Icons for objects */
#include "graphics/icons/objects/line.xpm"
#include "graphics/icons/objects/pline.xpm"
#include "graphics/icons/objects/ray.xpm"
#include "graphics/icons/objects/construction.xpm"
#include "graphics/icons/objects/spline.xpm"
#include "graphics/icons/objects/arc3p.xpm"
#include "graphics/icons/objects/circleradius.xpm"
#include "graphics/icons/objects/circlediameter.xpm"
#include "graphics/icons/objects/circle2p.xpm"
#include "graphics/icons/objects/circle3p.xpm"
#include "graphics/icons/objects/circlettr.xpm"
#include "graphics/icons/objects/circlettt.xpm"
#include "graphics/icons/objects/circledonut.xpm"
#include "graphics/icons/objects/ellipsecenter.xpm"
#include "graphics/icons/objects/ellipseaxis.xpm"
#include "graphics/icons/objects/ellipsearc.xpm"
#include "graphics/icons/objects/polygon.xpm"
#include "graphics/icons/objects/rectangle.xpm"
#include "graphics/icons/objects/point.xpm"
#include "graphics/icons/objects/text.xpm"

/* Icons for dimensions */
#include "graphics/icons/dimension/dimhoriz.xpm"
#include "graphics/icons/dimension/dimvert.xpm"
#include "graphics/icons/dimension/dimalign.xpm"
#include "graphics/icons/dimension/dimbaseline.xpm"
#include "graphics/icons/dimension/dimcontinue.xpm"
#include "graphics/icons/dimension/dimradius.xpm"
#include "graphics/icons/dimension/dimdiameter.xpm"
#include "graphics/icons/dimension/dimcenter.xpm"
#include "graphics/icons/dimension/dimangular.xpm"
#include "graphics/icons/dimension/dimleader.xpm"

/* Icons for zoom */
#include "graphics/icons/zoom/zoom_redraw.xpm"
#include "graphics/icons/zoom/zoom_previous.xpm"
#include "graphics/icons/zoom/zoom_window.xpm"
#include "graphics/icons/zoom/zoom_pan.xpm"
#include "graphics/icons/zoom/zoom_all.xpm"

/* Icons for osnap */
#include "graphics/icons/osnap/osnap_cen.xpm"
#include "graphics/icons/osnap/osnap_end.xpm"
#include "graphics/icons/osnap/osnap_ins.xpm"
#include "graphics/icons/osnap/osnap_int.xpm"
#include "graphics/icons/osnap/osnap_mid.xpm"
#include "graphics/icons/osnap/osnap_nea.xpm"
#include "graphics/icons/osnap/osnap_nod.xpm"
#include "graphics/icons/osnap/osnap_per.xpm"
#include "graphics/icons/osnap/osnap_qua.xpm"
#include "graphics/icons/osnap/osnap_tan.xpm"
#include "graphics/icons/osnap/osnap_track.xpm"
#include "graphics/icons/osnap/osnap_none.xpm"

/* Icons for construct */
#include "graphics/icons/construct/cons_copy.xpm"
#include "graphics/icons/construct/cons_offset.xpm"
#include "graphics/icons/construct/cons_mirror.xpm"
#include "graphics/icons/construct/cons_array_polar.xpm"
#include "graphics/icons/construct/cons_array_rect.xpm"
#include "graphics/icons/construct/cons_chamfer.xpm"
#include "graphics/icons/construct/cons_fillet.xpm"

/* Icons for modify */
#include "graphics/icons/modify/mod_move.xpm"
#include "graphics/icons/modify/mod_rotate.xpm"
#include "graphics/icons/modify/mod_stretch.xpm"
#include "graphics/icons/modify/mod_scale.xpm"
#include "graphics/icons/modify/mod_lengthen.xpm"
#include "graphics/icons/modify/mod_trim.xpm"
#include "graphics/icons/modify/mod_extend.xpm"
#include "graphics/icons/modify/mod_break.xpm"
#include "graphics/icons/modify/mod_edit_pline.xpm"
#include "graphics/icons/modify/mod_edit_spline.xpm"
#include "graphics/icons/modify/mod_edit_text.xpm"
#include "graphics/icons/modify/mod_edit_prop.xpm"
#include "graphics/icons/modify/mod_explode.xpm"
#include "graphics/icons/modify/mod_erase.xpm"

/* Icons for selections */
#include "graphics/icons/select/select_add.xpm"
#include "graphics/icons/select/select_sub.xpm"
#include "graphics/icons/select/select_sing.xpm"
#include "graphics/icons/select/select_win.xpm"
#include "graphics/icons/select/select_cross.xpm"

/* Icons for measurements */
#include "graphics/icons/measurements/meas_list.xpm"
#include "graphics/icons/measurements/meas_idpoint.xpm"
#include "graphics/icons/measurements/meas_dist.xpm"
#include "graphics/icons/measurements/meas_area.xpm"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* Size definitions */
#define ICON_WIDTH 32
#define ICON_HEIGHT 32
#define TOOLBOX_WIDTH 5
#define TOOLBOX_HEIGHT 4

#define LONGPRESS_TIME 400

/* Struct for holding a toolbox */
typedef struct s_toolbox
{
  GtkWidget *window;
  GtkWidget *table;

  GtkWidget *notebook;
  GtkWidget *create_label;
  GtkWidget *modify_label;
  GtkWidget *meas_label;

  GtkWidget *create_table;
  GList *icongroup_create;

  GtkWidget *modify_table;
  GList *icongroup_modify;

  GtkWidget *meas_table;
  GList *icongroup_meas;

  GtkWidget *zoom_table;
  GList *icongroup_zoom;

  GtkWidget *select_table;
  GList *icongroup_select;

  GtkWidget *osnap_table;
  GList *icongroup_osnap;
} _TOOLBOX;

/* Struct for holding a icon group */
typedef struct s_icongroup
{
  /* Name of this icongroup */
  gchar *name;

  /* Holder of the toolbox button 
     (this is the one the user can see
     and select). */
  GtkWidget *button;

  /* Holder of the iconmenu if this button
     contains many icons */
  GtkWidget *iconmenu;

  /* TRUE if this button contains subicons */
  gboolean manyicons;

  /* List of subicons belonging to this
     buttons */
  GList *icons;

  /* Position in table of this button */
  gint x,y;

  /* Pointer to current icon */
  gint current_icon_no;

  /* Pointer to the table this icongroup belongs to */
  GtkWidget *table;

} _ICONGROUP;

/* Struct for holding a single icon */
typedef struct s_icon
{
  /* Pointer to the icongroup this icons belongs to */
  _ICONGROUP *icongroup;

  gchar *menupath;
  gchar *tooltips;

  gchar **img;
} _ICON;

static _TOOLBOX *dia_toolbox=NULL;

static gboolean visible;

/* --  P R O T O T Y P E S  -- */
gint dia_show_toolbox(GtkWidget *widget, gpointer data);
static GtkWidget *make_toolbox_button(_ICONGROUP *icongroup, _ICON *icon);
static GtkWidget *make_view_icongroup(_ICONGROUP *icongroup);
static gint toolbox_clicked(GtkWidget *widget, GdkEventButton *event, _ICON *icon);


/* --  F U N C T I O N S  -- */
static void change_toolbox_button(_ICON *selicon)
{
  _ICONGROUP *icongroup;
  
  /* Delete old icon and incmenu */
  gtk_widget_destroy(selicon->icongroup->button);
  gtk_widget_destroy(selicon->icongroup->iconmenu);
  
  icongroup=selicon->icongroup;

  /* Create new toolbox button and iconmenu */
  icongroup->button=make_toolbox_button(icongroup, selicon);
  icongroup->iconmenu=make_view_icongroup(icongroup);

  /* Add signals */ 
  gtk_signal_connect (GTK_OBJECT (icongroup->button), "button_press_event",
		      GTK_SIGNAL_FUNC (toolbox_clicked), selicon);
  
  /* Add this to table */
  gtk_table_attach (GTK_TABLE (icongroup->table), icongroup->button,
		    icongroup->x, icongroup->x+1, icongroup->y, icongroup->y+1,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    0, 0);

  /* Set current icon number */
  icongroup->current_icon_no=g_list_index(selicon->icongroup->icons, selicon);
}

void toolbox_set_button(gchar *group, gint no)
{
  gboolean found_icon=FALSE;
  _ICONGROUP *icongroup;
  GList *list;

  if(dia_toolbox==NULL)
    return;

  /* Go through all zoom-icons */
  list=g_list_first(dia_toolbox->icongroup_zoom);

  while(list!=NULL)
    {
      icongroup=(_ICONGROUP *)list->data;

      /* If we found a zoom then stop */
      if(!strcmp( icongroup->name, group))
	{
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), TRUE);
	  found_icon=TRUE;
	}
      else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), FALSE);
            
      list=g_list_next(list);
    }

  if(found_icon)
    return;

  /* Go through all select-icons */
  list=g_list_first(dia_toolbox->icongroup_select);

  /* Special case for Select Add and Select Sub */
  if(!strcmp( group, "Select Add"))
    {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(((_ICONGROUP *)list->data)->button), TRUE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(((_ICONGROUP *)list->next->data)->button), FALSE);
      return;
    }
  else if(!strcmp( group, "Select Sub"))
    {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(((_ICONGROUP *)list->data)->button), FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(((_ICONGROUP *)list->next->data)->button), TRUE);
      return;
    }

  /* Skip the first two icons */
  list=g_list_next(list);
  list=g_list_next(list);

  while(list!=NULL)
    {
      icongroup=(_ICONGROUP *)list->data;

      /* If we found a zoom then stop */
      if(!strcmp( icongroup->name, group))
	{
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), TRUE);
	  found_icon=TRUE;
	}
      else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), FALSE);
            
      list=g_list_next(list);
    }

  if(found_icon)
    return;

  /* Go through all create-icons */
  list=g_list_first(dia_toolbox->icongroup_create);

  while(list!=NULL)
    {
      icongroup=(_ICONGROUP *)list->data;

      if(!strcmp(icongroup->name, group))
	{
	  /* Check if this is the current icon.
	     if not so change the icon. */
	  if(icongroup->current_icon_no!=no)
	    change_toolbox_button((_ICON *)g_list_nth(icongroup->icons,no)->data);

	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), TRUE);
	}
      else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), FALSE);
      
      list=g_list_next(list);
    }

  /* Go through all modify-icons */
  list=g_list_first(dia_toolbox->icongroup_modify);

  while(list!=NULL)
    {
      icongroup=(_ICONGROUP *)list->data;

      if(!strcmp(icongroup->name, group))
	{
	  /* Check if this is the current icon.
	     if not so change the icon. */
	  if(icongroup->current_icon_no!=no)
	    change_toolbox_button((_ICON *)g_list_nth(icongroup->icons,no)->data);

	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), TRUE);
	}
      else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), FALSE);
      
      list=g_list_next(list);
    }

  /* Go through all meas-icons */
  list=g_list_first(dia_toolbox->icongroup_meas);

  while(list!=NULL)
    {
      icongroup=(_ICONGROUP *)list->data;

      if(!strcmp(icongroup->name, group))
	{
	  /* Check if this is the current icon.
	     if not so change the icon. */
	  if(icongroup->current_icon_no!=no)
	    change_toolbox_button((_ICON *)g_list_nth(icongroup->icons,no)->data);

	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), TRUE);
	}
      else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), FALSE);
      
      list=g_list_next(list);
    }
}

static _ICONGROUP *get_icongroup(GList *list, gchar *name)
{
  _ICONGROUP *icongroup;
  
  while(list!=NULL)
    {
      icongroup=(_ICONGROUP *)list->data;
      
      if(!strcmp(name, icongroup->name))
	 return icongroup;

      list=g_list_next(list);
    }

  return NULL;
}

void toolbox_set_osnap(gint osnap)
{
  _ICONGROUP *icongroup;

  if(dia_toolbox==NULL)
    return;
  
  /* Center point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap cen");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_CEN);

  /* End point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap end");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_END);

  /* Insert point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap ins");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_INS);

  /* Intersectionpoint */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap int");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_INT);

  /* Midpoint */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap mid");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_MID);

  /* Nearest point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap nea");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_NEA);

  /* Node point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap nod");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_NOD);

  /* Perpendicular point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap per");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_PER);

  /* Quadrant point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap qua");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_QUA);

  /* Tangent point */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap tan");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_TAN);

  /* Track */
  //  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap track");
  //if(icongroup!=NULL)  
  //  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), osnap & OSNAP_TRACK);

  /* None */
  icongroup=get_icongroup(dia_toolbox->icongroup_osnap, "Osnap none");
  if(icongroup!=NULL)  
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icongroup->button), (osnap==OSNAP_NONE) );
}


static void menuitem_response(_ICON *selicon)
{
  GtkWidget *widget;

  /* Change the button */
  change_toolbox_button(selicon);
  
  /* Activate this icon */
  widget=menus_get_widget(selicon->menupath);

  if(widget!=NULL)
    gtk_signal_emit_by_name(GTK_OBJECT(widget),
			    "activate", NULL);     
}

static GtkWidget *make_view_icongroup(_ICONGROUP *icongroup)
{
  GtkTooltips *tooltips;
  GtkWidget *menu;
  GtkWidget *menu_item;
  GtkWidget *pixmapwid;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkStyle *style; 
  GList *list;
  _ICON *icon;

  /* Make icon list */
  list=g_list_first(icongroup->icons);

  /* Get the style of the button to get the
   * background color. */
  style=gtk_widget_get_style(dia_toolbox->window);
  
  menu=gtk_menu_new();
  gtk_widget_show(menu);
 
  while(list!=NULL)
    {
      icon=(_ICON *)list->data;

      /* Now on to the xpm stuff */
      pixmap=gdk_pixmap_create_from_xpm_d(dia_toolbox->window->window, &mask,
					  &style->bg[GTK_STATE_NORMAL],
					  icon->img);
      pixmapwid=gtk_pixmap_new (pixmap, mask);
      
      gtk_widget_show(pixmapwid);      
      
      menu_item=gtk_menu_item_new();
      gtk_widget_show(menu_item);
      
      gtk_container_add(GTK_CONTAINER(menu_item), pixmapwid);
      
      gtk_menu_append(GTK_MENU(menu), menu_item);
      
      /* Add tooltip to button */
      tooltips=gtk_tooltips_new();
      gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), menu_item, icon->tooltips, icon->tooltips);
      
      gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
				GTK_SIGNAL_FUNC(menuitem_response), (gpointer) icon);

      list=g_list_next(list);
    }
  
  return menu;
}

static gint toolbox_clicked(GtkWidget *widget, GdkEventButton *event, _ICON *icon)
{
  gint x,y;
  GdkModifierType state;
  guint32 st, ct; /* Holder for start time and current time */

  /* Get mouse data */
  gdk_window_get_pointer(event->window, &x, &y, &state);

  /* Get start time */
  ct=st=gdk_time_get();
  
  while(state & GDK_BUTTON1_MASK && !(ct-st>LONGPRESS_TIME))
    {
      /* Get mouse data */
      gdk_window_get_pointer(event->window, &x, &y, &state);

      /* Get current time */
      ct=gdk_time_get();
    }

  /* If long press */
  if((state & GDK_BUTTON1_MASK) && ct-st>LONGPRESS_TIME)
    {
      /* Is there many icons for this group? */
      if(icon->icongroup->manyicons)
	{
	  /* If so show these */
	  gtk_menu_popup(GTK_MENU(icon->icongroup->iconmenu), NULL, NULL, NULL, NULL, event->button, event->time);
	}
      else
	{
	  GtkWidget *widget2;

	  /* Wait for mouse release */
	  gdk_window_get_pointer(event->window, &x, &y, &state);

	  while( !(state & GDK_BUTTON1_MASK))
	    {
	      /* Get mouse data */
	      gdk_window_get_pointer(event->window, &x, &y, &state);
	    }

	  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
				   "button_press_event");
	  widget2=menus_get_widget(icon->menupath);
	  if(widget2!=NULL)
	    gtk_signal_emit_by_name(GTK_OBJECT(widget2),
				    "activate", NULL);
	}
    }
  else
    {
      GtkWidget *widget2;

      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
			      "button_press_event");
      widget2=menus_get_widget(icon->menupath);
      if(widget2!=NULL)
	gtk_signal_emit_by_name(GTK_OBJECT(widget2),
				"activate");
    }

  return TRUE;
}

static GtkWidget *xpm_button(gchar *orig_img[], gboolean manyicons)
{
  GtkWidget *box1;
  GtkWidget *pixmapwid;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkStyle *style;
  
  /* Create box for xpm */
  box1=gtk_hbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER (box1), 0);

  gtk_widget_set_usize(GTK_WIDGET(box1), ICON_WIDTH, ICON_HEIGHT);
  
  /* Get the style of the button to get the
   * background color. */
  style=gtk_widget_get_style(dia_toolbox->window);

  /* Add a small arrow if many icons in this group */
  if(manyicons)
    {
      gint i,j;
      gchar *xpm_img[32+4];

      xpm_img[0]=g_strdup("    32    32        3            1");
      xpm_img[1]=g_strdup("  c None");
      xpm_img[2]=g_strdup("# c #000000");
      xpm_img[3]=g_strdup(". c #ffffff");

      /* Copy the orginal images */
      for(i=0;i<32;i++)
	{
	  xpm_img[i+4]=g_new(gchar, 32);
	  memcpy(xpm_img[i+4], orig_img[i+4], 32);
	}

      /* Overlay orginal with a picture of an arrow */
      for(i=0;i<32;i++)
	for(j=0;j<32;j++)
	  {
	    if(arrow_xpm[i+4][j]!=' ')
	      xpm_img[i+4][j]=arrow_xpm[i+4][j];
	  }

      /* Now on to the xpm stuff */
      pixmap=gdk_pixmap_create_from_xpm_d(dia_toolbox->window->window, &mask,
					  &style->bg[GTK_STATE_NORMAL],
					  xpm_img);
      
      /* Free whole images */
      for(i=0;i<32+4;i++)
	g_free(xpm_img[i]);

    }
  else
    pixmap=gdk_pixmap_create_from_xpm_d(dia_toolbox->window->window, &mask,
				      &style->bg[GTK_STATE_NORMAL],
				      orig_img);

  pixmapwid=gtk_pixmap_new (pixmap, mask);

  /* Pack the pixmap into the box */
  gtk_box_pack_start (GTK_BOX (box1),
		      pixmapwid, FALSE, FALSE, 0);
 

  gtk_widget_show(pixmapwid);
  
  return(box1);
}

static GtkWidget *make_toolbox_button(_ICONGROUP *icongroup, _ICON *icon)
{
  GtkTooltips *tooltips;
  GtkWidget *button;
  GtkWidget *box;
  GtkWidget *alignment;

  /* Create a new button */
  button=gtk_toggle_button_new();
  
  /* Create toolbar button */
  box = xpm_button(icon->img, icongroup->manyicons);
  
  /* Pack and show all our widgets */
  gtk_widget_show(box);

  /* Make picture in box center */
  alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
  gtk_container_border_width (GTK_CONTAINER (alignment), 0);
  gtk_container_add (GTK_CONTAINER (button), alignment);
  gtk_widget_show(alignment);

  gtk_container_add (GTK_CONTAINER (alignment), box);
  
  gtk_widget_set_events(GTK_WIDGET(button), GDK_POINTER_MOTION_MASK|
			GDK_POINTER_MOTION_HINT_MASK|GDK_BUTTON_PRESS_MASK);
  gtk_widget_show(button);

  /* Add tooltip to button */
  tooltips=gtk_tooltips_new();
  gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), button, icon->tooltips, icon->tooltips);

  return button;
}

static GtkWidget *make_views(GList *icongroup_list, gint width, gint height)
     /* This function creates an table with buttons for the icon
	list and also add signals to each button */
{
  GtkWidget *table;
  GList *list;
  gint x,y;
  _ICONGROUP *icongroup;
  _ICON *icon;

  /* Make table */
  table=gtk_table_new (height, width, TRUE);
  gtk_widget_show(table);

  x=0;
  y=0;

  list=g_list_first(icongroup_list);
  
  while(list!=NULL)
    {
      /* Extract current group */
      icongroup=(_ICONGROUP *)list->data;

      /* Update to which table this icongroup belongs */
      icongroup->table=table;

      /* Many icons in this group? */
      icongroup->manyicons=(g_list_length(icongroup->icons)>1);

      /* Save location in table for this icongroup */
      icongroup->x=x;
      icongroup->y=y;

      /* Extract first icon in icon group */
      icongroup->current_icon_no=0;
      icon=(_ICON *)icongroup->icons->data;

      /* Create main button */
      icongroup->button=make_toolbox_button(icongroup, icon);

      /* Add signals */ 
      gtk_signal_connect (GTK_OBJECT (icongroup->button), "button_press_event",
			  GTK_SIGNAL_FUNC (toolbox_clicked), icongroup->icons->data);
      
      /* Add this to table */
      gtk_table_attach (GTK_TABLE (table), icongroup->button,
			icongroup->x, icongroup->x+1, icongroup->y, icongroup->y+1,
			(GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
			(GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
			0, 0);  
      
      /* Make icongroup menu */
      icongroup->iconmenu=make_view_icongroup(icongroup);

      x++;

      if(x>(width-1))
	{
	  x=0;
	  y++;
	}

      list=g_list_next(list);
    }

  return table;
}

static void make_new_icon(_ICONGROUP *group, gchar **xpm_img, gchar *menupath, gchar *tooltips)
{
  _ICON *icon;

  /* Create new icon */
  icon=g_new(_ICON, 1);

  /* Icon data */
  icon->img=xpm_img;

  /* Copy menu path */
  icon->menupath=g_strdup(menupath);

  /* Copy tooltip */
  icon->tooltips=g_strdup(tooltips);

  /* Path to icongroup  */
  icon->icongroup=group;

  /* Append this icon to the icon list */
  group->icons=g_list_append(group->icons, icon);
}


static void add_icon(GList **icongroup_list, gchar *group, gchar **xpm_img, gchar *menupath, gchar *tooltips)
{
  _ICONGROUP *new_icongroup;
  GList *list;

  /* Look for this group */
  list=g_list_first(*icongroup_list);

  while(list!=NULL)
    {
      if(!strcmp( ((_ICONGROUP *)list->data)->name, group))
	{
	  make_new_icon((_ICONGROUP *)list->data, xpm_img, menupath, tooltips);
	  return;
	}

      list=g_list_next(list);
    }


  /* This group doesnt exist => Create it */

  /* Create new struct for icongroup */
  new_icongroup=g_new(_ICONGROUP, 1);  

  /* Copy name */
  new_icongroup->name=g_strdup(group);

  /* Make sure this list with icons is empty */
  new_icongroup->icons=NULL;

  /* Create new icon */
  make_new_icon(new_icongroup, xpm_img, menupath, tooltips);

  /* Append this icon group to main list */
  *icongroup_list=g_list_append(*icongroup_list, new_icongroup);
}

static void make_dialog_toolbox(void)
{
  GtkWidget *hr;

  /* Create struct */
  dia_toolbox=g_new(_TOOLBOX,1);

  /* Create main window */
  dia_toolbox->window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize(dia_toolbox->window);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_toolbox->window), _("oCADis: Toolbox"));

  gtk_window_set_modal(GTK_WINDOW(dia_toolbox->window), FALSE);

  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_toolbox->window),"delete_event",
                     GTK_SIGNAL_FUNC(dia_show_toolbox), NULL);

  /* Make table */
  dia_toolbox->table=gtk_table_new (8, 1, FALSE);
  gtk_container_add(GTK_CONTAINER(dia_toolbox->window),dia_toolbox->table);  
  gtk_widget_show(dia_toolbox->table);

  /* Make notebook and set the position of the tabs */
  dia_toolbox->notebook=gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(dia_toolbox->notebook), GTK_POS_TOP);
  gtk_table_attach (GTK_TABLE (dia_toolbox->table), dia_toolbox->notebook,
		    1, 2, 1, 2,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    3, 3);  
  gtk_widget_show(dia_toolbox->notebook);

  /* Make horizontal line */
  hr=gtk_hseparator_new();
  gtk_widget_show(hr);

  gtk_table_attach (GTK_TABLE (dia_toolbox->table), hr,
		    1, 2, 2, 3,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    0, 0);  

  /* Make page for objects and dimensions */

  /* Make labels for notebook */
  dia_toolbox->create_label=gtk_label_new ( _("Create"));
  gtk_widget_show(dia_toolbox->create_label);
  dia_toolbox->modify_label=gtk_label_new ( _("Modify"));
  gtk_widget_show(dia_toolbox->modify_label);
  dia_toolbox->meas_label=gtk_label_new ( _("Meas"));
  gtk_widget_show(dia_toolbox->meas_label);
   
  /* Set default values */
  dia_toolbox->icongroup_zoom=NULL;
  dia_toolbox->icongroup_create=NULL;
  dia_toolbox->icongroup_modify=NULL;
  dia_toolbox->icongroup_osnap=NULL;
  dia_toolbox->icongroup_select=NULL;
  dia_toolbox->icongroup_meas=NULL;

  /* CREATE THE ICONS IN THE TOOLBOX */

/** O B J E C T S   A N D   D I M E N S I O N **/

  /* Icons for objects */
#include "icon_objects.inc"

  /* Icons for dimension */
#include "icon_dimensions.inc"
  
  /* Create table for objects and dimension */
  dia_toolbox->create_table=make_views(dia_toolbox->icongroup_create, TOOLBOX_WIDTH, TOOLBOX_HEIGHT);

  /* Append to notebook */
  gtk_notebook_append_page(GTK_NOTEBOOK (dia_toolbox->notebook), dia_toolbox->create_table, dia_toolbox->create_label);
  
/** C O N S T R U C T   A N D   M O D I F Y **/

  /* Icons for construct */
#include "icon_construct.inc"

  /* Icons for modify */
#include "icon_modify.inc"

  dia_toolbox->modify_table=make_views(dia_toolbox->icongroup_modify, TOOLBOX_WIDTH, TOOLBOX_HEIGHT);
  
  /* Append to notebook */
  gtk_notebook_append_page(GTK_NOTEBOOK (dia_toolbox->notebook), dia_toolbox->modify_table, dia_toolbox->modify_label);

/** M E A S U R E M E N T S **/
#include "icon_meas.inc"

  dia_toolbox->meas_table=make_views(dia_toolbox->icongroup_meas, TOOLBOX_WIDTH, TOOLBOX_HEIGHT);
  
  /* Append to notebook */
  gtk_notebook_append_page(GTK_NOTEBOOK (dia_toolbox->notebook), dia_toolbox->meas_table, dia_toolbox->meas_label);

/** Z O O M **/

  /* Icons for zoom */
#include "icon_zoom.inc"

  /* Create table for zoom */
  dia_toolbox->zoom_table=make_views(dia_toolbox->icongroup_zoom, 5, 1);  

  /* Attach this table to main window */
  gtk_table_attach (GTK_TABLE (dia_toolbox->table), dia_toolbox->zoom_table,
		    1, 2, 3, 4,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    3, 3);  

 /* Make horizontal line */
  hr=gtk_hseparator_new();
  gtk_widget_show(hr);
  
  gtk_table_attach (GTK_TABLE (dia_toolbox->table), hr,
		    1, 2, 4, 5,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    0, 0);
  
/** S E L E C T **/
#include "icon_select.inc"

  /* Create table for select */
  dia_toolbox->select_table=make_views(dia_toolbox->icongroup_select, 5, 1);  
  
  /* Attach this table to main window */
  gtk_table_attach (GTK_TABLE (dia_toolbox->table), dia_toolbox->select_table,
		    1, 2, 5, 6,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    3, 3);  

  /* Make horizontal line */
  hr=gtk_hseparator_new();
  gtk_widget_show(hr);
  
  gtk_table_attach (GTK_TABLE (dia_toolbox->table), hr,
		    1, 2, 6, 7,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    0, 0);  

/** O B J E C T   S N A P S **/

  /* Icons for osnap */
#include "icon_osnap.inc"

  /* Create table for osnap */
  dia_toolbox->osnap_table=make_views(dia_toolbox->icongroup_osnap, 5, 3);

  /* Attach this table to main window */
  gtk_table_attach (GTK_TABLE (dia_toolbox->table), dia_toolbox->osnap_table,
		    1, 2, 7, 8,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    3, 3);
  
  /* Make horizontal line */
  hr=gtk_hseparator_new();
  gtk_widget_show(hr);
  
  gtk_table_attach (GTK_TABLE (dia_toolbox->table), hr,
		    1, 2, 4, 5,
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
		    0, 0);  

/****************************************/

  /* You are not allowed to resize this window */
  gtk_window_set_policy(GTK_WINDOW(dia_toolbox->window), FALSE, FALSE, FALSE);

  /* View current osnap mode */
  toolbox_set_osnap(gl_osnap);

  /* View current tool */
  UpdateCoordsLabel();

}

gint dia_show_toolbox(GtkWidget *widget, gpointer data)
{
  GtkWidget *menuitem;

  if(dia_toolbox==NULL)
    make_dialog_toolbox();

  menuitem=menus_get_widget( _("/View/Show Toolbox"));      

  if(visible)
    {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(menuitem)->child), _("Show Toolbox"));
      gtk_widget_hide(dia_toolbox->window);
    }
  else
    {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(menuitem)->child), _("Hide Toolbox"));
      gtk_widget_show(dia_toolbox->window);
    }

  visible=!visible;

  return TRUE;
}












