/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include "struct.h"
#include "button.h"
#include "xor.h"
#include "initmenu.h"
#include "struct/config.h"

/* --  G L O B A L   V A R I A B L E S  -- */
#define DIGIT_NO_BUTTON -1

extern CONFIG configuration;  /* Defined in main.c */

/* --  L O C A L   V A R I A B L E S  -- */

#define TABLET_TYPE_MENU 0
#define TABLET_TYPE_TEXT 1

typedef struct s_tablet_button 
{
  gint type;
  gdouble x,y;
  gdouble width,height;
  gchar *path;
  gpointer data;
} TABLET_BUTTON;

typedef struct s_tablet_config
{
  gdouble screen_area[4];
  GList *buttons;
} TABLET_CONFIG;


static TABLET_CONFIG tablet;

static DOCUMENT *digit_doc=NULL;
static gdouble x,y;
static gint button;
static struct termios options;
static int fd;
static gint tag;



void digitizer_read(gpointer data,gint source,GdkInputCondition condition);
void digitizer_move(DOCUMENT *doc, gdouble x, gdouble y);
void digitizer_button(DOCUMENT *doc,gint button, gdouble x, gdouble y);
void tablet_config(void);

/* --  F U N C T I O N S  -- */
gint digitizer_init(void)
{
  fd=open(configuration.path_to_device, O_RDWR | O_NOCTTY | O_NDELAY );
  // fd=open(configuration.path_to_device, O_RDWR | O_NOCTTY );

  if(fd==-1)
    return FALSE;

  /* Setup port: 9600, 7 data, None paritet, stop */
  /* Use Canonical Input */
  tcgetattr(fd, &options);
  
  /* Set baud rate */ 
  cfsetispeed(&options, B9600);
  cfsetospeed(&options, B9600);

  options.c_cflag &= ~PARENB;
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS7;
  options.c_cflag &= ~(CLOCAL | CREAD );
  options.c_lflag = ICANON;
  options.c_iflag=IGNPAR | ICRNL;

  tcsetattr(fd, TCSANOW, &options);

  tag=gdk_input_add(fd,GDK_INPUT_READ,digitizer_read,NULL);

  tablet_config();
  return TRUE;
}

void digitizer_close(void)
{
  gdk_input_remove(tag);
  close(fd);
}

gint digitizer_get_status(gint *tbutton, gdouble *tx, gdouble *ty)
{
  *tbutton=button;
  *tx=x;
  *ty=y; 
  return TRUE;
}

void digitizer_read(gpointer data,gint source,GdkInputCondition condition)
{
  static gdouble ox=0,oy=0;
  static gint obutton=DIGIT_NO_BUTTON;

  gchar buffer[16];

  read(source,buffer,15);

  /* Scan y coord */
  y=g_strtod(&buffer[8],NULL);

  /* Scan x coord */
  buffer[8]='\0';
  x=g_strtod(&buffer[3],NULL);

  /* Scan button */
  if(buffer[2]=='U')
    button=DIGIT_NO_BUTTON;
  else
    button=buffer[2]-'0';

  if(obutton!=DIGIT_NO_BUTTON && button==DIGIT_NO_BUTTON)
    digitizer_button(digit_doc,obutton,x,y);
  else if(ox!=x || oy!=y)
    digitizer_move(digit_doc,x,y);
  
  ox=x;
  oy=y;
  obutton=button;


  g_warning("%f  %f", x,y);
}

gint digitizer_grab(DOCUMENT *doc)
{
  if(digit_doc!=NULL)
    digitizer_close();

  if(!digitizer_init())
    return FALSE;

  digit_doc=doc;

  return TRUE;
}

void digitizer_ungrab(DOCUMENT *doc)
{
  if(digit_doc==doc)
    digitizer_close();
}

void digitizer_move(DOCUMENT *doc,gdouble x, gdouble y)
{
  /* Only reports moves within the fixed area */
  if(x>tablet.screen_area[0] && x<tablet.screen_area[2] && 
     y>tablet.screen_area[3] && y<tablet.screen_area[1])
    {
      
      x-=tablet.screen_area[0];
      y=tablet.screen_area[1]-y;

      x=((gdouble)doc->widgets.draw->allocation.width/(tablet.screen_area[2]-tablet.screen_area[0]))*x;
      y=((gdouble)doc->widgets.draw->allocation.height/(tablet.screen_area[1]-tablet.screen_area[3]))*y;

      mousemove(digit_doc,x,y);
    }
}

void digitizer_button(DOCUMENT *doc,gint button, gdouble x, gdouble y)
{
  GList *but_list;
  TABLET_BUTTON *d;

  /* Only reports moves within the fixed area */
  if(x>tablet.screen_area[0] && x<tablet.screen_area[2] && 
     y>tablet.screen_area[3] && y<tablet.screen_area[1])
    {
      
      x-=tablet.screen_area[0];
      y=tablet.screen_area[1]-y;

      x=((gdouble)doc->widgets.draw->allocation.width/(tablet.screen_area[2]-tablet.screen_area[0]))*x;
      y=((gdouble)doc->widgets.draw->allocation.height/(tablet.screen_area[1]-tablet.screen_area[3]))*y;

      mousebutton(digit_doc,button+1,x,y);     
      return;
    }

  /* Check for symbols */
  but_list=tablet.buttons;

  while(but_list!=NULL)
    {
      d=(TABLET_BUTTON *)but_list->data;

      if(x>(d->x) && x<(d->x+d->width) &&
	 y>(d->y) && y<(d->y+d->height))
	{

	  //	  g_warning("Path : %s ", d->path);

	  gtk_signal_emit_by_name(GTK_OBJECT(menus_get_widget(d->path)),"activate",d->data);
	  return;
	}

      but_list=g_list_next(but_list);
    }
  

}

void add_button(gint type, gdouble x, gdouble y,gdouble w, gdouble h, gchar *path, gpointer data)
{
  TABLET_BUTTON *temp;
  
  temp=(TABLET_BUTTON *)g_new(TABLET_BUTTON,1);

  temp->type=type;
  temp->x=x;
  temp->y=y;
  temp->width=w;
  temp->height=h;
  temp->path=g_strdup(path);
  temp->data=data;

  tablet.buttons=g_list_append(tablet.buttons,temp);
}

void tablet_config(void)
{
  tablet.screen_area[0]=4440;
  tablet.screen_area[1]=7540;
  tablet.screen_area[2]=9080;
  tablet.screen_area[3]=3860;

  tablet.buttons=NULL;

  add_button(TABLET_TYPE_MENU,3624,6891,550,335,"/Objects/Line", NULL);

}







