/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <string.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gdouble fx,fy,sx,sy,tx,ty;

/* --  F U N C T I O N S  -- */
OBJECT *dim_add_aligned(DOCUMENT *doc, LAYER *layer, gchar *name, gchar *style, gint type_colour, COLOUR *colour, gint type, 
			gdouble fx, gdouble fy, gdouble sx, gdouble sy, gdouble tx, gdouble ty,
			gchar *text)
{
  OBJECT *obj;

  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }
  
  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_DIMENSION;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, BYLAYER, NULL, BYLAYER, 0); 

  /* Set object specific attributes */
  if(style==NULL)
    obj->data.dimension.style=doc->current_dimensionstyle;
  else
    obj->data.dimension.style=dimstyle_get_style(doc, style);

  obj->data.dimension.type=DIM_ALIGNED;
  obj->data.dimension.data=NULL;
  obj->data.dimension.type_data=DIM_DATA_SINGEL;

  dim_add_node(obj,fx,fy, tx, ty,text);
  dim_add_node(obj,sx,sy, 0, 0,text);
  
  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);
  
  /* Make bounding box for object */
  dim_bound_dimension(doc, obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void dim_set_prompt_aligned(void)
{
  gchar *t;

  t= g_strdup( _("Dimension text <%.1f>: "));

  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("First extension line origin: "));
      break;
    case 1:
      set_prompt_text( _("Second extension line origin: "));
      break;
    case 2:
      set_prompt_text( _("Dimension line location: "));
      break;
    case 3:
      set_prompt_text(dim_set_decimal(t, work_document->current_dimensionstyle->dimdec), dist(sx,sy,fx,fy));
      break;
    }

  g_free(t);
}

void dim_set_tool_aligned(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimali"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_ALIGNED;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void dim_button_aligned(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_stage=1;
      gl_oldmx=fx=x;
      gl_oldmy=fy=y;
      set_prompt();
      break;
    case 1:
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
      gl_oldmx=sx=x;
      gl_oldmy=sy=y;
      gl_stage=2;
      set_prompt();
      break;
    case 2:
      remove_old();

      if(doc->gl_snap)
	make_snap(doc, &x, &y);

      tx=x;
      ty=y;

      gl_stage=3;
      set_prompt();
      break;
    }
}

void dim_prompt_aligned(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_aligned(doc,1,x,y);
	break;
      }
      break;
    case 1:
    case 2:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_aligned(doc,1,x,y);
	break;
      }
      break;
    case 3:
      if(strcmp(buffer,""))
	dim_draw_dimension(doc, NULL, dim_add_aligned(doc,doc->current_layer,NULL,NULL,doc->type_colour, doc->current_colour, 
						      DIM_ALIGNED, fx,fy,sx,sy,tx,ty,g_strdup(buffer)));
      else
	dim_draw_dimension(doc, NULL, dim_add_aligned(doc,doc->current_layer,NULL,NULL,doc->type_colour, doc->current_colour, 
						      DIM_ALIGNED, fx,fy,sx,sy,tx,ty,NULL));

			 /*g_strdup_printf(dim_set_decimal(t, 
			   doc->current_dimensionstyle->dimdec),dist(fx,fy,sx,sy)))); */
      tool_idle(doc);
      break;
    }
}
void dim_xor_aligned_1(DOCUMENT *doc, gdouble x,gdouble y)
{
  gdouble angle;
  gchar t[40]="%.1f";
  GdkGC *gc;
  
  gc=misc_setup_gc(doc);

  angle=get_angle(fx,fy,x,y)*M_PI/180;

  /* Dimension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,fy),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));

  /* Draw dimension text */
  if(angle>M_PI/2 && angle<3*M_PI/2)
    draw_text(doc,gc,TEXT_JUST_BC,
	      WLDtoSCR_x(doc, (x-fx)/2+fx+doc->current_dimensionstyle->dimgap*cos(angle-M_PI/2)),
	      WLDtoSCR_y(doc,(y-fy)/2+fy-doc->current_dimensionstyle->dimgap*sin(angle-M_PI/2)), 
	      WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,
	      angle-M_PI,
	      g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),dist(fx,fy,x,y)));
  else
    draw_text(doc,gc,TEXT_JUST_BC,
	      WLDtoSCR_x(doc, (x-fx)/2+fx+doc->current_dimensionstyle->dimgap*cos(angle+M_PI/2)),
	      WLDtoSCR_y(doc,(y-fy)/2+fy-doc->current_dimensionstyle->dimgap*sin(angle+M_PI/2)), 
	      WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,
	      angle,
	      g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),dist(fx,fy,x,y)));
}

void dim_xor_aligned_2(DOCUMENT *doc, gdouble x,gdouble y)
{
  gdouble ex1,ey1,ex2,ey2;
  gdouble L,r,d;
  gdouble angle;
  gchar t[40]="%.1f";
  GdkGC *gc;
  
  gc=misc_setup_gc(doc);

  angle=get_angle(fx,fy,sx,sy)*M_PI/180;

  /* Get distance from point to dimension line */
  L=sqrt( (sx-fx)*(sx-fx) + (sy-fy)*(sy-fy) );

  r=((fy-y)*(fy-sy)-(fx-x)*(sx-fx))/(L*L);
  d=L*((fy-y)*(sx-fx)-(fx-x)*(sy-fy))/(L*L);

  ex1=fx+d*cos(angle+M_PI/2);
  ey1=fy-d*sin(angle+M_PI/2);
  ex2=sx+d*cos(angle+M_PI/2);
  ey2=sy-d*sin(angle+M_PI/2);

  /* First extension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,fy),
		WLDtoSCR_x(doc,ex1),WLDtoSCR_y(doc,ey1));

  /* Second extension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,sx),WLDtoSCR_y(doc,sy),
		WLDtoSCR_x(doc,ex2),WLDtoSCR_y(doc,ey2));

  /* Dimension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,ex1),WLDtoSCR_y(doc,ey1),
		WLDtoSCR_x(doc,ex2),WLDtoSCR_y(doc,ey2));

  /* Draw dimension text */
  if(angle>M_PI/2 && angle<3*M_PI/2)
    draw_text(doc,gc,TEXT_JUST_BC,
	      WLDtoSCR_x(doc, (ex2-ex1)/2+ex1+doc->current_dimensionstyle->dimgap*cos(angle-M_PI/2)),
	      WLDtoSCR_y(doc,(ey2-ey1)/2+ey1-doc->current_dimensionstyle->dimgap*sin(angle-M_PI/2)), 
	      WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,
	      angle-M_PI,
	      g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),dist(ex1,ey1,ex2,ey2)));
  else
    draw_text(doc,gc,TEXT_JUST_BC,
	      WLDtoSCR_x(doc, (ex2-ex1)/2+ex1+doc->current_dimensionstyle->dimgap*cos(angle+M_PI/2)),
	      WLDtoSCR_y(doc,(ey2-ey1)/2+ey1-doc->current_dimensionstyle->dimgap*sin(angle+M_PI/2)), 
	      WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,
	      angle,
	      g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),dist(ex1,ey1,ex2,ey2)));

}

void dim_transform_aligned(OBJECT *aligned, TRANSFORM *trans)
{  
  gdouble tx,ty;
  DIMENSION_POINT *tpoint;

  /* Extract points */
  tpoint=(DIMENSION_POINT *)g_list_first(aligned->data.dimension.data)->data;

  misc_transform_point(trans,tpoint->insert_point.x, tpoint->insert_point.y, &tx,&ty);
  tpoint->insert_point.x=tx; 
  tpoint->insert_point.y=ty;

  misc_transform_point(trans,tpoint->text_point.x, tpoint->text_point.y, &tx,&ty);
  tpoint->text_point.x=tx;
  tpoint->text_point.y=ty;

  tpoint=(DIMENSION_POINT *)g_list_first(aligned->data.dimension.data)->next->data;
  
  misc_transform_point(trans,tpoint->insert_point.x, tpoint->insert_point.y, &tx,&ty);
  tpoint->insert_point.x=tx;
  tpoint->insert_point.y=ty;
}

void dim_draw_aligned_block(DOCUMENT *doc, GdkGC *gc, GRDRAW *grdraw, OBJECT *aligned, TRANSFORM *trans)
{
  gdouble L,r,d, angle, sign;
  gdouble ex1,ey1,ex2,ey2;
  gdouble sx,sy,fx,fy, tx,ty;
  gchar *text;
  gdouble text_rot;
  gchar t[40]="%.1f";
  DIMENSION_POINT *tpoint;
  DIMENSIONSTYLE *style;

  /* Just in case Should be removed ASAP! */
  if(grdraw==NULL)
    {
      g_warning("dim_draw_horiz_block: grdraw==NULL => FIX ME!");
      grdraw=&gr_gdk_draw;
    }
  
  style=aligned->data.dimension.style;
  
  /* Extract points */
  tpoint=(DIMENSION_POINT *)g_list_first(aligned->data.dimension.data)->data;

  misc_transform_point(trans,tpoint->insert_point.x, tpoint->insert_point.y, &fx,&fy);
  misc_transform_point(trans,tpoint->text_point.x, tpoint->text_point.y, &tx,&ty);
  text=tpoint->text;

  tpoint=(DIMENSION_POINT *)g_list_first(aligned->data.dimension.data)->next->data;
  
  misc_transform_point(trans,tpoint->insert_point.x, tpoint->insert_point.y, &sx,&sy);

  /* Calc angle of dimension line */
  angle=get_angle(fx,fy,sx,sy)*M_PI/180;

  /* Get distance from point to dimension line */
  L=sqrt( (sx-fx)*(sx-fx) + (sy-fy)*(sy-fy) );

  r=((fy-ty)*(fy-sy)-(fx-tx)*(sx-fx))/(L*L);
  d=L*((fy-ty)*(sx-fx)-(fx-tx)*(sy-fy))/(L*L);

  ex1=fx+d*cos(angle+M_PI/2);
  ey1=fy-d*sin(angle+M_PI/2);
  ex2=sx+d*cos(angle+M_PI/2);
  ey2=sy-d*sin(angle+M_PI/2);

  sign=(d<0?-1:1);

  /* First extension line */
  grdraw->gr_draw_line(doc, gc,
		       fx+style->dimexo*cos(angle+M_PI/2)*sign,
		       fy-style->dimexo*sin(angle+M_PI/2)*sign,
		       ex1+style->dimexe*cos(angle+M_PI/2)*sign,
		       ey1-style->dimexe*sin(angle+M_PI/2)*sign);

  /* Second extension line */
  grdraw->gr_draw_line(doc, gc,
		       sx+style->dimexo*cos(angle+M_PI/2)*sign,
		       sy-style->dimexo*sin(angle+M_PI/2)*sign,
		       ex2+style->dimexe*cos(angle+M_PI/2)*sign,
		       ey2-style->dimexe*sin(angle+M_PI/2)*sign);
  
  /* Dimension line */
  grdraw->gr_draw_line(doc, gc,
		       ex1, ey1,
		       ex2, ey2);
  
  /* Draw arrows */
  grdraw->gr_draw_arrow(doc,gc,ex1,ey1,2*M_PI-angle,style->dimasz);
  grdraw->gr_draw_arrow(doc,gc,ex2,ey2,2*M_PI-angle-M_PI,style->dimasz);

  /* Draw dimension text */
  if(angle>M_PI/2 && angle<3*M_PI/2)
    {
      text_rot=angle-M_PI;
      angle-=M_PI/2;
    }
  else
    {
      text_rot=angle;
      angle+=M_PI/2;
    }
  
  if(text!=NULL)
    grdraw->gr_draw_text(doc,gc,TEXT_JUST_BC,
			 (ex2-ex1)/2+ex1+style->dimgap*cos(angle),
			 (ey2-ey1)/2+ey1-style->dimgap*sin(angle), 
			 style->dimtxt,1,
			 text_rot,
			 text);
    else
      {	  
	text=g_strdup_printf(dim_set_decimal(t,style->dimdec),dist(fx,fy,sx,sy));
	grdraw->gr_draw_text(doc,gc,TEXT_JUST_BC,
			     (ex2-ex1)/2+ex1+style->dimgap*cos(angle),
			     (ey2-ey1)/2+ey1-style->dimgap*sin(angle), 
			     style->dimtxt,1,
			     text_rot,
			     text);
	g_free(text);
      }     
}

void dim_draw_aligned(DOCUMENT *doc, GRDRAW *grdraw, OBJECT *aligned)
{
  dim_draw_aligned_block(doc, misc_setup_gc_object(doc, aligned), grdraw, aligned, NULL);
}























