/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "redraw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "intersect.h"
#include "obj_polyline.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static OBJECT *work_obj1, *work_obj2;
static gdouble gcx, gcy, gsa, gea, gsr, ger, gtr;
static gdouble gsv, gev;


/* This object is for holding information about line in a polyline */
static OBJECT gobj1;
static OBJECT gobj2;

/* This is TRUE if object selected is an arc, false otherwise */
static gboolean gobjarc=FALSE;

/* --  F U N C T I O N S  -- */
OBJECT *dim_add_angular(DOCUMENT *doc, LAYER *layer, gchar *name, gchar *style, gint type_colour, COLOUR *colour,
			gdouble cx, gdouble cy, 			
			gdouble first_angle, gdouble first_radius, gboolean vis1,
			gdouble second_angle, gdouble second_radius, gboolean vis2, 
			gdouble text_radius, gchar *text)
{
  DIMENSION_ANGULAR *tpoint;
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_DIMENSION;

  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, BYLAYER, NULL, BYLAYER, 0); 

  /* Set object specific attributes */
  if(style==NULL)
    obj->data.dimension.style=doc->current_dimensionstyle;
  else
    obj->data.dimension.style=dimstyle_get_style(doc, style);

  obj->data.dimension.type=DIM_ANGULAR;
  obj->data.dimension.data=NULL;
  obj->data.dimension.type_data=DIM_DATA_SINGEL;

  tpoint=g_new(DIMENSION_ANGULAR,1);
  tpoint->centrum_point.x=cx;
  tpoint->centrum_point.y=cy;

  tpoint->first_extension_angle=first_angle;
  tpoint->first_extension_radius=first_radius;
  tpoint->first_extension_visible=vis1;

  tpoint->second_extension_angle=second_angle;
  tpoint->second_extension_radius=second_radius;
  tpoint->second_extension_visible=vis2;

  tpoint->text_radius=text_radius;
  tpoint->text=text;

  obj->data.dimension.data=g_list_append(obj->data.dimension.data, (gpointer)tpoint);
  
  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);
  
  /* Make bounding box for object */
  dim_bound_dimension(doc, obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void dim_set_prompt_angular(void)
{
  gdouble a;
  gchar *t;

  t=g_strdup( _("Dimension text <%.1f>: "));
  
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Select arc, polyline or line: "));
      break;
    case 4:
    case 1:
      set_prompt_text( _("Dimension line location: "));
      break;
    case 2:
      if(gsa<gea)
	a=ABS(gea-gsa);     
      else
	a=360-ABS(gea-gsa);     
      
      set_prompt_text(dim_set_decimal(t, work_document->current_dimensionstyle->dimdec), a);
      break;
    case 3:
      set_prompt_text( _("Select second line or line segment of polyline: "));
      break;
    case 5:
      set_prompt_text( _("Length of first extension line (enter=unvisible): "));
      break;
    case 6:
      set_prompt_text( _("Length of second extension line (enter=unvisible): "));
      break;
    }
  
  g_free(t);
}

void dim_set_tool_angular(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimangular"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_ANGULAR;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
  work_obj1=NULL;
  work_obj2=NULL;
}

void dim_button_angular(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  gdouble px,py;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      deselect_all_objects(doc);            
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }
  switch(gl_stage)
    {
    case 0:
      work_obj1=select_singel_object_without_check(doc,x,y);
      if(work_obj1!=NULL)
	{
	  if(work_obj1->type==TYPE_ARC)
	    {
	      work_obj1->selected=OBJECT_SEL;
	      redraw_object(doc, work_obj1);

	      /* Set the gobal values */
	      gcx=work_obj1->data.arc.centrum.x;
	      gcy=work_obj1->data.arc.centrum.y;
	      gsa=work_obj1->data.arc.start_angle;
	      gsr=work_obj1->data.arc.radius;
	      gea=work_obj1->data.arc.end_angle;
	      ger=work_obj1->data.arc.radius;
	      gsv=TRUE;
	      gev=TRUE;
	      gobjarc=TRUE;
	      gtr=0;	      

	      gl_stage=1;
	    }
	  else if(work_obj1->type==TYPE_LINE)
	    {
	      work_obj1->selected=OBJECT_SEL;
	      redraw_object(doc, work_obj1);
	   
	      /* Set global values */
	      gobjarc=FALSE;

	      gl_stage=3;
	    }
	  else if(work_obj1->type==TYPE_POLYLINE)
	    {
	      gint type;
	      OBJECT_UNION s;

	      work_obj1->selected=OBJECT_SEL;
	      redraw_object(doc, work_obj1);

	      obj_select_polyline_object(work_obj1, x, y, gl_select_sensitivity/doc->zoom, &type, &s);

	      if(type==NODE_ARC)
		{

		  /* Set the gobal values */
		  gcx=s.arc.centrum.x;
		  gcy=s.arc.centrum.y;
		  gsa=s.arc.start_angle;
		  gsr=s.arc.radius;
		  gea=s.arc.end_angle;
		  ger=s.arc.radius;
		  gsv=TRUE;
		  gev=TRUE;
		  gobjarc=TRUE;
		  gtr=0;	      
		  
		  gl_stage=1;
		}
	      else
		{
		  /* Set object */
		  gobj1.data.line.p1.x=s.line.p1.x;
		  gobj1.data.line.p1.y=s.line.p1.y;
		  gobj1.data.line.p2.x=s.line.p2.x;
		  gobj1.data.line.p2.y=s.line.p2.y;

		  work_obj1=&gobj1;

		  /* Set global values */
		  gobjarc=FALSE;
		  
		  gl_stage=3;		  
		}

	    }
	  else
	    print( _("\n  Object must be a line, polyline or an arc."));
	}	  
      else
	print( _("\n  No object found."));
      set_prompt();
      break;
    case 4:
    case 1:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gtr=dist(gcx,gcy,x,y);
      gl_stage=2;
      set_prompt();
      break;
    case 3:
      work_obj2=select_singel_object_without_check(doc,x,y);
      if(work_obj2!=NULL)
	{
	  if(work_obj2->type==TYPE_LINE || work_obj2->type==TYPE_POLYLINE)
	    {
	      gint num=0;
	      POINT intersect;

	      /* Find vertex of polyline */
	      if(work_obj2->type==TYPE_POLYLINE)
		{
		  gint type;
		  OBJECT_UNION s;
		  		  
		  obj_select_polyline_object(work_obj2, x, y, gl_select_sensitivity/doc->zoom, &type, &s);
		  
		  if(type==NODE_ARC)
		    {
		      print( _("\n  Only line segments of polyline allowed."));
		      set_prompt();
		      return;
		    }
		  else
		    {
		      work_obj2->selected=OBJECT_SEL;
		      redraw_object(doc, work_obj2);

		      /* Set object */
		      gobj2.data.line.p1.x=s.line.p1.x;
		      gobj2.data.line.p1.y=s.line.p1.y;
		      gobj2.data.line.p2.x=s.line.p2.x;
		      gobj2.data.line.p2.y=s.line.p2.y;
		      
		      work_obj2=&gobj2;
		      
		      /* Set global values */
		      gobjarc=FALSE;
		      
		      gl_stage=3;		  
		    }
		}
	      else
		{
		  work_obj2->selected=OBJECT_SEL;
		  redraw_object(doc, work_obj2);
		}
	     
	      /* Find intersection point */
	      intersect_line_vs_line_infinite(work_obj1, work_obj2, &num, &intersect);

	      if(num>0)
		{
		  gcx=intersect.x;
		  gcy=intersect.y;

		  /* Set the gobal values */
		  if(dist(gcx, gcy, work_obj1->data.line.p1.x, work_obj1->data.line.p1.y)<
		     dist(gcx, gcy, work_obj1->data.line.p2.x, work_obj1->data.line.p2.y) )
		    {
		      gsr=dist(gcx, gcy, work_obj1->data.line.p2.x, work_obj1->data.line.p2.y);
		      gsa=get_angle(work_obj1->data.line.p1.x, work_obj1->data.line.p1.y, 
				    work_obj1->data.line.p2.x, work_obj1->data.line.p2.y);
		    }
		  else
		    {
		      gsr=dist(gcx, gcy, work_obj1->data.line.p1.x, work_obj1->data.line.p1.y);
		      gsa=get_angle(work_obj1->data.line.p2.x, work_obj1->data.line.p2.y, 
				    work_obj1->data.line.p1.x, work_obj1->data.line.p1.y);
		    }
		  if(dist(gcx, gcy, work_obj2->data.line.p1.x, work_obj2->data.line.p1.y)<
		     dist(gcx, gcy, work_obj2->data.line.p2.x, work_obj2->data.line.p2.y) )
		    {
		      ger=dist(gcx, gcy, work_obj2->data.line.p2.x, work_obj2->data.line.p2.y);
		      gea=get_angle(work_obj2->data.line.p1.x, work_obj2->data.line.p1.y, 
				    work_obj2->data.line.p2.x, work_obj2->data.line.p2.y);
		    }
		  else
		    {
		      ger=dist(gcx, gcy, work_obj2->data.line.p1.x, work_obj2->data.line.p1.y);
		      gea=get_angle(work_obj2->data.line.p2.x, work_obj2->data.line.p2.y, 
				  work_obj2->data.line.p1.x, work_obj2->data.line.p1.y);
		    }

		  gl_stage=4;
		}
	      else  /* No intersect point found */
		{
		  print( _("\n  Lines are parallell."));
		  deselect_all_objects(doc);
		  tool_idle(doc);
		  return;
		}

	    }
	  else
	    print( _("\n  Object must be a line or polyline."));
	}	  
      else
	print( _("\n  No object found."));
      set_prompt();
      break;
    case 5:
      misc_project_point_on_line(gcx+gsr*cos(gsa*M_PI/180.0), gcy-gsr*sin(gsa*M_PI/180.0),
				 gcx+gtr*cos(gsa*M_PI/180.0), gcy-gtr*sin(gsa*M_PI/180.0), 
				 x, y, &px, &py);
      gsr=dist(gcx,gcy,px,py);
      gsv=TRUE;

      gl_stage=6;
      set_prompt();
      break;
    case 6:
      misc_project_point_on_line(gcx+ger*cos(gea*M_PI/180.0), gcy-ger*sin(gea*M_PI/180.0),
				 gcx+gtr*cos(gea*M_PI/180.0), gcy-gtr*sin(gea*M_PI/180.0), 
				 x, y, &px, &py);
      ger=dist(gcx,gcy,px,py);
      gev=TRUE;

      gl_stage=2;
      set_prompt();
      break;
    }
}

void dim_prompt_angular(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx,ty;

  switch(gl_stage)
    {
    case 0:
    case 1:
    case 3:
    case 4:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
        dim_button_angular(doc,1,tx,ty);
	break;
      }
      break;
    case 2:
      if(strcmp(buffer,""))
	dim_draw_dimension(doc, NULL, dim_add_angular(doc,doc->current_layer,NULL,NULL,
						      doc->type_colour, doc->current_colour, 
						      gcx, gcy, 
						      gsa ,gsr, gsv,
						      gea, ger, gev,
						      gtr, g_strdup(buffer)));
      else
	dim_draw_dimension(doc, NULL, dim_add_angular(doc,doc->current_layer,NULL,NULL,
						      doc->type_colour, doc->current_colour, 
						      gcx, gcy, 
						      gsa ,gsr, gsv,
						      gea, ger, gev,
						      gtr, NULL));	 
      deselect_all_objects(doc);      
      tool_idle(doc);
      break;
    case 5:
      break;
    case 6:
      break;
    }
}

void dim_xor_angular_first_line(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble px,py;
  gdouble ta;
  gchar t[40]="%.1f";
  GdkGC *gc;
  gdouble angle;
  
  gc=misc_setup_gc(doc);

  /* Draw first extension line */
  misc_project_point_on_line(gcx+gsr*cos(gsa*M_PI/180.0), gcy-gsr*sin(gsa*M_PI/180.0),
			     gcx+gtr*cos(gsa*M_PI/180.0), gcy-gtr*sin(gsa*M_PI/180.0), 
			     x, y, &px, &py);

  gdk_draw_line(doc->widgets.draw->window, gc, 
		/*WLDtoSCR_x(doc, gcx+gsr*cos(gsa*M_PI/180.0)), 
		  WLDtoSCR_y(doc, gcy-gsr*sin(gsa*M_PI/180.0)), */
		px,py,
		WLDtoSCR_x(doc, gcx+gtr*cos(gsa*M_PI/180.0)),
		WLDtoSCR_y(doc, gcy-gtr*sin(gsa*M_PI/180.0)));
  
  /* Draw second extension line */
  gdk_draw_line(doc->widgets.draw->window, gc,  
		WLDtoSCR_x(doc, gcx+ger*cos(gea*M_PI/180.0)),
		WLDtoSCR_y(doc, gcy-ger*sin(gea*M_PI/180.0)),
		WLDtoSCR_x(doc, gcx+gtr*cos(gea*M_PI/180.0)),
		WLDtoSCR_y(doc, gcy-gtr*sin(gea*M_PI/180.0)));

  /* Draw arc */
  draw_arc(doc->widgets.draw->window,gc, FALSE,
	  WLDtoSCR_x(doc, gcx-gtr),WLDtoSCR_y(doc, gcy-gtr),
	  WLDtoSCR(doc,gtr*2), WLDtoSCR(doc,gtr*2),
	  gsa, gea);  


  /* Draw dimension text */
  /* Find half angle */
  if(gsa<gea)
    {
      ta=ABS(gea-gsa);     
      angle=( gsa+gea )/2;
    }
  else
    {
      ta=360-ABS(gea-gsa);
      angle=((360-gsa)+gea)/2+gsa;
      while(angle>360) angle-=360;
    }
  
  
  if(angle>0 && angle<180)
    draw_text(doc,gc,TEXT_JUST_BC,
	      WLDtoSCR_x(doc, gcx+(gtr+doc->current_dimensionstyle->dimgap)*cos(angle*M_PI/180.0)),
	      WLDtoSCR_y(doc, gcy-(gtr+doc->current_dimensionstyle->dimgap)*sin(angle*M_PI/180.0)),
	      WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,angle*M_PI/180.0-M_PI/2,
	      g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec), ta));
  else
    draw_text(doc,gc,TEXT_JUST_BC,
	      WLDtoSCR_x(doc, gcx+(gtr-doc->current_dimensionstyle->dimgap)*cos(angle*M_PI/180.0)),
	      WLDtoSCR_y(doc, gcy-(gtr-doc->current_dimensionstyle->dimgap)*sin(angle*M_PI/180.0)),
	      WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,angle*M_PI/180.0+M_PI/2,
	      g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec), ta));
}

void dim_xor_angular_second_line(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble px,py;
  gdouble ta;
  gchar t[40]="%.1f";
  GdkGC *gc;
  gdouble angle;
  
  gc=misc_setup_gc(doc);

  /* Draw first extension line */
  gdk_draw_line(doc->widgets.draw->window, gc, 
		gcx+gsr*cos(gsa*M_PI/180.0),
		gcy-gsr*sin(gsa*M_PI/180.0),
		gcx+gtr*cos(gsa*M_PI/180.0),
		gcy-gtr*sin(gsa*M_PI/180.0));

  /* Draw second extension line */
  misc_project_point_on_line(gcx+gsr*cos(gea*M_PI/180.0), gcy-gsr*sin(gea*M_PI/180.0),
			     gcx+gtr*cos(gea*M_PI/180.0), gcy-gtr*sin(gea*M_PI/180.0), 
			     x, y, &px, &py);
  
  gdk_draw_line(doc->widgets.draw->window, gc, 
		px,py,
		WLDtoSCR_x(doc, gcx+gtr*cos(gea*M_PI/180.0)),
		WLDtoSCR_y(doc, gcy-gtr*sin(gea*M_PI/180.0)));
  
  /* Draw arc */
  draw_arc(doc->widgets.draw->window,gc, FALSE,
	  WLDtoSCR_x(doc, gcx-gtr),WLDtoSCR_y(doc, gcy-gtr),
	  WLDtoSCR(doc,gtr*2), WLDtoSCR(doc,gtr*2),
	  gsa, gea);  


 /* Draw dimension text */
 /* Find half angle */
 if(gsa<gea)
   {
     ta=ABS(gea-gsa);     
     angle=( gsa+gea )/2;
   }
 else
   {
     ta=360-ABS(gea-gsa);
     angle=((360-gsa)+gea)/2+gsa;
     while(angle>360) angle-=360;
   }

 
 if(angle>0 && angle<180)
   draw_text(doc,gc,TEXT_JUST_BC,
	     WLDtoSCR_x(doc, gcx+(gtr+doc->current_dimensionstyle->dimgap)*cos(angle*M_PI/180.0)),
	     WLDtoSCR_y(doc, gcy-(gtr+doc->current_dimensionstyle->dimgap)*sin(angle*M_PI/180.0)),
	     WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,angle*M_PI/180.0-M_PI/2,
	     g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec), ta));
 else
   draw_text(doc,gc,TEXT_JUST_BC,
	     WLDtoSCR_x(doc, gcx+(gtr-doc->current_dimensionstyle->dimgap)*cos(angle*M_PI/180.0)),
	     WLDtoSCR_y(doc, gcy-(gtr-doc->current_dimensionstyle->dimgap)*sin(angle*M_PI/180.0)),
	     WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,angle*M_PI/180.0+M_PI/2,
	     g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec), ta));
}

void dim_xor_angular(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble ta;
  gchar t[40]="%.1f";
  GdkGC *gc;
  gdouble angle;

  gc=misc_setup_gc(doc);

  gtr=dist(gcx, gcy, x, y);


  /* Check if to draw extension lines */
  if(!gobjarc)
    {
      gsv=gtr>gsr;
      gev=gtr>ger; 
    }

  /* Draw first extension line */
  if(gsv)
    gdk_draw_line(doc->widgets.draw->window, gc, 
		  WLDtoSCR_x(doc, gcx+gsr*cos(gsa*M_PI/180.0)),
		  WLDtoSCR_y(doc, gcy-gsr*sin(gsa*M_PI/180.0)),
		  WLDtoSCR_x(doc, gcx+gtr*cos(gsa*M_PI/180.0)),
		  WLDtoSCR_y(doc, gcy-gtr*sin(gsa*M_PI/180.0)));
  
  /* Draw second extension line */
  if(gev)
    gdk_draw_line(doc->widgets.draw->window, gc,  
		  WLDtoSCR_x(doc, gcx+ger*cos(gea*M_PI/180.0)),
		  WLDtoSCR_y(doc, gcy-ger*sin(gea*M_PI/180.0)),
		  WLDtoSCR_x(doc, gcx+gtr*cos(gea*M_PI/180.0)),
		  WLDtoSCR_y(doc, gcy-gtr*sin(gea*M_PI/180.0)));
  
  /* Check position of cursor */
  if (!misc_within_angle(gsa, gea, get_angle(gcx, gcy, x,y)))
    {
      gdouble temp;
      temp=gsa;
      gsa=gea;
      gea=temp;
      temp=ger;
      ger=gsr;
      gsr=temp;
    }

  /* Draw arc */
  draw_arc(doc->widgets.draw->window,gc, FALSE,
	  WLDtoSCR_x(doc, gcx-gtr),WLDtoSCR_y(doc, gcy-gtr),
	  WLDtoSCR(doc,gtr*2), WLDtoSCR(doc,gtr*2),
	  gsa, gea);  


 /* Draw dimension text */
 /* Find half angle */
 if(gsa<gea)
   {
     ta=ABS(gea-gsa);     
     angle=( gsa+gea )/2;
   }
 else
   {
     ta=360-ABS(gea-gsa);
     angle=((360-gsa)+gea)/2+gsa;
     while(angle>360) angle-=360;
   }

 
 if(angle>0 && angle<180)
   draw_text(doc,gc,TEXT_JUST_BC,
	     WLDtoSCR_x(doc, gcx+(gtr+doc->current_dimensionstyle->dimgap)*cos(angle*M_PI/180.0)),
	     WLDtoSCR_y(doc, gcy-(gtr+doc->current_dimensionstyle->dimgap)*sin(angle*M_PI/180.0)),
	     WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,angle*M_PI/180.0-M_PI/2,
	     g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec), ta));
 else
   draw_text(doc,gc,TEXT_JUST_BC,
	     WLDtoSCR_x(doc, gcx+(gtr-doc->current_dimensionstyle->dimgap)*cos(angle*M_PI/180.0)),
	     WLDtoSCR_y(doc, gcy-(gtr-doc->current_dimensionstyle->dimgap)*sin(angle*M_PI/180.0)),
	     WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,angle*M_PI/180.0+M_PI/2,
	     g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec), ta));
}

void dim_transform_angular(OBJECT *dim, TRANSFORM *trans)
{
  DIMENSION_ANGULAR *ang;
  gdouble tx,ty;

  ang=(DIMENSION_ANGULAR *)g_list_first(dim->data.dimension.data)->data;

  misc_transform_point(trans, ang->centrum_point.x, ang->centrum_point.y, &tx, &ty);
  ang->centrum_point.x=tx;
  ang->centrum_point.y=ty;
}

void dim_draw_angular_block(DOCUMENT *doc, GdkGC *gc, GRDRAW *grdraw, OBJECT *angular, TRANSFORM *trans)
{
  gchar tr[40]="%.1f";
  gchar *ta;
  gdouble angle;
  gdouble cx,cy;
  gchar *text;
  DIMENSION_ANGULAR *t;
  DIMENSIONSTYLE *style;

  /* Just in case Should be removed ASAP! */
  if(grdraw==NULL)
    {
      g_warning("dim_draw_angular_block: grdraw==NULL => FIX ME!");
      grdraw=&gr_gdk_draw;
    }

  style=angular->data.dimension.style;

  t=(DIMENSION_ANGULAR *)g_list_first(angular->data.dimension.data)->data;
  
  misc_transform_point(trans,t->centrum_point.x, t->centrum_point.y, &cx,&cy);
  text=t->text;

  /* Draw first extension line */
  if(t->first_extension_visible)
    {
      if(t->text_radius>t->first_extension_radius)
        grdraw->gr_draw_line(doc, gc, 
			     cx+(t->first_extension_radius+style->dimexo)*cos(t->first_extension_angle*M_PI/180.0),
			     cy-(t->first_extension_radius+style->dimexo)*sin(t->first_extension_angle*M_PI/180.0),
			     cx+(t->text_radius+style->dimexe)*cos(t->first_extension_angle*M_PI/180.0),
			     cy-(t->text_radius+style->dimexe)*sin(t->first_extension_angle*M_PI/180.0));
      else
        grdraw->gr_draw_line(doc, gc, 
			     cx+(t->first_extension_radius-style->dimexo)*cos(t->first_extension_angle*M_PI/180.0),
			     cy-(t->first_extension_radius-style->dimexo)*sin(t->first_extension_angle*M_PI/180.0),
			     cx+(t->text_radius-style->dimexe)*cos(t->first_extension_angle*M_PI/180.0),
			     cy-(t->text_radius-style->dimexe)*sin(t->first_extension_angle*M_PI/180.0));

    }

  /* Draw second extension line */
  if(t->second_extension_visible)
    {
      if(t->text_radius>t->second_extension_radius)
        grdraw->gr_draw_line(doc, gc,  
			     cx+(t->second_extension_radius+style->dimexo)*cos(t->second_extension_angle*M_PI/180.0),
			     cy-(t->second_extension_radius+style->dimexo)*sin(t->second_extension_angle*M_PI/180.0),
			     cx+(t->text_radius+style->dimexe)*cos(t->second_extension_angle*M_PI/180.0),
			     cy-(t->text_radius+style->dimexe)*sin(t->second_extension_angle*M_PI/180.0));
      else
        grdraw->gr_draw_line(doc, gc,  
			     cx+(t->second_extension_radius-style->dimexo)*cos(t->second_extension_angle*M_PI/180.0),
			     cy-(t->second_extension_radius-style->dimexo)*sin(t->second_extension_angle*M_PI/180.0),
			     cx+(t->text_radius-style->dimexe)*cos(t->second_extension_angle*M_PI/180.0),
			     cy-(t->text_radius-style->dimexe)*sin(t->second_extension_angle*M_PI/180.0));
    }

  /* Draw arc */
  grdraw->gr_draw_arc(doc, gc, cx,cy, t->text_radius, t->first_extension_angle,t->second_extension_angle);

  /* Draw arrows */
  grdraw->gr_draw_arrow(doc, gc, 
			cx+t->text_radius*cos(t->first_extension_angle*M_PI/180.0), 
			cy-t->text_radius*sin(t->first_extension_angle*M_PI/180.0),
			-t->first_extension_angle*M_PI/180.0-M_PI/2, style->dimasz);
  grdraw->gr_draw_arrow(doc, gc, 
			cx+t->text_radius*cos(t->second_extension_angle*M_PI/180.0), 
			cy-t->text_radius*sin(t->second_extension_angle*M_PI/180.0),
			-t->second_extension_angle*M_PI/180.0+M_PI/2, style->dimasz);

  /* Draw dimension text */
  /* Find half angle */

  if(t->first_extension_angle<t->second_extension_angle)
    {
      angle=( t->first_extension_angle+t->second_extension_angle )/2;
    }
  else
    {
      angle=((360-t->first_extension_angle)+t->second_extension_angle)/2+t->first_extension_angle;
      while(angle>360) angle-=360;
    }
  
  if(t->text==NULL)
    {
      gdouble a;

      if(t->first_extension_angle<t->second_extension_angle)
	a=ABS(t->second_extension_angle-t->first_extension_angle);     
      else
	a=360-ABS(t->second_extension_angle-t->first_extension_angle);     
      
      ta=g_strdup_printf(dim_set_decimal(tr, style->dimdec),
			 a);
      
      text=ta;
    }
  else
    {
      ta=NULL;
      text=t->text;
    }

  if(angle>0 && angle<180)
    grdraw->gr_draw_text(doc,gc,TEXT_JUST_BC,
			 cx+(t->text_radius+style->dimgap)*cos(angle*M_PI/180.0),
			 cy-(t->text_radius+style->dimgap)*sin(angle*M_PI/180.0),
			 WLDtoSCR(doc, style->dimtxt),1,angle*M_PI/180.0-M_PI/2,
			 text);
  else
    grdraw->gr_draw_text(doc,gc,TEXT_JUST_BC,
			 cx+(t->text_radius-style->dimgap)*cos(angle*M_PI/180.0),
			 cy-(t->text_radius-style->dimgap)*sin(angle*M_PI/180.0),
			 WLDtoSCR(doc, style->dimtxt),1,angle*M_PI/180.0+M_PI/2,
			 text);
  
  if(ta!=NULL)
    g_free(ta);
}

void dim_draw_angular(DOCUMENT *doc, GRDRAW *grdraw, OBJECT *radius)
{
  dim_draw_angular_block(doc, misc_setup_gc_object(doc, radius), grdraw, radius, NULL);
}
