/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "redraw.h"
#include "calc.h"
#include "draw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "obj_text.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static OBJECT *work_dim;
static gdouble d_length;

/* --  F U N C T I O N S  -- */
void dim_add_baseline_point(OBJECT *dim, gdouble fx, gdouble fy, gchar *text)
{
  dim->data.dimension.type_data=DIM_DATA_BASELINE;

  dim_add_node(dim,fx,fy,0,0,text);
}

void dim_set_prompt_baseline(void)
{
  gchar *t;
  
  t=g_strdup( _("Dimension text <%.1f>: "));

  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Select linear dimension to baseline from: "));
      break;
    case 1:
      set_prompt_text( _("Extension line origin (Return to stop): "));
      break;
    case 2:
      set_prompt_text(dim_set_decimal(t, work_document->current_dimensionstyle->dimdec), d_length);
      break;
    }

  g_free(t);
}

void dim_set_tool_baseline(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimbase"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_BASELINE;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
  work_dim=NULL;
}

void dim_button_baseline(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      if(doc!=NULL)
	{
	  remove_old();
	  deselect_all_objects(doc);
	}
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      work_dim=select_singel_object(doc,x,y);
      if(work_dim!=NULL)
	{
	  if(work_dim->type==TYPE_DIMENSION)
	    {
	      /* Make sure that it is a singel or a baseline dimension */
	      if((work_dim->data.dimension.type==DIM_HORIZ ||
		  work_dim->data.dimension.type==DIM_VERT) &&
		 (work_dim->data.dimension.type_data==DIM_DATA_BASELINE || 
		  work_dim->data.dimension.type_data==DIM_DATA_SINGEL))
		{
		  work_dim->selected=OBJECT_SEL;
		  redraw_object(doc, work_dim);
		  gl_stage++;
		}
	      else
		print( _("\n  The dimension must be a singel or a previously started baseline dimension."));
	    }
	  else
	    print( _("\n  Object must be a linear dimension."));
	}	  
      else
	print( _("\n  No object found."));
      set_prompt();
      break;
    case 1:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);

      /* Test if between two extension lines */
       if( (work_dim->data.dimension.type==DIM_HORIZ && 
	  x > (((DIMENSION_POINT *)g_list_first(work_dim->data.dimension.data)->data)->insert_point.x) &&
	  x < (((DIMENSION_POINT *)g_list_last(work_dim->data.dimension.data)->data)->insert_point.x)) 
	  ||
	  (work_dim->data.dimension.type==DIM_VERT &&
	   y > (((DIMENSION_POINT *)g_list_first(work_dim->data.dimension.data)->data)->insert_point.y) &&
	   y < (((DIMENSION_POINT *)g_list_last(work_dim->data.dimension.data)->data)->insert_point.y)))
	print( _("\n  The origin may not fall between two existing extension lines."));
      else
	{
	  gl_stage=2;
	  gl_oldmx=x;
	  gl_oldmy=y;
	}
       set_prompt();
      break;
    }
}

void dim_prompt_baseline(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_baseline(doc,1,x,y);
	break;
      }
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_RETURN,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_RETURN:
	remove_old();
	work_dim->selected=OBJECT_NOT_SEL;
	redraw_object(doc, work_dim);
	tool_idle(doc);
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_baseline(doc,1,x,y);
	break;
      }
      break;
    case 2:
      remove_old();
      work_dim->selected=OBJECT_ERASE;
      redraw_object(doc, work_dim);
      if(strcmp(buffer,""))
	dim_add_baseline_point(work_dim, gl_oldmx, gl_oldmy,g_strdup(buffer));
      else
	dim_add_baseline_point(work_dim, gl_oldmx, gl_oldmy,NULL);
      work_dim->selected=OBJECT_SEL;
      redraw_object(doc, work_dim);
      /* Make bounding box */
      dim_bound_dimension(doc, work_dim);
      gl_stage=1;
      set_prompt();
      break;
    }
}

void dim_xor_baseline(DOCUMENT *doc, gdouble x,gdouble y)
{
  gdouble tx,ty,offset,r;
  GList *node;
  GdkGC *gc;
  gchar t[40]="%.1f";

  gc=misc_setup_gc(doc);

  offset=(g_list_length(work_dim->data.dimension.data)-1)*doc->current_dimensionstyle->dimdli;

  node=g_list_first(work_dim->data.dimension.data);

  if(work_dim->data.dimension.type==DIM_HORIZ)
    {
      tx=((DIMENSION_POINT *)node->data)->insert_point.x;
      r=((DIMENSION_POINT *)node->data)->insert_point.y;
      ty=((DIMENSION_POINT *)node->data)->text_point.y;
      
      if(ty>r)
	offset=-offset;
      
      /* Extension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,ty-offset),
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
      
      /* Dimension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,tx),WLDtoSCR_y(doc,ty-offset),
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,ty-offset));
      
      /* Draw dimension text */
      d_length=ABS(tx-x);
      draw_text(doc,gc,TEXT_JUST_BC,WLDtoSCR_x(doc, (x-tx)/2+tx),
		WLDtoSCR_y(doc,ty-doc->current_dimensionstyle->dimgap-offset), 
		WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,0,
		g_strdup_printf(dim_set_decimal(t, doc->current_dimensionstyle->dimdec),d_length));
    }
  else
    {
      tx=((DIMENSION_POINT *)node->data)->text_point.x;
      r=((DIMENSION_POINT *)node->data)->insert_point.x;
      ty=((DIMENSION_POINT *)node->data)->insert_point.y;
      
      if(tx>r)
	offset=-offset;
      
      /* Extension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,tx-offset),WLDtoSCR_y(doc,y),
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
      
      /* Dimension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,tx-offset),WLDtoSCR_y(doc,ty),
		    WLDtoSCR_x(doc,tx-offset),WLDtoSCR_y(doc,y));
      
      /* Draw dimension text */
      d_length=ABS(ty-y);
      draw_text(doc,gc,TEXT_JUST_BC,
		WLDtoSCR_x(doc,tx-doc->current_dimensionstyle->dimgap-offset),WLDtoSCR_y(doc, (y-ty)/2+ty), 
		WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,M_PI/2,
		g_strdup_printf(dim_set_decimal(t, doc->current_dimensionstyle->dimdec),d_length));
    }
}






