/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "misc.h"
#include "dimension.h"
#include "draw.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* -- Select dimension lines -- */
static gboolean gl_bound_first;
static gdouble bound_area[4];

/* --  F U N C T I O N S  -- */
static void dim_bound_line(DOCUMENT *doc, GdkGC *gc, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  if(gl_bound_first)
    {
      bound_area[0]=MIN(x1,x2);
      bound_area[1]=MIN(y1,y2);
      bound_area[2]=MAX(x1,x2);
      bound_area[3]=MAX(y1,y2);
    }
  else
    {
      bound_area[0]=MIN(bound_area[0], x1);
      bound_area[0]=MIN(bound_area[0], x2);
      bound_area[1]=MIN(bound_area[1], y1);
      bound_area[1]=MIN(bound_area[1], y2);
      bound_area[2]=MAX(bound_area[2], x1);
      bound_area[2]=MAX(bound_area[2], x2);
      bound_area[3]=MAX(bound_area[3], y1);
      bound_area[3]=MAX(bound_area[3], y2);
    }

  gl_bound_first=FALSE;
}

static void dim_bound_arc(DOCUMENT *doc, GdkGC *gc, gdouble xcen, gdouble ycen, gdouble radius, 
			  gdouble sangle, gdouble eangle)
{
  gdouble sx,sy,ex,ey;
  gint i;

  sx=xcen+radius*cos(sangle*M_PI/180);
  sy=ycen-radius*sin(sangle*M_PI/180);

  ex=xcen+radius*cos(eangle*M_PI/180);
  ey=ycen-radius*sin(eangle*M_PI/180);

  bound_area[0]=MIN(sx,ex);
  bound_area[1]=MIN(sy,ey);
  bound_area[2]=MAX(sx,ex);
  bound_area[3]=MAX(sy,ey);

  /* Check each quadrant */
  for(i=90;i<=360;i+=90)
    {

      if(misc_within_angle(sangle,eangle,i))
	{
	  sx=xcen+radius*cos(i*M_PI/180);
	  sy=ycen-radius*sin(i*M_PI/180);

	  bound_area[0]=MIN(bound_area[0],sx);
	  bound_area[1]=MIN(bound_area[1],sy);
	  bound_area[2]=MAX(bound_area[2],sx);
	  bound_area[3]=MAX(bound_area[3],sy);
	}
    }
}

static void dim_bound_arrow(DOCUMENT *doc, GdkGC *gc, gdouble x, gdouble y, gdouble angle,gdouble size)
{
  gdouble tx,ty;
  gdouble a[3][2];
  gint i;

  tx=x+size*cos(angle);
  ty=y+size*sin(angle);
        
  a[0][0]=x;
  a[0][1]=y;
  a[1][0]=tx+0.4*size*cos(angle+M_PI/2);
  a[1][1]=ty+0.4*size*sin(angle+M_PI/2);
  a[2][0]=tx+0.4*size*cos(angle-M_PI/2);
  a[2][1]=ty+0.4*size*sin(angle-M_PI/2);

  for(i=0; i<3; i++)
    {
      if(gl_bound_first)
	{
	  bound_area[0]=bound_area[2]=a[i][0];
	  bound_area[1]=bound_area[3]=a[i][1];
	}
      else
	{
	  bound_area[0]=MIN(bound_area[0],a[i][0]);
	  bound_area[1]=MIN(bound_area[1],a[i][1]);
	  bound_area[2]=MAX(bound_area[2],a[i][0]);
	  bound_area[3]=MAX(bound_area[3],a[i][1]);
	}

      gl_bound_first=FALSE;
    }
}

static void dim_bound_text(DOCUMENT *doc, GdkGC *gc, gint just, gdouble x, gdouble y, 
			   gdouble height, gdouble width, gdouble rot, gchar *text)
{
  gint type;
  gdouble tx,ty;

  draw_text_set_text(just, x, y, height, width, rot, text);
  
  draw_text_get_vertex(&type, &tx,&ty);

  if(gl_bound_first)
    {
      bound_area[0]=bound_area[2]=tx;
      bound_area[1]=bound_area[3]=ty;
    }
  else
    {
      bound_area[0]=MIN(bound_area[0],tx);
      bound_area[1]=MIN(bound_area[1],ty);
      bound_area[2]=MAX(bound_area[2],tx);
      bound_area[3]=MAX(bound_area[3],ty);
    }

  while(draw_text_get_vertex(&type, &tx,&ty))
    {
      bound_area[0]=MIN(bound_area[0],tx);
      bound_area[1]=MIN(bound_area[1],ty);
      bound_area[2]=MAX(bound_area[2],tx);
      bound_area[3]=MAX(bound_area[3],ty);
    }

  gl_bound_first=FALSE;
}

static GRDRAW gr_bound_draw={dim_bound_line, dim_bound_arc, dim_bound_arrow, dim_bound_text};

/* --  M O R E  F U N C T I O N S  -- */
void dim_bound_dimension(DOCUMENT *doc, OBJECT *obj)
{
  gl_bound_first=TRUE;

  /* Init global variables */

  dim_draw_dimension(doc, &gr_bound_draw, obj);

  obj->bounding_box[0]=bound_area[0];
  obj->bounding_box[1]=bound_area[1];
  obj->bounding_box[2]=bound_area[2];
  obj->bounding_box[3]=bound_area[3];

  /*  gdk_draw_rectangle(doc->widgets.draw->window, doc->xor_gc, FALSE, bound_area[0], bound_area[1], 
      bound_area[2]-bound_area[0], bound_area[3]-bound_area[1] ); */

}









