/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "redraw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "obj_text.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  F U N C T I O N S  -- */
OBJECT *dim_add_center(DOCUMENT *doc, LAYER *layer, gchar *name, gchar *style, gint type_colour, COLOUR *colour,
		       gdouble r, gdouble cx, gdouble cy)
{
  DIMENSION_RADIUS *tpoint;
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_DIMENSION;

  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, BYLAYER, NULL, BYLAYER, 0); 

  /* Set object specific attributes */
  if(style==NULL)
    obj->data.dimension.style=doc->current_dimensionstyle;
  else
    obj->data.dimension.style=dimstyle_get_style(doc, style);

  obj->data.dimension.type=DIM_CENTER;
  obj->data.dimension.data=NULL;
  obj->data.dimension.type_data=DIM_DATA_SINGEL;

  tpoint=g_new(DIMENSION_RADIUS,1);
  tpoint->radius=r;
  tpoint->centrum_point.x=cx;
  tpoint->centrum_point.y=cy;

  obj->data.dimension.data=g_list_append(obj->data.dimension.data, (gpointer)tpoint);
  
  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);
  
  /* Make bounding box for object */
  dim_bound_dimension(doc, obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}


void dim_set_prompt_center(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Select arc or circle: "));
      break;
    }
}

void dim_set_tool_center(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimcenter"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_CENTER;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void dim_button_center(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  OBJECT *work_obj;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      work_obj=select_singel_object(doc,x,y);
      if(work_obj!=NULL)
	{
	  if(work_obj->type==TYPE_CIRCLE)
	    {
	      dim_draw_dimension(doc, NULL, dim_add_center(doc,doc->current_layer,NULL,NULL,
							   doc->type_colour, doc->current_colour,
							   work_obj->data.circle.radius,
							   work_obj->data.circle.centrum.x,
							   work_obj->data.circle.centrum.y));
	      deselect_all_objects(doc);
	      tool_idle(doc);
	      return;
	    }
	  else if(work_obj->type==TYPE_ARC)
	    {
	      dim_draw_dimension(doc, NULL, dim_add_center(doc,doc->current_layer,NULL,NULL,
							   doc->type_colour, doc->current_colour,
							   work_obj->data.arc.radius,
							   work_obj->data.arc.centrum.x,
							   work_obj->data.arc.centrum.y));
	      deselect_all_objects(doc);     
	      tool_idle(doc);
	      return;
	    }
	  else
	    print( _("\n  Object must be a circle or an arc."));
	}	  
      else
	print( _("\n  No object found."));
      set_prompt();
      break;
    }
}

void dim_prompt_center(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;
  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_center(doc,1,x,y);
	break;
      }
      break;
    }
}

void dim_transform_center(OBJECT *center, TRANSFORM *transform)
{
  gdouble tx,ty;
  DIMENSION_RADIUS *t;

  t=(DIMENSION_RADIUS *)g_list_first(center->data.dimension.data)->data;

  misc_transform_point(transform,t->centrum_point.x, t->centrum_point.y, &tx,&ty);
  t->centrum_point.x=tx;
  t->centrum_point.y=ty;

  if(transform!=NULL && (transform->type & OBJ_SCALE))
    t->radius=ABS(t->radius*transform->scale_x);
}

void dim_draw_center_block(DOCUMENT *doc, GdkGC *gc, GRDRAW *grdraw, OBJECT *center, TRANSFORM *trans)
{
  gdouble cx,cy,r;
  DIMENSION_RADIUS *t;
  DIMENSIONSTYLE *style;
  
  /* Just in case Should be removed ASAP! */
  if(grdraw==NULL)
    {
      g_warning("dim_draw_horiz_block: grdraw==NULL => FIX ME!");
      grdraw=&gr_gdk_draw;
    }

  style=center->data.dimension.style;

  t=(DIMENSION_RADIUS *)g_list_first(center->data.dimension.data)->data;
  
  misc_transform_point(trans,t->centrum_point.x, t->centrum_point.y, &cx,&cy);

  r=t->radius;
  if(trans!=NULL && (trans->type & OBJ_SCALE))
    r=ABS(r*trans->scale_x);

  /* Draw center mark */
  if(style->dimmtp==0 || style->dimmtp==1)
    {
      grdraw->gr_draw_line(doc, gc, cx-style->dimmsz/2, cy, cx+style->dimmsz/2, cy);
      grdraw->gr_draw_line(doc, gc, cx, cy-style->dimmsz/2, cx, cy+style->dimmsz/2);
    }
  
  /* Draw extension lines */
  if(style->dimmtp==1 ||  style->dimmtp==2)
    {
      grdraw->gr_draw_line(doc, gc, cx+style->dimmsz*1.5, cy, cx+r+style->dimmsz, cy);
      grdraw->gr_draw_line(doc, gc, cx-style->dimmsz*1.5, cy, cx-r-style->dimmsz, cy);
      grdraw->gr_draw_line(doc, gc, cx, cy+style->dimmsz*1.5, cx, cy+r+style->dimmsz);
      grdraw->gr_draw_line(doc, gc, cx, cy-style->dimmsz*1.5, cx, cy-r-style->dimmsz);
    }
}

void dim_draw_center(DOCUMENT *doc, GRDRAW *grdraw, OBJECT *center)
{
  dim_draw_center_block(doc, misc_setup_gc_object(doc, center), grdraw, center, NULL);
}








