/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "redraw.h"
#include "calc.h"
#include "draw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "obj_text.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static OBJECT *work_dim;
static gdouble d_length;

/* --  F U N C T I O N S  -- */
static void sort_nodes(OBJECT *dim)
{
  gdouble t;
  DIMENSION_POINT *tpoint1, *tpoint2;
  
  tpoint1=(DIMENSION_POINT *)g_list_first(dim->data.dimension.data)->data;
  tpoint2=(DIMENSION_POINT *)g_list_last(dim->data.dimension.data)->data;

  if(dim->data.dimension.type==DIM_HORIZ && dim->data.dimension.type_data==DIM_DATA_SINGEL)
    {
      if(tpoint1->insert_point.x>tpoint2->insert_point.x)
	{
	  t=tpoint1->insert_point.x;
	  tpoint1->insert_point.x=tpoint2->insert_point.x;
	  tpoint2->insert_point.x=t;
	  t=tpoint1->insert_point.y;
	  tpoint1->insert_point.y=tpoint2->insert_point.y;
	  tpoint2->insert_point.y=t;
	}
    }
  else if(dim->data.dimension.type==DIM_VERT && dim->data.dimension.type_data==DIM_DATA_SINGEL)
    {
      if(tpoint1->insert_point.y>tpoint2->insert_point.y)
	{
	  t=tpoint1->insert_point.x;
	  tpoint1->insert_point.x=tpoint2->insert_point.x;
	  tpoint2->insert_point.x=t;
	  t=tpoint1->insert_point.y;
	  tpoint1->insert_point.y=tpoint2->insert_point.y;
	  tpoint2->insert_point.y=t;
	}
    }
}

void dim_add_continue_point(OBJECT *dim, gdouble fx, gdouble fy, gchar *text)
{
  gdouble tx,ty;
  DIMENSION_POINT *tpoint;
  GList *node;

  dim->data.dimension.type_data=DIM_DATA_CONTINUE;

  tpoint=g_new(DIMENSION_POINT,1);
  tpoint->text=NULL;
  tpoint->insert_point.x=fx;
  tpoint->insert_point.y=fy;
  tpoint->text_point.x=0;
  tpoint->text_point.y=0;
  
  node=g_list_first(work_dim->data.dimension.data);

  if(dim->data.dimension.type==DIM_HORIZ)
    {
      tx=((DIMENSION_POINT *)node->data)->insert_point.x;
      ty=((DIMENSION_POINT *)node->data)->text_point.y;
      
      if(fx<tx)
	{
	  tpoint->text_point.x=tx;
	  tpoint->text_point.y=ty;
	  tpoint->text=text;
	  
	  dim->data.dimension.data=g_list_prepend(dim->data.dimension.data, (gpointer)tpoint);
	}
      else
	{
	  node=g_list_last(work_dim->data.dimension.data);
	  
	  tx=((DIMENSION_POINT *)node->data)->insert_point.x;
	  ty=((DIMENSION_POINT *)g_list_previous(node)->data)->text_point.y;
	  
	  if(fx<tx)
	    return;
	  
	  dim->data.dimension.data=g_list_append(dim->data.dimension.data, (gpointer)tpoint);
	  
	  ((DIMENSION_POINT *)node->data)->text_point.x=tx;
	  ((DIMENSION_POINT *)node->data)->text_point.y=ty;
	  ((DIMENSION_POINT *)node->data)->text=text;
	}
    }
  else
    {
      tx=((DIMENSION_POINT *)node->data)->text_point.x;
      ty=((DIMENSION_POINT *)node->data)->insert_point.y;
      
      if(fy<ty)
	{
	  tpoint->text_point.x=tx;
	  tpoint->text_point.y=ty;
	  tpoint->text=text;	 
	  dim->data.dimension.data=g_list_prepend(dim->data.dimension.data, (gpointer)tpoint);
	}
      else
	{
	  node=g_list_last(work_dim->data.dimension.data);
	  
	  tx=((DIMENSION_POINT *)g_list_previous(node)->data)->text_point.x;
	  ty=((DIMENSION_POINT *)node->data)->insert_point.y;
	  
	  if(fy<ty)
	    return;
	  
	  dim->data.dimension.data=g_list_append(dim->data.dimension.data, (gpointer)tpoint);
	  
	  ((DIMENSION_POINT *)node->data)->text_point.x=tx;
	  ((DIMENSION_POINT *)node->data)->text_point.y=ty;
	  ((DIMENSION_POINT *)node->data)->text=text;
	}
    }
}

void dim_set_prompt_continue(void)
{
  gchar *t;

  t=g_strdup( _("Dimension text <%.1f>: "));

  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Select linear dimension to continue from: "));
      break;
    case 1:
      set_prompt_text( _("Extension line origin (enter to stop): "));
      break;
    case 2:
      set_prompt_text(dim_set_decimal(t, work_document->current_dimensionstyle->dimdec), d_length);
      break;
    }

  g_free(t);
}

void dim_set_tool_continue(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimcont"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_CONTINUE;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
  work_dim=NULL;
}

void dim_button_continue(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      if(doc!=NULL)
	{
	  remove_old();
	  deselect_all_objects(doc);	
	}
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      work_dim=select_singel_object(doc,x,y);
      if(work_dim!=NULL)
	{
	  if(work_dim->type==TYPE_DIMENSION)
	    {
	      /* Make sure that it is a singel or a continue dimension */
	      if((work_dim->data.dimension.type==DIM_HORIZ ||
		  work_dim->data.dimension.type==DIM_VERT) &&
		 (work_dim->data.dimension.type_data==DIM_DATA_CONTINUE || 
		  work_dim->data.dimension.type_data==DIM_DATA_SINGEL))
		{
		  sort_nodes(work_dim);
		  work_dim->selected=OBJECT_SEL;
		  redraw_object(doc, work_dim);
		  gl_stage++;
		}
	      else
		print( _("\n  The dimension must be a singel or a previously started continue dimension."));
	    }
	  else
	    print( _("\n  Object must be a linear dimension."));
	}	  
      else
	print( _("\n  No object found."));
      set_prompt();
      break;
    case 1:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);

      /* Test if between two extension lines */
      if( (work_dim->data.dimension.type==DIM_HORIZ && 
	  x > (((DIMENSION_POINT *)g_list_first(work_dim->data.dimension.data)->data)->insert_point.x) &&
	  x < (((DIMENSION_POINT *)g_list_last(work_dim->data.dimension.data)->data)->insert_point.x)) 
	  ||
	  (work_dim->data.dimension.type==DIM_VERT &&
	   y > (((DIMENSION_POINT *)g_list_first(work_dim->data.dimension.data)->data)->insert_point.y) &&
	   y < (((DIMENSION_POINT *)g_list_last(work_dim->data.dimension.data)->data)->insert_point.y)))
	print( _("\n  The origin may not fall between two existing extension lines."));
      else
	{
	  gl_stage=2;
	  gl_oldmx=x;
	  gl_oldmy=y;
	}

      set_prompt();
      break;
    }
}

void dim_prompt_continue(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_continue(doc,1,x,y);
	break;
      }
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_RETURN,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_RETURN:
	remove_old();
	work_dim->selected=OBJECT_NOT_SEL;
	redraw_object(doc, work_dim);
	tool_idle(doc);
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_continue(doc,1,x,y);
	break;
      }
      break;
    case 2:
      remove_old();
      work_dim->selected=OBJECT_ERASE;
      redraw_object(doc, work_dim);
      if(strcmp(buffer,""))
	dim_add_continue_point(work_dim, gl_oldmx, gl_oldmy,g_strdup(buffer));
      else
	dim_add_continue_point(work_dim, gl_oldmx, gl_oldmy,NULL);
      work_dim->selected=OBJECT_SEL;
      redraw_object(doc, work_dim);
      /* Make bounding box */
      dim_bound_dimension(doc, work_dim);
      gl_stage=1;
      set_prompt();
      break;
    }
}

void dim_xor_continue(DOCUMENT *doc, gdouble x,gdouble y)
{
  gchar t[40]="%.1f";
  gdouble tx,ty;
  GList *node;
  GdkGC *gc;
  
  gc=misc_setup_gc(doc);

  node=g_list_first(work_dim->data.dimension.data);

  if(work_dim->data.dimension.type==DIM_HORIZ)
    {
      tx=((DIMENSION_POINT *)node->data)->insert_point.x;
      ty=((DIMENSION_POINT *)node->data)->text_point.y;

      if(x>tx)
	{
	  node=g_list_last(work_dim->data.dimension.data);
      
	  tx=((DIMENSION_POINT *)node->data)->insert_point.x;
	  ty=((DIMENSION_POINT *)g_list_previous(node)->data)->text_point.y;
	  
	  if(x<tx)   /* Mouse between two extension lines */
	    return;
	}
      
      /* Extension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,ty),
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
      
      /* Dimension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,tx),WLDtoSCR_y(doc,ty),
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,ty));
      
      /* Draw dimension text */
      d_length=ABS(tx-x);
      draw_text(doc,gc,TEXT_JUST_BC,WLDtoSCR_x(doc, (x-tx)/2+tx),
		WLDtoSCR_y(doc,ty-doc->current_dimensionstyle->dimgap), 
		WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,0,
		g_strdup_printf(dim_set_decimal(t, doc->current_dimensionstyle->dimdec),d_length));
    }
  else
    {
      tx=((DIMENSION_POINT *)node->data)->text_point.x;
      ty=((DIMENSION_POINT *)node->data)->insert_point.y;

      if(y>ty)
	{
	  node=g_list_last(work_dim->data.dimension.data);
      
	  tx=((DIMENSION_POINT *)g_list_previous(node)->data)->text_point.x;
	  ty=((DIMENSION_POINT *)node->data)->insert_point.y;
	  
	  if(y<ty)   /* Mouse between two extension lines */
	    return;
	}
      
      /* Extension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y),
		    WLDtoSCR_x(doc,tx),WLDtoSCR_y(doc,y));
      
      /* Dimension line */
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,tx),WLDtoSCR_y(doc,y),
		    WLDtoSCR_x(doc,tx),WLDtoSCR_y(doc,ty));
      
      /* Draw dimension text */
      d_length=ABS(ty-y);
      draw_text(doc,gc,TEXT_JUST_BC,WLDtoSCR_x(doc,tx-doc->current_dimensionstyle->dimgap), 
		WLDtoSCR_y(doc, (y-ty)/2+ty), 
		WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,M_PI/2,
		g_strdup_printf(dim_set_decimal(t, doc->current_dimensionstyle->dimdec),d_length));
    }
}





