/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gdouble fx,fy,sx,sy,tx,ty;

/* --  F U N C T I O N S  -- */
OBJECT *dim_add_horiz(DOCUMENT *doc, LAYER *layer, gchar *name, gchar *style, gint type_colour, COLOUR *colour, gint type, 
		      gdouble fx, gdouble fy, gdouble sx, gdouble sy, gdouble tx, gdouble ty,
		      gchar *text)
{
  OBJECT *obj;

  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_DIMENSION;

  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, BYLAYER, NULL, BYLAYER, 0); 

  /* Set object specific attributes */
  if(style==NULL)
    obj->data.dimension.style=doc->current_dimensionstyle;
  else
    obj->data.dimension.style=dimstyle_get_style(doc, style);

  obj->data.dimension.type=DIM_HORIZ;
  obj->data.dimension.data=NULL;
  obj->data.dimension.type_data=DIM_DATA_SINGEL;

  dim_add_node(obj, fx, fy, tx, ty, text);
  dim_add_node(obj, sx, sy, 0, 0, text);

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);
  
  /* Make bounding box for object */
  dim_bound_dimension(doc, obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void dim_set_prompt_horiz(void)
{
  gchar *t;

  t=g_strdup( _("Dimension text <%.1f>: "));

  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("First extension line origin: "));
      break;
    case 1:
      set_prompt_text( _("Second extension line origin: "));
      break;
    case 2:
      set_prompt_text( _("Dimension line location: "));
      break;
    case 3:
      set_prompt_text(dim_set_decimal(t, work_document->current_dimensionstyle->dimdec), ABS(sx-fx));
      break;
    }

  g_free(t);
}

void dim_set_tool_horiz(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimhor"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_HORIZ;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void dim_button_horiz(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_stage=1;
      gl_oldmx=fx=x;
      gl_oldmy=fy=y;
      set_prompt();
      break;
    case 1:
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
      gl_oldmx=sx=x;
      gl_oldmy=sy=y;
      gl_stage=2;
      set_prompt();
      break;
    case 2:
      remove_old();

      if(doc->gl_snap)
	make_snap(doc, &x, &y);

      tx=x;
      ty=y;

      gl_stage=3;
      set_prompt();
      break;
    }
}

void dim_prompt_horiz(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_horiz(doc,1,x,y);
	break;
      }
      break;
    case 1:
    case 2:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_horiz(doc,1,x,y);
	break;
      }
      break;
    case 3:
      if(strcmp(buffer,""))
	dim_draw_dimension(doc, NULL, dim_add_horiz(doc,doc->current_layer,NULL, NULL, doc->type_colour, doc->current_colour, 
						    DIM_HORIZ, fx,fy,sx,sy,tx,ty,g_strdup(buffer)));
      else
	dim_draw_dimension(doc, NULL, dim_add_horiz(doc,doc->current_layer,NULL,NULL, doc->type_colour, doc->current_colour, 
						    DIM_HORIZ, fx,fy,sx,sy,tx,ty, NULL));

      tool_idle(doc);
      break;
    }
}
void dim_xor_horiz_1(DOCUMENT *doc, gdouble x,gdouble y)
{
  gchar t[40]="%.1f";
  GdkGC *gc;
  
  gc=misc_setup_gc(doc);

  /* First extension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,fy),
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,y));

  /* Dimension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,y),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));

  /* Draw dimension text */
  draw_text(doc,gc,TEXT_JUST_BC,WLDtoSCR_x(doc, (x-fx)/2+fx),
	    WLDtoSCR_y(doc,y-doc->current_dimensionstyle->dimgap), WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,0,
	    g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),ABS(fx-x)));
}

void dim_xor_horiz_2(DOCUMENT *doc, gdouble x,gdouble y)
{
  gchar t[40]="%.1f";
  GdkGC *gc;
  
  gc=misc_setup_gc(doc);

  /* First extension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,fy),
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,y));

  /* Second extension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,sx),WLDtoSCR_y(doc,sy),
		WLDtoSCR_x(doc,sx),WLDtoSCR_y(doc,y));
 
  /* Dimension line */
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,sx),WLDtoSCR_y(doc,y),
		WLDtoSCR_x(doc,fx),WLDtoSCR_y(doc,y));

  /* Draw dimension text */
  draw_text(doc,gc,TEXT_JUST_BC,WLDtoSCR_x(doc, (sx-fx)/2+fx),
	    WLDtoSCR_y(doc,y-doc->current_dimensionstyle->dimgap),
	    WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,0,
	    g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),ABS(fx-sx)));
}

void dim_transform_horiz(OBJECT *horiz, TRANSFORM *trans)
{
  gdouble tx,ty;
  DIMENSION_POINT *t;
  GList *p;

  p=g_list_first(horiz->data.dimension.data);

  t=(DIMENSION_POINT *)p->data;
  misc_transform_point_no_rotate(trans,t->insert_point.x, t->insert_point.y, &tx,&ty);
  t->insert_point.x=tx;
  t->insert_point.y=ty;
  misc_transform_point_no_rotate(trans,t->text_point.x, t->text_point.y, &tx,&ty);
  t->text_point.x=tx;
  t->text_point.y=ty;

  p=g_list_next(p);

  while(p!=NULL)
    {
      t=(DIMENSION_POINT *)p->data;      
           
      misc_transform_point_no_rotate(trans,t->insert_point.x, t->insert_point.y, &tx,&ty);
      t->insert_point.x=tx;
      t->insert_point.y=ty;
      p=g_list_next(p);
    }
}

void dim_draw_horiz_baseline(DOCUMENT *doc, GdkGC *gc, GRDRAW *grdraw, OBJECT *horiz, TRANSFORM *trans)
{
  gdouble sign;
  gdouble Ax,Ay, Bx, By,Cx,Cy;
  GList *p;
  DIMENSION_POINT *dim;
  gchar *text;
  DIMENSIONSTYLE *style;
  gchar t[40]="%.1f";

  style=horiz->data.dimension.style;
  p=g_list_first(horiz->data.dimension.data);

  dim=(DIMENSION_POINT *)p->data;
  misc_transform_point_no_rotate(trans,dim->insert_point.x, dim->insert_point.y, &Ax,&Ay);
  misc_transform_point_no_rotate(trans,dim->text_point.x, dim->text_point.y, &Cx,&Cy);

  if(Cy<Ay)
    sign=1;
  else
    sign=-1;

  p=g_list_next(p);

  while(p!=NULL)
    {
      dim=(DIMENSION_POINT *)p->data;      
           
      misc_transform_point_no_rotate(trans,dim->insert_point.x, dim->insert_point.y, &Bx,&By);
      text=dim->text;

      /* Dimension line */
      grdraw->gr_draw_line(doc, gc, Ax, Cy, Bx, Cy);

      /* Extension line */
      if(Cy<By)
	grdraw->gr_draw_line(doc, gc, Bx, Cy-style->dimexe, Bx, By-style->dimexo);
      else
	grdraw->gr_draw_line(doc, gc, Bx, Cy+style->dimexe, Bx, By+style->dimexo);

      /* Draw dimension text */
      if(text!=NULL)
	grdraw->gr_draw_text(doc, gc, TEXT_JUST_BC,(Bx-Ax)/2+Ax, Cy-style->dimgap, style->dimtxt, 1, 0, text);
      else
	{	  
	  text=g_strdup_printf(dim_set_decimal(t,style->dimdec),ABS(Ax-Bx));
	  grdraw->gr_draw_text(doc, gc, TEXT_JUST_BC, (Bx-Ax)/2+Ax, Cy-style->dimgap, style->dimtxt, 1, 0, text);
	  g_free(text);
	}

      /* Draw arrows */
      if(Ax<Bx)
	{
	  grdraw->gr_draw_arrow(doc,gc,Ax,Cy,0,style->dimasz);
	  grdraw->gr_draw_arrow(doc,gc,Bx,Cy,M_PI,style->dimasz);
	}
      else
	{
	  grdraw->gr_draw_arrow(doc,gc,Bx,Cy,0,style->dimasz);
	  grdraw->gr_draw_arrow(doc,gc,Ax,Cy,M_PI,style->dimasz);
	}

      Cy-=style->dimdli*sign;
      p=g_list_next(p);
    }

  /* Extension line */
  Cy+=style->dimdli*sign;
  if(Cy<Ay)
    grdraw->gr_draw_line(doc, gc, Ax, Cy-style->dimexe, Ax, Ay-style->dimexo);
  else
    grdraw->gr_draw_line(doc, gc, Ax, Cy+style->dimexe, Ax, Ay+style->dimexo);
}

void dim_draw_horiz_continue(DOCUMENT *doc, GdkGC *gc, GRDRAW *grdraw, OBJECT *horiz, TRANSFORM *trans)
{
  gdouble Ax, Ay, Bx, By,Cx,Cy;
  GList *p;
  DIMENSION_POINT *dim;
  gchar *text;
  DIMENSIONSTYLE *style;
  gchar t[40]="%.1f";

  style=horiz->data.dimension.style;
  
  p=g_list_first(horiz->data.dimension.data);

  dim=(DIMENSION_POINT *)p->data;
  misc_transform_point_no_rotate(trans,dim->insert_point.x, dim->insert_point.y, &Ax,&Ay);
  misc_transform_point_no_rotate(trans,dim->text_point.x, dim->text_point.y, &Cx,&Cy);
  text=dim->text;

  p=g_list_next(p);

  while(p!=NULL)
    {
      dim=(DIMENSION_POINT *)p->data;      
      
      misc_transform_point_no_rotate(trans,dim->insert_point.x, dim->insert_point.y, &Bx,&By);

      /* First extension line */
      if(Cy<Ay)
	grdraw->gr_draw_line(doc, gc, Ax, Ay-style->dimexo, Ax, Cy-style->dimexe);
      else
	grdraw->gr_draw_line(doc, gc, Ax, Ay+style->dimexo, Ax, Cy+style->dimexe);
      
      /* Second extension line */
      if(Cy<By)
	grdraw->gr_draw_line(doc, gc, Bx, By-style->dimexo, Bx, Cy-style->dimexe);
      else
	grdraw->gr_draw_line(doc, gc, Bx, By+style->dimexo, Bx, Cy+style->dimexe);

      /* Dimension line */
      grdraw->gr_draw_line(doc, gc, Bx, Cy, Ax, Cy);

      /* Draw arrows */
      if(Ax<Bx)
	{
	  grdraw->gr_draw_arrow(doc,gc,Ax,Cy,0,style->dimasz);
	  grdraw->gr_draw_arrow(doc,gc,Bx,Cy,M_PI,style->dimasz);
	}
      else
	{
	  grdraw->gr_draw_arrow(doc,gc,Ax,Cy,M_PI,style->dimasz);
	  grdraw->gr_draw_arrow(doc,gc,Bx,Cy,0,style->dimasz);
	}
      
      /* Draw dimension text */
      if(text!=NULL)
	grdraw->gr_draw_text(doc, gc, TEXT_JUST_BC, (Bx-Ax)/2+Ax, Cy-style->dimgap, style->dimtxt, 
			     1, 0, text);
      else
	{	  
	  text=g_strdup_printf(dim_set_decimal(t,style->dimdec),ABS(Ax-Bx));
	  grdraw->gr_draw_text(doc, gc, TEXT_JUST_BC,(Bx-Ax)/2+Ax, Cy-style->dimgap, style->dimtxt,
			       1, 0, text);
	  g_free(text);
	}

      /* Get the next point in list */
      Ax=Bx;
      Ay=By;

      text=dim->text;
     
      p=g_list_next(p);
    }
}


void dim_draw_horiz_block(DOCUMENT *doc, GdkGC *gc, GRDRAW *gr_draw, OBJECT *horiz, TRANSFORM *trans)
{
  /* Just in case Should be removed ASAP! */
  if(gr_draw==NULL)
    {
      g_warning("dim_draw_horiz_block: grdraw==NULL => FIX ME!");
      gr_draw=&gr_gdk_draw;
    }
  
  switch(horiz->data.dimension.type_data)
    {
    case DIM_DATA_SINGEL:
    case DIM_DATA_CONTINUE:
      dim_draw_horiz_continue(doc, gc, gr_draw, horiz, trans);
      break;
    case DIM_DATA_BASELINE:
      dim_draw_horiz_baseline(doc, gc, gr_draw, horiz, trans);
      break;
    }
}

void dim_draw_horiz(DOCUMENT *doc, GRDRAW *gr_draw, OBJECT *horiz)
{
  dim_draw_horiz_block(doc,misc_setup_gc_object(doc, horiz), gr_draw, horiz,NULL);
}
