/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "redraw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static OBJECT *work_dim;

/* --  F U N C T I O N S  -- */
DIMENSION_RADIUS *add_leader_node(OBJECT *obj, gdouble x, gdouble y)
{
  DIMENSION_RADIUS *tpoint;

  tpoint=g_new(DIMENSION_RADIUS,1);
  tpoint->insert_point.x=x;
  tpoint->insert_point.y=y; 
  tpoint->text=NULL;

  obj->data.dimension.data=g_list_append(obj->data.dimension.data, (gpointer)tpoint);

  dim_bound_dimension(obj->layer->doc, obj);

  return tpoint;
}

OBJECT *dim_add_leader(DOCUMENT *doc, LAYER *layer, gchar *name, gchar *style,
		       gint type_colour, COLOUR *colour)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_DIMENSION;

  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, BYLAYER, NULL, BYLAYER, 0); 

  /* Set object specific attributes */
  if(style==NULL)
    obj->data.dimension.style=doc->current_dimensionstyle;
  else
    obj->data.dimension.style=dimstyle_get_style(doc, style);

  obj->data.dimension.type=DIM_LEADER;
  obj->data.dimension.data=NULL;
  obj->data.dimension.type_data=DIM_DATA_SINGEL;

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);
  
  /* Make bounding box for object */
  dim_bound_dimension(doc, obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}


void dim_set_prompt_leader(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Start Point: "));
      break;
    case 1:
      set_prompt_text( _("Next Point (enter to stop): "));
      break;
    case 2:
      set_prompt_text( _("Text: "));
      break;
    }
}

void dim_set_tool_leader(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimleader"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_LEADER;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
  work_dim=NULL;
}

void dim_button_leader(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      if(gl_stage==0 || gl_stage==2)
	{
	  remove_old();
	  print( _("*Cancel*"));
	  tool_idle(doc);
	  return;
	}
      else
	{
	  remove_old();
	  gl_stage=2;
	  set_prompt();
	}
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);

      work_dim=dim_add_leader(doc,doc->current_layer,NULL,NULL,doc->type_colour, doc->current_colour);
      add_leader_node(work_dim,x,y);

      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      add_leader_node(work_dim, x,y);

      gl_oldmx=x;
      gl_oldmy=y;
      set_prompt();
      break;
    }
}

void dim_prompt_leader(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_leader(doc,1,x,y);
	break;
      }
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&x,&y,NULL,NULL)){
      case PROMPT_RETURN:
	remove_old();
	gl_stage=2;
	set_prompt();
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_leader(doc,1,x,y);
	break;
      }
    case 2:
      remove_old();

      if(strcmp(buffer,""))
      	((DIMENSION_RADIUS *)g_list_last(work_dim->data.dimension.data)->data)->text=g_strdup(buffer);

      dim_bound_dimension(doc, work_dim);
      dim_draw_dimension(doc, NULL, work_dim);
      doc->saved=FALSE;
      make_window_title(doc);
      tool_idle(doc);
      break;
    }
}

void dim_xor_leader(DOCUMENT *doc,gdouble x, gdouble y)
{
  GdkGC *gc;
  DIMENSION_RADIUS *data;

  data=add_leader_node(work_dim, x,y);

  gc=misc_setup_gc(doc);
  dim_draw_dimension_block(doc,gc,work_dim,NULL);

  work_dim->data.dimension.data=g_list_remove(work_dim->data.dimension.data,data);
}

void dim_transform_leader(OBJECT *leader, TRANSFORM *transform)
{
  gdouble tx,ty;
  GList *list;

  list=g_list_first(leader->data.dimension.data);

  while(list!=NULL)
    {
      misc_transform_point(transform,((DIMENSION_RADIUS *)list->data)->insert_point.x,
			   ((DIMENSION_RADIUS *)list->data)->insert_point.y,
			   &tx, &ty);
      ((DIMENSION_RADIUS *)list->data)->insert_point.x=tx;
      ((DIMENSION_RADIUS *)list->data)->insert_point.y=ty;

      list=g_list_next(list);
    }
}

void dim_draw_leader_block(DOCUMENT *doc, GdkGC *gc, GRDRAW *grdraw, OBJECT *leader, TRANSFORM *transform)
{
  GList *list;
  gdouble ox,oy,x,y;
  DIMENSIONSTYLE *style;

  /* Just in case Should be removed ASAP! */
  if(grdraw==NULL)
    {
      g_warning("dim_draw_horiz_block: grdraw==NULL => FIX ME!");
      grdraw=&gr_gdk_draw;
    }

  style=leader->data.dimension.style;

  list=g_list_first(leader->data.dimension.data);

  /* We have no nothing to draw */
  if(list==NULL)
     return;

  misc_transform_point(transform,((DIMENSION_RADIUS *)list->data)->insert_point.x,
		       ((DIMENSION_RADIUS *)list->data)->insert_point.y,
		       &ox, &oy);

  list=g_list_next(list);

  while(list!=NULL)
    {
      misc_transform_point(transform,((DIMENSION_RADIUS *)list->data)->insert_point.x,
			   ((DIMENSION_RADIUS *)list->data)->insert_point.y,
			   &x, &y);

      grdraw->gr_draw_line(doc, gc, x, y, ox, oy);      
      ox=x;
      oy=y;
      list=g_list_next(list);
    }

  /* Draw text */
  if(((DIMENSION_RADIUS *)g_list_last(leader->data.dimension.data)->data)->text!=NULL)
    {
      gint sign;
      gdouble dummy;

      misc_transform_point(transform,((DIMENSION_RADIUS *)g_list_last(leader->data.dimension.data)->data)->insert_point.x,
			   ((DIMENSION_RADIUS *)g_list_last(leader->data.dimension.data)->data)->insert_point.y,
			   &x, &dummy);
      misc_transform_point(transform, ((DIMENSION_RADIUS *)g_list_last(leader->data.dimension.data)->prev->data)->insert_point.x,
			   ((DIMENSION_RADIUS *)g_list_last(leader->data.dimension.data)->prev->data)->insert_point.y,
			   &y, &dummy);

      sign=(x<y)?1:-1;

      grdraw->gr_draw_text(doc,gc,(sign==1?TEXT_JUST_MR:TEXT_JUST_ML),
			   ox-style->dimtxt*sign, oy, 
			   style->dimtxt,1,0,
			   ((DIMENSION_RADIUS *)g_list_last(leader->data.dimension.data)->data)->text);
    }

  /* Draw nice arrow */
  if(g_list_length(leader->data.dimension.data)>1)
    {
      misc_transform_point(transform,
			   ((DIMENSION_RADIUS *)g_list_first(leader->data.dimension.data)->data)->insert_point.x,
			   ((DIMENSION_RADIUS *)g_list_first(leader->data.dimension.data)->data)->insert_point.y,
			   &x, &y);
      misc_transform_point(transform,
			   ((DIMENSION_RADIUS *)g_list_first(leader->data.dimension.data)->next->data)->insert_point.x,
			   ((DIMENSION_RADIUS *)g_list_first(leader->data.dimension.data)->next->data)->insert_point.y,
			   &ox, &oy);

      grdraw->gr_draw_arrow(doc,gc,x,y,M_PI-get_angle(ox,oy,x,y)*M_PI/180.0,style->dimasz);
    }
}

void dim_draw_leader(DOCUMENT *doc, GRDRAW *grdraw, OBJECT *leader)
{
  dim_draw_leader_block(doc, misc_setup_gc_object(doc, leader), grdraw, leader, NULL);
}




