/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "redraw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "dimension.h"
#include "dim_bound.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static OBJECT *work_obj;
static gdouble r,cx,cy,rx,ry, ix,iy;

/* --  F U N C T I O N S  -- */
OBJECT *dim_add_radius(DOCUMENT *doc, LAYER *layer, gchar *name, gchar *style, gint type_colour, COLOUR *colour,
		       gdouble r, gdouble cx, gdouble cy, gdouble rx,gdouble ry,
		       gdouble ix, gdouble iy, gchar *text)
{
  DIMENSION_RADIUS *tpoint;
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }
  
  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_DIMENSION;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, BYLAYER, NULL, BYLAYER, 0); 

  /* Set object specific attributes */
  if(style==NULL)
    obj->data.dimension.style=doc->current_dimensionstyle;
  else
    obj->data.dimension.style=dimstyle_get_style(doc, style);

  obj->data.dimension.type=DIM_RADIUS;
  obj->data.dimension.data=NULL;
  obj->data.dimension.type_data=DIM_DATA_SINGEL;

  tpoint=g_new(DIMENSION_RADIUS,1);
  tpoint->radius=r;
  tpoint->centrum_point.x=cx;
  tpoint->centrum_point.y=cy;
  tpoint->radius_point.x=rx;
  tpoint->radius_point.y=ry;
  tpoint->insert_point.x=ix;
  tpoint->insert_point.y=iy;

  tpoint->text=text;

  obj->data.dimension.data=g_list_append(obj->data.dimension.data, (gpointer)tpoint);
   
  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);
  
  /* Make bounding box for object */
  dim_bound_dimension(doc, obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}


void dim_set_prompt_radius(void)
{
  gchar *t;

  t=g_strdup( _("Dimension text <R%.1f>: "));

  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Select arc or circle: "));
      break;
    case 1:
      set_prompt_text( _("Dimension line location: "));
      break;
    case 2:
      set_prompt_text(dim_set_decimal(t, work_document->current_dimensionstyle->dimdec), r);
      break;
    }

  g_free(t);
}

void dim_set_tool_radius(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("dimradius"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_RADIUS;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
  work_obj=NULL;
}

void dim_button_radius(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      if(doc!=NULL)
	deselect_all_objects(doc);
      
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }
  switch(gl_stage)
    {
    case 0:
      work_obj=select_singel_object(doc,x,y);
      if(work_obj!=NULL)
	{
	  if(work_obj->type==TYPE_CIRCLE || work_obj->type==TYPE_ARC)
	    {
	      work_obj->selected=OBJECT_SEL;
	      redraw_object(doc, work_obj);
	      gl_stage=1;
	    }
	  else
	    print( _("\n  Object must be a circle or an arc."));
	}	  
      else
	print( _("\n  No object found."));
      set_prompt();
      break;
    case 1:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_stage=2;
      set_prompt();
      break;
    }
}

void dim_prompt_radius(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;
  gchar t[40]="R%.1f";

  switch(gl_stage)
    {
    case 1:
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	dim_button_radius(doc,1,x,y);
	break;
      }
      break;
    case 2:
      work_obj->selected=OBJECT_NOT_SEL;
      redraw_object(doc, work_obj);

      if(strcmp(buffer,""))
	dim_draw_dimension(doc, NULL, dim_add_radius(doc,doc->current_layer,NULL,NULL,
						     doc->type_colour, doc->current_colour, 
						     r,cx,cy,rx,ry,ix,iy,g_strdup(buffer)));
      else
	dim_draw_dimension(doc, NULL, dim_add_radius(doc,doc->current_layer,NULL,NULL,
						     doc->type_colour, doc->current_colour, 
						     r,cx,cy,rx,ry,ix,iy,g_strdup_printf(dim_set_decimal(t, 
													 doc->current_dimensionstyle->dimdec),r)));

      deselect_all_objects(doc);      
      tool_idle(doc);
      break;
    }
}

void dim_xor_radius(DOCUMENT *doc, gdouble x,gdouble y)
{
  gchar t[40]="R%.1f";
  gdouble sign;
  gdouble angle;
  GdkGC *gc;

  gc=misc_setup_gc(doc);

  if(work_obj->type==TYPE_CIRCLE)
    {
      cx=work_obj->data.circle.centrum.x;
      cy=work_obj->data.circle.centrum.y;
      r=work_obj->data.circle.radius;
    }
  else if(work_obj->type==TYPE_ARC)
    {
      cx=work_obj->data.arc.centrum.x;
      cy=work_obj->data.arc.centrum.y;
      r=work_obj->data.arc.radius;
    }
  else /* Wrong type  (should not be possible) */
    return;

  angle=get_angle(cx,cy,x,y)*M_PI/180;

  rx=cx+r*cos(angle);
  ry=cy-r*sin(angle);
  ix=x;
  iy=y;

  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,rx),WLDtoSCR_y(doc,ry),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));

  if(dist(x,y,cx,cy)>r)
    sign=1;
  else 
    sign=-1;
  
  /* Draw extension line */
  if(angle>M_PI/2 && angle<3*M_PI/2)
    {
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y),
		    WLDtoSCR_x(doc,x-doc->current_dimensionstyle->dimtxt*sign), WLDtoSCR_y(doc,y));

      /* Draw dimension text */
      draw_text(doc,gc,(sign==1?TEXT_JUST_MR:TEXT_JUST_ML), 
		WLDtoSCR_x(doc,x-doc->current_dimensionstyle->dimtxt*sign*1.5), WLDtoSCR_y(doc,y), 
		WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,0, 
		g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),r));
    }
  else
    {
      gdk_draw_line(doc->widgets.draw->window,gc,
		    WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y),
		    WLDtoSCR_x(doc,x+doc->current_dimensionstyle->dimtxt*sign), WLDtoSCR_y(doc,y));

      /* Draw dimension text */
      draw_text(doc,gc,(sign==1?TEXT_JUST_ML:TEXT_JUST_MR), 
		WLDtoSCR_x(doc,x+doc->current_dimensionstyle->dimtxt*sign*1.5), WLDtoSCR_y(doc,y), 
		WLDtoSCR(doc,doc->current_dimensionstyle->dimtxt),1,0, 
		g_strdup_printf(g_strdup_printf(dim_set_decimal(t,doc->current_dimensionstyle->dimdec),r),r));
    }
  
  /* Draw arrow */
  draw_arrow(doc,gc,rx,ry,(sign==1?2*M_PI-angle:M_PI-angle),doc->current_dimensionstyle->dimasz);
}

void dim_transform_radius(OBJECT *radius, TRANSFORM *trans)
{  
  gdouble tx,ty;
  DIMENSION_RADIUS *t;

  t=(DIMENSION_RADIUS *)g_list_first(radius->data.dimension.data)->data;
  
  if(trans!=NULL && (trans->type & OBJ_SCALE))
    t->radius=ABS(t->radius*trans->scale_x);

  misc_transform_point(trans,t->centrum_point.x, t->centrum_point.y, &tx,&ty);
  t->centrum_point.x=tx;
  t->centrum_point.y=ty;

  misc_transform_point(trans,t->radius_point.x, t->radius_point.y, &tx,&ty);
  t->radius_point.x=tx;
  t->radius_point.y=ty;

  misc_transform_point(trans,t->insert_point.x, t->insert_point.y, &tx,&ty);
  t->insert_point.x=tx;
  t->insert_point.y=ty;
}

void dim_draw_radius_block(DOCUMENT *doc, GdkGC *gc, GRDRAW *grdraw, OBJECT *radius, TRANSFORM *trans)
{
  gdouble angle;
  gdouble cx,cy,r,ix,iy,rx,ry;
  gdouble sign;
  gchar *text;
  DIMENSION_RADIUS *t;
  DIMENSIONSTYLE *style;

  /* Just in case Should be removed ASAP! */
  if(grdraw==NULL)
    {
      g_warning("dim_draw_horiz_block: grdraw==NULL => FIX ME!");
      grdraw=&gr_gdk_draw;
    }

  style=radius->data.dimension.style;

  t=(DIMENSION_RADIUS *)g_list_first(radius->data.dimension.data)->data;
  
  r=t->radius;
  if(trans!=NULL && (trans->type & OBJ_SCALE))
    r=ABS(r*trans->scale_x);

  misc_transform_point(trans,t->centrum_point.x, t->centrum_point.y, &cx,&cy);
  misc_transform_point(trans,t->radius_point.x, t->radius_point.y, &rx,&ry);
  misc_transform_point(trans,t->insert_point.x, t->insert_point.y, &ix,&iy);

  text=t->text;

  if(dist(ix,iy,cx,cy)>r)
    sign=1;
  else 
    sign=-1;

  angle=get_angle(cx,cy,ix,iy)*M_PI/180;

  grdraw->gr_draw_line(doc, gc, rx, ry, ix, iy);

  /* Draw extension line */
  if(angle>M_PI/2 && angle<3*M_PI/2)
    {
      grdraw->gr_draw_line(doc, gc, ix, iy, ix-style->dimtxt*sign, iy);

      /* Draw dimension text */
      grdraw->gr_draw_text(doc,gc,(sign==1?TEXT_JUST_MR:TEXT_JUST_ML), 
			   ix-style->dimtxt*sign*1.5, iy, 
			   style->dimtxt,1,0, text);
    }
  else
    {
      grdraw->gr_draw_line(doc, gc, ix, iy, ix+style->dimtxt*sign, iy);

      /* Draw dimension text */
      grdraw->gr_draw_text(doc,gc,(sign==1?TEXT_JUST_ML:TEXT_JUST_MR), 
			   ix+style->dimtxt*sign*1.5, iy, 
			   style->dimtxt,1,0,text);
    }
 
  /* Draw arrow */
  grdraw->gr_draw_arrow(doc,gc,rx,ry,(sign==1?2*M_PI-angle:M_PI-angle),style->dimasz);
}

void dim_draw_radius(DOCUMENT *doc, GRDRAW *grdraw, OBJECT *radius)
{
  dim_draw_radius_block(doc, misc_setup_gc_object(doc, radius), grdraw, radius, NULL);
}





