/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "calc.h"
#include "dim_horiz.h"
#include "dim_vert.h"
#include "dim_aligned.h"
#include "dim_radius.h"
#include "dim_diameter.h"
#include "dim_center.h"
#include "dim_leader.h"
#include "dim_angular.h"
#include "misc.h"
#include "draw.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* -- Draw dimension graphics -- */
void dim_draw_line(DOCUMENT *doc, GdkGC *gc, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  gdk_draw_line(doc->widgets.draw->window,gc,
		WLDtoSCR_x(doc,x1),WLDtoSCR_y(doc,y1),
		WLDtoSCR_x(doc,x2),WLDtoSCR_y(doc,y2));
}

void dim_draw_arc(DOCUMENT *doc, GdkGC *gc, gdouble xcen, gdouble ycen, gdouble radius, 
		  gdouble start_angle, gdouble end_angle)
{
  draw_arc(doc->widgets.draw->window,gc, FALSE,
	   WLDtoSCR_x(doc,xcen-radius),WLDtoSCR_y(doc,ycen-radius),
	   WLDtoSCR(doc,radius*2),WLDtoSCR(doc,radius*2),
	   start_angle, end_angle);
}

void dim_draw_arrow(DOCUMENT *doc, GdkGC *gc, gdouble x, gdouble y, gdouble angle,gdouble size)
{
  gdouble tx,ty;
  GdkPoint a[3];
  
  tx=x+size*cos(angle);
  ty=y+size*sin(angle);
        
  a[0].x=WLDtoSCR_x(doc,x);
  a[0].y=WLDtoSCR_y(doc,y);
  a[1].x=WLDtoSCR_x(doc,tx+0.4*size*cos(angle+M_PI/2));
  a[1].y=WLDtoSCR_y(doc,ty+0.4*size*sin(angle+M_PI/2));
  a[2].x=WLDtoSCR_x(doc,tx+0.4*size*cos(angle-M_PI/2));
  a[2].y=WLDtoSCR_y(doc,ty+0.4*size*sin(angle-M_PI/2));

  switch(doc->current_dimensionstyle->dimatp)
    {
    case 0:  /* Closed filled */
      gdk_draw_polygon(doc->widgets.draw->window,gc,TRUE, a, 3);  
      break;
    case 1:  /* Closed */
      gdk_draw_polygon(doc->widgets.draw->window,gc,FALSE, a, 3);
      break;
    case 2:  /* Open */
      gdk_draw_line(doc->widgets.draw->window,gc,a[0].x,a[0].y,a[1].x,a[1].y);
      gdk_draw_line(doc->widgets.draw->window,gc,a[0].x,a[0].y,a[2].x,a[2].y);
      break;
    case 3:  /* Line */
      a[0].x=WLDtoSCR_x(doc,x+0.5*size*cos(angle+M_PI/4));
      a[0].y=WLDtoSCR_y(doc,y-0.5*size*sin(angle+M_PI/4));
      a[1].x=WLDtoSCR_x(doc,x-0.5*size*cos(angle+M_PI/4));
      a[1].y=WLDtoSCR_y(doc,y+0.5*size*sin(angle+M_PI/4));
      gdk_draw_line(doc->widgets.draw->window,gc,a[0].x,a[0].y,a[1].x,a[1].y);
      break;
    }
}

void dim_draw_text(DOCUMENT *doc, GdkGC *gc, gint just, gdouble x, gdouble y, 
		   gdouble height, gdouble width, gdouble rot, gchar *text)
{
  draw_text(doc, gc, just, WLDtoSCR_x(doc, x), WLDtoSCR_y(doc, y), WLDtoSCR(doc, height), 1, rot, text);
}

/* This should be static */
GRDRAW gr_gdk_draw={dim_draw_line, dim_draw_arc, dim_draw_arrow,dim_draw_text};

/* --  F U N C T I O N S  -- */
DIMENSIONSTYLE *dimstyle_get_style(DOCUMENT *doc, gchar *name)
{
  GList *list;

  list=g_list_first(doc->dimensionstyle_list);

  while(list!=NULL)
    {
      if(!strcmp(((DIMENSIONSTYLE *)list->data)->name,name))
	return (DIMENSIONSTYLE *)list->data;
      list=g_list_next(list);
    }

  return doc->current_dimensionstyle;
}

DIMENSIONSTYLE *dimstyle_new(DOCUMENT *doc, gchar *name)
{
  DIMENSIONSTYLE *temp;

  temp=g_new(DIMENSIONSTYLE,1);
  temp->name=g_strdup(name);
  doc->dimensionstyle_list=g_list_append(doc->dimensionstyle_list,temp);

  /* Set some default values */
  temp->dimexe=5;
  temp->dimexo=3;
  temp->dimgap=1;
  temp->dimtxt=10;
  temp->dimdec=0;
  temp->dimdli=25;
  temp->dimatp=0;
  temp->dimasz=10;
  temp->dimmtp=1;
  temp->dimmsz=10;

  return temp;
}


DIMENSION_POINT *dim_add_node(OBJECT *dim, gdouble ix, gdouble iy, gdouble tx, gdouble ty, gchar *text)
{
  DIMENSION_POINT *tpoint;

  tpoint=g_new(DIMENSION_POINT,1);
  tpoint->insert_point.x=ix;
  tpoint->insert_point.y=iy;
  tpoint->text_point.x=tx;
  tpoint->text_point.y=ty;
  if(text!=NULL)
    tpoint->text=g_strdup(text);
  else
    tpoint->text=NULL;

  dim->data.dimension.data=g_list_append(dim->data.dimension.data, (gpointer)tpoint);  

  return tpoint;
}
 
OBJECT *dim_add_linear(DOCUMENT *doc, LAYER *layer, gchar *name, gchar *style, 
		       gint type_colour, COLOUR *colour, gint dim_type, gint type_data)
{
  OBJECT *temp;
  static gint no_dim=1;

  temp=(OBJECT *)g_malloc(sizeof(OBJECT));
  
  temp->type=TYPE_DIMENSION;
  temp->selected=OBJECT_NOT_SEL;
  if(name==NULL)
    {
      temp->name=g_strdup_printf( _("Dimension %d"),no_dim);
      no_dim++;
    }
  else
    temp->name=g_strdup_printf("%s",name);

  if(style==NULL)
    temp->data.dimension.style=doc->current_dimensionstyle;
  else
    temp->data.dimension.style=dimstyle_get_style(doc, style);

  temp->data.dimension.type=dim_type;
  temp->data.dimension.data=NULL;
  temp->data.dimension.type_data=type_data;
  
  /* Make colour */
  temp->type_colour=type_colour;
  if(temp->type_colour==BYLAYER)
    temp->colour=layer->colour;
  else 
    {
      temp->colour=(COLOUR *)g_malloc(sizeof(COLOUR));
      memcpy(temp->colour,colour, sizeof(COLOUR));
    }

  temp->layer=layer;

  layer->object_list=g_list_append(layer->object_list,temp);

  doc->saved=FALSE;
  make_window_title(doc);

  return temp;
}

gchar *dim_set_decimal(gchar *text, gint num)
{
  gint i=0;

  while(text[i]!='\0')
    {
      if(text[i]=='%' && text[i+1]=='.')
	text[i+2]=num+'0';
      i++;
    }

  return text;
}

void dim_draw_dimension(DOCUMENT *doc, GRDRAW *gr_draw, OBJECT *obj)
{
  if(gr_draw==NULL)
    gr_draw=&gr_gdk_draw;

  switch(obj->data.dimension.type)
    {
    case DIM_HORIZ:
      dim_draw_horiz(doc, gr_draw, obj);
      break;
    case DIM_VERT:
      dim_draw_vert(doc, gr_draw, obj);
      break;
    case DIM_ALIGNED:
      dim_draw_aligned(doc, gr_draw, obj);
      break;
    case DIM_RADIUS:
      dim_draw_radius(doc, gr_draw, obj);
      break;
    case DIM_DIAMETER:
      dim_draw_diameter(doc, gr_draw, obj);
      break;
    case DIM_CENTER:
      dim_draw_center(doc, gr_draw, obj);
      break;
    case DIM_LEADER:
      dim_draw_leader(doc, gr_draw, obj);
      break;
    case DIM_ANGULAR:
      dim_draw_angular(doc, gr_draw, obj);
      break;
    }  
}

void dim_transform_dimension(OBJECT *obj, TRANSFORM *trans)
{
  switch(obj->data.dimension.type)
    {
    case DIM_HORIZ:
      dim_transform_horiz(obj, trans);
      break;
    case DIM_VERT:
      dim_transform_vert(obj, trans);
      break;
    case DIM_ALIGNED:
      dim_transform_aligned(obj, trans);
      break;
    case DIM_RADIUS:
      dim_transform_radius(obj, trans);
      break;
    case DIM_DIAMETER:
      dim_transform_diameter(obj, trans);
      break;
    case DIM_CENTER:
      dim_transform_center(obj, trans);
      break;
    case DIM_LEADER:
      dim_transform_leader(obj, trans);
      break;
    case DIM_ANGULAR:
      dim_transform_angular(obj, trans);
      break;
    }  
}

void dim_draw_dimension_block(DOCUMENT *doc, GdkGC *gc, OBJECT *obj, TRANSFORM *trans)
{
  switch(obj->data.dimension.type)
    {
    case DIM_HORIZ:
      dim_draw_horiz_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    case DIM_VERT:
      dim_draw_vert_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    case DIM_ALIGNED:
      dim_draw_aligned_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    case DIM_RADIUS:
      dim_draw_radius_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    case DIM_DIAMETER:
      dim_draw_diameter_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    case DIM_CENTER:
      dim_draw_center_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    case DIM_LEADER:
      dim_draw_leader_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    case DIM_ANGULAR:
      dim_draw_angular_block(doc, gc, &gr_gdk_draw, obj, trans);
      break;
    }  
}





