/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include "struct.h"
#include "calc.h"
#include "misc.h"
#include "standard.h"
#include "calc_arc.h"
#include "obj_polyline.h"

void draw_arc(GdkDrawable *win, GdkGC *gc, gint filled, gint x, gint y, gint w, gint h, 
	      gdouble sangle, gdouble eangle);

/* --  G L O B A L   V A R I A B L E S  -- */
typedef struct 
{
  gchar character;

  /* Distance to add for next character */
  gdouble width;

  GList *data;
} _CHARACTER;

typedef struct
{
  gint type;
  OBJECT_UNION s;
} _CHARACTER_OBJECT;

/* --  L O C A L   V A R I A B L E S  -- */

/* The font height */
static gdouble font_height;

/* Use a hash table for looking up character definitions */
static GHashTable *hash_standardfont;

/* --  F U N C T I O N S  -- */
void get_bounding_box_of_arc(OBJECT_ARC *arc, gdouble *bounding_box)
{
  gdouble sx,sy,ex,ey;
  gint i;

  sx=arc->centrum.x+arc->radius*cos(arc->start_angle*M_PI/180);
  sy=arc->centrum.y-arc->radius*sin(arc->start_angle*M_PI/180);

  ex=arc->centrum.x+arc->radius*cos(arc->end_angle*M_PI/180);
  ey=arc->centrum.y-arc->radius*sin(arc->end_angle*M_PI/180);

  bounding_box[0]=MIN(sx,ex);
  bounding_box[1]=MIN(sy,ey);
  bounding_box[2]=MAX(sx,ex);
  bounding_box[3]=MAX(sy,ey);

  /* Check each quadrant */
  for(i=90;i<=360;i+=90)
    {

      if(misc_within_angle(arc->start_angle, arc->end_angle,i))
	{
	  sx=arc->centrum.x+arc->radius*cos(i*M_PI/180);
	  sy=arc->centrum.y-arc->radius*sin(i*M_PI/180);

	  bounding_box[0]=MIN(bounding_box[0],sx);
	  bounding_box[1]=MIN(bounding_box[1],sy);
	  bounding_box[2]=MAX(bounding_box[2],sx);
	  bounding_box[3]=MAX(bounding_box[3],sy);
	}
    }
}

void init_font_table(void)
{
  /* This includes the actually font-data */
#include "fonts/standard.h"

  gdouble ax=0,ay=0;
  _CHARACTER_OBJECT *s;
  gint i=0;
  gint no;
  gchar c;
  _CHARACTER *t;
  gchar **token;
  gdouble bound[4]={0,0,0,0};
  gdouble tbound[4];
  gboolean first_width;
  gboolean first_height;

  /* Create a new hashtable for font */
  hash_standardfont=g_hash_table_new(NULL, NULL);

  /* We have not yet found the height of the font */
  first_height=TRUE;

  while(TRUE)
    {
      /* Extract character */
      c=standard_font[i][1];

      /* Extract number of objects character depends on */
      no=(gint)g_strtod(&standard_font[i][4], NULL);

      i++;

      /* Make new character */
      t=g_new(_CHARACTER, 1);

      t->character=c;
      t->data=NULL;

      /* Read objects */
      no+=i;

      /* This is the first object in character that we read the width from */
      first_width=TRUE;

      while(i<no)
	{

	  /* Create new union */
	  s=g_new( _CHARACTER_OBJECT, 1);

	  token=g_strsplit(standard_font[i]," ", 255);      

	  /* Check for type */

	  /* Line */
	  if(!strcmp(token[0], "L"))
	    {
	      s->type=TYPE_LINE;
	      s->s.line.p1.x=g_strtod(token[1], NULL);
	      s->s.line.p1.y=g_strtod(token[2], NULL);
	      ax=s->s.line.p2.x=g_strtod(token[3], NULL);
	      ay=s->s.line.p2.y=g_strtod(token[4], NULL);

	      /* Create bounding box */
	      tbound[0]=s->s.line.p1.x;
	      tbound[1]=s->s.line.p1.y;
	      tbound[2]=s->s.line.p2.x;
	      tbound[3]=s->s.line.p2.y;
	    }

	  /* Line Relative */
	  if(!strcmp(token[0], "LT"))
	    {
	      s->type=TYPE_LINE;
	      s->s.line.p1.x=ax;
	      s->s.line.p1.y=ay;
	      ax=s->s.line.p2.x=g_strtod(token[1], NULL);
	      ay=s->s.line.p2.y=g_strtod(token[2], NULL);

	      /* Create bounding box */
	      tbound[0]=s->s.line.p1.x;
	      tbound[1]=s->s.line.p1.y;
	      tbound[2]=s->s.line.p2.x;
	      tbound[3]=s->s.line.p2.y;	      
	    }

	  /* Arc */
	  if(!strcmp(token[0], "A"))
	    {
	      s->type=TYPE_ARC;
	      s->s.arc.centrum.x=g_strtod(token[1], NULL);
	      s->s.arc.centrum.y=g_strtod(token[2], NULL);
	      s->s.arc.radius=g_strtod(token[3], NULL);
	      s->s.arc.start_angle=g_strtod(token[4], NULL);
	      s->s.arc.end_angle=g_strtod(token[5], NULL);

	      /* Fix angle */
	      while(s->s.arc.start_angle<0) s->s.arc.start_angle+=360;
	      while(s->s.arc.start_angle>360) s->s.arc.start_angle-=360;

	      while(s->s.arc.end_angle<0) s->s.arc.end_angle+=360;
	      while(s->s.arc.end_angle>360) s->s.arc.end_angle-=360;

	      /* Create bounding box */
	      get_bounding_box_of_arc(&s->s.arc, tbound);	     
	    }

	  /* Arc reverse */
	  if(!strcmp(token[0], "AR"))
	    {
	      s->type=TYPE_ARC;
	      s->s.arc.centrum.x=g_strtod(token[1], NULL);
	      s->s.arc.centrum.y=g_strtod(token[2], NULL);
	      s->s.arc.radius=g_strtod(token[3], NULL);
	      s->s.arc.start_angle=g_strtod(token[5], NULL);
	      s->s.arc.end_angle=g_strtod(token[4], NULL);

	      /* Fix angle */
	      while(s->s.arc.start_angle<0) s->s.arc.start_angle+=360;
	      while(s->s.arc.start_angle>360) s->s.arc.start_angle-=360;

	      while(s->s.arc.end_angle<0) s->s.arc.end_angle+=360;
	      while(s->s.arc.end_angle>360) s->s.arc.end_angle-=360;

	      /* Create bounding box */
	      get_bounding_box_of_arc(&s->s.arc, tbound);
	    }

	  /* Check first so this is a valid character. Just check size for A-Zaz */
	  if(isalpha(c))
	    {
	      /* Calculate the width of the character */
	      if(!first_width)
		{
		  bound[0]=MIN(tbound[0], bound[0]);
		  bound[0]=MIN(tbound[2], bound[0]);
		  
		  bound[2]=MAX(tbound[0], bound[2]);
		  bound[2]=MAX(tbound[2], bound[2]);
		}
	      else
		{
		  bound[0]=MIN(tbound[0], tbound[2]);
		  bound[2]=MAX(tbound[0], tbound[2]);
		  first_width=FALSE;
		}	     
	      
	      /* Calculate the height of the font */
	      if(!first_height)
		{
		  bound[1]=MIN(tbound[1], bound[1]);
		  bound[1]=MIN(tbound[3], bound[1]);
		  
		  bound[3]=MAX(tbound[1], bound[3]);
		  bound[3]=MAX(tbound[3], bound[3]);
		}
	      else
		{
		  bound[1]=MIN(tbound[1], tbound[3]);
		  bound[3]=MAX(tbound[1], tbound[3]);
		  first_height=FALSE;
		}	     
	    }

	  /* Remove memory */
	  g_strfreev(token);

	  /* Append object to list */
	  t->data=g_list_append(t->data, s);

	  i++;
	}

      /* Width of character */
      t->width=MAX(ABS(bound[2]-bound[0]), 1);
      
      /* Add character to hash list */
      g_hash_table_insert(hash_standardfont, GINT_TO_POINTER( (gint)c ), (gpointer)t);

      if(!strcmp(standard_font[i], "# EOF"))
	break;
    }

  /* Calculate the font height */
  font_height=ABS(bound[3]-bound[1]);
}

void draw_arrow(DOCUMENT *doc,GdkGC *gc, gdouble x, gdouble y, gdouble angle,gdouble size)
{
  gdouble tx,ty;
  GdkPoint a[3];
 
  tx=x+size*cos(angle);
  ty=y+size*sin(angle);
        
  a[0].x=WLDtoSCR_x(doc,x);
  a[0].y=WLDtoSCR_y(doc,y);
  a[1].x=WLDtoSCR_x(doc,tx+0.4*size*cos(angle+M_PI/2));
  a[1].y=WLDtoSCR_y(doc,ty+0.4*size*sin(angle+M_PI/2));
  a[2].x=WLDtoSCR_x(doc,tx+0.4*size*cos(angle-M_PI/2));
  a[2].y=WLDtoSCR_y(doc,ty+0.4*size*sin(angle-M_PI/2));

  switch(doc->current_dimensionstyle->dimatp)
    {
    case 0:  /* Closed filled */
      gdk_draw_polygon(doc->widgets.draw->window,gc,TRUE, a, 3);  
      break;
    case 1:  /* Closed */
      gdk_draw_polygon(doc->widgets.draw->window,gc,FALSE, a, 3);
      break;
    case 2:  /* Open */
      gdk_draw_line(doc->widgets.draw->window,gc,a[0].x,a[0].y,a[1].x,a[1].y);
      gdk_draw_line(doc->widgets.draw->window,gc,a[0].x,a[0].y,a[2].x,a[2].y);
      break;
    case 3:  /* Line */
      a[0].x=WLDtoSCR_x(doc,x+0.5*size*cos(angle+M_PI/4));
      a[0].y=WLDtoSCR_y(doc,y-0.5*size*sin(angle+M_PI/4));
      a[1].x=WLDtoSCR_x(doc,x-0.5*size*cos(angle+M_PI/4));
      a[1].y=WLDtoSCR_y(doc,y+0.5*size*sin(angle+M_PI/4));
      gdk_draw_line(doc->widgets.draw->window,gc,a[0].x,a[0].y,a[1].x,a[1].y);
      break;
    }
}

/* -- D R A W   P O L Y L I N E */
static GList *curr_node_list;
static TRANSFORM *curr_trans;
static OBJECT *polyline;
static gint polyline_draw_stage=0;


gboolean draw_polyline_get_vertex_polyline(gint *type, OBJECT_UNION *s)
{
  OBJECT_ARC arc;
  gdouble ex,ey;
  gdouble x,y;
  POINT t;
  static gdouble ox=0,oy=0;
  NODE *node;

  if(curr_node_list==NULL)
    {
      if(polyline_draw_stage==0 && polyline->data.polyline.closed==POLYLINE_CLOSE_LINE)
	{
	  /* Make last closure line */
	  *type=NODE_LINE;
	  t.x=((NODE *)g_list_first(polyline->data.polyline.node_list)->data)->x;
	  t.y=((NODE *)g_list_first(polyline->data.polyline.node_list)->data)->y;
	  t.selected=((NODE *)g_list_first(polyline->data.polyline.node_list)->data)->selected;
	  misc_transform_point_select(curr_trans, &t, &x, &y);
	  
	  s->line.p1.x=ox;
	  s->line.p1.y=oy;
	  ox=s->line.p2.x=x;
	  oy=s->line.p2.y=y;

	  polyline_draw_stage++;
	  return TRUE;
	}
      else if(polyline_draw_stage==0 && polyline->data.polyline.closed==POLYLINE_CLOSE_ARC)
	{
	  t.x=((NODE *)g_list_first(polyline->data.polyline.node_list)->data)->x;
	  t.y=((NODE *)g_list_first(polyline->data.polyline.node_list)->data)->y;
	  t.selected=((NODE *)g_list_first(polyline->data.polyline.node_list)->data)->selected;

	  misc_transform_point_select(curr_trans, &t, &x, &y);

	  calc_arc_2p( calc_tangent(polyline->data.polyline.node_list, curr_trans), 
		       ox, oy, x,y, NULL, NULL, &arc);
	  
	  *type=NODE_ARC;
	  
	  s->arc.centrum.x=arc.centrum.x;
	  s->arc.centrum.y=arc.centrum.y;
	  s->arc.radius=arc.radius/2;
	  s->arc.start_angle=arc.start_angle;
	  s->arc.end_angle=arc.end_angle;

	  polyline_draw_stage++;
	  return TRUE;
	}      
      else
	return FALSE;
    }

  node=(NODE *)curr_node_list->data;
  *type=NODE_LINE;
  
  t.x=node->x;
  t.y=node->y;
  t.selected=node->selected;
  misc_transform_point_select(curr_trans, &t, &x, &y);

  if(node->type==NODE_LINE)
    {
      *type=NODE_LINE;
      s->line.p1.x=ox;
      s->line.p1.y=oy;
      ox=s->line.p2.x=x;
      oy=s->line.p2.y=y;
    }
  else if(node->type==NODE_ARC)
    {
      if(curr_node_list->next!=NULL)
	{
	  t.x=((NODE *)curr_node_list->next->data)->x;
	  t.y=((NODE *)curr_node_list->next->data)->y;
	  t.selected=((NODE *)curr_node_list->next->data)->selected;
	  misc_transform_point_select(curr_trans, &t, &ex,&ey);
	  
	  if(calc_arc_3p(ox,oy,x,y,ex,ey,&arc))
	    {
	      *type=NODE_ARC;
	      
	      s->arc.centrum.x=arc.centrum.x;
	      s->arc.centrum.y=arc.centrum.y;
	      s->arc.radius=arc.radius;
	      s->arc.start_angle=arc.start_angle;
	      s->arc.end_angle=arc.end_angle;

	      ox=ex;
	      oy=ey;
	    }
	  
	  curr_node_list=g_list_next(curr_node_list);
	}	
      else
	{
	  *type=NODE_LINE;
	  s->line.p1.x=ox;
	  s->line.p1.y=oy;
	  s->line.p2.x=ox;
	  s->line.p2.y=oy;
	}
    }
  
  curr_node_list=g_list_next(curr_node_list);
  return TRUE;
}

void *draw_polyline_set_polyline(OBJECT *pline, TRANSFORM *trans)
{
  polyline_draw_stage=0;
  polyline=pline;
  curr_trans=trans;
  curr_node_list=g_list_first(pline->data.polyline.node_list);

  switch(pline->data.polyline.type)
    {
    case 0: /* Normal polyline */
      return draw_polyline_get_vertex_polyline;
    case 1: /* Spline */
      /*      return draw_polyline_get_vertex_spline; */
      break;
    }

  return NULL;

}

/* -- Functions and variables for handling text */
#define TEXT_VERTEX_MOVETO 0
#define TEXT_VERTEX_LINETO 1

#define TEXTFLAGS_NONE 0
#define TEXTFLAGS_OVERSCORE 1
#define TEXTFLAGS_UNDERSCORE 2
#define TEXTFLAGS_DEGREE 3
#define TEXTFLAGS_PLUSMINUS 4
#define TEXTFLAGS_CIRCLEDIAMETER 5
#define TEXTFLAGS_PERCENT 6

static gint text_letter_length;
static gint text_letter_counter;
static gint text_vertex_counter;
static gint text_x, text_y, text_height, text_width;
static gchar *text;
static gdouble text_sx, text_sy;
static gdouble cost,sint;
static gint text_flag;

static void draw_calc_text_justify(gint just, gdouble height, gdouble width, gdouble rot, gdouble *dx, gdouble *dy)
{
  switch(just)
    {
    case TEXT_JUST_TL:
      *dx=(gdouble)height*cos(-M_PI/2+rot);
      *dy=(gdouble)height*sin(-M_PI/2+rot);
      break;
    case TEXT_JUST_TC:
      *dx=(gdouble)height*cos(-M_PI/2+rot);
      *dy=(gdouble)height*sin(-M_PI/2+rot);
      (*dx)+=width/2*cos(rot+M_PI);
      (*dy)+=width/2*sin(rot+M_PI);
      break;
    case TEXT_JUST_TR:
      *dx=(gdouble)height*cos(-M_PI/2+rot);
      *dy=(gdouble)height*sin(-M_PI/2+rot);
      (*dx)+=width*cos(rot+M_PI);
      (*dy)+=width*sin(rot+M_PI);
      break;
    case TEXT_JUST_ML:
      *dx=(gdouble)height*cos(-M_PI/2+rot)/2;
      *dy=(gdouble)height*sin(-M_PI/2+rot)/2;
      break;
    case TEXT_JUST_MC:
      *dx=(gdouble)height*cos(-M_PI/2+rot)/2;
      *dy=(gdouble)height*sin(-M_PI/2+rot)/2;
      (*dx)+=width/2*cos(rot+M_PI);
      (*dy)+=width/2*sin(rot+M_PI);
      break;
    case TEXT_JUST_MR:
      *dx=(gdouble)height*cos(-M_PI/2+rot)/2;
      *dy=(gdouble)height*sin(-M_PI/2+rot)/2;
      (*dx)+=width*cos(rot+M_PI);
      (*dy)+=width*sin(rot+M_PI);
      break;
    case TEXT_JUST_BL:  /* <- Default */
      *dx=0;
      *dy=0;
      break;
    case TEXT_JUST_BC:
      *dx=width/2*cos(rot+M_PI);
      *dy=width/2*sin(rot+M_PI);
      break;
    case TEXT_JUST_BR:
      *dx=width*cos(rot+M_PI);
      *dy=width*sin(rot+M_PI);
      break;
    }

}

void draw_text_set_text(gint ojust, gint ox, gint oy, gint oheight, gdouble owidth, gdouble angle, gchar *otext)
{
  gdouble dx,dy;

  text_letter_length=strlen(otext);
  text_letter_counter=0;
  text_vertex_counter=0;
  text_x=ox;
  text_y=oy;
  text_height=oheight;
  text_width=owidth;
  text_flag=TEXTFLAGS_NONE;
  text=otext;
  //  g_strup(text);

  text_sx=(gdouble)(text_height*text_width)/6; 
  text_sy=(gdouble)text_height/6;

  cost=cos(angle);
  sint=sin(angle);

  draw_calc_text_justify(ojust, oheight, text_letter_length*(text_height*text_width)-2*text_sx,
			 angle, &dx,&dy);

  text_x+=dx;
  text_y-=dy;
}

gboolean draw_text_get_vertex(gint *type, gdouble *x, gdouble *y)
{
  gdouble wchar,hchar,xd,yd;
  gint p;

  /* Are we at the end of the string? If so, quit */
  if(text_letter_length==text_letter_counter)
    return FALSE;

  wchar=hchar=6;
  
  p=(text[text_letter_counter])-32;
  
  /* check range to make sure that we dont write any illegal letters*/
  if((p<0) || (p>95))
    p=94;
     
  /* Check range of vertex */
  if(text_vertex_counter<charVert[p])
    {
      if(numChar[p][2*text_vertex_counter]<0 || numChar[p][2*text_vertex_counter+1]<0 || text_vertex_counter==0)
	*type=TEXT_VERTEX_MOVETO;
      else
	*type=TEXT_VERTEX_LINETO;
      
      xd=ABS(numChar[p][2*text_vertex_counter])+wchar*text_letter_counter;
      yd=hchar-ABS(numChar[p][2*text_vertex_counter+1]);
      // yd=ABS(numChar[p][2*text_vertex_counter+1]);
      
      *x=text_x+((text_sx*xd*cost-text_sy*yd*sint));
      *y=text_y-((text_sx*xd*sint+text_sy*yd*cost));
      
      text_vertex_counter++;
      return TRUE;
    }
  else
    {
      text_letter_counter++;
      text_vertex_counter=0;
      return draw_text_get_vertex(type,x,y);
    }

  return FALSE;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

void draw_text2(DOCUMENT *doc, GdkGC *gc, gint just, gdouble x, gdouble y, gdouble height, 
		gdouble width, gdouble rot, gchar *text)
{
  gint i;
  GList *list;
  _CHARACTER *character;
  _CHARACTER_OBJECT *object;

  /* We need to scale the font so it has a correct height */
  height=height/font_height;

  for(i=0;i<strlen(text);i++)
    {

      /* Find current character in hash table */
      character=(_CHARACTER *)g_hash_table_lookup(hash_standardfont, GINT_TO_POINTER( (gint)text[i]));

      /* If we found a character */
      if(character!=NULL)
	{
	  list=g_list_first(character->data);

	  while(list!=NULL)
	    {
	      object=(_CHARACTER_OBJECT *)list->data;

	      switch(object->type)
		{
		case TYPE_LINE:
		  gdk_draw_line(doc->widgets.draw->window, 
				gc,
				WLDtoSCR_x(doc, object->s.line.p1.x*height*width+x), 
				WLDtoSCR_y(doc, y-object->s.line.p1.y*height),
				WLDtoSCR_x(doc, object->s.line.p2.x*height*width+x), 
				WLDtoSCR_y(doc, y-object->s.line.p2.y*height));
		  break;
		case TYPE_ARC:
		  draw_arc(doc->widgets.draw->window, gc, FALSE, 
			   WLDtoSCR_x(doc, object->s.arc.centrum.x*height+x-object->s.arc.radius*height*width), 
			   WLDtoSCR_y(doc, y-(object->s.arc.centrum.y*height)-object->s.arc.radius*height),
			   WLDtoSCR(doc, object->s.arc.radius*height*2*width), 
			   WLDtoSCR(doc, object->s.arc.radius*height*2),
			   object->s.arc.start_angle, object->s.arc.end_angle);
		  break;
		}

	      list=g_list_next(list);
	    }

	  /* Move cursor */
	  x+=character->width*height*1.2*width;
	}
      else
	{
	  /* Check if this is a space */
	  if( isspace(text[i]) )
	    {
	      width+=height*1.2*width;
	    }

	}




    }


}











void draw_text(DOCUMENT *doc, GdkGC *gc, gint just, gint x, gint y, gint height, gdouble width, gdouble rot, gchar *text)
{
  GdkPoint *list;
  gint type,j;
  gdouble tx,ty;

  draw_text_set_text(just,x,y,height,width,rot,text);

  list=(GdkPoint *)g_malloc(sizeof(GdkPoint)*32);

  j=0;

  while(draw_text_get_vertex(&type, &tx,&ty))
	{
	  if(type==TEXT_VERTEX_MOVETO)
	    {
	      if(j>0)
		gdk_draw_lines(doc->widgets.draw->window, gc, list, j);	      
	      j=0;
	    }

	  list[j].x=tx;
	  list[j].y=ty;
	  j++;
	}

  if(j>1)
    gdk_draw_lines(doc->widgets.draw->window, 
		   gc,
		   list,j);	      

  g_free(list);
}

gint select_text(gdouble mx, gdouble my,gdouble d,gint just, gdouble x, gdouble y, 
		 gdouble height, gdouble width, gdouble rot, gchar *text)
{
  gint type;
  gdouble tx,ty;
  gdouble ox=0,oy=0;

  draw_text_set_text(just,x,y,height,width,rot,text);

  while(draw_text_get_vertex(&type, &tx,&ty))
	{
	  if(type==TEXT_VERTEX_LINETO)
	    {
	      if(misc_point_on_line(mx,my,ox,oy,tx,ty,d))
		return TRUE;
	    }
	  ox=tx;
	  oy=ty;
	}

  return FALSE;
}

void draw_elarc(DOCUMENT *doc, GdkWindow *window, GdkGC *gc, gint xcen, gint ycen, gint ma, gint mi, 
		gdouble angle, gdouble start_angle, gdouble end_angle)
     /* Draws an ellipical arc */
{
  gint i;
  gint num;
  gdouble sa,ea,an;
  gdouble sina,cosa,dt;
  POINT *list;
  GdkPoint *rot_list;

  if(start_angle>end_angle)
    {
      draw_elarc(doc,window,gc,xcen,ycen,ma,mi,angle,start_angle,360);
      draw_elarc(doc,window,gc,xcen,ycen,ma,mi,angle,0,end_angle);
      return;     
    }

  num=MAX(65, ma*mi/1000);
  num=MIN(num, 700);

  sa=start_angle*M_PI/180.0;
  ea=end_angle*M_PI/180.0;

  if (ABS(sa-ea)<GFLOAT_SENSITIVE) return; /* Nothing to draw */

  list=g_new(POINT,num+5);

  dt=ABS(sa-ea)/(gdouble)num;

  for(i=0, an=sa;i<=num;i++, an+=dt)
  {
    list[i].x=((gdouble)ma)*cos(2*M_PI-an);
    list[i].y=((gdouble)mi)*sin(2*M_PI-an);
  }

  rot_list=g_new(GdkPoint, num+5);

  sina=sin(-angle*M_PI/180);
  cosa=cos(-angle*M_PI/180);
  
  for(i=0;i<=num;i++)
    {
      rot_list[i].x=xcen+(gint) (list[i].x*cosa-list[i].y*sina);
      rot_list[i].y=ycen+(gint) (list[i].x*sina+list[i].y*cosa);
    }
 
  gdk_draw_lines(window, gc, rot_list, num+1);

  g_free(rot_list);
  g_free(list);
}

void draw_arc(GdkDrawable *win, GdkGC *gc, gint filled, gint x, gint y, gint w, gint h, 
	      gdouble sangle, gdouble eangle)
     /* Draw an arc */
{
  if(eangle>sangle)
    gdk_draw_arc(win, gc, filled, x,y,w,h,(sangle*64.0),((eangle-sangle)*64.0));
  else
    {
      gdk_draw_arc(win, gc, filled, x,y,w,h,(sangle*64.0),((360.0-sangle)*64.0));
      gdk_draw_arc(win, gc, filled, x,y,w,h,0,(eangle*64.0));
    }

}

void draw_ellipse(DOCUMENT *doc, GdkWindow *window, GdkGC *gc, gint xcen, gint ycen, gint ma, gint mi, gdouble angle)
     /* Draws an ellipse with centar at xcen,ycen. Major axis radius ma and minor mi with
	the rotation of angle degrees */
{
  gint i;
  gint num;
  gdouble dt,dCos,dSin,sina,cosa,adivb,bdiva;
  POINT *list;
  GdkPoint *rot_list;

  num=MAX(65, mi*ma/1000);
  num=MIN(num, 700);

  dt=2.*M_PI/(gdouble)num;

  dCos=cos(dt);
  dSin=sin(dt);
  sina=sin(-angle*M_PI/180);
  cosa=cos(-angle*M_PI/180);
  adivb=(gdouble)ma/(gdouble)mi;
  bdiva=(gdouble)mi/(gdouble)ma;

  list=g_new(POINT,num);

  rot_list=g_new(GdkPoint, num+1);

  list[0].x=ma;
  list[0].y=0;

  for(i=0;i<(num-1); i++)
    {
      list[i+1].x=list[i].x*dCos-adivb*list[i].y*dSin;
      list[i+1].y=bdiva*list[i].x*dSin+list[i].y*dCos;
    }

  for(i=0;i<num;i++)
    {
      rot_list[i].x=xcen+(int) (list[i].x*cosa-list[i].y*sina);
      rot_list[i].y=ycen+(int) (list[i].x*sina+list[i].y*cosa);
    }

  rot_list[num].x=rot_list[0].x;
  rot_list[num].y=rot_list[0].y;

  gdk_draw_lines(window, gc, rot_list, num+1);

  g_free(rot_list);
  g_free(list);

}

void draw_point(GdkWindow *win, GdkGC *gc, gint style, gint size,gint x, gint y)
     /* Draw a point */
{
  if(style & POINT_DOT)
    gdk_draw_point(win,gc,x,y);

  if(style & POINT_LINE)
    gdk_draw_line(win,gc,x,y, x, y-size);

  if(style & POINT_CIRCLE)
    gdk_draw_arc(win,gc,FALSE,
		 x-size,y-size, 
		 size*2, size*2,0,FULL_CIRCLE);

  if(style & POINT_BOX)
    gdk_draw_rectangle(win,gc,FALSE,
		       x-size,y-size, 
		       size*2, size*2);

  if(style & POINT_MARK)
    {
      gdk_draw_line(win,gc,x,y-size, x, y+size);
      gdk_draw_line(win,gc,x-size, y, x+size, y);
    }

  if(style & POINT_XMARK)
    {
      gdk_draw_line(win,gc,x-size,y-size, 
		    x+size, y+size);
      gdk_draw_line(win,gc,x+size,y-size, 
		    x-size, y+size);
    }
}

void set_point(DOCUMENT *doc, gint black, gdouble x, gdouble y)
     /* Draw a pixel */
{
  if(doc==NULL)
    return;

  if(black)
    {
      gdk_gc_set_foreground(doc->replace_gc, &doc->black);
      gdk_draw_point(doc->widgets.draw->window, 
		     doc->replace_gc,
		     WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
    }
  else
    gdk_draw_point(doc->widgets.draw->window, 
		   doc->erase_gc,
		   WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
}








