/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <strings.h>
#include "struct.h"
#include "global.h"
#include "initmenu.h"
#include "misc.h"
#include "redraw.h"
#include "dia_message.h"
#include "dia_misc.h"
#include "dia_comm.h"
#include "parse.h"
#include "xor.h"
#include "dia_coords.h"
#include "color.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
#define DONT_CHANGE (-1)

typedef struct {
  GtkWidget *window;
  
  GtkWidget *vbox1;
 
  GtkWidget *table;
  GtkWidget *text[4];

  GtkWidget *option_layer;
  GtkWidget *menu_layer;

  GtkWidget *option_colour;
  GtkWidget *menu_colour;

  GtkWidget *option_linestyle;
  GtkWidget *menu_linestyle;

  GtkWidget *combo_linewidth;

  GtkWidget *hr;

  GtkWidget *hbuttonbox1;
  GtkWidget *button_ok;
  GtkWidget *button_cancel;
} _EDITPROP;

_EDITPROP *dia_editprop;

static DOCUMENT *grab_document;

static LAYER *new_layer=NULL;
static gint new_type_linestyle=DONT_CHANGE;
static LINESTYLE *new_linestyle=NULL;
static gint new_type_colour=DONT_CHANGE;
static COLOUR *new_colour=NULL;

/* --  P R O T O T Y P E S  -- */
static void editprop_get_dialog(DOCUMENT *doc)
{
  gboolean change=FALSE;
  gchar *string;
  OBJECT *object;
  GList *list;

  /* For each object in selection list... */
  list=g_list_first(doc->selection_list);

  while(list!=NULL)
    {
      /* Extract object */
      object=(OBJECT *)list->data;

      /* Change layer? */
      if(new_layer!=NULL)
	{
	  /* Remove object from old layer */
	  object->layer->object_list=g_list_remove(object->layer->object_list, 
						   (gpointer) object); 

	  /* Add layer to new */
	  new_layer->object_list=g_list_append(new_layer->object_list, 
					       (gpointer) object); 

	  /* Update pointer to layer */
	  object->layer=new_layer;

	  /* Update linestyle */
	  if(object->type_linestyle==BYLAYER || object->type_linestyle==BYBLOCK)
	    object->linestyle=new_layer->linestyle;

	  /* Update colour */
	  if(object->type_colour==BYLAYER || object->type_colour==BYBLOCK)
	    object->colour=new_layer->colour;

	  /* Update linewidth */
	  if(object->type_linewidth==BYLAYER || object->type_linewidth==BYBLOCK)
	    object->linewidth=new_layer->linewidth;

	  change=TRUE;
	}

      /* Change colour? */
      if(new_type_colour!=DONT_CHANGE)
	{
	  if(object->type_colour==USERDEFINED)
	    g_free(object->colour);

	  object->type_colour=new_type_colour;

	  switch(new_type_colour)
	    {
	    case BYLAYER:
	    case BYBLOCK:
	      object->colour=object->layer->colour;
	      break;	     
	    case PREDEFINED:
	    case USERDEFINED:
	      object->colour=new_colour;
	      break;
	    }	

	  change=TRUE;
	}

      /* Change linestyle */
      if(new_type_linestyle!=DONT_CHANGE)
	{
	  if(object->type_linestyle==USERDEFINED)
	    g_free(object->linestyle);
	  
	  object->type_linestyle=new_type_linestyle;
	  
	  switch(new_type_linestyle)
	    {
	    case BYLAYER:
	    case BYBLOCK:
	      object->linestyle=object->layer->linestyle;
	      break;	     
	    case PREDEFINED:
	    case USERDEFINED:
	      object->linestyle=new_linestyle;
	      break;
	    }	

	  change=TRUE;
	}

      /* Change linewidth */
      string=gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dia_editprop->combo_linewidth)->entry));

      if(!strcmp( _("By Layer"), string))
	{
	  object->type_linewidth=BYLAYER;
	  object->linewidth=object->layer->linewidth;
	  change=TRUE;
	}
      else if(!strcmp( _("By Block"), string))
	{
	  object->type_linewidth=BYBLOCK;
	  object->linewidth=object->layer->linewidth;
	  change=TRUE;
	}
      else if(strcmp( _("Don't Change"), string))
	{
	  object->type_linewidth=USERDEFINED;
	  object->linewidth=g_strtod(string, NULL);
	  change=TRUE;
	}

      list=g_list_next(list);
    }

  /* If objects have been changed mark the drawing as not saved and
     make a nice window title */
  if(change)
    {
      doc->saved=FALSE;
      make_window_title(doc);
    }
}

static gint editprop_close(GtkWidget *widget, gpointer data)
{
  /* Remove all in selection list */
  deselect_all_objects(grab_document);

  tool_idle(grab_document);

  gtk_widget_hide(dia_editprop->window);
  menus_set_sensitive("/Modify/Edit/Properties", TRUE);
  return TRUE;
}

static gint editprop_ok(GtkWidget *widget, gpointer data)
{
  editprop_get_dialog(grab_document);

  return editprop_close(widget, data);
}

static void make_dialog_editprop(void)
{
  GList *linewidth_glist=NULL;
  gint i;

  /* Create struct */
  dia_editprop=g_new(_EDITPROP,1);

  /* Create main window */
  dia_editprop->window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize(dia_editprop->window);

  /* Set window title */
  gtk_window_set_title(GTK_WINDOW(dia_editprop->window), _("oCADis: Edit properties"));

  gtk_window_set_modal(GTK_WINDOW(dia_editprop->window),TRUE);
  
  /* Attach signals to window */
  gtk_signal_connect(GTK_OBJECT(dia_editprop->window),"delete_event",
                     GTK_SIGNAL_FUNC(editprop_close),NULL);

  /* -- Create main vbox -- */
  dia_editprop->vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (dia_editprop->vbox1);
  gtk_container_add (GTK_CONTAINER (dia_editprop->window), dia_editprop->vbox1);

  /* -- Create table -- */
  dia_editprop->table=gtk_table_new(2, 4, FALSE);
  gtk_widget_show(dia_editprop->table);
  gtk_box_pack_start (GTK_BOX (dia_editprop->vbox1), dia_editprop->table, FALSE, FALSE, 0);


  dia_editprop->text[0]=gtk_label_new( _("Layer: "));
  dia_editprop->text[1]=gtk_label_new( _("Colour: "));
  dia_editprop->text[2]=gtk_label_new( _("Linestyle: "));
  dia_editprop->text[3]=gtk_label_new( _("Linewidth: "));

  for(i=0;i<4;i++)
    {
      gtk_misc_set_alignment(GTK_MISC(dia_editprop->text[i]),0.0,0.0);
      gtk_table_attach_defaults(GTK_TABLE(dia_editprop->table), dia_editprop->text[i], 0, 1, i, i+1);
      gtk_widget_show(dia_editprop->text[i]);
    }

  /* -- Create option menu for layers -- */
  dia_editprop->option_layer=gtk_option_menu_new();
  gtk_widget_set_usize(GTK_WIDGET(dia_editprop->option_layer), 150, 24);
  gtk_table_attach_defaults(GTK_TABLE(dia_editprop->table), dia_editprop->option_layer, 1, 2, 0, 1);
  gtk_widget_show(dia_editprop->option_layer);
  
  dia_editprop->menu_layer=NULL;

  /* -- Create option menu for colour -- */
  dia_editprop->option_colour=gtk_option_menu_new();
  gtk_widget_set_usize(GTK_WIDGET(dia_editprop->option_colour), 150, 24);
  gtk_table_attach_defaults(GTK_TABLE(dia_editprop->table), dia_editprop->option_colour, 1, 2, 1, 2);
  gtk_widget_show(dia_editprop->option_colour);

  dia_editprop->menu_colour=NULL;

  /* -- Create option menu for linestyle -- */
  dia_editprop->option_linestyle=gtk_option_menu_new();
  gtk_widget_set_usize(GTK_WIDGET(dia_editprop->option_linestyle), 150, 24);
  gtk_table_attach_defaults(GTK_TABLE(dia_editprop->table), dia_editprop->option_linestyle, 1, 2, 2, 3);
  gtk_widget_show(dia_editprop->option_linestyle);

  dia_editprop->menu_linestyle=NULL;

  /* -- Create combo menu for linewidth -- */
  dia_editprop->combo_linewidth=gtk_combo_new();
  gtk_widget_set_usize(GTK_WIDGET(dia_editprop->combo_linewidth), 150, 24);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dia_editprop->combo_linewidth)->entry), _("Don't Change"));
  gtk_combo_disable_activate(GTK_COMBO(dia_editprop->combo_linewidth));
  gtk_table_attach_defaults(GTK_TABLE(dia_editprop->table), dia_editprop->combo_linewidth, 1, 2, 3, 4);
  gtk_widget_show(dia_editprop->combo_linewidth);

  /* Create linewidth glist */
  linewidth_glist=g_list_append(linewidth_glist, _("Don't Change"));
  linewidth_glist=g_list_append(linewidth_glist, _("By Layer"));
  linewidth_glist=g_list_append(linewidth_glist, _("By Block"));
  linewidth_glist=g_list_append(linewidth_glist, "0.0");
  linewidth_glist=g_list_append(linewidth_glist, "0.5");
  linewidth_glist=g_list_append(linewidth_glist, "1.0");
  linewidth_glist=g_list_append(linewidth_glist, "2.0");
  linewidth_glist=g_list_append(linewidth_glist, "5.0");
  linewidth_glist=g_list_append(linewidth_glist, "10.0");
  linewidth_glist=g_list_append(linewidth_glist, "15.0");
  linewidth_glist=g_list_append(linewidth_glist, "20.0");
  linewidth_glist=g_list_append(linewidth_glist, "30.0");
  gtk_combo_set_popdown_strings(GTK_COMBO(dia_editprop->combo_linewidth), linewidth_glist); 

  /* -- Create Separator -- */
  dia_editprop->hr = gtk_hseparator_new ();
  gtk_widget_show (dia_editprop->hr);
  gtk_box_pack_start (GTK_BOX (dia_editprop->vbox1), dia_editprop->hr, FALSE, FALSE, 0);

  /* -- Create box for buttons -- */
  gtk_hbutton_box_set_spacing_default (4);
  gtk_hbutton_box_set_layout_default (GTK_BUTTONBOX_END);
  dia_editprop->hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_container_set_border_width (GTK_CONTAINER (dia_editprop->hbuttonbox1), 4);
  gtk_widget_show (dia_editprop->hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dia_editprop->vbox1), dia_editprop->hbuttonbox1, FALSE, FALSE, 0);

  /* Create OK button */
  dia_editprop->button_ok = gtk_button_new_with_label ( _("OK"));
  gtk_widget_show (dia_editprop->button_ok);
  gtk_container_add (GTK_CONTAINER (dia_editprop->hbuttonbox1), dia_editprop->button_ok);
  gtk_signal_connect(GTK_OBJECT(dia_editprop->button_ok),"clicked",
		     GTK_SIGNAL_FUNC(editprop_ok), NULL); 

  /* Create close button */
  dia_editprop->button_cancel = gtk_button_new_with_label ( _("Cancel"));
  gtk_widget_show (dia_editprop->button_cancel);
  gtk_container_add (GTK_CONTAINER (dia_editprop->hbuttonbox1), dia_editprop->button_cancel);
  gtk_signal_connect(GTK_OBJECT(dia_editprop->button_cancel),"clicked",
		     GTK_SIGNAL_FUNC(editprop_close), NULL); 
}

static gint change_layer(GtkWidget *widget, gpointer layer)
{
  new_layer=(LAYER *)layer;
  return TRUE;
}

static void make_layer(DOCUMENT *doc)
{
  GList *list;

  /* Realase previous layer menu */
  if(dia_editprop->menu_layer!=NULL)
    gtk_widget_destroy(dia_editprop->menu_layer);

  /* Create new menu */
  dia_editprop->menu_layer=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(dia_editprop->option_layer), dia_editprop->menu_layer);

  /* Create first menu item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_layer),  make_menuitem( _("Don't Change"), 
								     GTK_SIGNAL_FUNC (change_layer), 
								     (gpointer)NULL));      
  
  /* Make item Separator */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_layer), make_menuitem_hr()); 

  /* Create a menuitem for each layer in document */
  list=g_list_first(doc->layer_list);
 
  while(list!=NULL)
    {
      /* Create layer menulist and connect it */
      gtk_menu_append(GTK_MENU(dia_editprop->menu_layer),  make_menuitem(((LAYER *)list->data)->name, 
									 GTK_SIGNAL_FUNC (change_layer), 
									 (gpointer)list->data));      
      list=g_list_next(list);
    }

  /* Set the current layer to layer number 0 */
  gtk_option_menu_set_history(GTK_OPTION_MENU(dia_editprop->option_layer),0);
}

static gint change_type_colour(GtkWidget *widget, gpointer type_colour)
{
  new_type_colour=GPOINTER_TO_INT(type_colour);
  new_colour=NULL;
  return TRUE;
}

static gint change_colour_predefined(GtkWidget *widget, gpointer colour)
{
  new_type_colour=PREDEFINED;
  new_colour=(COLOUR *)colour;
  return TRUE;
}

static gint change_colour_userdefined(GtkWidget *widget, gpointer *doc)
{
  gdouble colour[3];

  if(dia_colour_selection(colour))
    {
      if(new_type_colour==USERDEFINED && new_colour!=NULL)
	g_free(new_colour);
      
      new_type_colour=USERDEFINED;
 
      new_colour=make_new_userdefined_colour((DOCUMENT *)doc, colour[0], colour[1], colour[2]);

    }

  return TRUE;
}
static void make_colour(DOCUMENT *doc)
{
  GtkWidget *menuitem, *draw;
  gint i;

  /* Realase previous layer menu */
  if(dia_editprop->menu_colour!=NULL)
    gtk_widget_destroy(dia_editprop->menu_colour);

  /* Create new menu */
  dia_editprop->menu_colour=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(dia_editprop->option_colour), dia_editprop->menu_colour);

  /* Create first menu item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), make_menuitem( _("Don't Change"), GTK_SIGNAL_FUNC (change_type_colour), 
								     GINT_TO_POINTER(DONT_CHANGE))); 

  /* Make item Separator */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), make_menuitem_hr()); 

  /* Create By Layer item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), make_menuitem( _("By Layer"), GTK_SIGNAL_FUNC (change_type_colour), 
									GINT_TO_POINTER(BYLAYER))); 
  /* Create By Block item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), make_menuitem( _("By Block"), GTK_SIGNAL_FUNC (change_type_colour), 
									GINT_TO_POINTER(BYBLOCK))); 

  /* Make item Separator */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), make_menuitem_hr()); 

  /* Create a colour item */
  for(i=0;i<doc->no_predefined_colours;i++)
    {
      menuitem=gtk_menu_item_new();

      gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
			 GTK_SIGNAL_FUNC (change_colour_predefined),
			 (gpointer)&doc->predefined_colours[i]);

      gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), menuitem);
      gtk_widget_show(menuitem);

      gtk_widget_realize(menuitem);

      draw=gtk_drawing_area_new();
      gtk_drawing_area_size(GTK_DRAWING_AREA(draw), 70,14);
      gtk_widget_show(draw);

      /* Set colour of drawing box */
      gtk_container_add (GTK_CONTAINER (menuitem), draw);

      gdk_window_set_background(draw->window, &doc->predefined_colours[i].colour);
      gdk_window_clear(draw->window);
    }

  /* Make item Separator */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), make_menuitem_hr()); 

  /* Create Userdefined item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_colour), make_menuitem( _("Userdefined"), 
								      GTK_SIGNAL_FUNC (change_colour_userdefined), 
								      doc)); 

  /* Set the current layer to layer number 0 */
  gtk_option_menu_set_history(GTK_OPTION_MENU(dia_editprop->option_colour),0);
}

static gint change_type_linestyle(GtkWidget *widget, gpointer linestyle)
{
  new_type_linestyle=GPOINTER_TO_INT(linestyle);
  new_linestyle=NULL;
  return TRUE;
}

static gint change_linestyle(GtkWidget *widget, gpointer linestyle)
{
  new_type_linestyle=PREDEFINED;
  new_linestyle=(LINESTYLE *)linestyle;
  return TRUE;
}

static void make_linestyle(DOCUMENT *doc)
{
  GList *list;

  /* Release previous layer menu */
  if(dia_editprop->menu_linestyle!=NULL)
    gtk_widget_destroy(dia_editprop->menu_linestyle);

  /* Create new menu */
  dia_editprop->menu_linestyle=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(dia_editprop->option_linestyle), dia_editprop->menu_linestyle);

  /* Create first menu item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_linestyle), make_menuitem( _("Don't Change"), 
									 GTK_SIGNAL_FUNC (change_type_linestyle), 
									 GINT_TO_POINTER(DONT_CHANGE))); 

  /* Make item Separator */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_linestyle), make_menuitem_hr()); 

  /* Create By Layer item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_linestyle), make_menuitem( _("By Layer"), 
									 GTK_SIGNAL_FUNC (change_type_linestyle), 
									 GINT_TO_POINTER(BYLAYER))); 
  /* Create By Block item */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_linestyle), make_menuitem( _("By Block"), 
									 GTK_SIGNAL_FUNC (change_type_linestyle), 
									 GINT_TO_POINTER(BYBLOCK))); 

  /* Make item Separator */
  gtk_menu_append(GTK_MENU(dia_editprop->menu_linestyle), make_menuitem_hr()); 

  /* Create a menuitem for each layer in document */
  list=g_list_first(doc->linestyle_list);
 
  while(list!=NULL)
    {
      /* Create linestyle menuitem and connect it */
      gtk_menu_append(GTK_MENU(dia_editprop->menu_linestyle),  make_menuitem(((LINESTYLE*)list->data)->name, 
									     GTK_SIGNAL_FUNC (change_linestyle), 
									     (gpointer)list->data));       
      list=g_list_next(list);
    }

  /* Set the current layer to layer number 0 */
  gtk_option_menu_set_history(GTK_OPTION_MENU(dia_editprop->option_linestyle),0);
}

void eprop_select_object(DOCUMENT *doc)
{
 grab_document=doc;

  if(grab_document!=NULL)
    {
      if(dia_editprop==NULL)
	make_dialog_editprop();

      menus_set_sensitive("/Modify/Edit/Properties", FALSE);

      /* Make menu for layers */
      make_layer(grab_document);

      /* Make menu for colour */
      make_colour(grab_document);

      /* Make menu for linestyle */
      make_linestyle(grab_document);

      /* Make some default values */
      new_layer=NULL;
      new_type_linestyle=DONT_CHANGE;
      new_linestyle=NULL;
      new_type_colour=DONT_CHANGE;
      new_colour=NULL;
      gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dia_editprop->combo_linewidth)->entry), _("Don't Change"));

      gtk_widget_show(dia_editprop->window);
    }
}

void tool_set_tool_eprop(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("eprop"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_EPROP;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to edit: "), _("Selection - Edit properties"));
  set_prompt();

  work_document=NULL;
}





