/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "xor.h"
#include "misc.h"
#include "dia_misc.h"
#include "select.h"
#include "layer.h"
#include "parse.h"
#include "redraw.h"
#include "dia_comm.h"
#include "dia_coords.h"
#include "obj_text.h"
#include "obj_attribute.h"
#include "obj_insert.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  F U N C T I O N S  -- */
GList *select_attribute_in_block(OBJECT *insert, gdouble x, gdouble y, gdouble d, gchar **prompt)
{
  LAYER *layer;
  TRANSFORM insert_transform;
  OBJECT object;
  BLOCK_OBJECT *block_object;
  GList *list;
  GList *attr_list;

  /* Make tranform matrix */
  insert_transform.type=OBJ_TRANSLATE;
  insert_transform.trans_x=insert->data.block.insert_point.x;
  insert_transform.trans_y=insert->data.block.insert_point.y;
  insert_transform.cen_x=0;
  insert_transform.cen_y=0;

  /* Check if need to scale block */
  if(ABS(insert->data.block.scale_x-1)>GFLOAT_SENSITIVE ||
     ABS(insert->data.block.scale_y-1)>GFLOAT_SENSITIVE)
    {
      insert_transform.type|=OBJ_SCALE;
      insert_transform.scale_x=insert->data.block.scale_x;
      insert_transform.scale_y=insert->data.block.scale_y;
    }

  /* Check if need to rotate block */
  if(ABS(insert->data.block.rotation)>GFLOAT_SENSITIVE)
    {
      insert_transform.type|=OBJ_ROTATE;
      insert_transform.rot_angle=-insert->data.block.rotation*M_PI/180.0;
    }

  /* Get list of attributes in this block */
  attr_list=g_list_first(insert->data.block.attr);

  /* Get first object in block */
  list=g_list_first(insert->data.block.block->object_list);

  while(list!=NULL)
    {
      block_object=(BLOCK_OBJECT *)list->data;

      /* Check if we should use the layer of the block */
      if(!strcmp(block_object->name_of_layer,"0"))
	layer=insert->layer;
      else
	{
	  layer=layer_find(insert->layer->doc, block_object->name_of_layer);
	  if(layer==NULL)
	    {
	      g_warning( _("Layer %s not found. Creating it."), block_object->name_of_layer );
	      layer=layer_new(insert->layer->doc, -1, insert->layer->doc->current_linestyle, 0.0 , 0.0, 0.0, 0.0,  
			  block_object->name_of_layer, TRUE, TRUE);
	    }
	}

      /* Make nice object to draw */
      object.type=block_object->type;	 
      object.layer=layer;

      /* Make linewidth */
      object.type_linewidth=block_object->type_linewidth;
      switch(object.type_linewidth)
	{
	case BYLAYER:
	  object.linewidth=layer->linewidth;
	  break;
	case BYBLOCK:
	  object.linewidth=insert->linewidth;
	  break;
	case USERDEFINED:
	  object.linewidth=block_object->linewidth;
	  break;
	}
      
      memcpy(&object.data, &block_object->data, sizeof(OBJECT_UNION));
      make_object_selected(&object, OBJECT_SEL, FALSE);
	  
      switch(object.type)
	{
	case TYPE_ATTRIBUTE:
	  obj_transform_attribute(&object, &insert_transform);
	  /* Fix text of attribute */
	  object.data.attribute.text.text=(gchar *)attr_list->data;

	  if(obj_select_attribute(&object, x, y, d))	    
	    {
	      *prompt=block_object->data.attribute.prompt;
	      return attr_list;
	    }
	  attr_list=g_list_next(attr_list);
	  break;
	}
      
      list=g_list_next(list);
    }

  return NULL;
}

void tool_set_prompt_etext(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Select text or attribute to edit: "));
      break;
    }

}

void tool_set_tool_etext(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("etext"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_ETEXT;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  set_prompt();

  work_document=NULL;
}

void tool_button_etext(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  OBJECT *selobj;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }
  
  switch(gl_stage)
    {
    case 0:
      selobj=select_singel_object(doc,x,y);
      if(selobj!=NULL)
	{
	  GList *attr;
	  gchar *prompt=NULL;
	  gchar *tmp;

	  switch(selobj->type)
	    {
	    case TYPE_TEXT:
	      if((tmp=dia_enter_text( _("Edit text"), _("New text: "), selobj->data.text.text))!=NULL)
		{
		  selobj->selected=OBJECT_ERASE;
		  redraw_object(doc, selobj);
		  
		  g_free(selobj->data.text.text);
		  selobj->data.text.text=g_strdup(tmp);
		  obj_bound_text(selobj);
		}	    	      
	      deselect_all_objects(doc);
	      tool_idle(doc);
	      return;
	    case TYPE_ATTRIBUTE:
	      prompt=g_strdup_printf("%s: ", selobj->data.attribute.prompt);
	      if((tmp=dia_enter_text( _("Edit attribute"), prompt, selobj->data.attribute.text.text))!=NULL)
		{
		  selobj->selected=OBJECT_ERASE;
		  redraw_object(doc, selobj);
		  
		  g_free(selobj->data.attribute.text.text);
		  selobj->data.attribute.text.text=g_strdup(tmp);
		  obj_bound_attribute(selobj);
		}
	      g_free(prompt);
	      deselect_all_objects(doc);
	      tool_idle(doc);
	      return; 
	    case TYPE_BLOCK:
	      attr=select_attribute_in_block(selobj, x,y, gl_select_sensitivity/doc->zoom, &prompt);
	      if(attr!=NULL)
		{
		  if((tmp=dia_enter_text( _("Edit attribute"), prompt, (gchar *) attr->data))!=NULL)
		    {
		      selobj->selected=OBJECT_ERASE;
		      redraw_object(doc, selobj);

		      g_free(attr->data);
		      attr->data=g_strdup(tmp);
		      obj_bound_insert(selobj);
		    }

		  deselect_all_objects(doc);
		  tool_idle(doc);
		  return; 		  
		}	      
	      deselect_all_objects(doc);
	      break;
	    default:
	      print( _("\n  Object must be a text or attribute."));
	      deselect_all_objects(doc);
	    }
	}	  
      else
	print( _("\n  No object found!"));
      set_prompt();
      break;
    }
}

void tool_prompt_etext(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx,ty;

  switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
		      FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
  case PROMPT_POINT2:
    work_document=doc;
    remove_old();
    tool_button_etext(doc,1,tx,ty);    
    break;
  }
}












