/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "struct.h"
#include "color.h"
#include "file_ocadis.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */


/* --  S A V E   B L O C K S  -- */
static void write_attribute(FILE *fp, BLOCK_OBJECT *block_object)
{
  /* Colour */
  if(block_object->type_colour==BYLAYER)
    fprintf(fp, ",L");
  else if(block_object->type_colour==BYBLOCK)
    fprintf(fp, ",B");
  else
    fprintf(fp,",U,%f,%f,%f",block_object->colour->c[0], block_object->colour->c[1], block_object->colour->c[2]);
  
  /* Linestyle */
  if(block_object->type_linestyle==BYLAYER)
    fprintf(fp, ",L");
  else if(block_object->type_linestyle==BYBLOCK)
    fprintf(fp, ",B");
  else
    fprintf(fp, ",U,\"%s\"", block_object->name_of_linestyle);
  
  /* Linewidth */
  if(block_object->type_linewidth==BYLAYER)
    fprintf(fp, ",L");
  else if(block_object->type_linewidth==BYBLOCK)
    fprintf(fp, ",B");
  else
    fprintf(fp, ",U,%f", block_object->linewidth);
 
 fprintf(fp, "\n");
}

static void write_object_line(FILE *fp, BLOCK_OBJECT *block_object)
{
  fprintf( fp," OL,\"%s\",%f,%f,%f,%f", block_object->name_of_layer,
	   block_object->data.line.p1.x, block_object->data.line.p1.y,
	   block_object->data.line.p2.x, block_object->data.line.p2.y);
  write_attribute(fp, block_object);
}

static void write_object_circle(FILE *fp, BLOCK_OBJECT *block_object)
{
  fprintf( fp," OC,\"%s\",%f,%f,%f", block_object->name_of_layer, 
	      block_object->data.circle.centrum.x, block_object->data.circle.centrum.y,
	      block_object->data.circle.radius);
  write_attribute(fp, block_object);
}

static void write_object_arc(FILE *fp, BLOCK_OBJECT *block_object)
{
  fprintf( fp," OA,\"%s\",%f,%f,%f,%f,%f",block_object->name_of_layer,
	   block_object->data.arc.centrum.x,block_object->data.arc.centrum.y,
	   block_object->data.arc.radius, block_object->data.arc.start_angle, block_object->data.arc.end_angle);
  write_attribute(fp, block_object);
}

static void write_object_text(FILE *fp, BLOCK_OBJECT *block_object)
{
  if(block_object->data.text.style!=NULL)
    fprintf( fp," OT,\"%s\",\"%s\",%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\"",block_object->name_of_layer, 
	     block_object->data.text.style->name, 
	     block_object->data.text.insert.x, block_object->data.text.insert.y,
	     block_object->data.text.alignment.x, block_object->data.text.alignment.y,
	     block_object->data.text.angle, block_object->data.text.oblique,
	     block_object->data.text.height, block_object->data.text.width,
	     block_object->data.text.justification, block_object->data.text.flags,
	     block_object->data.text.text);
  else
    fprintf( fp," OT,\"%s\",NULL,%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\"",block_object->name_of_layer,
	     block_object->data.text.insert.x, block_object->data.text.insert.y,
	     block_object->data.text.alignment.x, block_object->data.text.alignment.y,
	     block_object->data.text.angle, block_object->data.text.oblique,
	     block_object->data.text.height, block_object->data.text.width,
	     block_object->data.text.justification, block_object->data.text.flags,
	     block_object->data.text.text);

  write_attribute(fp, block_object);
}

static void write_object_attribute(FILE *fp, BLOCK_OBJECT *block_object)
    {
  if(block_object->data.attribute.text.style!=NULL)
    fprintf(fp," OW,\"%s\",\"%s\",%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\",\"%s\"",block_object->name_of_layer, 
	    block_object->data.attribute.text.style->name, 
	    block_object->data.attribute.text.insert.x, block_object->data.attribute.text.insert.y,
	    block_object->data.attribute.text.alignment.x, block_object->data.attribute.text.alignment.y,
	    block_object->data.attribute.text.angle, block_object->data.attribute.text.oblique,
	    block_object->data.attribute.text.height, block_object->data.attribute.text.width,
	    block_object->data.attribute.text.justification, block_object->data.attribute.text.flags,
	    block_object->data.attribute.prompt, block_object->data.attribute.text.text);
  else
    fprintf(fp," OW,\"%s\",NULL,%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\",\"%s\"",block_object->name_of_layer,
	    block_object->data.attribute.text.insert.x, block_object->data.attribute.text.insert.y,
	    block_object->data.attribute.text.alignment.x, block_object->data.attribute.text.alignment.y,
	    block_object->data.attribute.text.angle, block_object->data.attribute.text.oblique,
	    block_object->data.attribute.text.height, block_object->data.attribute.text.width,
	    block_object->data.attribute.text.justification, block_object->data.attribute.text.flags,
	    block_object->data.attribute.prompt, block_object->data.attribute.text.text);

  write_attribute(fp, block_object);
}

void file_block_save(FILE *fp, BLOCK *block)
{
  BLOCK_OBJECT *block_object;
  GList *list;

  /* Write block name and number of objects */
  fprintf(fp,"B,\"%s\",%d\n", block->name, g_list_length(block->object_list));

  list=g_list_first(block->object_list);
  
  while(list!=NULL)
    {
      block_object=(BLOCK_OBJECT *)list->data;

      switch(block_object->type)
	{
	case TYPE_LINE:
	  write_object_line(fp, block_object);
	  break;
	case TYPE_CIRCLE:
	  write_object_circle(fp, block_object);
	  break;
	case TYPE_ARC:
	  write_object_arc(fp, block_object);
	  break;
	case TYPE_TEXT:
	  write_object_text(fp, block_object);
	  break;
	case TYPE_ATTRIBUTE:
	  write_object_attribute(fp, block_object);
	  break;
	}

      list=g_list_next(list);
    }
}

/* --  L O A D  B L O C K S  -- */
static gchar arg[25][50];

static void get_all_attributes(DOCUMENT *doc, gint sarg, gint *tcol, COLOUR **col, gint *tstyle, gchar **style, 
			       gint *twidth, gdouble *width)
{
  gint index;

  /* Get colour of object */
  if(arg[sarg][0]=='B')  
    {
      *tcol=BYBLOCK;
      index=0;
    }
  else if(arg[sarg][0]=='L')  
    {
      *tcol=BYLAYER;
      index=0;
    }
  else
    {
      *tcol=USERDEFINED;

      *col=make_new_userdefined_colour(doc,g_strtod(arg[sarg+1],NULL),
				       g_strtod(arg[sarg+2],NULL),g_strtod(arg[sarg+3],NULL));
      index=3;
    }
  
  /* Get linestyle of object */
  if(arg[sarg+1+index][0]=='B')
    *tstyle=BYBLOCK;
  else if(arg[sarg+1+index][0]=='L')
    *tstyle=BYLAYER;
  else
    {
      *tstyle=USERDEFINED;
      *style=g_strdup(arg[sarg+2+index]);
      index++;
    }
  
  /* Get linewidth of object */
  if(arg[sarg+2+index][0]=='B')
    *twidth=BYBLOCK;
  else if(arg[sarg+2+index][0]=='L')
    *twidth=BYLAYER;
  else
    {
      *twidth=USERDEFINED;
      *width=g_strtod(arg[sarg+3+index],NULL);
    }  
}

void file_block_load(FILE *fp, DOCUMENT *doc, gchar *name, gint no)
{
  BLOCK *block;
  BLOCK_OBJECT *block_object;
  gint ts;
  gint i;

  block=g_new(BLOCK,1);
  
  block->name=g_strdup(name);
  block->object_list=NULL;
  block->attribute=FALSE;

  for(i=0;i<no;i++)
    {
      read_line(fp,&ts,arg[0]);		      

      /* Create new block object */
      block_object=g_new(BLOCK_OBJECT, 1);

      /* Fix layer name */
      block_object->name_of_layer=g_strdup(arg[1]);

      switch(arg[0][1])
	{
	case 'L':   /* Line */
	  block_object->type=TYPE_LINE;
	  /* Fix objects attributes */
	  get_all_attributes(doc, 6, &(block_object->type_colour), &(block_object->colour), 
			     &(block_object->type_linestyle), &(block_object->name_of_linestyle), 
			     &(block_object->type_linewidth), &(block_object->linewidth));	  

	  /* Fix object specific data */
	  block_object->data.line.p1.x=g_strtod(arg[2],NULL);
	  block_object->data.line.p1.y=g_strtod(arg[3],NULL);
	  block_object->data.line.p2.x=g_strtod(arg[4],NULL);
	  block_object->data.line.p2.y=g_strtod(arg[5],NULL);
	  break;
	case 'C':   /* Circle */
	  block_object->type=TYPE_CIRCLE;
	  /* Fix objects attributes */
	  get_all_attributes(doc, 5, &(block_object->type_colour), &(block_object->colour), 
			     &(block_object->type_linestyle), &(block_object->name_of_linestyle), 
			     &(block_object->type_linewidth), &(block_object->linewidth));	  

	  /* Fix object specific data */
	  block_object->data.circle.centrum.x=g_strtod(arg[2],NULL);
	  block_object->data.circle.centrum.y=g_strtod(arg[3],NULL);
	  block_object->data.circle.radius=g_strtod(arg[4],NULL);
	  break;
	case 'A':   /* Arc */
	  block_object->type=TYPE_ARC;
	  /* Fix objects attributes */
	  get_all_attributes(doc, 7, &(block_object->type_colour), &(block_object->colour), 
			     &(block_object->type_linestyle), &(block_object->name_of_linestyle), 
			     &(block_object->type_linewidth), &(block_object->linewidth));	  

	  /* Fix object specific data */
	  block_object->data.arc.centrum.x=g_strtod(arg[2],NULL);
	  block_object->data.arc.centrum.y=g_strtod(arg[3],NULL);
	  block_object->data.arc.radius=g_strtod(arg[4],NULL);
	  block_object->data.arc.start_angle=g_strtod(arg[5],NULL);
	  block_object->data.arc.end_angle=g_strtod(arg[6],NULL);
	  break;
	case 'T':   /* Text*/
	  block_object->type=TYPE_TEXT;
	  /* Fix objects attributes */
	  get_all_attributes(doc, 15, &(block_object->type_colour), &(block_object->colour), 
			     &(block_object->type_linestyle), &(block_object->name_of_linestyle), 
			     &(block_object->type_linewidth), &(block_object->linewidth));	  
	  
	  /* Fix object specific data */
	  if(!strcmp(arg[2],"NULL"))
	    {
	      block_object->data.text.insert.x=g_strtod(arg[3],NULL);
	      block_object->data.text.insert.y=g_strtod(arg[4],NULL);
	      block_object->data.text.alignment.x=g_strtod(arg[5],NULL);
	      block_object->data.text.alignment.y=g_strtod(arg[6],NULL);
	      block_object->data.text.angle=g_strtod(arg[7],NULL);
	      block_object->data.text.oblique=g_strtod(arg[8],NULL);
	      block_object->data.text.height=g_strtod(arg[9],NULL);
	      block_object->data.text.width=g_strtod(arg[10],NULL);
	      block_object->data.text.justification=g_strtod(arg[11],NULL);
	      block_object->data.text.flags=g_strtod(arg[12],NULL);
	      block_object->data.text.text=g_strdup(arg[13]);	      
	    }
	  else
	    {
	      /* Write this! */
	    }
	  break;
	case 'W':   /* Attribute */
	  block_object->type=TYPE_ATTRIBUTE;
	  /* Fix objects attributes */
	  get_all_attributes(doc, 15, &(block_object->type_colour), &(block_object->colour), 
			     &(block_object->type_linestyle), &(block_object->name_of_linestyle), 
			     &(block_object->type_linewidth), &(block_object->linewidth));	  
	  
	  /* Fix object specific data */
	  if(!strcmp(arg[2],"NULL"))
	    {
	      block_object->data.attribute.text.insert.x=g_strtod(arg[3],NULL);
	      block_object->data.attribute.text.insert.y=g_strtod(arg[4],NULL);
	      block_object->data.attribute.text.alignment.x=g_strtod(arg[5],NULL);
	      block_object->data.attribute.text.alignment.y=g_strtod(arg[6],NULL);
	      block_object->data.attribute.text.angle=g_strtod(arg[7],NULL);
	      block_object->data.attribute.text.oblique=g_strtod(arg[8],NULL);
	      block_object->data.attribute.text.height=g_strtod(arg[9],NULL);
	      block_object->data.attribute.text.width=g_strtod(arg[10],NULL);
	      block_object->data.attribute.text.justification=g_strtod(arg[11],NULL);
	      block_object->data.attribute.text.flags=g_strtod(arg[12],NULL);
	      block_object->data.attribute.prompt=g_strdup(arg[13]);
	      block_object->data.attribute.text.text=g_strdup(arg[14]);	      
	    }
	  else
	    {
	      /* Write this! */
	    }

	  /* We have an attribute in the block */
	  block->attribute=TRUE;
	  break;
	}

      /* Add this object to the block */
      block->object_list=g_list_append(block->object_list, block_object);
    }  

  /* Appending block to document */
  doc->block_list=g_list_append(doc->block_list,block);

}


