/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* This code to read/write a DXF-file is taken from 
   QCad, version 1.2.0 (www.qcad.org) but changed 
   to C-language and heavily modified to fit oCADis. 

   Any bugs found in this code is most likly created 
   by me and not by Andrew Mustun who wrote the 
   orginal code. */

/* TODO
   
   2000-08-11
   Fix so that import of polylines with arcs works 
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "misc.h"
#include "layer.h"
#include "textstyle.h"
#include "linestyle.h"
#include "dimension.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_polyline.h"
#include "obj_spline.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
#define WLDtoPLT_x(x) ((gdouble) ( x ))
#define WLDtoPLT_y(y) ((gdouble) ( bound[1]-y ))
#define WLDtoPLT(r)   ((gdouble) ( r ))

static gdouble bound_area[4];

/* --  F U N C T I O N S  -- */

/* -- H E L P   F U N C T I O N S -- */
static gint to_int(gchar *string)
     /* Reads and returns an integer from string */
{
  return (gint)g_strtod(string,NULL);
}

static gdouble to_double(gchar *string)
     /* Reads and returns a double from string */
{
  return (gdouble)g_strtod(string,NULL);
}

static gdouble corr_angle(gdouble angle)
     /* Returns an angle that fits between 0 and 360 */
{
  while(angle<   0.0) angle+=360.0;
  while(angle>=360.0) angle-=360.0;
  return angle;
}

static gint dxf_get_colour(gdouble *c)
     /* Selects correct colour number based
	on the predefined colours */
{
  return 1;
}


/* -- W R I T E   D X F -- */

gboolean file_export_dxf(DOCUMENT *doc, FILE *dxf_file)
     /* Exports a DXF-file */
{
  misc_get_size_of_drawing(doc, &bound_area[0], &bound_area[1], &bound_area[2], &bound_area[3]);
  
  
  return TRUE;
}
 
/* -- R E A D   D X F -- */
static void make_default_values(DOCUMENT *doc, LAYER **layer, 
				gint *type_linewidth, gdouble *linewidth,
				gint *type_linestyle, LINESTYLE **linestyle,
				gint *type_colour, COLOUR **colour)
  /* Make the default values for a object */
{
  /* Set linewidth */
  *type_linewidth=BYLAYER;
  *linewidth=doc->current_linewidth;
  /* Set linestyle */
  *type_linestyle=BYLAYER;
  *linestyle=doc->current_linestyle;
  /* Set linecolour */
  *type_colour=BYLAYER;
  *colour=doc->current_colour;
  /* Set layer */
  *layer=doc->current_layer;
}

static gboolean check_standard_code(DOCUMENT *doc, gint code, gchar *dxf_line, LAYER **layer, 
				    gint *type_linewidth, gdouble *linewidth,
				    gint *type_linestyle, LINESTYLE **linestyle,
				    gint *type_colour, COLOUR **colour)
  /* Checks and setup the common codes for objects.
     Returns TRUE if attribut(s) changed or else FALSE */
{
  switch(code)
    {
    case  6:  /* style */
      *type_linestyle=USERDEFINED;
      *linestyle=linestyle_find(doc, dxf_line);

      if(*linestyle==NULL)
	{
	  *linestyle=linestyle_new(doc, -1, dxf_line,  LINE_SOLID, 1, 0,NULL);
	  make_linestyle_popmenu(doc);
	}
      return TRUE;
      break;
    case  8:  /* Layer */
      *layer=layer_find(doc, dxf_line);
      
      /* If that layer was not found then create it */
      if(*layer==NULL)
	{
	  *layer=layer_new(doc, -1, doc->linestyle_list->data, 0, 0, 0, 0, 
			  dxf_line, TRUE, TRUE);
	  make_layer_popmenu(doc);
	}
      return TRUE;
      break;
    case 39:  /* Thickness */
      *type_linewidth=USERDEFINED;
      *linewidth=to_double(dxf_line);
      return TRUE;
      break;
    case 62:  /* Color */
      *type_colour=USERDEFINED;
      *colour=&doc->predefined_colours[to_int(dxf_line) % doc->no_predefined_colours];
      break;
    }

  return FALSE;
}

static gboolean read_entities(DOCUMENT *doc, FILE *dxf_file)
     /* Reads entities found in dxf_file */
{
  gboolean  ret;                    /* returned value */
  gint      code=-1;                /* Dxf-code as number */
  gdouble   vx1=0.0, vy1=0.0,       /* Start point */
            vx2=0.0, vy2=0.0,       /* End point */
            vcx=0.0, vcy=0.0,       /* Centre */
            vcr=0.0,                /* Radius */
            va1=0.0, va2=0.0,       /* Start / End Angle */
            vab=0.0,                /* Bulge */
            vpx=0.0, vpy=0.0;       /* First Polyline point */
  gdouble   ax=0.0, ay=0.0;         /* Current coordinate */
  gboolean  plClose=FALSE;          /* Polyline closed-flag */
  gchar     dxf_line[255];          /* A line in the dxf file */
  gchar     dxf_code[255];          /* A Code in the dxf file as string */
  gint      type_linewidth;         /* Type of linewidth */
  gdouble   linewidth;              /* Current linewidth */
  gint      type_linestyle;         /* Type of linestyle */
  LINESTYLE *linestyle;             /* Current linestyle */
  gint      type_colour;            /* Type of colour */
  COLOUR    *colour;                /* Current colour */
  LAYER     *layer=NULL;            /* Holds current layer */
  gdouble   switch_y=0.0;           /* Variable to mirror the y-coordinate */

  g_warning("read_entities");

  while(!feof(dxf_file))
    {
      fgets(dxf_line,255,dxf_file);
      g_strstrip(dxf_line);

      if(dxf_line[0]>='A' && dxf_line[0]<='Z') 
	{
	  /* We have reach the end! */
	  if(!strcmp(dxf_line,"ENDSEC")) 
	    return TRUE;
	  /* LINE */
	  else if(!strcmp(dxf_line, "LINE")) 
	    {
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);

	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{
			  switch(code) 
			    {			      
			    case 10:  /* X1 */
			      vx1 = to_double(dxf_line);
			      break;
			    case 20:  /* Y1 */
			      vy1 = to_double(dxf_line);
			      break;
			    case 11:  /* X2 */
			      vx2 = to_double(dxf_line);
			      break;
			    case 21:  /* Y2 */
			      vy2 = to_double(dxf_line);
			      break;			
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 

	      obj_add_line(doc, layer, NULL, type_colour, colour, 
			   type_linestyle, linestyle, type_linewidth, linewidth,
			   vx1,switch_y-vy1,vx2,switch_y-vy2);
	    }
	  /* RAY */
	  else if(!strcmp(dxf_line, "RAY")) 
	    {
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);
	      
	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);
		  
		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);
		      
		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{
			  switch(code) 
			    {			      
			    case 10:  /* X1 */
			      vx1 = to_double(dxf_line);
			      break;
			    case 20:  /* Y1 */
			      vy1 = to_double(dxf_line);
			      break;
			    case 11:  /* X2 */
			      vx2 = to_double(dxf_line);
			      break;
			    case 21:  /* Y2 */
			      vy2 = to_double(dxf_line);
			      break;			
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 
	      
	      obj_add_rline(doc, layer, NULL, type_colour, colour, 
			    type_linestyle, linestyle, type_linewidth, linewidth,
			    vx1,switch_y-vy1,vx2+vx1,switch_y-(vy2+vy1));
	    }
	  /* XLINE */
	  else if(!strcmp(dxf_line, "XLINE")) 
	    {
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);

	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{
			  switch(code) 
			    {			      
			    case 10:  /* X1 */
			      vx1 = to_double(dxf_line);
			      break;
			    case 20:  /* Y1 */
			      vy1 = to_double(dxf_line);
			      break;
			    case 11:  /* X2 */
			      vx2 = to_double(dxf_line);
			      break;
			    case 21:  /* Y2 */
			      vy2 = to_double(dxf_line);
			      break;			
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 

	      obj_add_cline(doc, layer, NULL, type_colour, colour, 
			    type_linestyle, linestyle, type_linewidth, linewidth,
			    vx1,switch_y-vy1,vx2+vx1,switch_y-(vy2+vy1));
	    }
	  /* CIRCLE */
	  else if(!strcmp(dxf_line, "CIRCLE")) 	  
	    {
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);	    

	      do 
		{

		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{		    
			  switch(code) 
			    {
			    case 10:  /* Centre X */
			      vcx = to_double(dxf_line);
			      break;
			    case 20:  /* Centre Y */
			      vcy = to_double(dxf_line);
			      break;
			    case 40:  /* Radius */
			      vcr = to_double(dxf_line);
			      break;
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 

	      obj_add_circle(doc, layer, NULL, type_colour, colour, 
			     type_linestyle, linestyle, type_linewidth, linewidth,
			     vcx,switch_y-vcy,vcr);
	    }
	  /* ARC */
	  else if(!strcmp(dxf_line, "ARC")) 	  
	    {
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);

	      do 
		{

		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{		    
			  switch(code) 
			    {
			    case 10:  /* Centre X */
			      vcx = to_double(dxf_line);
			      break;
			    case 20:  /* Centre Y */
			      vcy = to_double(dxf_line);
			      break;
			    case 40:  /* Radius */
			      vcr = to_double(dxf_line);
			      break;
			    case 50:  /* Start Angle */
			      va1 = corr_angle(to_double(dxf_line));
			      break;
			    case 51:  /* End Angle */
			      va2 = corr_angle(to_double(dxf_line));
			      break;
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 

	      /* Here should the angles of the arc be fixed so
		 that an arc is drawn at the right direction */

	      obj_add_arc(doc, layer, NULL, type_colour, colour, 
			  type_linestyle, linestyle, type_linewidth, linewidth,
			  vcx, switch_y-vcy, vcr, va1, va2);
	    }
	  /* ELLIPSE or elliptical arc */
	  else if(!strcmp(dxf_line, "ELLIPSE")) 	  
	    {
	      gdouble vapx=0.0, vapy=0.0; /* Apex (endpoint of center axis) */
	      gdouble vratio=1.0;         /* Ratio of minor axis to major axis */

	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);
	      
	      va1=0.0;
	      va2=2*M_PI;

	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              		 
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);
		      
		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{		    
			  switch(code) 
			    {
			    case 10:  /* Centre X */
			      vcx = to_double(dxf_line);
			      break;
			    case 20:  /* Centre Y */
			      vcy = to_double(dxf_line);
			      break;
			    case 11:  /* Endpoint of major axis X */
			      vapx = to_double(dxf_line);
			      break;
			    case 21:  /* Endpoint of major axis Y */
			      vapy = to_double(dxf_line);
			      break;
			    case 40:  /* Ratio of minor axis to major axis (eg.: 0.75) */
			      vratio = to_double(dxf_line);
			      break; 
			    case 41:  /* Start angle in rad (0.0 for a full ellipse) */
			      va1 = to_double(dxf_line);
			      break;
			    case 42:  /* End angle in rad (2PI=6.283 for a full ellipse) */
			      va2 = to_double(dxf_line);
			      break;
			    }
			}
		    }
		} while(code!=0); 
	      
	      if(fabs(va1)<GFLOAT_SENSITIVE && fabs(va2-2*M_PI)<GFLOAT_SENSITIVE)
		obj_add_ellipse(doc, layer, NULL, type_colour, colour, 
				type_linestyle, linestyle, type_linewidth, linewidth, 
				vcx, switch_y-vcy, dist(0,0,vapx,vapy), 
				dist(0,0,vapx,vapy)*vratio,
				-get_angle(0,0,vapx,vapy));
	      else
		obj_add_elarc(doc, layer, NULL, type_colour, colour, 
				type_linestyle, linestyle, type_linewidth, linewidth, 
				vcx, switch_y-vcy, dist(0,0,vapx,vapy), 
				dist(0,0,vapx,vapy)*vratio,
				-get_angle(0,0,vapx,vapy), va1*180/M_PI, va2*180/M_PI);
	    }
	  /* TEXT */
	  else if(!strcmp(dxf_line, "TEXT")) 	  
	    {
	      gchar vtext[255];
	      gdouble vheight=0.0;
	      gdouble vtextang=0.0;
	      gint valignv=0, valignh=0, vfl=TEXT_JUST_BL;
	      TEXTSTYLE *textstyle=NULL;
	      gint valigntable[4][6]={{TEXT_JUST_BL, TEXT_JUST_BC, TEXT_JUST_BR, TEXT_JUST_AL, 0, TEXT_JUST_FI},
				      {TEXT_JUST_BL, TEXT_JUST_BC, TEXT_JUST_BR,            0, 0,            0},
				      {TEXT_JUST_ML, TEXT_JUST_MC, TEXT_JUST_MR,            0, 0,            0},
				      {TEXT_JUST_TL, TEXT_JUST_TC, TEXT_JUST_TR,            0, 0,            0}};
				    
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);
	      
	      vtext[0]='\0';
	      
	      do 
		{
		  
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
		  
		  code=to_int(dxf_code);
		  
		  if(code!=0) 
		    {              		 
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);
		      
		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{		    
			  switch(code) 
			    {
			    case  1:  /* Text itself */
			      strcpy(vtext, dxf_line);
			      break;
			    case  7:  /* Text style */
			      textstyle=textstyle_find(doc, dxf_line);
			      break;
			    case 10:  /* X1 */
			      vx1 = to_double(dxf_line);
			      break;
			    case 20:  /* Y1 */
			      vy1 = to_double(dxf_line);
			      break;
			    case 11:  /* X1 Aligment */
			      vx2 = to_double(dxf_line);
			      break;
			    case 21:  /* Y1 Aligment */
			      vy2 = to_double(dxf_line);
			      break;
			    case 40:  /* height */
			      vheight = to_double(dxf_line);
			      break;
			    case 50:  /* angle */
			      vtextang = to_double(dxf_line);
			      break;
			    case 72:  /* alignment horizontel */
			      valignh = to_int(dxf_line);
			    case 73:  /* alignment vertical */
			      valignv = to_int(dxf_line);
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 
	      
	      /* Check for alignment */
	      if(valignv>=0 && valignv<=3 && valignh>=0 && valignh<=5)
		vfl=valigntable[valignv][valignh];

	      obj_add_text(doc, layer, NULL, type_colour, colour, 
			   type_linestyle, linestyle, type_linewidth, linewidth, 
			   textstyle, vfl, vtextang, vheight, 
			   vx1, switch_y-vy1, vx2, switch_y-vy2, vtext);
	    }
	  /* POLYLINE */
	  else if(!strcmp(dxf_line, "LWPOLYLINE")) 
	    {
	      gint pclose=POLYLINE_OPEN;
	      OBJECT *pline=NULL;
	      
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);
	      
	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{
			  switch(code) 
			    {			      
			    case 70:  /* Flag "Polyline closed" */
			      if( 1 & to_int(dxf_line))
				pclose=POLYLINE_CLOSE_LINE; 
			      break;
			    case 10:  /* X1 */
			      vx1 = to_double(dxf_line);
			      break;
			    case 20:  /* Y1 */
			      vy1 = to_double(dxf_line);

			      /* Hopefully we now got all values for polyline */
			      if(pline==NULL)
				pline=create_new_polyline(doc, layer, NULL, type_colour, colour, 
							  type_linestyle, linestyle, type_linewidth, linewidth,
							  pclose);
			      add_node(pline, NODE_LINE, vx1, switch_y-vy1, 1,1,1);
			      break;
			    case 42:  /* Bulge angle */
			      va1=to_double(dxf_line)*4;
			      if(pline!=NULL)
				{
				  NODE *last;

				  g_warning("Adding bulge: %f", va1);
				  last=(NODE *)g_list_last(pline->data.polyline.node_list)->data;
				  /*  last->type=NODE_ARC;
				      add_node(pline, NODE_LINE, last->x, switch_y-last->y, 1,1,1); */
				  /* Calc arc-node considering the bulge-value
				     The bulge value is 1/4 of the included angle for an arc segement(?) */

				  /*				  obj_add_line(doc, layer, NULL, type_colour, colour, 
					       type_linestyle, linestyle, type_linewidth, linewidth,
					       last->x,last->y,last->x+10*cos(va1),(last->y+10*sin(va1)));
				  */
				}
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 

	      obj_bound_polyline(pline);
	      layer->object_list=g_list_append(layer->object_list,(gpointer)pline);
	    }
	  /* SPLINE */
	  else if(!strcmp(dxf_line, "SPLINE")) 
	    {
	      gint pclose=FALSE;
	      OBJECT *spline=NULL;
	      
	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);
	      
	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{
			  switch(code) 
			    {			      
			    case 70:  /* Flag "Polyline closed" */
			      if( 1 & to_int(dxf_line))
				pclose=TRUE;
			      break;
			    case 10:  /* X1 */
			      vx1 = to_double(dxf_line);
			      break;
			    case 20:  /* Y1 */
			      vy1 = to_double(dxf_line);

			      if(spline==NULL)
				spline=create_new_spline(doc, layer, NULL, type_colour, colour, 
							 type_linestyle, linestyle, type_linewidth, linewidth,
							 pclose);
			      spline_add_node(spline, vx1, switch_y-vy1, 1);
			      break;
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 

	      obj_bound_spline(spline);
	      layer->object_list=g_list_append(layer->object_list,(gpointer)spline);
	    }
	  /* DIMENSION */
	  else if(!strcmp(dxf_line, "DIMENSION")) 
	    {
	      OBJECT *dim;
	      gint typ=1;
	      gdouble v10=0.0, v20=0.0,
                      v13=0.0, v23=0.0,
                      v14=0.0, v24=0.0,
                      v15=0.0, v25=0.0,
                      v16=0.0, v26=0.0,
                      v40=0.0, v50=0.0;

	      /* Make default values */
	      make_default_values(doc, &layer, &type_linewidth, &linewidth, &type_linestyle, &linestyle,
				  &type_colour, &colour);
	      
	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      if(!check_standard_code(doc, code, dxf_line, &layer, &type_linewidth, &linewidth, 
					      &type_linestyle, &linestyle, &type_colour, &colour))
			{
			  switch(code) 
			    {			      
			    case 10:  /* line position x */
			      v10 = to_double(dxf_line);
			      break;
			    case 20:  /* line position y */
			      v20 = to_double(dxf_line);
			      break;
			    case 13:  /* X1 */ 
			      v13 = to_double(dxf_line);
			      break;
			    case 23:  /* Y1 */
			      v23 = to_double(dxf_line);
			      break;
			    case 14:  /* X2 */
			      v14 = to_double(dxf_line);
			      break;
			    case 24:  /* Y2 */
			      v24 = to_double(dxf_line);
			      break;
			    case 15:  /* X2 */
			      v15 = to_double(dxf_line);
			      break;
			    case 25:  /* Y2 */
			      v25 = to_double(dxf_line);
			    case 16:  /* X2 */
			      v16 = to_double(dxf_line);
			      break;
			    case 26:  /* Y2 */
			      v26 = to_double(dxf_line);
			      break;
			    case 40:
			      v40 = to_double(dxf_line);
			      break;
			    case 50:
			      v50 = to_double(dxf_line);
			      break;
			    case 70:  /* Typ */
			      typ = to_double(dxf_line);
			      break;			      
			    default:
			      break;
			    }
			}
		    }
		} while(code!=0); 

	      /* Remove Bit values: */
	      if(typ>=128) typ-=128;   /* Location of Text */
	      if(typ>= 64) typ-= 64;   /* Ordinate */

	      switch(typ) 
		{
		case 0:
		  /* Horizontal */		  
		  if(fabs(v50)<GFLOAT_SENSITIVE)
		    {
		      dim=dim_add_linear(doc, layer, NULL, doc->dimensionstyle_list->data, type_colour,colour,
					 DIM_HORIZ,2);		     
		      dim_add_node(dim, v13, switch_y-v23, v10, switch_y-v20, NULL);
		      dim_add_node(dim, v14, switch_y-v24, v10, switch_y-v20, NULL);
		    }
		  else /* Vertical */
		    {
		      dim=dim_add_linear(doc, layer, NULL, doc->dimensionstyle_list->data, type_colour,colour,
					 DIM_VERT,2);
		      dim_add_node(dim, v13, switch_y-v23, v10, switch_y-v20, NULL);
		      dim_add_node(dim, v14, switch_y-v24, v10, switch_y-v20, NULL);
		    }		 
		  break;
		  /* Aligned */ 
		case 1: 
		      dim=dim_add_linear(doc, layer, NULL, doc->dimensionstyle_list->data, type_colour,colour,
					 DIM_ALIGNED,2);		     
		      dim_add_node(dim, v13, switch_y-v23, v10, switch_y-v20, NULL);
		      dim_add_node(dim, v14, switch_y-v24, v10, switch_y-v20, NULL);
		  break;
		}
	    }
	}
    }
  
  /* Unexcepted end of DXF-file */
  return FALSE;
}

static gboolean read_tables(DOCUMENT *doc, FILE *dxf_file)
     /* Reads tables in DXF-file (layers and linestyles) */
{ 
  gint      code=-1;                /* Dxf-code as number */
  gchar     dxf_line[255];          /* A line in the dxf file */
  gchar     dxf_code[255];          /* A Code in the dxf file as string */

  g_warning("tables");

  while(!feof(dxf_file))
    {
      fgets(dxf_line,255,dxf_file);
      g_strstrip(dxf_line);
  
      if(dxf_line[0]>='A' && dxf_line[0]<='Z')       
	{
	  if(!strcmp(dxf_line,"ENDSEC"))
	    return TRUE;
	  /* LINESTYLE */
	  if(!strcmp(dxf_line,"LTYPE"))
	    {
	      gchar ltype_name[255];       /* Linestyle name */
	      gdouble ltype_length=0.0;    /* Linestyle length */
	      gdouble ltype_no=0.0;        /* Linestyle number of dash length item */
	      gdouble *ltype_pattern=NULL; /* Linestyle dash item */
	      gint count=0;

	      ltype_name[0]='\0';

	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      switch(code) 
			{
			case 2:   /* Linestyle name */
			  strcpy(ltype_name, dxf_line);
			  break;
			case 73:   /* Linestyle number of dash */
			  ltype_no=to_int(dxf_line);
			  ltype_pattern=g_new(gdouble, ltype_no);
			  break;
			case 40:   /* Linestyle total length */
			  ltype_length=to_double(dxf_line);
			  break;
			case 49:  /* Dash item */
			  ltype_pattern[count]=fabs(to_double(dxf_line));
			  count++;
			  break;
			}
		    }
		} while(code!=0); 	 
	      
	      /* Check for legal names */
	      if(ltype_name!='\0' && strcmp("BYLAYER", ltype_name) && 
		 strcmp("BYBLOCK", ltype_name) && strcmp("", ltype_name))
		{
		  if(ltype_length>0.0)
		    linestyle_new(doc, -1, ltype_name, LINE_NOT_SOLID, ltype_length,  ltype_no, ltype_pattern);
		  else
		    linestyle_new(doc, -1, ltype_name, LINE_SOLID, 1.0, 0, NULL);
		  make_linestyle_popmenu(doc);
		}
	    }
	  /* LAYER */
	  if(!strcmp(dxf_line,"LAYER"))
	    {
	      gchar layer_name[255];       /* Layer name */
	      gint layer_flags=0;          /* Layer flags */
	      gint layer_colour=0;         /* Layer colour */
	      gboolean layer_visible=TRUE; /* Layer visible */
	      gchar layer_linestyle[255];  /* Layer name */
	      COLOUR *colour;

	      layer_name[0]='\0';
	      layer_linestyle[0]='\0';
	      colour=&doc->predefined_colours[0]; 

	      do 
		{
		  fgets(dxf_code,255,dxf_file);
		  g_strstrip(dxf_code);
	 
		  code=to_int(dxf_code);

		  if(code!=0) 
		    {              
		      fgets(dxf_line,255,dxf_file);
		      g_strstrip(dxf_line);

		      switch(code) 
			{
			case 2:   /* Layer name */
			  strcpy(layer_name, dxf_line);
			  break;
			case 70:  /* Layer flags */
			  layer_flags=to_int(dxf_line);
			  break;
			case 62:  /* Colour number (if negative layer invisible) */
			  layer_colour=to_int(dxf_line);
			  if(layer_colour<0)
			    {
			      layer_colour=-layer_colour;
			      layer_visible=FALSE;
			    }
			  colour=&doc->predefined_colours[(layer_colour-1) % doc->no_predefined_colours];
			  break;
			case 6:   /* Linestyle */
			  strcpy(layer_linestyle, dxf_line);
			  break;
			}
		    }
		} while(code!=0); 	      

	      /* Dont add a layer if it dont have a name */
	      if(layer_name[0]!='\0')
		{
		  layer_new(doc, -1, doc->linestyle_list->data, 0, 
			    colour->c[0], colour->c[1],colour->c[2],
			    layer_name, layer_visible, !(layer_flags & 4));	      
		  make_layer_popmenu(doc);	     
		  
		}
	    }
	}
    }
  
  return FALSE;
}

gboolean file_import_dxf(DOCUMENT *doc, FILE *dxf_file) 
     /* Imports a DXF-file */
{
  gchar     dxf_line[255];          /* A line in the dxf file */

  while(!feof(dxf_file))
    {
      fgets(dxf_line,255,dxf_file);
      g_strstrip(dxf_line);
      
      if(dxf_line[0]>='A' && dxf_line[0]<='Z')       
	{
	  if(!strcmp(dxf_line,"EOF"))
	    return TRUE;
	  else if(!strcmp(dxf_line,"TABLES"))
	    read_tables(doc,dxf_file);		  
	  else if(!strcmp(dxf_line,"ENTITIES"))
	    read_entities(doc,dxf_file);		  
	}
    }

  return FALSE;
}
 
