/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "misc.h"
#include "draw.h"
#include "calc_arc.h"
#include "dia_pagesetup.h"
#include "obj_polyline.h"
#include "dimension.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

#define MM2UNIT 40

#define WLDtoPLT_x(x) ((gint) rint(MM2UNIT*( (x-plot_xcen) * plot_zoom + plot_xpan) ))
#define WLDtoPLT_y(y) ((gint) rint(MM2UNIT*( (plot_ycen-y) * plot_zoom + plot_ypan) ))
#define WLDtoPLT(r)   ((gint) rint(r*plot_zoom*MM2UNIT))

static gdouble plot_zoom, plot_xpan, plot_ypan;
static gdouble plot_xcen, plot_ycen;
static gdouble bound_area[4];



static FILE *hpgl_file;

/* --  F U N C T I O N S  -- */

static void hpgl_draw_line(DOCUMENT *doc, GdkGC *gc, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  fprintf(hpgl_file, "PA%d,%d;PD;PA%d,%d;PU;",
	  WLDtoPLT_x(x1), WLDtoPLT_y(y1),
	  WLDtoPLT_x(x2), WLDtoPLT_y(y2));
}

static void hpgl_draw_arrow(DOCUMENT *doc,GdkGC *gc, gdouble x, gdouble y, gdouble angle,gdouble size)
{

}

static void hpgl_draw_text(DOCUMENT *doc, GdkGC *gc, gint just, gdouble x, gdouble y, 
			   gdouble height, gdouble width, gdouble rot, gchar *text)
{
  gint type;
  gdouble tx,ty;

  draw_text_set_text(just,x,y,height,width,rot,text);

  while(draw_text_get_vertex(&type, &tx,&ty))
	{
	  if(type==TEXT_VERTEX_MOVETO)
	    fprintf(hpgl_file, "PA%d,%d;PD;", WLDtoPLT_x(tx), WLDtoPLT_y(ty));
	  else
	    fprintf(hpgl_file, "PA%d,%d;PU;", WLDtoPLT_x(tx), WLDtoPLT_y(ty));
	}
}

static GRDRAW hpgl_grdraw={hpgl_draw_line, hpgl_draw_arrow, hpgl_draw_text};

static void set_layer(LAYER *layer)
{
}

static gint set_object(OBJECT *object)
{
  return FALSE;
}

static void plot_arc(OBJECT_ARC *arc)
{
  gdouble tangle;

  if(arc->start_angle<=arc->end_angle)
    tangle=arc->end_angle-arc->start_angle;
  else
   tangle=arc->end_angle+(360-arc->start_angle);
  
  fprintf(hpgl_file, "PU;PA%d,%d;PD;AA%d,%d,%d;PU;",
	  WLDtoPLT_x(arc->centrum.x+arc->radius*
		     cos(arc->start_angle*M_PI/180.0)), 
	  WLDtoPLT_y(arc->centrum.y+arc->radius*
		     sin(arc->start_angle*M_PI/180.0)), 
	  WLDtoPLT_x(arc->centrum.x),
	  WLDtoPLT_y(arc->centrum.y),
	  (gint)tangle);
}

static void plot_polyline(OBJECT *pline)
{
}

static void plot_spline(OBJECT *spline)
{
}

static void plot_ellipse(OBJECT *ellipse)
{
}

static void plot_elarc(OBJECT *elarc)
{
}

static void plot_text(OBJECT *text)
{
  gint type;
  gdouble tx,ty;

 if(text->data.text.justification==TEXT_JUST_AL)
    {
      gdouble h,a;
      
      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      h=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text));
      
      draw_text_set_text(text->data.text.justification,text->data.text.insert.x,text->data.text.insert.y,
			 h, 1, a, text->data.text.text);
    }
  else if(text->data.text.justification==TEXT_JUST_FI)
    {
      gdouble w,a;

      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      w=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text)*text->data.text.height);

      draw_text_set_text(text->data.text.justification,text->data.text.insert.x,text->data.text.insert.y,
			 text->data.text.height, w, a, text->data.text.text);
    }
  else
    draw_text_set_text(text->data.text.justification,text->data.text.insert.x,text->data.text.insert.y,
		       text->data.text.height,text->data.text.width, text->data.text.angle*M_PI/180.0,text->data.text.text);

  while(draw_text_get_vertex(&type, &tx,&ty))
	{
	  if(type==TEXT_VERTEX_MOVETO)
	    fprintf(hpgl_file, "PA%d,%d;", WLDtoPLT_x(tx), WLDtoPLT_y(ty));
	  else
	    fprintf(hpgl_file, "PD;PA%d,%d;PU;", WLDtoPLT_x(tx), WLDtoPLT_y(ty));
	}
}

static void plot_point(OBJECT *point)
{
}

static void generate_entities(DOCUMENT *doc)
{
  gint change;
  GList *layer_list;
  GList *object_list;
  LAYER *layer;
  OBJECT *object;
  layer_list=g_list_first(doc->layer_list);

  while(layer_list!=NULL)
    {
      layer=(LAYER *)layer_list->data;

      object_list=g_list_first(layer->object_list);

      /* Set default colour and linewidth for current layer */
      set_layer(layer);

      while(object_list!=NULL)
	{
	  object=((OBJECT *)object_list->data);

	  change=set_object(object);

	  switch(object->type)
	    {
	    case TYPE_LINE:	    
	      fprintf(hpgl_file,"PA%d,%d;PD;PA%d,%d;PU;",
		     WLDtoPLT_x(object->data.line.p1.x), WLDtoPLT_y(object->data.line.p1.y),
		     WLDtoPLT_x(object->data.line.p2.x), WLDtoPLT_y(object->data.line.p2.y));
	      break;
	    case TYPE_CIRCLE:
	      fprintf(hpgl_file, "PA%d,%d;CI%d;",
		      WLDtoPLT_x(object->data.circle.centrum.x), WLDtoPLT_y(object->data.circle.centrum.y),
		      WLDtoPLT(object->data.circle.radius));
	      break;
	    case TYPE_ARC:
	      plot_arc(&object->data.arc);
	      break;
	    case TYPE_POLYLINE:
	      plot_polyline(object);
	      break;
	    case TYPE_SPLINE:
	      plot_spline(object);
	      break;
	    case TYPE_POINT:
	      plot_point(object);
	      break;
	    case TYPE_TEXT:
	      plot_text(object);
	      break;
	    case TYPE_ELLIPSE:
	      plot_ellipse(object);
	      break;
	    case TYPE_ELARC:
	      plot_elarc(object);
	      break;
	    case TYPE_DIMENSION:
	      dim_draw_dimension(doc, &hpgl_grdraw, object);
	      break;
	    }
	  object_list=g_list_next(object_list);
	}

      /* Restore graphic settings */
      layer_list=g_list_next(layer_list);
    }
}

gint file_export_hpgl(DOCUMENT *doc, FILE *my_file)
{
  misc_get_size_of_drawing(doc, &bound_area[0], &bound_area[1], &bound_area[2], &bound_area[3]);

  plot_xpan=pagesetup.width/2;
  plot_ypan=pagesetup.height/2;

  /*  plot_xpan=0;
  plot_ypan=0;
  */
  plot_xcen=fabs(bound_area[0]-bound_area[2])/2+bound_area[0];
  plot_ycen=fabs(bound_area[1]-bound_area[3])/2+bound_area[1];

  if(pagesetup.fittopage)
    {
      gdouble width,height;
      gdouble draw_width, draw_height;

      draw_width=ABS(bound_area[0]-bound_area[2]);
      draw_height=ABS(bound_area[1]-bound_area[3]);

      width=pagesetup.width-pagesetup.margin_left-pagesetup.margin_right;
      height=pagesetup.height-pagesetup.margin_top-pagesetup.margin_bottom;
      
      plot_zoom=MIN(width/draw_width, height/draw_height);
    }
  else
    plot_zoom=pagesetup.scale_mm/pagesetup.scale_unit;

  hpgl_file=my_file;

  fprintf(hpgl_file, "%cE%c.(;%c.I81;;17:%c.N;19:IN;SC;PU;SP1;LT;VS18;", 27, 27, 27, 27); 

  generate_entities(doc);

  return TRUE;
}






