/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "obj_polyline.h"
#include "redraw.h"
#include "misc.h"

/* --  L O C A L   V A R I A B L E S  -- */
#define CONN_SENS 0.001

/* --  F U N C T I O N S  -- */

/* Check if start and endpoints of polyline are the same 
   if so make a closed polygon */
void check_close(OBJECT *obj)
{
  gdouble sx,sy,ex,ey;

  /* Get coordinates */
  sx=((NODE *)g_list_first(obj->data.polyline.node_list)->data)->x;
  sy=((NODE *)g_list_first(obj->data.polyline.node_list)->data)->y;

  ex=((NODE *)g_list_last(obj->data.polyline.node_list)->data)->x;
  ey=((NODE *)g_list_last(obj->data.polyline.node_list)->data)->y;

  /* If they are close make close polyline and delete last point */
  if(fabs(sx-ex)<CONN_SENS && fabs(sy-ey)<CONN_SENS)
    {
      obj->data.polyline.closed=POLYLINE_CLOSE_LINE;
      g_list_remove(obj->data.polyline.node_list,g_list_last(obj->data.polyline.node_list));
    }
}

gboolean file_import_mapgen(DOCUMENT *doc, FILE *my_file)
{
  OBJECT *obj=NULL;
  gchar buffer[255];

  while(!feof(my_file))
    {
      fgets(buffer,255,my_file);

      if(buffer[0]=='#')  /* We want to create a new polyline */
	{
	  if(obj!=NULL)
	    {
	      check_close(obj);
	      obj_bound_polyline(obj);
	      set_current_attributes(doc,obj);
	      doc->current_layer->object_list=g_list_append(doc->current_layer->object_list,(gpointer)obj);
	    }
	  obj=create_new_polyline(doc, doc->current_layer, NULL, 
				  doc->type_colour, doc->current_colour, 
				  doc->type_linestyle, doc->current_linestyle,
				  doc->type_linewidth, doc->current_linewidth,
				  POLYLINE_OPEN);
	}
      else  /* Add a node to current polyline */
	{
	  float x,y;
	  sscanf(buffer,"%f\t%f",&x,&y);
	  add_node(obj, NODE_LINE, x, -y, 0, 0, 1);
	}

    }

  if(obj!=NULL)
    {
      check_close(obj);
      obj_bound_polyline(obj);
      set_current_attributes(doc,obj);
      doc->current_layer->object_list=g_list_append(doc->current_layer->object_list,(gpointer)obj);
    }

  redraw_window(doc);
  return TRUE;
}








