/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "initmenu.h"
#include "struct.h"
#include "struct/grdraw.h"
#include "misc.h"
#include "layer.h"
#include "color.h"
#include "pop_colour.h"
#include "linestyle.h"
#include "textstyle.h"
#include "dia_message.h"
#include "new_draw.h"
#include "obj_line.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_text.h"
#include "obj_attribute.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_polyline.h"
#include "obj_spline.h"
#include "obj_point.h"
#include "obj_insert.h"
#include "dim_center.h"
#include "dim_radius.h"
#include "dim_diameter.h"
#include "dim_leader.h"
#include "dim_angular.h"
#include "dimension.h"
#include "dim_bound.h"
#include "file_block.h"
#include "tool_block.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
DOCUMENT *save_document;

/* --  F U N C T I O N S  -- */
gint read_line(FILE *fp,gint *num, gchar *array){
  gchar buffer[300];
  gint i, j;
  gint exit=FALSE;
  gint citat=FALSE;

  fgets(buffer,300,fp);

  if(!strcmp(buffer,"EOF") || feof(fp) )
     return FALSE;

  i=j=*num=0;

  while(!exit){

    if(buffer[i]=='\0' || buffer[i]=='\n'){

      array[(*num)*50+j]='\0';

      (*num)++;

      exit=TRUE;
    }
    else if( (buffer[i]==',') && (!citat)){

      array[(*num)*50+j]='\0';

      (*num)++;
      j=0;

    }
    else if(buffer[i]=='\"'){
      citat=1-citat;
    }
    else{
      if(buffer[i]!=' ' && !citat){
        array[*num*50+j]=buffer[i];
        j++;
      }
      else if(citat){
        array[*num*50+j]=buffer[i];
        j++;
      }
    }


    i++;
  }
  return TRUE;
}

gint check_version(FILE *fp){
  gchar buffer[100];

  fgets(buffer,100,fp);

  return (!strcmp(buffer,"H,\"oCADis 1.0\"\n"));
}

static gchar arg[25][50];

static void get_all_attributes(DOCUMENT *doc, gint sarg, gint *tcol, COLOUR **col, gint *tstyle, LINESTYLE **style, 
			       gint *twidth, gdouble *width)
{
  gint index;

  /* Get colour of object */
  if(arg[sarg][0]=='B')  
    {
      *tcol=BYBLOCK;
      index=0;
    }
  else if(arg[sarg][0]=='L')  
    {
      *tcol=BYLAYER;
      index=0;
    }
  else
    {
      *tcol=USERDEFINED;

      *col=make_new_userdefined_colour(doc,g_strtod(arg[sarg+1],NULL),
				       g_strtod(arg[sarg+2],NULL),g_strtod(arg[sarg+3],NULL));
      index=3;
    }
  
  /* Get linestyle of object */
  if(arg[sarg+1+index][0]=='B')
    *tstyle=BYBLOCK;
  else if(arg[sarg+1+index][0]=='L')
    *tstyle=BYLAYER;
  else
    {
      *tstyle=USERDEFINED;
      *style=(LINESTYLE *)g_list_nth(doc->linestyle_list,(gint)g_strtod(arg[sarg+2+index],NULL))->data; 
      index++;
    }
  
  /* Get linewidth of object */
  if(arg[sarg+2+index][0]=='B')
    *twidth=BYBLOCK;
  else if(arg[sarg+2+index][0]=='L')
    *twidth=BYLAYER;
  else
    {
      *twidth=USERDEFINED;
      *width=g_strtod(arg[sarg+3+index],NULL);
    }  
}

void get_colour_attribute(DOCUMENT *doc, gint sarg, gint *tcol, COLOUR **col)
{
  if(arg[sarg][0]=='L')  
    *tcol=BYLAYER;
  else if(arg[sarg][0]=='B')
    *tcol=BYBLOCK;
  else
    {
      *tcol=USERDEFINED;
      
      *col=make_new_userdefined_colour(doc,g_strtod(arg[sarg+1],NULL),
				       g_strtod(arg[sarg+2],NULL),g_strtod(arg[sarg+3],NULL));
    }
}

DOCUMENT *file_open(gchar *filename)
{
  gdouble *pattern;
  gint i, no, ts;
  gint num;
  FILE *fp;

  DOCUMENT *load_doc=NULL;
  LAYER *load_layer=NULL;
  gint type_colour, type_linestyle, type_linewidth;
  COLOUR *colour=NULL;
  LINESTYLE *linestyle=NULL;
  DIMENSIONSTYLE *dimstyle;
  gdouble linewidth=0;
  OBJECT *temp_obj;

  fp=fopen(filename,"r");

  if(fp==NULL)
    {
      message(ICON_INFO, _("The file %s\nwas not found. Please make sure that this is\nthe correct path and filename."), filename);
      return NULL;
    }

  /* Check so this really is a oCADis drawing file */
  if(!check_version(fp)){
    message(ICON_INFO, _("The file %s\nis not supported by oCADis and\ncan therefor not be loaded."), filename);
    fclose(fp);
    return NULL;
  }

  load_doc=make_new_drawing(filename, 100, 100);       

  while(read_line(fp,&num,arg[0]))
    {
      switch(arg[0][0])
	{
	case 'V':  /* Viewport */
	  load_doc->xpan=g_strtod(arg[1],NULL);
	  load_doc->ypan=g_strtod(arg[2],NULL);
	  load_doc->zoom=g_strtod(arg[3],NULL);
	  break;
	case 'G':  /* Grid data */
	  load_doc->gl_ortho=(gint)g_strtod(arg[1],NULL);

	  load_doc->gl_grid=(gint)g_strtod(arg[2],NULL);
	  load_doc->gl_grid_x=(gint)g_strtod(arg[3],NULL);
	  load_doc->gl_grid_y=(gint)g_strtod(arg[4],NULL);

	  load_doc->gl_iso_view=(gint)g_strtod(arg[5],NULL);
	  load_doc->gl_iso_view=(gint)g_strtod(arg[6],NULL);

	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(load_doc->widgets.toolbar_ortho), load_doc->gl_ortho);
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(load_doc->widgets.toolbar_grid), load_doc->gl_grid);  
	  break;
	case 'N':  /* Snap data */
	  load_doc->gl_snap=(gint)g_strtod(arg[1],NULL);

	  load_doc->gl_snap_basex=g_strtod(arg[2],NULL);
	  load_doc->gl_snap_basey=g_strtod(arg[3],NULL);
	  load_doc->gl_snap_width=g_strtod(arg[4],NULL);
	  load_doc->gl_snap_height=g_strtod(arg[5],NULL);     
	  load_doc->gl_snap_angle=g_strtod(arg[6],NULL);

	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(load_doc->widgets.toolbar_snap), load_doc->gl_snap);
	  break;
	case 'C':  /* Colour */
	  no=(gint)g_strtod(arg[1],NULL);
	  make_new_colour(load_doc,no,&load_doc->predefined_colours[no],
			  g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),g_strtod(arg[4],NULL));
	  break;
	case 'S':  /* Linestyle */
	  if(num==4)  /* Line solid */
	    linestyle_new(load_doc,(gint)g_strtod(arg[1],NULL),(gchar *)arg[2],LINE_SOLID,
			  g_strtod(arg[3],NULL), 0, NULL);
	  else
	    {
	      no=(gint)g_strtod(arg[4],NULL);
	      
	      pattern=(gdouble *)g_malloc(sizeof(gdouble)*no);
	      for(i=0;i<no;i++)
		  pattern[i]=g_strtod(arg[5+i],NULL);

	      linestyle_new(load_doc,(gint)g_strtod(arg[1],NULL),(gchar *)arg[2],LINE_NOT_SOLID, 
			    g_strtod(arg[3],NULL), no, pattern);
	    }
	  break;
	case 'T':  /* Textstyle */
	  textstyle_new(load_doc,-1,arg[2],
			(gint)g_strtod(arg[1],NULL),(gint)g_strtod(arg[3],NULL),
			g_strtod(arg[4],NULL),g_strtod(arg[5],NULL),g_strtod(arg[6],NULL));
	  break;
	case 'Q':  /* Dimension style */
	  dimstyle=dimstyle_new(load_doc,arg[2]);
	  dimstyle->dimexe=g_strtod(arg[3],NULL);
	  dimstyle->dimexo=g_strtod(arg[4],NULL);
	  dimstyle->dimgap=g_strtod(arg[5],NULL);
	  dimstyle->dimtxt=g_strtod(arg[6],NULL);
	  dimstyle->dimdec=(gint)g_strtod(arg[7],NULL);
	  dimstyle->dimdli=g_strtod(arg[8],NULL);
	  dimstyle->dimatp=(gint)g_strtod(arg[9],NULL);
	  dimstyle->dimasz=g_strtod(arg[10],NULL);
	  dimstyle->dimmtp=(gint)g_strtod(arg[11],NULL);
	  dimstyle->dimmsz=g_strtod(arg[12],NULL);
	  break;
	case 'B':  /* Block definition */
	  file_block_load(fp, load_doc, arg[1], (gint)g_strtod(arg[2],NULL));
	  break;
	case 'L':  /* Layer */
	  /* Make sure that this layer doesnt exists */
	  load_layer=layer_find(load_doc, (gchar *)arg[1]);

	  if(load_layer==NULL)	    
	    {
	      no=(gint)g_strtod(arg[6],NULL);

	      load_layer=layer_new(load_doc, -1,(LINESTYLE *)g_list_nth(load_doc->linestyle_list,no)->data, 
				   g_strtod(arg[5],NULL),
				   g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),
				   (gchar *)arg[1],(gint)g_strtod(arg[7],NULL),(gint)g_strtod(arg[8],NULL));
	    }
	  break;
	case 'O':  /* Object */
	  switch(arg[0][1])
	    {
	    case 'L':  /* Line */
	      get_all_attributes(load_doc, 6, &type_colour, &colour, &type_linestyle, &linestyle, 
			     &type_linewidth, &linewidth);

	      obj_add_line(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			   type_linewidth, linewidth,
			   g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),
			   g_strtod(arg[4],NULL),g_strtod(arg[5],NULL));
	      break;
	    case 'R':  /* Ray */
	      get_all_attributes(load_doc, 6, &type_colour, &colour, &type_linestyle, &linestyle, 
			     &type_linewidth, &linewidth);

	      obj_add_rline(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			    type_linewidth, linewidth,
			    g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),
			    g_strtod(arg[4],NULL),g_strtod(arg[5],NULL));
	      break;
	    case 'O':  /* Construction line */
	      get_all_attributes(load_doc, 6, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      obj_add_cline(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			    type_linewidth, linewidth,
			    g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),
			    g_strtod(arg[4],NULL),g_strtod(arg[5],NULL));
	      break;
	    case 'C':  /* Circle */
	      get_all_attributes(load_doc, 5, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      obj_add_circle(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			     type_linewidth, linewidth,
			     g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),
			     g_strtod(arg[4],NULL));
	      break;
	    case 'A':  /* Arc */
	      get_all_attributes(load_doc, 7, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      obj_add_arc(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
	      		  type_linewidth, linewidth,
			  g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),
			  g_strtod(arg[4],NULL),g_strtod(arg[5],NULL),
			  g_strtod(arg[6],NULL));
	      break;
	    case 'T':  /* Text */
	      get_all_attributes(load_doc, 14, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      if(!strcmp("NULL", arg[2]))
		obj_add_text(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			     type_linewidth, linewidth,
			     textstyle_find(load_doc, arg[2]),(gint)g_strtod(arg[11],NULL), 
			     g_strtod(arg[7],NULL), g_strtod(arg[9],NULL),
			     g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),
			     g_strtod(arg[5],NULL),g_strtod(arg[6],NULL),
			     arg[13]);
	      else
		obj_add_text(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			     type_linewidth, linewidth,
			     NULL,(gint)g_strtod(arg[11],NULL), 
			     g_strtod(arg[7],NULL), g_strtod(arg[9],NULL),
			     g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),
			     g_strtod(arg[5],NULL),g_strtod(arg[6],NULL),
			     arg[13]);
	      break;
	    case 'W':  /* Attribute */
	      get_all_attributes(load_doc, 15, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      if(!strcmp("NULL", arg[2]))
		obj_add_attribute(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
				  type_linewidth, linewidth,
				  textstyle_find(load_doc, arg[2]),(gint)g_strtod(arg[11],NULL), 
				  g_strtod(arg[7],NULL), g_strtod(arg[9],NULL),
				  g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),
				  g_strtod(arg[5],NULL),g_strtod(arg[6],NULL),
				  arg[13], arg[14]);
	      else
		obj_add_attribute(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
				  type_linewidth, linewidth,
				  NULL,(gint)g_strtod(arg[11],NULL), 
				  g_strtod(arg[7],NULL), g_strtod(arg[9],NULL),
				  g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),
				  g_strtod(arg[5],NULL),g_strtod(arg[6],NULL),
				  arg[13], arg[14]);
	      break;
	    case 'P':  /* Point */
	      get_all_attributes(load_doc, 7, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      obj_add_point(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			    type_linewidth, linewidth,(gint)g_strtod(arg[6],NULL),(gint)g_strtod(arg[4],NULL),
			    g_strtod(arg[5],NULL),g_strtod(arg[2],NULL),g_strtod(arg[3],NULL));
	      break;
	    case 'E':  /* Ellipse */
	      get_all_attributes(load_doc, 7, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      obj_add_ellipse(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			      type_linewidth, linewidth,
			      g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),
			      g_strtod(arg[5],NULL),g_strtod(arg[6],NULL));

	      break;
	    case 'Z':  /* Elarc */
	      get_all_attributes(load_doc, 9, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);
	      
	      obj_add_elarc(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
			      type_linewidth, linewidth,
			      g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),
			      g_strtod(arg[5],NULL),g_strtod(arg[6],NULL),
			      g_strtod(arg[7],NULL),g_strtod(arg[8],NULL));
	      break;
	    case 'Y':  /* Polyline */
	      no=(gint)g_strtod(arg[4],NULL);

	      get_all_attributes(load_doc, 5, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      temp_obj=create_new_polyline(load_doc, load_layer,  arg[1], 
					   type_colour,colour,type_linestyle,linestyle, 
					   type_linewidth, linewidth,
					   (gint)g_strtod(arg[3],NULL));

	      for(i=0;i<no;i++)
		{		  
		  read_line(fp,&ts,arg[0]);
		    
		  add_node(temp_obj, (gint)g_strtod(arg[1],NULL),
			   g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),
			   g_strtod(arg[4],NULL),g_strtod(arg[5],NULL),
			   g_strtod(arg[6],NULL));
		}

	      obj_bound_polyline(temp_obj);
	      load_layer->object_list=g_list_append(load_layer->object_list,(gpointer)temp_obj);

	      break;
	    case 'S':  /* Spline */
	      no=(gint)g_strtod(arg[3],NULL);

	      get_all_attributes(load_doc, 4, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);

	      temp_obj=create_new_spline(load_doc, load_layer,  arg[1], 
					 type_colour,colour,type_linestyle,linestyle, 
					 type_linewidth, linewidth,
					 (gint)(gint)g_strtod(arg[2],NULL));

	      for(i=0;i<no;i++)
		{		  
		  read_line(fp,&ts,arg[0]);
		    
		  spline_add_node(temp_obj, g_strtod(arg[2],NULL),
				  g_strtod(arg[3],NULL),g_strtod(arg[6],NULL));
		}

	      obj_bound_spline(temp_obj);
	      load_layer->object_list=g_list_append(load_layer->object_list,(gpointer)temp_obj);
	      break;
	    case 'B':  /* Insert */
	      get_all_attributes(load_doc, 9, &type_colour, &colour, &type_linestyle, &linestyle, 
				 &type_linewidth, &linewidth);
	      
	      /* Check if block exists */
	      if(find_block_name(load_doc, arg[7])!=NULL)
		{
		  GList *load_list;
		 
		  temp_obj=obj_add_insert(load_doc, load_layer, arg[1],type_colour,colour,type_linestyle,linestyle, 
					  type_linewidth, linewidth,
					  g_strtod(arg[2],NULL),g_strtod(arg[3],NULL),g_strtod(arg[5],NULL),
					  g_strtod(arg[6],NULL),g_strtod(arg[4],NULL), NULL, find_block_name(load_doc, arg[7]));
		  
		  /* If block has attribute load them */
		  no=(gint)g_strtod(arg[8],NULL);		 
		  for(i=0, load_list=NULL ; i<no ; i++)
		    {		
		      read_line(fp,&ts,arg[0]);
		      load_list=g_list_append(load_list, g_strdup(arg[1]));
		    }
		  
		  if(load_list!=NULL)
		    temp_obj->data.block.attr=load_list;

		  obj_bound_insert(temp_obj);
		}
	      else
		g_warning("Block \"%s\" is not defined.", arg[7]);	     
	      break;
	    }
	  break;
	case 'D':  /* Dimension */
	  switch(arg[0][1])
	    {
	    case 'C':  /* Center */
	      get_colour_attribute(load_doc, 6, &type_colour, &colour);

	      dim_bound_dimension(load_doc, dim_add_center(load_doc, load_layer, arg[1], arg[2], type_colour,colour,
							   g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),g_strtod(arg[5],NULL)));
	      break;
	    case 'R':  /* Radius */
	      get_colour_attribute(load_doc, 11, &type_colour, &colour);

	      dim_bound_dimension(load_doc, dim_add_radius(load_doc, load_layer, arg[1], arg[2], type_colour,colour,
							   g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),g_strtod(arg[5],NULL),
							   g_strtod(arg[6],NULL),g_strtod(arg[7],NULL),g_strtod(arg[8],NULL),
							   g_strtod(arg[9],NULL),g_strdup(arg[10])));
	      break;
	    case 'D':  /* Diameter */
	      get_colour_attribute(load_doc, 13, &type_colour, &colour);

	      dim_bound_dimension(load_doc, dim_add_diameter(load_doc, load_layer, arg[1], arg[2], type_colour,colour,
							     g_strtod(arg[3],NULL),g_strtod(arg[4],NULL),g_strtod(arg[5],NULL),
							     g_strtod(arg[6],NULL),g_strtod(arg[7],NULL),g_strtod(arg[8],NULL),
							     g_strtod(arg[9],NULL),g_strtod(arg[10],NULL),g_strtod(arg[11],NULL),
							     g_strdup(arg[12])));
	      break;
	    case 'H':  /* Linear, horizontal */
	      get_colour_attribute(load_doc, 5, &type_colour, &colour);
	      temp_obj=dim_add_linear(load_doc, load_layer, arg[1], arg[2], type_colour,colour,
				      DIM_HORIZ, (gint)g_strtod(arg[3],NULL));
	      
	      no=(gint)g_strtod(arg[4],NULL);
	      for(i=0;i<no;i++)
		{		  
		  read_line(fp,&ts,arg[0]);
		  
		  if(ts==5)
		    dim_add_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL), 
				 g_strtod(arg[3],NULL), g_strtod(arg[4],NULL), NULL); 
		  else
		    dim_add_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL), 
				 g_strtod(arg[3],NULL), g_strtod(arg[4],NULL), arg[5]); 
		}	      
	      dim_bound_dimension(load_doc, temp_obj);
	      break;
	    case 'V':  /* Linear, vertical */
	      get_colour_attribute(load_doc, 5, &type_colour, &colour);
	      temp_obj=dim_add_linear(load_doc, load_layer, arg[1], arg[2], type_colour,colour,
				      DIM_VERT, (gint)g_strtod(arg[3],NULL));
	      
	      no=(gint)g_strtod(arg[4],NULL);
	      for(i=0;i<no;i++)
		{		  
		  read_line(fp,&ts,arg[0]);
		  
		  if(ts==5)		    
		    dim_add_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL), 
				 g_strtod(arg[3],NULL), g_strtod(arg[4],NULL), NULL); 
		  else		    
		    dim_add_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL), 
				 g_strtod(arg[3],NULL), g_strtod(arg[4],NULL), arg[5]); 
		}
	      dim_bound_dimension(load_doc, temp_obj);
	      break;
	    case 'A':  /* Linear, Aligned */
	      get_colour_attribute(load_doc, 5, &type_colour, &colour);
	      temp_obj=dim_add_linear(load_doc, load_layer, arg[1], arg[2], type_colour,colour,
				      DIM_ALIGNED, (gint)g_strtod(arg[3],NULL));
	      
	      no=(gint)g_strtod(arg[4],NULL);
	      for(i=0;i<no;i++)
		{		  
		  read_line(fp,&ts,arg[0]);
		 
		  if(ts==5)
		    dim_add_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL), 
				 g_strtod(arg[3],NULL), g_strtod(arg[4],NULL), NULL); 
		  else
		    dim_add_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL), 
				 g_strtod(arg[3],NULL), g_strtod(arg[4],NULL), arg[5]); 
		}
	      dim_bound_dimension(load_doc, temp_obj);
	      break;
	    case 'X':  /* Angular dimension */
	      get_colour_attribute(load_doc, 13, &type_colour, &colour);

	      if(strcmp(arg[12],"NULL"))
		dim_bound_dimension(load_doc, dim_add_angular(load_doc, load_layer, arg[1], arg[2], type_colour, colour,
							      g_strtod(arg[3],NULL), g_strtod(arg[4],NULL),
							      g_strtod(arg[5],NULL), g_strtod(arg[6],NULL), (gint)g_strtod(arg[7],NULL),
							      g_strtod(arg[8],NULL), g_strtod(arg[9],NULL), (gint)g_strtod(arg[10],NULL),
							      g_strtod(arg[11],NULL),
							      g_strdup(arg[12])));
	      else
		dim_bound_dimension(load_doc, dim_add_angular(load_doc, load_layer, arg[1], arg[2], type_colour, colour,
							      g_strtod(arg[3],NULL), g_strtod(arg[4],NULL),
							      g_strtod(arg[5],NULL), g_strtod(arg[6],NULL), (gint)g_strtod(arg[7],NULL),
							      g_strtod(arg[8],NULL), g_strtod(arg[9],NULL), (gint)g_strtod(arg[10],NULL),
							      g_strtod(arg[11],NULL),
							      NULL));	      
	      break;
	    case 'L':  /* Leader */
	      get_colour_attribute(load_doc, 4, &type_colour, &colour);
	      
	      no=(gint)g_strtod(arg[3],NULL);
	      
	      temp_obj=dim_add_leader(load_doc, load_layer, arg[1], arg[2],type_colour,colour);
	      
	      for(i=0;i<no;i++)
		{		  
		  read_line(fp,&ts,arg[0]);
		  
		  if(ts==3)
		    add_leader_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL));
		  else
		    {
		      add_leader_node(temp_obj, g_strtod(arg[1],NULL), g_strtod(arg[2],NULL));
		      ((DIMENSION_RADIUS *)g_list_last(temp_obj->data.dimension.data)->data)->text=
			g_strdup(arg[3]);
		    }
		}	      
	      dim_bound_dimension(load_doc, temp_obj);
	      break;
	    }
	}
    }

  load_doc->current_layer=load_doc->layer_list->data;
  load_doc->no_predefined_colours=8;

  load_doc->type_colour=BYLAYER;
  load_doc->current_colour=load_doc->current_layer->colour;
  load_doc->type_linestyle=BYLAYER;
  load_doc->current_linestyle=load_doc->current_layer->linestyle;
  load_doc->type_linewidth=BYLAYER;
  load_doc->current_linewidth=load_doc->current_layer->linewidth;

  make_layer_popmenu(load_doc);
  make_colour_popmenu(load_doc);
  make_linestyle_popmenu(load_doc);

  gtk_option_menu_set_history(GTK_OPTION_MENU(load_doc->widgets.toolbar_colour),0);
  gtk_option_menu_set_history(GTK_OPTION_MENU(load_doc->widgets.toolbar_option),0);
  gtk_option_menu_set_history(GTK_OPTION_MENU(load_doc->widgets.toolbar_linestyle),0);
  
  fclose(fp);

  load_doc->saved=TRUE;
  make_window_title(load_doc);

  new_make_default(load_doc);

  load_doc->current_dimensionstyle=(DIMENSIONSTYLE *)load_doc->dimensionstyle_list->data;

  return load_doc;
}


/* -- W R I T E  F I L E -- */

void write_object_linestyle(OBJECT *obj, gchar *string)
{
  if(obj->type_linestyle==BYLAYER)
    g_snprintf(string,29,"L");
  else if(obj->type_linestyle==BYBLOCK)
    g_snprintf(string,29,"B");
  else
    g_snprintf(string,29,"U,%d",g_list_index(obj->layer->doc->linestyle_list, obj->linestyle));
}

void write_object_linewidth(OBJECT *obj, gchar *string)
{
  if(obj->type_linewidth==BYLAYER)
    g_snprintf(string,29,"L");
  else if(obj->type_linewidth==BYBLOCK)
    g_snprintf(string,29,"B");
  else
    g_snprintf(string,29,"U,%f",obj->linewidth);
}

void write_object_colour(OBJECT *obj, gchar *string)
{
  if(obj->type_colour==BYLAYER)
      g_snprintf(string,29,"L");
  else if(obj->type_colour==BYBLOCK)
      g_snprintf(string,29,"B");
  else
      g_snprintf(string,29,"U,%f,%f,%f",obj->colour->c[0],obj->colour->c[1],obj->colour->c[2]);
}

void write_object(gpointer data, gpointer fp){
  OBJECT *obj;
  GList *list;
  gchar junk1[30]="Error 1";
  gchar junk2[30]="Error 2";
  gchar junk3[30]="Error 3";

  obj=(OBJECT *)data;

  write_object_colour(obj, junk1);
  write_object_linestyle(obj, junk2);
  write_object_linewidth(obj, junk3);
  
  switch(obj->type)
    {
    case TYPE_LINE:
      fprintf((FILE *)fp," OL,\"%s\",%f,%f,%f,%f", obj->name,
	      obj->data.line.p1.x,obj->data.line.p1.y,
	      obj->data.line.p2.x,obj->data.line.p2.y);
      break;
    case TYPE_CLINE:
      fprintf((FILE *)fp," OO,\"%s\",%f,%f,%f,%f", obj->name,
	      obj->data.line.p1.x,obj->data.line.p1.y,
	      obj->data.line.p2.x,obj->data.line.p2.y);
      break;
    case TYPE_RLINE:
      fprintf((FILE *)fp," OR,\"%s\",%f,%f,%f,%f", obj->name,
	      obj->data.line.p1.x,obj->data.line.p1.y,
	      obj->data.line.p2.x,obj->data.line.p2.y);
      break;
    case TYPE_CIRCLE:
      fprintf((FILE *)fp," OC,\"%s\",%f,%f,%f", obj->name, 
	      obj->data.circle.centrum.x,obj->data.circle.centrum.y,
	      obj->data.circle.radius);
      break;
    case TYPE_ARC:
      fprintf((FILE *)fp," OA,\"%s\",%f,%f,%f,%f,%f",obj->name,
	      obj->data.arc.centrum.x,obj->data.arc.centrum.y,
	      obj->data.arc.radius, obj->data.arc.start_angle, obj->data.arc.end_angle);
      break;
    case TYPE_ELLIPSE:
      fprintf((FILE *)fp," OE,\"%s\",%f,%f,%f,%f,%f",obj->name,
	      obj->data.ellipse.centrum.x,obj->data.ellipse.centrum.y,
	      obj->data.ellipse.major, obj->data.ellipse.minor, 
	      obj->data.ellipse.angle);
      break;
    case TYPE_ELARC:
      fprintf((FILE *)fp," OZ,\"%s\",%f,%f,%f,%f,%f,%f,%f",obj->name,
	      obj->data.elarc.centrum.x,obj->data.elarc.centrum.y,
	      obj->data.elarc.major, obj->data.elarc.minor, 
	      obj->data.elarc.angle,obj->data.elarc.start_angle,obj->data.elarc.end_angle);
      break;
    case TYPE_TEXT:
      if(obj->data.text.style!=NULL)
	fprintf((FILE *)fp," OT,\"%s\",\"%s\",%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\"",obj->name, obj->data.text.style->name, 
		obj->data.text.insert.x, obj->data.text.insert.y,
		obj->data.text.alignment.x, obj->data.text.alignment.y,
		obj->data.text.angle, obj->data.text.oblique,
		obj->data.text.height, obj->data.text.width,
		obj->data.text.justification, obj->data.text.flags,
		obj->data.text.text);
      else
	fprintf((FILE *)fp," OT,\"%s\",NULL,%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\"",obj->name,
		obj->data.text.insert.x, obj->data.text.insert.y,
		obj->data.text.alignment.x, obj->data.text.alignment.y,
		obj->data.text.angle, obj->data.text.oblique,
		obj->data.text.height, obj->data.text.width,
		obj->data.text.justification, obj->data.text.flags,
		obj->data.text.text);
      break;
    case TYPE_ATTRIBUTE:
      if(obj->data.attribute.text.style!=NULL)
	fprintf((FILE *)fp," OW,\"%s\",\"%s\",%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\",\"%s\"",obj->name, 
		obj->data.attribute.text.style->name, 
		obj->data.attribute.text.insert.x, obj->data.attribute.text.insert.y,
		obj->data.attribute.text.alignment.x, obj->data.attribute.text.alignment.y,
		obj->data.attribute.text.angle, obj->data.attribute.text.oblique,
		obj->data.attribute.text.height, obj->data.attribute.text.width,
		obj->data.attribute.text.justification, obj->data.attribute.text.flags,
		obj->data.attribute.prompt, obj->data.attribute.text.text);
      else
	fprintf((FILE *)fp," OW,\"%s\",NULL,%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,\"%s\",\"%s\"",obj->name,
		obj->data.attribute.text.insert.x, obj->data.attribute.text.insert.y,
		obj->data.attribute.text.alignment.x, obj->data.attribute.text.alignment.y,
		obj->data.attribute.text.angle, obj->data.attribute.text.oblique,
		obj->data.attribute.text.height, obj->data.attribute.text.width,
		obj->data.attribute.text.justification, obj->data.attribute.text.flags,
		obj->data.attribute.prompt, obj->data.attribute.text.text);
      break;
    case TYPE_POINT:
      fprintf((FILE *)fp," OP,\"%s\",%f,%f,%d,%f,%d",obj->name, 
	      obj->data.point.centrum.x, obj->data.point.centrum.y, 
	      obj->data.point.point_type, obj->data.point.point_size,
	      obj->data.point.point_style);
      break;
    case TYPE_POLYLINE:
      fprintf((FILE *)fp," OY,\"%s\",%d,%d,%d",obj->name, 
	      obj->data.polyline.type, obj->data.polyline.closed,
	      g_list_length(obj->data.polyline.node_list));
      fprintf((FILE *)fp,",%s,%s,%s\n", junk1, junk2, junk3);
      list=obj->data.polyline.node_list;

      while(list!=NULL)
	{
	  fprintf((FILE *)fp, "    N,%d,%f,%f,%f,%f,%f\n",
		  ((NODE *)list->data)->type, ((NODE *)list->data)->x,((NODE *)list->data)->y,
		  ((NODE *)list->data)->sw,((NODE *)list->data)->ew,
		  ((NODE *)list->data)->weight),
	  list=g_list_next(list);
	}
      break;
    case TYPE_SPLINE:
      fprintf((FILE *)fp," OS,\"%s\",%d,%d",obj->name, 
	      obj->data.spline.closed,
	      g_list_length(obj->data.spline.node_list));
      fprintf((FILE *)fp,",%s,%s,%s\n", junk1, junk2, junk3);
      list=obj->data.spline.node_list;

      while(list!=NULL)
	{
	  fprintf((FILE *)fp, "    N,%d,%f,%f,%f,%f,%f\n",
		  ((NODE *)list->data)->type, ((NODE *)list->data)->x,((NODE *)list->data)->y,
		  ((NODE *)list->data)->sw,((NODE *)list->data)->ew,
		  ((NODE *)list->data)->weight),
	  list=g_list_next(list);
	}
      break;
    case TYPE_BLOCK:
      fprintf((FILE *)fp," OB,\"%s\",%f,%f,%f,%f,%f,\"%s\",%d",obj->name,
	      obj->data.block.insert_point.x,obj->data.block.insert_point.y,
	      obj->data.block.rotation, obj->data.block.scale_x, 
	      obj->data.block.scale_y, obj->data.block.block->name, g_list_length(obj->data.block.attr) );
      fprintf((FILE *)fp,",%s,%s,%s\n", junk1, junk2, junk3);

      list=g_list_first(obj->data.block.attr);

      /* Write attributes belonging to this object */
      while(list!=NULL)
	{
	  if(list->data==NULL)
	    fprintf((FILE *)fp,"    AT,NULL\n");
	  else
	    fprintf((FILE *)fp,"    AT,\"%s\"\n", (gchar *)list->data);
	  list=g_list_next(list);
	}
      break;
    case TYPE_DIMENSION:
      switch(obj->data.dimension.type)
	{
	case DIM_HORIZ:	  
	  fprintf((FILE *)fp," DH,\"%s\",\"Standard\",%d,%d,%s\n", obj->name, obj->data.dimension.type_data,
	      g_list_length(obj->data.dimension.data),junk1);
	  
	  list=obj->data.dimension.data;

	  while(list!=NULL)
	    {
	      if(((DIMENSION_POINT *)list->data)->text!=NULL)
		fprintf((FILE *)fp, "    N,%f,%f,%f,%f,\"%s\"\n",
			((DIMENSION_POINT *)list->data)->insert_point.x,((DIMENSION_POINT *)list->data)->insert_point.y,
			((DIMENSION_POINT *)list->data)->text_point.x,((DIMENSION_POINT *)list->data)->text_point.y,
			((DIMENSION_POINT *)list->data)->text);
	      else
		fprintf((FILE *)fp, "    N,%f,%f,%f,%f\n",
			((DIMENSION_POINT *)list->data)->insert_point.x,((DIMENSION_POINT *)list->data)->insert_point.y,
			((DIMENSION_POINT *)list->data)->text_point.x,((DIMENSION_POINT *)list->data)->text_point.y);
	      
	      list=g_list_next(list);
	    }
	  break;
	case DIM_VERT:	  
	  fprintf((FILE *)fp," DV,\"%s\",\"Standard\",%d,%d,%s\n", obj->name, obj->data.dimension.type_data,
	      g_list_length(obj->data.dimension.data),junk1);
	  
	  list=obj->data.dimension.data;

	  while(list!=NULL)
	    {
	      if(((DIMENSION_POINT *)list->data)->text!=NULL)
		fprintf((FILE *)fp, "    N,%f,%f,%f,%f,\"%s\"\n",
			((DIMENSION_POINT *)list->data)->insert_point.x,((DIMENSION_POINT *)list->data)->insert_point.y,
			((DIMENSION_POINT *)list->data)->text_point.x,((DIMENSION_POINT *)list->data)->text_point.y,
			((DIMENSION_POINT *)list->data)->text);
	      else
		fprintf((FILE *)fp, "    N,%f,%f,%f,%f\n",
			((DIMENSION_POINT *)list->data)->insert_point.x,((DIMENSION_POINT *)list->data)->insert_point.y,
			((DIMENSION_POINT *)list->data)->text_point.x,((DIMENSION_POINT *)list->data)->text_point.y);
	      
	      list=g_list_next(list);
	    }
	  break;
	case DIM_ALIGNED:
	  fprintf((FILE *)fp," DA,\"%s\",\"Standard\",%d,%d,%s\n", obj->name, obj->data.dimension.type_data,
	      g_list_length(obj->data.dimension.data),junk1);
	  
	  list=obj->data.dimension.data;

	  while(list!=NULL)
	    {
	      if(((DIMENSION_POINT *)list->data)->text!=NULL)
		fprintf((FILE *)fp, "    N,%f,%f,%f,%f,\"%s\"\n",
			((DIMENSION_POINT *)list->data)->insert_point.x,((DIMENSION_POINT *)list->data)->insert_point.y,
			((DIMENSION_POINT *)list->data)->text_point.x,((DIMENSION_POINT *)list->data)->text_point.y,
			((DIMENSION_POINT *)list->data)->text);
	      else
		fprintf((FILE *)fp, "    N,%f,%f,%f,%f\n",
			((DIMENSION_POINT *)list->data)->insert_point.x,((DIMENSION_POINT *)list->data)->insert_point.y,
			((DIMENSION_POINT *)list->data)->text_point.x,((DIMENSION_POINT *)list->data)->text_point.y);
	      
	      list=g_list_next(list);
	    }
	  break;
	case DIM_RADIUS:
	  fprintf((FILE *)fp," DR,\"%s\",\"Standard\",%f,%f,%f,%f,%f,%f,%f,\"%s\",%s\n",obj->name, 
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->centrum_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->centrum_point.y,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius_point.y,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->insert_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->insert_point.y,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->text,
		  junk1);
	  break;
	case DIM_DIAMETER:
	  fprintf((FILE *)fp," DD,\"%s\",\"Standard\",%f,%f,%f,%f,%f,%f,%f,%f,%f,\"%s\",%s\n",obj->name, 
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->centrum_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->centrum_point.y,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius_point.y,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius2_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius2_point.y,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->insert_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->insert_point.y,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->text,
		  junk1);
	  break;
	case DIM_CENTER:
	  fprintf((FILE *)fp," DC,\"%s\",\"%s\",%f,%f,%f,%s\n",obj->name, obj->data.dimension.style->name,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->radius,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->centrum_point.x,
		  ((DIMENSION_RADIUS *)obj->data.dimension.data->data)->centrum_point.y,
		  junk1);
	  break;
	case DIM_ANGULAR:
	  if (((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->text!=NULL)
	    {
	      fprintf((FILE *)fp," DX,\"%s\",\"Standard\",%f,%f,%f,%f,%d,%f,%f,%d,%f,\"%s\",%s\n",obj->name, 
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->centrum_point.x,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->centrum_point.y,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->first_extension_angle,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->first_extension_radius,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->first_extension_visible,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->second_extension_angle,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->second_extension_radius,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->second_extension_visible,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->text_radius,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->text,
		      junk1);
	    }
	  else
	    {
	      fprintf((FILE *)fp," DX,\"%s\",\"Standard\",%f,%f,%f,%f,%d,%f,%f,%d,%f,NULL,%s\n",obj->name, 
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->centrum_point.x,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->centrum_point.y,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->first_extension_angle,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->first_extension_radius,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->first_extension_visible,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->second_extension_angle,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->second_extension_radius,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->second_extension_visible,
		      ((DIMENSION_ANGULAR *)obj->data.dimension.data->data)->text_radius,		      
		      junk1);
	    }

	  break;
	case DIM_LEADER:
	  fprintf((FILE *)fp," DL,\"%s\",\"%s\",%d,%s\n", obj->name,obj->data.dimension.style->name,
		  g_list_length(obj->data.dimension.data),junk1);
	  
	  list=obj->data.dimension.data;

	  while(list!=NULL)
	    {
	      if(((DIMENSION_RADIUS *)list->data)->text!=NULL)
		fprintf((FILE *)fp, "    N,%f,%f,\"%s\"\n",
			((DIMENSION_RADIUS *)list->data)->insert_point.x,((DIMENSION_RADIUS *)list->data)->insert_point.y,
			((DIMENSION_RADIUS *)list->data)->text);
	      else
		fprintf((FILE *)fp, "    N,%f,%f\n",
			((DIMENSION_RADIUS *)list->data)->insert_point.x,((DIMENSION_RADIUS *)list->data)->insert_point.y);
	      
	      list=g_list_next(list);
	    }
	  break;
	}
    }

  if(obj->type!=TYPE_SPLINE && obj->type!=TYPE_POLYLINE && obj->type!=TYPE_DIMENSION && obj->type!=TYPE_BLOCK)
    fprintf((FILE *)fp,",%s,%s,%s\n", junk1, junk2, junk3);

}

void write_all_objects(gpointer data, gpointer fp)
{
  LAYER *layer;
  
  layer=(LAYER *)data;
  
  fprintf((FILE *)fp,"L,\"%s\"\n", layer->name);

  /* Write objects */
  g_list_foreach(layer->object_list,write_object,(gpointer)fp);
}

void write_layers(gpointer data, gpointer fp)
{
  LAYER *layer;
  
  layer=(LAYER *)data;
  
  fprintf((FILE *)fp,"L,\"%s\",%f,%f,%f,%f,%d,%d,%d\n", layer->name, layer->colour->c[0],layer->colour->c[1],layer->colour->c[2],
	  layer->linewidth,g_list_index(layer->doc->linestyle_list, layer->linestyle),layer->visible,layer->editable);
}

void write_linestyle(gpointer data, gpointer fp)
{
  LINESTYLE *style;
  gint i;

  style=(LINESTYLE *)data;

  i=g_list_index(style->doc->linestyle_list, data);

  if(style->type==LINE_SOLID)
    fprintf((FILE *)fp, "S,%d,\"%s\",%f\n",i, style->name, style->dash_length);
  else
    {
      fprintf((FILE *)fp, "S,%d,\"%s\",%f,%d", i,style->name, style->dash_length, style->dash_no);
      for(i=0;i<style->dash_no;i++)
	{
	  fprintf((FILE *)fp, ",%f", style->dash_dlist[i]);
	}
      fprintf((FILE *)fp, "\n");
    }
}


void write_predefined_colour(FILE *fp, COLOUR *colour)
{
  fprintf((FILE *)fp,"C,%d,%f,%f,%f\n",
	  colour->predefined_number, colour->c[0],colour->c[1], colour->c[2]);
}

void write_textstyle(gpointer data, gpointer fp)
{
  TEXTSTYLE *style;

  style=(TEXTSTYLE *)data;

  fprintf((FILE *)fp,"T,%d,\"%s\",%d,%f,%f,%f\n",
	  style->type, style->name, style->flags, style->height, style->oblique,style->width);
}

void write_block(gpointer data, gpointer fp)
{
  file_block_save((FILE *)fp, (BLOCK *)data);
}

gint file_save(DOCUMENT *doc, gchar *filename)
{
  FILE *fp;
  gint i;
  struct stat buf;
  gint err;
  gint t1,t2,t3,t4,t5;
  gchar *tmp;

  err = stat (filename, &buf);
  if (err == 0)
    {      
      if(alert(ICON_QUESTION, _("Overwrite file?"), _("Yes"), _("No"),NULL, _("The file %s\ndoes already exist.\nDo you want to overwrite it?"),filename)!=1)
	return FALSE;
    }

  fp=fopen(filename,"w"); 
  
  if(fp==NULL)
    {
      message(ICON_WARNING, _("File %s cannot be saved.\nPlease make sure that you have\nthe permission to write in the specific path."), filename);	
      return FALSE;
    }
  
  /* Save document */
  
  /* Write Header */
  fprintf(fp,"H,\"oCADis 1.0\"\n");
  
  /* Write Title */
  
  /* Write Viewport */
  fprintf(fp,"V,%f,%f,%f\n", doc->xpan, doc->ypan, doc->zoom);

  /* Write Window Position and size */
  gdk_window_get_geometry(doc->widgets.window->window, &t1,&t2,&t3,&t4,&t5);
  fprintf(fp,"W,%d,%d,%d,%d\n", t1,t2,t3,t4);

  /* Write grid, ortho and isoview data */
  fprintf(fp,"G,%d,%d,%d,%d,%d,%d\n", doc->gl_ortho, doc->gl_grid, doc->gl_grid_x, 
	  doc->gl_grid_y, doc->gl_iso_view, doc->gl_iso_type);

  /* Write snap data */
  fprintf(fp,"N,%d,%f,%f,%f,%f,%f\n", doc->gl_snap, doc->gl_snap_basex, doc->gl_snap_basey, 
	  doc->gl_snap_width, doc->gl_snap_height, doc->gl_snap_angle);

  /* Write predefined colours */
  for(i=0;i<MAX_NO_PREDEFINED_COLOURS;i++)
    write_predefined_colour(fp, &doc->predefined_colours[i]);

  /* Write linestyle list */
  g_list_foreach(doc->linestyle_list,write_linestyle,(gpointer)fp);

  /* Write textstyle list */
  g_list_foreach(doc->textstyle_list,write_textstyle,(gpointer)fp);

  /* Write dimensionstyle list */
  fprintf((FILE *)fp,"Q,%d,\"Standard\",%f,%f,%f,%f,%i,%f,%i,%f,%i,%f\n",0,
	  doc->current_dimensionstyle->dimexe,
	  doc->current_dimensionstyle->dimexo,
	  doc->current_dimensionstyle->dimgap,
	  doc->current_dimensionstyle->dimtxt,
	  doc->current_dimensionstyle->dimdec,
	  doc->current_dimensionstyle->dimdli,
	  doc->current_dimensionstyle->dimatp,
	  doc->current_dimensionstyle->dimasz,
	  doc->current_dimensionstyle->dimmtp,
	  doc->current_dimensionstyle->dimmsz);

  /* Write Layers */
  g_list_foreach(doc->layer_list,write_layers,(gpointer)fp);

  /* Write blocks */
  g_list_foreach(doc->block_list,write_block,(gpointer)fp);

  /* Write Objects */
  g_list_foreach(doc->layer_list,write_all_objects,(gpointer)fp);

  fprintf(fp,"EOF");
  
  fclose(fp);   
  
  doc->saved=TRUE;

  tmp=doc->filename;
  doc->filename=g_strdup(filename);
  g_free(tmp);

  make_window_title(doc);

  return TRUE;
}








