/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "misc.h"
#include "draw.h"
#include "calc_arc.h"
#include "select.h"
#include "dia_pagesetup.h"
#include "obj_polyline.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_text.h"
#include "obj_attribute.h"
#include "layer.h"
#include "linestyle.h"
#include "dimension.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
#define WEIGHT_RED 0.299
#define WEIGHT_GREEN 0.587
#define WEIGHT_BLUE 0.114

#define MM2UNIT 2.83464

#define WLDtoPLT_x(x) ((gdouble) MM2UNIT*( (x-plot_xcen) * plot_zoom + plot_xpan) )
#define WLDtoPLT_y(y) ((gdouble) MM2UNIT*( (plot_ycen-y) * plot_zoom + plot_ypan) )
#define WLDtoPLT(r)   ((gdouble)r*plot_zoom*MM2UNIT)

static gdouble plot_zoom, plot_xpan, plot_ypan;
static gdouble plot_xcen, plot_ycen;
static gdouble bound_area[4];



static FILE *ps_file;

/* --  F U N C T I O N S  -- */

static void ps_draw_line(DOCUMENT *doc, GdkGC *gc, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  fprintf(ps_file, "  newpath %f %f moveto %f %f lineto stroke\n",
	  WLDtoPLT_x(x1), WLDtoPLT_y(y1),
	  WLDtoPLT_x(x2), WLDtoPLT_y(y2));
}

static void ps_draw_arc(DOCUMENT *doc, GdkGC *gc, gdouble xcen, gdouble ycen, gdouble radius, 
			gdouble start_angle, gdouble end_angle)
{
  fprintf(ps_file, "  newpath %f %f %f %f %f arc stroke\n",
	  WLDtoPLT_x(xcen), WLDtoPLT_y(ycen),
	  WLDtoPLT(radius), start_angle, end_angle);
}

static void ps_draw_arrow(DOCUMENT *doc,GdkGC *gc, gdouble x, gdouble y, gdouble angle,gdouble size)
{
  gdouble tx,ty;
  gdouble a[3][2];
  
  tx=x+size*cos(angle);
  ty=y+size*sin(angle);
        
  a[0][0]=x;
  a[0][1]=y;
  a[1][0]=tx+0.4*size*cos(angle+M_PI/2);
  a[1][1]=ty+0.4*size*sin(angle+M_PI/2);
  a[2][0]=tx+0.4*size*cos(angle-M_PI/2);
  a[2][1]=ty+0.4*size*sin(angle-M_PI/2);

  switch(doc->current_dimensionstyle->dimatp)
    {
    case 0:  /* Closed filled */
      fprintf(ps_file, "  newpath %f %f moveto %f %f lineto %f %f lineto fill stroke\n",
	      WLDtoPLT_x(a[0][0]), WLDtoPLT_y(a[0][1]), WLDtoPLT_x(a[1][0]), 
	      WLDtoPLT_y(a[1][1]), WLDtoPLT_x(a[2][0]), WLDtoPLT_y(a[2][1]));
      break;
    case 1:  /* Closed */
      fprintf(ps_file, "  newpath %f %f moveto %f %f lineto %f %f lineto closepath stroke\n",
	      WLDtoPLT_x(a[0][0]), WLDtoPLT_y(a[0][1]), WLDtoPLT_x(a[1][0]), 
	      WLDtoPLT_y(a[1][1]), WLDtoPLT_x(a[2][0]), WLDtoPLT_y(a[2][1]));
      break;
    case 2:  /* Open */
      ps_draw_line(doc, gc, WLDtoPLT_x(a[0][0]), WLDtoPLT_y(a[0][1]), 
		   WLDtoPLT_x(a[1][0]), WLDtoPLT_y(a[1][1]));
      ps_draw_line(doc, gc, WLDtoPLT_x(a[0][0]), WLDtoPLT_y(a[0][1]), 
		   WLDtoPLT_x(a[2][0]), WLDtoPLT_y(a[2][1]));
      break;
    case 3:  /* Line */
      a[0][0]=x+0.5*size*cos(angle+M_PI/4);
      a[0][1]=y-0.5*size*sin(angle+M_PI/4);
      a[1][0]=x-0.5*size*cos(angle+M_PI/4);
      a[1][1]=y+0.5*size*sin(angle+M_PI/4);

      ps_draw_line(doc, gc, WLDtoPLT_x(a[0][0]), WLDtoPLT_y(a[0][1]), 
		   WLDtoPLT_x(a[1][0]), WLDtoPLT_y(a[1][1]));
      break;
    }
}

static void ps_draw_text(DOCUMENT *doc, GdkGC *gc, gint just, gdouble x, gdouble y, 
			 gdouble height, gdouble width, gdouble rot, gchar *text)
{
  gint type;
  gdouble tx,ty;

  draw_text_set_text(just,x,y,height,width,rot,text);

  fprintf(ps_file, "  newpath   %% Text\n");
  while(draw_text_get_vertex(&type, &tx,&ty))
	{
	  if(type==TEXT_VERTEX_MOVETO)
	    fprintf(ps_file, "          %f %f moveto\n", WLDtoPLT_x(tx), WLDtoPLT_y(ty));
	  else
	    fprintf(ps_file, "          %f %f lineto\n", WLDtoPLT_x(tx), WLDtoPLT_y(ty));
	}
  fprintf(ps_file, "  stroke   %% End text\n");
}

static GRDRAW ps_grdraw={ps_draw_line, ps_draw_arc, ps_draw_arrow, ps_draw_text};

static gdouble getintensity(COLOUR *c)
{
  return( WEIGHT_RED*(c->c[0])+ WEIGHT_GREEN*(c->c[1])+ WEIGHT_BLUE*(c->c[2]) );
}

static void generate_header(DOCUMENT *doc)
{
  fprintf(ps_file, "%%!PS-Adobe-2.0 EPSF-2.0\n");

  if(doc->filename!=NULL)
    fprintf(ps_file,"%%%%Title: %s\n", doc->filename);
  else
    fprintf(ps_file,"%%%%Title: Untitled\n");

  fprintf(ps_file,"%%%%Creator: oCADis\n");
 
  fprintf(ps_file,"%%%%Pages: 1\n");

  fprintf(ps_file,"%%%%EndComments\n");

  if(pagesetup.landscape)
    {
      fprintf(ps_file,"%f %f translate\n", pagesetup.height/2*MM2UNIT, pagesetup.width/2*MM2UNIT);
      fprintf(ps_file,"90 rotate\n");
    }
  else
    fprintf(ps_file,"%f %f translate\n", pagesetup.width/2*MM2UNIT, pagesetup.height/2*MM2UNIT);

}

static void set_layer(LAYER *layer)
{
  fprintf(ps_file,"gsave      %% Start Layer: %s\n",layer->name);

  fprintf(ps_file,"  %f setgray\n", getintensity(layer->colour));
  fprintf(ps_file,"  %f setlinewidth\n", WLDtoPLT(layer->linewidth));

  if(layer->linestyle->type==LINE_SOLID)
    fprintf(ps_file,"  [] 0 setdash\n");
  else
    {
      gint i;
      
      fprintf(ps_file,"  [ ");
      for(i=0;i<layer->linestyle->dash_no;i++)
	fprintf(ps_file,"%f ",MAX(1,WLDtoPLT(layer->linestyle->dash_dlist[i]*layer->linestyle->dash_length)));

      fprintf(ps_file,"] 0 setdash\n");
    }
}

static gint set_object(OBJECT *object)
{
  gint ret=FALSE;

  if(object->type_colour!=BYLAYER)
    {
      if(!ret)
	fprintf(ps_file,"  gsave\n");

      fprintf(ps_file,"    %f setgray\n", getintensity(object->colour));
      ret=TRUE;
    }

  if(object->type!=TYPE_DIMENSION && object->type_linewidth!=BYLAYER)
    {
      if(!ret)
	fprintf(ps_file,"gsave\n");

      fprintf(ps_file,"    %f setlinewidth\n", WLDtoPLT(object->linewidth));
      ret=TRUE;
    }

  if(object->type!=TYPE_DIMENSION && object->type_linestyle!=BYLAYER)
    {
      if(!ret)
	fprintf(ps_file,"  gsave\n");

      if(object->linestyle->type==LINE_SOLID)
	fprintf(ps_file,"    [] 0 setdash\n");
      else
	{
	  gint i;
	  
	  fprintf(ps_file,"    [ ");
	  for(i=0;i<object->linestyle->dash_no;i++)
	    fprintf(ps_file,"%f ",MAX(1,WLDtoPLT(object->linestyle->dash_dlist[i]*object->linestyle->dash_length)));
	  
	  fprintf(ps_file,"] 0 setdash\n");
	}
      ret=TRUE;
    }

  return ret;
}

static void plot_line(gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  fprintf(ps_file, "  newpath %f %f moveto %f %f lineto stroke\n",
	  WLDtoPLT_x(x1), WLDtoPLT_y(y1),
	  WLDtoPLT_x(x2), WLDtoPLT_y(y2));
}

static void plot_arc(OBJECT_ARC *arc)
{
  fprintf(ps_file, "  newpath %f %f %f %f %f arc stroke\n",
	  WLDtoPLT_x(arc->centrum.x), WLDtoPLT_y(arc->centrum.y),
	  WLDtoPLT(arc->radius),
	  arc->start_angle, arc->end_angle);
}

static void plot_polyline(OBJECT *pline)
{
  gdouble sx,sy,Ax, Ay, Bx, By, Cx, Cy;
  GList *node_list;
  OBJECT_ARC arc;

  node_list=pline->data.polyline.node_list;

  sx=Ax=((NODE *)node_list->data)->x;
  sy=Ay=((NODE *)node_list->data)->y;

  node_list=g_list_next(node_list);

  while(node_list!=NULL)
    {
      Bx=((NODE *)node_list->data)->x;
      By=((NODE *)node_list->data)->y;

      if(((NODE *)node_list->data)->type==NODE_LINE)
	{
	  plot_line(Ax, Ay, Bx, By);

	  Ax=Bx;
	  Ay=By;
	}
      else
	{
	  if(node_list->next!=NULL)
	    {
	      Cx=((NODE *)node_list->next->data)->x;
	      Cy=((NODE *)node_list->next->data)->y;

	      if(calc_arc_3p(Ax,Ay,Bx,By,Cx,Cy,&arc))
		fprintf(ps_file, "  newpath %f %f %f %f %f arc stroke\n",
			WLDtoPLT_x(arc.centrum.x), WLDtoPLT_y(arc.centrum.y),
			WLDtoPLT(arc.radius), arc.start_angle, arc.end_angle);
	      
	      Ax=Cx;
	      Ay=Cy;
	    
	      node_list=g_list_next(node_list);
	    }	  
	}
      node_list=g_list_next(node_list);
    }

  switch(pline->data.polyline.closed)
    {
    case POLYLINE_CLOSE_LINE:
      plot_line(Ax, Ay, sx, sy);
      break;
    case POLYLINE_CLOSE_ARC:
      calc_arc_2p( calc_tangent(pline->data.polyline.node_list, NULL), 
		   Ax, Ay, sx,sy, NULL, NULL, &arc);
      arc.radius/=2;
      
      fprintf(ps_file, "  newpath %f %f %f %f %f arc stroke\n",
	      WLDtoPLT_x(arc.centrum.x), WLDtoPLT_y(arc.centrum.y),
	      WLDtoPLT(arc.radius), arc.start_angle, arc.end_angle);
      break;
    default:
    }

}

static void plot_spline(OBJECT *spline)
{
  POINT *tlist;
  int num,i;

  num=spline->data.spline.num;

  tlist=spline->data.spline.cache_point;
  
  i=0;
  
  while(i<num)
    {
      if(i==0)
	fprintf(ps_file,"  newpath %f %f moveto\n", WLDtoPLT_x(tlist[i].x), WLDtoPLT_y(tlist[i].y));
      else
	fprintf(ps_file,"  %f %f lineto\n", WLDtoPLT_x(tlist[i].x), WLDtoPLT_y(tlist[i].y));
      i++;
    }
  
  fprintf(ps_file,"  stroke\n");
}

static void plot_ellipse(OBJECT *ellipse)
{
  fprintf(ps_file,"  gsave\n");
  fprintf(ps_file,"  %f %f translate\n", WLDtoPLT_x(ellipse->data.ellipse.centrum.x), 
	  WLDtoPLT_y(ellipse->data.ellipse.centrum.y));
  fprintf(ps_file,"  %f rotate\n", ellipse->data.ellipse.angle); 
  fprintf(ps_file,"  %f %f scale\n", WLDtoPLT(ellipse->data.ellipse.major), 
	  WLDtoPLT(ellipse->data.ellipse.minor));
  fprintf(ps_file, "  newpath %f %f %f 0 360 arc stroke\n  grestore\n",
	  0.0,0.0,
	  1.0);
}

static void plot_elarc(OBJECT *elarc)
{
  fprintf(ps_file,"  gsave\n");
  fprintf(ps_file,"  %f %f translate\n", WLDtoPLT_x(elarc->data.elarc.centrum.x), 
	  WLDtoPLT_y(elarc->data.elarc.centrum.y));
  fprintf(ps_file,"  %f rotate\n", elarc->data.elarc.angle); 
  fprintf(ps_file,"  %f %f scale\n", WLDtoPLT(elarc->data.elarc.major), 
	  WLDtoPLT(elarc->data.elarc.minor));
 fprintf(ps_file, "  newpath %f %f %f %f %f arc stroke\n  grestore\n",
	 0.0, 0.0, 1.0, elarc->data.elarc.start_angle, elarc->data.elarc.end_angle);
}

static void plot_text(OBJECT *text)
{
  gint type;
  gdouble tx,ty;

  if(text->data.text.justification==TEXT_JUST_AL)
    {
      gdouble h,a;
      
      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      h=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text));
      
      draw_text_set_text(text->data.text.justification,text->data.text.insert.x,text->data.text.insert.y,
			 h, 1, a, text->data.text.text);
    }
  else if(text->data.text.justification==TEXT_JUST_FI)
    {
      gdouble w,a;

      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      w=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text)*text->data.text.height);

      draw_text_set_text(text->data.text.justification,text->data.text.insert.x,text->data.text.insert.y,
			 text->data.text.height, w, a, text->data.text.text);
    }
  else
    draw_text_set_text(text->data.text.justification, WLDtoPLT_x(text->data.text.insert.x), WLDtoPLT_y(text->data.text.insert.y),
		       WLDtoPLT(text->data.text.height),text->data.text.width, text->data.text.angle*M_PI/180.0,text->data.text.text);

  fprintf(ps_file, "  newpath   %% Text\n");
  while(draw_text_get_vertex(&type, &tx,&ty))
	{
	  if(type==TEXT_VERTEX_MOVETO)
	    fprintf(ps_file, "          %f %f moveto\n", tx, ty);
	  else
	    fprintf(ps_file, "          %f %f lineto\n", tx, ty);
	}
  fprintf(ps_file, "  stroke   %% End text\n");
}

static void plot_block(DOCUMENT *doc, OBJECT *insert)
{
  gboolean change;
  TRANSFORM insert_transform;
  LINESTYLE *linestyle;
  LAYER *layer;
  OBJECT object;
  BLOCK_OBJECT *block_object;
  GList *list;
  GList *attr_list;

  /* Make tranform matrix */
  insert_transform.type=OBJ_TRANSLATE;
  insert_transform.trans_x=insert->data.block.insert_point.x;
  insert_transform.trans_y=insert->data.block.insert_point.y;
  insert_transform.cen_x=0;
  insert_transform.cen_y=0;
  
  /* Check if need to scale block */
  if(ABS(insert->data.block.scale_x-1)>GFLOAT_SENSITIVE ||
     ABS(insert->data.block.scale_y-1)>GFLOAT_SENSITIVE)
    {
      insert_transform.type|=OBJ_SCALE;
      insert_transform.scale_x=insert->data.block.scale_x;
      insert_transform.scale_y=insert->data.block.scale_y;
    }

  /* Check if need to rotate block */
  if(ABS(insert->data.block.rotation)>GFLOAT_SENSITIVE)
    {
      insert_transform.type|=OBJ_ROTATE;
      insert_transform.rot_angle=-insert->data.block.rotation*M_PI/180.0;
    }

  /* Get list of attributes in this block */
  attr_list=g_list_first(insert->data.block.attr);

  /* Get first object in block */
  list=g_list_first(insert->data.block.block->object_list);

  while(list!=NULL)
    {
      block_object=(BLOCK_OBJECT *)list->data;

      /* Get layer for this object */

      /* Check if we should use the layer of the block */
      if(!strcmp(block_object->name_of_layer,"0"))
	layer=insert->layer;
      else
	{
	  layer=layer_find(doc, block_object->name_of_layer);
	  if(layer==NULL)
	    {
	      g_warning( _("Layer %s not found. Creating it."), block_object->name_of_layer );
	      layer=layer_new(doc, -1, doc->current_linestyle, 0.0 , 0.0, 0.0, 0.0,  
			  block_object->name_of_layer, TRUE, TRUE);
	    }
	}

      /* Draw only if layer is visible */      
      if(layer->visible)
	{
	  /* Make nice object to draw */
	  object.type=block_object->type;
	  object.layer=layer;
	  
	  /* Make colour */
	  object.type_colour=block_object->type_colour;
	  switch(object.type_colour)
	    {
	    case BYLAYER:	
	      object.colour=layer->colour;
	      break;
	    case BYBLOCK:
	      object.colour=insert->colour;
	      break;
	    case PREDEFINED:
	    case USERDEFINED:
	      object.colour=block_object->colour;
	      break;
	    }
	  
	  /* Make linestyle */
	  object.type_linestyle=block_object->type_linestyle;
	  switch(object.type_linestyle)
	    {
	    case BYLAYER:
	      object.linestyle=layer->linestyle;
	      break;
	    case BYBLOCK:
	      object.linestyle=insert->linestyle;
	      break;
	    case USERDEFINED:
	      linestyle=linestyle_find(doc, block_object->name_of_linestyle);
	      
	      if(linestyle==NULL)
		{
		  g_warning( _("Linestyle %s not found. Creating."), block_object->name_of_linestyle);
		  linestyle=linestyle_new(doc, -1, block_object->name_of_linestyle, LINE_SOLID, 1, 0,NULL);
		}
	      
	      object.linestyle=linestyle;
	      break;
	    }
	  
	  /* Make linewidth */
	  object.type_linewidth=block_object->type_linewidth;
	  switch(object.type_linewidth)
	    {
	    case BYLAYER:
	      object.linewidth=layer->linewidth;
	      break;
	    case BYBLOCK:
	      object.linewidth=insert->linewidth;
	      break;
	    case USERDEFINED:
	      object.linewidth=block_object->linewidth;
	      break;
	    }
	  
	  memcpy(&object.data, &block_object->data, sizeof(OBJECT_UNION));
	  
	  make_object_selected(&object, OBJECT_SEL, FALSE);
	  object.selected=insert->selected;

	  change=set_object(&object);
	  
	  switch(object.type)
	    {
	    case TYPE_LINE:
	      obj_transform_line(&object, &insert_transform);
	      fprintf(ps_file, "  newpath %f %f moveto %f %f lineto stroke\n",
		      WLDtoPLT_x(object.data.line.p1.x), WLDtoPLT_y(object.data.line.p1.y),
		      WLDtoPLT_x(object.data.line.p2.x), WLDtoPLT_y(object.data.line.p2.y));
	      break;
	    case TYPE_CIRCLE:
	      obj_transform_circle(&object, &insert_transform);
	      fprintf(ps_file, "  newpath %f %f %f 0 360 arc stroke\n",
		      WLDtoPLT_x(object.data.circle.centrum.x), WLDtoPLT_y(object.data.circle.centrum.y),
		      WLDtoPLT(object.data.circle.radius));
	      break;
	    case TYPE_ARC:
	      obj_transform_arc(&object, &insert_transform);
	      plot_arc(&object.data.arc);
	      break;
	    case TYPE_TEXT:
	      obj_transform_text(&object, &insert_transform);
	      plot_text(&object);
	      break;
	    case TYPE_ATTRIBUTE:
	      /* Fix text of attribute */
	      object.data.attribute.text.text=(gchar *)attr_list->data;
	      attr_list=g_list_next(attr_list);
	      obj_transform_attribute( &object, &insert_transform);
	      plot_text(&object);
	      break;
	    }

	  if(change)
	    fprintf(ps_file,"  grestore\n");
	}

      list=g_list_next(list);
    }
}

static void plot_point(OBJECT *point)
{
  gdouble x,y;
  gdouble size;
  gint style;

  style=point->data.point.point_style;
  x=point->data.point.centrum.x;
  y=point->data.point.centrum.y;
  size=point->data.point.point_size;
  
  if(style & POINT_DOT)
    plot_line(x, y, x, y);

  if(style & POINT_LINE)
    plot_line(x, y, x, y-size);
  
  
  if(style & POINT_CIRCLE)
    fprintf(ps_file, "  newpath %f %f %f 0 360 arc stroke\n",
	    WLDtoPLT_x(x), WLDtoPLT_y(y), WLDtoPLT(size));
  
  if(style & POINT_BOX)
    fprintf(ps_file, "  newpath %f %f moveto %f %f lineto %f %f lineto %f %f lineto closepath stroke\n",
	    WLDtoPLT_x(x-size), WLDtoPLT_y(y-size),
	    WLDtoPLT_x(x+size), WLDtoPLT_y(y-size),
	    WLDtoPLT_x(x+size), WLDtoPLT_y(y+size),
	    WLDtoPLT_x(x-size), WLDtoPLT_y(y+size));
  
  if(style & POINT_MARK)
    {
      plot_line(x-size, y, x+size, y);
      plot_line(x, y+size, x, y+size);
    }
  
  if(style & POINT_XMARK)
    {
      plot_line(x-size, y-size, x+size, y+size);
      plot_line(x+size, y-size, x-size, y+size);
    }
}

static void generate_entities(DOCUMENT *doc)
{
  gint change;
  GList *layer_list;
  GList *object_list;
  LAYER *layer;
  OBJECT *object;
  layer_list=g_list_first(doc->layer_list);

  while(layer_list!=NULL)
    {
      layer=(LAYER *)layer_list->data;

      object_list=g_list_first(layer->object_list);

      /* Set default colour and linewidth for current layer */
      set_layer(layer);

      while(object_list!=NULL)
	{
	  object=((OBJECT *)object_list->data);

	  change=set_object(object);

	  switch(object->type)
	    {
	    case TYPE_LINE:	    
	      fprintf(ps_file, "  newpath %f %f moveto %f %f lineto stroke\n",
		      WLDtoPLT_x(object->data.line.p1.x), WLDtoPLT_y(object->data.line.p1.y),
		      WLDtoPLT_x(object->data.line.p2.x), WLDtoPLT_y(object->data.line.p2.y));
	      break;
	    case TYPE_CIRCLE:
	      fprintf(ps_file, "  newpath %f %f %f 0 360 arc stroke\n",
		      WLDtoPLT_x(object->data.circle.centrum.x), WLDtoPLT_y(object->data.circle.centrum.y),
		      WLDtoPLT(object->data.circle.radius));
	      break;
	    case TYPE_ARC:
	      plot_arc(&object->data.arc);
	      break;
	    case TYPE_POLYLINE:
	      plot_polyline(object);
	      break;
	    case TYPE_SPLINE:
	      plot_spline(object);
	      break;
	    case TYPE_POINT:
	      plot_point(object);
	      break;
	    case TYPE_TEXT:
	      plot_text(object);
	      break;
	    case TYPE_ELLIPSE:
	      plot_ellipse(object);
	      break;
	    case TYPE_ELARC:
	      plot_elarc(object);
	      break;
	    case TYPE_BLOCK:
	      plot_block(doc, object);
	      break;
	    case TYPE_DIMENSION:
	      dim_draw_dimension(doc, &ps_grdraw, object);
	      break;
	    }

	  if(change)
	    fprintf(ps_file,"  grestore\n");

	  object_list=g_list_next(object_list);
	}

      /* Restore graphic settings */
      fprintf(ps_file,"grestore   %% End Layer: %s\n", layer->name);
      layer_list=g_list_next(layer_list);
    }
}

static void generate_eof(void)
{
  fprintf(ps_file,"showpage\n%%%%EOF\n");
}

gint file_export_ps(DOCUMENT *doc, FILE *my_file)
{
  misc_get_size_of_drawing(doc, &bound_area[0], &bound_area[1], &bound_area[2], &bound_area[3]);

  plot_xpan=pagesetup.width/2;
  plot_ypan=pagesetup.height/2;

  plot_xpan=0;
  plot_ypan=0;

  plot_xcen=fabs(bound_area[0]-bound_area[2])/2+bound_area[0];
  plot_ycen=fabs(bound_area[1]-bound_area[3])/2+bound_area[1];

  if(pagesetup.fittopage)
    {
      gdouble width,height;
      gdouble draw_width, draw_height;

      draw_width=ABS(bound_area[0]-bound_area[2]);
      draw_height=ABS(bound_area[1]-bound_area[3]);

      width=pagesetup.width-pagesetup.margin_left-pagesetup.margin_right;
      height=pagesetup.height-pagesetup.margin_top-pagesetup.margin_bottom;
      
      plot_zoom=MIN(width/draw_width, height/draw_height);
    }
  else
    plot_zoom=pagesetup.scale_mm/pagesetup.scale_unit;

  ps_file=my_file;

  generate_header(doc);
  generate_entities(doc);
  generate_eof();

  return TRUE;
}






