/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include "struct.h"
#include "dia_message.h"
#include "parse.h"
#include "new_draw.h"
#include "translation.h"

/* M A N U A L   F O R   S C R I P T S
   ===================================================================

   oCADis is able to handle script-files. These are simple text-files
   where each row is a command, just the kind you could have written
   in the command prompt. You are also able to use the more compact
   way of writing (that is for example line|100,100|200,200|).

   Script-specific commands
   -------------------------------------------------------------------

   Scripts also have some special commands that must only be found in
   a script files and is unvalid everywere else. These are:

     grab new

     This commando makes a new drawing and tells oCADis that all the
     following commands in the script files should be applied in this
     drawing. 

   A line starting with a character # is ignored.

   Other type of special script-commands
   -------------------------------------------------------------------
   These are commands that you can enter directly in the command prompt 
   but the greatest use for these commands are when dealing with scripts.

     dim X as [string, int, float, point]

     This command makes a new variable with the name X and the type
     as a string, int, float or point.

     let X=[number or string surrounded by "]

     This command sets the varible X to a specific value depending
     on the variable-type.

     let X=show_input("[prompt]")

     This command shows a dialog with the text prompt and 
     lets the user input a valid number or string. oCADis will
     store this value in the variable X.

     list var

     This command lists all the defined variables and its
     values. It can be good for debugging.

   
 */ 


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static DOCUMENT *script_doc;

/* --  F U N C T I O N S  -- */
void file_script(gchar *filename)
{
  gchar buffer[1024];
  FILE *fp;

  script_doc=NULL;

  fp=fopen(filename,"r");
  
  if(fp==NULL)
    {
      message(ICON_INFO, _("The file %s\nwas not found. Please make sure that this is\nthe correct path and filename."), filename);
      return;
    }


  while(!feof(fp))
    {
      fgets(buffer, 1024, fp);

      /* Check for script-specific commands */
      if(!strcmp(buffer, _("grab new\n")))
	script_doc=new_draw();
      else if(buffer[0]=='#')
	{
	  /* This is a comment line. Ignore me! */
	}
      else
	parse(script_doc, buffer);
    }


  fclose(fp);

}









