/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "struct/dialog.h"
#include "translation.h"
#include "redraw.h"
#include "menuitem.h"
#include "select.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_attribute.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_rectangle.h"
#include "obj_polygon.h"
#include "obj_spline.h"
#include "obj_insert.h"
#include "dim_horiz.h"
#include "dim_vert.h"
#include "dim_aligned.h"
#include "dim_radius.h"
#include "dim_diameter.h"
#include "dim_center.h"
#include "dim_continue.h"
#include "dim_baseline.h"
#include "dim_leader.h"
#include "dim_angular.h"
#include "tool_zoom.h"
#include "tool_copy.h"
#include "tool_erase.h"
#include "tool_offset.h"
#include "tool_array.h"
#include "tool_move.h"
#include "tool_mirror.h"
#include "tool_rotate.h"
#include "tool_trim.h"
#include "tool_idpoint.h"
#include "tool_dist.h"
#include "tool_list.h"
#include "tool_fillet.h"
#include "tool_break.h"
#include "tool_chamfer.h"
#include "tool_stretch.h"
#include "tool_scale.h"
#include "tool_block.h"
#include "edit_text.h"
#include "edit_prop.h"
#include "osnap.h"
#include "file_ocadis.h"

extern GtkWidget *menubar; 

/* --  L O C A L   V A R I A B L E S  -- */
static GtkItemFactoryEntry menu_items[] =
{
	{ N_("/_File"),                       NULL,           NULL,                  0,           "<Branch>"},
	{ N_("/File/New..."),                 "<control>N",   menu_new,              0,           NULL},
	{ N_("/File/sep1"),                   NULL,           NULL,                  0,           "<Separator>"},
	{ N_("/File/Open Drawing..."),        "<control>O",   menu_load,             0,           NULL},
	{ N_("/File/Open Script..."), NULL,           menu_script,           0,           NULL},
	{ N_("/File/sep2"),                   NULL,           NULL,                  0,           "<Separator>"},
	{ N_("/File/Save"),                   "<control>S",   menu_save,             0,        NULL},
	{ N_("/File/Save As..."),             NULL,           menu_saveas,           0,        NULL},
	{ N_("/File/Close"),                  NULL,           menu_close,            0,           NULL},
	{ N_("/File/sep3"),                   NULL,           NULL,                  0,           "<Separator>"},
	{ N_("/File/Import/DXF"),             NULL,           menu_import_file,      0,           NULL},
	{ N_("/File/Import/Mapgen"),          NULL,           menu_import_file,      1,           NULL},
	{ N_("/File/Export/DXF"),             NULL,           menu_export_file,      0,           NULL},	
	{ N_("/File/Export/Postscript"),      NULL,           menu_export_file,      1,           NULL},
	{ N_("/File/Export/HPGL"),            NULL,           menu_export_file,      2,           NULL},
	{ N_("/File/sep4"),                   NULL,           NULL,                  0,           "<Separator>"},
	{ N_("/File/Page Setup..."),          NULL,           menu_show_dialog,      DIALOG_PAGESETUP,           NULL},
	{ N_("/File/sep5"),                   NULL,           NULL,                  0,           "<Separator>"},
       	{ N_("/File/Preferences..."),         NULL,           menu_show_dialog,      DIALOG_PREFERENCES,        NULL},
	{ N_("/File/sep5"),                   NULL,           NULL,                  0,           "<Separator>"},
	{ N_("/File/Quit"),                   "<control>Q",   menu_quit,             0,           NULL},
    	
     	{ N_("/_View"),                       NULL,           NULL,               0,                "<Branch>"},
	{ N_("/View/Redraw"),                 NULL,           redraw,             0,                NULL},
     	{ N_("/View/<separator>"),            NULL,           NULL,               0,                "<Separator>"},
     	{ N_("/View/Zoom, Window"),           NULL,           tool_set_tool_zoom, 0,                NULL},
     	{ N_("/View/Zoom, Previous"),         NULL,           menu_zoom_previous, 0,                NULL},
     	{ N_("/View/Zoom, Relative/16:1"),    NULL,           tool_zoom_relative, 0,                NULL},
     	{ N_("/View/Zoom, Relative/8:1"),     NULL,           tool_zoom_relative, 1,                NULL},
     	{ N_("/View/Zoom, Relative/4:1"),     NULL,           tool_zoom_relative, 2,                NULL},
     	{ N_("/View/Zoom, Relative/2:1"),     NULL,           tool_zoom_relative, 3,                NULL},
     	{ N_("/View/Zoom, Relative/sep"),     NULL,           NULL,               0,                "<Separator>"},
     	{ N_("/View/Zoom, Relative/1:2"),     NULL,           tool_zoom_relative, 4,                NULL},
     	{ N_("/View/Zoom, Relative/1:4"),     NULL,           tool_zoom_relative, 5,                NULL},
     	{ N_("/View/Zoom, Relative/1:8"),     NULL,           tool_zoom_relative, 6,                NULL},
     	{ N_("/View/Zoom, Relative/1:16"),    NULL,           tool_zoom_relative, 7,                NULL},
     	{ N_("/View/Zoom, All"),              NULL,           menu_zoom_all,      0,                NULL},
     	{ N_("/View/Pan"),                    NULL,           tool_set_tool_zoom, 2,                NULL},
     	{ N_("/View/<separator>"),            NULL,           NULL,               0,                "<Separator>"},
	{ N_("/View/Show Coordinates"),       NULL,           menu_show_dialog,   DIALOG_COORD,     NULL},       
	{ N_("/View/Show Command"),           NULL,           menu_show_dialog,   DIALOG_COMM,      NULL},       
	{ N_("/View/Show Toolbox"),           NULL,           menu_show_dialog,   DIALOG_TOOLBOX,   NULL},       
	{ N_("/View/Show Block Libraries"),   NULL,           menu_show_dialog,   DIALOG_BLOCK,     NULL},       

	{ N_("/_Assist"),                            NULL,           NULL,                         0,                   "<Branch>"},
	{ N_("/Assist/Undo"),                        NULL,           NULL,                         0,                   NULL},
	{ N_("/Assist/Redo"),                        NULL,           NULL,                         0,                   NULL},
     	{ N_("/Assist/sep4"),                        NULL,           NULL,                         0,                   "<Separator>"},
	{ N_("/Assist/Drawing Aids..."),             NULL,           menu_show_dialog,             DIALOG_DRAWAIDS,     NULL},
       	{ N_("/Assist/Format/Layer..."),             NULL,           menu_show_dialog,             DIALOG_LAYER,        NULL},
       	{ N_("/Assist/Format/Colour..."),            NULL,           menu_show_dialog,             DIALOG_COLOUR,       NULL},
       	{ N_("/Assist/Format/Linetype..."),          NULL,           menu_show_dialog,             DIALOG_LINESTYLE,    NULL},
     	{ N_("/Assist/Format/sep1"),                 NULL,           NULL,                         0,                   "<Separator>"},
       	{ N_("/Assist/Format/Text Style..."),        NULL,           menu_show_dialog,             DIALOG_TEXTSTYLE,    NULL},
       	{ N_("/Assist/Format/Dimension Setup..."),   NULL,           menu_show_dialog,             DIALOG_DIMSTYLE,     NULL},
       	{ N_("/Assist/Format/Point Style..."),       NULL,           menu_show_dialog,             DIALOG_POINTSTYLE,   NULL},
     	{ N_("/Assist/Format/sep2"),                 NULL,           NULL,                         0,                   "<Separator>"},
       	{ N_("/Assist/Format/Units..."),             NULL,           NULL,                         0,                   NULL},
       	{ N_("/Assist/Format/Drawing Limits..."),    NULL,           NULL,                         0,                   NULL},
     	{ N_("/Assist/sep4"),                        NULL,           NULL,                         0,                   "<Separator>"},
       	{ N_("/Assist/Layers..."),                   NULL,           menu_show_dialog,             DIALOG_LAYER,        NULL},
       	{ N_("/Assist/Linetype..."),                 NULL,           menu_show_dialog,             DIALOG_LINESTYLE,    NULL},
       	{ N_("/Assist/sep5"),                        NULL,           NULL,                         0,                   "<Separator>"},
       	{ N_("/Assist/Object Snap Settings..."),     NULL,           menu_show_dialog,             DIALOG_OSNAP,        NULL},
       	{ N_("/Assist/Object Snap/Center"),          NULL,           set_snap,             OSNAP_CEN,         NULL},
       	{ N_("/Assist/Object Snap/Endpoint"),        NULL,           set_snap,             OSNAP_END,         NULL},
       	{ N_("/Assist/Object Snap/Insertion"),       NULL,           set_snap,             OSNAP_INS,         NULL},
       	{ N_("/Assist/Object Snap/Intersection"),    NULL,           set_snap,             OSNAP_INT,         NULL},
       	{ N_("/Assist/Object Snap/Midpoint"),        NULL,           set_snap,             OSNAP_MID,         NULL},
       	{ N_("/Assist/Object Snap/Nearest"),         NULL,           set_snap,             OSNAP_NEA,         NULL},
       	{ N_("/Assist/Object Snap/Node"),            NULL,           set_snap,             OSNAP_NOD,         NULL},
       	{ N_("/Assist/Object Snap/Perpendicular"),   NULL,           set_snap,             OSNAP_PER,         NULL},
       	{ N_("/Assist/Object Snap/Quadrant"),        NULL,           set_snap,             OSNAP_QUA,         NULL},
       	{ N_("/Assist/Object Snap/Tangent"),         NULL,           set_snap,             OSNAP_TAN,         NULL},
       	{ N_("/Assist/Object Snap/sep1"),            NULL,           NULL,                         0,         "<Separator>"},
       	{ N_("/Assist/Object Snap/None"),            NULL,           set_snap,            OSNAP_NONE,         NULL},
       	{ N_("/Assist/sep8"),                        NULL,           NULL,                         0,         "<Separator>"},
       	{ N_("/Assist/Selection..."),                NULL,           menu_show_dialog,             DIALOG_SELECT,         NULL},
       	{ N_("/Assist/Select"),                      NULL,           NULL,                         0,         "<Branch>"},
       	{ N_("/Assist/Select/Add"),                  NULL,           select_add_remove_object,  TRUE,         NULL},
       	{ N_("/Assist/Select/Remove"),               NULL,           select_add_remove_object,  FALSE,         NULL},
       	{ N_("/Assist/Select/sep8"),                 NULL,           NULL,                         0,        "<Separator>"},
       	{ N_("/Assist/Select/All"),                  NULL,           select_all,                   0,        NULL},
       	{ N_("/Assist/Select/Invert"),               NULL,           select_invert,                0,        NULL},
       	{ N_("/Assist/Select/Point"),                NULL,           select_point,                 0,        NULL},
       	{ N_("/Assist/Select/Window"),               NULL,           select_window,                0,        NULL},
       	{ N_("/Assist/Select/Crossing"),             NULL,           select_crossing,              0,        NULL},
     	{ N_("/Assist/sep6"),                        NULL,           NULL,                         0,        "<Separator>"},
       	{ N_("/Assist/Inguiry/List"),                NULL,           tool_set_tool_list,           0,        NULL},
       	{ N_("/Assist/Inguiry/sep7"),                NULL,           NULL,                         0,        "<Separator>"},
       	{ N_("/Assist/Inguiry/ID Point"),            NULL,           tool_set_tool_idpoint,        0,        NULL},
       	{ N_("/Assist/Inguiry/Distance"),            NULL,           tool_set_tool_dist,           0,        NULL},
       	{ N_("/Assist/Inguiry/Area"),                NULL,           NULL,                         0,        NULL},

	{ N_("/_Objects"),                            NULL,           NULL,                     0,          "<Branch>"},
	{ N_("/Objects/Line"),                        NULL,           obj_set_tool_line,        0,          NULL},
       	{ N_("/Objects/Polyline"),                    NULL,           obj_set_tool_polyline,    0,          NULL},
       	{ N_("/Objects/Guideline/Ray"),               NULL,           obj_set_tool_rline,       0,          NULL},
       	{ N_("/Objects/Guideline/Construction"),      NULL,           obj_set_tool_cline,       0,          NULL},
       	{ N_("/Objects/<separator>"),                 NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Spline"),                      NULL,           obj_set_tool_spline,      0,          NULL},
       	{ N_("/Objects/Arc/3 Points"),                NULL,           obj_set_tool_arc,         0,          NULL},
       	{ N_("/Objects/Arc/<separator>"),             NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Arc/Start, Center, End"),      NULL,           obj_set_tool_arc,         1,          NULL},
       	{ N_("/Objects/Arc/Start, Center, Angle"),    NULL,           obj_set_tool_arc,         2,          NULL},
       	{ N_("/Objects/Arc/Start, Center, Length"),   NULL,           obj_set_tool_arc,         3,          NULL},
       	{ N_("/Objects/Arc/<separator>"),             NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Arc/Start, End, Angle"),       NULL,           obj_set_tool_arc,         4,          NULL},
       	{ N_("/Objects/Arc/Start, End, Direction"),   NULL,           obj_set_tool_arc,         5,          NULL},
       	{ N_("/Objects/Arc/Start, End, Radius"),      NULL,           obj_set_tool_arc,         6,          NULL},
       	{ N_("/Objects/Arc/<separator>"),             NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Arc/Center, Start, End"),      NULL,           obj_set_tool_arc,         7,          NULL},
       	{ N_("/Objects/Arc/Center, Start, Angle"),    NULL,           obj_set_tool_arc,         8,          NULL},
       	{ N_("/Objects/Arc/Center, Start, Length"),   NULL,           obj_set_tool_arc,         9,          NULL},
       	{ N_("/Objects/Arc/<separator>"),             NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Arc/Continue"),                NULL,           obj_set_tool_arc,        10,         NULL},
       	{ N_("/Objects/Circle/Center, Radius"),       NULL,           obj_set_tool_circle,      0,          NULL},
       	{ N_("/Objects/Circle/Center, Diameter"),     NULL,           obj_set_tool_circle,      2,          NULL},
       	{ N_("/Objects/Circle/<separator>"),          NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Circle/2 Points"),             NULL,           obj_set_tool_circle,      8,          NULL},
       	{ N_("/Objects/Circle/3 Points"),             NULL,           obj_set_tool_circle,      4,          NULL},
       	{ N_("/Objects/Circle/<separator>"),          NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Circle/Tan, Tan, Radius"),     NULL,           obj_set_tool_circle,      13,         NULL},
       	{ N_("/Objects/Circle/Tan, Tan, Tan"),        NULL,           obj_set_tool_circle,      17,         NULL},
       	{ N_("/Objects/Circle/<separator>"),          NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Circle/Donut"),                NULL,           obj_set_tool_circle,     10,         NULL},
       	{ N_("/Objects/Ellipse/Center"),              NULL,           obj_set_tool_ellipse,     3,          NULL},
       	{ N_("/Objects/Ellipse/Axis, End"),           NULL,           obj_set_tool_ellipse,     0,          NULL},
       	{ N_("/Objects/Ellipse/<separator>"),         NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Ellipse/Arc"),                 NULL,           obj_set_tool_elarc,       0,          NULL},
       	{ N_("/Objects/<separator>"),                 NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Polygon"),                     NULL,           obj_set_tool_polygon,     0,          NULL},
       	{ N_("/Objects/Rectangle"),                   NULL,           obj_set_tool_rectangle,   0,          NULL},
       	{ N_("/Objects/<separator>"),                 NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Point"),                       NULL,           obj_set_tool_point,       0,          NULL},
       	{ N_("/Objects/<separator>"),                 NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Text"),                        NULL,           obj_set_tool_text,        0,          NULL},
       	{ N_("/Objects/<separator>"),                 NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Insert Block"),                NULL,           obj_set_tool_insert,      0,          NULL},
       	{ N_("/Objects/<separator>"),                 NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Attribute"),                   NULL,           obj_set_tool_attribute,   0,          NULL},
       	{ N_("/Objects/<separator>"),                 NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Objects/Hatch..."),                    NULL,           NULL,                     0,          NULL},
	
	{ N_("/_Dimensioning"),                       NULL,           NULL,                     0,          "<Branch>"},
	{ N_("/Dimensioning/Linear/Horizontel"),      NULL,           dim_set_tool_horiz,       0,          NULL},
	{ N_("/Dimensioning/Linear/Vertical"),        NULL,           dim_set_tool_vert,        0,          NULL},
       	{ N_("/Dimensioning/Aligned"),                NULL,           dim_set_tool_aligned,     0,          NULL},
       	{ N_("/Dimensioning/Baseline"),               NULL,           dim_set_tool_baseline,    0,          NULL},
       	{ N_("/Dimensioning/Continue"),               NULL,           dim_set_tool_continue,    0,          NULL},
       	{ N_("/Dimensioning/<separator>"),            NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Dimensioning/Radial/Radius"),          NULL,           dim_set_tool_radius,      0,          NULL},
       	{ N_("/Dimensioning/Radial/Diameter"),        NULL,           dim_set_tool_diameter,    0,          NULL},
	{ N_("/Dimensioning/Center Mark"),            NULL,           dim_set_tool_center,      0,          NULL},
       	{ N_("/Dimensioning/<separator>"),            NULL,           NULL,                     0,          "<Separator>"},
       	{ N_("/Dimensioning/Angular"),                NULL,           dim_set_tool_angular,     0,          NULL},
	{ N_("/Dimensioning/Leader"),                 NULL,           dim_set_tool_leader,      0,          NULL},

	{ N_("/_Construct"),                     NULL,        NULL,                  0,          "<Branch>"},	
	{ N_("/Construct/Copy"),                 NULL,        tool_set_tool_copy,    0,          NULL},	
	{ N_("/Construct/Offset"),               NULL,        tool_set_tool_offset,  0,          NULL},
	{ N_("/Construct/Mirror"),               NULL,        tool_set_tool_mirror,  0,          NULL},
	{ N_("/Construct/Array/Rectangular"),    NULL,        tool_set_tool_array,   0,          NULL},
	{ N_("/Construct/Array/Polar"),          NULL,        tool_set_tool_array,   1,          NULL},
	{ N_("/Construct/sep1"),                 NULL,        NULL,                  0,          "<Separator>"},
 	{ N_("/Construct/Chamfer"),              NULL,        tool_set_tool_chamfer, 0,          NULL},
 	{ N_("/Construct/Fillet"),               NULL,        tool_set_tool_fillet,  0,          NULL},
	{ N_("/Construct/sep1"),                 NULL,        NULL,                  0,          "<Separator>"},
 	{ N_("/Construct/Block"),                NULL,        tool_set_tool_block,   0,          NULL},

 	{ N_("/_Modify"),                NULL,        NULL,                  0,          "<Branch>"},
	{ N_("/Modify/Move"),            NULL,        tool_set_tool_move,    0,          NULL},
	{ N_("/Modify/Rotate"),          NULL,        tool_set_tool_rotate,  0,          NULL},
 	{ N_("/Modify/sep3"),            NULL,        NULL,                  0,          "<Separator>"},
 	{ N_("/Modify/Stretch"),         NULL,        tool_set_tool_stretch, 0,          NULL},
 	{ N_("/Modify/Scale"),           NULL,        tool_set_tool_scale,   0,          NULL},
 	{ N_("/Modify/Lengthen"),        NULL,        NULL,                  0,          NULL},
   	{ N_("/Modify/sep1"),            NULL,        NULL,                  0,          "<Separator>"},
 	{ N_("/Modify/Trim"),            NULL,        tool_set_tool_trim,    0,          NULL},
 	{ N_("/Modify/Extend"),          NULL,        NULL,                  0,          NULL},
 	{ N_("/Modify/Break"),           NULL,        tool_set_tool_break,   0,          NULL},
 	{ N_("/Modify/sep2"),            NULL,        NULL,                  0,          "<Separator>"},
 	{ N_("/Modify/Edit/Polyline"),   NULL,        NULL,                  0,          NULL},
 	{ N_("/Modify/Edit/Spline"),     NULL,        NULL,                  0,          NULL},
 	{ N_("/Modify/Edit/Text"),       NULL,        tool_set_tool_etext,   0,          NULL},
 	{ N_("/Modify/Edit/sep5"),       NULL,        NULL,                  0,          "<Separator>"},
 	{ N_("/Modify/Edit/Properties"), NULL,        tool_set_tool_eprop,   0,          NULL},
 	{ N_("/Modify/Explode"),         NULL,        NULL,                  0,          NULL},
 	{ N_("/Modify/sep4"),            NULL,        NULL,                  0,          "<Separator>"},
 	{ N_("/Modify/Erase"),           NULL,        tool_set_tool_erase,   0,          NULL},
	
	{ N_("/_Help"),                NULL,        NULL,                    0,             "<LastBranch>"},
	{ N_("/Help/_GPL"),            NULL,        menu_show_dialog,        DIALOG_GPL,    NULL},
	{ N_("/Help/_About oCADis"),   NULL,        menu_show_dialog,        DIALOG_ABOUT,  NULL}
};


/* --  F U N C T I O N S  -- */
GtkWidget *menus_get_widget(char *path);


GtkItemFactory *item_factory;

GtkWidget *create_main_menu(GtkWidget  *window)
{
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
  gint i;

  accel_group = gtk_accel_group_new ();
  
  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
     or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
     the accelerator table while generating menus.
  */  
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
				       accel_group);
  
  /* Translate every menuitem with getttext */
  for(i=0;i<nmenu_items;i++)
    menu_items[i].path=_(menu_items[i].path);
  
  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
  
  /* Attach the new accelerator group to the window. */
  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));
  
  return gtk_item_factory_get_widget (item_factory, "<main>");  
}

GtkWidget *menus_get_widget(char *path)
{
  gint i,j;
  gchar new_path[1024];
  GtkWidget *widget;

  /* Remove all underscore */
  for(i=0,j=0;i<strlen(path);i++)
    {
      if(path[i]!='_')
	{
	  new_path[j]=path[i];
	  j++;
	}
    }
  new_path[j]='\0';

  widget=gtk_item_factory_get_widget(item_factory, new_path);

  if(widget==NULL)
    g_warning("Cannot find widget in path %s", new_path);

  return widget;
}

void menus_set_sensitive(char *path, int sensitive)
{
  GtkWidget *widget;

  widget=menus_get_widget(path);
  
  if(widget!=NULL)
    gtk_widget_set_sensitive ( widget, sensitive);  
}

void menus_not_selectable(void)
{
  gint i;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);


  for(i=0;i<nmenu_items;i++)
    {
      if(menu_items[i].callback==NULL && menu_items[i].item_type==NULL)
	menus_set_sensitive(menu_items[i].path,FALSE);
    }

}

GtkWidget *make_menuitem(gchar *label, GtkSignalFunc func, gpointer data)
     /* Creates a menu item */
{
  GtkWidget *menuitem;

  menuitem=gtk_menu_item_new_with_label(label);
  gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
		     GTK_SIGNAL_FUNC (func),
		     data);
  gtk_widget_show(menuitem);  

  return menuitem;
}

GtkWidget *make_menuitem_hr(void)
     /* Creates a horizontal separator for menus */
{
  GtkWidget *menuitem;

  menuitem=gtk_menu_item_new();
  gtk_widget_set_sensitive(GTK_WIDGET(menuitem), FALSE);
  gtk_widget_show(menuitem);  

  return menuitem;
}













