/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "draw.h"

/* Intersection diagram


                            C 
                            o
                            n
                            s
                            t
                    P       r
                    o       u               E 
                    l       c   S       C   l   
                    y       t   p       i   l
                L   l       t   l       r   i   
                i   i   R   i   i   A   c   p 
                n   n   a   o   n   r   l   s 
                e   e   y   n   e   c   e   e 
             +---------------------------------
Line         |  D   D               D   D     
Polyline     |  X                             
Ray          |  XXXXX                         
Construction |  XXXXXXXXX                     
Spline       |  XXXXXXXXXXXXX                 
Arc          |  XXXXXXXXXXXXXXXXX   D   D     
Circle       |  XXXXXXXXXXXXXXXXXXXX    D     
Ellipse      |  XXXXXXXXXXXXXXXXXXXXXXXXX      

W - Working on that. Not done yet.
D - Done.
  - Currently unsupported.

 */












/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
static gint test_angle(OBJECT *obj, gdouble test_x, gdouble test_y)
{
  return misc_within_angle(obj->data.arc.start_angle-GFLOAT_SENSITIVE,obj->data.arc.end_angle+GFLOAT_SENSITIVE, 
			   get_angle(obj->data.arc.centrum.x, obj->data.arc.centrum.y, test_x,test_y));
}

static gint test_angle_short(OBJECT_UNION *obj, gdouble test_x, gdouble test_y)
{
  return misc_within_angle(obj->arc.start_angle-GFLOAT_SENSITIVE,obj->arc.end_angle+GFLOAT_SENSITIVE, 
			   get_angle(obj->arc.centrum.x, obj->arc.centrum.y, test_x,test_y));
}

void intersect_line_vs_line(OBJECT *line1, OBJECT *line2, gint *num, POINT *list)
{
  gdouble Ax,Ay,Bx,By,Cx,Cy,Dx,Dy;
  gdouble r,s;

  Ax=line1->data.line.p1.x;
  Ay=line1->data.line.p1.y;
  Bx=line1->data.line.p2.x;
  By=line1->data.line.p2.y;
  Cx=line2->data.line.p1.x;
  Cy=line2->data.line.p1.y;
  Dx=line2->data.line.p2.x;
  Dy=line2->data.line.p2.y;
  
  if(fabs((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx))<GFLOAT_SENSITIVE)
    r=((Ay-Cy)*(Dx-Cx)-(Ax-Cx)*(Dy-Cy))/GFLOAT_SENSITIVE;
  else
    r=((Ay-Cy)*(Dx-Cx)-(Ax-Cx)*(Dy-Cy))/((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx));

  if(fabs((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx))<GFLOAT_SENSITIVE)
    s=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/GFLOAT_SENSITIVE;
  else
    s=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx));

  if( (r>=(0-GFLOAT_SENSITIVE)) && (r<=(1+GFLOAT_SENSITIVE)) && (s>=(0-GFLOAT_SENSITIVE)) && (s<=(1+GFLOAT_SENSITIVE)))
    {
      list[*num].x=Ax+r*(Bx-Ax);
      list[*num].y=Ay+r*(By-Ay);
      (*num)++;
    }
}

void intersect_line_vs_line_infinite(OBJECT *line1, OBJECT *line2, gint *num, POINT *list)
{
  gdouble Ax,Ay,Bx,By,Cx,Cy,Dx,Dy;
  gdouble k1,k2;
  gdouble m1,m2;

  Ax=line1->data.line.p1.x;
  Ay=line1->data.line.p1.y;
  Bx=line1->data.line.p2.x;
  By=line1->data.line.p2.y;
  Cx=line2->data.line.p1.x;
  Cy=line2->data.line.p1.y;
  Dx=line2->data.line.p2.x;
  Dy=line2->data.line.p2.y;

  if(fabs(Bx-Ax)<GFLOAT_SENSITIVE)
    k1=(By-Ay)/GFLOAT_SENSITIVE;
  else
    k1=(By-Ay)/(Bx-Ax);

  m1=Ay-k1*Ax;

  if(fabs(Dx-Cx)<GFLOAT_SENSITIVE)
    k2=(Dy-Cy)/GFLOAT_SENSITIVE;
  else
    k2=(Dy-Cy)/(Dx-Cx);

  m2=Cy-k2*Cx;

  if(  (TRUE) ) 
    {
      list[*num].x=(m2-m1)/(k1-k2);
      list[*num].y=k1*list[*num].x + m1;
      (*num)++;
    }  
}

void intersect_line_vs_circle(OBJECT *line, OBJECT *circle, gint *num, POINT *list)
{
  gdouble Ax,Ay,Bx,By,Cx,Cy,Dx,Dy;
  gdouble r,s,a,b,c,L;

  Ax=line->data.line.p1.x;
  Ay=line->data.line.p1.y;
  Bx=line->data.line.p2.x;
  By=line->data.line.p2.y;
  Cx=circle->data.circle.centrum.x;
  Cy=circle->data.circle.centrum.y;
  r=circle->data.circle.radius;
  
  a=(Bx-Ax)*(Bx-Ax)+(By-Ay)*(By-Ay);
  b=2*( (Bx-Ax)*(Ax-Cx)+(By-Ay)*(Ay-Cy) );
  c=Cx*Cx+Cy*Cy+Ax*Ax+Ay*Ay-2*(Cx*Ax+Cy*Ay)-r*r;
  
  L=b*b-4*a*c;
  
  if(L>0)
    {		  		
      r=sqrt( (Bx-Ax)*(Bx-Ax)+ (By-Ay)*(By-Ay)  );
      
      /* First intersection point */
      s=(-b+sqrt(L))/(2*a);
      Cx=Ax+s*(Bx-Ax);
      Cy=Ay+s*(By-Ay);
      
      Dx=((Ay-Cy)*(Ay-By)-(Ax-Cx)*(Bx-Ax))/(r*r);
      Dy=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/(r*r);
      
      /* Check to see that the intersection point is on the line segment */
      if((Dx>=(0-GFLOAT_SENSITIVE) && Dx<=(1+GFLOAT_SENSITIVE) && ABS(Dy)<GFLOAT_SENSITIVE ))
	{
	  list[*num].x=Cx;
	  list[*num].y=Cy;
	  (*num)++;
	}

      /* Second intersection point */
      s=(-b-sqrt(L))/(2*a);
      Cx=Ax+s*(Bx-Ax);
      Cy=Ay+s*(By-Ay);
      
      Dx=((Ay-Cy)*(Ay-By)-(Ax-Cx)*(Bx-Ax))/(r*r);
      Dy=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/(r*r);
      
      /* Check to see that the intersection point is on the line segment */
      if((Dx>=(0-GFLOAT_SENSITIVE) && Dx<=(1+GFLOAT_SENSITIVE) && ABS(Dy)<GFLOAT_SENSITIVE ))
	{
	  list[*num].x=Cx;
	  list[*num].y=Cy;
	  (*num)++;
	}
    }
}

void intersect_line_vs_pline(OBJECT *line, OBJECT *pline, gint *num, POINT *list)
{
  gdouble Ax,Ay,Bx,By,Cx,Cy,Dx,Dy;
  gdouble r,s, a, b, c, L;
  OBJECT_UNION u;
  gint type;
  gboolean ret;
  gboolean (*draw_vertex)(gint *type, OBJECT_UNION *s);

  draw_vertex=draw_polyline_set_polyline(pline, NULL);

  ret=draw_vertex(&type,&u);

  while(ret)
    {
      ret=draw_vertex(&type,&u);

      if(!ret)
	break;
      
      switch(type)
	{
	case NODE_LINE:
	  Ax=line->data.line.p1.x;
	  Ay=line->data.line.p1.y;
	  Bx=line->data.line.p2.x;
	  By=line->data.line.p2.y;
	  Cx=u.line.p1.x;
	  Cy=u.line.p1.y;
	  Dx=u.line.p2.x;
	  Dy=u.line.p2.y;
	  
	  if(fabs((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx))<GFLOAT_SENSITIVE)
	    r=((Ay-Cy)*(Dx-Cx)-(Ax-Cx)*(Dy-Cy))/GFLOAT_SENSITIVE;
	  else
	    r=((Ay-Cy)*(Dx-Cx)-(Ax-Cx)*(Dy-Cy))/((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx));
	  
	  if(fabs((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx))<GFLOAT_SENSITIVE)
	    s=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/GFLOAT_SENSITIVE;
	  else
	    s=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/((Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx));
	  
	  if( (r>=(0-GFLOAT_SENSITIVE)) && (r<=(1+GFLOAT_SENSITIVE)) && (s>=(0-GFLOAT_SENSITIVE)) && (s<=(1+GFLOAT_SENSITIVE)))
	    {
	      list[*num].x=Ax+r*(Bx-Ax);
	      list[*num].y=Ay+r*(By-Ay);
	      (*num)++;
	    }
	  break;
	case NODE_ARC:
	  Ax=line->data.line.p1.x;
	  Ay=line->data.line.p1.y;
	  Bx=line->data.line.p2.x;
	  By=line->data.line.p2.y;
	  Cx=u.arc.centrum.x;
	  Cy=u.arc.centrum.y;
	  r=u.arc.radius;
	  
	  a=(Bx-Ax)*(Bx-Ax)+(By-Ay)*(By-Ay);
	  b=2*( (Bx-Ax)*(Ax-Cx)+(By-Ay)*(Ay-Cy) );
	  c=Cx*Cx+Cy*Cy+Ax*Ax+Ay*Ay-2*(Cx*Ax+Cy*Ay)-r*r;
	  
	  L=b*b-4*a*c;
	  
	  if(L>0)
	    {		  		
	      r=sqrt( (Bx-Ax)*(Bx-Ax)+ (By-Ay)*(By-Ay)  );
	      
	      /* First intersection point */
	      s=(-b+sqrt(L))/(2*a);
	      Cx=Ax+s*(Bx-Ax);
	      Cy=Ay+s*(By-Ay);
	      
	      Dx=((Ay-Cy)*(Ay-By)-(Ax-Cx)*(Bx-Ax))/(r*r);
	      Dy=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/(r*r);
	      
	      /* Check to see that the intersection point is on the line segment
		 and its poit lies between the angles of the arc */
	      if((Dx>=(0-GFLOAT_SENSITIVE) && Dx<=(1+GFLOAT_SENSITIVE) && ABS(Dy)<GFLOAT_SENSITIVE && test_angle_short(&u, Cx,Cy)))
		{
		  list[*num].x=Cx;
		  list[*num].y=Cy;
		  (*num)++;
		}
	      
	      /* Second intersection point */
	      s=(-b-sqrt(L))/(2*a);
	      Cx=Ax+s*(Bx-Ax);
	      Cy=Ay+s*(By-Ay);
	      
	      Dx=((Ay-Cy)*(Ay-By)-(Ax-Cx)*(Bx-Ax))/(r*r);
	      Dy=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/(r*r);
	      
	      /* Check to see that the intersection point is on the line segment
		 and its point lies between the angles of the arc */
	      if((Dx>=(0-GFLOAT_SENSITIVE) && Dx<=(1+GFLOAT_SENSITIVE) && ABS(Dy)<GFLOAT_SENSITIVE && test_angle_short(&u,Cx,Cy)))
		{
		  list[*num].x=Cx;
		  list[*num].y=Cy;
		  (*num)++;
		}
	      
	    }	   
	  break;
	}
    }
  
}



void intersect_line_vs_arc(OBJECT *line, OBJECT *arc,gint *num, POINT *list) 
{
  gdouble Ax,Ay,Bx,By,Cx,Cy,Dx,Dy;
  gdouble r,s,a,b,c,L;

  Ax=line->data.line.p1.x;
  Ay=line->data.line.p1.y;
  Bx=line->data.line.p2.x;
  By=line->data.line.p2.y;
  Cx=arc->data.arc.centrum.x;
  Cy=arc->data.arc.centrum.y;
  r=arc->data.arc.radius;
	    
  a=(Bx-Ax)*(Bx-Ax)+(By-Ay)*(By-Ay);
  b=2*( (Bx-Ax)*(Ax-Cx)+(By-Ay)*(Ay-Cy) );
  c=Cx*Cx+Cy*Cy+Ax*Ax+Ay*Ay-2*(Cx*Ax+Cy*Ay)-r*r;
	    
  L=b*b-4*a*c;
  
  if(L>0)
    {		  		
      r=sqrt( (Bx-Ax)*(Bx-Ax)+ (By-Ay)*(By-Ay)  );
      
      /* First intersection point */
      s=(-b+sqrt(L))/(2*a);
      Cx=Ax+s*(Bx-Ax);
      Cy=Ay+s*(By-Ay);
      
      Dx=((Ay-Cy)*(Ay-By)-(Ax-Cx)*(Bx-Ax))/(r*r);
      Dy=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/(r*r);
      
      /* Check to see that the intersection point is on the line segment
	 and its poit lies between the angles of the arc */
      if((Dx>=(0-GFLOAT_SENSITIVE) && Dx<=(1+GFLOAT_SENSITIVE) && ABS(Dy)<GFLOAT_SENSITIVE && test_angle(arc, Cx,Cy)))
	{
	  list[*num].x=Cx;
	  list[*num].y=Cy;
	  (*num)++;
	}

      /* Second intersection point */
      s=(-b-sqrt(L))/(2*a);
      Cx=Ax+s*(Bx-Ax);
      Cy=Ay+s*(By-Ay);
      
      Dx=((Ay-Cy)*(Ay-By)-(Ax-Cx)*(Bx-Ax))/(r*r);
      Dy=((Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay))/(r*r);
      
      /* Check to see that the intersection point is on the line segment
	 and its point lies between the angles of the arc */
      if((Dx>=(0-GFLOAT_SENSITIVE) && Dx<=(1+GFLOAT_SENSITIVE) && ABS(Dy)<GFLOAT_SENSITIVE && test_angle(arc,Cx,Cy)))
	{
	  list[*num].x=Cx;
	  list[*num].y=Cy;
	  (*num)++;
	}
     
    }	   
}

void intersect_circle_vs_circle(OBJECT *circle1, OBJECT *circle2,gint *num, POINT *list) 
{
  gdouble Ax,Ay,Bx,By, Cx,Cy;
  gdouble r0, r1, a,h,d;

  Ax=circle1->data.circle.centrum.x;
  Ay=circle1->data.circle.centrum.y;
  r0=circle1->data.circle.radius;

  Bx=circle2->data.circle.centrum.x;
  By=circle2->data.circle.centrum.y;
  r1=circle2->data.circle.radius;

  d=dist(Ax,Ay,Bx,By);

  if(d<=(r0+r1)+GFLOAT_SENSITIVE && d+GFLOAT_SENSITIVE>=((r0>r1)?(r0-r1):(r1-r0)))
    {
      a=(r0*r0-r1*r1+d*d)/(2*d);
      Cx=Ax+a*(Bx-Ax)/d;
      Cy=Ay+a*(By-Ay)/d;

      h=sqrt(r0*r0-a*a);
      
      list[*num].x=Cx+h*(By-Ay)/d;
      list[*num].y=Cy-h*(Bx-Ax)/d;
      (*num)++;

      list[*num].x=Cx-h*(By-Ay)/d;
      list[*num].y=Cy+h*(Bx-Ax)/d;
      (*num)++;
    }
}

void intersect_circle_vs_arc(OBJECT *circle, OBJECT *arc,gint *num, POINT *list) 
{
  gdouble Ax,Ay,Bx,By, Cx,Cy;
  gdouble r0, r1, a,h,d;

  Ax=circle->data.circle.centrum.x;
  Ay=circle->data.circle.centrum.y;
  r0=circle->data.circle.radius;

  Bx=arc->data.arc.centrum.x;
  By=arc->data.arc.centrum.y;
  r1=arc->data.arc.radius;

  d=dist(Ax,Ay,Bx,By);

  if(d<=(r0+r1)+GFLOAT_SENSITIVE && d+GFLOAT_SENSITIVE>=((r0>r1)?(r0-r1):(r1-r0)))
    {
      a=(r0*r0-r1*r1+d*d)/(2*d);
      Cx=Ax+a*(Bx-Ax)/d;
      Cy=Ay+a*(By-Ay)/d;

      h=sqrt(r0*r0-a*a);

      if(test_angle(arc, Cx+h*(By-Ay)/d, Cy-h*(Bx-Ax)/d))
	{
	  list[*num].x=Cx+h*(By-Ay)/d;
	  list[*num].y=Cy-h*(Bx-Ax)/d;
	  (*num)++;
	}

      if(test_angle(arc, Cx-h*(By-Ay)/d,Cy+h*(Bx-Ax)/d))
	{
	  list[*num].x=Cx-h*(By-Ay)/d;
	  list[*num].y=Cy+h*(Bx-Ax)/d;
	  (*num)++;
	}
    }
}

void intersect_arc_vs_arc(OBJECT *arc1, OBJECT *arc2, gint *num, POINT *list) 
{
  gdouble Ax,Ay,Bx,By, Cx,Cy;
  gdouble r0, r1, a,h,d;

  Ax=arc1->data.arc.centrum.x;
  Ay=arc1->data.arc.centrum.y;
  r0=arc1->data.arc.radius;

  Bx=arc2->data.arc.centrum.x;
  By=arc2->data.arc.centrum.y;
  r1=arc2->data.arc.radius;

  d=dist(Ax,Ay,Bx,By);

  if(d<=(r0+r1)+GFLOAT_SENSITIVE && d+GFLOAT_SENSITIVE>=((r0>r1)?(r0-r1):(r1-r0)))
    {
      a=(r0*r0-r1*r1+d*d)/(2*d);
      Cx=Ax+a*(Bx-Ax)/d;
      Cy=Ay+a*(By-Ay)/d;

      h=sqrt(r0*r0-a*a);

      if(test_angle(arc1, Cx+h*(By-Ay)/d,Cy-h*(Bx-Ax)/d ) &&  
	 test_angle(arc2, Cx+h*(By-Ay)/d, Cy-h*(Bx-Ax)/d))
	{
	  list[*num].x=Cx+h*(By-Ay)/d;
	  list[*num].y=Cy-h*(Bx-Ax)/d;
	  (*num)++;
	}

      if(test_angle(arc1, Cx-h*(By-Ay)/d, Cy+h*(Bx-Ax)/d) &&  
	 test_angle(arc2, Cx-h*(By-Ay)/d, Cy+h*(Bx-Ax)/d))     
	{
	  list[*num].x=Cx-h*(By-Ay)/d;
	  list[*num].y=Cy+h*(Bx-Ax)/d;
	  (*num)++;
	}
    }
}








