/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <string.h>
#include "struct.h"
#include "color.h"
#include "dia_message.h"
#include "initmenu.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
LAYER *layer_find(DOCUMENT *doc, gchar *name)
  /* Returns the layer with the name or NULL
     if no layer was found with that name */
{
  GList *list;

  list=g_list_first(doc->layer_list);

  while(list!=NULL)
    {
      if(!strcmp(((LAYER*)list->data)->name, name))
	return (LAYER *)list->data;

      list=g_list_next(list);
    }

  return NULL;
}

static gint change_layer(GtkWidget *widget, LAYER *layer)
  /* Callback to layer-popup. 
     Change the current layer. */
{
  DOCUMENT *doc;

  if(!layer->editable)
    message(ICON_INFO, _("This layer is locked.\nTo unlock it, select Assist/Layers."));

  doc=layer->doc;

  doc->current_layer=layer;
  
  if(doc->type_colour==BYLAYER)
    doc->current_colour=layer->colour;

  if(doc->type_linestyle==BYLAYER)
    doc->current_linestyle=layer->linestyle;

  return TRUE;
}

void make_layer_popmenu(DOCUMENT *doc)
  /* This function creates the popup-layer-menu 
     on the drawing windows. It should be called
     after adding or modifying layers. */
{
  GList *list;

  /* Remove old menu */
  gtk_widget_destroy(doc->widgets.toolbar_layermenu);

  /* Make new */
  doc->widgets.toolbar_layermenu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(doc->widgets.toolbar_option),
			   doc->widgets.toolbar_layermenu);

  list=g_list_first(doc->layer_list);

  while(list!=NULL)
    {
      /* Create layer menulist and connect it */
      gtk_menu_append(GTK_MENU(doc->widgets.toolbar_layermenu), make_menuitem(((LAYER *)list->data)->name, 
									      GTK_SIGNAL_FUNC (change_layer), 
									      (gpointer)list->data));       
      list=g_list_next(list);
    }

  /* Set the current layer to layer number 0 */
  gtk_option_menu_set_history(GTK_OPTION_MENU(doc->widgets.toolbar_option),0);
}

LAYER *layer_new(DOCUMENT *doc, gint row, LINESTYLE *linestyle, gdouble linewidth,gdouble r, gdouble g, gdouble b, 
		 gchar *name, gboolean visible, gboolean editable)
  /* Creates a new layer called name.
     if row==-1 add this layer last in the layer list
     otherwise put the layer in position row in the
     layer list. */
{
  LAYER *temp;

  temp=(LAYER *)g_malloc(sizeof(LAYER));
  
  temp->name=(gchar *)g_malloc(sizeof(gchar)*(strlen(name)+1));
  strcpy(temp->name, name);

  temp->visible=visible;
  temp->editable=editable;
  temp->linestyle=linestyle;
  temp->linewidth=linewidth;
  temp->object_list=NULL;
  temp->colour=make_new_userdefined_colour(doc, r,g,b);

  temp->doc=doc;

  if(row==-1)
    doc->layer_list=g_list_append(doc->layer_list,temp);
  else
    doc->layer_list=g_list_insert(doc->layer_list,temp, row);
  
  return temp;
}

void layer_delete(DOCUMENT *doc, LAYER *layer)
  /* Deletes the layer named name.
     Does not destroy the objects in layer! 
     It should first be tested if any objects
     exist in this layer. If so remove them
     before calling this function. */
{
  doc->layer_list=g_list_remove(doc->layer_list, (gpointer) layer);
  
  if(doc->layer_list==NULL)
    doc->current_layer=NULL;
  else
    doc->current_layer=doc->layer_list->data;
}







