/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <string.h>
#include "struct.h"
#include "initmenu.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
LINESTYLE *linestyle_find(DOCUMENT *doc, gchar *name)
  /* Returns the linestyle with the name or NULL
     if no linestyle was found with that name */
{
  GList *list;

  list=g_list_first(doc->linestyle_list);

  while(list!=NULL)
    {
      if(!strcmp(((LINESTYLE *)list->data)->name, name))
	return (LINESTYLE *)list->data;

      list=g_list_next(list);
    }

  return NULL;
}

void make_picture_of_line(DOCUMENT *doc, LINESTYLE *style)
{
  /*  if(style->pic.pixmap!=NULL)
    g_free(style->pic.pixmap);
  */
  /* Create pixmap */
  style->pic.pixmap=gdk_pixmap_new(doc->widgets.window->window,100,5,-1);
  style->pic.mask=NULL;

  /* Clear background */
  gdk_draw_rectangle(style->pic.pixmap,
		     doc->widgets.window->style->white_gc,
		     TRUE, 0, 0, 100,5);

  /* Draw nice line */
  gdk_gc_set_foreground(doc->replace_gc, &doc->black);
  if(style->type==LINE_SOLID)
    gdk_gc_set_line_attributes(doc->replace_gc,3,
			       GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
  else
    {
      gint i;

      for(i=0;i<style->dash_no;i++)
	style->dash_list[i]=MAX(1,style->dash_dlist[i]*10);
	   
      gdk_gc_set_line_attributes(doc->replace_gc,3,
			       GDK_LINE_ON_OFF_DASH, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
      gdk_gc_set_dashes(doc->replace_gc, 0, style->dash_list, style->dash_no);
    }

  gdk_draw_line(style->pic.pixmap,doc->replace_gc,1,2,99,2);

}

static gint change_linestyle_predefined(GtkWidget *widget, LINESTYLE *linestyle)
{
  if(linestyle->doc->type_linestyle==USERDEFINED)
    g_free(linestyle->doc->current_linestyle);

  linestyle->doc->type_linestyle=PREDEFINED;
  linestyle->doc->current_linestyle=linestyle;

  return TRUE;
}

static gint change_linestyle_bylayer(GtkWidget *widget, DOCUMENT *doc)
{
  doc->type_linestyle=BYLAYER;
  doc->current_linestyle=doc->current_layer->linestyle; 
  return TRUE;
}

static gint change_linestyle_byblock(GtkWidget *widget, DOCUMENT *doc)
{
  doc->type_linestyle=BYBLOCK;
  doc->current_linestyle=doc->current_layer->linestyle; 
  return TRUE;
}

void make_linestyle_popmenu(DOCUMENT *doc)
{
  GList *list;

  /* Remove old menu */
  gtk_widget_destroy(doc->widgets.toolbar_linestylemenu);

  /* Make new */
  doc->widgets.toolbar_linestylemenu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(doc->widgets.toolbar_linestyle),
			   doc->widgets.toolbar_linestylemenu);

  /* Make item By Layer */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_linestylemenu),
		  make_menuitem( _("By Layer"), GTK_SIGNAL_FUNC(change_linestyle_bylayer), (gpointer) doc));

  /* Make item By Block */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_linestylemenu),
		  make_menuitem( _("By Block"), GTK_SIGNAL_FUNC(change_linestyle_byblock), (gpointer) doc));
      
  /* Make item Separator */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_linestylemenu), make_menuitem_hr()); 


  /* For each linestyle create a menu item */
  list=doc->linestyle_list;

  while(list!=NULL)
    {
      /* Create linestyle menulist and connect it */
      gtk_menu_append(GTK_MENU(doc->widgets.toolbar_linestylemenu),
		      make_menuitem(((LINESTYLE*)list->data)->name, GTK_SIGNAL_FUNC(change_linestyle_predefined), 
				    (gpointer)(LINESTYLE *)list->data));
      
      list=g_list_next(list);
    }

  /* Set the current layer to layer number 0 */
  gtk_option_menu_set_history(GTK_OPTION_MENU(doc->widgets.toolbar_linestyle),0);
}

LINESTYLE *linestyle_new(DOCUMENT *doc, gint row, gchar *name, gint type_linestyle, gdouble length, gint no, gdouble *pattern)
{
  LINESTYLE *temp;

  temp=(LINESTYLE *)g_malloc(sizeof(LINESTYLE));
  
  temp->name=(gchar *)g_malloc(sizeof(gchar)*(strlen(name)+1));
  strcpy(temp->name, name);

  temp->type=type_linestyle;
  temp->dash_offset=0;
  temp->doc=doc;

  temp->dash_length=length;

  if(type_linestyle==LINE_SOLID)
    {
      temp->dash_list=NULL;
      temp->dash_no=0;
    }
  else
    {
      temp->dash_dlist=pattern;
      temp->dash_list=(gchar *)g_malloc(sizeof(gchar)*no);
      temp->dash_no=no;
    }

  /* Make picture */
  temp->pic.pixmap=NULL;

  make_picture_of_line(doc, temp);

  if(row==-1)
    doc->linestyle_list=g_list_append(doc->linestyle_list,temp);
  else
    doc->linestyle_list=g_list_insert(doc->linestyle_list,temp, row);

  return temp;
}




