/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "translation.h"
#include "gtk_mtext.h"
#include "struct.h"
#include "initmenu.h"
#include "menuitem.h"
#include "digitizer.h"
#include "parse.h"
#include "dia_comm.h"
#include "dia_preference.h"
#include "dia_pagesetup.h"
#include "dia_drawaids.h"
#include "dia_layer.h"
#include "dia_gpl.h"
#include "dia_about.h"
#include "dia_message.h"
#include "dia_osnap.h"
#include "dia_select.h"
#include "dia_list.h"
#include "dia_colour.h"
#include "dia_dimsetup.h"
#include "select.h"
#include "misc.h"
#include "struct/config.h"

/* --  G L O B A L   V A R I A B L E S  -- */
GList *document_list=NULL;     /* List of all documents */
DOCUMENT *work_document=NULL;  /* Current work document */
DOCUMENT *focus_document=NULL; /* Lastest focused document by mouse pointer */

gint gl_tool=TOOL_IDLE;                   /* Current selected tool */
gint gl_stage=0;                          /* Current stage for selected tool */
gint gl_osnap=OSNAP_NONE;                 /* Type of object snap, if any */
gint gl_osnap_sensitivity=4;              /* Sensitivity of object selection mode */
gint gl_osnap_apply_to=APPLY_ALL_LAYERS;  /* What layers to apply object snap */
gint gl_osnap_is=OSNAP_ONCE;              /* Running snap or once time snap */
gint gl_select_mask=-1;                   /* What type of objects is user able to select */
gint gl_select_sensitivity=4;             /* Sensitivity of selection mode */
gint gl_select_apply_to=APPLY_ALL_LAYERS; /* What layers to apply selection */
gdouble gl_oldmx, gl_oldmy;               /* Global variables for holding previous mouse position */

CONFIG configuration;                     /* Configuration settings. This is globals setting that 
					     applies to all drawings */

extern GtkWidget *dia_mtext;              /* Pointer to the command widget */

gint command_from_keyboard=FALSE;         /* Sets to TRUE if user selects a command via the keyboard */
 
GdkCursor *mousecursor[15];               /* List of different mouse cursors */

GList *linewidth_glist=NULL;              /* List of linewidths for option menus */

ICON_STRUCT icon_list[NO_ICONS];          /* List of icons used in message- and alert-boxes */

/* --  L O C A L   V A R I A B L E S  -- */
GtkWidget *main_window;
static GtkWidget *main_vbox;
static GtkWidget *menubar; 

extern void init_font_table(void);


/* --  M A I N   F U N C T I O N  -- */
static gint window_keyboard( GtkWidget *widget, GdkEventKey *event)
{
  gtk_mtext_key_press(dia_mtext,event);
  return TRUE;
}

static gint mainwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  return(try_to_quit(widget, data));
}

static void set_default_configuration(CONFIG *config)
{
  config->use_background=TRUE;

  config->path_to_tablet=NULL;
  config->path_to_device=g_strdup("/dev/ttyS1");
  config->path_to_symbols=g_strdup("./symbols/");
  config->digitizer_data=NULL;
}

int main(int argc, char *argv[])
{
  gchar *tmp;

  /* Set language */
#ifdef ENABLE_NLS
  tmp=g_getenv("NLSPATH");
  if(tmp)
    bindtextdomain(PACKAGE, tmp);
  else
    bindtextdomain(PACKAGE, LOCALEDIR);
 
  textdomain(PACKAGE);
#endif

  /* Init gtk */
  gtk_set_locale();
  gtk_init(&argc, &argv);

  /* Create cursors */
  mousecursor[0]=gdk_cursor_new(GDK_CROSSHAIR);
  mousecursor[1]=gdk_cursor_new(GDK_TOP_LEFT_CORNER);
  mousecursor[2]=gdk_cursor_new(GDK_DOTBOX);
  mousecursor[3]=gdk_cursor_new(GDK_LEFT_PTR);
  mousecursor[4]=gdk_cursor_new(GDK_XTERM);
  mousecursor[5]=gdk_cursor_new(GDK_WATCH);
  mousecursor[6]=gdk_cursor_new(GDK_FLEUR); /* Cursor when panning from keyboard/menu */
  mousecursor[7]=gdk_cursor_new(GDK_FLEUR); /* Cursor when panning with middle mouse button, should be some type of hand */
  
  /* Set default values for configuration */
  set_default_configuration(&configuration);

  /* Create all dialogs */
  make_dialog_command();
  make_dialog_drawaids();
  make_dialog_layer();
  make_dialog_gpl();
  make_dialog_about();
  make_dialog_osnap();
  make_dialog_select();
  make_dialog_list();
  make_dialog_colour();
  make_dialog_preferences();
  make_dialog_pagesetup();
  make_dialog_dimsetup();

  /* Create table for font */
  init_font_table();

  /* Init the parse table for commands */
  parse_init_hashtable();

  /* Create linewidth glist */
  linewidth_glist=g_list_append(linewidth_glist, _("By Layer"));
  linewidth_glist=g_list_append(linewidth_glist, _("By Block"));
  linewidth_glist=g_list_append(linewidth_glist, "0.0");
  linewidth_glist=g_list_append(linewidth_glist, "0.5");
  linewidth_glist=g_list_append(linewidth_glist, "1.0");
  linewidth_glist=g_list_append(linewidth_glist, "2.0");
  linewidth_glist=g_list_append(linewidth_glist, "5.0");
  linewidth_glist=g_list_append(linewidth_glist, "10.0");
  linewidth_glist=g_list_append(linewidth_glist, "15.0");
  linewidth_glist=g_list_append(linewidth_glist, "20.0");
  linewidth_glist=g_list_append(linewidth_glist, "30.0");
    
  /* Creating the main window */
  main_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(main_window),_("oCADis: Main menu"));

  gtk_signal_connect(GTK_OBJECT(main_window),"delete_event", 
		     GTK_SIGNAL_FUNC(mainwindow_close),NULL);
  gtk_signal_connect(GTK_OBJECT(main_window),"key_press_event",
		     (GtkSignalFunc)window_keyboard, NULL);

  /* Create the vbox that holds the menubar */
  main_vbox=gtk_vbox_new(FALSE, 1);
  gtk_container_border_width(GTK_CONTAINER(main_vbox), 1);
  gtk_container_add(GTK_CONTAINER(main_window), main_vbox);
  gtk_widget_show(main_vbox);

  /* Create the menubar */
  menubar=create_main_menu(main_window);
  gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE ,0 );
  gtk_widget_show(menubar);

  /* Turn off menus */
  menus_not_selectable();
  select_submenu(FALSE);

  /* Ok, ready for action! */
  gtk_widget_show(main_window);

  /* Make some icons */
  make_all_icons();

  gtk_main();

  return 0;
}








