/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include "struct.h"
#include "struct/dialog.h"
#include "global.h"
#include "dia_message.h"
#include "new_draw.h"
#include "misc.h"
#include "dia_comm.h"
#include "dia_coords.h"
#include "dia_preference.h"
#include "dia_pagesetup.h"
#include "dia_drawaids.h"
#include "dia_layer.h"


#include "dia_style.h"


#include "dia_dimsetup.h"
#include "dia_gpl.h"
#include "dia_about.h"
#include "dia_block.h"
#include "dia_osnap.h"
#include "dia_select.h"
#include "dia_point.h"
#include "dia_colour.h"
#include "dia_misc.h"
#include "dia_toolbox.h"
#include "file_ocadis.h"
#include "file_dxf.h"
#include "file_ps.h"
#include "file_hpgl.h"
#include "file_mapgen.h"
#include "file_script.h"
#include "tool_zoom.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
gboolean try_to_quit()
{
  gboolean ok=FALSE;
  GList *list;

  list=document_list;

  /* Are all the drawings saved? */
  while(list!=NULL)
    {
      if(!((DOCUMENT *)list->data)->saved)
	ok=TRUE;

      list=list->next;
    }

  /* There are some drawgin not saved yet. Warn the user. */
  if(ok)
    {
      if(alert(ICON_WARNING, _("Quitting"), _("Yes"), _("No"), NULL,
	       _("There are some modified drawings that are\nnot saved.\nAre you sure you want to quit?"))==1)
	gtk_main_quit();      
    }
  else
    gtk_main_quit();

  return ok;
}

void menu_quit(GtkWidget *widget, gpointer data)
{
  try_to_quit();
}

void menu_close(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  doc=get_document();

  if(doc==NULL)
    message(0, _("There is no drawing to close."));
  else
    window_close(widget, NULL, (gpointer)doc);
}

void menu_save(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;
  gchar *tmp;

  doc=(DOCUMENT *)data;

  if(doc==NULL)
    {
      doc=get_document();
      if(doc==NULL)
	{
	  message(0, _("No active drawing found.\nPlease make a new drawing or open an existing drawing."));
	  return;
	}
    }

  if(doc->filename==NULL)
    {
      tmp=dia_fileselector( _("Save drawing"), "");

      if(tmp!=NULL)
	file_save(doc, tmp);
    }
  else
    file_save(doc, doc->filename);
}

void menu_saveas(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;
  gchar *tmp;

  doc=get_document();
  if(doc==NULL)
    {
      message(0, _("No active drawing found.\nPlease make a new drawing or open an existing drawing."));
      return;
    }

  if(doc->filename==NULL)
    {
      tmp=dia_fileselector( _("Save drawing"), NULL);
    }
  else
      tmp=dia_fileselector( _("Save drawing"), NULL);

  if(tmp!=NULL)
    file_save(doc, tmp);
}

void menu_import_file(GtkWidget *widget, gpointer type)
{
  DOCUMENT *doc;
  gint ret=FALSE;
  FILE *fd;
  gchar *filename=NULL;

  switch(GPOINTER_TO_INT(type))
    {
    case 0: /* Import DXF */
      filename=dia_fileselector( _("Import a DXF drawing"), "*.dxf");
      break;
    case 1: /* Import Mapgen */
      filename=dia_fileselector( _("Import a Mapgen-file"), "*.dat");
      break;
    }

  if(filename==NULL)
    return;

  fd=fopen(filename,"r");
 
  if(fd==NULL)
    {
      message(ICON_WARNING, _("File %s cannot be loaded"), filename);	
      return;
    }

  /* Make a document to load the file in to */
  doc=new_draw();       

  if(doc==NULL)
    {
      message(ICON_WARNING, _("Cannot create a empty document."));
      fclose(fd);
      return;
    }

  switch(GPOINTER_TO_INT(type))
    {
    case 0:   /* Import DXF */
      ret=file_import_dxf(doc,fd);
      break;
    case 1:   /* Import Mapgen */
      ret=file_import_mapgen(doc,fd);
      break;
    }

  if(!ret)
      message(ICON_WARNING, _("Error while loading file %d"), filename);	

  if(ret)
    {
      doc->saved=TRUE;
      if(doc->title!=NULL)
	g_free(doc->filename);
      doc->filename=g_strdup(filename);
      if(doc->title!=NULL)
	g_free(doc->title);
      doc->title=g_strdup(filename);
      make_window_title(doc);
    }

  fclose(fd);
}

void menu_export_file(GtkWidget *widget, gpointer type)
{
  gint ret=FALSE;
  FILE *fd;
  struct stat buf;
  DOCUMENT *doc;
  gchar *filename=NULL;
  gint err;

  doc=get_document();
  if(doc==NULL)
    {
      message(0, _("No active drawing found.\n"));
      return;
    }

  switch(GPOINTER_TO_INT(type))
    {
    case 0:   /* DXF */
      filename=dia_fileselector( _("Export drawing as DXF"), "*.dxf");
      break;
    case 1:   /* PostScript */
      filename=dia_fileselector( _("Export drawing as PostScript"), "*.ps");
      break;
    case 2:   /* HPGL */
      filename=dia_fileselector( _("Export drawing as HPGL"), "*.hpgl");
      break;
    }

  if(filename==NULL)
    return;

  err = stat (filename, &buf);
  if (err == 0)
    {      
      if(alert(ICON_QUESTION, _("Overwrite file?"), _("Yes"), _("No"), NULL,
	       _("This file does already exist.\nDo you want to overwrite it?"))!=1)
	return;
    }

  fd=fopen(filename,"w"); 
  
  if(fd==NULL)
    {
      message(ICON_WARNING, _("File %s cannot be saved.\nPlease make sure that you have\nthe permission to write in the specific path."), filename);	
      return;
    }

  switch(GPOINTER_TO_INT(type))
    {
    case 0:   /* DXF */
      ret=file_export_dxf(doc,fd);
      break;
    case 1:   /* PostScript */
      ret=file_export_ps(doc,fd);
      break;
    case 2:   /* HPGL */
      ret=file_export_hpgl(doc,fd);
      break;
    }

  if(!ret)
      message(ICON_WARNING, _("Error while saving file %d"), filename);	

  fclose(fd);
}

void menu_load(GtkWidget *widget, gpointer data)
{
  gchar *tmp;

  tmp=dia_fileselector( _("Open Drawing"), NULL);

  if(tmp!=NULL)
    file_open(tmp);
}

void menu_script(GtkWidget *widget, gpointer data)
{
  gchar *tmp;
  
  tmp=dia_fileselector( _("Open Script"), NULL);
  
  if(tmp!=NULL)
      file_script(tmp);
}

void menu_new(GtkWidget *widget, gpointer data)
{
  new_draw();
}

void menu_zoom_all(GtkWidget *widget, gpointer type)
{
  tool_zoom_all(NULL);
}

void menu_zoom_previous(GtkWidget *widget, gpointer type)
{
  tool_zoom_previous(NULL);
}


void menu_show_dialog(GtkWidget *widget, gpointer func)
{
  switch(GPOINTER_TO_INT(func))
    {
    case DIALOG_COORD:
      dia_show_coords(widget,0);
      break;
    case DIALOG_COMM:
      dia_show_command(widget,0);
      break;
    case DIALOG_PAGESETUP:
      dia_show_pagesetup(widget,0);
      break;
    case DIALOG_DRAWAIDS:
      dia_show_drawaids(widget,0);
      break;
    case DIALOG_LAYER:
      dia_show_layer(widget,0);
      break;
    case DIALOG_COLOUR:
      dia_show_colour(widget,0);
      break;
    case DIALOG_LINESTYLE:
      dia_show_style(DIALOG_LINESTYLE);
      break;
    case DIALOG_TEXTSTYLE:
      dia_show_style(DIALOG_TEXTSTYLE);
      break;
    case DIALOG_DIMSTYLE:
      dia_show_dimsetup(widget,0);
      break;
    case DIALOG_POINTSTYLE:
      dia_show_style(DIALOG_POINTSTYLE);
      break;
    case DIALOG_OSNAP:
      dia_show_osnap(widget,0);
      break;
    case DIALOG_SELECT:
      dia_show_select(widget,0);
      break;
    case DIALOG_PREFERENCES:
      dia_show_preferences(widget,0);
      break;
    case DIALOG_GPL:
      dia_show_gpl(widget,0);
      break;
    case DIALOG_ABOUT:
      dia_show_about(widget,0);
      break;
    case DIALOG_TOOLBOX:
      dia_show_toolbox(widget,0);
      break;
    case DIALOG_BLOCK:
      dia_show_block();
      break;
    default:
      break;
    }
}



