/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_misc.h"
#include "file_ocadis.h"
#include "new_draw.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */
#define OCADIS TRUE;

/* --  L O C A L   V A R I A B L E S  -- */
void misc_setup_black_line(DOCUMENT *doc);

/* --  P R O T O T Y P E S  -- */
void misc_rotate_point(gdouble angle, gdouble origo_x, gdouble origo_y, gdouble *x, gdouble *y);


/* --  F U N C T I O N S  -- */
gint misc_within_angle(gdouble sangle, gdouble eangle, gdouble tangle)
{
  if(eangle>sangle)
    return( (tangle>sangle) && (tangle<eangle));
  else
    return( (tangle>sangle) || (tangle<eangle));
}

void misc_xor_line(DOCUMENT *doc,gdouble x, gdouble y)
{
  misc_setup_black_line(doc);
  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,gl_oldmx),WLDtoSCR_y(doc,gl_oldmy),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
}

void make_window_title(DOCUMENT *doc)
{
  gchar temp[200]="\0";

  if(TRUE)
    strcat(temp, _("oCADis: "));

  if(!doc->saved)
    strcat(temp, "*");

  if(doc->filename==NULL)
    strcat(temp, _("[no name]"));
  else 
    strcat(temp, doc->filename);

  if(doc->title!=NULL)
    g_free(doc->title);

  doc->title=g_strdup(temp);

  gtk_window_set_title(GTK_WINDOW(doc->widgets.window),doc->title);
}

void update_rulers(DOCUMENT *doc)
{
  gtk_ruler_set_range(GTK_RULER(doc->widgets.horiz_ruler),SCRtoWLD_x(doc,0) , 
		       SCRtoWLD_x(doc,doc->widgets.draw->allocation.width),0,1000  );

  gtk_ruler_set_range(GTK_RULER(doc->widgets.vert_ruler),SCRtoWLD_y(doc,0) , 
		       SCRtoWLD_y(doc,doc->widgets.draw->allocation.height),0,1000  );
}

gdouble get_angle(gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  gdouble k;

  k=atan2( y2-y1 , x2-x1 )*180.0/M_PI;

  if(k<0)
    k=-k;
  else
    k=360-k;

  if(ABS(k-360.0)<GFLOAT_SENSITIVE)
    k=0.0;

  return k;
}

DOCUMENT *get_document(void)
{
  if(work_document!=NULL)
    return work_document;
  else if(focus_document!=NULL)
    return focus_document;
  else if(document_list!=NULL)
    return document_list->data;

 return NULL;
}

gdouble dist(gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  return( (gdouble)sqrt( (x1-x2)*(x1-x2)+(y1-y2)*(y1-y2) ) );
}

gdouble dist_arc(gdouble sangle, gdouble eangle, gdouble radius)
{
  if(eangle<sangle)
    return (ABS(eangle)/360*2*M_PI*radius)+(ABS(360-sangle)/360*2*M_PI*radius);
  else
    return (ABS(sangle-eangle)/360*2*3.14159*radius);
}

gdouble dist_arc2(gdouble sangle, gdouble eangle, gdouble radius)
{
  return (ABS(sangle-eangle)/360*2*M_PI*radius);
}

void make_snap(DOCUMENT *doc, gdouble *x, gdouble *y)
{
  misc_rotate_point(doc->gl_snap_angle, doc->gl_snap_basex,doc->gl_snap_basey,x,y);

  *x=rint(((*x)-doc->gl_snap_basex)/doc->gl_snap_width)
    *doc->gl_snap_width+doc->gl_snap_basex;
  *y=rint(((*y)-doc->gl_snap_basey)/doc->gl_snap_height)
    *doc->gl_snap_height+doc->gl_snap_basey;

  misc_rotate_point(-doc->gl_snap_angle, doc->gl_snap_basex,doc->gl_snap_basey,x,y);
}

void make_ortho(DOCUMENT *doc,  gdouble x1, gdouble y1, gdouble *x2, gdouble *y2)
{
  gdouble rot;

  rot=doc->gl_snap_angle;

  misc_rotate_point(rot, doc->gl_snap_basex,doc->gl_snap_basey,&x1,&y1);

  misc_rotate_point(rot, doc->gl_snap_basex,doc->gl_snap_basey,x2,y2);

  if( ABS(*x2-x1) > ABS(*y2-y1) )
    *y2=y1;
  else
    *x2=x1;

  misc_rotate_point(-rot, doc->gl_snap_basex,doc->gl_snap_basey,x2,y2);
}

void make_ortho_snap(DOCUMENT *doc, gdouble ox, gdouble oy,
		     gdouble *nx, gdouble *ny)
{
  if(doc->gl_snap)
    make_snap(doc,nx,ny);

  if(doc->gl_ortho)
    make_ortho(doc,ox,oy,nx,ny);

}

static void Centroid3( NODE *p1, NODE *p2, NODE *p3, POINT *c)
     /* Returns three times the centroid.  The factor of 3 is
        left in to permit division to be avoided until later. */
{
  c->x = p1->x + p2->x + p3->x;
  c->y = p1->y + p2->y + p3->y;
  return;
}

static gdouble Area2(NODE *a, NODE *b, NODE *c )
     /* Returns twice the signed area of the triangle determined by a,b,c,
        positive if a,b,c are oriented ccw, and negative if cw. */
{
  return (b->x - a->x)*(c->y - a->y) - (c->x - a->x)*(b->y - a->y);
}

void misc_calc_centroid(GList *node_list, POINT *p)
     /* Calculates the centroid of a polyline.
	Returns the centroid point in p */
{
  gint i, n;
  gdouble A2, Areasum2 = 0;        /* Partial area sum */    
  POINT Cent3;

  p->x=0;
  p->y=0;
        
  n=g_list_length(node_list);

  if(n==1)
    {
      p->x=((NODE *)node_list->data)->x;
      p->y=((NODE *)node_list->data)->y;
    }
  else if(n==2)
    {
      p->x=((NODE *)node_list->data)->x+((NODE *)node_list->next->data)->x;
      p->y=((NODE *)node_list->data)->y+((NODE *)node_list->next->data)->y;
      p->x/=2;
      p->y/=2;
    }
  else if(n>2)
    {
      for(i=1; i < n-1; i++) 
	{
	  Centroid3((NODE *)g_list_first(node_list)->data, 
		    (NODE *)g_list_nth(node_list, i)->data, 		 
		    (NODE *)g_list_nth(node_list, i+1)->data, 		 
		    &Cent3);
	  A2=Area2((NODE *)g_list_first(node_list)->data, 
		   (NODE *)g_list_nth(node_list, i)->data, 		 
		   (NODE *)g_list_nth(node_list, i+1)->data);
	  p->x += A2 * Cent3.x;
	  p->y += A2 * Cent3.y;
	  Areasum2 += A2;
	}
      
      p->x/=3*Areasum2;
      p->y/=3*Areasum2;
    }
}

void set_cursortype(DOCUMENT *doc, gint cursortype)
{
  if(cursortype==-1)
    gdk_window_set_cursor(doc->widgets.draw->window,NULL);
  else
    gdk_window_set_cursor(doc->widgets.draw->window,mousecursor[cursortype]);
}

void set_cursor(DOCUMENT *doc)
{
  if(doc==NULL)
    doc=get_document();

  if(doc==NULL)
    return;

  if(doc->use_digitizer)
    set_cursortype(doc,-1);  /* Should be hide cursor */
  else if(gl_osnap!=OSNAP_NONE && gl_tool!=TOOL_IDLE)
    set_cursortype(doc,2);
  else
    {
      switch(gl_tool)
	{
	case TOOL_IDLE:
	  set_cursortype(doc,-1);
	  break;
	case TOOL_ZOOM:
	  if(gl_stage==2 || gl_stage==3)	   
	    set_cursortype(doc,6);	    
	  else
	    set_cursortype(doc,0);
	  break;
	case TOOL_BREAK:
	case TOOL_OFFSET:
	  if(gl_stage==0)
	    set_cursortype(doc,2);
	  else
	    set_cursortype(doc,0);
	  break;
	case TOOL_ETEXT:
	case TOOL_FILLET:
	case TOOL_CHAMFER:
	case TOOL_SELECT:
	  set_cursortype(doc,2);
	  break;
	case TOOL_ARRAY:
	  if(gl_stage==9)
	    set_cursortype(doc,2);
	  else
	    set_cursortype(doc,0);
	  break;
	default:
	  set_cursortype(doc,0);
	  break;
	}
    }
}

void tool_idle(DOCUMENT *doc)
{
  gl_stage=0;
  gl_tool=TOOL_IDLE;
  work_document=NULL;
  set_prompt();
  set_cursor(doc);
  UpdateCoordsLabel();
}

gint intersect_box(gdouble *box1,gdouble *box2)
{
  return !(box1[2]<box2[0] || box1[3]<box2[1] || 
	   box2[2]<box1[0] || box2[3]<box1[1]);
}

gint foreach_object(DOCUMENT *doc, void (*f)(DOCUMENT *,LAYER *,OBJECT *))
{
  GList *layer;
  GList *object;
  GList *temp;

  if(doc==NULL)
    return FALSE;

  layer=g_list_first(doc->layer_list);

  while(layer!=NULL){
    object=((LAYER *)layer->data)->object_list;
    while(object!=NULL){

      temp=g_list_next(object);

      f(doc, (LAYER *)layer->data, (OBJECT *)object->data);

      object=temp;
    }

    layer=g_list_next(layer);
  }
  return TRUE;
}

void misc_setup_black_line(DOCUMENT *doc)
{
  gdk_gc_set_foreground(doc->xor_gc, &doc->white);
  gdk_gc_set_line_attributes(doc->xor_gc,0,
			     GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_ROUND);
}

static void misc_setup_linestyle(DOCUMENT *doc, GdkGC *gc, OBJECT *obj)
     /* Function for setting a linestyle of an object */
{
    /* Object of type dimension should always use a solid line and the thinnest possible */
  if(obj->type==TYPE_DIMENSION)
    gdk_gc_set_line_attributes(gc,0,
			       GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
  else if(obj->linestyle->type==LINE_SOLID)
    gdk_gc_set_line_attributes(gc,WLDtoSCR(doc, obj->linewidth),
			       GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
  else
    {
      gint i, value;

      for(i=0;i<obj->linestyle->dash_no;i++)
	{
	  /* Get length of line segment. At least 1 pixel long */
	  value=MAX(1,WLDtoSCR(doc, ABS(obj->linestyle->dash_dlist[i]*obj->linestyle->dash_length)));

	  obj->linestyle->dash_list[i]=value;
	}

      gdk_gc_set_dashes(gc, obj->linestyle->dash_offset, obj->linestyle->dash_list, obj->linestyle->dash_no);
      gdk_gc_set_line_attributes(gc,WLDtoSCR(doc, obj->linewidth),
				 GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
    }
}

GdkGC *misc_setup_gc_object_xor(DOCUMENT *doc, OBJECT *obj)
{
  GdkGC *gc;

  gc=doc->xor_gc;

  gdk_gc_set_foreground(gc, &obj->colour->xor_colour);
  
  misc_setup_linestyle(doc,gc,obj);

  return gc;
}

GdkGC *misc_setup_gc_object(DOCUMENT *doc, OBJECT *obj)
{
  GdkGC *gc;

  /* Which gc should we use */
  switch(obj->selected)
    {
    case OBJECT_SEL:
      gc=doc->select_gc;
      if((obj->colour->colour.red+obj->colour->colour.green+obj->colour->colour.blue)<GFLOAT_SENSITIVE)
	gdk_gc_set_foreground(gc, &doc->green);
      else
	gdk_gc_set_foreground(gc, &obj->colour->xor_colour);
      break;
    case OBJECT_ERASE:
      gc=doc->erase_gc;
      gdk_gc_set_foreground(gc, &doc->white);
      break;      
    case OBJECT_NOT_SEL:
    default:
      gc=doc->replace_gc;
      gdk_gc_set_foreground(gc, &obj->colour->colour);
      break;
    }

  misc_setup_linestyle(doc,gc,obj);

  return gc;
}

GdkGC *misc_setup_gc(DOCUMENT *doc)
{
  GdkGC *gc;

  gc=doc->xor_gc;

  /* Set colour */
  gdk_gc_set_foreground(gc, &doc->current_colour->xor_colour);
  
  /* Set dashes */
  if(doc->current_linestyle->type==LINE_SOLID)
    {
      gdk_gc_set_line_attributes(gc,WLDtoSCR(doc, doc->current_linewidth),
				 GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
    }
  else
    {
      gint i;

      for(i=0;i<doc->current_linestyle->dash_no;i++)
	doc->current_linestyle->dash_list[i]=MAX(1,WLDtoSCR(doc,
						      doc->current_linestyle->dash_dlist[i]*doc->current_linestyle->dash_length));

      gdk_gc_set_dashes(gc, doc->current_linestyle->dash_offset, doc->current_linestyle->dash_list, 
			doc->current_linestyle->dash_no);
      gdk_gc_set_line_attributes(gc,WLDtoSCR(doc, doc->current_linewidth),
				 GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
    }

  return gc;
}

void obj_set_attributes(DOCUMENT *doc, OBJECT *obj, gchar *name, gint type_colour, COLOUR *colour, 
			gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, gdouble linewidth)
{
  static gint obj_counter=0;

  /* A newly created object is not selected */
  obj->selected=OBJECT_NOT_SEL;  

  /* Set name */
  if(name==NULL)
    {
      obj->name=g_strdup_printf( _("Object %d"), obj_counter);
      obj_counter++;
    }
  else
    obj->name=g_strdup_printf("%s",name);
  
  /* Make colour */
  obj->type_colour=type_colour;
  if(obj->type_colour==BYLAYER || obj->type_colour==BYBLOCK)
    obj->colour=obj->layer->colour;
  else 
    {
      obj->colour=g_new(COLOUR,1);
      memcpy(obj->colour,colour, sizeof(COLOUR));
    }
  
  /* Make linestyle */
  obj->type_linestyle=type_linestyle;
  if(type_linestyle==BYLAYER || obj->type_linestyle==BYBLOCK)
    obj->linestyle=obj->layer->linestyle;
  else
    obj->linestyle=linestyle;
    
  /* Make linewidth */
  obj->type_linewidth=type_linewidth;
  if(type_linewidth==BYLAYER || obj->type_linewidth==BYBLOCK)
    obj->linewidth=obj->layer->linewidth;
  else
    obj->linewidth=linewidth;
}
 
void set_current_attributes(DOCUMENT *doc, OBJECT *obj)
{
  /* Make colour */
  obj->type_colour=doc->type_colour;
  if((obj->type_colour==BYLAYER) || (obj->type_colour==BYBLOCK))
    obj->colour=obj->layer->colour;
  else 
    {
      obj->colour=(COLOUR *)g_malloc(sizeof(COLOUR));
      memcpy(obj->colour,doc->current_colour, sizeof(COLOUR));
    }

  /* Make linestyle */
  obj->type_linestyle=doc->type_linestyle;
  if(doc->type_linestyle==BYLAYER || doc->type_linestyle==BYBLOCK)
    obj->linestyle=obj->layer->linestyle;
  else
    obj->linestyle=doc->current_linestyle;
  
  /* Make linewidth */
  obj->type_linewidth=doc->type_linewidth;
  if(doc->type_linewidth==BYLAYER || doc->type_linewidth==BYBLOCK)
    obj->linewidth=obj->layer->linewidth;
  else
    obj->linewidth=doc->current_linewidth;
}

DOCUMENT *misc_object_get_document(void)
{
  DOCUMENT *doc;
  gint ret;
  gchar *tmp;

  doc=get_document();

  if(doc!=NULL)
     return doc;

  ret=alert(ICON_INFO, _("Message"), _("New"), _("Open"), _("Cancel"),
	    _("No drawing found.\nEither start a new drawing or open a old."));

  switch(ret)
    {
    case 1:  /* New */
      doc=new_draw();
      break;
    case 2:  /* Open */
      tmp=dia_fileselector( _("Open drawing"), "");

      if(tmp!=NULL)
	doc=file_open(tmp);
      else
	{
	  doc=NULL;
	  set_prompt();
	}
      break;
    case 0:  /* Window cancel */
    case 3:  /* Cancel */
      doc=NULL;     
      set_prompt();
      break;
    }

  return doc;
}

DOCUMENT *misc_tool_get_document(void)
{
  DOCUMENT *doc;
  gint ret;
  gchar *tmp;

  doc=get_document();

  if(doc!=NULL)
     return doc;

  ret=alert(ICON_INFO, _("Message"), _("Open"), _("Cancel"), NULL, _("No drawing found.\nYou must open a previous drawing."));

  switch(ret)
    {
    case 1:  /* Open */
      tmp=dia_fileselector( _("Open drawing"), "");

      if(tmp!=NULL)
	doc=file_open(tmp);
      else
	{
	  doc=NULL;
	  set_prompt();
	}
      break;
    case 0:  /* Window cancel */
    case 2:  /* Cancel */
      doc=NULL;     
      set_prompt();
      break;
    }

  return doc;
}

void misc_rotate_point(gdouble angle, gdouble origo_x, gdouble origo_y, gdouble *x, gdouble *y)
{
  gdouble tx;
  
  *x=(*x)-origo_x;
  *y=(*y)-origo_y;

  tx=(*x);

  *x=tx*cos(angle)-(*y)*sin(angle);
  *y=tx*sin(angle)+(*y)*cos(angle);  

  *x=(*x)+origo_x;
  *y=(*y)+origo_y;
}

void misc_mirror_point(gdouble angle, gdouble origo_x, gdouble origo_y,
		       gdouble *x, gdouble *y)
{
  gdouble tx,ty;

  tx=*x-origo_x;
  ty=*y-origo_y;

  misc_rotate_point(-angle,0,0,&tx,&ty);

  ty=-ty;

  misc_rotate_point(angle,0,0,&tx,&ty);

  *x=tx+origo_x;
  *y=ty+origo_y;
}

static gdouble bx1,by1,bx2,by2;
static gint first;

gint misc_find_bound_box_of_selection(DOCUMENT *doc, gdouble *x1,gdouble *y1, gdouble *x2, gdouble *y2)
     /* Calculate boundbox for all selected objects */
{
  gboolean first=TRUE;
  GList *select_list;
  OBJECT *object;

  select_list=g_list_first(doc->selection_list);
  
  while(select_list!=NULL)
    {
      object=(OBJECT *)select_list->data;
      if(!(object->type==TYPE_RLINE || object->type==TYPE_CLINE))
	{
	  if(first)
	    {
	      bx1=object->bounding_box[0];
	      by1=object->bounding_box[1];
	      bx2=object->bounding_box[2];
	      by2=object->bounding_box[3];
	      first=FALSE;
	    }
	  else
	    {
	      bx1=MIN(bx1,object->bounding_box[0]);
	      by1=MIN(by1,object->bounding_box[1]);
	      bx2=MAX(bx2,object->bounding_box[2]);
	      by2=MAX(by2,object->bounding_box[3]);
	    }
	}

      select_list=g_list_next(select_list);      
    }

  if(x1!=NULL)
    *x1=bx1;
  if(x2!=NULL)
    *x2=bx2;
  if(y1!=NULL)
    *y1=by1;
  if(y2!=NULL)
    *y2=by2;

  return !first;
}

static void calc_bound_box(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  if(object->type==TYPE_RLINE || object->type==TYPE_CLINE)
    return;

  if(first)
    {
      bx1=MIN(bx1, object->bounding_box[0]);
      by1=MIN(by1, object->bounding_box[1]);
      bx2=MAX(bx2, object->bounding_box[2]);
      by2=MAX(by2, object->bounding_box[3]);
    }
  else
    {
      bx1=object->bounding_box[0];
      by1=object->bounding_box[1];
      bx2=object->bounding_box[2];
      by2=object->bounding_box[3];
      first=TRUE;
    }
}

gint misc_get_size_of_drawing(DOCUMENT *doc, gdouble *x1,gdouble *y1, gdouble *x2, gdouble *y2)
{
  first=FALSE;

  /* Create drawing box */
  foreach_object(doc, calc_bound_box);
  
 if(x1!=NULL)
    *x1=bx1;
  if(x2!=NULL)
    *x2=bx2;
  if(y1!=NULL)
    *y1=by1;
  if(y2!=NULL)
    *y2=by2;

  return first;
}

gint misc_find_midpoint_of_selection(DOCUMENT *doc, gdouble *x, gdouble *y)
{
  gdouble x1,y1,x2,y2;
  gint ret;

  ret=misc_find_bound_box_of_selection(doc,&x1,&y1,&x2,&y2);

  *x=ABS(x1-x2)/2+x1;
  *y=ABS(y1-y2)/2+y1;
  return ret;
}

gint misc_point_on_line(gdouble x, gdouble y, gdouble Ax, gdouble Ay, gdouble Bx, gdouble By, gdouble d)
{
  gdouble L;
  gdouble s,r;

  L=sqrt( (Bx-Ax)*(Bx-Ax) + (By-Ay)*(By-Ay) );

  r=((Ay-y)*(Ay-By)-(Ax-x)*(Bx-Ax))/(L*L);

  if((r>=(-GFLOAT_SENSITIVE)) && (r<=(1+GFLOAT_SENSITIVE))){
    s=((Ay-y)*(Bx-Ax)-(Ax-x)*(By-Ay))/(L*L);
    return( ABS(s*L)<d);
  }

  return FALSE;
}

gint misc_point_on_arc(gdouble x, gdouble y, OBJECT_ARC *arc, gdouble d)
{
  gdouble xc, yc, radius, r;
  gdouble tangle;


  xc=arc->centrum.x;
  yc=arc->centrum.y;
  radius=arc->radius;

  r=sqrt((xc-x)*(xc-x)+(yc-y)*(yc-y));

  if( (r>radius-d) && (r<radius+d) )
    {
      tangle=get_angle(xc,yc,x,y);
      
      if(arc->end_angle>arc->start_angle)
	{
	  if( (tangle>arc->start_angle) && (tangle<arc->end_angle))
	    return TRUE;
	}
      else
	{
	  if( (tangle>arc->start_angle) || (tangle<arc->end_angle))
	    return TRUE;
	}
    }
  
  return FALSE;
}

void misc_transform_point(TRANSFORM *transform,
			  gdouble org_x, gdouble org_y, 
			  gdouble *new_x, gdouble *new_y)
{
  *new_x=org_x;
  *new_y=org_y;

  if(transform!=NULL)
    {
      if(transform->type & OBJ_SCALE)
	{
	  *new_x=(org_x-transform->cen_x)*transform->scale_x+transform->cen_x;
	  *new_y=(org_y-transform->cen_y)*transform->scale_y+transform->cen_y;
	}
      
      if(transform->type & OBJ_ROTATE)
	misc_rotate_point(transform->rot_angle,transform->cen_x,transform->cen_y, new_x,new_y);	  
      
      if(transform->type & OBJ_TRANSLATE)
	{
	  (*new_x)+=transform->trans_x;
	  (*new_y)+=transform->trans_y;
	}
    }
}

void misc_transform_point_select(TRANSFORM *transform,
				 POINT *p, 
				 gdouble *new_x, gdouble *new_y)
{
  *new_x=p->x;
  *new_y=p->y;

  if(transform!=NULL && p->selected)
    {
      if(transform->type & OBJ_SCALE)
	{
	  *new_x=(p->x-transform->cen_x)*transform->scale_x+transform->cen_x;
	  *new_y=(p->y-transform->cen_y)*transform->scale_y+transform->cen_y;
	}
      
      if(transform->type & OBJ_ROTATE)
	misc_rotate_point(transform->rot_angle,transform->cen_x,transform->cen_y, new_x,new_y);	  
      
      if(transform->type & OBJ_TRANSLATE)
	{
	  (*new_x)+=transform->trans_x;
	  (*new_y)+=transform->trans_y;
	}
    }
}

void misc_transform_point_no_rotate(TRANSFORM *transform,
				    gdouble org_x, gdouble org_y, 
				    gdouble *new_x, gdouble *new_y)
{
  *new_x=org_x;
  *new_y=org_y;

  if(transform!=NULL)
    {
      if(transform->type & OBJ_SCALE)
	{
	  *new_x=(org_x-transform->cen_x)*transform->scale_x+transform->cen_x;
	  *new_y=(org_y-transform->cen_y)*transform->scale_y+transform->cen_y;
	}
      
      if(transform->type & OBJ_TRANSLATE)
	{
	  (*new_x)+=transform->trans_x;
	  (*new_y)+=transform->trans_y;
	}
    }
}

void misc_project_point_on_line(gdouble Ax, gdouble Ay, gdouble Bx, gdouble By,
				gdouble x, gdouble y, gdouble *px, gdouble *py)
{
  gdouble L;
  gdouble r;

  L=sqrt( (Bx-Ax)*(Bx-Ax) + (By-Ay)*(By-Ay) );

  r=((Ay-y)*(Ay-By)-(Ax-x)*(Bx-Ax))/(L*L);

  *px = Ax + r*(Bx-Ax);
  *py = Ay + r*(By-Ay);  
}
