/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "gtk_mtext.h"
#include "calc.h"
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "redraw.h"
#include "button.h"
#include "layer.h"
#include "select.h"
#include "linestyle.h"
#include "textstyle.h"
#include "xor.h"
#include "color.h"
#include "misc.h"
#include "grid.h"
#include "parse.h"
#include "menuitem.h"
#include "pop_colour.h"
#include "dia_layer.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "digitizer.h"
#include "dimension.h"
#include "tool_zoom.h"
#include "graphics/save.xpm"
#include "graphics/snap.xpm"
#include "graphics/grid.xpm"
#include "graphics/ortho.xpm"
#include "graphics/digit.xpm"
#include "struct/config.h"
#include "translation.h"

/* --  D E F I N E S  -- */
#define EVENT_METHOD(i,x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x


/* --  G L O B A L   V A R I A B L E S  -- */
extern CONFIG configuration;  /* Defined in main.c */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
static gint window_linewidth(GtkWidget *widget, DOCUMENT *doc)
{
  gchar *string;

  string=gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry));

  if(!strcmp( _("By Layer"), string))
    {
      doc->type_linewidth=BYLAYER;
      doc->current_linewidth=doc->current_layer->linewidth;
    }
  else if(!strcmp( _("By Block"), string))
    {
      doc->type_linewidth=BYBLOCK;
      doc->current_linewidth=doc->current_layer->linewidth;
    }
  else
    {
      doc->type_linewidth=USERDEFINED;
      doc->current_linewidth=g_strtod(string, NULL);
    }

  string=g_strdup_printf("%.1f", doc->current_linewidth);
  if(doc->type_linewidth==BYLAYER)
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry), _("By Layer"));
  else if(doc->type_linewidth==BYBLOCK)
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry), _("By Block"));
  else
      gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry), string);

  g_free(string);

  /* Unfocus entry */
  GTK_WIDGET_UNSET_FLAGS(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry, GTK_HAS_FOCUS);

  return TRUE;
}

static gint window_unfocuslinewidth(GtkWidget *widget, GdkEventFocus *event, DOCUMENT *doc)
{
  window_linewidth(widget, doc);
  return TRUE;
}

gint window_close(GtkWidget *widget, GdkEvent *event, DOCUMENT *doc)
{ /* This routine is also used by menu_close */
  gint ret;
  
  if(!doc->saved)
    {
      ret=alert(ICON_QUESTION, _("Close drawing"), _("Yes"), _("No"), _("Cancel"), 
		_("This drawing has been modified.\nDo you want to save it before closing it?"));

      switch(ret)
	{
	case 3:  /* Cancel */
	case 0:  /* Window close==Cancel */
	  return TRUE;
	case 1:  /* Save */
	  menu_save(widget,doc);
	  break;
	}
    }

  /* Close background redraw-functions */
  if(doc->redraw_tag!=-1)
    gtk_idle_remove(doc->redraw_tag);
  
  /* A LOT of memory leaks here! Delete document here! */
  gtk_widget_hide(doc->widgets.window);

  /* This line creates a segmentation fault */
  //  gtk_widget_destroy(doc->widgets.window);

  if(work_document==doc)
    work_document=NULL;

  focus_document=NULL;

  /* Remove document from the document list */
  document_list=g_list_remove(document_list,(gpointer)doc);

  g_free(doc);

  return TRUE; 
}

static gint window_configure(GtkWidget *widget, GdkEvent *event, DOCUMENT *doc)
{
  update_rulers(doc);
  return TRUE; 
}

static gint window_button(GtkWidget *widget, GdkEventButton *event, DOCUMENT *doc)
{
  int x, y;
  GdkModifierType state;

  x = event->x;
  y = event->y;
  state = event->state;

  if(!doc->use_digitizer)
    mousebutton(doc, event->button, SCRtoWLD_x(doc,x), SCRtoWLD_y(doc,y));

  return TRUE; 
}

static gint window_motion(GtkWidget *widget, GdkEventMotion *event, DOCUMENT *doc)
{
  int x, y;
  GdkModifierType state;

  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }
  
  if(!doc->use_digitizer)
    mousemove(doc, SCRtoWLD_x(doc,x), SCRtoWLD_y(doc,y));

  return TRUE;
}

static gint window_enter(GtkWidget *widget, GdkEvent *event, DOCUMENT *doc)
{
  focus_document=doc;

  set_cursor(doc);
  return TRUE;
}

static gint window_leave(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  remove_old();
  return TRUE;
}

static gint window_redraw(GtkWidget *widget, GdkEvent *event, gpointer doc)
{
  redraw_window((DOCUMENT *)doc);
  return TRUE;
}

static gint window_keyboard( GtkWidget *widget, GdkEventKey *event, gpointer doc)
{
  /* If linewidth has focus dont send keypress to command window! */
  if(GTK_WIDGET_HAS_FOCUS(GTK_COMBO(((DOCUMENT *)doc)->widgets.toolbar_linewidthmenu)->entry))
    return TRUE;
 
  comm_keyboard((DOCUMENT *)doc,event);

  return TRUE;
}

static gint window_toggle( GtkWidget *widget, DOCUMENT *doc)
{
  doc->gl_ortho=GTK_TOGGLE_BUTTON(doc->widgets.toolbar_ortho)->active;

  if(GTK_TOGGLE_BUTTON(doc->widgets.toolbar_grid)->active && !doc->gl_grid)
    {
      /* We have turned on the grid. Draw it! */
      doc->gl_grid=TRUE;
      draw_grid(doc);

    }
  else if(!GTK_TOGGLE_BUTTON(doc->widgets.toolbar_grid)->active && doc->gl_grid)
    {
      doc->gl_grid=FALSE;
      draw_grid(doc);
    }

  doc->gl_snap=GTK_TOGGLE_BUTTON(doc->widgets.toolbar_snap)->active;

 if(GTK_TOGGLE_BUTTON(doc->widgets.toolbar_digit)->active && !doc->use_digitizer)
    {
      if(digitizer_grab(doc))
	doc->use_digitizer=TRUE;
      else
	{
	  message(ICON_KILL, _("Cannot init the digitizer."));
	  /* Should perform a sort of redrawing the digit-button here also */
	  GTK_TOGGLE_BUTTON(doc->widgets.toolbar_digit)->active=FALSE;
	}
    }
  else if(!GTK_TOGGLE_BUTTON(doc->widgets.toolbar_digit)->active && doc->use_digitizer)
    {
      digitizer_ungrab(doc);
      doc->use_digitizer=FALSE;
    }

  return TRUE;
}

DOCUMENT *make_new_drawing(gchar *title, gdouble width, gdouble height)
{
  GdkBitmap *tmpmask;
  GdkPixmap *tmpicon;
  DOCUMENT *doc;

  doc=(DOCUMENT *)g_malloc(sizeof(DOCUMENT));

  /* Add new document to list */
  document_list=g_list_append(document_list,doc);

  doc->width=width;
  doc->height=height;
  doc->title=NULL;
  doc->filename=NULL;
  doc->saved=TRUE;
  doc->xpan=0;
  doc->ypan=0;
  doc->zoom=1.0;
  doc->gl_ortho=FALSE;
  doc->gl_snap=FALSE;
  doc->gl_grid=FALSE;
  doc->gl_iso_view=FALSE;
  doc->gl_iso_type=ISOMETRIC_LEFT;
  doc->current_layer=NULL;
  doc->layer_list=NULL;
  doc->selection_list=NULL;
  doc->current_colour=NULL;
  doc->type_colour=BYLAYER;
  doc->type_linewidth=BYLAYER;
  doc->current_linewidth=0;
  doc->type_linestyle=BYLAYER;
  doc->current_linestyle=NULL;
  doc->linestyle_list=NULL;
  doc->textstyle_list=NULL;
  doc->dimensionstyle_list=NULL;
  doc->variable_list=NULL;
  doc->zoom_previous=NULL;
  doc->cont_exist=FALSE;
  doc->working_polyline=NULL;
  doc->undo_list=NULL;
  doc->redo_list=NULL;
  doc->block_list=NULL;

  doc->point_style=(POINT_CIRCLE|POINT_MARK);
  doc->point_type=POINT_ABSOLUTE;
  doc->point_size=10;

  doc->gl_snap_basex=0;
  doc->gl_snap_basey=0;
  doc->gl_snap_width=30;
  doc->gl_snap_height=30;
  doc->gl_snap_angle=0;
  doc->gl_grid_x=1;
  doc->gl_grid_y=1;

  doc->use_digitizer=FALSE;
  doc->redraw_tag=-1;

  /* Make nice title */
  if(title!=NULL)
    doc->filename=g_strdup(title);

  /* Create main widget */
  doc->widgets.window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_policy(GTK_WINDOW(doc->widgets.window), TRUE, TRUE, TRUE);
  gtk_window_set_default_size(GTK_WINDOW( doc->widgets.window),400,400);
  make_window_title(doc);
  gtk_signal_connect(GTK_OBJECT(doc->widgets.window),"delete_event", 
                     GTK_SIGNAL_FUNC(window_close),(gpointer)doc);
  gtk_signal_connect(GTK_OBJECT(doc->widgets.window),"key_press_event",
		     GTK_SIGNAL_FUNC(window_keyboard), (gpointer)doc);

  gtk_widget_realize(doc->widgets.window);

  /* create vbox */
  doc->widgets.vbox=gtk_vbox_new(FALSE,0);
  gtk_widget_show(doc->widgets.vbox);

  /* Create table */
  doc->widgets.table = gtk_table_new (3, 3, FALSE);
  gtk_table_set_col_spacing (GTK_TABLE (doc->widgets.table), 0, 1);
  gtk_table_set_col_spacing (GTK_TABLE (doc->widgets.table), 1, 2);
  gtk_table_set_row_spacing (GTK_TABLE (doc->widgets.table), 0, 1);
  gtk_table_set_row_spacing (GTK_TABLE (doc->widgets.table), 1, 2);
  gtk_container_border_width (GTK_CONTAINER (doc->widgets.table), 2);
  gtk_widget_show(doc->widgets.table);

  /* Create origion */
  doc->widgets.origin = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (doc->widgets.origin), GTK_SHADOW_OUT);
  gtk_widget_show(doc->widgets.origin);

  /* Create toolbar */
  doc->widgets.toolbar=gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_show(doc->widgets.toolbar);

  gtk_toolbar_append_space(GTK_TOOLBAR(doc->widgets.toolbar));
  
  tmpicon = gdk_pixmap_create_from_xpm_d ( doc->widgets.window->window, &tmpmask,
					   &doc->widgets.window->style->white, save_icon_xpm );
  doc->widgets.toolbar_saveicon=gtk_pixmap_new(tmpicon, tmpmask);

  doc->widgets.toolbar_save=gtk_toolbar_append_item(GTK_TOOLBAR(doc->widgets.toolbar),
						    _("Save"), _("Save drawing"), NULL,
						    doc->widgets.toolbar_saveicon,
						    GTK_SIGNAL_FUNC(menu_save),
						    (gpointer)doc);

  gtk_toolbar_append_space(GTK_TOOLBAR(doc->widgets.toolbar));

  tmpicon = gdk_pixmap_create_from_xpm_d ( doc->widgets.window->window, &tmpmask,
					   &doc->widgets.window->style->white, snap_xpm );
  doc->widgets.toolbar_snapicon=gtk_pixmap_new(tmpicon, tmpmask);

  doc->widgets.toolbar_snap=gtk_toolbar_append_element(GTK_TOOLBAR(doc->widgets.toolbar),
						       GTK_TOOLBAR_CHILD_TOGGLEBUTTON,NULL,
						       _("Snap"), _("Turn On/Off Snap"), NULL,
						       doc->widgets.toolbar_snapicon,
						       GTK_SIGNAL_FUNC(window_toggle),
						       (gpointer)doc);

  tmpicon = gdk_pixmap_create_from_xpm_d ( doc->widgets.window->window, &tmpmask,
					   &doc->widgets.window->style->white, grid_xpm );
  doc->widgets.toolbar_gridicon=gtk_pixmap_new(tmpicon, tmpmask);

  doc->widgets.toolbar_grid=gtk_toolbar_append_element(GTK_TOOLBAR(doc->widgets.toolbar),
						       GTK_TOOLBAR_CHILD_TOGGLEBUTTON,NULL,
						       _("Grid"), _("Turn On/Off Grid"), NULL,
						       doc->widgets.toolbar_gridicon,
						       GTK_SIGNAL_FUNC(window_toggle),
						       (gpointer)doc);

  tmpicon = gdk_pixmap_create_from_xpm_d ( doc->widgets.window->window, &tmpmask,
					   &doc->widgets.window->style->white, ortho_xpm );
  doc->widgets.toolbar_orthoicon=gtk_pixmap_new(tmpicon, tmpmask);

  doc->widgets.toolbar_ortho=gtk_toolbar_append_element(GTK_TOOLBAR(doc->widgets.toolbar),
							GTK_TOOLBAR_CHILD_TOGGLEBUTTON,NULL,
							_("Ortho"), _("Turn On/Off Ortho"), NULL,
							doc->widgets.toolbar_orthoicon,
							GTK_SIGNAL_FUNC(window_toggle),
							(gpointer)doc);

  tmpicon = gdk_pixmap_create_from_xpm_d ( doc->widgets.window->window, &tmpmask,
					   &doc->widgets.window->style->white, digit_xpm );
  doc->widgets.toolbar_digiticon=gtk_pixmap_new(tmpicon, tmpmask);

  doc->widgets.toolbar_digit=gtk_toolbar_append_element(GTK_TOOLBAR(doc->widgets.toolbar),
							GTK_TOOLBAR_CHILD_TOGGLEBUTTON,NULL,
							_("Digitizer"), _("Turn On/Off Digitizer"), NULL,
							doc->widgets.toolbar_digiticon,
							GTK_SIGNAL_FUNC(window_toggle),
							(gpointer)doc);

  gtk_toolbar_append_space(GTK_TOOLBAR(doc->widgets.toolbar));
  
  /* Create layer menu */
  doc->widgets.toolbar_option=gtk_option_menu_new();
  gtk_widget_set_usize(GTK_WIDGET( doc->widgets.toolbar_option),150,24);
  gtk_widget_show(doc->widgets.toolbar_option);    

  doc->widgets.toolbar_layermenu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(doc->widgets.toolbar_option),
			   doc->widgets.toolbar_layermenu);

  gtk_toolbar_append_widget(GTK_TOOLBAR(doc->widgets.toolbar),doc->widgets.toolbar_option, 
			    _("Current layer"), NULL);

  gtk_toolbar_append_space(GTK_TOOLBAR(doc->widgets.toolbar));

  /* Create colour menu */
  doc->widgets.toolbar_colour=gtk_option_menu_new();
  gtk_widget_set_usize(GTK_WIDGET( doc->widgets.toolbar_colour),102,24);
  gtk_widget_show(doc->widgets.toolbar_colour);    

  doc->widgets.toolbar_colourmenu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(doc->widgets.toolbar_colour),
			   doc->widgets.toolbar_colourmenu);

  gtk_toolbar_append_widget(GTK_TOOLBAR(doc->widgets.toolbar),doc->widgets.toolbar_colour, 
			    _("Current colour"), NULL);

  gtk_toolbar_append_space(GTK_TOOLBAR(doc->widgets.toolbar));

  /* Create line style menu */
  doc->widgets.toolbar_linestyle=gtk_option_menu_new();
  gtk_widget_set_usize(GTK_WIDGET( doc->widgets.toolbar_linestyle),150,24);
  gtk_widget_show(doc->widgets.toolbar_linestyle);    

  doc->widgets.toolbar_linestylemenu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(doc->widgets.toolbar_linestyle),
			   doc->widgets.toolbar_linestylemenu);

  gtk_toolbar_append_widget(GTK_TOOLBAR(doc->widgets.toolbar),doc->widgets.toolbar_linestyle, 
			    _("Current linestyle"), NULL);

  gtk_toolbar_append_space(GTK_TOOLBAR(doc->widgets.toolbar));

  /* Create line width menu */
  doc->widgets.toolbar_linewidthmenu=gtk_combo_new();
  gtk_widget_set_usize(GTK_WIDGET( doc->widgets.toolbar_linewidthmenu),80,24);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry), _("By Layer"));

  gtk_combo_set_popdown_strings(GTK_COMBO(doc->widgets.toolbar_linewidthmenu), linewidth_glist);
  gtk_combo_disable_activate(GTK_COMBO(doc->widgets.toolbar_linewidthmenu));

  gtk_widget_show(doc->widgets.toolbar_linewidthmenu);

  gtk_toolbar_append_widget(GTK_TOOLBAR(doc->widgets.toolbar),doc->widgets.toolbar_linewidthmenu, 
			    _("Current linewidth"), NULL);

  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry), "activate",
                            GTK_SIGNAL_FUNC (window_linewidth), (gpointer)doc);

  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(doc->widgets.toolbar_linewidthmenu)->entry), "focus_out_event",
                            GTK_SIGNAL_FUNC (window_unfocuslinewidth), (gpointer)doc);

  gtk_toolbar_append_space(GTK_TOOLBAR(doc->widgets.toolbar));

  /* Create drawarea */
  doc->widgets.draw=gtk_drawing_area_new();
  gtk_widget_show(doc->widgets.draw);
  
  gtk_widget_set_events(GTK_WIDGET(doc->widgets.draw), GDK_POINTER_MOTION_MASK|
                        GDK_POINTER_MOTION_HINT_MASK|GDK_BUTTON_PRESS_MASK|
                        GDK_ENTER_NOTIFY_MASK|GDK_LEAVE_NOTIFY_MASK|
			GDK_KEY_PRESS_MASK);

  /* Attach signals to draw area */
  gtk_signal_connect(GTK_OBJECT(doc->widgets.draw),"button_press_event",
		     (GtkSignalFunc)window_button,(gpointer)doc);
  gtk_signal_connect(GTK_OBJECT(doc->widgets.draw),"expose_event",
		     GTK_SIGNAL_FUNC(window_redraw),(gpointer)doc);
  gtk_signal_connect(GTK_OBJECT(doc->widgets.draw),"motion_notify_event",
		     (GtkSignalFunc)window_motion,(gpointer)doc);
  gtk_signal_connect(GTK_OBJECT(doc->widgets.draw),"enter_notify_event",
		     (GtkSignalFunc)window_enter,(gpointer)doc);
  gtk_signal_connect(GTK_OBJECT(doc->widgets.draw),"leave_notify_event",
		     (GtkSignalFunc)window_leave,(gpointer)doc);
  gtk_signal_connect(GTK_OBJECT(doc->widgets.draw),"configure_event",
		     GTK_SIGNAL_FUNC(window_configure), (gpointer)doc);

  /*  The adjustment datums  */
  doc->widgets.hsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 100, 1, 1, 100));
  doc->widgets.vsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 100, 1, 1, 100));

  /* Create range widgets */
  doc->widgets.hsb = gtk_hscrollbar_new (doc->widgets.hsbdata);
  gtk_widget_show(doc->widgets.hsb);
  doc->widgets.vsb = gtk_vscrollbar_new (doc->widgets.vsbdata);
  gtk_widget_show(doc->widgets.vsb);

  /* Create rulers */
  doc->widgets.horiz_ruler=gtk_hruler_new();
  gtk_ruler_set_metric(GTK_RULER(doc->widgets.horiz_ruler),GTK_PIXELS);
  gtk_signal_connect_object(GTK_OBJECT(doc->widgets.draw),"motion_notify_event",
                            (GtkSignalFunc)EVENT_METHOD(doc->widgets.horiz_ruler, 
							motion_notify_event),
                            GTK_OBJECT(doc->widgets.horiz_ruler));
  gtk_widget_show(doc->widgets.horiz_ruler);

  doc->widgets.vert_ruler=gtk_vruler_new();
  gtk_ruler_set_metric(GTK_RULER(doc->widgets.vert_ruler),GTK_PIXELS);
  gtk_signal_connect_object(GTK_OBJECT(doc->widgets.draw),"motion_notify_event",
                            (GtkSignalFunc)EVENT_METHOD(doc->widgets.vert_ruler, 
							motion_notify_event),
                            GTK_OBJECT(doc->widgets.vert_ruler));
  gtk_widget_show(doc->widgets.vert_ruler);

  /* Attach rulers and scroll window to the table */
  gtk_table_attach (GTK_TABLE (doc->widgets.table), doc->widgets.origin, 0, 1, 0, 1,
                    GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach (GTK_TABLE (doc->widgets.table), doc->widgets.horiz_ruler, 1, 2, 0, 1,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach (GTK_TABLE (doc->widgets.table), doc->widgets.vert_ruler, 0, 1, 1, 2,
                    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_table_attach (GTK_TABLE (doc->widgets.table), doc->widgets.draw, 1, 2, 1, 2,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_table_attach (GTK_TABLE (doc->widgets.table), doc->widgets.hsb, 0, 2, 2, 3,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach (GTK_TABLE (doc->widgets.table), doc->widgets.vsb, 2, 3, 0, 2,
                    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  
  /* Attach table to window */
  gtk_container_add(GTK_CONTAINER(doc->widgets.window),doc->widgets.vbox);  
  gtk_box_pack_start(GTK_BOX(doc->widgets.vbox),doc->widgets.toolbar,FALSE,TRUE,4); 
  gtk_box_pack_start(GTK_BOX(doc->widgets.vbox),doc->widgets.table,TRUE,TRUE,0); 

  /* Show window */
  gtk_widget_show(doc->widgets.window);

  /* Create colors and GC */
  make_color(doc);  

  make_colour_popmenu(doc);
  gtk_option_menu_set_history(GTK_OPTION_MENU(doc->widgets.toolbar_colour),0);

  /* Update rulers */
  update_rulers(doc);

  return doc;
}

/* If anything is missing in the document, create some default values */
void new_make_default(DOCUMENT *doc)
{
  /* Make default linestyle */
  if(doc->linestyle_list==NULL)
    {
      /* Create default linestyles */
      doc->type_linestyle=BYLAYER;
      doc->current_linestyle=linestyle_new(doc, -1, _("Solid"), LINE_SOLID, 1, 0,NULL);      
          
      make_linestyle_popmenu(doc);
      
      gtk_option_menu_set_history(GTK_OPTION_MENU(doc->widgets.toolbar_linestyle),0);
    }

  /* Make default layer list */
  if(doc->layer_list==NULL)
    {
      /* Create default layer */      
      doc->current_layer=layer_new(doc,-1,doc->linestyle_list->data,0,0,0,0, _("Default Layer"), TRUE, TRUE);

      make_layer_popmenu(doc);
      gtk_option_menu_set_history(GTK_OPTION_MENU(doc->widgets.toolbar_option),0);
    }

  
  /* Make a standard textstyle */
  if(doc->textstyle_list==NULL)
    textstyle_new(doc,-1, _("Standard"),TEXTSTYLE_OCADIS, 0,0,0,1);

  /* Make a dimension style */
  if(doc->dimensionstyle_list==NULL)
    doc->current_dimensionstyle=dimstyle_new(doc, _("Standard"));      

}

DOCUMENT *new_draw(void)
{
  DOCUMENT *doc;

  doc=make_new_drawing(NULL, 100, 100);
  new_make_default(doc);

  doc->predefined_colour=1;
  doc->type_colour=BYLAYER;
  doc->current_colour=doc->current_layer->colour;

  return doc;
}









