/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "dia_comm.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "calc_arc.h"
#include "translation.h"

/* --  P R O T O T Y P E S  -- */
void obj_draw_arc(DOCUMENT *doc, OBJECT *arc);
void make_continue_end(DOCUMENT *doc, OBJECT_ARC *arc);

/* --  L O C A L   V A R I A B L E S  -- */
static gdouble ax,ay,bx,by,cx,cy;
static gint use_table=FALSE;
static gint table_nr, table_counter;
static gint first_time;

static gint table[11][3]={
  {0,1,2},   /* 3 Point */
  {0,3,5},   /* SCE */
  {0,3,6},   /* SCA */
  {0,3,7},   /* SCL */
  {0,8,14},  /* SEA */
  {0,8,10},  /* SED */
  {0,8,11},  /* SER */
  {3,4,5},   /* CSE */
  {3,4,6},   /* CSA */
  {3,4,7},   /* CSL */
  {15,15,15}    /* Continue */
};

/* --  P R O T O T Y P E S  -- */

/* --  F U N C T I O N S  -- */
void obj_set_prompt_arc(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Center/<Start point>: "));
      break;
    case 1:
      set_prompt_text( _("Center/End/<Second point>: "));
      break;
    case 15:
    case 8:
    case 2:
      set_prompt_text( _("End point: "));
      break;
    case 12:
    case 3:
      set_prompt_text( _("Center: "));
      break;
    case 4:
      set_prompt_text( _("Start point: "));
      break;
    case 13:
    case 5:
      set_prompt_text( _("Angle/Length of coord/<End point>: "));
      break;
    case 14:
    case 6:
      set_prompt_text( _("Included angle: "));
      break;
    case 7:
      set_prompt_text( _("Length of coord: "));
      break;
    case 9:
      set_prompt_text( _("Angle/Direction/Radius/<Center point>: "));
      break;
     case 10:
      set_prompt_text( _("Direction from start point: "));
      break;
    case 11:
      set_prompt_text( _("Radius: "));
      break;
      break;
    }
}

void obj_bound_arc(OBJECT *obj)
{
  gdouble sx,sy,ex,ey;
  gint i;

  sx=obj->data.arc.centrum.x+obj->data.arc.radius*cos(obj->data.arc.start_angle*M_PI/180);
  sy=obj->data.arc.centrum.y-obj->data.arc.radius*sin(obj->data.arc.start_angle*M_PI/180);

  ex=obj->data.arc.centrum.x+obj->data.arc.radius*cos(obj->data.arc.end_angle*M_PI/180);
  ey=obj->data.arc.centrum.y-obj->data.arc.radius*sin(obj->data.arc.end_angle*M_PI/180);

  obj->bounding_box[0]=MIN(sx,ex);
  obj->bounding_box[1]=MIN(sy,ey);
  obj->bounding_box[2]=MAX(sx,ex);
  obj->bounding_box[3]=MAX(sy,ey);

  /* Check each quadrant */
  for(i=90;i<=360;i+=90)
    {

      if(misc_within_angle(obj->data.arc.start_angle,obj->data.arc.end_angle,i))
	{
	  sx=obj->data.arc.centrum.x+obj->data.arc.radius*cos(i*M_PI/180);
	  sy=obj->data.arc.centrum.y-obj->data.arc.radius*sin(i*M_PI/180);

	  obj->bounding_box[0]=MIN(obj->bounding_box[0],sx);
	  obj->bounding_box[1]=MIN(obj->bounding_box[1],sy);
	  obj->bounding_box[2]=MAX(obj->bounding_box[2],sx);
	  obj->bounding_box[3]=MAX(obj->bounding_box[3],sy);
	}
    }
}


OBJECT *obj_add_arc(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		    gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		    gdouble linewidth, gdouble xc, gdouble yc, gdouble r, 
		    gdouble start_angle, gdouble end_angle)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_ARC;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */
  obj->data.arc.centrum.x=xc;
  obj->data.arc.centrum.y=yc;
  obj->data.arc.radius=r;
  obj->data.arc.start_angle=start_angle;
  obj->data.arc.end_angle=end_angle;

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_arc(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

OBJECT *obj_add_arc_spec(DOCUMENT *doc, OBJECT_ARC *arc)
{
  remove_old();
  tool_idle(doc);
  return obj_add_arc(doc,doc->current_layer, NULL, doc->type_colour, doc->current_colour, 
		     doc->type_linestyle, doc->current_linestyle,
		     doc->type_linewidth, doc->current_linewidth,
		     arc->centrum.x, arc->centrum.y, 
		     arc->radius, arc->start_angle,arc->end_angle);
}


void obj_set_tool_arc(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  use_table=FALSE;
  table_counter=0;

  remove_old();

  if(!command_from_keyboard)
    print( _("arc"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(!command_from_keyboard)
    {
      use_table=TRUE;
      table_nr=GPOINTER_TO_INT(data);
    }

  doc=misc_object_get_document();
  if(doc==NULL)
    return;

  if(use_table)
    gl_stage=table[table_nr][table_counter];
  else
    gl_stage=0;

  /* If continue arcs is selected */
  if(!doc->cont_exist && gl_stage==15)
    {
      print( _("\n  Not possible since no arc or line has been drawn."));
      set_prompt();
      return;
    }
  else if(doc->cont_exist && gl_stage==15)
    {
      gl_oldmx=bx=doc->cont_x;
      gl_oldmy=by=doc->cont_y;
      work_document=doc;
    }
  else
    work_document=NULL;

  gl_tool=TOOL_ARC;
  set_prompt();
  UpdateCoordsLabel();
  set_cursor(NULL);
  first_time=TRUE;
}

void obj_button_arc(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  gdouble sangle,eangle;
  gdouble d;
  OBJECT_ARC arc;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  if(first_time)
    {
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
    }
  else
    make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);

  switch(gl_stage)
    {
    case 0:   /* Center/<Start point> */
      gl_oldmx=ax=x;
      gl_oldmy=ay=y;
      if(use_table)
	gl_stage=table[table_nr][++table_counter];
      else
	gl_stage++;
      set_prompt();
      break;
    case 1:  /* Center/End/<Second point> */
      remove_old();
      gl_oldmx=bx=x;
      gl_oldmy=by=y;
      if(use_table)
	gl_stage=table[table_nr][++table_counter];
      else
	gl_stage++;
      set_prompt();
      break;
    case 2:  /* End point */
      calc_arc_3p(ax, ay, bx, by, x, y, &arc);
      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));

      /* Calculate for continue arc */
      doc->cont_exist=TRUE;
      doc->cont_x=x;
      doc->cont_y=y;
      d=dist(ax,ay,x,y);
      if(  ((ay-by)*(x-ax)-(ax-bx)*(y-ay))/(d*d)<0)	
	doc->cont_tan=get_angle(arc.centrum.x, arc.centrum.y, x,y)*M_PI/180;
      else
	doc->cont_tan=get_angle(x,y,arc.centrum.x, arc.centrum.y)*M_PI/180;	

      break;
    case 3:   /* Center */
      remove_old();
      gl_oldmx=cx=x;
      gl_oldmy=cy=y;
      if(use_table)
	gl_stage=table[table_nr][++table_counter];
      else
	gl_stage++;
      set_prompt();
      break;
    case 4:  /* Start Point */
      remove_old();
      gl_oldmx=ax=x;
      gl_oldmy=ay=y;
      if(use_table)
	gl_stage=table[table_nr][++table_counter];
      else
	gl_stage++;
      set_prompt();
      break;
    case 13:
    case 5:  /* Angle/Length/<End point>: */
      remove_old();
      calc_arc_CSE(cx, cy, ax, ay, x, y, &arc);
      make_continue_end(doc,&arc);

      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
      break;
    case 6:  /* Included angle: */
      remove_old();

      sangle=get_angle(cx,cy,ax,ay);
      eangle=get_angle(cx,cy,x,y);

      if(sangle<eangle)
	eangle-=sangle;
      else
	eangle=eangle+(360.0-sangle);

      calc_arc_CSA(cx, cy, ax, ay,eangle, &arc);
      make_continue_end(doc,&arc);
      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
      break;
    case 7:  /* Length of coord */
      remove_old();
      calc_arc_CSL(cx, cy, ax, ay, dist(x,y,ax,ay), &arc);
      make_continue_end(doc,&arc);
      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
      break;
    case 8:  /* End point */
      remove_old();
      gl_oldmx=ax;
      gl_oldmy=ay;
      bx=x;
      by=y;
      if(use_table)
	gl_stage=table[table_nr][++table_counter];
      else
	gl_stage++;
      set_prompt();
      break;
    case 10: /* Direction */
      remove_old();
      d=dist(ax,ay,bx,by);
      calc_arc_SED(ax, ay, bx, by, atan2(-(ay-y),ax-x),
		   (((ay-y)*(bx-ax)-(ax-x)*(by-ay))/(d*d))>0, &arc);
      make_continue_end(doc,&arc);
      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
      break;
    case 11: /* Radius */
      remove_old();
      d=dist(ax,ay,bx,by);
      calc_arc_SER(ax, ay, bx, by,((((ay-y)*(bx-ax)-(ax-x)*(by-ay))/(d*d))>0),
		   dist(x,y,ax,ay), &arc);      
      make_continue_end(doc,&arc);
      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
      break;
    case 12: /* Center 2 */
      remove_old();
      gl_oldmx=cx=x;
      gl_oldmy=cy=y;
      if(use_table)
	gl_stage=table[table_nr][++table_counter];
      else
	gl_stage++;
      set_prompt();
      break;
    case 14: /* Included angle */
      remove_old();
      sangle=get_angle(ax,ay,bx,by);
      eangle=get_angle(ax,ay,x,y);

      if(eangle>sangle)
	eangle=eangle-sangle;
      else
	eangle=eangle+(360.0-sangle);
      
      calc_arc_SEA(ax, ay, bx, by,360.0-eangle, &arc ); 

      make_continue_end(doc,&arc);
      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
      break;
    case 15: /* End point for continue arcs */
      remove_old();
      calc_arc_2p(doc->cont_tan, doc->cont_x, doc->cont_y, x,y, &bx, &by, &arc);
      arc.radius=arc.radius/2;
      
      obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));

      /* Calculate for continue arc */
      doc->cont_exist=TRUE;
      d=dist(doc->cont_x,doc->cont_x,x,y);
      if(  ((doc->cont_y-by)*(x-doc->cont_x)-(doc->cont_x-bx)*(y-doc->cont_y))/(d*d)<0)	
	doc->cont_tan=get_angle(arc.centrum.x, arc.centrum.y, x,y)*M_PI/180;
      else
	doc->cont_tan=get_angle(x,y,arc.centrum.x, arc.centrum.y)*M_PI/180;	

      doc->cont_x=x;
      doc->cont_y=y;

      break;
    }

  first_time=FALSE;
  UpdateCoordsLabel();
}

void make_continue_end(DOCUMENT *doc, OBJECT_ARC *arc)
{
  gdouble d;
  gdouble sx,sy;
  gdouble ex,ey;
  gdouble sangle,eangle;

  sangle=arc->start_angle;
  eangle=arc->end_angle;

  /* Find a point between start and end angle */
  if(sangle<eangle)
    sangle=(eangle-sangle)/2+sangle;
  else
    {
      sangle=(360-sangle+eangle)/2+sangle;
      if(sangle>360)
	sangle-=360;
    }

  bx=arc->centrum.x+arc->radius*cos(sangle*M_PI/180);
  by=arc->centrum.y-arc->radius*sin(sangle*M_PI/180); 

  sx=arc->centrum.x+arc->radius*cos(arc->start_angle*M_PI/180.0);
  sy=arc->centrum.y-arc->radius*sin(arc->start_angle*M_PI/180.0);
  ex=arc->centrum.x+arc->radius*cos(arc->end_angle*M_PI/180.0);
  ey=arc->centrum.y-arc->radius*sin(arc->end_angle*M_PI/180.0);
  

  d=dist(sx,sx,ex,ey);
  if(  ((sy-by)*(ex-sx)-(sx-bx)*(ey-sy))/(d*d)<0)	
    doc->cont_tan=get_angle(arc->centrum.x, arc->centrum.y, ex,ey)*M_PI/180;
  else
    doc->cont_tan=get_angle(ex,ey,arc->centrum.x, arc->centrum.y)*M_PI/180;	

  doc->cont_exist=TRUE;
  doc->cont_x=ex;
  doc->cont_y=ey;
}

gint obj_menu_arc(DOCUMENT *doc, gint menusel)
{
  remove_old();
  print("*Cancel*");
  tool_idle(doc);
  return TRUE;
}

void obj_prompt_arc(DOCUMENT *doc, gchar *buffer)
{
  gint mn;
  gdouble tx,ty,value;
  OBJECT_ARC arc;

  remove_old();

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  FALSE, _("C/"),0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      case PROMPT_MENU:
	gl_stage=3;
	use_table=FALSE;
	set_prompt();
	break;
      }
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  TRUE, _("C/E/"),ax,ay,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      case PROMPT_MENU:
	switch(mn){
	case 0:
	  gl_stage=12;
	  use_table=FALSE;
	  break;
	case 1:
	  gl_stage=8;
	  use_table=FALSE;
	  break;
	}
	set_prompt();
	break;
      }
      break;
    case 2:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,bx,by,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      }
      break;
    case 12:
    case 3:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      }
      break;
    case 4:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,cx,cy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      }
      break;
    case 13:
    case 5:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  TRUE, _("A/L/"),ax,ay,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      case PROMPT_MENU:
	switch(mn){
	case 0:
	  gl_stage=6;
	  use_table=FALSE;
	  break;
	case 1:
	  gl_stage=7;
	  use_table=FALSE;
	  break;
	}
	set_prompt();
	break;
      }
      break;
    case 6:  /* Included angle */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	work_document=doc;
	if(value>=0.0)	  
	  calc_arc_CSA(cx,cy,ax,ay,value, &arc);
	else
	{
	  arc.centrum.x=cx;
	  arc.centrum.y=cy;
	  arc.radius=dist(ax,ay,cx,cy);
	  arc.end_angle=get_angle(cx,cy,ax,ay);
	  arc.start_angle=arc.end_angle+value;
	}

	make_continue_end(doc,&arc);
	obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
       	break;
      }
      break;
    case 7:  /* Length of coord */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	work_document=doc;
	calc_arc_CSL(cx,cy,ax,ay,value,&arc);
	make_continue_end(doc,&arc);
	obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
       	break;
      }   
      break;
    case 8:  /* End point */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,ax,ay,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      }
      break;
    case 9:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  TRUE, _("A/D/R/"),ax,ay,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      case PROMPT_MENU:
	switch(mn){
	case 0:
	  gl_stage=14;
	  use_table=FALSE;
	  break;
	case 1:
	  gl_stage=10;
	  use_table=FALSE;
	  break;
	case 2:
	  gl_stage=11;
	  use_table=FALSE;
	  break;
	}
	set_prompt();
	break;
      }
      break;
    case 10: /* Direction from start */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,FALSE,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	work_document=doc;

	if(value>0.0)
	  tx=get_angle(bx,by,ax,ay)-value;
	else
	  tx=get_angle(bx,by,ax,ay)-value;

	calc_arc_SED(ax, ay, bx, by, tx*M_PI/180.0,value<0.0, &arc);
	make_continue_end(doc,&arc);
	obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
       	break;
      }
      break;
    case 11: /* Radius */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,FALSE,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	work_document=doc;
	if(value<0)
	  calc_arc_SER(ax, ay, bx, by, TRUE, -value, &arc);
	else
	  calc_arc_SER(ax, ay, bx, by, FALSE, value, &arc);
	make_continue_end(doc,&arc);
	obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
       	break;
      }
      break;
    case 14: /* Included angle */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,FALSE,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	work_document=doc;
	if(value<0)
	  value=-(360.0-ABS(value));
	else
	  value=360.0-value;

	calc_arc_SEA(ax, ay, bx, by, value, &arc);
	make_continue_end(doc,&arc);
	obj_draw_arc(doc, obj_add_arc_spec(doc, &arc));
       	break;
      }
      break;
    case 15: /* End point (for continue arcs) */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,bx,by,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_arc(doc,1,tx,ty);
       	break;
      }
      break;
    }

  UpdateCoordsLabel();
}

void obj_xor_arc_cont(DOCUMENT *doc,gdouble x, gdouble y)
{
  OBJECT_ARC arc;

  calc_arc_2p(doc->cont_tan, doc->cont_x, doc->cont_y, x,y, NULL, NULL, &arc);

  draw_arc(doc->widgets.draw->window, 
	   misc_setup_gc(doc),FALSE,
	   WLDtoSCR_x(doc, arc.centrum.x-arc.radius/2),WLDtoSCR_y(doc,arc.centrum.y-arc.radius/2),
	   WLDtoSCR(doc,arc.radius),WLDtoSCR(doc,arc.radius),arc.start_angle, 
	   arc.end_angle);

}

void obj_xor_arc_CSL(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble e;
  OBJECT_ARC arc;

  e=dist(x,y,ax,ay);

  UpdateSingelCoord(0,e);

  misc_setup_black_line(doc);
  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,ax),WLDtoSCR_y(doc,ay),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
  
  if(calc_arc_CSL(cx,cy,ax,ay,e,&arc))
    {
      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);
    }
}

void obj_xor_arc_CSA(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble eangle,sangle;
  OBJECT_ARC arc;

  sangle=get_angle(cx,cy,ax,ay);
  eangle=get_angle(cx,cy,x,y);

  if(sangle<eangle)
    eangle-=sangle;
  else
    eangle=eangle+(360.0-sangle);

  UpdateSingelCoord(0,eangle);

  misc_setup_black_line(doc);
  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,cx),WLDtoSCR_y(doc,cy),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
  
  if(calc_arc_CSA(cx,cy,ax,ay,eangle,&arc))
    {
      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);
    }
}

void obj_xor_arc_CSE(DOCUMENT *doc,gdouble x, gdouble y)
{
  OBJECT_ARC arc;

  misc_setup_black_line(doc);
  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,cx),WLDtoSCR_y(doc,cy),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));

  if(calc_arc_CSE(cx,cy,ax,ay,x,y,&arc))
    {
      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);
    }
}

void obj_xor_arc_SEC(DOCUMENT *doc,gdouble x, gdouble y)
{
  OBJECT_ARC arc;

  if(calc_arc_CSE(x,y,ax,ay,bx,by,&arc))
    {
      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc), FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);
    }
}

void obj_xor_arc_SED(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble ta, d, s,c;
  OBJECT_ARC arc;

  misc_setup_black_line(doc);
  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,ax),WLDtoSCR_y(doc,ay),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));

  ta=get_angle(x,y,ax,ay);
  s=get_angle(bx,by,ax,ay);

  d=dist(ax,ay,bx,by);
 
  if(ta>s)
    c=360-(ta-s);
  else
    c=360-(ta+(360.0-s));

  if(c>180)
    c=c-360;

  UpdateSingelCoord(0,c);
  
  if(calc_arc_SED(ax,ay,bx,by,ta*M_PI/180.0,(((ay-y)*(bx-ax)-(ax-x)*(by-ay))/(d*d))>0,&arc))
    {
      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);
    }
}

void obj_xor_arc_SER(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble d, dir;
  OBJECT_ARC arc;

  misc_setup_black_line(doc);
  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,ax),WLDtoSCR_y(doc,ay),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
  
  d=dist(ax,ay,bx,by);
  dir=((((ay-y)*(bx-ax)-(ax-x)*(by-ay))/(d*d))>0);

  if(calc_arc_SER(ax,ay,bx,by,dir,dist(x,y,ax,ay), &arc))
    {
      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc), FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);

      if(dir)
	UpdateSingelCoord(0, -arc.radius);
      else
	UpdateSingelCoord(0, arc.radius);
    }
}

void obj_xor_arc_SEA(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble sangle,eangle;
  OBJECT_ARC arc;

  misc_setup_black_line(doc);
  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,ax),WLDtoSCR_y(doc,ay),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y));
  
  sangle=get_angle(ax,ay,bx,by);
  eangle=get_angle(ax,ay,x,y);

  if(eangle>sangle)
    eangle=eangle-sangle;
  else
    eangle=eangle+(360.0-sangle);

  if(calc_arc_SEA(ax,ay,bx,by,360.0-eangle,&arc))
    {
      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);

      UpdateSingelCoord(0, eangle);
    }
}

void obj_xor_arc_3p(DOCUMENT *doc,gdouble x, gdouble y)
{
  OBJECT_ARC arc;
  
  if(calc_arc_3p(ax,ay,bx,by,x,y,&arc))
    {

      draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
	       WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
	       arc.end_angle);
    }
}

void obj_transform_arc(OBJECT *arc, TRANSFORM *trans)
{
  gdouble cx,cy;
  gdouble rx,ry;

  misc_transform_point(trans,arc->data.arc.centrum.x,arc->data.arc.centrum.y,&cx,&cy);
  arc->data.arc.centrum.x=cx;
  arc->data.arc.centrum.y=cy;

  rx=arc->data.arc.radius;
  ry=arc->data.arc.radius;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      rx=ABS(rx*trans->scale_x);      
      ry=ABS(ry*trans->scale_y);      

      /* If the two radius isnt equal: arc=>elarc */
      if(ABS(rx-ry)>GFLOAT_SENSITIVE)
	{
	  /* WRITE THIS */
	}
      else
	arc->data.arc.radius=rx;
    }

  if(trans!=NULL && (trans->type & OBJ_ROTATE))
    {
      arc->data.arc.start_angle-=trans->rot_angle*180/M_PI;
      while(arc->data.arc.start_angle<0) arc->data.arc.start_angle+=360;
      while(arc->data.arc.start_angle>360) arc->data.arc.start_angle-=360;

      arc->data.arc.end_angle-=trans->rot_angle*180/M_PI;
      while(arc->data.arc.end_angle<0) arc->data.arc.end_angle+=360;
      while(arc->data.arc.end_angle>360) arc->data.arc.end_angle-=360;
    }
  
  obj_bound_arc(arc);
}

void obj_draw_arc_block(DOCUMENT *doc, GdkGC *gc, OBJECT *arc, TRANSFORM *trans)
{
  gdouble angle=0;
  gdouble Ax, Ay, rx, ry;
 
  misc_transform_point(trans,arc->data.arc.centrum.x,arc->data.arc.centrum.y,&Ax,&Ay);

  rx=arc->data.arc.radius;
  ry=arc->data.arc.radius;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      rx=ABS(rx*trans->scale_x);
      ry=ABS(ry*trans->scale_y);
    }

  if(trans!=NULL && (trans->type & OBJ_ROTATE))
    angle=-trans->rot_angle*180/M_PI;

  draw_elarc(doc, doc->widgets.draw->window, gc, WLDtoSCR_x(doc, Ax), WLDtoSCR_y(doc, Ay), WLDtoSCR(doc, rx), WLDtoSCR(doc, ry),
	     angle, arc->data.arc.start_angle, arc->data.arc.end_angle);
}

void obj_draw_arc(DOCUMENT *doc, OBJECT *arc)
{
  obj_draw_arc_block(doc,misc_setup_gc_object(doc, arc),arc,NULL);
}

gint obj_select_arc(OBJECT *arc, gdouble x, gdouble y, gdouble d)
{
  gdouble xc, yc, radius, r;
  gdouble tangle;

  xc=arc->data.arc.centrum.x;
  yc=arc->data.arc.centrum.y;
  radius=arc->data.arc.radius;

  r=sqrt((xc-x)*(xc-x)+(yc-y)*(yc-y));

  if( (r>radius-(d+arc->linewidth/2)) && (r<radius+(d+arc->linewidth/2)) )
    {
      tangle=get_angle(xc,yc,x,y);

      if(arc->data.arc.end_angle>arc->data.arc.start_angle)
	return( (tangle>arc->data.arc.start_angle) && (tangle<arc->data.arc.end_angle));
      else{
	return( (tangle>arc->data.arc.start_angle) || (tangle<arc->data.arc.end_angle));
      }
    }

  return FALSE;
}





