/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "translation.h"
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_coords.h"
#include "dia_message.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "draw.h"
#include "redraw.h"
#include "obj_ellipse.h"
#include "math_tangent.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gdouble a_x, a_y, b_x, b_y, g_radius;
static gdouble inner_dia=0, outer_dia=10;

static OBJECT *objsel[3];

/* -- P R O T O T Y P E S -- */
void obj_draw_circle(DOCUMENT *doc, OBJECT *circle);

/* --  F U N C T I O N S  -- */
void obj_set_prompt_circle(void)
{
  switch(gl_stage)
    {
    case 0: 
    case 2: 
      set_prompt_text( _("2 Point/3 Point/TTR/TTT/<Center>: "));
      break;
    case 1: 
      set_prompt_text( _("Diameter/<Radius>: "));
      break;
    case 3: 
      set_prompt_text( _("Radius/<Diameter>: "));
      break;
    case 4: 
      set_prompt_text( _("Center/2 Point/TTR/TTT/<First Point>: "));
      break;
    case 5: 
      set_prompt_text( _("Second Point: "));
      break;
    case 6: 
      set_prompt_text( _("Third Point: "));
      break;
    case 8: 
      set_prompt_text( _("Center/3 Point/TTR/TTT/<First Point>: "));
      break;
    case 9: 
      set_prompt_text( _("Second Point: "));
      break; 
    case 10: 
      set_prompt_text( _("Diameter/<Center>: "));
      break;
    case 11:
      set_prompt_text( _("Inner diameter <%f>: "), inner_dia);
      break; 
    case 12: 
      set_prompt_text( _("Outer diameter <%f>: "), outer_dia);
      break; 
    case 13:
      set_prompt_text( _("Radius: "));
      break;
    case 14:
    case 17:
      set_prompt_text( _("Select first tangent object: "));
      break;
    case 15:
    case 18:
      set_prompt_text( _("Select second tangent object: "));
      break;
    case 16:
    case 20:
      set_prompt_text( _("Select position of circle: "));
      break;
    case 19:
      set_prompt_text( _("Select third tangent object: "));
      break;
    }
}

void obj_bound_circle(OBJECT *obj)
{
  obj->bounding_box[0]=obj->data.circle.centrum.x - obj->data.circle.radius - obj->linewidth/2;
  obj->bounding_box[1]=obj->data.circle.centrum.y - obj->data.circle.radius - obj->linewidth/2;
  obj->bounding_box[2]=obj->data.circle.centrum.x + obj->data.circle.radius + obj->linewidth/2;
  obj->bounding_box[3]=obj->data.circle.centrum.y + obj->data.circle.radius + obj->linewidth/2;
}


OBJECT *obj_add_circle(DOCUMENT *doc, LAYER *layer,  gchar *name, gint type_colour, COLOUR *colour, 
		     gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		     gdouble linewidth, gdouble xc, gdouble yc, gdouble radius)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_CIRCLE;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */
  obj->data.circle.centrum.x=xc;
  obj->data.circle.centrum.y=yc;
  obj->data.circle.radius=radius;

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_circle(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void obj_set_tool_circle(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("circle"));
  
  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;

  gl_tool=TOOL_CIRCLE;
  gl_stage=GPOINTER_TO_INT(data);

  UpdateCoordsLabel();

  if(gl_stage==10)
    {
      UpdateSingelCoord(2,inner_dia);
      UpdateSingelCoord(3,outer_dia);
      print( _("\n  Current Inner diameter: %f, Outer diameter: %f"), inner_dia, outer_dia);
    }

  work_document=NULL;
  set_prompt();
  set_cursor(NULL);

}

void obj_button_circle(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  gdouble A,B,C,D,E,F,G;
  gdouble p_x,p_y;
  gdouble d;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      deselect_all_objects(doc);
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0: // Cen, Rad, part 1
    case 2: // Cen, Dia, part 1
    case 8: // 2P, part 1
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage++;
      set_prompt();
      break;
    case 1:  // Cen, Rad, part 2
    case 3:  // Cen, Dia, part 2
           
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      d=dist(gl_oldmx, gl_oldmy, x,y);
      

      remove_old();
      
      obj_draw_circle(doc, obj_add_circle(doc,doc->current_layer, NULL, doc->type_colour, doc->current_colour,
					  doc->type_linestyle, doc->current_linestyle, 
					  doc->type_linewidth, doc->current_linewidth,
					  gl_oldmx, gl_oldmy, d));
      
      tool_idle(doc);
      break;
    case 4:  // 3P, part 1
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=a_x=x;
      gl_oldmy=a_y=y;
      gl_stage++;
      set_prompt();
      break;
    case 5:  // 3P, part 2
      remove_old();
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=b_x=x;
      gl_oldmy=b_y=y;
      gl_stage++;
      set_prompt();
      break;
    case 6:  // 3P, part 3
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      A=b_x-a_x;
      B=b_y-a_y;
      C=x-a_x;
      D=y-a_y;
      E=A*(a_x+b_x)+B*(a_y+b_y);
      F=C*(a_x+x)+D*(a_y+y);
      
      G=2.0*(A*(y-b_y)-B*(x-b_x));
      
      if(G!=0)
	{
	  p_x=(D*E-B*F)/G;
	  p_y=(A*F-C*E)/G;
	
	  d=dist(a_x, a_y, p_x,p_y);
	  
	  remove_old();
      
	  obj_draw_circle(doc, obj_add_circle(doc,doc->current_layer,NULL,doc->type_colour, doc->current_colour,
					      doc->type_linestyle, doc->current_linestyle, 
					      doc->type_linewidth, doc->current_linewidth,
					      p_x, p_y, d));

	  tool_idle(doc);
	}
      else
	print( _("\n-- Points are colinear!\n"));
      break;
    case 9:  // 2P, part 2
      make_ortho_snap( doc, gl_oldmx,gl_oldmy, &x,&y); 
      
      d=dist(gl_oldmx, gl_oldmy, x,y)/2;
      
      
      remove_old();

      obj_draw_circle(doc, obj_add_circle(doc,doc->current_layer, NULL, doc->type_colour, doc->current_colour,
					  doc->type_linestyle, doc->current_linestyle, 
					  doc->type_linewidth, doc->current_linewidth,
					  (gl_oldmx+x)/2, (gl_oldmy+y)/2, d));
      
      tool_idle(doc);

      break;
    case 10:  /* Donut */
      remove_old();
      if(doc->gl_snap)
	make_snap(doc, &x,&y);

      obj_draw_circle(doc, obj_add_circle(doc,doc->current_layer, NULL, doc->type_colour, doc->current_colour,
					  doc->type_linestyle, doc->current_linestyle, 
					  USERDEFINED, (outer_dia-inner_dia)/2,
					  x, y, ((outer_dia-inner_dia)/2+inner_dia)/2));
      set_prompt();
      break;
    case 14:  /* Select first object */
    case 17:
      objsel[0]=select_singel_object(doc,x,y);
      if(objsel[0]!=NULL)
        {
	  if(objsel[0]->type==TYPE_LINE || objsel[0]->type==TYPE_CIRCLE || objsel[0]->type==TYPE_ARC || objsel[0]->type==TYPE_POINT)
	    {
	      gl_stage++;
	      objsel[0]->selected=OBJECT_SEL;
	      redraw_object(doc,objsel[0]);
	    }
	  else
	    print( _("\n  Only points, lines, arcs or circles are allowed as tangent-objects."));
        }
      else
        print( _("\n  No object found."));

      set_prompt();
      break;
    case 15:  /* Select second object */
    case 18:
      objsel[1]=select_singel_object(doc,x,y);
      if(objsel[1]!=NULL)
        {
	  if(objsel[1]->type==TYPE_LINE || objsel[1]->type==TYPE_CIRCLE || objsel[1]->type==TYPE_ARC || objsel[1]->type==TYPE_POINT)
	    {
	      gl_stage++;
	      objsel[1]->selected=OBJECT_SEL;
	      redraw_object(doc,objsel[1]);
	    }
	  else
	    print( _("\n  Only point, lines, arcs or circles are allowed as tangent-objects."));
        }
      else
        print( _("\n  No object found."));

      set_prompt();
      break;
    case 16:  /* Select position of circle */
      remove_old();
      deselect_all_objects(doc);
      
      {
	gdouble cx,cy,cr;
	gdouble xo[10],yo[10],ro[10];
	gint nsol,pos;
	
	math_circle_tangent(g_radius, &objsel[0], xo, yo, ro, &nsol);
	
	if(nsol>0)
	  {
	    math_find_nearest_circle(xo, yo, ro, nsol, x, y, &cx, &cy, &cr, &pos);
	
	    obj_draw_circle(doc, obj_add_circle(doc,doc->current_layer, NULL, doc->type_colour, doc->current_colour,
						doc->type_linestyle, doc->current_linestyle, 
						doc->type_linewidth, doc->current_linewidth,
						cx, cy, cr));	       
	  }
	else
	  print( _("\n  No solution could be found."));
      }

      tool_idle(doc);
      break;
    case 19:  /* Select third object */
      objsel[2]=select_singel_object(doc,x,y);
      if(objsel[2]!=NULL)
        {
	  if(objsel[2]->type==TYPE_LINE || objsel[2]->type==TYPE_CIRCLE || objsel[2]->type==TYPE_ARC || objsel[2]->type==TYPE_POINT)
	    {
	      gl_stage++;
	      objsel[2]->selected=OBJECT_SEL;
	      redraw_object(doc,objsel[2]);
	    }
	  else
	    print( _("\n  Only points, lines, arcs or circles are allowed as tangent-objects."));
        }
      else
        print( _("\n  No object found."));

      set_prompt();
      break;
    case 20:  /* Select position of circle */
      remove_old();
      deselect_all_objects(doc);

      {
	gdouble cx,cy,cr;
	gdouble xo[10],yo[10],ro[10];
	gint nsol,pos;
	
	math_apollonius(&objsel[0], xo, yo, ro, &nsol);
	
	if(nsol>0)
	  {
	    math_find_nearest_circle(xo, yo, ro, nsol, x, y, &cx, &cy, &cr, &pos);
	    
	    obj_draw_circle(doc, obj_add_circle(doc,doc->current_layer, NULL, doc->type_colour, doc->current_colour,
						doc->type_linestyle, doc->current_linestyle, 
						doc->type_linewidth, doc->current_linewidth,
						cx, cy, cr));	       
	  }
	else
	  print( _("\n  No solution could be found."));  
      }

      tool_idle(doc);

      break;
    }

}

gint obj_menu_circle(gint menusel)
{
  switch(menusel){
  case 0:  /* Center, Radius */
    gl_stage=0;
    break;
  case 1:  /* Center, Diameter */
    gl_stage=2;
    break;
  case 2:  /* 3P Circle */
    gl_stage=4;
    break;
  case 3:  /* 2P Circle */
    gl_stage=8;
    break;
  case 4:  /* TTR */
    gl_stage=13;
    break;
  case 5:  /* TTT */
    gl_stage=17;
    break;
  case 7:  /* Cancel */
    UpdateCoordsLabel();
    break;
  }
 return FALSE;
}

void obj_prompt_circle(DOCUMENT *doc, gchar *buffer)
{
  gdouble x, y;
  gdouble e,d=1;
  gint mn;

  switch(gl_stage)
    {
    case 0:
    case 2:
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU),
			  buffer, FALSE,
			  _("2P/3P/TTR/TTT/"),
			  0,0,&x,&y,NULL,&mn))
	{
	case PROMPT_POINT2:
	  work_document=doc;
	  obj_button_circle(doc,1,x,y);
	  break;
	case PROMPT_MENU:
	  switch(mn)
	    {
	    case 0:
	      obj_menu_circle(3);
	      break;
	    case 1:
	      obj_menu_circle(2);
	      break;
	    case 2:
	      obj_menu_circle(4);
	      break;
	    case 3:
	      obj_menu_circle(5);
	      break;
	    }
	  set_prompt();
	  break;
	}
      break;
    case 3:  /* Ask diameter */
      d=0.5;
    case 1:  /* Ask diameter */
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT1|PROMPT_MENU),
			  buffer,FALSE,
			  _("D/R/"),
			  0,0,NULL,NULL,&e,&mn))
	{
	case PROMPT_POINT1:
	  d*=e;
	  remove_old();
	  work_document=doc;
	  obj_draw_circle(doc, obj_add_circle(doc,doc->current_layer,NULL,doc->type_colour, doc->current_colour,
					      doc->type_linestyle, doc->current_linestyle, 
					      doc->type_linewidth, doc->current_linewidth,
					      gl_oldmx, gl_oldmy, d));	  
	  tool_idle(doc);
	  break;
	case PROMPT_MENU:
	  if(gl_stage==1 && mn==0){
	    gl_stage=3;
	    set_prompt();
	  }else if(gl_stage==3 && mn==1){
	    gl_stage=1;
	    set_prompt();
	  }else{
	    print( _("\n-- Syntax Error\n"));
	    prompt();
	  }
	  break;
	}
      break;
    case 4:   // 3p, first point
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU),
			  buffer, FALSE,
			  _("C/2P/TTR/TTT/"),
			  0,0,&x,&y,NULL,&mn))
	{
	case PROMPT_POINT2:
	  work_document=doc;
	  obj_button_circle(doc,1,x,y);
	  break;
	case PROMPT_MENU:
	  switch(mn)
	    {
	    case 0:
	      obj_menu_circle(0);
	      break;
	    case 1:
	      obj_menu_circle(3);
	      break;
	    case 2:
	      obj_menu_circle(4);
	      break;
	    case 3:
	      obj_menu_circle(5);
	      break;
	    }
	  set_prompt();
	  break;
	}
      break;
      
    case 5:  /* 3p, second point */
    case 6:  /* 3p, third point  */
    case 9:  /* 2p, second point */
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT2),
			  buffer, TRUE, NULL,gl_oldmx,gl_oldmy,&x,&y,NULL,NULL))
	{
	case PROMPT_POINT2:
	  work_document=doc;
	  obj_button_circle(doc,1,x,y);
	  break;
	}
      break;
    case 8:  // 2p, first point
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU),
			  buffer, FALSE,
			  _("C/3P/TTR/TTT/"),
			  0,0,&x,&y,NULL,&mn))
	{
	case PROMPT_POINT2:
	  work_document=doc;
	  obj_button_circle(doc,1,x,y);
	  break;
	case PROMPT_MENU:
	  switch(mn)
	    {
	    case 0:
	      obj_menu_circle(0);
	      break;
	    case 1:
	      obj_menu_circle(2);
	      break;
	    case 2:
	      obj_menu_circle(4);
	      break;
	    case 3:
	      obj_menu_circle(5);
	      break;
	    }
	  set_prompt();
	  break;
	}
      break;
    case 10:
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  FALSE, _("D/"),0,0,&x,&y,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_circle(doc,1,x,y);
	break;
      case PROMPT_MENU:
	gl_stage=11;
	set_prompt();
	break;
      }
      break;
    case 11:  /* Inner diameter */
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&e,NULL)){
      case PROMPT_POINT1:
	inner_dia=ABS(e);
      case PROMPT_RETURN:
	gl_stage=12;
	UpdateSingelCoord(2,inner_dia);
	set_prompt();
	break;
      }
      break;
    case 12:  /* Outer diameter */
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&e,NULL)){
      case PROMPT_POINT1:
	outer_dia=ABS(e);
      case PROMPT_RETURN:
	if(outer_dia<inner_dia)
	  print( _("\n  Outer diameter must be greater or equal than inner diameter."));
	else
	  {
	    gl_stage=10;
	    UpdateSingelCoord(3,outer_dia);
	  }
	set_prompt();
	break;
      }
      break;
    case 13:  /* Radius */
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&e,NULL)){
      case PROMPT_POINT1:
	g_radius=ABS(e);
      case PROMPT_RETURN:
	gl_stage=14;
	set_prompt();
	break;
      }
      break;
    case 14: /* Select first object */
    case 17:
    case 15: /* Select second object */
    case 18:
    case 19: /* Select third object */
    case 20:
    case 16: /* Select position */
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&x,&y,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	obj_button_circle(doc,1,x,y);
	break;
      }
      break;
    }
}

void obj_xor_circle(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble d;

  d=dist(gl_oldmx, gl_oldmy, x,y);

  gdk_draw_arc(doc->widgets.draw->window, 
               misc_setup_gc(doc),FALSE,
               WLDtoSCR_x(doc,gl_oldmx-d),WLDtoSCR_y(doc,gl_oldmy-d),
               WLDtoSCR(doc,d*2),WLDtoSCR(doc,d*2),0, FULL_CIRCLE);

}

void obj_xor_circle_2P(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble d;

  d=dist(gl_oldmx, gl_oldmy, x,y)/2;

  gdk_draw_arc(doc->widgets.draw->window, 
               misc_setup_gc(doc),FALSE,
               WLDtoSCR_x(doc, (gl_oldmx+x)/2-d),WLDtoSCR_y(doc,(gl_oldmy+y)/2-d),
               WLDtoSCR(doc,d*2),WLDtoSCR(doc,d*2),0, FULL_CIRCLE);
}

void obj_xor_circle_3P(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble A,B,C,D,E,F,G;
  gdouble d,p_x,p_y;

  A=b_x-a_x;
  B=b_y-a_y;
  C=x-a_x;
  D=y-a_y;

  E=A*(a_x+b_x)+B*(a_y+b_y);
  F=C*(a_x+x)+D*(a_y+y);

  G=2.0*(A*(y-b_y)-B*(x-b_x));

  if(G!=0){
    p_x=(D*E-B*F)/G;
    p_y=(A*F-C*E)/G;

    d=dist(a_x, a_y, p_x,p_y);

    UpdateSingelCoord(2, d);
 
    gdk_draw_arc(doc->widgets.draw->window, 
                 misc_setup_gc(doc),FALSE,
                 WLDtoSCR_x(doc, p_x-d),WLDtoSCR_y(doc,p_y-d),
                 WLDtoSCR(doc,d*2),WLDtoSCR(doc,d*2),0, FULL_CIRCLE);
  }
}

void obj_xor_circle_TTR(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble cx,cy,cr;
  gdouble xo[10],yo[10],ro[10];
  gint nsol,pos;

  math_circle_tangent(g_radius, &objsel[0], xo, yo, ro, &nsol);

  math_find_nearest_circle(xo, yo, ro, nsol, x, y, &cx, &cy, &cr, &pos);

  gdk_draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, cx-cr),WLDtoSCR_y(doc, cy-cr),
	       WLDtoSCR(doc, cr*2),WLDtoSCR(doc, cr*2), 0, FULL_CIRCLE);
      
}

void obj_xor_circle_TTT(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble cx,cy,cr;
  gdouble xo[10],yo[10],ro[10];
  gint nsol,pos;

  math_apollonius(&objsel[0], xo, yo, ro, &nsol);

  math_find_nearest_circle(xo, yo, ro, nsol, x, y, &cx, &cy, &cr, &pos);

  gdk_draw_arc(doc->widgets.draw->window, 
	       misc_setup_gc(doc),FALSE,
	       WLDtoSCR_x(doc, cx-cr),WLDtoSCR_y(doc, cy-cr),
	       WLDtoSCR(doc, cr*2),WLDtoSCR(doc, cr*2), 0, FULL_CIRCLE);
}


void obj_transform_circle(OBJECT *circle, TRANSFORM *trans)
{
  gdouble tx,ty;
  gdouble rx,ry;

  misc_transform_point_select(trans, &circle->data.circle.centrum, &tx, &ty);
  circle->data.circle.centrum.x=tx;
  circle->data.circle.centrum.y=ty;

  rx=circle->data.circle.radius;
  ry=circle->data.circle.radius;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      rx=ABS(rx*trans->scale_x);
      ry=ABS(ry*trans->scale_y);

      if(ABS(rx-ry)<GFLOAT_SENSITIVE)
	circle->data.circle.radius=rx;
      else
	{
	  circle->type=TYPE_ELLIPSE;
	  circle->data.ellipse.centrum.x=tx;
	  circle->data.ellipse.centrum.y=ty;
	  circle->data.ellipse.major=rx;
	  circle->data.ellipse.minor=ry;
	  circle->data.ellipse.angle=0;
	  obj_bound_ellipse(circle);
	  return;
	}
    }

  obj_bound_circle(circle);
}

void obj_draw_circle_block(DOCUMENT *doc, GdkGC *gc, OBJECT *circle, TRANSFORM *trans)
{
  gdouble Ax,Ay;
  gdouble rx,ry;

  misc_transform_point_select(trans, &circle->data.circle.centrum, &Ax, &Ay);

  rx=circle->data.circle.radius;
  ry=circle->data.circle.radius;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      rx=ABS(rx*trans->scale_x);
      ry=ABS(ry*trans->scale_y);
    }

  if(trans!=NULL && (trans->type & OBJ_ROTATE))
    draw_ellipse(doc, doc->widgets.draw->window, gc, 
		 WLDtoSCR_x(doc,Ax), WLDtoSCR_y(doc,Ay), 
		 WLDtoSCR(doc,rx), WLDtoSCR(doc,ry), -trans->rot_angle*180/M_PI);
  else
    gdk_draw_arc(doc->widgets.draw->window, 
		 gc,FALSE,
		 WLDtoSCR_x(doc,Ax-rx),
		 WLDtoSCR_y(doc,Ay-ry),    
		 WLDtoSCR(doc,rx*2),
		 WLDtoSCR(doc,ry*2),   
		 0, FULL_CIRCLE);
}


void obj_draw_circle(DOCUMENT *doc, OBJECT *circle)
{
  obj_draw_circle_block(doc,misc_setup_gc_object(doc, circle),circle,NULL);
}

gint obj_select_circle(OBJECT *circle, gdouble x, gdouble y, gdouble d)
{
  gdouble xc, yc, radius, r;
  
  xc=circle->data.circle.centrum.x;
  yc=circle->data.circle.centrum.y;
  radius=circle->data.circle.radius;

  r=sqrt((xc-x)*(xc-x)+(yc-y)*(yc-y));

  return( (r>radius-(d+circle->linewidth/2)) && (r<radius+(d+circle->linewidth/2)) );
}




