/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gdouble cline_angle=M_PI/4;

/* --  P R O T O T Y P E S  -- */
void obj_draw_cline(DOCUMENT *doc, OBJECT *cline);

/* --  F U N C T I O N S  -- */
void obj_set_prompt_cline(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Horizontal/Vertical/Angle/<From point>: "));
      break;
    case 1: 
    case 2: /* Hor */
    case 3: /* Ver */
    case 5: /* Angle */
      set_prompt_text( _("Through point: "));
      break;
    case 4: /* Angle */
      set_prompt_text( _("Angle: "));
      break;
    }
}

void obj_bound_cline(OBJECT *obj)
{
  obj->bounding_box[0]=MIN(obj->data.line.p1.x,obj->data.line.p2.x);
  obj->bounding_box[1]=MIN(obj->data.line.p1.y,obj->data.line.p2.y);
  obj->bounding_box[2]=MAX(obj->data.line.p1.x,obj->data.line.p2.x);
  obj->bounding_box[3]=MAX(obj->data.line.p1.y,obj->data.line.p2.y);
}


OBJECT *obj_add_cline(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		      gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		      gdouble linewidth, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_CLINE;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */
  obj->data.line.p1.x=x1;
  obj->data.line.p1.y=y1;
  obj->data.line.p2.x=x2;
  obj->data.line.p2.y=y2;
  
  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_cline(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void obj_set_tool_cline(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("xline"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_CLINE;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void obj_button_cline(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      //      print("*Cancel*");
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      obj_draw_cline(doc, obj_add_cline(doc,doc->current_layer, NULL, 
					doc->type_colour, doc->current_colour, 
					doc->type_linestyle, doc->current_linestyle,
					doc->type_linewidth, doc->current_linewidth,
					gl_oldmx, gl_oldmy, x,y));
      set_prompt();     
      break;
    case 2:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      obj_draw_cline(doc, obj_add_cline(doc,doc->current_layer, NULL, 
					doc->type_colour, doc->current_colour, 
					doc->type_linestyle, doc->current_linestyle,
					doc->type_linewidth, doc->current_linewidth,
					x,y, x+10,y));
      set_prompt();

      break;
    case 3:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      obj_draw_cline(doc, obj_add_cline(doc,doc->current_layer, NULL, 
					doc->type_colour, doc->current_colour, 
					doc->type_linestyle, doc->current_linestyle,
					doc->type_linewidth, doc->current_linewidth,
					x,y, x,y+10));
      set_prompt();

      break;
    case 5:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      obj_draw_cline(doc, obj_add_cline(doc,doc->current_layer, NULL, 
					doc->type_colour, doc->current_colour, 
					doc->type_linestyle, doc->current_linestyle,
					doc->type_linewidth, doc->current_linewidth,
					x,y, x+10*cos(cline_angle),y-10*sin(cline_angle)));
      set_prompt();

      break;
    }
}

void obj_prompt_cline(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty, value;
  gint mn;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  FALSE, _("H/V/A/"),0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_cline(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	switch(mn)
	  {
	  case 0: /* Hor */
	    work_document=doc;
	    gl_stage=2;
	    break;
	  case 1: /* Ver */
	    work_document=doc;
	    gl_stage=3;
	    break;
	  case 2: /* Angle */
	    gl_stage=4;
	    break;
	  }
	set_prompt();
	break;
      }
    }
  else if(gl_stage==4)
    {
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	cline_angle=value*M_PI/180;
      case PROMPT_RETURN:
	gl_stage=5;
	work_document=doc;
	//	UpdateSingelCoord(2,dist1);
	set_prompt();
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_RETURN:
	remove_old();
	tool_idle(doc);
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_cline(doc,1,tx,ty);
	break;
      }
    }  
}

void calc_endpoint_for_cline(DOCUMENT *doc, gdouble x1,gdouble y1, gdouble x2, gdouble y2, 
			     gdouble *nx1, gdouble *ny1, gdouble *nx2, gdouble *ny2)
{
  gdouble wx1,wy1,wx2,wy2;
  gdouble m,b;

  wx1=SCRtoWLD_x(doc,-1);
  wy1=SCRtoWLD_y(doc,-1);
  wx2=SCRtoWLD_x(doc,doc->widgets.draw->allocation.width);
  wy2=SCRtoWLD_y(doc,doc->widgets.draw->allocation.height);


  if( ABS(x2-x1)<GFLOAT_SENSITIVE)  /* A vertical line */
    {
      *nx2=*nx1=x1;
      *ny1=wy1;
      *ny2=wy2;
    }
  else if( ABS(y2-y1)<GFLOAT_SENSITIVE) /* A horizontal line */
    {
      *ny2=*ny1=y1;
      *nx1=wx1;
      *nx2=wx2;
    }
  else
    {
      m=(y2-y1)/(x2-x1);

      b=y1-m*x1;

      /* Calculate inersection with left */
      *nx1=wx1;
      *ny1=m*wx1+b;
      
      /* Calculate intersection with right */
      *nx2=wx2;
      *ny2=m*wx2+b;
      
      if(*ny1<wy1)
	{
	  *ny1=wy1;
	  *nx1=(wy1-b)/m;
	}
      if(*ny1>wy2)
	{
	  *ny1=wy2;
	  *nx1=(wy2-b)/m;
	}

      if(*ny2<wy1)
	{
	  *ny2=wy1;
	  *nx2=(wy1-b)/m;
	}
      if(*ny2>wy2)
	{
	  *ny2=wy2;
	  *nx2=(wy2-b)/m;
	} 
    }
}

void obj_xor_cline(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble nx1,ny1;
  gdouble nx2,ny2;

  if(gl_stage==1)
    calc_endpoint_for_cline(doc,gl_oldmx,gl_oldmy,x,y, &nx1, &ny1, &nx2, &ny2);
  else if(gl_stage==2)
    calc_endpoint_for_cline(doc,x,y,x+10,y, &nx1, &ny1, &nx2, &ny2);   
  else if(gl_stage==3)
    calc_endpoint_for_cline(doc,x,y,x,y+10, &nx1, &ny1, &nx2, &ny2);   
  else if(gl_stage==5)
      calc_endpoint_for_cline(doc,x,y,x+10*cos(cline_angle),y-10*sin(cline_angle), &nx1, &ny1, &nx2, &ny2);   

  gdk_draw_line(doc->widgets.draw->window, 
		misc_setup_gc(doc),      		
		WLDtoSCR_x(doc,nx1),WLDtoSCR_y(doc,ny1),
		WLDtoSCR_x(doc,nx2),WLDtoSCR_y(doc,ny2));
}

void obj_transform_cline(OBJECT *line, TRANSFORM *trans)
{
  gdouble tx,ty;

  misc_transform_point(trans,line->data.line.p1.x,line->data.line.p1.y,&tx,&ty);
  line->data.line.p1.x=tx;
  line->data.line.p1.y=ty;

  misc_transform_point(trans,line->data.line.p2.x,line->data.line.p2.y,&tx,&ty);
  line->data.line.p2.x=tx;
  line->data.line.p2.y=ty;
  
  obj_bound_cline(line);
}

void obj_draw_cline_block(DOCUMENT *doc, GdkGC *gc, OBJECT *cline, TRANSFORM *trans)
{
  gdouble Ax, Ay, Bx, By;

  misc_transform_point(trans,cline->data.line.p1.x,cline->data.line.p1.y,&Ax,&Ay);
  misc_transform_point(trans,cline->data.line.p2.x,cline->data.line.p2.y,&Bx,&By);

  calc_endpoint_for_cline(doc, Ax,Ay, Bx,By, &Ax,&Ay, &Bx, &By);

  gdk_draw_line(doc->widgets.draw->window, 
		gc,
		WLDtoSCR_x(doc,Ax), 
		WLDtoSCR_y(doc,Ay),
		WLDtoSCR_x(doc,Bx), 
		WLDtoSCR_y(doc,By));

}

void obj_draw_cline(DOCUMENT *doc, OBJECT *cline)
{
  obj_draw_cline_block(doc,misc_setup_gc_object(doc, cline),cline,NULL);
}


gint obj_select_cline(OBJECT *cline, gdouble x, gdouble y, gdouble d)
{
  gdouble L;
  gdouble s,r;
  gdouble Ax, Ay, Bx, By;
  
  Ax=cline->data.line.p1.x;
  Ay=cline->data.line.p1.y;
  Bx=cline->data.line.p2.x;
  By=cline->data.line.p2.y;

  calc_endpoint_for_cline(cline->layer->doc,Ax,Ay,Bx,By, &Ax,&Ay,&Bx, &By);

  L=sqrt( (Bx-Ax)*(Bx-Ax) + (By-Ay)*(By-Ay) );

  r=((Ay-y)*(Ay-By)-(Ax-x)*(Bx-Ax))/(L*L);

  if((r>=0) && (r<=1)){
    s=((Ay-y)*(Bx-Ax)-(Ax-x)*(By-Ay))/(L*L);
    return( ABS(s*L)<(d+cline->linewidth/2));
  }
    
  return FALSE;
}







