/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "obj_elarc.h"
#include "draw.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
gdouble xc,yc;

/* --  P R O T O T Y P E S  -- */
void obj_draw_ellipse(DOCUMENT *doc, OBJECT *ellipse);

/* --  F U N C T I O N S  -- */
void calc_ellipse(gdouble x, gdouble y,gdouble *rot, gdouble *nx,gdouble *ny)
{
  gdouble L,d;
  gdouble angle;

  *rot=get_angle(xc,yc, gl_oldmx, gl_oldmy);
  
  L=dist(gl_oldmx, gl_oldmy, xc,yc);
  d=L*((gl_oldmy-y)*(xc-gl_oldmx)-(gl_oldmx-x)*(yc-gl_oldmy))/(L*L);
  angle=(*rot)*M_PI/180-M_PI/2;
  
  *nx=xc+d*cos(angle);
  *ny=yc-d*sin(angle);
}

void obj_set_prompt_ellipse(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Arc/Center/<Axis endpoint 1>: "));
      break;
    case 1:
      set_prompt_text( _("Axis endpoint 2: "));
      break;
    case 2:
    case 5:
      set_prompt_text( _("Rotation/<Other axis distance>: "));
      break;
    case 3:
      set_prompt_text( _("Center of ellipse: "));
      break;
    case 4:
      set_prompt_text( _("Axis endpoint: "));
      break;
    case 6:
      set_prompt_text( _("Rotation around major axis: "));
      break;
    }
}

void obj_bound_ellipse(OBJECT *obj)
{
  gint i;
  gint num;
  gdouble dt,dCos,dSin,sina,cosa,adivb,bdiva;
  gdouble ma,mi;
  POINT *list;
  GdkPoint *rot_list;

  ma=obj->data.ellipse.major;
  mi=obj->data.ellipse.minor;

  num=MAX(65, mi*ma/1000);
  num=MIN(num, 700);

  dt=2.*M_PI/(gdouble)num;

  dCos=cos(dt);
  dSin=sin(dt);
  sina=sin(-obj->data.ellipse.angle*M_PI/180);
  cosa=cos(-obj->data.ellipse.angle*M_PI/180);
  adivb=ma/mi;
  bdiva=mi/ma;

  list=g_new(POINT,num);

  rot_list=g_new(GdkPoint, num+1);

  list[0].x=ma;
  list[0].y=0;

  for(i=0;i<(num-1); i++)
    {
      list[i+1].x=list[i].x*dCos-adivb*list[i].y*dSin;
      list[i+1].y=bdiva*list[i].x*dSin+list[i].y*dCos;
    }

  obj->bounding_box[0]=obj->data.ellipse.centrum.x+(list[i].x*cosa-list[i].y*sina);
  obj->bounding_box[1]=obj->data.ellipse.centrum.y+(list[i].x*sina+list[i].y*cosa);

  for(i=1;i<num;i++)
    {
      obj->bounding_box[0]=MIN(obj->data.ellipse.centrum.x+(list[i].x*cosa-list[i].y*sina), obj->bounding_box[0]);
      obj->bounding_box[1]=MIN(obj->data.ellipse.centrum.y+(list[i].x*sina+list[i].y*cosa), obj->bounding_box[1]);
      obj->bounding_box[2]=MAX(obj->data.ellipse.centrum.x+(list[i].x*cosa-list[i].y*sina), obj->bounding_box[2]);
      obj->bounding_box[3]=MAX(obj->data.ellipse.centrum.y+(list[i].x*sina+list[i].y*cosa), obj->bounding_box[3]);
    }

  g_free(list);
}


OBJECT *obj_add_ellipse(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
			gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
			gdouble linewidth,
			gdouble xcen, gdouble ycen, gdouble major, gdouble minor, gdouble angle)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_ELLIPSE;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */
  obj->data.ellipse.centrum.x=xcen;
  obj->data.ellipse.centrum.y=ycen;
  obj->data.ellipse.major=major;
  obj->data.ellipse.minor=minor;
  obj->data.ellipse.angle=angle;

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_ellipse(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void obj_set_tool_ellipse(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("ellipse"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;

  gl_tool=TOOL_ELLIPSE;
  gl_stage=GPOINTER_TO_INT(data);
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void obj_button_ellipse(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  gdouble minor,major;
  gdouble rot,nx,ny,r,r2;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      //      print("*Cancel*");
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:   /* Axis endpoint 1*/
      if(doc->gl_snap)
	make_snap(doc,&x,&y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage++;
      set_prompt();
      break;
    case 1:   /* Axis endpoint 2 */
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy,  &x, &y);
      xc=x;
      yc=y;
      gl_stage++;
      set_prompt();
      break;
    case 3:   /* Center point of ellipse */
      if(doc->gl_snap)
	make_snap(doc,&x,&y);
      xc=gl_oldmx=x;
      yc=gl_oldmy=y;
      gl_stage++;
      set_prompt();
      break;
    case 4:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy,  &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage++;
      set_prompt();
      break;
    case 2:   /* Axis distance */
    case 5:
      remove_old();
      if(doc->gl_snap)
	make_snap(doc,&x,&y);

      calc_ellipse(x,y,&rot,&nx,&ny);
      
      if( dist(xc,yc,nx,ny)<dist(xc,yc,gl_oldmx,gl_oldmy) )
	{
	  minor=dist(xc,yc,nx,ny);
	  major=dist(xc,yc,gl_oldmx,gl_oldmy);
	  rot=get_angle(xc,yc, gl_oldmx, gl_oldmy);
	}
      else
	{
	  major=dist(xc,yc,nx,ny);
	  minor=dist(xc,yc,gl_oldmx,gl_oldmy);
	  rot=get_angle(xc,yc, nx, ny);
	}

      obj_draw_ellipse(doc, obj_add_ellipse(doc, doc->current_layer, NULL, 
					    doc->type_colour, doc->current_colour, 
					    doc->type_linestyle, doc->current_linestyle,
					    doc->type_linewidth, doc->current_linewidth,
					    xc,yc, major,minor, rot));
      tool_idle(doc);     
      break;
    case 6:  /* Rotation */
      remove_old();
      if(doc->gl_snap)
	make_snap(doc,&x,&y);

      calc_ellipse(x,y,&rot,&nx,&ny);

      r=dist(xc,yc,gl_oldmx,gl_oldmy);
      r2=dist(xc,yc,nx,ny);

      if(r2>r)
	r2=r;

      if( r2<r )
	{
	  minor=r2;
	  major=r;
	  rot=get_angle(xc,yc, gl_oldmx, gl_oldmy);
	}
      else
	{
	  major=r2;
	  minor=r;
	  rot=get_angle(xc,yc, nx, ny);
	}

      obj_draw_ellipse(doc, obj_add_ellipse(doc, doc->current_layer, NULL, 
					    doc->type_colour, doc->current_colour, 
					    doc->type_linestyle, doc->current_linestyle,
					    doc->type_linewidth, doc->current_linewidth,
					    xc,yc,major,minor,rot));
      tool_idle(doc);     
      break;
    }
}

void obj_prompt_ellipse(DOCUMENT *doc, gchar *buffer)
{
  gdouble major,minor,rot;
  gdouble tx, ty, value;
  gint mn;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  FALSE, _("A/C/"),0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_ellipse(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	if(mn==0)
	  obj_set_tool_elarc(NULL,NULL);
	else
	  {
	    gl_stage=3;	    
	    set_prompt();
	  }
	break;
      }
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_ellipse(doc,1,tx,ty);
	break;
      }
      break;
    case 3:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_ellipse(doc,1,tx,ty);
	break;
      }
      break;
    case 2:
    case 5:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1|PROMPT_MENU,buffer,
			  FALSE, _("R/"),0,0,NULL,NULL,&value,&mn)){
      case PROMPT_POINT1:
	remove_old();
	
	if( value<dist(xc,yc,gl_oldmx,gl_oldmy) )
	{
	  minor=value;
	  major=dist(xc,yc,gl_oldmx,gl_oldmy);
	  rot=get_angle(xc,yc, gl_oldmx, gl_oldmy);
	}
	else
	  {
	    major=value;
	    minor=dist(xc,yc,gl_oldmx,gl_oldmy);
	    rot=get_angle(xc,yc, gl_oldmx, gl_oldmy)+90;
	  }

	obj_draw_ellipse(doc, obj_add_ellipse(doc, doc->current_layer, NULL, 
					      doc->type_colour, doc->current_colour, 
					      doc->type_linestyle, doc->current_linestyle,
					      doc->type_linewidth, doc->current_linewidth,
					      xc, yc, major, minor, rot));
	tool_idle(doc);     
	break;
      case PROMPT_MENU:
	gl_stage=6;
	set_prompt();
	break;
      }
      break;
    case 4:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_ellipse(doc,1,tx,ty);
	break;
      }
      break;
    case 6:  /* Rotation */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE, NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	remove_old();
	obj_draw_ellipse(doc, obj_add_ellipse(doc, doc->current_layer, NULL, 
					      doc->type_colour, doc->current_colour, 
					      doc->type_linestyle, doc->current_linestyle,
					      doc->type_linewidth, doc->current_linewidth,
					      xc,yc,
					      dist(xc,yc,gl_oldmx,gl_oldmy),		   
					      dist(xc,yc,gl_oldmx,gl_oldmy)*cos(value*M_PI/180.0), 
					      get_angle(xc,yc, gl_oldmx, gl_oldmy)));
	tool_idle(doc);     
	break;
      }
      break;
    }
}

void obj_xor_ellipse(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble nx, ny;
  gdouble minor,major;
  gdouble rot;
  
  misc_setup_black_line(doc);

  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,gl_oldmx),WLDtoSCR_y(doc,gl_oldmy),
		WLDtoSCR_x(doc,xc),WLDtoSCR_y(doc,yc));
  
  calc_ellipse(x,y,&rot, &nx, &ny);

  gdk_draw_line(doc->widgets.draw->window, 
		doc->xor_gc,
		WLDtoSCR_x(doc,xc),WLDtoSCR_y(doc,yc),
		WLDtoSCR_x(doc,nx),WLDtoSCR_y(doc,ny));

  if( dist(xc,yc,nx,ny)<dist(xc,yc,gl_oldmx,gl_oldmy) )
	{
	  minor=dist(xc,yc,nx,ny);
	  major=dist(xc,yc,gl_oldmx,gl_oldmy);
	  rot=get_angle(xc,yc, gl_oldmx, gl_oldmy);
	}
      else
	{
	  major=dist(xc,yc,nx,ny);
	  minor=dist(xc,yc,gl_oldmx,gl_oldmy);
	  rot=get_angle(xc,yc, nx, ny);
	}

  draw_ellipse(doc,doc->widgets.draw->window, 
	       misc_setup_gc(doc),
	       WLDtoSCR_x(doc,xc), WLDtoSCR_y(doc,yc), 
	       WLDtoSCR(doc,major),WLDtoSCR(doc, minor),rot);

  UpdateSingelCoord(2,major);
  UpdateSingelCoord(3,minor);

}

void obj_xor_ellipse_rot(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble r,r2, rot, nx,ny;

  calc_ellipse(x,y,&rot, &nx, &ny);

  r=dist(xc,yc,gl_oldmx,gl_oldmy);
  r2=dist(xc,yc,nx,ny);
  
  if(r2>r)
    r2=r;
 
  draw_ellipse(doc,doc->widgets.draw->window, 
	       misc_setup_gc(doc),
	       WLDtoSCR_x(doc,xc), WLDtoSCR_y(doc,yc), 
	       WLDtoSCR(doc,r),WLDtoSCR(doc,r2),
	       get_angle(xc,yc, gl_oldmx, gl_oldmy));

  UpdateSingelCoord(2,r);
  UpdateSingelCoord(3,acos(r2/r)*180.0/M_PI);
}

void obj_transform_ellipse(OBJECT *ellipse, TRANSFORM *trans)
{
  gdouble tx,ty;

  misc_transform_point(trans,ellipse->data.ellipse.centrum.x,ellipse->data.ellipse.centrum.y,&tx,&ty);
  ellipse->data.ellipse.centrum.x=tx;
  ellipse->data.ellipse.centrum.y=ty;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      ellipse->data.ellipse.major=ABS(ellipse->data.ellipse.major*trans->scale_x);
      ellipse->data.ellipse.minor=ABS(ellipse->data.ellipse.minor*trans->scale_y);
    }

  if(trans!=NULL && (trans->type & OBJ_ROTATE))
    ellipse->data.ellipse.angle-=trans->rot_angle*180/M_PI;

  obj_bound_ellipse(ellipse);
}

void obj_draw_ellipse_block(DOCUMENT *doc, GdkGC *gc, OBJECT *ellipse, TRANSFORM *trans)
{
  gdouble Ax, Ay, rx,ry,a;
  
  misc_transform_point(trans,ellipse->data.ellipse.centrum.x,ellipse->data.ellipse.centrum.y,&Ax,&Ay);

  rx=ellipse->data.ellipse.major;
  ry=ellipse->data.ellipse.minor;
  a=ellipse->data.ellipse.angle;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      rx=ABS(rx*trans->scale_x);
      ry=ABS(ry*trans->scale_y);
    }

  if(trans!=NULL && (trans->type & OBJ_ROTATE))
    a-=trans->rot_angle*180/M_PI;

  /* Draw bounding box */
  /*  gdk_draw_line(doc->widgets.draw->window, gc, 
		WLDtoSCR_x(doc, ellipse->bounding_box[0]),WLDtoSCR_y(doc, ellipse->bounding_box[1]),
		WLDtoSCR_x(doc, ellipse->bounding_box[2]),WLDtoSCR_y(doc, ellipse->bounding_box[1]));
  gdk_draw_line(doc->widgets.draw->window, gc, 
		WLDtoSCR_x(doc, ellipse->bounding_box[2]),WLDtoSCR_y(doc, ellipse->bounding_box[1]),
		WLDtoSCR_x(doc, ellipse->bounding_box[2]),WLDtoSCR_y(doc, ellipse->bounding_box[3]));
  gdk_draw_line(doc->widgets.draw->window, gc, 
		WLDtoSCR_x(doc, ellipse->bounding_box[2]),WLDtoSCR_y(doc, ellipse->bounding_box[3]),
		WLDtoSCR_x(doc, ellipse->bounding_box[0]),WLDtoSCR_y(doc, ellipse->bounding_box[3]));
  gdk_draw_line(doc->widgets.draw->window, gc, 
		WLDtoSCR_x(doc, ellipse->bounding_box[0]),WLDtoSCR_y(doc, ellipse->bounding_box[1]),
		WLDtoSCR_x(doc, ellipse->bounding_box[0]),WLDtoSCR_y(doc, ellipse->bounding_box[3]));
  */

  draw_ellipse(doc,doc->widgets.draw->window, 
	       gc,      
	       WLDtoSCR_x(doc,Ax), WLDtoSCR_y(doc,Ay), 
	       WLDtoSCR(doc,rx),WLDtoSCR(doc,ry),a);
}

void obj_draw_ellipse(DOCUMENT *doc, OBJECT *ellipse)
{
  obj_draw_ellipse_block(doc,misc_setup_gc_object(doc, ellipse),ellipse,NULL);
}

gint obj_select_ellipse(OBJECT *ellipse, gdouble x, gdouble y, gdouble d)
{
  gdouble c;
  gdouble rx,ry;
  gdouble tx,ty;

  x-=ellipse->data.ellipse.centrum.x;
  y-=ellipse->data.ellipse.centrum.y;
  
  rx=ellipse->data.ellipse.minor;
  ry=ellipse->data.ellipse.major;

  c=ellipse->data.ellipse.angle*M_PI/180;
  tx=x*cos(-c)+y*sin(-c);
  ty=-x*sin(-c)+y*cos(-c);

  return( (tx*tx)/((ry+d)*(ry+d))+(ty*ty)/((rx+d)*(rx+d))<1   && 
	  (tx*tx)/((ry-d)*(ry-d))+(ty*ty)/((rx-d)*(rx-d))>1  );

}







