/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gdouble close_x, close_y;

/* --  P R O T O T Y P E S  -- */
void obj_draw_line(DOCUMENT *doc, OBJECT *line);


/* --  F U N C T I O N S  -- */
void obj_set_prompt_line(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Start Point: "));
      break;
    case 1:
      set_prompt_text( _("Close/<End Point>: "));
      break;
    }
}

void obj_bound_line(OBJECT *obj)
{
  obj->bounding_box[0]=MIN(obj->data.line.p1.x,obj->data.line.p2.x);
  obj->bounding_box[1]=MIN(obj->data.line.p1.y,obj->data.line.p2.y);
  obj->bounding_box[2]=MAX(obj->data.line.p1.x,obj->data.line.p2.x);
  obj->bounding_box[3]=MAX(obj->data.line.p1.y,obj->data.line.p2.y);
}


OBJECT *obj_add_line(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		     gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		     gdouble linewidth, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  OBJECT *obj;

  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_LINE;

  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 

  /* Set object specific attributes */
  obj->data.line.p1.x=x1;
  obj->data.line.p1.y=y1;
  obj->data.line.p2.x=x2;
  obj->data.line.p2.y=y2;

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_line(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void obj_set_tool_line(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("line"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_LINE;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void obj_button_line(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      //      print("*Cancel*");
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      close_x=gl_oldmx=x;
      close_y=gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
    
      doc->cont_exist=TRUE;
      doc->cont_x=x;
      doc->cont_y=y;
      doc->cont_tan=get_angle(x,y,gl_oldmx,gl_oldmy)*M_PI/180.0+M_PI/2;

      obj_draw_line(doc, obj_add_line(doc,doc->current_layer, NULL, doc->type_colour, doc->current_colour, 
				      doc->type_linestyle, doc->current_linestyle,
				      doc->type_linewidth, doc->current_linewidth,
				      gl_oldmx, gl_oldmy, x,y));

      gl_oldmx=x;
      gl_oldmy=y;
      set_prompt();
      
      UpdateSingelCoord(0, gl_oldmx);
      UpdateSingelCoord(1, gl_oldmy);

      break;
    }
}

void obj_prompt_line(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gint mn;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_line(doc,1,tx,ty);
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2|PROMPT_MENU,buffer,
			  TRUE, _("C/"),gl_oldmx,gl_oldmy,&tx,&ty,NULL,&mn)){
      case PROMPT_RETURN:
	remove_old();
	tool_idle(doc);
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_line(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	if(gl_stage>0){
	  remove_old();

	  doc->cont_exist=TRUE;
	  doc->cont_x=close_x;
	  doc->cont_y=close_y;
	  doc->cont_tan=get_angle(close_x,close_y,gl_oldmx,gl_oldmy)*M_PI/180.0+M_PI/2;

	  gl_stage=0;
	  obj_draw_line(doc, obj_add_line(doc,doc->current_layer, NULL,doc->type_colour, doc->current_colour, 
					  doc->type_linestyle, doc->current_linestyle,
					  doc->type_linewidth, doc->current_linewidth,
					  gl_oldmx, gl_oldmy, close_x,close_y));
	}
	tool_idle(doc);
	break;
      }
    }  
}

void obj_xor_line(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdk_draw_line(doc->widgets.draw->window, 
		misc_setup_gc(doc),
		WLDtoSCR_x(doc,gl_oldmx),WLDtoSCR_y(doc,gl_oldmy),
		WLDtoSCR_x(doc,x),WLDtoSCR_y(doc,y)); 
}

void obj_transform_line(OBJECT *line, TRANSFORM *trans)
{
  gdouble tx,ty;

  misc_transform_point_select(trans,&line->data.line.p1,&tx,&ty);
  line->data.line.p1.x=tx;
  line->data.line.p1.y=ty;
  
  misc_transform_point_select(trans,&line->data.line.p2,&tx,&ty);
  line->data.line.p2.x=tx;
  line->data.line.p2.y=ty;

  obj_bound_line(line);
}

void obj_draw_line_block(DOCUMENT *doc, GdkGC *gc, OBJECT *line, TRANSFORM *trans)
{
  gdouble Ax, Ay, Bx, By;

  misc_transform_point_select(trans,&line->data.line.p1,&Ax,&Ay);
  misc_transform_point_select(trans,&line->data.line.p2,&Bx,&By);

  gdk_draw_line(doc->widgets.draw->window, 
		gc,
		WLDtoSCR_x(doc,Ax), 
		WLDtoSCR_y(doc,Ay),
		WLDtoSCR_x(doc,Bx), 
		WLDtoSCR_y(doc,By));
}

void obj_draw_line(DOCUMENT *doc, OBJECT *line)
{
  obj_draw_line_block(doc,misc_setup_gc_object(doc, line),line,NULL);
}

gint obj_select_line(OBJECT *line, gdouble x, gdouble y, gdouble d)
{
  return(misc_point_on_line(x,y,
			    line->data.line.p1.x, line->data.line.p1.y,
			    line->data.line.p2.x, line->data.line.p2.y,
			    d+line->linewidth/2));

}







