/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "draw.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
void obj_draw_point(DOCUMENT *doc, OBJECT *point);

/* --  F U N C T I O N S  -- */
void obj_set_prompt_point(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Multiple/<Point>: "));
      break;
    case 1:
      set_prompt_text( _("Point: "));
      break;
    }
}

void obj_set_tool_point(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("point"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_POINT;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void obj_bound_point(OBJECT *obj)
{
  obj->bounding_box[0]=obj->data.point.centrum.x-obj->data.point.point_size;
  obj->bounding_box[1]=obj->data.point.centrum.y-obj->data.point.point_size;
  obj->bounding_box[2]=obj->data.point.centrum.x+obj->data.point.point_size;
  obj->bounding_box[3]=obj->data.point.centrum.y+obj->data.point.point_size;
}


OBJECT *obj_add_point(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		      gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		      gdouble linewidth, gint style, gint sizetype, 
		      gfloat size, gdouble x, gdouble y)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }
  
  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_POINT;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */
  obj->data.point.centrum.x=x;
  obj->data.point.centrum.y=y;
  obj->data.point.point_style=style;
  obj->data.point.point_type=sizetype;
  obj->data.point.point_size=size;

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_point(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}


void obj_button_point(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      obj_draw_point(doc, obj_add_point(doc,doc->current_layer,NULL, doc->type_colour,
					doc->current_colour,  doc->type_linestyle, doc->current_linestyle,
					doc->type_linewidth, doc->current_linewidth,
					doc->point_style, doc->point_type, 
					doc->point_size,x,y));
      tool_idle(doc);
      break;
    case 1:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      obj_draw_point(doc, obj_add_point(doc,doc->current_layer, NULL, doc->type_colour,
					doc->current_colour,  doc->type_linestyle, doc->current_linestyle,
					doc->type_linewidth, doc->current_linewidth, 
					doc->point_style, doc->point_type,
					doc->point_size,x,y));
      gl_oldmx=x;
      gl_oldmy=y;
      set_prompt();
      break;
    }
}

void obj_prompt_point(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gint mn;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  FALSE, _("M/"),0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_point(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	gl_stage=1;
	set_prompt();
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL, NULL)){
      case PROMPT_RETURN:
	remove_old();
	tool_idle(doc);
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_point(doc,1,tx,ty);
	break;
      }
    }  
}

void obj_transform_point(OBJECT *point, TRANSFORM *trans)
{
  gdouble tx,ty;

  misc_transform_point(trans,point->data.point.centrum.x,point->data.point.centrum.y,&tx,&ty);
  point->data.point.centrum.x=tx;
  point->data.point.centrum.y=ty;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      if(point->data.point.point_type==POINT_RELATIVE)
	point->data.point.point_size=ABS(point->data.point.point_size*trans->scale_y);
    }
}

void obj_draw_point_block(DOCUMENT *doc, GdkGC *gc, OBJECT *point, TRANSFORM *trans)
{
  gdouble Ax, Ay, r;

  misc_transform_point(trans,point->data.point.centrum.x,point->data.point.centrum.y,&Ax,&Ay);
  
  r=point->data.point.point_size;

  if(trans!=NULL && (trans->type & OBJ_SCALE))
    {
      if(point->data.point.point_type==POINT_RELATIVE)
	r=ABS(r*trans->scale_y);
    }

  gdk_gc_set_line_attributes(gc,0,GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
  
  if(point->data.point.point_type==POINT_ABSOLUTE)
    draw_point(doc->widgets.draw->window, gc,
	       point->data.point.point_style, r,
	       WLDtoSCR_x(doc,Ax),WLDtoSCR_y(doc,Ay));
  else
    draw_point(doc->widgets.draw->window, gc,
	       point->data.point.point_style, WLDtoSCR(doc,r),
	       WLDtoSCR_x(doc,Ax), WLDtoSCR_y(doc,Ay));
}



void obj_draw_point(DOCUMENT *doc, OBJECT *point)
{
  obj_draw_point_block(doc,misc_setup_gc_object(doc, point),point,NULL);
}

gint obj_select_point(OBJECT *point, gdouble x, gdouble y, gdouble d)
{
  gdouble x1, y1, x2, y2;

  x1=point->data.point.centrum.x-point->data.point.point_size-d;
  y1=point->data.point.centrum.y-point->data.point.point_size-d;
  x2=point->data.point.centrum.x+point->data.point.point_size+d;
  y2=point->data.point.centrum.y+point->data.point.point_size+d;

  return ((x1<x) && (x<x2) && (y1<y) && (y<y2));
}





