/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "obj_polyline.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gint no_sides=4;
static GdkPoint *point_list=NULL;

/* --  P R O T O T Y P E S  -- */


/* --  F U N C T I O N S  -- */
void obj_set_prompt_polygon(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Number of sides <%d>: "),no_sides);
      break;
    case 1:
      set_prompt_text( _("Edge/<Center of polygon>: "));
      break;
    case 2:
      set_prompt_text( _("Radius of circle: "));
      break;
    case 3:
      set_prompt_text( _("First point of edge: "));
      break;
    case 4:
      set_prompt_text( _("Second point of edge: "));
      break;
    }
}

OBJECT *obj_add_polygon(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		     gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		     gdouble linewidth, gint sides, gdouble xcen, gdouble ycen, gdouble radius, gdouble sangle)
{
  OBJECT *temp=NULL;
  gint i;
  gdouble angle,x,y;

  if(layer==NULL)
    {
      message(0, _("There is no layer selected.\nPlease select a layer to use for the object."));
      return NULL;
    }

  angle=(2*M_PI)/sides;
  sangle=sangle*M_PI/180;

  for(i=0;i<no_sides;i++)
    {
      x=xcen+radius*cos(angle*i-sangle);
      y=ycen+radius*sin(angle*i-sangle);

      if(i==0)
	temp=create_new_polyline(doc,layer,NULL, type_colour, colour, type_linestyle, 
				 linestyle, type_linewidth, linewidth, TRUE);

      add_node(temp,NODE_LINE, x, y, linewidth, linewidth, 1);
    }

  obj_bound_polyline(temp);

  layer->object_list=g_list_append(layer->object_list,(gpointer)temp);

  doc->saved=FALSE;
  make_window_title(doc);

  return temp;
}

OBJECT *obj_add_polygon_spec(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		     gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		     gdouble linewidth, gint sides, gdouble x1,gdouble y1, gdouble x2,gdouble y2)
{
  OBJECT *temp=NULL;
  gint i;
  gdouble angle,tangle,r;
  gdouble ox1,oy1, ox2, oy2;
  GList *list;

  if(layer==NULL)
    {
      message(0, _("There is no layer selected.\nPlease select a layer to use for the object."));
      return NULL;
    }

  r=dist(x1,y1,x2,y2);

  tangle=get_angle(x1, y1,x2,y2)*M_PI/180;

  angle=2*M_PI/sides;

  temp=create_new_polyline(doc,layer,NULL, type_colour, colour, type_linestyle, 
			   linestyle, type_linewidth, linewidth, TRUE);

  add_node(temp,NODE_LINE, x1, y1, linewidth, linewidth,1);
  add_node(temp,NODE_LINE, x2, y2, linewidth, linewidth, 1);

  for(i=2;i<no_sides;i++)
    {
      list=g_list_last(temp->data.polyline.node_list);
      ox1=((NODE *)g_list_previous(list)->data)->x;
      oy1=((NODE *)g_list_previous(list)->data)->y;
      ox2=((NODE *)list->data)->x;
      oy2=((NODE *)list->data)->y;
      
      tangle=get_angle(ox1,oy1,ox2,oy2)*M_PI/180;

      add_node(temp,NODE_LINE, r*cos(-tangle-angle)+ox2,  r*sin(-tangle-angle)+oy2, linewidth, linewidth, 1);
    }

  obj_bound_polyline(temp);

  layer->object_list=g_list_append(layer->object_list,(gpointer)temp);

  doc->saved=FALSE;
  make_window_title(doc);

  return temp;
}

void obj_set_tool_polygon(GtkWidget *widget, gpointer data)
{
  remove_old();
  
  if(!command_from_keyboard)
    print( _("polygon"));
  
  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_POLYGON;
  gl_stage=0;
  work_document=NULL;
  obj_set_prompt_polygon();
  UpdateCoordsLabel();
}

void obj_button_polygon(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      //      print("*Cancel*");
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 1:  /* Center of polygon */
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=2;
      set_prompt();
      break;
    case 2:  /* Radius */
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
      
      obj_draw_polyline(doc, obj_add_polygon(doc,doc->current_layer, NULL, 
					     doc->type_colour, doc->current_colour, 
					     doc->type_linestyle, doc->current_linestyle,
					     doc->type_linewidth, doc->current_linewidth,
					     no_sides, gl_oldmx, gl_oldmy, 
					     dist(gl_oldmx, gl_oldmy, x,y),
					     get_angle(gl_oldmx, gl_oldmy, x,y)));     
      tool_idle(doc);
      break;
    case 3:  /* First point */
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=4;
      set_prompt();
      break;
    case 4:  /* Second point */
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      obj_draw_polyline(doc, obj_add_polygon_spec(doc,doc->current_layer, NULL, 
						  doc->type_colour, doc->current_colour, 
						  doc->type_linestyle, doc->current_linestyle,
						  doc->type_linewidth, doc->current_linewidth,
						  no_sides, gl_oldmx, gl_oldmy, x,y));      
      tool_idle(doc);
    }
}

void obj_prompt_polygon(DOCUMENT *doc, gchar *buffer)
{
  gdouble value, tx,ty;
  gint mn;

  switch(gl_stage)
    {
    case 0:  /* Number of sides */
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	if (value>2)
	  no_sides=value;
	else
	  {
	    print( _("\n  Number of sides in a polygon must be at least 3."));
	    set_prompt();
	    return;
	  }
      case PROMPT_RETURN:
	gl_stage=1;
	if(point_list!=NULL)  /* Make list for points */
	  g_free(point_list);
	point_list=(GdkPoint *)g_malloc((no_sides+1)*sizeof(GdkPoint));
	//	UpdateSingelCoord(2,dist1);
	set_prompt();
	break;
      }
      break;
    case 1:  /* Center point */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  FALSE, _("E/"),0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_polygon(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	gl_stage=3;
	UpdateCoordsLabel();
	set_prompt();
	break;
      }
      break;
    case 2:  /* Radius */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	remove_old();
             
	obj_draw_polyline(doc, obj_add_polygon(doc,doc->current_layer, NULL, 
					       doc->type_colour, doc->current_colour, 
					       doc->type_linestyle, doc->current_linestyle,
					       doc->type_linewidth, doc->current_linewidth,
					       no_sides, gl_oldmx, gl_oldmy, 
					       value,
					       0));     
	tool_idle(doc);
	break;
      }
      break;
    case 3:  /* First point */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_polygon(doc,1,tx,ty);
	break;
      }
      break;
    case 4:  /* Second point */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_polygon(doc,1,tx,ty);
	break;
      }
      break;
    }
}

void obj_xor_polygon(DOCUMENT *doc,gdouble x, gdouble y)
{
  gint i;
  gdouble r;
  gdouble angle;
  gdouble tangle;

  misc_xor_line(doc,x,y);

  r=dist(gl_oldmx,gl_oldmy,x,y);
  UpdateSingelCoord(2, r);

  tangle=get_angle(gl_oldmx, gl_oldmy,x,y)*M_PI/180;

  angle=(2*M_PI)/no_sides;

  for(i=0;i<no_sides;i++)
    {
      point_list[i].x=WLDtoSCR_x(doc,gl_oldmx+r*cos(angle*i-tangle));
      point_list[i].y=WLDtoSCR_y(doc,gl_oldmy+r*sin(angle*i-tangle));
    }

  point_list[no_sides].x=point_list[0].x;
  point_list[no_sides].y=point_list[0].y;

  gdk_draw_lines(doc->widgets.draw->window, 
		 misc_setup_gc(doc), point_list, no_sides+1);
}

void obj_xor_polygon_edge(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble tangle,angle, r;
  gint i;

  r=dist(gl_oldmx,gl_oldmy,x,y);

  tangle=get_angle(gl_oldmx, gl_oldmy,x,y)*M_PI/180;

  angle=2*M_PI/no_sides;

  point_list[0].x=WLDtoSCR_x(doc,gl_oldmx);
  point_list[0].y=WLDtoSCR_y(doc,gl_oldmy);
  point_list[1].x=WLDtoSCR_x(doc,x);
  point_list[1].y=WLDtoSCR_y(doc,y);

  for(i=2;i<no_sides;i++)
    {
      tangle=get_angle(point_list[i-2].x,point_list[i-2].y,point_list[i-1].x,point_list[i-1].y)*M_PI/180;

      point_list[i].x=WLDtoSCR(doc, r*cos(-tangle-angle)) + point_list[i-1].x;
      point_list[i].y=WLDtoSCR(doc, r*sin(-tangle-angle)) + point_list[i-1].y;
    }

  point_list[no_sides].x=point_list[0].x;
  point_list[no_sides].y=point_list[0].y;

  gdk_draw_lines(doc->widgets.draw->window, 
		 misc_setup_gc(doc), point_list, no_sides+1);

}






