/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "obj_polyline.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  P R O T O T Y P E S  -- */


/* --  F U N C T I O N S  -- */
void obj_set_prompt_rectangle(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("First corner: "));
      break;
    case 1:
      set_prompt_text( _("Other corner: "));
      break;
    }
}

OBJECT *obj_add_rectangle(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		     gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		     gdouble linewidth, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  OBJECT *temp;

  if(layer==NULL)
    {
      message(0, _("There is no layer selected.\nPlease select a layer to use for the object."));
      return NULL;
    }

  temp=create_new_polyline(doc,layer,NULL, type_colour, colour, type_linestyle, 
			   linestyle, type_linewidth, linewidth, TRUE);

  add_node(temp,NODE_LINE, x1, y1, linewidth, linewidth, 1); 
  add_node(temp,NODE_LINE, x2, y1, linewidth, linewidth, 1);
  add_node(temp,NODE_LINE, x2, y2, linewidth, linewidth, 1);
  add_node(temp,NODE_LINE, x1, y2, linewidth, linewidth, 1);

  obj_bound_polyline(temp);

  layer->object_list=g_list_append(layer->object_list,(gpointer)temp);

  doc->saved=FALSE;
  make_window_title(doc);

  return temp;
}

void obj_set_tool_rectangle(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("rectangle"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_RECTANGLE;
  gl_stage=0;
  work_document=NULL;
  obj_set_prompt_rectangle();
  UpdateCoordsLabel();
}

void obj_button_rectangle(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      //      print("*Cancel*");
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      obj_draw_polyline(doc, obj_add_rectangle(doc,doc->current_layer, NULL, 
					       doc->type_colour, doc->current_colour, 
					       doc->type_linestyle, doc->current_linestyle,
					       doc->type_linewidth, doc->current_linewidth,
					       gl_oldmx, gl_oldmy, x,y));
      tool_idle(doc);
      break;
    }
}

void obj_prompt_rectangle(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_rectangle(doc,1,tx,ty);
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_RETURN:
	remove_old();
	tool_idle(doc);
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_rectangle(doc,1,tx,ty);
	break;
      }
    }  
}

void obj_xor_rectangle(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdk_draw_rectangle(doc->widgets.draw->window, 
		     misc_setup_gc(doc),FALSE,
		     WLDtoSCR_x(doc,MIN(x,gl_oldmx)),
		     WLDtoSCR_y(doc,MIN(y,gl_oldmy)),
		     WLDtoSCR(doc,ABS(x-gl_oldmx)),WLDtoSCR(doc,ABS(y-gl_oldmy)));
}

