/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  P R O T O T Y P E S  -- */
void obj_draw_rline(DOCUMENT *doc, OBJECT *rline);

/* --  F U N C T I O N S  -- */
void obj_set_prompt_rline(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Start Point: "));
      break;
    case 1:
      set_prompt_text( _("Through point: "));
      break;
    }
}

void obj_bound_rline(OBJECT *obj)
{
  obj->bounding_box[0]=MIN(obj->data.line.p1.x,obj->data.line.p2.x);
  obj->bounding_box[1]=MIN(obj->data.line.p1.y,obj->data.line.p2.y);
  obj->bounding_box[2]=MAX(obj->data.line.p1.x,obj->data.line.p2.x);
  obj->bounding_box[3]=MAX(obj->data.line.p1.y,obj->data.line.p2.y);
}


OBJECT *obj_add_rline(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		      gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, 
		      gdouble linewidth, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }

  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_RLINE;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */
  obj->data.line.p1.x=x1;
  obj->data.line.p1.y=y1;
  obj->data.line.p2.x=x2;
  obj->data.line.p2.y=y2;
  
  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_rline(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void obj_set_tool_rline(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("ray"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_RLINE;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void obj_button_rline(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      //      print("*Cancel*");
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      obj_draw_rline(doc, obj_add_rline(doc,doc->current_layer, NULL, 
					doc->type_colour, doc->current_colour, 
					doc->type_linestyle, doc->current_linestyle,
					doc->type_linewidth, doc->current_linewidth,
					gl_oldmx, gl_oldmy, x,y));
      set_prompt();
      
      break;
    }
}

void obj_prompt_rline(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_rline(doc,1,tx,ty);
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_RETURN:
	remove_old();
	tool_idle(doc);
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_rline(doc,1,tx,ty);
	break;
      }
    }  
}

void calc_endpoint_for_ray(DOCUMENT *doc, gdouble x1,gdouble y1, gdouble x2, gdouble y2, gdouble *x, gdouble *y)
{
  gdouble wx1,wy1,wx2,wy2;
  gdouble m,b;
  gdouble nx1, ny1, nx2, ny2;

  wx1=SCRtoWLD_x(doc,-1);
  wy1=SCRtoWLD_y(doc,-1);
  wx2=SCRtoWLD_x(doc,doc->widgets.draw->allocation.width);
  wy2=SCRtoWLD_y(doc,doc->widgets.draw->allocation.height);


  if( ABS(x2-x1)<GFLOAT_SENSITIVE)  /* A vertical line */
    {
      nx2=nx1=x1;
      ny1=wy1;
      ny2=wy2;
    }
  else if( ABS(y2-y1)<GFLOAT_SENSITIVE) /* A horizontal line */
    {
      ny2=ny1=y1;
      nx1=wx1;
      nx2=wx2;
    }
  else
    {
      m=(y2-y1)/(x2-x1);

      b=y1-m*x1;

      /* Calculate intersection with left */
      nx1=wx1;
      ny1=m*wx1+b;
      
      /* Calculate intersection with right */
      nx2=wx2;
      ny2=m*wx2+b;
      
      if(ny1<wy1)
	{
	  ny1=wy1;
	  nx1=(wy1-b)/m;
	}
      if(ny1>wy2)
	{
	  ny1=wy2;
	  nx1=(wy2-b)/m;
	}

      if(ny2<wy1)
	{
	  ny2=wy1;
	  nx2=(wy1-b)/m;
	}
      if(ny2>wy2)
	{
	  ny2=wy2;
	  nx2=(wy2-b)/m;
	} 
    }

  /* Which point is closest to x2,y2 */

  if(dist(nx1,ny1, x2,y2)<dist(x1,y1,nx1,ny1))
    {
      *x=nx1;
      *y=ny1;
    }
  else
    {
      *x=nx2;
      *y=ny2;
    }
}

void obj_xor_rline(DOCUMENT *doc,gdouble x, gdouble y)
{
  gdouble nx,ny;

  calc_endpoint_for_ray(doc,gl_oldmx,gl_oldmy,x,y, &nx, &ny);

  gdk_draw_line(doc->widgets.draw->window, 
		misc_setup_gc(doc),
		WLDtoSCR_x(doc,gl_oldmx),WLDtoSCR_y(doc,gl_oldmy),
		WLDtoSCR_x(doc,nx),WLDtoSCR_y(doc,ny));

}

void obj_transform_rline(OBJECT *rline, TRANSFORM *trans)
{
  gdouble tx,ty;

  misc_transform_point_select(trans,&rline->data.line.p1,&tx,&ty);
  rline->data.line.p1.x=tx;
  rline->data.line.p1.y=ty;

  misc_transform_point_select(trans,&rline->data.line.p2,&tx,&ty);
  rline->data.line.p2.x=tx;
  rline->data.line.p2.y=ty;
  
  obj_bound_rline(rline);
}

void obj_draw_rline_block(DOCUMENT *doc, GdkGC *gc, OBJECT *rline, TRANSFORM *trans)
{
  gdouble Ax, Ay, Bx, By;

  misc_transform_point_select(trans,&rline->data.line.p1,&Ax,&Ay);
  misc_transform_point_select(trans,&rline->data.line.p2,&Bx,&By);

  calc_endpoint_for_ray(doc, Ax,Ay, Bx,By, &Bx, &By);

  gdk_draw_line(doc->widgets.draw->window, 
		gc,
		WLDtoSCR_x(doc,Ax), 
		WLDtoSCR_y(doc,Ay),
		WLDtoSCR_x(doc,Bx), 
		WLDtoSCR_y(doc,By));
}

void obj_draw_rline(DOCUMENT *doc, OBJECT *rline)
{
  obj_draw_rline_block(doc,misc_setup_gc_object(doc, rline),rline,NULL);
}

gint obj_select_rline(OBJECT *rline, gdouble x, gdouble y, gdouble d)
{
  gdouble L;
  gdouble s,r;
  gdouble Ax, Ay, Bx, By;
  
  Ax=rline->data.line.p1.x;
  Ay=rline->data.line.p1.y;
  Bx=rline->data.line.p2.x;
  By=rline->data.line.p2.y;

  calc_endpoint_for_ray(rline->layer->doc,Ax,Ay,Bx,By,&Bx,&By);

  L=sqrt( (Bx-Ax)*(Bx-Ax) + (By-Ay)*(By-Ay) );

  r=((Ay-y)*(Ay-By)-(Ax-x)*(Bx-Ax))/(L*L);

  if((r>=0) && (r<=1)){
    s=((Ay-y)*(Bx-Ax)-(Ax-x)*(By-Ay))/(L*L);
    return( ABS(s*L)<(d+rline->linewidth/2));
  }
    
  return FALSE;
}









