/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  P R O T O T Y P E S  -- */
void calc_bspline(OBJECT *spline);
void calc_closed_bspline(OBJECT *spline);
void obj_draw_spline(DOCUMENT *doc, OBJECT *spline);
void obj_draw_spline_block(DOCUMENT *doc, GdkGC *gc, OBJECT *spline, TRANSFORM *trans);


/* --  F U N C T I O N S  -- */
void obj_set_prompt_spline(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Start Point: "));
      break;
    case 1:
      set_prompt_text( _("Next Point: "));
      break;
    case 2:
      set_prompt_text( _("Close/Undo/<Next Point>: "));
      break;
    }
}

void obj_bound_spline(OBJECT *obj)
{
  gint i,num;
  POINT *list;

  /* If no cache list exists create one */
  if(obj->data.spline.cache_point==NULL)
    calc_bspline(obj);
  
  num=obj->data.spline.num;

  list=obj->data.spline.cache_point;


  obj->bounding_box[0]=list[0].x;
  obj->bounding_box[1]=list[0].y;
  obj->bounding_box[2]=list[0].x;
  obj->bounding_box[3]=list[0].y;

  i=1;

  while(i<num)
    {
        obj->bounding_box[0]=MIN( obj->bounding_box[0],list[i].x);
        obj->bounding_box[1]=MIN( obj->bounding_box[1],list[i].y);
        obj->bounding_box[2]=MAX( obj->bounding_box[2],list[i].x);
        obj->bounding_box[3]=MAX( obj->bounding_box[3],list[i].y);
      i++;
    }
}

NODE *spline_add_node(OBJECT *spline, gdouble x, gdouble y, gdouble w)
{
  NODE *temp;

  temp=(NODE *)g_malloc(sizeof(NODE));

  temp->selected=FALSE;
  temp->type=0;
  temp->x=x;
  temp->y=y;
  temp->weight=w;

  spline->data.spline.node_list=g_list_append(spline->data.spline.node_list, (gpointer)temp);

  if(spline->data.spline.cache_point!=NULL)
    g_free(spline->data.spline.cache_point);

  spline->data.spline.cache_point=NULL;

  return temp;
}

OBJECT *create_new_spline(DOCUMENT *doc, LAYER *layer, gchar *name, 
			  gint type_colour, COLOUR *colour, 
			  gint type_linestyle, LINESTYLE *linestyle, 
			  gint type_linewidth, gdouble linewidth,
			  gint closed)
{
  OBJECT *obj;

  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }
  
  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_SPLINE;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */  
  obj->data.spline.node_list=NULL;
  obj->data.spline.closed=closed;
  obj->data.spline.cache_point=NULL;

 /* Return new object */
  return obj;
}

void obj_set_tool_spline(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("spline"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_SPLINE;
  gl_stage=0;
  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
}

void obj_button_spline(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {    
      remove_old();
      if(work_document!=NULL && doc->working_polyline!=NULL)
	{
	  obj_bound_spline(doc->working_polyline);
	  set_current_attributes(doc, doc->working_polyline);
	  obj_draw_spline(doc, doc->working_polyline);
	  doc->current_layer->object_list=g_list_append(doc->current_layer->object_list,(gpointer)doc->working_polyline);
	  doc->working_polyline=NULL;      
	  doc->saved=FALSE;
	  make_window_title(doc);
	}
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      /* Do we have an old polyline? If so erase it. */
      if(doc->working_polyline!=NULL)  /* <-- Memory leak!!! The nodes are not released */
	g_free(doc->working_polyline);
      
      doc->working_polyline=NULL;
      
      if(doc->gl_snap)
	make_snap(doc, &x, &y);

      doc->working_polyline=create_new_spline(doc, doc->current_layer, NULL, 
					      doc->type_colour, doc->current_colour, 
					      doc->type_linestyle, doc->current_linestyle,
					      doc->type_linewidth, doc->current_linewidth,
					      FALSE);
      spline_add_node(doc->working_polyline, x,y,1);

      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      spline_add_node(doc->working_polyline, x,y, 1);

      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=2;
      set_prompt();
      break;
    case 2:
      remove_old();

      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      spline_add_node(doc->working_polyline, x,y, 1);

      gl_oldmx=x;
      gl_oldmy=y;
      set_prompt();
      break;
    }

  UpdateSingelCoord(0, gl_oldmx);
  UpdateSingelCoord(1, gl_oldmy);
}

void undo_spline(DOCUMENT *doc, OBJECT *spline)
{
  GList *list;

  list=g_list_last(spline->data.spline.node_list);

  if(g_list_previous(list)==NULL && g_list_next(list)==NULL)
    {
      print( _("\n  Cannot undo first point."));
    }
  else
    {
      remove_old();

      gl_oldmx=((NODE *)g_list_previous(list)->data)->x;
      gl_oldmy=((NODE *)g_list_previous(list)->data)->y;

      UpdateSingelCoord(0, gl_oldmx);
      UpdateSingelCoord(1, gl_oldmy);
     
      spline->data.polyline.node_list=g_list_remove(spline->data.spline.node_list, list->data);

      /* Is this the first point */
      if(g_list_length(spline->data.polyline.node_list)==1)
	gl_stage=1;
    }
  set_prompt();
}

void obj_prompt_spline(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gint mn;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_spline(doc,1,tx,ty);
	break;
      }
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_spline(doc,1,tx,ty);
	break;
      }
      break;
    case 2:

      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2|PROMPT_MENU,buffer,
			  TRUE, _("C/U/"),gl_oldmx,gl_oldmy,&tx,&ty,NULL,&mn)){
      case PROMPT_RETURN:
	remove_old();
	tool_idle(doc);
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_spline(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	if(mn==0)
	  {
	    remove_old();
	    doc->working_polyline->data.spline.closed=TRUE;
	    obj_bound_spline(doc->working_polyline);
	    set_current_attributes(doc, doc->working_polyline);
	    obj_draw_spline(doc, doc->working_polyline);
	    doc->current_layer->object_list=g_list_append(doc->current_layer->object_list,(gpointer)doc->working_polyline);
	    doc->working_polyline=NULL;      
	    doc->saved=FALSE;
	    make_window_title(doc);
	    tool_idle(doc);
	  }
	else
	    undo_spline(doc, doc->working_polyline);	    

	break;
      }
    }  
}

void obj_xor_spline(DOCUMENT *doc,gdouble x, gdouble y)
{
  NODE *data;

  data=spline_add_node(doc->working_polyline, x,y, 1);

  obj_draw_spline_block(doc, misc_setup_gc(doc), doc->working_polyline, NULL);

  doc->working_polyline->data.spline.node_list=g_list_remove(doc->working_polyline->data.spline.node_list,data);

  doc->working_polyline->data.spline.cache_point=NULL;  

}

void calc_bspline(OBJECT *spline)
{
  gint i,num,j;
  POINT *p;
  POINT *pn;
  GList *list;

  gdouble FAX,FBX,FCX,FDX;
  gdouble FAY,FBY,FCY,FDY;

  gint InterPts=20,n;
  gdouble T,FX,FY;

  if(spline->data.spline.closed)
    {
      calc_closed_bspline(spline);
      return;
    }

  num=g_list_length(spline->data.spline.node_list);

  pn=g_new(POINT, num+3);

  /* Transfer node_point to an XY-array */
  list=spline->data.spline.node_list;
  
  /* Repeat data at start point */
  pn[0].x=((NODE *)list->data)->x;
  pn[0].y=((NODE *)list->data)->y;

  i=1;
  while(list!=NULL)
    {
      pn[i].x=((NODE *)list->data)->x;
      pn[i].y=((NODE *)list->data)->y;
      list=g_list_next(list);
      i++;
    }

  /* Repeat data at start point */
  pn[i].x=pn[i-1].x;
  pn[i].y=pn[i-1].y;
  pn[i+1].x=pn[i-1].x;
  pn[i+1].y=pn[i-1].y;

  /* Somewhere to store points */
  p=g_new(POINT, num*(InterPts+1)+1);

  /* Calc B-spline */
  
  p[0].x=pn[0].x;
  p[0].y=pn[0].y;
  
  for(i=1, n=1; n<=num; n++)
    {
      FAX=(-1.0/6.0)*pn[n-1].x+(3.0/6.0)*pn[n].x+(-3.0/6.0)*pn[n+1].x+(1.0/6.0)*pn[n+2].x;
      FBX=(3.0/6.0)*pn[n-1].x+(-6.0/6.0)*pn[n].x+(3.0/6.0)*pn[n+1].x;
      FCX=(-3.0/6.0)*pn[n-1].x+(3.0/6.0)*pn[n+1].x;
      FDX=(1.0/6.0)*pn[n-1].x+(4.0/6.0)*pn[n].x+(1.0/6.0)*pn[n+1].x;
      
      FAY=(-1.0/6.0)*pn[n-1].y+(3.0/6.0)*pn[n].y+(-3.0/6.0)*pn[n+1].y+(1.0/6.0)*pn[n+2].y;
      FBY=(3.0/6.0)*pn[n-1].y+(-6.0/6.0)*pn[n].y+(3.0/6.0)*pn[n+1].y;
      FCY=(-3.0/6.0)*pn[n-1].y+(3.0/6.0)*pn[n+1].y;
      FDY=(1.0/6.0)*pn[n-1].y+(4.0/6.0)*pn[n].y+(1.0/6.0)*pn[n+1].y;
      
      for(j=0;j<=InterPts;j++)
	{
	  T=(gdouble)j/(gdouble)(InterPts);
	  
	  FX=((FAX*T+FBX)*T+FCX)*T+FDX;
	  FY=((FAY*T+FBY)*T+FCY)*T+FDY;
	  
	  p[i].x=FX;
	  p[i].y=FY;
	  i++;
	}      
    }

  spline->data.spline.cache_point=p;
  spline->data.spline.num=i;
  
  /* Free temp lists */
  g_free(pn);
}

void calc_closed_bspline(OBJECT *spline)
{
  gint i,num,j;
  POINT *p;
  POINT *pn;
  GList *list;

  gdouble FAX,FBX,FCX,FDX;
  gdouble FAY,FBY,FCY,FDY;

  gint InterPts=20,n;
  gdouble T,FX,FY;

  num=g_list_length(spline->data.spline.node_list);

  pn=g_new(POINT, num+3+3+3);

  /* Transfer node_point to an XY-array */
  list=spline->data.spline.node_list;
  
  i=4;
  while(list!=NULL)
    {
      pn[i].x=((NODE *)list->data)->x;
      pn[i].y=((NODE *)list->data)->y;
      list=g_list_next(list);
      i++;
    }

  /* Repeat start data */
  pn[0].x=pn[i-3].x;
  pn[0].y=pn[i-3].y;
  pn[1].x=pn[i-3].x;
  pn[1].y=pn[i-3].y;
  pn[2].x=pn[i-2].x;
  pn[2].y=pn[i-2].y;
  pn[3].x=pn[i-1].x;
  pn[3].y=pn[i-1].y;

  /* Repeat end data */
  pn[i].x=pn[4].x;
  pn[i].y=pn[4].y;
  pn[i+1].x=pn[5].x;
  pn[i+1].y=pn[5].y;
  pn[i+2].x=pn[6].x;
  pn[i+2].y=pn[6].y;
  pn[i+3].x=pn[6].x;
  pn[i+3].y=pn[6].y;
  pn[i+3].x=pn[6].x;
  pn[i+3].y=pn[6].y;

  /* Somewhere to store points */
  p=g_new(POINT, num*(InterPts+1)+1);

  /* Calc B-spline */
  for(i=0, n=1; n<=num+3+3; n++)
    {
      FAX=(-1.0/6.0)*pn[n-1].x+(3.0/6.0)*pn[n].x+(-3.0/6.0)*pn[n+1].x+(1.0/6.0)*pn[n+2].x;
      FBX=(3.0/6.0)*pn[n-1].x+(-6.0/6.0)*pn[n].x+(3.0/6.0)*pn[n+1].x;
      FCX=(-3.0/6.0)*pn[n-1].x+(3.0/6.0)*pn[n+1].x;
      FDX=(1.0/6.0)*pn[n-1].x+(4.0/6.0)*pn[n].x+(1.0/6.0)*pn[n+1].x;
      
      FAY=(-1.0/6.0)*pn[n-1].y+(3.0/6.0)*pn[n].y+(-3.0/6.0)*pn[n+1].y+(1.0/6.0)*pn[n+2].y;
      FBY=(3.0/6.0)*pn[n-1].y+(-6.0/6.0)*pn[n].y+(3.0/6.0)*pn[n+1].y;
      FCY=(-3.0/6.0)*pn[n-1].y+(3.0/6.0)*pn[n+1].y;
      FDY=(1.0/6.0)*pn[n-1].y+(4.0/6.0)*pn[n].y+(1.0/6.0)*pn[n+1].y;
      
      for(j=0;j<=InterPts;j++)
	{
	  T=(gdouble)j/(gdouble)(InterPts);
	  
	  FX=((FAX*T+FBX)*T+FCX)*T+FDX;
	  FY=((FAY*T+FBY)*T+FCY)*T+FDY;
	
	  if(n>3 && n<=num+3)
	    { 
	      p[i].x=FX;
	      p[i].y=FY;
	      i++;
	    }
	}      
    }

  spline->data.spline.cache_point=p;
  spline->data.spline.num=i;
  
  /* Free temp lists */
  g_free(pn);
}

void obj_transform_spline(OBJECT *spline, TRANSFORM *trans)
{
  GList *node_list;
  gdouble tx,ty;

  node_list=g_list_first(spline->data.spline.node_list);
  
  while(node_list!=NULL)
    {
      misc_transform_point(trans, ((NODE *)node_list->data)->x, ((NODE *)node_list->data)->y,&tx,&ty);
      ((NODE *)node_list->data)->x=tx;
      ((NODE *)node_list->data)->y=ty;
      node_list=g_list_next(node_list);
    }

  calc_bspline(spline);

  obj_bound_spline(spline);
}

void obj_draw_spline_block(DOCUMENT *doc, GdkGC *gc, OBJECT *spline, TRANSFORM *trans)
{
  GdkPoint *p;
  POINT *list;
  gint i,num;
  gdouble x,y;

  /* If no cache list exists create one */
  if(spline->data.spline.cache_point==NULL)
     calc_bspline(spline);

  num=spline->data.spline.num;

  /* Transform world coordinates to screen */
  p=g_new(GdkPoint, num);

  list=spline->data.spline.cache_point;

  i=0;

  while(i<num)
    {
      misc_transform_point(trans, list[i].x, list[i].y, &x,&y);

      p[i].x=WLDtoSCR_x(doc, x);
      p[i].y=WLDtoSCR_y(doc, y);
      i++;
    }

  /* Draw lines */
  gdk_draw_lines(doc->widgets.draw->window, gc, p,num);     
}

void obj_draw_spline(DOCUMENT *doc, OBJECT *spline)
{
  obj_draw_spline_block(doc,misc_setup_gc_object(doc, spline),spline,NULL);
}


gint obj_select_spline(OBJECT *spline, gdouble x, gdouble y, gdouble d)
{
  POINT *list;
  gint i,num;
  gdouble Ax, Ay, Bx, By;

  /* If no cache list exists create one */
  if(spline->data.spline.cache_point==NULL)
     calc_bspline(spline);
 
  num=spline->data.spline.num;

  list=spline->data.spline.cache_point;

  i=1;

  while(i<num)
    {
      Ax=list[i-1].x;
      Ay=list[i-1].y;
      Bx=list[i].x;
      By=list[i].y;
 
      if(misc_point_on_line(x,y,Ax,Ay,Bx,By,d))
	return TRUE;
    
      i++;
    }

  return FALSE;
}












