/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "dia_coords.h"
#include "textstyle.h"
#include "xor.h"
#include "misc.h"
#include "draw.h"
#include "parse.h"
#include "select.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gint def_just=TEXT_JUST_BL;
static gdouble def_height=10, def_rot=0;
static gdouble def_ins_x, def_ins_y;
static gdouble def_align_x, def_align_y;
static TEXTSTYLE *def_style=NULL;

/* --  P R O T O T Y P E S  -- */
void obj_draw_text(DOCUMENT *doc, OBJECT *line);


/* --  F U N C T I O N S  -- */
void obj_set_prompt_text(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Style/Justify/<Insert Point>: "));
      break;
    case 1:
      set_prompt_text( _("Height <%f>: "), def_height);
      break;
    case 2:
      set_prompt_text( _("Rotation <%f>: "), def_rot);
      break;
    case 3:
      set_prompt_text( _("Text: "));
      break;
    case 7:
    case 9:
    case 4:
      set_prompt_text( _("Other point: "));
      break;
    case 5:
      set_prompt_text( _("Align/Fit/TL/TC/TR/ML/MC/MR/BL/BC/BR: "));
      break;
    case 6:
    case 8:
      set_prompt_text( _("First point: "));
      break;
    case 10:
      set_prompt_text( _("Select textstyle name: "));
      break;
    }
}

void obj_bound_text(OBJECT *obj)
{
  gint type;
  gdouble tx,ty;

  draw_text_set_text(obj->data.text.justification,obj->data.text.insert.x,obj->data.text.insert.y,
		     obj->data.text.height,obj->data.text.width, obj->data.text.angle*M_PI/180.0, obj->data.text.text);

  if(!draw_text_get_vertex(&type, &tx,&ty))
    return;

  obj->bounding_box[0]=obj->bounding_box[2]=tx;
  obj->bounding_box[1]=obj->bounding_box[3]=ty;

  while(draw_text_get_vertex(&type, &tx,&ty))
	{
	  obj->bounding_box[0]=MIN(obj->bounding_box[0],tx);
	  obj->bounding_box[1]=MIN(obj->bounding_box[1],ty);
	  obj->bounding_box[2]=MAX(obj->bounding_box[2],tx);
	  obj->bounding_box[3]=MAX(obj->bounding_box[3],ty);
	}
}

OBJECT *obj_add_text(DOCUMENT *doc, LAYER *layer, gchar *name, gint type_colour, COLOUR *colour, 
		     gint type_linestyle, LINESTYLE *linestyle, gint type_linewidth, gdouble linewidth, 
		     TEXTSTYLE *style, gint just, gdouble angle, gdouble height, 
		     gdouble ins_x, gdouble ins_y, gdouble align_x, gdouble align_y,
		     gchar *text)
{
  OBJECT *obj;
  
  /* Make sure we have valid parameters */
  if(doc==NULL || layer==NULL)
    {
      g_warning( _("Unvalid parameters for function %s in file %s"), __FUNCTION__, __FILE__);
      return NULL;
    }
  
  /* Create object */
  obj=g_new(OBJECT, 1);
  
  /* Type of object */
  obj->type=TYPE_TEXT;
  
  /* Pointer to layer */
  obj->layer=layer;

  /* Set up attributes for this object */
  obj_set_attributes(doc, obj, name, type_colour, colour, type_linestyle, 
		     linestyle, type_linewidth, linewidth); 
  
  /* Set object specific attributes */
  obj->data.text.insert.x=ins_x;
  obj->data.text.insert.y=ins_y;
  obj->data.text.alignment.x=align_x;
  obj->data.text.alignment.y=align_y;
  obj->data.text.justification=just;
  obj->data.text.text=g_strdup(text);
  obj->data.text.angle=angle;
  obj->data.text.style=style;

  if(style==NULL)
    {
      obj->data.text.flags=0;
      obj->data.text.height=height;
      obj->data.text.oblique=0;
      obj->data.text.width=1;
    }
  else
    {
      obj->data.text.flags=style->flags;
      if(style->height<GFLOAT_SENSITIVE)
	obj->data.text.height=height;
      else
	obj->data.text.height=style->height;

      obj->data.text.oblique=style->oblique;
      obj->data.text.width=style->width;
    }

  /* Add object to layer list */
  layer->object_list=g_list_append(layer->object_list,obj);

  /* Make bounding box for object */
  obj_bound_text(obj);

  doc->saved=FALSE;
  make_window_title(doc);

  /* Return new object */
  return obj;
}

void obj_set_tool_text(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("text"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_object_get_document()==NULL)
    return;
  
  gl_tool=TOOL_TEXT;
  gl_stage=0;
  // def_style=NULL;
  //  def_just=TEXT_JUST_BL;

  work_document=NULL;
  set_prompt();
  set_cursor(NULL);
  UpdateCoordsLabel();
  UpdateSingelCoord(2,def_height);
  UpdateSingelCoord(3,def_rot);
}

void obj_button_text(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      def_ins_x=x;
      def_ins_y=y;

      if(def_style!=NULL && def_style->height>GFLOAT_SENSITIVE)
	{
	  def_height=def_style->height;
	  UpdateSingelCoord(2,def_height);
	  gl_stage=2;
	}
      else
	gl_stage=1;
      set_prompt();
      break;
    case 1:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=4;
      set_prompt();
      break;
    case 4:
      remove_old();
      make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
      gl_stage=3;
      def_height=dist(gl_oldmx, gl_oldmy, x,y);
      def_rot=get_angle(gl_oldmx, gl_oldmy, x,y)-90;
      set_prompt();
      break;
    case 6:
    case 8:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=def_ins_x=x;
      gl_oldmy=def_ins_y=y;
      gl_stage++;
      set_prompt();
      break;
    case 7:
    case 9:
      remove_old();
      make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
      def_align_x=x;
      def_align_y=y;
      gl_stage=3;
      set_prompt();
    }
}

void obj_prompt_text(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gdouble value;
  gint mn;

  switch(gl_stage)
    {
    case 0:  /* Insert point */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,buffer,
			  FALSE, _("S/J/"),0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_text(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	if(mn==1)
	  gl_stage=5;
	if(mn==0)
	  gl_stage=10;
	set_prompt();
	break;
      }
      break;
    case 1:  /* Height */
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	def_height=value;
      case PROMPT_RETURN:
	gl_stage=2;
	UpdateSingelCoord(2,def_height);
	set_prompt();
	break;
      }   
      break;
    case 2:  /* Rotation */
      switch(parse_prompt(PROMPT_RETURN|PROMPT_OSNAP|PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	def_rot=value;
      case PROMPT_RETURN:
	gl_stage=3;
	UpdateSingelCoord(3,def_rot);
	set_prompt();
	break;
      }   
      break;
    case 3:  /* Text */      
      obj_draw_text(doc, obj_add_text(doc,doc->current_layer, NULL,doc->type_colour, doc->current_colour, 
				      doc->type_linestyle, doc->current_linestyle,
				      doc->type_linewidth, doc->current_linewidth,
				      def_style,def_just, def_rot, def_height,def_ins_x,def_ins_y,
				      def_align_x,def_align_y,buffer));
      tool_idle(doc);
      break;
    case 5:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_MENU,buffer,
			  FALSE, _("A/F/TL/TC/TR/ML/MC/MR/BL/BC/BR/"),0,0,NULL,NULL,NULL,&mn)){
      case PROMPT_MENU:
	if(mn==0)
	  gl_stage=6;
	else if(mn==1)
	  gl_stage=8;
	else
	  gl_stage=0;

	def_just=mn;
	set_prompt();
	break;
      }
      break;
    case 6:
    case 8:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_text(doc,1,tx,ty);
	break;
      }
      break;
    case 7:
    case 9:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	obj_button_text(doc,1,tx,ty);
	break;
      }
    case 10:
      def_style=textstyle_find(doc,buffer);
      if(def_style==NULL)
	print( _("\n  Cannot find textstyle."));

      gl_stage=0;
      set_prompt();
      break;
    }
}

void obj_xor_text(DOCUMENT *doc,gdouble x, gdouble y)
{
  misc_xor_line(doc,x,y);

  UpdateSingelCoord(2,dist(gl_oldmx, gl_oldmy, x,y) );
  UpdateSingelCoord(3,get_angle(gl_oldmx, gl_oldmy, x,y)-90 );
  
}

void obj_transform_text(OBJECT *text, TRANSFORM *trans)
{
  gdouble tx,ty;

  if( (text->data.text.justification==TEXT_JUST_AL) ||  (text->data.text.justification==TEXT_JUST_FI) )
    {     
      misc_transform_point_select(trans, &text->data.text.insert, &tx,&ty);
      text->data.text.insert.x=tx;
      text->data.text.insert.y=ty;
      misc_transform_point_select(trans, &text->data.text.alignment, &tx,&ty);
      text->data.text.alignment.x=tx;
      text->data.text.alignment.y=ty;
    }
  else
    {  
      /* Scale height */
      if( (trans!=NULL) && (trans->type & OBJ_SCALE))
	text->data.text.height*=trans->scale_x;
      
      misc_transform_point_select(trans, &text->data.text.insert, &tx,&ty);
      text->data.text.insert.x=tx;
      text->data.text.insert.y=ty;

      if(trans!=NULL && (trans->type & OBJ_ROTATE))
	text->data.text.angle-=(trans->rot_angle*180.0/M_PI);
    }
  
  obj_bound_text(text);
}

void obj_draw_text_block(DOCUMENT *doc, GdkGC *gc, OBJECT *text, TRANSFORM *trans)
{
  gdouble x,y,a;
  gdouble h,w;

  if(text->data.text.justification==TEXT_JUST_AL)
    {
      
      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      h=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text));
      
      misc_transform_point_select(trans, &text->data.text.insert, &x,&y);


      if(trans!=NULL && (trans->type & OBJ_ROTATE))
	 a-=trans->rot_angle;

      draw_text2(doc, gc, text->data.text.justification,
		x, y, h, 1, a, text->data.text.text);
    }
  else if(text->data.text.justification==TEXT_JUST_FI)
    {

      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      w=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text)*text->data.text.height);

      //      misc_transform_point(trans, text->data.text.insert.x, text->data.text.insert.y, &x,&y);
      misc_transform_point_select(trans, &text->data.text.insert, &x,&y);

      if(trans!=NULL && (trans->type & OBJ_ROTATE))
	 a-=trans->rot_angle;

      draw_text2(doc,gc,text->data.text.justification,
		x, y, text->data.text.height, w, a, text->data.text.text);
    }
  else
    {  
      a=text->data.text.angle*M_PI/180.0;

      misc_transform_point_select(trans, &text->data.text.insert, &x,&y);

      h=text->data.text.height;
      w=text->data.text.width;

      /* Scale height */
      if( (trans!=NULL) && (trans->type & OBJ_SCALE))
	h*=trans->scale_x;

      if(trans!=NULL && (trans->type & OBJ_ROTATE))
	 a-=trans->rot_angle;

      draw_text2(doc, gc, text->data.text.justification, 
		 x, y, h, w, a, text->data.text.text);
    }
}

void obj_draw_text(DOCUMENT *doc, OBJECT *text)
{
  obj_draw_text_block(doc,misc_setup_gc_object(doc, text),text,NULL);
}

gint obj_select_text(OBJECT *text, gdouble x, gdouble y, gdouble d)
{
  if(text->data.text.justification==TEXT_JUST_AL)
    {
      gdouble h,a;
      
      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      h=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text));
      
      return(select_text(x,y,d,text->data.text.justification, text->data.text.insert.x,text->data.text.insert.y,
			 h,1,a,text->data.text.text));
    }
  else if(text->data.text.justification==TEXT_JUST_FI)
    {
      gdouble w,a;

      a=get_angle(text->data.text.insert.x,text->data.text.insert.y,
		  text->data.text.alignment.x,text->data.text.alignment.y)*M_PI/180;

      w=dist(text->data.text.insert.x,text->data.text.insert.y,text->data.text.alignment.x,text->data.text.alignment.y)
	/((gdouble)strlen(text->data.text.text)*text->data.text.height);

      return(select_text(x,y,d,text->data.text.justification, text->data.text.insert.x,text->data.text.insert.y,
			 text->data.text.height,1,a,text->data.text.text));
    }
  else
    return(select_text(x,y,d,text->data.text.justification, text->data.text.insert.x,text->data.text.insert.y,
		       text->data.text.height,text->data.text.width,text->data.text.angle*M_PI/180.0,text->data.text.text));

  return FALSE;
}












