/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "translation.h"
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "osnap.h"
#include "dia_comm.h"
#include "menuitem.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_attribute.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_rectangle.h"
#include "obj_polygon.h"
#include "obj_spline.h"
#include "obj_insert.h"
#include "dim_horiz.h"
#include "dim_vert.h"
#include "dim_aligned.h"
#include "dim_radius.h"
#include "dim_diameter.h"
#include "dim_center.h"
#include "dim_continue.h"
#include "dim_baseline.h"
#include "dim_leader.h"
#include "dim_angular.h"
#include "tool_zoom.h"
#include "tool_copy.h"
#include "tool_erase.h"
#include "tool_offset.h"
#include "tool_array.h"
#include "tool_move.h"
#include "tool_mirror.h"
#include "tool_rotate.h"
#include "tool_trim.h"
#include "tool_idpoint.h"
#include "tool_dist.h"
#include "tool_list.h"
#include "tool_fillet.h"
#include "tool_break.h"
#include "tool_chamfer.h"
#include "tool_stretch.h"
#include "tool_scale.h"
#include "tool_block.h"
#include "edit_text.h"
#include "edit_prop.h"
#include "variable.h"
#include "select.h"
#include "misc.h"
#include "redraw.h"
#include "xor.h"
#include "tool_block.h"
#include "exprparser.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
#define NO_OF_MENUTOKENS 12

typedef struct 
{
  gchar *com;
  void (*func)(GtkWidget *, gpointer);
  guint data;
  gint update_prompt;
} PARSE_WORDS;

static PARSE_WORDS parse_list[]=
{
  /* File */
  { N_("new"),       menu_new,                 0,  TRUE},
  { N_("quit"),      menu_quit,                0,  TRUE},

  /* View */
  { N_("zoom"),      tool_set_tool_zoom,       0, FALSE},
  { N_("pan"),       tool_set_tool_zoom,       2, FALSE},
  { N_("redraw"),    redraw,                   0, FALSE},

  /* Assist */
  { N_("cen"),       set_snap,         OSNAP_CEN, FALSE},
  { N_("end"),       set_snap,         OSNAP_END, FALSE},
  { N_("ins"),       set_snap,         OSNAP_INS, FALSE},
  { N_("int"),       set_snap,         OSNAP_INT, FALSE},
  { N_("mid"),       set_snap,         OSNAP_MID, FALSE},
  { N_("nea"),       set_snap,         OSNAP_NEA, FALSE},
  { N_("nod"),       set_snap,         OSNAP_NOD, FALSE},
  { N_("per"),       set_snap,         OSNAP_PER, FALSE},
  { N_("qua"),       set_snap,         OSNAP_QUA, FALSE},
  { N_("tan"),       set_snap,         OSNAP_TAN, FALSE},
  { N_("idpoint"),   tool_set_tool_idpoint,   0, FALSE},
  { N_("distance"),  tool_set_tool_dist,      0, FALSE},
  { N_("dist"),      tool_set_tool_dist,      0, FALSE},
  { N_("list"),      tool_set_tool_list,      0, FALSE},

  /* Objects */
  { N_("line"),      obj_set_tool_line,        0, FALSE},
  { N_("pline"),     obj_set_tool_polyline,    0, FALSE},
  { N_("ray"),       obj_set_tool_rline,       0, FALSE},
  { N_("xline"),     obj_set_tool_cline,       0, FALSE},
  { N_("spline"),    obj_set_tool_spline,      0, FALSE},
  { N_("arc"),       obj_set_tool_arc,         0, FALSE},
  { N_("circle"),    obj_set_tool_circle,      0, FALSE},
  { N_("donut"),     obj_set_tool_circle,     10, FALSE},
  { N_("ellipse"),   obj_set_tool_ellipse,     0, FALSE},
  { N_("elarc"),     obj_set_tool_elarc,       0, FALSE},
  { N_("polygon"),   obj_set_tool_polygon,     0, FALSE},
  { N_("rectangle"), obj_set_tool_rectangle,   0, FALSE},
  { N_("point"),     obj_set_tool_point,       0, FALSE},
  { N_("text"),      obj_set_tool_text,        0, FALSE},
  { N_("insert"),    obj_set_tool_insert,      0, FALSE},

  /* Construct */
  { N_("copy"),      tool_set_tool_copy,       0, FALSE},
  { N_("offset"),    tool_set_tool_offset,     0, FALSE},
  { N_("mirror"),    tool_set_tool_mirror,     0, FALSE},
  { N_("array"),     tool_set_tool_array,      0, FALSE},
  { N_("chamfer"),   tool_set_tool_chamfer,    0, FALSE},
  { N_("fillet"),    tool_set_tool_fillet,     0, FALSE},
  { N_("block"),     tool_set_tool_block,     0, FALSE},

  /* Modifiy */
  { N_("move"),      tool_set_tool_move,       0, FALSE},
  { N_("rotate"),    tool_set_tool_rotate,     0, FALSE},
  { N_("stretch"),   tool_set_tool_stretch,    0, FALSE},
  { N_("scale"),     tool_set_tool_scale,      0, FALSE},
  { N_("trim"),      tool_set_tool_trim,       0, FALSE},
  { N_("break"),     tool_set_tool_break,      0, FALSE},
  { N_("erase"),     tool_set_tool_erase,      0, FALSE},
  { N_("etext"),     tool_set_tool_etext,      0, FALSE},  
  { N_("eprop"),     tool_set_tool_eprop,      0, FALSE},

  /* Special commands */
  { N_("list var"),    tool_var_list,       0, FALSE},
  { N_("list block"),  tool_block_list,     0, FALSE}
};

static gint size_parse_list=sizeof(parse_list)/sizeof(PARSE_WORDS);

/* Use a hash table for speed */
static GHashTable *comm_hashtable;

/* For autocompletion of commands */
static GCompletion *comm_completion;


/* --  F U N C T I O N S  -- */
static gchar *completion_cmp(gpointer data)
{
  return (gchar *)data;
}

void parse_init_hashtable()
{
  GList *comm_list=NULL;
  int i;
  
  comm_hashtable=g_hash_table_new(g_str_hash,g_str_equal);

  /* We are comparing strings so no function need as a parameter to g_completion_new */
  comm_completion=g_completion_new(completion_cmp);

  for(i=0;i<size_parse_list;i++)
    {
      comm_list=g_list_append(comm_list,(gpointer) _(parse_list[i].com));

      g_hash_table_insert(comm_hashtable, _(parse_list[i].com),(gpointer)&parse_list[i]);
    }

  /* Make completion */
  g_completion_add_items(comm_completion, comm_list);
}

gint parse_autocompletion(gchar *s, gchar **n_prefix)
{
  GList *item;

  item=g_completion_complete(comm_completion, s, n_prefix);

  if(g_list_length(item)>1)
    {
      print("\n");
      while(item!=NULL)
	{
	  print("%s ", item->data);
	  item=g_list_next(item);
	}

      /* Found many prefix */
      return 0;
    }
 
  /* Found one prefix */
  if(*n_prefix!=NULL)
    return 1;

  /* No prefix found! */
  return -1;
}

gint tool_prompt_idle(DOCUMENT *doc, gchar *buffer)
{
  PARSE_WORDS *t;

  /* Remove leading and trailing spaces */
  g_strstrip(buffer);

  /* First check for the words let or dim */
  if(!strncmp( _("dim"), buffer, 3))
    tool_var_dim(doc, buffer);
  else if(!strncmp( _("let"), buffer, 3))
    tool_var_let(doc, buffer);
  else
    {
      t= (PARSE_WORDS *) g_hash_table_lookup(comm_hashtable, buffer);

      if(t!=NULL)
	{
	  /* This is _very_ ugly solution */
	  command_from_keyboard=TRUE;
	  t->func(NULL, (gpointer)t->data);
	  command_from_keyboard=FALSE;
	  
	  if(t->update_prompt)
	    set_prompt();
	  
	  return TRUE;
	}
      return FALSE;
    }

  set_prompt();
  return TRUE;
}

static void parse_command(DOCUMENT *doc, gchar *string)
{
  /* Is this a normal command */
  if(tool_prompt_idle(doc,string))
    return;

  if(gl_tool==TOOL_IDLE && string[0]=='\0')
    {
      set_prompt();
      return;
    }

  switch(gl_tool)
    {
    case TOOL_LINE:
      obj_prompt_line(doc,string);
      break;
    case TOOL_CIRCLE:
      obj_prompt_circle(doc,string);
      break;
    case TOOL_ARC:
      obj_prompt_arc(doc,string);
      break;
    case TOOL_POLYLINE:
      obj_prompt_polyline(doc,string);
      break;
    case TOOL_RLINE:
      obj_prompt_rline(doc,string);
      break;
    case TOOL_CLINE:
      obj_prompt_cline(doc,string);
      break;
    case TOOL_POINT:
      obj_prompt_point(doc,string);
      break;
    case TOOL_SPLINE:
      obj_prompt_spline(doc,string);
      break;
    case TOOL_ELLIPSE:
      obj_prompt_ellipse(doc,string);
      break;
    case TOOL_ELARC:
      obj_prompt_elarc(doc,string);
      break;
    case TOOL_TEXT:
      obj_prompt_text(doc,string);
      break;
    case TOOL_ATTRIBUTE:
      obj_prompt_attribute(doc,string);
      break;
    case TOOL_RECTANGLE:
      obj_prompt_rectangle(doc,string);
      break;
    case TOOL_POLYGON:
      obj_prompt_polygon(doc,string);
      break;
    case TOOL_INSERT:
      obj_prompt_insert(doc,string);
      break;
    case TOOL_ZOOM:
      tool_prompt_zoom(doc,string); 
      break;
    case TOOL_SELECT:
      spec_prompt_select(doc,string); 
      break;
    case TOOL_COPY:
      tool_prompt_copy(doc,string); 
      break;
    case TOOL_OFFSET:
      tool_prompt_offset(doc,string); 
      break;
    case TOOL_MIRROR:
      tool_prompt_mirror(doc,string); 
      break;
    case TOOL_ARRAY:
      tool_prompt_array(doc,string); 
      break;
    case TOOL_MOVE:
      tool_prompt_move(doc,string); 
      break;
    case TOOL_ROTATE:
      tool_prompt_rotate(doc,string); 
      break;
    case TOOL_TRIM:
      tool_prompt_trim(doc,string); 
      break;
    case TOOL_IDPOINT:
      tool_prompt_idpoint(doc,string); 
      break;
    case TOOL_DIST:
      tool_prompt_dist(doc,string); 
      break;
    case TOOL_FILLET:
      tool_prompt_fillet(doc,string); 
      break;
    case TOOL_BREAK:
      tool_prompt_break(doc,string); 
      break;
    case TOOL_CHAMFER:
      tool_prompt_chamfer(doc,string); 
      break;
    case TOOL_STRETCH:
      tool_prompt_stretch(doc,string); 
      break;
    case TOOL_SCALE:
      tool_prompt_scale(doc,string); 
      break;
    case TOOL_BLOCK:
      tool_prompt_block(doc,string); 
      break;
    case TOOL_HORIZ:
      dim_prompt_horiz(doc,string); 
      break;
    case TOOL_VERT:
      dim_prompt_vert(doc,string); 
      break;
    case TOOL_ALIGNED:
      dim_prompt_aligned(doc,string); 
      break;
    case TOOL_RADIUS:
      dim_prompt_radius(doc,string); 
      break;
    case TOOL_DIAMETER:
      dim_prompt_diameter(doc,string); 
      break;
    case TOOL_CENTER:
      dim_prompt_center(doc,string); 
      break;
    case TOOL_CONTINUE:
      dim_prompt_continue(doc,string); 
      break;
    case TOOL_BASELINE:
      dim_prompt_baseline(doc,string); 
      break;
    case TOOL_LEADER:
      dim_prompt_leader(doc,string); 
      break;
    case TOOL_ANGULAR:
      dim_prompt_angular(doc,string); 
      break;
    case TOOL_ETEXT:
      tool_prompt_etext(doc,string); 
      break;
    default:
      print( _("\n  Syntax Error.\n"));
      prompt();
      break;
    }
}

void parse(DOCUMENT *doc, gchar *buffer)
{
  if(doc==NULL)
    doc=get_document();

  /* No token at all => User pressed enter */
  if(buffer[0]=='\0')
    parse_command(doc,buffer);
  else
    {
      int i;
      gchar **token;

      token=g_strsplit(buffer,"|", 255);      
      for(i=0;token[i]!=NULL;i++)
	{
	  /* Remove all leading and trailing spaces */
	  g_strstrip(token[i]);
	  
	  if(i>0)
	    print(token[i]);
	  
	  parse_command(doc,token[i]);
	}

      if(i>1 && buffer[strlen(buffer)-1]!='|' && gl_tool!=TOOL_IDLE)
	{
	  gchar temp='\0';
	  parse_command(doc,&temp);
	}

      g_strfreev(token);
    }
}

/* -- F U N C T I O N S   T O   G E T   A   C O O R D I N A T E   O R   A   V A L U E -- */
static void print_error(gint err)
{
  switch(err)
    {
    case EXPRPARSER_DIVIDE_BY_ZERO:
      print( _("\n  Divide by zero."));
      break;
    case EXPRPARSER_PARENTHESES_NOT_BALANCED:
      print( _("\n  Parentheses not balanced."));
      break;
    case EXPRPARSER_OPEN_PARENTHESIS_EXPECTED:
      print( _("\n  Open parenthesis expected."));
      break;
    case EXPRPARSER_CLOSE_PARENTHESIS_EXPECTED:
      print( _("\n  Close paraenthesis expected."));
      break;
    case EXPRPARSER_COMMA_EXPECTED:
      print( _("\n  Comma expected."));
      break;
    case EXPRPARSER_UNKNOWN_VARIABLE:
      print( _("\n  Unknown variable."));
      break;
    case EXPRPARSER_BAD_IDENTIFIER:
      print( _("\n  Bad identifier."));
      break;
    case EXPRPARSER_DOMAIN_ERROR:
      print( _("\n  Domain error."));
      break;
    }

}

#define INPUT_ERROR         0
#define INPUT_ABSOLUTE_RECT 1
#define INPUT_ABSOLUTE_POL  2
#define INPUT_RELATIVE_RECT 3
#define INPUT_RELATIVE_POL  4

gboolean parse_single_value(DOCUMENT *doc, gchar *buffer, gdouble *r)
{ 
  gint ret;

  ret=exprparser(doc, buffer, r);

  if(ret!=EXPRPARSER_OK)
    {
      print_error(ret);
      return FALSE;
    }

  return TRUE;
}

gint parse_coordinate(DOCUMENT *doc, gchar *buffer, gdouble *x, gdouble *y)
{
  gchar fbuf[255];
  gint open_parentheses=0;
  gint close_parentheses=0;
  gint i,k;
  gchar sep='\0';
  gboolean rel;

  rel = (buffer[0]=='@');

  /* Skip first character if relative */
  if(rel==TRUE)
    i=1;
  else
    i=0;

  for(k=0;i<strlen(buffer);i++,k++)
    {
      if(buffer[i]=='(')
	open_parentheses++;
      else if(buffer[i]==')')
	close_parentheses++;
      else if( ( buffer[i]==',' || buffer[i]=='<' ) &&  (open_parentheses==close_parentheses))
	{
	  sep=buffer[i];
	  break;
	}
      fbuf[k]=buffer[i];
    }

  fbuf[k]='\0';

  /* Did we found a separator? */
  if(sep=='\0')
    {
      print( _("\n  Cannot find separator in string."));
      return INPUT_ERROR;
    }

  if(!parse_single_value(doc, fbuf, x))
      return INPUT_ERROR;

  if(!parse_single_value(doc, &buffer[i+1], y))
    return INPUT_ERROR;
  
  /* Find type of coordinate */
  if(sep==',' && !rel)
    return INPUT_ABSOLUTE_RECT;
  else  if(sep==',' && rel)
    return INPUT_RELATIVE_RECT;
  else if(sep=='<' && !rel)
    return INPUT_ABSOLUTE_POL;
  else  if(sep=='<' && rel)
    return INPUT_RELATIVE_POL;

  return INPUT_ERROR;
}

gboolean get_coordinate(DOCUMENT *doc, gchar *buffer, gboolean rel, gdouble ox, gdouble oy, gdouble *x, gdouble *y)
{
  gint ret;
  gdouble tx,ty;

  ret=parse_coordinate(doc, buffer, &tx, &ty);

  if(ret==INPUT_ABSOLUTE_RECT)
    {
      *x=tx;
      *y=ty;
      return TRUE;
    }
  else if(ret==INPUT_ABSOLUTE_POL)
    {
      *x=tx*cos(ty*M_PI/180.0);
      *y=-tx*sin(ty*M_PI/180.0);
      return TRUE;
    }
  else if(ret==INPUT_RELATIVE_RECT && rel)
    {
      *x=tx+ox;
      *y=ty+oy;
      return TRUE;
    }
  else if(ret==INPUT_RELATIVE_POL && rel)
    {
      *x=ox+tx*cos(ty*M_PI/180.0);
      *y=oy-tx*sin(ty*M_PI/180.0);
      return TRUE;
    }
  else if((ret==INPUT_RELATIVE_POL || ret==INPUT_RELATIVE_RECT) && !rel)
    {
      print( _("\n  Relative coordinates are not allowed."));
      return FALSE;
    }
  else if(ret==INPUT_ERROR)
    return FALSE;

  print( _("\n  Syntax error."));
  return FALSE;
}

/* -- M A I N   P A R S I N G   R O U T I N E -- */
gint parse_prompt(gint mask, gchar *buffer, gint relative,gchar *menus,
                  gdouble ox, gdouble oy, gdouble *x, gdouble *y, 
                  gdouble *num,gint *mn)
{
  /* Has the user just pressed return? */
  if(buffer[0]=='\0')
    {
      if(mask & PROMPT_RETURN)
	return PROMPT_RETURN;
      else  /* Repeat prompt */
	{
	  set_prompt();
	  return PROMPT_IDLE;
	}
    }

  /* Has the user selected selcen? */
  if(!strcmp(buffer, _("selcen")) && (mask & PROMPT_POINT2) && work_document!=NULL)
    {
      if(misc_find_midpoint_of_selection(work_document,x,y))
	return PROMPT_POINT2;
      else
	{
	  print( _("\n  No object(s) selected."));
	  set_prompt();
	  return PROMPT_IDLE;
	}
    }

  /* Check if user has entered a menucommand */
  if(mask & PROMPT_MENU && menus!=NULL)
    {
      int i;
      gchar **token;

      token=g_strsplit(menus,"/", NO_OF_MENUTOKENS);

      for(i=0;token[i]!=NULL;i++)
	{
	  if(!strcasecmp(token[i],buffer))
	    {
	      *mn=i;
	      return PROMPT_MENU;
	    }
	}

      g_strfreev(token);
    }

  /* Has the user entered a number? */
  if(mask & PROMPT_POINT1)
    {         
      if(parse_single_value(work_document, buffer,num))		
	return PROMPT_POINT1;
      else
	{
	  set_prompt();
	  return PROMPT_ERROR;
	}
    }
  
  /* Has the user entered a coordinate? */
  if(mask & PROMPT_POINT2)
    {  
      if(get_coordinate(work_document, buffer, relative, ox, oy, x, y))      
	return PROMPT_POINT2;
      else
	{
	  set_prompt();
	  return PROMPT_ERROR;
	}
    }

  if(!(mask & PROMPT_ERROR))
    {
      print( _("\n  Syntax error."));
      set_prompt();
    }
  return PROMPT_ERROR;
}













