/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "global.h"
#include "dia_message.h"
#include "color.h"
#include "misc.h"
#include "dia_misc.h"
#include "initmenu.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */

static gint change_colour_predefined(GtkWidget *widget, COLOUR *colour)
{
  if(colour->doc->type_colour==USERDEFINED)
    g_free(colour->doc->current_colour);

  colour->doc->type_colour=PREDEFINED;
  colour->doc->current_colour=colour;
  colour->doc->predefined_colour=colour->predefined_number;
  return TRUE;
}

static gint change_colour_bylayer(GtkWidget *widget, DOCUMENT *doc)
{
  if(doc->type_colour==USERDEFINED)
    g_free(doc->current_colour);

  doc->type_colour=BYLAYER;
  doc->current_colour=doc->current_layer->colour; 
  return TRUE;
}

static gint change_colour_byblock(GtkWidget *widget, DOCUMENT *doc)
{
  if(doc->type_colour==USERDEFINED)
    g_free(doc->current_colour);

  doc->type_colour=BYBLOCK;
  doc->current_colour=doc->current_layer->colour; 
  return TRUE;
}

static gint change_colour_userdefined(GtkWidget *widget, DOCUMENT *doc)
{
  gdouble colour[3];

  if(dia_colour_selection(colour))
    {
      if(doc->type_colour==USERDEFINED)
	g_free(doc->current_colour);
      
      doc->type_colour=USERDEFINED;
 
      doc->current_colour=make_new_userdefined_colour(doc,colour[0], colour[1], colour[2]);

      //      gdk_window_set_background( doc->widgets.toolbar_colouruserdefined->window,&(doc->current_colour->colour));

    }

  return TRUE;
}

void make_colour_popmenu(DOCUMENT *doc)
{
  gint i;
  GtkWidget *menu;

  /* Remove old menu */
  gtk_widget_destroy(doc->widgets.toolbar_colourmenu);

  /* Make new */
  doc->widgets.toolbar_colourmenu=gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(doc->widgets.toolbar_colour),
			   doc->widgets.toolbar_colourmenu);

  /* Menu: By Layer */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_colourmenu),
		  make_menuitem( _("By Layer"), GTK_SIGNAL_FUNC(change_colour_bylayer), (gpointer) doc));

  /* Menu: By block */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_colourmenu),
		  make_menuitem( _("By Block"), GTK_SIGNAL_FUNC(change_colour_byblock), (gpointer) doc));

  /* Make item Separator */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_colourmenu), make_menuitem_hr()); 

  /* Make colour plates */
  for(i=0;i<doc->no_predefined_colours;i++)
    {
      menu=gtk_menu_item_new();

      gtk_signal_connect(GTK_OBJECT(menu),"activate",
			 GTK_SIGNAL_FUNC (change_colour_predefined),
			 (gpointer)&doc->predefined_colours[i]);

      gtk_menu_append(GTK_MENU(doc->widgets.toolbar_colourmenu), menu);
      gtk_widget_show(menu);

      gtk_widget_realize(menu);

      doc->widgets.toolbar_colourbox[i]=gtk_drawing_area_new();
      gtk_drawing_area_size(GTK_DRAWING_AREA(doc->widgets.toolbar_colourbox[i]), 70,14);
      gtk_widget_show(doc->widgets.toolbar_colourbox[i]);

      /* Set colour of drawing box */
      gtk_container_add (GTK_CONTAINER (menu), doc->widgets.toolbar_colourbox[i]);

      gdk_window_set_background(doc->widgets.toolbar_colourbox[i]->window, &doc->predefined_colours[i].colour);
      gdk_window_clear(doc->widgets.toolbar_colourbox[i]->window);

    }

  /* Make item Separator */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_colourmenu), make_menuitem_hr()); 

  /* Menu: Userdefined */
  gtk_menu_append(GTK_MENU(doc->widgets.toolbar_colourmenu),
		  make_menuitem( _("Userdefined"), GTK_SIGNAL_FUNC(change_colour_userdefined), (gpointer) doc));
}



