/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "grid.h"
#include "misc.h"
#include "calc.h"
#include "parse.h"
#include "xor.h"
#include "dia_comm.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_attribute.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_spline.h"
#include "obj_insert.h"
#include "dimension.h"
#include "struct/config.h"

/* --  G L O B A L   V A R I A B L E S  -- */
extern CONFIG configuration;

/* --  L O C A L   V A R I A B L E S  -- */

/* --  F U N C T I O N S  -- */
static void redraw_object_clip(gpointer obj, gpointer doc)
{
  switch(((OBJECT *)obj)->type)
    {
    case TYPE_LINE:
      obj_draw_line( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_SPLINE:
      obj_draw_spline( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_RLINE:
      obj_draw_rline( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_CLINE:
      obj_draw_cline( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_CIRCLE:
      obj_draw_circle( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_ARC:
      obj_draw_arc( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_POLYLINE:
      obj_draw_polyline( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_POINT:
      obj_draw_point( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_TEXT:
      obj_draw_text( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_ATTRIBUTE:
      obj_draw_attribute( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_ELLIPSE:
      obj_draw_ellipse( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_ELARC:
      obj_draw_elarc( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_BLOCK:
      obj_draw_insert( (DOCUMENT *)doc, (OBJECT *)obj);
      break;
    case TYPE_DIMENSION:
      dim_draw_dimension((DOCUMENT *)doc, NULL, (OBJECT *)obj);
      break;
    }
}

void redraw_object(DOCUMENT *doc, OBJECT *obj)
{
  if(doc==NULL || obj==NULL)
    return;

  switch(obj->type)
    {
    case TYPE_LINE:
      obj_draw_line(doc, obj);
      break;
    case TYPE_RLINE:
      obj_draw_rline(doc, obj);
      break;
    case TYPE_CLINE:
      obj_draw_cline(doc, obj);
      break;
    case TYPE_SPLINE:
      obj_draw_spline(doc, obj);
      break;
    case TYPE_CIRCLE:
      obj_draw_circle(doc, obj);
      break;
    case TYPE_ARC:
      obj_draw_arc(doc, obj);
      break;
    case TYPE_POLYLINE:
      obj_draw_polyline(doc, obj);
      break;
    case TYPE_POINT:
      obj_draw_point(doc, obj);
      break;
    case TYPE_TEXT:
      obj_draw_text(doc, obj);
      break;
    case TYPE_ATTRIBUTE:
      obj_draw_attribute(doc, obj);
      break;
    case TYPE_ELLIPSE:
      obj_draw_ellipse(doc, obj);
      break;
    case TYPE_ELARC:
      obj_draw_elarc(doc, obj);
      break;
    case TYPE_BLOCK:
      obj_draw_insert(doc, obj);
      break;
    case TYPE_DIMENSION:
      dim_draw_dimension((DOCUMENT *)doc, NULL, (OBJECT *)obj);
      break;
    }
}

static void redraw_layer(gpointer layer, gpointer doc)
{
  if( ((LAYER *)layer)->visible )
    g_list_foreach( ((LAYER *)layer)->object_list,redraw_object_clip, doc);
}

static GList *get_next_visible_layer(GList *list)
     /* Get the next visible layer in the GList */
{
  while(list!=NULL)
    {
      if(((LAYER *)list->data)->visible)
	return list;
      list=g_list_next(list);
    }
  return NULL;
}

static gint redraw_idle_callback( gpointer data )
     /* Function to redraw objects during the
	idle phase of the program */
{
  DOCUMENT *doc;

  doc=(DOCUMENT *)data;


  if(doc->current_redraw_object==NULL)
    {
      doc->current_redraw_layer=g_list_next(doc->current_redraw_layer);

      doc->current_redraw_layer=get_next_visible_layer(doc->current_redraw_layer);
      if(doc->current_redraw_layer!=NULL)
	{
	  doc->current_redraw_object=((LAYER *)doc->current_redraw_layer->data)->object_list;	  
	  return TRUE;
	}
      else
	{
	  /* Nothing to draw */
	  doc->redraw_tag=-1;
	  return FALSE;
	}
    }

  redraw_object_clip((gpointer)doc->current_redraw_object->data, doc); 
  doc->current_redraw_object=g_list_next(doc->current_redraw_object);

  return TRUE;
}


void redraw_window(DOCUMENT *doc)
     /* Redraws the drawing doc */
{
  /* Draw white background */
  gdk_draw_rectangle(doc->widgets.draw->window, 
		     doc->widgets.window->style->white_gc,
		     TRUE,
		     0,0,
		     doc->widgets.draw->allocation.width,
		     doc->widgets.draw->allocation.height);


  /* Check if to use background drawing */
  if(configuration.use_background)
    {
      /* Check if we already has a redrawibng process,
	 if so, close it */
      if(doc->redraw_tag!=-1)
	gtk_idle_remove(doc->redraw_tag);

      /* Check if we have any layers to draw */
      doc->current_redraw_layer=get_next_visible_layer(doc->layer_list);

      if(doc->current_redraw_layer!=NULL)
	{
	  doc->current_redraw_object=((LAYER *)doc->current_redraw_layer->data)->object_list;

	  /* Start a idle process */
	  doc->redraw_tag=gtk_idle_add(redraw_idle_callback, (gpointer)doc);
	}
    }
  else
    {
      /* Redraw each layer */
      g_list_foreach(doc->layer_list,redraw_layer, (gpointer)doc);   
    }

  /* Draw grid */
  if(doc->gl_grid)
    draw_grid(doc);
}

void redraw(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  remove_old();

  doc=get_document();
  if(doc!=NULL)
    redraw_window(doc);

  if(command_from_keyboard)
    {
      print("\n");
      prompt();
    }

}





