/* -- Float sensitive */
#define GFLOAT_SENSITIVE 0.00001

/* -- Maximum number of predefined colours for popup menu -- */
#define MAX_NO_PREDEFINED_COLOURS 16

/* -- Object transform defines -- */
#define OBJ_NONE 0x00
#define OBJ_TRANSLATE 0x01
#define OBJ_SCALE 0x02
#define OBJ_ROTATE 0x04

/* -- Transform struct -- */
typedef struct s_transform
{
  gint type;                /* Type of transformation */
  gdouble rot_angle;        /* Rotation angle */
  gdouble cen_x,cen_y;      /* Center point of rotation and scale */
  gdouble scale_x,scale_y;  /* Scale point */
  gdouble trans_x,trans_y;  /* Move point */
} TRANSFORM; 

/* -- Defines for ISOMETRIC VIEW -- */
#define ISOMETRIC_LEFT  0
#define ISOMETRIC_TOP   1
#define ISOMETRIC_RIGHT 2

/* -- Defines for tool POINT -- */
#define POINT_DOT    0x01
#define POINT_LINE   0x02
#define POINT_CIRCLE 0x04
#define POINT_BOX    0x08
#define POINT_MARK   0x10
#define POINT_XMARK  0x20

#define POINT_ABSOLUTE 0
#define POINT_RELATIVE 1

/* -- Defines for tool TEXT -- */
#define TEXT_JUST_AL  0
#define TEXT_JUST_FI  1
#define TEXT_JUST_TL  2
#define TEXT_JUST_TC  3
#define TEXT_JUST_TR  4
#define TEXT_JUST_ML  5
#define TEXT_JUST_MC  6
#define TEXT_JUST_MR  7
#define TEXT_JUST_BL  8
#define TEXT_JUST_BC  9
#define TEXT_JUST_BR 10

/* -- Defines for tool POLYLINE -- */
#define POLYLINE_OPEN       0
#define POLYLINE_CLOSE_LINE 1
#define POLYLINE_CLOSE_ARC  2


/* -- Defines for osnap and selection -- */
#define APPLY_CURRENT_LAYER          0
#define APPLY_ALL_LAYERS             1
#define APPLY_ALL_LAYERS_BUT_CURRENT 2

#define OSNAP_ALWAYS 0    /* Runnning object snap mode */
#define OSNAP_ONCE   1    /* Normal object snap mode */

/* -- Defines for alertboxes -- */
#define NO_ICONS 7

typedef struct{
  GdkPixmap *pixmap;
  GdkBitmap *mask;
} ICON_STRUCT;

#define ICON_INFO     0
#define ICON_WARNING  1
#define ICON_QUESTION 2
#define ICON_KILL     3

/* Endangle for a full circle */ 
#define FULL_CIRCLE 360*64

/* No tool selected */
#define TOOL_IDLE       0

/* Inguiry */
#define TOOL_LIST       10
#define TOOL_IDPOINT    11
#define TOOL_DIST       12
#define TOOL_AREA       13

/* Objects, start on 20- */
#define TOOL_LINE       20
#define TOOL_POLYLINE   21
#define TOOL_CLINE      22
#define TOOL_RLINE      23
#define TOOL_SPLINE     24
#define TOOL_ARC        25
#define TOOL_CIRCLE     26
#define TOOL_ELLIPSE    27
#define TOOL_ELARC      100
#define TOOL_POLYGON    28
#define TOOL_RECTANGLE  29
#define TOOL_POINT      30
#define TOOL_TEXT       31
#define TOOL_INSERT     32
#define TOOL_ATTRIBUTE  34
#define TOOL_HATCH      35

/* Dimensioning */
#define TOOL_HORIZ     101
#define TOOL_VERT      102
#define TOOL_CONTINUE  103
#define TOOL_BASELINE  104
#define TOOL_RADIUS    105
#define TOOL_DIAMETER  106
#define TOOL_CENTER    107
#define TOOL_LEADER    108
#define TOOL_ALIGNED   109
#define TOOL_ANGULAR   110

/* Construct */
#define TOOL_COPY       40
#define TOOL_OFFSET     41
#define TOOL_MIRROR     42
#define TOOL_ARRAY      43
#define TOOL_CHAMFER    44
#define TOOL_FILLET     45
#define TOOL_BLOCK      47

/* Modify */
#define TOOL_MOVE       50
#define TOOL_ROTATE     51
#define TOOL_ALIGN      52
#define TOOL_STRETCH    53
#define TOOL_SCALE      54
#define TOOL_LENGTHEN   55
#define TOOL_MPOINT     56
#define TOOL_TRIM       57
#define TOOL_EXTEND     58
#define TOOL_BREAK      59
#define TOOL_EPLINE     60
#define TOOL_ESPLINE    61
#define TOOL_ETEXT      62
#define TOOL_EPROP      63
#define TOOL_EXPLODE    64
#define TOOL_ERASE      65

/* Speciell */
#define TOOL_ZOOM       90
#define TOOL_SELECT     91

/* defines for object snapping */
#define OSNAP_NONE  0x000
#define OSNAP_CEN   0x001
#define OSNAP_END   0x002
#define OSNAP_INS   0x004
#define OSNAP_INT   0x008
#define OSNAP_MID   0x010
#define OSNAP_NEA   0x020
#define OSNAP_NOD   0x040
#define OSNAP_PER   0x080
#define OSNAP_QUA   0x100
#define OSNAP_TAN   0x200

/* -- U N D O  D E F I N E S -- */
/* Defines for undo */
#define UNDO_ADD 0
#define UNDO_MODIFY 1
#define UNDO_ERASE 2

typedef struct s_undo
{
  gint type;
  GList object_list;
} UNDO;

/* --  D E F I N E S  -- */
#define BYLAYER 0        /* According to layer */
#define PREDEFINED 1     /* According to one of predefined values */ 
#define USERDEFINED 2    /* Userdefined colour */
#define BYBLOCK 3        /* According to block if object included in one or else by layer */

#define NODE_LINE 0
#define NODE_ARC 1

/* -- F O N T  S T R U C T S -- */
#define TEXTSTYLE_OCADIS 0   /* The only current support file format is oCADis own */

#define TEXTSTYLE_BACKWARDS 0x01
#define TEXTSTYLE_UPSIDE_DOWN 0x02
#define TEXTSTYLE_VERTICAL 0x04

typedef struct s_textstyle
{
  gint type;                        /* Type of font */
  gchar *name;                      /* Name of font */
  gint flags;                       /* Flags */
  gdouble height, oblique, width;   /* Settings */
  struct s_document *doc;           /* Shortcut */
} TEXTSTYLE;

/* -- C O L O U R  S T R U C T S -- */
typedef struct s_colour
{
  gchar *name;

  gint type;

  gint predefined_number;

  /* Userdefined */
  gdouble c[3];
  GdkColor colour;  
  GdkColor xor_colour;

  /* Makes life easier */
  struct s_document *doc;

} COLOUR;

/* -- L I N E S T Y L E  S T R U C T S -- */
#define LINE_SOLID 0
#define LINE_NOT_SOLID 1

typedef struct s_linestyle
{
  gchar *name;

  gint type;    /* the same has in GdkLineStyle */

  gdouble dash_length;
  gint dash_offset;
  gchar *dash_list;
  gdouble *dash_dlist;
  gint dash_no;

  /* Pixmap of linestyle */
  ICON_STRUCT pic;

  struct s_document *doc;
} LINESTYLE;


/* -- D I M E N S I O N   S T R U C T S -- */
typedef struct s_dimensionstyle
{
  gchar *name;  /* Name of dimension style */

  gdouble dimexe;   /* Distance extension line from dimension line */
  gdouble dimexo;   /* Distance between defpoint and start of line */
  gdouble dimgap;   /* Distance between dimension text and dimension line */
  gdouble dimtxt;   /* Size of dimension text */
  gint dimdec;      /* Number of unit decimal places */
  gdouble dimdli;   /* Distance between 2 dimension lines in baseline */
  gint dimatp;      /* Type of arrow */
  gdouble dimasz;   /* Size of arrow */
  gint dimmtp;      /* Type of centermark */
  gdouble dimmsz;   /* Size of centermark */

  struct s_document *doc;           /* Shortcut */
} DIMENSIONSTYLE;

/* --  D O C U M E N T  A N D  O B J E C T   D E F I N E S  -- */

/* Object selectmode */
enum object_sel {OBJECT_NOT_SEL, OBJECT_SEL, OBJECT_ERASE};

/* Object-identifier */
#define TYPE_LINE      0x0001     /* Normal line */
#define TYPE_CIRCLE    0x0002     /* Circle */
#define TYPE_ARC       0x0004     /* Arc */
#define TYPE_POLYLINE  0x0008     /* Polyline */
#define TYPE_ELLIPSE   0x0010     /* Ellipse */
#define TYPE_TEXT      0x0020     /* Text */
#define TYPE_POINT     0x0040     /* Point */
#define TYPE_CLINE     0x0080     /* Construction-line */
#define TYPE_RLINE     0x0100     /* Ray line */
#define TYPE_SPLINE    0x0200     /* Spline */
#define TYPE_BLOCK     0x0400     /* Block */
#define TYPE_SYMBOL    0x0800     /* Symbol */
#define TYPE_ATTRIBUTE 0x1000     /* Attribute */
#define TYPE_HATCH     0x2000     /* Hatch */
#define TYPE_ELARC     0x4000     /* Ellipcal Arc */
#define TYPE_DIMENSION 0x8000     /* Dimension */

typedef struct s_coordinate{
  gint selected;
  gdouble x;
  gdouble y;
} POINT;

typedef struct s_node{
  gint selected;
  gint type;
  gdouble x,y;
  gdouble sw,ew;
  gdouble weight;
} NODE;

typedef struct s_point{
  POINT centrum;
  gint point_type;
  gfloat point_size;
  gint point_style;
} OBJECT_POINT;

typedef struct s_line{
  POINT p1,p2;
} OBJECT_LINE;

typedef struct s_polyline{
  gint type;  /* Polyline, spline, bezier NURBS etc. */

  gint closed;

  GList *node_list; 

} OBJECT_POLYLINE;

typedef struct s_spline{
  gint closed;

  GList *node_list; 

  /* Cache info */
  gint num;
  POINT *cache_point;
} OBJECT_SPLINE;


typedef struct s_circle{
  POINT centrum;
  gdouble radius;
} OBJECT_CIRCLE;

typedef struct s_arc{
  POINT centrum;
  gdouble radius;
  gdouble start_angle;
  gdouble end_angle;
  gint selected_startpoint;
  gint selected_endpoint;
} OBJECT_ARC;

typedef struct s_ellipse{
  POINT centrum;
  gdouble major;
  gdouble minor;
  gdouble angle;
} OBJECT_ELLIPSE;

typedef struct s_elarc{
  POINT centrum;
  gdouble major;
  gdouble minor;
  gdouble angle;
  gdouble start_angle;
  gdouble end_angle;
  gint selected_startpoint;
  gint selected_endpoint;
} OBJECT_ELARC;

typedef struct s_text{
  POINT insert;
  POINT alignment;  /* For align and fit */

  gdouble angle;
  gdouble oblique;
  gdouble height;
  gdouble width;
  
  gint justification;
  gint flags;

  gchar *text;

  struct s_textstyle *style;
} OBJECT_TEXT;

typedef struct s_attribute{
  OBJECT_TEXT text;

  gchar *prompt;
} OBJECT_ATTRIBUTE;

typedef struct s_object_block{
  POINT insert_point; /* Insert point for block */
  gdouble rotation;   /* Degree of rotation */
  gdouble scale_x;    /* Scale x */
  gdouble scale_y;    /* Scale y */

  GList *attr;        /* List of defined attributes */

  struct s_block *block;  /* Pointer to block */
  gchar *name_of_block;   /* Pointer to name of block */
} OBJECT_BLOCK;

#define DIM_HORIZ    0
#define DIM_VERT     1
#define DIM_RADIUS   2
#define DIM_DIAMETER 3
#define DIM_ANGULAR  4
#define DIM_ALIGNED  5
#define DIM_CENTER   6
#define DIM_LEADER   7

#define DIM_TEXT_CONST 0
#define DIM_TEXT_MOVE 1

#define DIM_DATA_CONTINUE 0
#define DIM_DATA_BASELINE 1
#define DIM_DATA_SINGEL   2

typedef struct s_dimension_point
{
  gint type;
  gint flags;

  POINT insert_point;
  POINT text_point;

  gint type_text;
  gchar *text;

} DIMENSION_POINT;

typedef struct s_dimension_radius
{
  gint type;
  gint flags;

  gdouble radius;
  POINT centrum_point;
  POINT radius_point;
  POINT radius2_point;
  POINT insert_point;

  gint type_text;
  gchar *text;

} DIMENSION_RADIUS;

typedef struct s_dimension_angular
{
  gint type;
  gint flags;

  POINT centrum_point;
  
  gboolean first_extension_visible;
  gdouble first_extension_angle;
  gdouble first_extension_radius;

  gboolean second_extension_visible;
  gdouble second_extension_angle;
  gdouble second_extension_radius;

  gdouble text_radius;

  gboolean text_side;

  gint type_text;
  gchar *text;

} DIMENSION_ANGULAR;

typedef struct s_dimension{
  gint type;

  DIMENSIONSTYLE *style;

  gint type_data;
  GList *data;

} OBJECT_DIMENSION;

typedef union u_object{
  OBJECT_POINT point;
  OBJECT_LINE line;
  OBJECT_CIRCLE circle;
  OBJECT_POLYLINE polyline;
  OBJECT_ARC arc;
  OBJECT_TEXT text;
  OBJECT_ATTRIBUTE attribute;
  OBJECT_ELLIPSE ellipse;
  OBJECT_SPLINE spline;
  OBJECT_BLOCK block;
  OBJECT_ELARC elarc;
  OBJECT_DIMENSION dimension;
} OBJECT_UNION;

typedef struct s_object{
  /* -- Type of object -- */
  gint type;

  /* -- Selection status of object -- */
  enum object_sel selected;

  /* -- Name of object -- */
  gchar *name;

  /* -- The type and colour of the object -- */
  gint type_colour;
  COLOUR *colour;

  /* -- The type and linewidth of the object -- */
  gint type_linewidth; 
  gdouble linewidth;

  /* -- The type and linestyle of the object -- */
  gint type_linestyle; 
  LINESTYLE *linestyle;

  /* -- Bounding box of the object -- */
  gdouble bounding_box[4];

  /* -- Object specific data -- */
  OBJECT_UNION data;
 
  /* -- Pointer to the layer object is placed in -- */
  struct s_layer *layer;

} OBJECT;

typedef struct s_layer{
  /* -- Name of layer -- */
  gchar *name;

  /* -- Default colour in this layer -- */
  COLOUR *colour;

  /* -- Default linewidth in this layer -- */
  gdouble linewidth;

  /* -- Default linestylr in this layer -- */
  LINESTYLE *linestyle;

  /* -- Is the layer visible? -- */
  gboolean visible;

  /* -- Is the layer locked? -- */
  gboolean editable;

  /* -- List of all objects in this layer -- */
  GList *object_list;

  /* -- Shortcut to document -- */ 
  struct s_document *doc;

} LAYER;

typedef struct s_window{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *table;
  GtkWidget *draw;
  GtkWidget *origin;
  GtkWidget *hsb;
  GtkAdjustment *hsbdata;
  GtkWidget *vsb;
  GtkAdjustment *vsbdata;
  GtkWidget *horiz_ruler;
  GtkWidget *vert_ruler;
  GtkWidget *toolbar;
  GtkWidget *toolbar_saveicon;
  GtkWidget *toolbar_save;
  GtkWidget *toolbar_snapicon;
  GtkWidget *toolbar_snap;
  GtkWidget *toolbar_gridicon;
  GtkWidget *toolbar_grid;
  GtkWidget *toolbar_orthoicon;
  GtkWidget *toolbar_ortho;
  GtkWidget *toolbar_digiticon;
  GtkWidget *toolbar_digit;
  GtkWidget *toolbar_layermenu;
  GtkWidget *toolbar_option;
  GtkWidget *toolbar_colourmenu;
  GtkWidget *toolbar_colour;
  GtkWidget *toolbar_linewidthmenu;
  GtkWidget *toolbar_linewidth;
  GtkWidget *toolbar_linestylemenu;
  GtkWidget *toolbar_linestyle;

  GtkWidget *toolbar_colourbox[MAX_NO_PREDEFINED_COLOURS];
  GtkWidget *toolbar_colourbylayer;
  GtkWidget *toolbar_colouruserdefined;

  GtkWidget *toolbar_linewidthbox[8];
  GtkWidget *toolbar_linewidthbylayer;
  GtkWidget *toolbar_linewidthuserdefined;

  GtkWidget *toolbar_linestylebox[8];
  GtkWidget *toolbar_linestylebylayer;
  GtkWidget *toolbar_linestyleuserdefined;
} WIDGETS;

typedef struct s_document{
  gchar *title;    /* Title of window */
  gchar *filename; /* Name of file and its path */

  gboolean saved;

  WIDGETS widgets;

  /* -- Height and Width of drawing -- */
  gdouble width;
  gdouble height;

  /* -- Viewport settings */
  gdouble zoom;
  gdouble xpan, ypan;

  gint gl_ortho;
  gint gl_snap;
  gint gl_grid;

  /* -- Snap settings -- */
  gdouble gl_snap_basex, gl_snap_basey;
  gdouble gl_snap_width, gl_snap_height;
  gdouble gl_snap_angle;
  gint gl_grid_x, gl_grid_y;
  gint gl_iso_view;
  gint gl_iso_type;

  /* -- Digitizer -- */
  gboolean use_digitizer;       /* Use digitizer instead of mouse */

  /* -- Redraw data -- */
  gint redraw_tag;              /* Tag of the redrawing process */
  GList *current_redraw_layer;  /* The currently redrawing layer */
  GList *current_redraw_object; /* The currently redrawing object */

  /* -- For continue arcs -- */
  gint cont_exist;
  gdouble cont_x,cont_y,cont_tan;
  
  /* -- Current Point settings -- */
  gint point_type;
  gfloat point_size;
  gint point_style;

  /* -- Working polyline -- */
  OBJECT *working_polyline;

  /* -- Pointer to current layer -- */
  LAYER *current_layer;

  /* -- Current Colour settings -- */
  gint type_colour; 
  gint predefined_colour;
  COLOUR *current_colour;

  /* -- Current Linewidth settings -- */
  gint type_linewidth; 
  gdouble current_linewidth;

  /* -- Current Linestyle settings -- */
  gint type_linestyle; 
  LINESTYLE *current_linestyle;
  GList *linestyle_list;

  /* -- Current fontstyle settings -- */
  GList *textstyle_list;

  /* -- List of layers in document -- */
  GList *layer_list;

  /* -- The block list */
  GList *block_list;

  /* -- The dimension style list -- */
  GList *dimensionstyle_list;
  DIMENSIONSTYLE *current_dimensionstyle;

  /* -- Undo list --*/
  GList *undo_list;

  /* -- Redo list --*/
  GList *redo_list;

  /* -- List of previous zooms -- */
  GList *zoom_previous;

  /* -- List of selected objects -- */
  GList *selection_list;

  /* -- List of defiend variables -- */
  GHashTable *variable_list;

  /* -- Predefined colourlist -- */
  gint no_predefined_colours;
  COLOUR predefined_colours[MAX_NO_PREDEFINED_COLOURS];

  /* -- Graphics stuff -- */
  GdkGC *xor_gc;
  GdkGC *replace_gc;
  GdkGC *select_gc;
  GdkGC *erase_gc;
  GdkColor black;
  GdkColor green;
  GdkColor white;
} DOCUMENT;


/* -- B L O C K   S T R U C T S -- */

typedef struct s_block
{
  gchar *name;           /* Name of block */

  /* -- This is true if block contains attributes -- */
  gboolean attribute;

  GList *object_list;    /* List of objects */
} BLOCK;

typedef struct s_block_object{
  /* -- Type of object -- */
  gint type;

  /* -- The type and colour of the object -- */
  gint type_colour;
  COLOUR *colour;

  /* -- The type and linewidth of the object -- */
  gint type_linewidth; 
  gdouble linewidth;

  /* -- The type and linestyle of the object -- */
  gint type_linestyle; 
  gchar *name_of_linestyle;

  /* -- Object specific data -- */
  union u_object data;
 
  /* -- Layer name -- */
  gchar *name_of_layer;
} BLOCK_OBJECT;







