/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "xor.h"
#include "select.h"
#include "redraw.h"
#include "parse.h"
#include "drag.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gint num_rows, num_cols;
static gdouble row_dist, col_dist;

/* --  F U N C T I O N S  -- */
static void rectangular_array(DOCUMENT *doc)
{
  TRANSFORM transform;
  gint tx,ty;

  remove_old();

  transform.type=OBJ_TRANSLATE;
  transform.trans_y=0;

  for(ty=0;ty<num_rows;ty++)
    {
      transform.trans_x=0;

      for(tx=0;tx<num_cols;tx++)
	{
	  if((tx!=0) || (ty!=0))
	    copy_selected_object(doc, doc, &transform);
	   transform.trans_x+=col_dist;
	}

       transform.trans_y+=row_dist;
    }
}

static void polar_array(DOCUMENT *doc, gint num_items, gdouble angle_to_fill, gboolean rotate_objects)
{
  TRANSFORM transform;
  gint count;
  gdouble selxcen, selycen, sangle,radius;
  gdouble da;

  misc_find_midpoint_of_selection(doc, &selxcen, &selycen);

  sangle=get_angle(gl_oldmx, gl_oldmy,selxcen,selycen);
  radius=dist(selxcen,selycen, gl_oldmx,gl_oldmy);

  if( ABS(ABS(angle_to_fill)-360)<GFLOAT_SENSITIVE)
    da=angle_to_fill/num_items;
  else
    da=angle_to_fill/(num_items-1);

  if(rotate_objects)  /* Rotate objects around the center point */
    {
      /* Make tranform struct */
      transform.type=OBJ_ROTATE;
      transform.cen_x=gl_oldmx;
      transform.cen_y=gl_oldmy;

      for(count=1;count<num_items;count++)
	{
	  transform.rot_angle=-(da*count)*M_PI/180.0;
	  copy_selected_object(doc, doc, &transform);
	}
    }
  else
    {
      transform.type=OBJ_TRANSLATE;

      for(count=1;count<num_items;count++)
	{
	  transform.trans_x=(gl_oldmx-selxcen)+radius*cos((sangle+da*count)*M_PI/180.0);
	  transform.trans_y=(gl_oldmy-selycen)-radius*sin((sangle+da*count)*M_PI/180.0);
	  copy_selected_object(doc, doc, &transform);
	}

    }
}

void tool_set_prompt_array(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Number of rows (---): "));
      break;
    case 1:
      set_prompt_text( _("Number of columns (|||): "));
      break;
    case 2: 
      set_prompt_text( _("Unit cell or distance between rows: "));
      break;
    case 3:
      set_prompt_text( _("Distance between columns: "));
      break;
    case 4:
      set_prompt_text( _("Other corner: "));
      break;
    case 5:
      set_prompt_text( _("Specify centerpoint of array: "));
      break;
    case 10:
    case 6:
      set_prompt_text( _("Number of items: "));
      break;
    case 7:
      set_prompt_text( _("Angle to fill (+=ccw, -=cw): "));
      break;
    }
}

void tool_button_array(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 2:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=4;
      set_prompt();
      break;
    case 4:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      col_dist=x-gl_oldmx;
      row_dist=y-gl_oldmy;
      rectangular_array(doc);
      remove_old();
      deselect_all_objects(doc);
      tool_idle(doc);
      break;
    case 5:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=6;
      set_prompt();
      break;
    }
}

void tool_prompt_array(DOCUMENT *doc, gchar *buffer)
{
  gint rotate;
  gdouble value, tx, ty;
  
  if(gl_stage<4)
    {
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT1),
			  buffer,FALSE,NULL,0,0,NULL,NULL,&value,NULL))
	{
	case PROMPT_POINT1:
	  
	  switch(gl_stage)
	    {
	    case 0:
	      num_rows=value;
	      break;
	    case 1:
	      num_cols=value;
	      break;
	    case 2:
	      row_dist=value;	  
	      break;
	    case 3:
	      work_document=doc;
	      col_dist=value;
	      rectangular_array(doc);
	      remove_old();
	      deselect_all_objects(doc);
	      tool_idle(doc);
	      
	      return;
	      break;
	    }
	  gl_stage++;
	  set_prompt();
	  break;
	}
    }
  else if(gl_stage==4)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_array(doc,1,tx,ty);
	break;
      }
    }
  else if(gl_stage==5 || gl_stage==8 || gl_stage==9)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_array(doc,1,tx,ty);
	break;
      }     
    }
  else if(gl_stage==6 || gl_stage==7 || gl_stage==10)
    {
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT1),
			  buffer,FALSE,NULL,0,0,NULL,NULL,&value,NULL))
	{
	case PROMPT_POINT1:	  
	  switch(gl_stage)
	    {
	    case 6:
	      num_rows=value;
	      gl_stage++;
	      set_prompt();
	      break;
	    case 7:
	      rotate=alert(ICON_QUESTION, _("Array"), _("Yes"), _("No"),NULL, _("Rotate objects as they are copied?"));
	      if(rotate>0)
		 polar_array(doc, num_rows, value, rotate==1);
	      deselect_all_objects(doc);
	      tool_idle(doc);
	      break;
	    }
	  break;
	}
    }
}

void tool_set_tool_array(GtkWidget *widget, gpointer data)
{
  remove_old();
  
  if(!command_from_keyboard)
    print( _("array"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_ARRAY;
  if(GPOINTER_TO_INT(data)==0)
    gl_stage=0;    /* Rectangular array */
  else if (GPOINTER_TO_INT(data)==1)
    gl_stage=5;    /* Polar array */

  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to array: "), _("Selection - Array"));
  set_prompt();

  work_document=NULL;
}












