/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "xor.h"
#include "select.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "redraw.h"
#include "parse.h"
#include "color.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gchar *name_of_block;

/* --  F U N C T I O N S  -- */
void tool_block_list(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;
  
  doc=get_document();
  
  if(doc==NULL)
    {
      print( _("\n  No document found"));
      set_prompt();
      return;
    }
  
  if(doc->block_list==NULL)
    print( _("\n  No blocks defined."));
  else
    {
      GList *list;

      print("\n");

      list=g_list_first(doc->block_list);
      while(list!=NULL)
	{
	  print( _("\n  Block name: %s"), ((BLOCK *)list->data)->name);  
	  list=g_list_next(list);
	}
      print( _("\n\n  Number of defined block: %d\n"),  
	    g_list_length(doc->block_list) );
    }

  set_prompt();
}

void tool_set_prompt_block(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Block name (or ?): "));
      break;
    case 1: 
      set_prompt_text( _("Insertion base point: "));
      break;
    }
}

static BLOCK_OBJECT *duplicate_object_for_block(DOCUMENT *doc, OBJECT *object, gdouble x, gdouble y)
{
  BLOCK_OBJECT *temp;

  temp=g_new(BLOCK_OBJECT, 1);

  temp->type=object->type;

  /* Duplicate colour */
  temp->type_colour=object->type_colour;
  if(temp->type_colour!=USERDEFINED && temp->type_colour!=PREDEFINED)
    temp->colour=NULL;
  else
    temp->colour=make_new_userdefined_colour(doc, object->colour->c[0],object->colour->c[1], object->colour->c[2]);

  /* Duplicate linewidth */
  temp->type_linewidth=object->type_linewidth;
  if(temp->type_linewidth==USERDEFINED)
    temp->linewidth=object->linewidth;

  /* Duplicate linestyle */
  temp->type_linestyle=object->type_linestyle;
  if(temp->type_linestyle!=USERDEFINED)
    temp->name_of_linestyle=NULL;
  else
    temp->name_of_linestyle=g_strdup(object->linestyle->name);

  /* Get name of layer */
  temp->name_of_layer=g_strdup(object->layer->name);

  /* Copy object data */
  memcpy(&temp->data, &object->data, sizeof(OBJECT_UNION));

  /* Change coordinates for data */
  switch(temp->type)
    {
    case TYPE_RLINE:
    case TYPE_CLINE:
    case TYPE_LINE:
      temp->data.line.p1.x=object->data.line.p1.x-x;
      temp->data.line.p1.y=object->data.line.p1.y-y;
      temp->data.line.p2.x=object->data.line.p2.x-x;
      temp->data.line.p2.y=object->data.line.p2.y-y;
      break;
    case TYPE_CIRCLE:
      temp->data.circle.centrum.x=object->data.circle.centrum.x-x;
      temp->data.circle.centrum.y=object->data.circle.centrum.y-y;
      temp->data.circle.radius=object->data.circle.radius;
      break;
    case TYPE_ARC:
      temp->data.arc.centrum.x=object->data.arc.centrum.x-x;
      temp->data.arc.centrum.y=object->data.arc.centrum.y-y;
      temp->data.arc.radius=object->data.arc.radius;
      temp->data.arc.start_angle=object->data.arc.start_angle;
      temp->data.arc.end_angle=object->data.arc.end_angle;
      break;
    case TYPE_ATTRIBUTE:
      temp->data.attribute.text.insert.x=object->data.attribute.text.insert.x-x;
      temp->data.attribute.text.insert.y=object->data.attribute.text.insert.y-y;
      temp->data.attribute.text.alignment.x=object->data.attribute.text.alignment.x-x;
      temp->data.attribute.text.alignment.y=object->data.attribute.text.alignment.y-y;
      temp->data.attribute.text.text=g_strdup(object->data.attribute.text.text);
      temp->data.attribute.prompt=g_strdup(object->data.attribute.prompt);
      break;
    case TYPE_TEXT:
      temp->data.text.insert.x=object->data.text.insert.x-x;
      temp->data.text.insert.y=object->data.text.insert.y-y;
      temp->data.text.alignment.x=object->data.text.alignment.x-x;
      temp->data.text.alignment.y=object->data.text.alignment.y-y;
      temp->data.text.text=g_strdup(object->data.text.text);
      break;
    }
  
  return temp;
}

static void make_block(DOCUMENT *doc, gdouble x, gdouble y)
{
  OBJECT *obj;
  BLOCK_OBJECT *new_obj;
  BLOCK *temp;
  GList *list;

  temp=g_new(BLOCK,1);

  temp->name=name_of_block;
  temp->object_list=NULL;
  temp->attribute=FALSE;

  list=g_list_first(doc->selection_list);

  while(list!=NULL)
    {
      obj=(OBJECT *)list->data;

      new_obj=duplicate_object_for_block(doc, obj, x, y);

      if(new_obj->type==TYPE_ATTRIBUTE)
	temp->attribute=TRUE;

      temp->object_list=g_list_append(temp->object_list, new_obj);

      list=g_list_next(list);
    }
  
  doc->block_list=g_list_append(doc->block_list,temp);
}

void tool_button_block(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 1:
      make_block(doc,x,y);
      deselect_all_objects(doc);
      tool_idle(doc);
      break;
    }
}

BLOCK *find_block_name(DOCUMENT *doc, gchar *name)
{
  GList *list;

  list=doc->block_list;

  while(list!=NULL)
    {
      if(!strcmp( ((BLOCK *)list->data)->name, name))
	 return list->data;
      list=g_list_next(list);
    }
  return NULL;
}


void tool_prompt_block(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx,ty;

  switch(gl_stage)
    {
    case 0:  /* Enter name */
      if(!strcmp(buffer,"?"))
	{
	  tool_block_list(NULL, 0);
	  return;
	}
      else if(find_block_name(doc,buffer)==NULL)
	{
	  name_of_block=g_strdup(buffer);
	  gl_stage++;
	}
      else
	print( _("\n  This name is occupied."));	 

      set_prompt();
      break;
    case 1: /* Insertion base point */
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_block(doc,1,tx,ty);
	break;
      }
      break;
    }
}

void tool_set_tool_block(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("block"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_BLOCK;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to block: "), _("Selection - Block"));
  set_prompt();

  work_document=NULL;
}















