/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "xor.h"
#include "select.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "redraw.h"
#include "parse.h"
#include "calc.h"
#include "obj_elarc.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static OBJECT *selected_object;

/* --  F U N C T I O N S  -- */
void calculate_break(DOCUMENT *doc, OBJECT *object, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  gdouble prj_x1, prj_x2, prj_y1, prj_y2;
  gdouble Ax, Ay, Bx, By;
  gdouble L,r,s, dist_x1y1, dist_x2y2;
  gint found_p1, found_p2;

  switch(object->type)
    {
    case TYPE_LINE:
      found_p1=FALSE;
      found_p2=FALSE;

      Ax=object->data.line.p1.x;
      Ay=object->data.line.p1.y;
      Bx=object->data.line.p2.x;
      By=object->data.line.p2.y;

      L=sqrt( (Bx-Ax)*(Bx-Ax)+(By-Ay)*(By-Ay));

      /* Project both points to the line */
      r=((Ay-y1)*(Ay-By)-(Ax-x1)*(Bx-Ax))/(L*L);
      prj_x1=Ax+r*(Bx-Ax);
      prj_y1=Ay+r*(By-Ay);

      r=((Ay-y2)*(Ay-By)-(Ax-x2)*(Bx-Ax))/(L*L);
      prj_x2=Ax+r*(Bx-Ax);
      prj_y2=Ay+r*(By-Ay);

      r=((Ay-prj_y1)*(Ay-By)-(Ax-prj_x1)*(Bx-Ax))/(L*L);
      s=((Ay-y1)*(Bx-Ax)-(Ax-x1)*(By-Ay))/(L);
      found_p1=(r>=0 && r<=1) && s<4;

      r=((Ay-prj_y2)*(Ay-By)-(Ax-prj_x2)*(Bx-Ax))/(L*L);
      s=((Ay-y2)*(Bx-Ax)-(Ax-x2)*(By-Ay))/(L);
      found_p2=(r>=0 && r<=1) && s<4;

      if(found_p1 && found_p2)
	{
	  dist_x1y1=dist(Ax,Ay, prj_x1, prj_y1);
	  dist_x2y2=dist(Ax,Ay, prj_x2, prj_y2);

	  if(dist_x1y1<dist_x2y2)
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.line.p2.x=prj_x1;
	      object->data.line.p2.y=prj_y1;
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	      redraw_object(doc, obj_add_line(doc,object->layer, object->name, object->type_colour, object->colour, 
					      object->type_linestyle, object->linestyle,
					      object->type_linewidth, object->linewidth,
					      Bx, By , prj_x2, prj_y2));
	    }
	  else
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.line.p1.x=prj_x1;
	      object->data.line.p1.y=prj_y1;
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	      redraw_object(doc, obj_add_line(doc,object->layer, object->name, object->type_colour, object->colour, 
					      object->type_linestyle, object->linestyle,
					      object->type_linewidth, object->linewidth,
					      Ax, Ay , prj_x2, prj_y2));
	    }
	}
      else if(found_p1 && !found_p2)
	{
	  dist_x1y1=dist(prj_x2,prj_y2,Ax,Ay);
	  dist_x2y2=dist(prj_x2,prj_y2,Bx,By);

	  if(dist_x1y1<dist_x2y2)
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.line.p1.x=prj_x1;
	      object->data.line.p1.y=prj_y1;
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	  else
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.line.p2.x=prj_x1;
	      object->data.line.p2.y=prj_y1;
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	}
      else if(!found_p1 && found_p2)
	{
	  dist_x1y1=dist(prj_x1,prj_y1,Ax,Ay);
	  dist_x2y2=dist(prj_x1,prj_y1,Bx,By);

	  if(dist_x1y1<dist_x2y2)
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.line.p1.x=prj_x2;
	      object->data.line.p1.y=prj_y2;
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	  else
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.line.p2.x=prj_x2;
	      object->data.line.p2.y=prj_y2;
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	}
      else
	print( _("\n  Cannot break."));
      break;
    case TYPE_CIRCLE:
      found_p1=FALSE;
      found_p2=FALSE;

      Ax=object->data.circle.centrum.x;
      Ay=object->data.circle.centrum.y;
      L=object->data.circle.radius;      

      found_p1=ABS(dist(Ax,Ay,x1,y1)-L)<4;
      found_p2=ABS(dist(Ax,Ay,x2,y2)-L)<4;

      if(found_p1 && found_p2)
	{
	  object->selected=OBJECT_ERASE;
	  redraw_object(doc, object);
	  
	  object->selected=OBJECT_SEL;
	  object->type=TYPE_ARC;
	  object->data.arc.centrum.x=Ax;
	  object->data.arc.centrum.y=Ay;
	  object->data.arc.radius=L;
	  object->data.arc.start_angle=get_angle(Ax,Ay,x2,y2);
	  object->data.arc.end_angle=get_angle(Ax,Ay,x1,y1);
	  redraw_object(doc,object);
	}
      else
	print( _("\n  Cannot break."));
      break;
    case TYPE_ARC:
      found_p1=FALSE;
      found_p2=FALSE;

      Ax=object->data.arc.centrum.x;
      Ay=object->data.arc.centrum.y;
      Bx=object->data.arc.start_angle;
      By=object->data.arc.end_angle;

      L=object->data.arc.radius;      

      s=180-get_angle(x1,y1,Ax,Ay);
      prj_x1=L*cos(s*3.14159/180)+Ax;
      prj_y1=L*sin(s*3.14159/180)+Ay;

      r=180-get_angle(x2,y2,Ax,Ay);
      prj_x2=L*cos(r*3.14159/180)+Ax;
      prj_y2=L*sin(r*3.14159/180)+Ay;

      found_p1=ABS(dist(Ax,Ay,x1,y1)-L)<4 && misc_within_angle(Bx,By,get_angle(Ax,Ay,x1,y1));
      found_p2=ABS(dist(Ax,Ay,x2,y2)-L)<4 && misc_within_angle(Bx,By,get_angle(Ax,Ay,x2,y2));

      if(found_p1 && found_p2)
	{
	  dist_x1y1=dist_arc(Bx, get_angle(Ax,Ay,x1,y1), L);
	  dist_x2y2=dist_arc(Bx, get_angle(Ax,Ay,x2,y2), L);

	  if(dist_x1y1<dist_x2y2)
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      s=object->data.arc.end_angle;
	      object->data.arc.end_angle=get_angle(Ax,Ay,x1,y1);
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	      redraw_object(doc, obj_add_arc(doc,object->layer, object->name, object->type_colour, object->colour, 
					     object->type_linestyle, object->linestyle,
					     object->type_linewidth, object->linewidth,
					     Ax, Ay ,L, get_angle(Ax,Ay,x2,y2), s));
	    }
	  else
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      s=object->data.arc.end_angle;
	      object->data.arc.end_angle=get_angle(Ax,Ay,x2,y2);
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	      redraw_object(doc, obj_add_arc(doc,object->layer, object->name, object->type_colour, object->colour, 
					     object->type_linestyle, object->linestyle,
					     object->type_linewidth, object->linewidth,
					     Ax, Ay ,L, get_angle(Ax,Ay,x1,y1),s));
	    }
	}
      else if(found_p1 && !found_p2)
	{
	  dist_x1y1=dist_arc2(Bx, get_angle(Ax,Ay, x1,y1), L);
	  dist_x2y2=dist_arc2(By, get_angle(Ax,Ay,x1,y1), L);

	  if(dist_x1y1<dist_x2y2)
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.arc.start_angle=get_angle(Ax,Ay,x1,y1);
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	  else
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.arc.end_angle=get_angle(Ax,Ay,x1,y1);
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	}
      else if(!found_p1 && found_p2)      
	{
	  dist_x1y1=dist_arc2(Bx, get_angle(Ax,Ay, x2,y2), L);
	  dist_x2y2=dist_arc2(By, get_angle(Ax,Ay,x2,y2), L);

	  if(dist_x1y1<dist_x2y2)
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.arc.start_angle=get_angle(Ax,Ay,x2,y2);
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	  else
	    {
	      object->selected=OBJECT_ERASE;
	      redraw_object(doc,object);
	      object->data.arc.end_angle=get_angle(Ax,Ay,x2,y2);
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
	}
      else
	print( _("\n  Cannot break."));
      break;
    case TYPE_ELLIPSE:
      found_p1=FALSE;
      found_p2=FALSE;


      Ax=object->data.ellipse.centrum.x;
      Ay=object->data.ellipse.centrum.y;

      Bx=object->data.ellipse.major;
      By=object->data.ellipse.minor;


      found_p1=ABS(dist(Ax,Ay,x1,y1)-L)<4;
      found_p2=ABS(dist(Ax,Ay,x2,y2)-L)<4;

      //      if(found_p1 && found_p2)
	{
	  object->selected=OBJECT_ERASE;
	  redraw_object(doc, object);
	  
	  object->selected=OBJECT_SEL;
	  object->type=TYPE_ELARC;
	  object->data.elarc.centrum.x=Ax;
	  object->data.elarc.centrum.y=Ay;
	  object->data.elarc.major=Bx;
	  object->data.elarc.minor=By;
	  object->data.elarc.start_angle=ellipse_alfa2t(get_angle(x2,y2,Ax,Ay), Bx, By);
	  object->data.elarc.end_angle=ellipse_alfa2t(get_angle(x1,y1,Ax,Ay), Bx, By);
	  redraw_object(doc,object);
	}
	/*      else
		print("\n  Cannot break."); */
      break;
    }
}

void tool_set_prompt_break(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Select object: "));
      break;
    case 1: 
      set_prompt_text( _("First point/<Enter second point>: "));
      break;
    case 2: 
      set_prompt_text( _("Enter first point: "));
      break;
    case 3: 
      set_prompt_text( _("Enter second point: "));
      break;
    }
}

void tool_set_tool_break(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("break"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_BREAK;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  set_prompt();

  work_document=NULL;
  selected_object=NULL;
}

void tool_button_break(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }
  
  switch(gl_stage)
    {
    case 0:
      selected_object=select_singel_object(doc,x,y);
      if(selected_object!=NULL)
	{
	  selected_object->selected=OBJECT_SEL;
	  redraw_object(doc, selected_object);
	  gl_stage++;
	  gl_oldmx=x;
	  gl_oldmy=y;
	  set_cursor(doc);
	}
      else
	print( _("\n  No object found!"));
      set_prompt();
      break;
    case 3:
    case 1:
      remove_old();
      calculate_break(doc, selected_object, gl_oldmx, gl_oldmy, x, y);
      deselect_all_objects(doc);
      doc->saved=FALSE;
      tool_idle(doc);
      make_window_title(doc);
      break;
    case 2:
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=3;
      set_prompt();
      break;
    }
}

void tool_prompt_break(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx,ty;
  gint mn;

  switch(gl_stage)
    {
    case 2:
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_break(doc,1,tx,ty);
	break;
      }
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2|PROMPT_MENU,buffer,
			  TRUE, _("F/"),gl_oldmx,gl_oldmy,&tx,&ty,NULL,&mn)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_break(doc,1,tx,ty);
	break;
      case PROMPT_MENU:
	gl_stage=2;
	set_prompt();
	break;
      }
      break;
    case 3:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_break(doc,1,tx,ty);
	break;
      }
      break;
    }
}





