/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "parse.h"
#include "layer.h"
#include "linestyle.h"
#include "select.h"
#include "drag.h"
#include "xor.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  F U N C T I O N S  -- */
void tool_set_prompt_copy(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Multiple/<Base Point>: "));
      break;
    case 3: 
    case 1: 
      set_prompt_text( _("Second point of displacement: "));
      break;
    case 2: 
      set_prompt_text( _("Base point: "));
      break;
    }

}

void tool_button_copy(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  TRANSFORM transform;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {

    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
      /* Make tranform matrix */
      transform.type=OBJ_TRANSLATE;
      transform.trans_x=x-gl_oldmx;
      transform.trans_y=y-gl_oldmy;
      copy_selected_object(doc, doc, &transform);
      deselect_all_objects(doc);
      tool_idle(doc);
      break;
    case 2:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=3;
      set_prompt();
      break;
    case 3:
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
      /* Make tranform matrix */
      transform.type=OBJ_TRANSLATE;
      transform.trans_x=x-gl_oldmx;
      transform.trans_y=y-gl_oldmy;
      copy_selected_object(doc, doc, &transform);
      set_prompt();
      break;
    }
}

void tool_button_copy_drag(DOCUMENT *src_doc, DOCUMENT *dest_doc, gint button, gdouble x, gdouble y)
{
  TRANSFORM transform;

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(src_doc);
      tool_idle(src_doc);
      return;
    }

  remove_old();
  make_ortho_snap(dest_doc,gl_oldmx, gl_oldmy, &x, &y);

  /* Make tranform matrix */
  transform.type=OBJ_TRANSLATE;
  transform.trans_x=x-gl_oldmx;
  transform.trans_y=y-gl_oldmy;

  /* Copy selected objects from src to dest and deselect them in dest_doc */
  copy_selected_object(src_doc, dest_doc, &transform);
 
  dest_doc->saved=FALSE;
  make_window_title(dest_doc);

  /* Fix layer and linestyle optionmenus */
  make_linestyle_popmenu(dest_doc);
  make_layer_popmenu(dest_doc);

  if(gl_stage==1)
    {
      deselect_all_objects(src_doc);
      tool_idle(src_doc);
    }
  else
    set_prompt();
}

void tool_menu_copy(gint menusel)
{
  switch(menusel)
    {
    case 0: /* Multipe */
      gl_stage=2;
      set_prompt();
      break;
    case 1: /* Cancel */
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(work_document);
      tool_idle(work_document);
      return;
      break;
    }
}

void tool_prompt_copy(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gint mn;

  switch(gl_stage)
    {
    case 0:
    case 2:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_MENU|PROMPT_RETURN|PROMPT_POINT2,buffer,
                          FALSE, _("M/"),0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_RETURN:
        remove_old();
        tool_idle(doc);
        break;
      case PROMPT_POINT2:
        work_document=doc;
	remove_old();
        tool_button_copy(doc,1,tx,ty);
        break;
      case PROMPT_MENU:
	tool_menu_copy(0);
	break;
      case PROMPT_ERROR:
      default:
        return;
      }

      break;
    case 1:
    case 3:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
                          TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,&mn)){
      case PROMPT_RETURN:
        remove_old();
        tool_idle(doc);
        break;
      case PROMPT_POINT2:
	work_document=doc;
        remove_old();
        tool_button_copy(doc,1,tx,ty);
        break;
      case PROMPT_ERROR:
      default:
        return;
      }

      break;
    } 
}

void tool_set_tool_copy(GtkWidget *widget, gpointer data)
{
  remove_old();

  if(!command_from_keyboard)
    print( _("copy"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);

  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_COPY;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to copy: "), _("Selection - Copy"));
  set_prompt();

  work_document=NULL;
}












