/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "xor.h"
#include "redraw.h"
#include "select.h"
#include "parse.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
void tool_set_prompt_dist(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("First Point: "));
      break;
    case 1:
      set_prompt_text( _("Second Point: "));
      break;
    }
}

void tool_set_tool_dist(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("dist"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_DIST;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  set_prompt();

  work_document=NULL;
}

void tool_button_dist(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x ,&y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
       make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
       print( _("\n  Delta X: %f Y: %f  Length: %f Angle: %f"), x-gl_oldmx, y-gl_oldmy, dist(x, y, gl_oldmx, gl_oldmy),
	     get_angle(gl_oldmx, gl_oldmy, x, y));
       remove_old();
       tool_idle(doc);
       break;
    }
}

void tool_prompt_dist(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_dist(doc,1,tx,ty);
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_RETURN:
	remove_old();
	tool_idle(doc);
	return; 
	break;
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_dist(doc,1,tx,ty);
	break;
      }
    }  
}



