/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "xor.h"
#include "select.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "redraw.h"
#include "parse.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  F U N C T I O N S  -- */
static void delete_nodes(gpointer data, gpointer user)
{
  g_free(data);
}

void erase_select_object(DOCUMENT *doc)
{
  OBJECT *object;
  GList *select_list;

  remove_old();

  if(doc==NULL || doc->selection_list==NULL)
    return;

  select_list=g_list_first(doc->selection_list);

  while(select_list!=NULL)
    {
      object=(OBJECT *)select_list->data;

      object->selected=OBJECT_ERASE;
      redraw_object(doc, object);

      /* Make some clean-ups */
      /* Object specific */
      if(object->type==TYPE_POLYLINE)
	{
	  g_list_foreach(object->data.polyline.node_list,delete_nodes,NULL);
	  g_list_free(object->data.polyline.node_list);
	}
      if(object->type==TYPE_SPLINE)
	{
	  g_list_foreach(object->data.spline.node_list,delete_nodes,NULL);
	  g_list_free(object->data.spline.node_list);
	  g_free(object->data.spline.cache_point);
	}
      if(object->type==TYPE_TEXT)
	g_free(object->data.text.text);

      /* Generell */
      g_free(object->name);
      if(object->type_colour==USERDEFINED)
	g_free(object->colour);

      object->layer->object_list=g_list_remove(object->layer->object_list, 
					       (gpointer) object); 
      g_free(object);

      select_list=g_list_next(select_list);
    }

  /* Remove all in selection list */
  g_list_free(doc->selection_list);
  doc->selection_list=NULL;
  tool_idle(doc);
}

void tool_set_tool_erase(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("erase"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_ERASE;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to erase: "), _("Selection - Erase"));
  set_prompt();

  work_document=NULL;
}













