/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "select.h"
#include "parse.h"
#include "xor.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
static gchar *make_common_print(OBJECT *object)
{
  gchar *t, *s;

  /* Make name och layer name */
  t=g_strdup_printf( _("Name: \"%s\" Layer: \"%s\""), object->name, object->layer->name);

  /* Make colour */
  if(object->type_colour==BYLAYER)
    s=g_strdup_printf( _("%s Colour: By Layer"), t);
  else if(object->type_colour==BYBLOCK)
    s=g_strdup_printf( _("%s Colour: By Block"), t);
  else
    s=g_strdup_printf( _("%s Colour: %f,%f,%f"), t, object->colour->c[0], object->colour->c[1], object->colour->c[2]);

  g_free(t);

  /* Make linestyle */
  if(object->type_linestyle==BYLAYER)
    t=g_strdup_printf( _("%s\n  Linestyle: By Layer"),s);
  else if(object->type_linestyle==BYBLOCK)
    t=g_strdup_printf( _("%s\n  Linestyle: By Block"),s);
  else
    t=g_strdup_printf( _("%s\n  Linestyle: \"%s\""), s, object->linestyle->name);

  /* Make linewidth */
  if(object->type_linewidth==BYLAYER)
    s=g_strdup_printf( _("%s  Linewidth: By Layer"),t);
  else if(object->type_linewidth==BYBLOCK)
    s=g_strdup_printf( _("%s  Linewidth: By Block"),t);
  else
    s=g_strdup_printf( _("%s  Linewidth: %f"), t, object->linewidth);

  g_free(t);

  /* Make bounding box */
  t=g_strdup_printf( _("%s\n  Bounding box: %f,%f,%f,%f"), s, object->bounding_box[0], 
		    object->bounding_box[1],object->bounding_box[2],object->bounding_box[3]);

  g_free(s);

  return t;
}

void list_select_object(DOCUMENT *doc)
{
  gchar *common_prop;
  OBJECT *object;
  GList *select_list;
  gdouble x1,y1,x2,y2;

  remove_old();

  if(doc==NULL || doc->selection_list==NULL)
    {
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  select_list=g_list_first(doc->selection_list);

  print("\n");

  while(select_list!=NULL)
    {
      object=(OBJECT *)select_list->data;

      /* Write common properties for objects */
      common_prop=make_common_print(object);

      switch(object->type)
	{
	case TYPE_LINE:
	  x1=object->data.line.p1.x;
	  y1=object->data.line.p1.y;
	  x2=object->data.line.p2.x;
	  y2=object->data.line.p2.y;
	  print( _("\n  Type: Line %s\n  from X: %f, Y: %f to X: %f Y: %f\n  Length: %f  Angle: %f  Delta X: %f Y: %f\n"),
		common_prop, x1, y1, x2, y2, dist(x1,y1,x2,y2), get_angle(x1,y1,x2,y2),
		x2-x1,y2-y1);
	  break;
	case TYPE_CIRCLE:
	  x1=object->data.circle.radius;
	  print( _("\n  Type: Circle %s\n  Center X: %f Y: %f  Radius: %f\n  Circumference: %f  Area: %f\n"),
		common_prop, object->data.circle.centrum.x, object->data.circle.centrum.y,x1,
		2*3.14159*x1, 3.14159*x1*x1);
	  break;
	case TYPE_ARC:
	  x1=object->data.arc.radius;
	  x2=object->data.arc.start_angle;
	  y2=object->data.arc.end_angle;
	  print( _("\n  Type: Arc %s\n  Center X: %f Y: %f  Radius: %f\n  Start angle: %f  End angle: %f\n  Length: %f\n"),
		common_prop, object->data.arc.centrum.x, object->data.arc.centrum.y, x1,
		 x2,y2, ABS(x2-y2)/360.0*3.14159*x1*x1);
	  break;
	}

      g_free(common_prop);
      select_list=g_list_next(select_list);      
    }

  deselect_all_objects(doc);
  tool_idle(doc);
}


void tool_set_tool_list(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("list"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_LIST;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to list: "), _("Selection - List"));
  set_prompt();

  work_document=NULL;
}



