/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "draw.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "xor.h"
#include "select.h"
#include "calc_arc.h"
#include "obj_line.h"
#include "obj_spline.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_ellipse.h"
#include "drag.h"
#include "redraw.h"
#include "parse.h"
#include "calc.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */


/* --  F U N C T I O N S  -- */
static void mirror_objects(DOCUMENT *doc, gdouble angle)
{
  GList *select_list;
  OBJECT *object;
  OBJECT *new_object;
  GList *node_list;
  gdouble Ax,Ay, Bx,By, Cx,Cy,r, tangle;

  select_list=g_list_first(doc->selection_list);
  
  while(select_list!=NULL)
    {
      object=(OBJECT *)select_list->data;
      
      switch(object->type)
	{
	case TYPE_CLINE:
	case TYPE_RLINE:
	case TYPE_LINE:
	  Ax=object->data.line.p1.x;
	  Ay=object->data.line.p1.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);	  
	  Bx=object->data.line.p2.x;
	  By=object->data.line.p2.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Bx,&By);
	  if(object->type==TYPE_LINE)
	    {
	      redraw_object(doc, obj_add_line(doc,object->layer, object->name, object->type_colour, object->colour, 
					      object->type_linestyle, object->linestyle,
					      object->type_linewidth, object->linewidth, Ax, Ay , Bx, By));
	    }
	  else if(object->type==TYPE_RLINE)
	    {
	      redraw_object(doc, obj_add_rline(doc,object->layer, object->name, object->type_colour, object->colour, 
					       object->type_linestyle, object->linestyle,
					       object->type_linewidth, object->linewidth, Ax, Ay , Bx, By));
	    }
	  else if(object->type==TYPE_CLINE)
	    {
	      redraw_object(doc, obj_add_cline(doc,object->layer, object->name, object->type_colour, object->colour, 
					       object->type_linestyle, object->linestyle,
					       object->type_linewidth, object->linewidth, Ax, Ay , Bx, By));
	    }	  
	  break;
	case TYPE_CIRCLE:
	  Ax=object->data.circle.centrum.x;
	  Ay=object->data.circle.centrum.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	  redraw_object(doc, obj_add_circle(doc, object->layer,  object->name, object->type_colour, object->colour, 
					    object->type_linestyle, object->linestyle,
					    object->type_linewidth, object->linewidth, Ax,Ay,object->data.circle.radius));

	  break;
	case TYPE_ELLIPSE:
	  Ax=object->data.ellipse.centrum.x;
	  Ay=object->data.ellipse.centrum.y;
	  Bx=Ax+object->data.ellipse.major*cos(object->data.ellipse.angle*M_PI/180.0);
	  By=Ay-object->data.ellipse.major*sin(object->data.ellipse.angle*M_PI/180.0);
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Bx,&By);
	  tangle=get_angle(Ax,Ay,Bx,By);
	  redraw_object(doc, obj_add_ellipse(doc, object->layer,  object->name, object->type_colour, object->colour, 
					     object->type_linestyle, object->linestyle,
					     object->type_linewidth, object->linewidth, Ax,Ay,
					     object->data.ellipse.major, object->data.ellipse.minor,
					     tangle));
	  break;
	case TYPE_ARC:
	  r=object->data.arc.radius;
	  
	  Cx=object->data.arc.centrum.x;
	  Cy=object->data.arc.centrum.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Cx,&Cy);
	  
	  Ax=object->data.arc.centrum.x+r*cos(object->data.arc.start_angle*M_PI/180);
	  Ay=object->data.arc.centrum.y-r*sin(object->data.arc.start_angle*M_PI/180);
	  misc_mirror_point(angle,gl_oldmx,gl_oldmy, &Ax,&Ay);
	  
	  Bx=object->data.arc.centrum.x+r*cos(object->data.arc.end_angle*M_PI/180);
	  By=object->data.arc.centrum.y-r*sin(object->data.arc.end_angle*M_PI/180);
	  misc_mirror_point(angle,gl_oldmx,gl_oldmy, &Bx,&By);
	  redraw_object(doc, obj_add_arc(doc, object->layer, object->name, object->type_colour, object->colour, 
					 object->type_linestyle, object->linestyle,
					 object->type_linewidth, object->linewidth,
					 Cx,Cy,r, get_angle(Cx,Cy,Bx,By), get_angle(Cx,Cy,Ax,Ay)));

	  break;
	case TYPE_POLYLINE:
	  new_object=copy_polyline(doc,object->layer,object);

	  node_list=new_object->data.polyline.node_list;

	  while(node_list!=NULL)
	    {
	      Ax=((NODE *)node_list->data)->x;
	      Ay=((NODE *)node_list->data)->y;
	      misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	      ((NODE *)node_list->data)->x=Ax;
	      ((NODE *)node_list->data)->y=Ay;	     
	      node_list=node_list->next;
	    }
	  	  
	  obj_bound_polyline(new_object);
	  redraw_object(doc, new_object);	  
	  break;
	case TYPE_SPLINE:
	  new_object=copy_spline(doc,object->layer,object);

	  node_list=new_object->data.spline.node_list;

	  while(node_list!=NULL)
	    {
	      Ax=((NODE *)node_list->data)->x;
	      Ay=((NODE *)node_list->data)->y;
	      misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	      ((NODE *)node_list->data)->x=Ax;
	      ((NODE *)node_list->data)->y=Ay;	     
	      node_list=node_list->next;
	    }	 
	  if(new_object->data.spline.cache_point!=NULL)
	    g_free(new_object->data.spline.cache_point);

	  new_object->data.spline.cache_point=NULL;
	  obj_bound_spline(new_object);
	  redraw_object(doc, new_object);	  
	  break;
	case TYPE_POINT:
	  Ax=object->data.point.centrum.x;
	  Ay=object->data.point.centrum.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	  redraw_object(doc, obj_add_point(doc, object->layer, object->name, object->type_colour, object->colour, 
					   object->type_linestyle, object->linestyle,
					   object->type_linewidth, object->linewidth,
					   object->data.point.point_style, object->data.point.point_type,
					   object->data.point.point_size, 
					   Ax,Ay));
	  break;
	}

      select_list=g_list_next(select_list);     
    }
}


void tool_set_prompt_mirror(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("First point of mirror line: "));
      break;
    case 1: 
      set_prompt_text( _("Second point: "));
      break;
    }

}

void tool_set_tool_mirror(GtkWidget *widget, gpointer data)
{
  remove_old();
  
  if(!command_from_keyboard)
    print( _("mirror"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_MIRROR;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to mirror: "), _("Selection - Mirror"));
  set_prompt();

  work_document=NULL;
}

void tool_button_mirror(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);

      mirror_objects(doc, (360-get_angle(gl_oldmx, gl_oldmy, x,y))*M_PI/180.0);

      deselect_all_objects(doc);
      tool_idle(doc);
      break;
    }
}

void tool_prompt_mirror(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_mirror(doc,1,tx,ty);
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
			  TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_mirror(doc,1,tx,ty);
	break;
      }
    }  
}

void tool_xor_mirror(DOCUMENT *doc,gdouble x, gdouble y)
{
  GList *select_list;
  OBJECT *object;
  GdkGC *gc;
  gdouble Ax,Ay, Bx,By,Cx,Cy,r, tangle;
  gint i,num;
  GdkPoint *point_list;
  POINT *cache_list;
  GList *node_list;
  TRANSFORM trans;
  OBJECT_ARC arc;
  gdouble angle;

  angle=(360-get_angle(gl_oldmx, gl_oldmy, x,y))*M_PI/180.0;
  
  misc_xor_line(doc,x,y);
  
  select_list=g_list_first(doc->selection_list);
  
  while(select_list!=NULL)
    {
      object=(OBJECT *)select_list->data;
      
      gc=misc_setup_gc_object_xor(doc,object);      
      
      switch(object->type)
	{
	case TYPE_RLINE:
	case TYPE_CLINE:
	case TYPE_LINE:
	  Ax=object->data.line.p1.x;
	  Ay=object->data.line.p1.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	  
	  Bx=object->data.line.p2.x;
	  By=object->data.line.p2.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Bx,&By);
	  
	  if(object->type==TYPE_LINE)
	    {
	      gdk_draw_line(doc->widgets.draw->window, 
			    gc,
			    WLDtoSCR_x(doc,Ax), 
			    WLDtoSCR_y(doc,Ay),
			    WLDtoSCR_x(doc,Bx), 
			    WLDtoSCR_y(doc,By));
	      
	    }
	  else if(object->type==TYPE_RLINE)
	    {
	      gdouble nx,ny;	      
	      calc_endpoint_for_ray(doc,Ax,Ay,Bx,By, &nx, &ny);

	      gdk_draw_line(doc->widgets.draw->window, 
			    gc,
			    WLDtoSCR_x(doc,Ax),WLDtoSCR_y(doc,Ay),
			    WLDtoSCR_x(doc,nx),WLDtoSCR_y(doc,ny));
	    }
	  else if(object->type==TYPE_CLINE)
	    {
	      gdouble nx1,ny1;
	      gdouble nx2,ny2;

	      calc_endpoint_for_cline(doc,Ax,Ay,Bx,By, &nx1, &ny1, &nx2, &ny2);

	      gdk_draw_line(doc->widgets.draw->window, 
			    gc,
			    WLDtoSCR_x(doc,nx1), 
			    WLDtoSCR_y(doc,ny1),
			    WLDtoSCR_x(doc,nx2), 
			    WLDtoSCR_y(doc,ny2));
	    }
	  break;
	case TYPE_CIRCLE:
	  Ax=object->data.circle.centrum.x;
	  Ay=object->data.circle.centrum.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	  Ax=Ax-object->data.circle.centrum.x;
	  Ay=Ay-object->data.circle.centrum.y;
	  trans.type=OBJ_TRANSLATE;
	  trans.trans_x=Ax;
	  trans.trans_y=Ay;
	  obj_draw_circle_block(doc,gc,object,&trans);	  
	  break;
	case TYPE_ELLIPSE:
	  Ax=object->data.ellipse.centrum.x;
	  Ay=object->data.ellipse.centrum.y;
	  Bx=Ax+object->data.ellipse.major*cos(object->data.ellipse.angle*M_PI/180.0);
	  By=Ay-object->data.ellipse.major*sin(object->data.ellipse.angle*M_PI/180.0);
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Bx,&By);
	  tangle=get_angle(Ax,Ay,Bx,By);
	  draw_ellipse(doc,doc->widgets.draw->window, 
		       misc_setup_gc(doc),
		       WLDtoSCR_x(doc,Ax), WLDtoSCR_y(doc,Ay), 
		       WLDtoSCR(doc,object->data.ellipse.major),WLDtoSCR(doc, object->data.ellipse.minor),tangle);
	  break;
	case TYPE_ARC:
	  r=object->data.arc.radius;

	  Cx=object->data.arc.centrum.x;
	  Cy=object->data.arc.centrum.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Cx,&Cy);
	  
	  Ax=object->data.arc.centrum.x+r*cos(object->data.arc.start_angle*M_PI/180);
	  Ay=object->data.arc.centrum.y-r*sin(object->data.arc.start_angle*M_PI/180);
	  misc_mirror_point(angle,gl_oldmx,gl_oldmy, &Ax,&Ay);

	  Bx=object->data.arc.centrum.x+r*cos(object->data.arc.end_angle*M_PI/180);
	  By=object->data.arc.centrum.y-r*sin(object->data.arc.end_angle*M_PI/180);
	  misc_mirror_point(angle,gl_oldmx,gl_oldmy, &Bx,&By);

	  draw_arc(doc->widgets.draw->window,gc, FALSE,
		   WLDtoSCR_x(doc,Cx-r), 
		   WLDtoSCR_y(doc,Cy-r),
		   WLDtoSCR(doc,r*2), 
		   WLDtoSCR(doc,r*2),
		   get_angle(Cx,Cy,Bx,By),
		   get_angle(Cx,Cy,Ax,Ay) );

	  break;
	case TYPE_POLYLINE:
	  node_list=object->data.polyline.node_list;

	  Ax=((NODE *)node_list->data)->x;
	  Ay=((NODE *)node_list->data)->y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);

	  node_list=node_list->next;

	  while(node_list!=NULL)
	    {
	      Bx=((NODE *)node_list->data)->x;
	      By=((NODE *)node_list->data)->y;
	      misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Bx,&By);

	      if(((NODE *)node_list->data)->type==NODE_LINE)
		{
		  gdk_draw_line(doc->widgets.draw->window, gc, WLDtoSCR_x(doc,Ax),WLDtoSCR_y(doc,Ay),
				WLDtoSCR_x(doc,Bx),WLDtoSCR_y(doc,By));
		  Ax=Bx;
		  Ay=By;
		}
	      else
		{
		  if(node_list->next!=NULL)
		    {
		      Cx=((NODE *)node_list->next->data)->x;
		      Cy=((NODE *)node_list->next->data)->y;
		      misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Cx,&Cy);

		      if(calc_arc_3p(Ax,Ay,Bx,By,Cx,Cy,&arc))
			{
			  draw_arc(doc->widgets.draw->window,gc,FALSE,
				   WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
				   WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),
				   arc.start_angle, arc.end_angle);
			}
		      Ax=Cx;
		      Ay=Cy;
	      	     
		      node_list=g_list_next(node_list);
		    }
		  
		}
	      
	      node_list=node_list->next;
	    }
	  break;
	case TYPE_SPLINE:
	  num=object->data.spline.num;
	  
	  /* Transform world coordinates to screen */
	  point_list=g_new(GdkPoint, num);
	  
	  cache_list=object->data.spline.cache_point;
	  
	  i=0;
	  
	  while(i<num)
	    {
	      Ax=cache_list[i].x;
	      Ay=cache_list[i].y;
	      misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	      point_list[i].x=WLDtoSCR_x(doc, Ax);
	      point_list[i].y=WLDtoSCR_y(doc, Ay);	     	       
	      i++;
	    }
	  
	  /* Draw lines */
	  gdk_draw_lines(doc->widgets.draw->window, doc->xor_gc, point_list,num);
	  break;
	case TYPE_POINT:
	  Ax=object->data.point.centrum.x;
	  Ay=object->data.point.centrum.y;
	  misc_mirror_point(angle, gl_oldmx,gl_oldmy, &Ax,&Ay);
	  Ax-=object->data.point.centrum.x;
	  Ay-=object->data.point.centrum.y;
	  trans.type=OBJ_TRANSLATE;
	  trans.trans_x=Ax;
	  trans.trans_y=Ay;
	  obj_draw_point_block(doc,gc,object,&trans);
	  break;
	}

      select_list=g_list_next(select_list);
    }

}




