/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "struct.h"
#include "global.h"
#include "dia_comm.h"
#include "select.h"
#include "calc.h"
#include "drag.h"
#include "draw.h"
#include "redraw.h"
#include "misc.h"
#include "xor.h"
#include "dia_coords.h"
#include "dia_message.h"
#include "parse.h"
#include "tool_copy.h"
#include "tool_erase.h"

#include "obj_line.h"
#include "obj_polyline.h"
#include "obj_circle.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_arc.h"
#include "obj_cline.h"
#include "obj_rline.h"
#include "obj_text.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static OBJECT *object=NULL;

/* --  F U N C T I O N S  -- */
static void offset_polyline(DOCUMENT *doc,GdkGC *gc, gboolean xor, OBJECT *pline, gdouble x, gdouble y);

void offset_object_dist(DOCUMENT *doc, OBJECT *object, gdouble d)
{
  gdouble alfa, beta;
  gdouble r;
  gdouble Ax,Ay,Bx,By,width;

  switch(object->type)
    {
    case TYPE_LINE:
      r=dist(object->data.line.p1.x, 
	     object->data.line.p1.y, 
	     object->data.line.p2.x, 
	     object->data.line.p2.y  );

      alfa=-(object->data.line.p1.x-object->data.line.p2.x)/r;
      beta=(object->data.line.p1.y-object->data.line.p2.y)/r;
      obj_draw_line(doc, obj_add_line(doc, object->layer, object->name, object->type_colour, object->colour, 
				      object->type_linestyle, object->linestyle,
				      object->type_linewidth, object->linewidth,
				      object->data.line.p1.x+beta*d, object->data.line.p1.y+alfa*d,
				      object->data.line.p2.x+beta*d, object->data.line.p2.y+alfa*d));

      break;
    case TYPE_RLINE:
      r=dist(object->data.line.p1.x, 
	     object->data.line.p1.y, 
	     object->data.line.p2.x, 
	     object->data.line.p2.y  );

      alfa=-(object->data.line.p1.x-object->data.line.p2.x)/r;
      beta=(object->data.line.p1.y-object->data.line.p2.y)/r;
      obj_draw_rline(doc, obj_add_rline(doc, object->layer, object->name, object->type_colour, object->colour, 
					object->type_linestyle, object->linestyle,
					object->type_linewidth, object->linewidth,
					object->data.line.p1.x+beta*d, object->data.line.p1.y+alfa*d,
					object->data.line.p2.x+beta*d, object->data.line.p2.y+alfa*d));
      
      break;
    case TYPE_CLINE:
      r=dist(object->data.line.p1.x, 
	     object->data.line.p1.y, 
	     object->data.line.p2.x, 
	     object->data.line.p2.y  );
      
      alfa=-(object->data.line.p1.x-object->data.line.p2.x)/r;
      beta=(object->data.line.p1.y-object->data.line.p2.y)/r;
      obj_draw_cline(doc, obj_add_cline(doc, object->layer, object->name, object->type_colour, object->colour, 
					object->type_linestyle, object->linestyle,
					object->type_linewidth, object->linewidth,
					object->data.line.p1.x+beta*d, object->data.line.p1.y+alfa*d,
					object->data.line.p2.x+beta*d, object->data.line.p2.y+alfa*d));
      break;
    case TYPE_CIRCLE:
      obj_draw_circle(doc, obj_add_circle(doc, object->layer, object->name, object->type_colour, object->colour, 
					  object->type_linestyle, object->linestyle,
					  object->type_linewidth, object->linewidth,
					  object->data.circle.centrum.x,
					  object->data.circle.centrum.y,
					  object->data.circle.radius+d));
      break;
    case TYPE_ELLIPSE:
      obj_draw_ellipse(doc, obj_add_ellipse(doc, object->layer, object->name, object->type_colour, object->colour, 
					    object->type_linestyle, object->linestyle,
					    object->type_linewidth, object->linewidth,
					    object->data.ellipse.centrum.x,
					    object->data.ellipse.centrum.y,
					    object->data.ellipse.major+d,object->data.ellipse.minor+d,
					    object->data.ellipse.angle));
      break;
    case TYPE_ELARC:
      obj_draw_elarc(doc, obj_add_elarc(doc, object->layer, object->name, object->type_colour, object->colour, 
					object->type_linestyle, object->linestyle,
					object->type_linewidth, object->linewidth,
					object->data.elarc.centrum.x,
					object->data.elarc.centrum.y,
					object->data.elarc.major+d,object->data.elarc.minor+d,
					object->data.elarc.angle,
					object->data.elarc.start_angle,object->data.elarc.end_angle));
      break;
    case TYPE_ARC:
      obj_draw_arc(doc, obj_add_arc(doc, object->layer, object->name, object->type_colour, object->colour, 
				    object->type_linestyle, object->linestyle,
				    object->type_linewidth, object->linewidth,
				    object->data.arc.centrum.x,
				    object->data.arc.centrum.y,
				    object->data.arc.radius+d,
				    object->data.arc.start_angle,
				    object->data.arc.end_angle));
      break;
    case TYPE_TEXT:
      width=strlen(object->data.text.text)*(object->data.text.height*object->data.text.width);
      
      Ax=object->data.text.insert.x;
      Ay=object->data.text.insert.y;
      Bx=width*cos(-object->data.text.angle)+Ax;
      By=width*sin(-object->data.text.angle)+Ay;
      
      alfa=-(Ax-Bx)/width;
      beta=(Ay-By)/width;
      
      Ax=Ax+beta*d-object->data.text.insert.x;
      Ay=Ay+alfa*d-object->data.text.insert.y;

      obj_draw_text(doc, obj_add_text(doc, object->layer,
				      object->name, object->type_colour, object->colour, 
				      object->type_linestyle, object->linestyle,
				      object->type_linewidth, object->linewidth,
				      object->data.text.style, object->data.text.justification,
				      object->data.text.angle, object->data.text.height,
				      object->data.text.insert.x+Ax,object->data.text.insert.y+Ay,
				      object->data.text.alignment.x+Ax,object->data.text.alignment.y+Ay,
				      object->data.text.text));
      break;
    case TYPE_POLYLINE:

      break;
    }
}

void offset_object_button(DOCUMENT *doc, OBJECT *object, gdouble x,gdouble y)
{
  OBJECT *new_object;
  gdouble r;
  gdouble L,s;
  gdouble d1,d2;
  gdouble Ax,Ay,Bx,By;

  switch(object->type)
    {
    case TYPE_RLINE:
    case TYPE_CLINE:
    case TYPE_LINE:
      L=dist(object->data.line.p1.x, 
	     object->data.line.p1.y, 
	     object->data.line.p2.x, 
	     object->data.line.p2.y);
      s=((object->data.line.p1.y-y)*(object->data.line.p2.x-object->data.line.p1.x)-
	(object->data.line.p1.x-x)*(object->data.line.p2.y-object->data.line.p1.y))/(L*L);

      offset_object_dist(doc, object, -s*L);
      break;
    case TYPE_CIRCLE:
      r=dist(x,y,object->data.circle.centrum.x,object->data.circle.centrum.y);
      obj_draw_circle(doc, obj_add_circle(doc, object->layer, object->name, object->type_colour, object->colour, 
					  object->type_linestyle, object->linestyle,
					  object->type_linewidth, object->linewidth,
					  object->data.circle.centrum.x,
					  object->data.circle.centrum.y,
					  r));
      break;
    case TYPE_ELLIPSE:
      d2=get_angle(object->data.ellipse.centrum.x,object->data.ellipse.centrum.y,x,y)-object->data.ellipse.angle;
      
      while(d2<0) d2+=360;
      while(d2>360) d2-=360;
      
      d2=ellipse_alfa2t(d2,object->data.ellipse.major,object->data.ellipse.minor)*M_PI/180;
      
      gl_oldmx=object->data.ellipse.major*cos(d2)+object->data.ellipse.centrum.x;
      gl_oldmy=-object->data.ellipse.minor*sin(d2)+object->data.ellipse.centrum.y;
      
      misc_rotate_point(-object->data.ellipse.angle*M_PI/180.0, 
			object->data.ellipse.centrum.x,object->data.ellipse.centrum.y, 
			&gl_oldmx, &gl_oldmy);
      
      d1=dist(x, y, object->data.ellipse.centrum.x, object->data.ellipse.centrum.y)-
	dist(gl_oldmx, gl_oldmy, object->data.ellipse.centrum.x, object->data.ellipse.centrum.y);
      
      obj_draw_ellipse(doc, obj_add_ellipse(doc, object->layer, object->name, object->type_colour, object->colour, 
					    object->type_linestyle, object->linestyle,
					    object->type_linewidth, object->linewidth,
					    object->data.ellipse.centrum.x,
					    object->data.ellipse.centrum.y,
					    object->data.ellipse.major+d1,object->data.ellipse.minor+d1,
					    object->data.ellipse.angle));
      break;
    case TYPE_ELARC:
      gl_oldmx=x-object->data.elarc.centrum.x;
      gl_oldmy=y-object->data.elarc.centrum.y;

      r=object->data.elarc.angle*M_PI/180.0;

      x=gl_oldmx*cos(-r)+gl_oldmy*sin(-r);
      y=-gl_oldmx*sin(-r)+gl_oldmy*cos(-r);
      
      d2=object->data.elarc.minor/object->data.elarc.major;
      d1=sqrt( (d2*d2*x*x+y*y)/(d2*d2) );
      
      obj_draw_elarc(doc, obj_add_elarc(doc, object->layer, object->name, object->type_colour, object->colour, 
					object->type_linestyle, object->linestyle,
					object->type_linewidth, object->linewidth,
					object->data.elarc.centrum.x,
					object->data.elarc.centrum.y,
					d1,d2*d1,
					object->data.elarc.angle,
					object->data.elarc.start_angle, object->data.elarc.end_angle));
      break;
    case TYPE_ARC:
      r=dist(x,y,object->data.arc.centrum.x,object->data.arc.centrum.y);
      obj_draw_arc(doc, obj_add_arc(doc, object->layer,
				    object->name, object->type_colour, object->colour, 
				    object->type_linestyle, object->linestyle,
				    object->type_linewidth, object->linewidth,
				    object->data.arc.centrum.x,
				    object->data.arc.centrum.y,
				    r,
				    object->data.arc.start_angle,
				    object->data.arc.end_angle));
      break;
    case TYPE_TEXT:
      L=strlen(object->data.text.text)*(object->data.text.height*object->data.text.width);
      Ax=object->data.text.insert.x;
      Ay=object->data.text.insert.y;
      Bx=L*cos(-object->data.text.angle)+Ax;
      By=L*sin(-object->data.text.angle)+Ay;
      
      s=((Ay-y)*(Bx-Ax)-
	 (Ax-x)*(By-Ay))/(L*L);

      d1=-(Ax-Bx)/L;
      d2=(Ay-By)/L;            
     
      r=-s*L;
       
      x=Ax+d2*r-object->data.text.insert.x;
      y=Ay+d1*r-object->data.text.insert.y;

      obj_draw_text(doc, obj_add_text(doc, object->layer,
				      object->name, object->type_colour, object->colour, 
				      object->type_linestyle, object->linestyle,
				      object->type_linewidth, object->linewidth,
				      object->data.text.style, object->data.text.justification,
				      object->data.text.angle, object->data.text.height,
				      object->data.text.insert.x+x,object->data.text.insert.y+y,
				      object->data.text.alignment.x+x,object->data.text.alignment.y+y,
				      object->data.text.text));
      break;
    case TYPE_POLYLINE:
      new_object=copy_polyline(doc, object->layer, object);
      offset_polyline(doc, NULL, FALSE, new_object, x, y);
      break;
    }
}

void tool_set_prompt_offset(void)
{
  switch(gl_stage)
    {
    case 0:
      set_prompt_text( _("Select object to offset: "));
      break;
    case 1: 
      set_prompt_text( _("Through or offset point: "));
      break;
    }

}

void tool_button_offset(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{

  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      object=select_singel_object(doc,x,y);
      if(object!=NULL)
        {
	  if(object->type==TYPE_POINT)
	      print( _("\n  Unable to offset a point."));
	  else if(object->type==TYPE_DIMENSION)
	      print( _("\n  Unable to offset a dimension."));
	  else
	    {
	      gl_stage=1;
	      object->selected=OBJECT_SEL;
	      redraw_object(doc,object);
	    }
        }
      else
        print( _("\n  No object found."));
      set_cursor(doc);
      set_prompt();
      break; 
    case 1:
      remove_old();
      offset_object_button(doc, object, x,y);
      deselect_all_objects(doc);
      tool_idle(doc);
      break;
    }
}

void tool_prompt_offset(DOCUMENT *doc, gchar *buffer)
{
  gdouble d,tx,ty;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_offset(doc,1,tx,ty);
	break;
      }

    }
  else
    {
      switch(parse_prompt((PROMPT_OSNAP|PROMPT_POINT1|PROMPT_RETURN),
			  buffer,FALSE,
			  NULL,
			  0,0,NULL,NULL,&d,NULL))
	{
	case PROMPT_RETURN:
	  gl_tool=TOOL_IDLE;
	  work_document=NULL;
	  /*      ChangeCoordLabel(0); */
	  set_prompt();
	  break;
	case PROMPT_POINT1:
	  work_document=doc;
	  remove_old();
	  offset_object_dist(doc, object, d);
	  deselect_all_objects(doc);
	  tool_idle(doc);
	  break;
	case PROMPT_ERROR:
	default:
	  return;
	}
    }
}

void tool_set_tool_offset(GtkWidget *widget, gpointer data)
{
  remove_old();
  
  if(!command_from_keyboard)
    print( _("offset"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_OFFSET;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  set_prompt();
  work_document=NULL;
}

static int calc_intersection_line(DOCUMENT *doc, gdouble Ax,gdouble Ay,gdouble Bx, gdouble By, 
				  gdouble x, gdouble y, gdouble xcen, gdouble ycen, gdouble *d,
				  gdouble *d2)
{
  gdouble k, m;
  gdouble ik, im;
  gdouble ix, iy;

  /* Calculate user line */
  if(ABS(xcen-x)>GFLOAT_SENSITIVE)
    k=(ycen-y)/(xcen-x);
  else
    k=1000000;     /* Big value */

  m=ycen-k*xcen;

  /* Calculate polyline to test */
  if(ABS(Ax-Bx)>GFLOAT_SENSITIVE)
    ik=(Ay-By)/(Ax-Bx);
  else
    ik=1000000;     /* Big value */

  im=Ay-ik*Ax;

  /* Calculate intersection point */
  ix=(im-m)/(k-ik);
  iy=ik*ix+im;

  /* Is this point on the line? */
  if(misc_point_on_line(ix,iy,Ax,Ay,Bx,By, 0.001))
    {
      gdouble tk,tm;

      /* Calculate tangent to center point */
      if(ABS(k)>GFLOAT_SENSITIVE)
	tk=-1/k;
      else
	tk=-1000000;

      tm=ycen-tk*xcen;
    
      if((tk*ix-iy+tm<=GFLOAT_SENSITIVE && tk*x-y+tm<=GFLOAT_SENSITIVE) ||
	 (tk*ix-iy+tm>GFLOAT_SENSITIVE && tk*x-y+tm>GFLOAT_SENSITIVE))	
	{

	  *d=dist(ix,iy,x,y);

	  if((ik*x-y+im<=GFLOAT_SENSITIVE && ik*xcen-ycen+im<=GFLOAT_SENSITIVE) ||
	     (ik*x-y+im>GFLOAT_SENSITIVE && ik*xcen-ycen+im>GFLOAT_SENSITIVE))	
	    *d=-*d;

	  *d2=dist(ix,iy,xcen,ycen);

	  return TRUE;
	}
    }

  return FALSE;
}

static void offset_polyline(DOCUMENT *doc,GdkGC *gc, gboolean xor, OBJECT *pline, gdouble x, gdouble y)
{
  gint done=FALSE;
  gdouble d,d2;
  gdouble xcen=0,ycen=0;
  gdouble ox,oy;
  gdouble tx,ty;
  gdouble sx,sy;
  GList *node_list;
  POINT p;
  gdouble s,L;

  /* Calculate center of polyline */
  misc_calc_centroid(pline->data.polyline.node_list, &p);
  xcen=p.x;
  ycen=p.y;

  /* Calculate intersection */
  node_list=g_list_first(pline->data.polyline.node_list);

  sx=ox=((NODE *)node_list->data)->x;
  sy=oy=((NODE *)node_list->data)->y;

  node_list=g_list_next(node_list);

  while(node_list!=NULL && !done)
    {
      tx=((NODE *)node_list->data)->x;
      ty=((NODE *)node_list->data)->y;

      done=calc_intersection_line(doc,ox,oy,tx,ty,x,y,xcen,ycen, &d, &d2);

      if(done)
	{
	  /* Calc dist from line to point */
	  L=sqrt( (tx-ox)*(tx-ox) + (ty-oy)*(ty-oy) );
	  s=((oy-y)*(tx-ox)-(ox-x)*(ty-oy))/(L*L);
	}

      ox=tx;      
      oy=ty;
      node_list=g_list_next(node_list);
    }

  /* Closed polygon */
  if(pline->data.polyline.closed && !done)
    {
      done=calc_intersection_line(doc,ox,oy,sx,sy,x,y,xcen,ycen, &d, &d2);

      if(done)
	{
	  /* Calc dist from line to point */
	  L=sqrt( (sx-ox)*(sx-ox) + (sy-oy)*(sy-oy) );
	  s=((oy-y)*(sx-ox)-(ox-x)*(sy-oy))/(L*L);
	}
    }
  

  g_warning("d=%f    d2=%f    distance %f ", d,d2, s);
  /* We have a distance */
  if(done)
    {
      TRANSFORM trans;

      trans.type=OBJ_SCALE;
      trans.cen_x=xcen;
      trans.cen_y=ycen;
      trans.scale_x=d/d2+1.0;
      trans.scale_y=d/d2+1.0;

      if(xor)
	{
	  UpdateSingelCoord(2, d);	  
	  obj_draw_polyline_block(doc, gc, object, &trans);
	}
      else
	obj_transform_polyline(object, &trans);
    }

}


void tool_xor_offset(DOCUMENT *doc,gdouble x, gdouble y)
{
  TRANSFORM trans;
  GdkGC *gc;
  gdouble alfa, beta;
  gdouble L,s,d;
  gdouble d1,d2;
  gdouble Ax,Ay,Bx,By,width;

  if(object->selected==OBJECT_SEL)
    {
      gc=misc_setup_gc_object_xor(doc,object);     

      switch(object->type)
	{
	case TYPE_CLINE:
	case TYPE_RLINE:
	case TYPE_LINE:
	  L=dist(object->data.line.p1.x, 
		 object->data.line.p1.y, 
		 object->data.line.p2.x, 
		 object->data.line.p2.y  );
	  
	  alfa=-(object->data.line.p1.x-object->data.line.p2.x)/L;
	  beta=(object->data.line.p1.y-object->data.line.p2.y)/L;
	  
	  s=((object->data.line.p1.y-y)*(object->data.line.p2.x-object->data.line.p1.x)-
	     (object->data.line.p1.x-x)*(object->data.line.p2.y-object->data.line.p1.y))/(L*L);
	  
	  d=-s*L;
	  
	  UpdateSingelCoord(2, d);
	  
	  trans.type=OBJ_TRANSLATE;
	  trans.trans_x=beta*d;
	  trans.trans_y=alfa*d;

	  if(object->type==TYPE_LINE)
	    obj_draw_line_block(doc, gc, object, &trans);
	  else if(object->type==TYPE_CLINE)
	    obj_draw_cline_block(doc, gc, object, &trans);
	  else if (object->type==TYPE_RLINE)
	  obj_draw_rline_block(doc, gc, object, &trans);
	  break;
	case TYPE_CIRCLE:
	  d=dist(x,y,object->data.circle.centrum.x,object->data.circle.centrum.y);
	  UpdateSingelCoord(2, d-object->data.circle.radius);

	  trans.type=OBJ_SCALE;
	  trans.cen_x=object->data.circle.centrum.x;
	  trans.cen_y=object->data.circle.centrum.y;
	  trans.scale_x=d/object->data.circle.radius;
	  trans.scale_y=d/object->data.circle.radius;

	  obj_draw_circle_block(doc, gc, object, &trans);
	  break;
	case TYPE_ELLIPSE:
	    d=get_angle(object->data.ellipse.centrum.x,object->data.ellipse.centrum.y,x,y)-object->data.ellipse.angle;
	    
	    while(d<0) d+=360;
	    while(d>360) d-=360;
	    
	    d=ellipse_alfa2t(d,object->data.ellipse.major,object->data.ellipse.minor)*M_PI/180;

	    gl_oldmx=object->data.ellipse.major*cos(d)+object->data.ellipse.centrum.x;
	    gl_oldmy= -object->data.ellipse.minor*sin(d)+object->data.ellipse.centrum.y;

	    misc_rotate_point(-object->data.ellipse.angle*M_PI/180.0, 
			      object->data.ellipse.centrum.x,object->data.ellipse.centrum.y, 
			      &gl_oldmx, &gl_oldmy);

	    d1=dist(x, y, object->data.ellipse.centrum.x, object->data.ellipse.centrum.y)-
	      dist(gl_oldmx, gl_oldmy, object->data.ellipse.centrum.x, object->data.ellipse.centrum.y);
	    
	    UpdateSingelCoord(2, d1);
	    trans.type=OBJ_SCALE;
	    trans.cen_x=object->data.ellipse.centrum.x;
	    trans.cen_y=object->data.ellipse.centrum.y;
	    trans.scale_x=(object->data.ellipse.major+d1)/object->data.ellipse.major;
	    trans.scale_y=(object->data.ellipse.minor+d1)/object->data.ellipse.minor;

	    obj_draw_ellipse_block(doc, gc, object, &trans);
	  break;
	case TYPE_ELARC:
	  gl_oldmx=x-object->data.elarc.centrum.x;
	  gl_oldmy=y-object->data.elarc.centrum.y;

	  d=object->data.elarc.angle*M_PI/180.0;

	  x=gl_oldmx*cos(-d)+gl_oldmy*sin(-d);
	  y=-gl_oldmx*sin(-d)+gl_oldmy*cos(-d);
	  
	  d2=object->data.elarc.minor/object->data.elarc.major;
	  d1=sqrt( (d2*d2*x*x+y*y)/(d2*d2) );

	  trans.type=OBJ_SCALE;
	  trans.cen_x=object->data.elarc.centrum.x;
	  trans.cen_y=object->data.elarc.centrum.y;
	  trans.scale_x=d1/object->data.elarc.major;
	  trans.scale_y=d1/object->data.elarc.major;

	  UpdateSingelCoord(2, d1-object->data.elarc.major);

	  obj_draw_elarc_block(doc, gc, object, &trans);
	  break;
	case TYPE_ARC:
	  d=dist(x,y,object->data.arc.centrum.x,object->data.arc.centrum.y);
	  UpdateSingelCoord(2, d-object->data.arc.radius);

	  trans.type=OBJ_SCALE;
	  trans.cen_x=object->data.arc.centrum.x;
	  trans.cen_y=object->data.arc.centrum.y;
	  trans.scale_x=d/object->data.arc.radius;
	  trans.scale_y=d/object->data.arc.radius;

	  obj_draw_arc_block(doc, gc, object, &trans);
	  break;
	case TYPE_TEXT:
	  width=strlen(object->data.text.text)*(object->data.text.height*object->data.text.width);

	  Ax=object->data.text.insert.x;
	  Ay=object->data.text.insert.y;

	  Bx=width*cos(-object->data.text.angle*M_PI/180.0)+Ax;
	  By=width*sin(-object->data.text.angle*M_PI/180.0)+Ay;

	  alfa=-(Ax-Bx)/width;
	  beta=(Ay-By)/width;
	  
	  s=((Ay-y)*(Bx-Ax)-
	     (Ax-x)*(By-Ay))/(width*width);
	  
	  d=-s*width;
	 	  
	  trans.type=OBJ_TRANSLATE;
	  trans.trans_x=beta*d;
	  trans.trans_y=alfa*d;
	 
	  UpdateSingelCoord(2, d);
	  obj_draw_text_block(doc, gc, object, &trans);
	  break;
	case TYPE_POLYLINE:
	  offset_polyline(doc, gc, TRUE, object, x, y);
	  break;
	}      
    }
}




