/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "xor.h"
#include "select.h"
#include "calc_arc.h"
#include "obj_line.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_text.h"
#include "obj_spline.h"
#include "obj_insert.h"
#include "redraw.h"
#include "parse.h"
#include "calc.h"
#include "dimension.h"
#include "translation.h"

/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */

/* --  F U N C T I O N S  -- */
static void rotate_selected_objects(DOCUMENT *doc, TRANSFORM *transform)
{
  GList *select_list;
  OBJECT *object;
  
  select_list=g_list_first(doc->selection_list);
  
  while(select_list!=NULL)
    {
      object=(OBJECT *)select_list->data;

      object->selected=OBJECT_ERASE;
      redraw_object(doc,object);

      switch(object->type)
	{
	case TYPE_LINE:
	  obj_transform_line(object, transform);
	  break;
	case TYPE_RLINE:
	  obj_transform_rline(object, transform);
	  break;
	case TYPE_CLINE:
	  obj_transform_cline(object, transform);
	  break;
	case TYPE_CIRCLE:
	  obj_transform_circle(object, transform);
	  break;
	case TYPE_ELLIPSE:
	  obj_transform_ellipse(object, transform);
	  break;
	case TYPE_ELARC:
	  obj_transform_elarc(object, transform);
	  break;
	case TYPE_ARC:
	  obj_transform_arc(object, transform);
	  break;
	case TYPE_POLYLINE:
	  obj_transform_polyline(object, transform);
	  break;
	case TYPE_POINT:
	  obj_transform_point(object, transform);
	  break;
	case TYPE_TEXT:
	  obj_transform_text(object, transform);
	  break;
	case TYPE_SPLINE:
	  obj_transform_spline(object, transform);
	  break;
	case TYPE_BLOCK:
	  obj_transform_insert(object, transform);
	  break;
	case TYPE_DIMENSION:
	  dim_transform_dimension(object, transform);
	  break;
	}

      object->selected=OBJECT_NOT_SEL;
      redraw_object(doc,object);

      select_list=g_list_next(select_list);
    }
}

void tool_set_prompt_rotate(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Base point: "));
      break;
    case 1: 
      set_prompt_text( _("Rotation angle: "));
      break;
    }

}

void tool_set_tool_rotate(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("rotate"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_ROTATE;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to rotate: "), _("Selection - Rotate"));
  set_prompt();

  work_document=NULL;
}

void rotate_objects(DOCUMENT *doc, gdouble dangle)
{
  TRANSFORM transform;

  /* Make tranform struct */
  transform.type=OBJ_ROTATE;
  transform.cen_x=gl_oldmx;
  transform.cen_y=gl_oldmy;
  transform.rot_angle=dangle*(((gdouble)M_PI)/(gdouble)180);

  rotate_selected_objects(doc, &transform);
}

void tool_button_rotate(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      if(doc->gl_snap)
	make_snap(doc, &x, &y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      remove_old();
      make_ortho_snap(doc,gl_oldmx, gl_oldmy, &x, &y);
      rotate_objects(doc, -get_angle(gl_oldmx,gl_oldmy, x,y));
      deselect_all_objects(doc);
      doc->saved=FALSE;
      make_window_title(doc);
      tool_idle(doc);
      break;
    }
}

void tool_xor_rotate(DOCUMENT *doc,gdouble x, gdouble y)
{
  TRANSFORM transform;
  gdouble angle;

  misc_xor_line(doc,x,y);

  angle=get_angle(gl_oldmx,gl_oldmy, x,y);

  UpdateSingelCoord(2, angle);

  angle=-angle*M_PI/180.0f;

  /* Make tranform struct */
  transform.type=OBJ_ROTATE;
  transform.cen_x=gl_oldmx;
  transform.cen_y=gl_oldmy;
  transform.rot_angle=angle;

  xor_objects_transform(doc, doc, &transform);
}

void tool_prompt_rotate(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gdouble value;

  if(gl_stage==0)
    {
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
			  FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
      case PROMPT_POINT2:
	work_document=doc;
	remove_old();
	tool_button_rotate(doc,1,tx,ty);
	break;
      }
    }
  else
    {
      switch(parse_prompt(PROMPT_POINT1,buffer,
			  FALSE,NULL,0,0,NULL,NULL,&value,NULL)){
      case PROMPT_POINT1:
	work_document=doc;
	remove_old();
	rotate_objects(doc, -value);
	deselect_all_objects(doc);
	tool_idle(doc);
       	break;
      }
    }  
}















