/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "calc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "xor.h"
#include "select.h"
#include "obj_line.h"
#include "obj_rline.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_polyline.h"
#include "obj_point.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "calc_arc.h"
#include "redraw.h"
#include "draw.h"
#include "parse.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static TRANSFORM trans;

/* --  F U N C T I O N S  -- */
static void stretch_objects(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  if(object->selected==OBJECT_SEL)
    {
      object->selected=OBJECT_ERASE;
      redraw_object(doc,object);

      switch(object->type)
	{
	case TYPE_LINE:
	  obj_transform_line(object, &trans);
	  break;
	case TYPE_RLINE:
	  obj_transform_rline(object, &trans);
	  break;
	case TYPE_CIRCLE:
	  obj_transform_circle(object, &trans);
	  break;
	case TYPE_POLYLINE:
	  obj_transform_polyline(object, &trans);
	  break;
	case TYPE_TEXT:
	  obj_transform_text(object, &trans);
	  break;

	}

      object->selected=OBJECT_NOT_SEL;
      redraw_object(doc,object);
    }
}

void tool_set_prompt_stretch(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Base Point: "));
      break;
    case 1: 
      set_prompt_text( _("Second point of displacement: "));
      break;
    }

}

void tool_button_stretch(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }

  switch(gl_stage)
    {

    case 0:
      if(doc->gl_snap)
	make_snap(doc,&x,&y);
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      remove_old();
      make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
      trans.type=OBJ_TRANSLATE;
      trans.trans_x=(x-gl_oldmx);
      trans.trans_y=(y-gl_oldmy);
      foreach_object(doc,stretch_objects);
      deselect_all_objects(doc);
      doc->saved=FALSE;
      make_window_title(doc);
      tool_idle(doc);
      break;
    }
}

void tool_prompt_stretch(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gint mn;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
                          FALSE,NULL,0,0,&tx,&ty,NULL,&mn)){
      case PROMPT_RETURN:
        remove_old();
        tool_idle(doc);
        break;
      case PROMPT_POINT2:
        work_document=doc;
	remove_old();
        tool_button_stretch(doc,1,tx,ty);
        break;
      case PROMPT_ERROR:
      default:
        return;
      }

      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_RETURN|PROMPT_POINT2,buffer,
                          TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,&mn)){
      case PROMPT_RETURN:
        remove_old();
        tool_idle(doc);
        break;
      case PROMPT_POINT2:
	work_document=doc;
        remove_old();
        tool_button_stretch(doc,1,tx,ty);
        break;
      case PROMPT_ERROR:
      default:
        return;
      }

      break;
    } 
}

void tool_set_tool_stretch(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("stretch"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_STRETCH;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select object(s) to stretch: "), _("Selection - Stretch"));
  set_prompt();

  work_document=NULL;
}

static void xor_objects(DOCUMENT *doc, LAYER *layer, OBJECT *object){
  if(object->selected==OBJECT_SEL)
    {
      switch(object->type)
	{
	case TYPE_LINE:
	  obj_draw_line_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);
	  break;
	case TYPE_RLINE:
	  obj_draw_rline_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);
	  break;
	case TYPE_CIRCLE:
	  obj_draw_circle_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);
	  break;
	case TYPE_ELLIPSE:
	  obj_draw_ellipse_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);
	  break;
	case TYPE_TEXT:
	  obj_draw_text_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);
	  break;
	case TYPE_POINT:
	  obj_draw_point_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);	  
	  break;
	case TYPE_POLYLINE:
	  obj_draw_polyline_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);	  
	  break;
	case TYPE_ARC:
	  /*	  if(object->data.arc.selected_startpoint && object->data.arc.selected_endpoint)
	    obj_draw_arc_block(doc,misc_setup_gc_object_xor(doc, object),object,&trans);	  
	  else if(object->data.arc.selected_startpoint && !object->data.arc.selected_endpoint)
	    {
	      gdouble sx,sy,ex,ey;
	      OBJECT_ARC arc;

	      sx=object->data.arc.centrum.x+object->data.arc.radius*cos(object->data.arc.start_angle*M_PI/180.0);
	      sy=object->data.arc.centrum.y-object->data.arc.radius*sin(object->data.arc.start_angle*M_PI/180.0);
	      ex=object->data.arc.centrum.x+object->data.arc.radius*cos(object->data.arc.end_angle*M_PI/180.0);
	      ey=object->data.arc.centrum.y-object->data.arc.radius*sin(object->data.arc.end_angle*M_PI/180.0);

	      if(calc_arc_2p(object->data.arc.start_angle*M_PI/180.0, sx+toff_x, sy+toff_y,ex,ey,NULL,NULL, &arc))
		{
		  draw_arc(doc->widgets.draw->window, 
			   misc_setup_gc_object_xor(doc, object),FALSE,
			   WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
			   WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
			   arc.end_angle);
		}	   
	    }
	  else if(!object->data.arc.selected_startpoint && object->data.arc.selected_endpoint)
	    {
	      gdouble sx,sy,ex,ey;
	      OBJECT_ARC arc;

	      sx=object->data.arc.centrum.x+object->data.arc.radius*cos(object->data.arc.start_angle*M_PI/180.0);
	      sy=object->data.arc.centrum.y-object->data.arc.radius*sin(object->data.arc.start_angle*M_PI/180.0);
	      ex=object->data.arc.centrum.x+object->data.arc.radius*cos(object->data.arc.end_angle*M_PI/180.0);
	      ey=object->data.arc.centrum.y-object->data.arc.radius*sin(object->data.arc.end_angle*M_PI/180.0);

	      if(calc_arc_CSE(object->data.arc.centrum.x+toff_x/2,object->data.arc.centrum.y+toff_y/2,
			      sx,sy,ex+toff_x,ey+toff_y,&arc))		
		{
		  draw_arc(doc->widgets.draw->window, 
			   misc_setup_gc_object_xor(doc, object),FALSE,
			   WLDtoSCR_x(doc, arc.centrum.x-arc.radius),WLDtoSCR_y(doc,arc.centrum.y-arc.radius),
			   WLDtoSCR(doc,arc.radius*2),WLDtoSCR(doc,arc.radius*2),arc.start_angle, 
			   arc.end_angle);
		}	   	      
		} */	    
	  break;
	}
    }
}


void tool_xor_stretch(DOCUMENT *doc,gdouble x, gdouble y)
{
  misc_xor_line(doc,x,y);

  trans.type=OBJ_TRANSLATE;
  trans.trans_x=(x-gl_oldmx);
  trans.trans_y=(y-gl_oldmy);

  foreach_object(doc,xor_objects);
}


















