/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "misc.h"
#include "dia_coords.h"
#include "dia_comm.h"
#include "dia_message.h"
#include "xor.h"
#include "select.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "redraw.h"
#include "parse.h"
#include "calc.h"
#include "intersect.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static POINT inter_points[65];
static gint inter_num;

static gdouble int_x1, int_y1, int_dist1;
static gboolean int_found1=FALSE;
static gdouble int_x2, int_y2, int_dist2;
static gboolean int_found2=FALSE;
static gdouble Ax, Ay, Bx, By;

/* --  F U N C T I O N S  -- */

/* --- Object to trim LINE --- */
void calc_dist_of_line(gdouble x, gdouble y, gdouble Px, gdouble Py)
{
  gdouble Pdist, Tdist;

  Pdist=sqrt((Ax-Px)*(Ax-Px)+(Ay-Py)*(Ay-Py));
  Tdist=sqrt((Ax-x)*(Ax-x)+(Ay-y)*(Ay-y));
  
  if(Tdist<=Pdist){
    if(int_found1){
      if(Pdist<int_dist1){
	int_x1=Px;
	int_y1=Py;
	int_dist1=Pdist;
      }      
    }else{
      int_x1=Px;
      int_y1=Py;
      int_found1=TRUE;
      int_dist1=Pdist;
    }      
  }else{
    Pdist=sqrt((Bx-Px)*(Bx-Px)+(By-Py)*(By-Py));
    if(int_found2){
      if(Pdist<int_dist2){
	int_x2=Px;
	int_y2=Py;
	int_dist2=Pdist;
      }
      
    }else{
      int_x2=Px;
      int_y2=Py;
      int_found2=TRUE;
      int_dist2=Pdist;
    }
  }
}

void trim_line(DOCUMENT *doc, OBJECT *object, gdouble x, gdouble y)
{
  GList *layer_list;
  GList *cutt_list;
  OBJECT *cutt_edge;
  gdouble r, L;
  gint i;

  /* Init global variables */
  int_found1=FALSE;
  int_found2=FALSE;

  /* Extract data from line */
  Ax=object->data.line.p1.x;
  Ay=object->data.line.p1.y;
  Bx=object->data.line.p2.x;
  By=object->data.line.p2.y;

  /* Project down the userdefined point to a point that lies on the line */
  L=sqrt( (Bx-Ax)*(Bx-Ax)+(By-Ay)*(By-Ay));
  r=((Ay-y)*(Ay-By)-(Ax-x)*(Bx-Ax))/(L*L);
  x=Ax+r*(Bx-Ax);
  y=Ay+r*(By-Ay);
  
  layer_list=doc->layer_list;

  while(layer_list!=NULL){
    cutt_list=((LAYER *)layer_list->data)->object_list;
    while(cutt_list!=NULL){
      cutt_edge=(OBJECT *)cutt_list->data;

      if(cutt_edge->selected==OBJECT_SEL){
	switch(cutt_edge->type)
	  {
	  case TYPE_LINE:          /* Line vs line */
	    intersect_line_vs_line(object, cutt_edge, &inter_num, inter_points);
	    break;
	  case TYPE_CIRCLE:         /* Line vs Circle */
	    intersect_line_vs_circle(object, cutt_edge, &inter_num, inter_points);
	    break;
	  case TYPE_ARC:            /* Line vs arc */
	    intersect_line_vs_arc(object, cutt_edge, &inter_num, inter_points);
	    break;
	  case TYPE_POLYLINE:            /* Line vs polyline */
	    intersect_line_vs_pline(object, cutt_edge, &inter_num, inter_points);
	    break;
	  }
      }
      cutt_list=g_list_next(cutt_list);
    }
    layer_list=g_list_next(layer_list);
  }
  
  /* Calc the two closest intersections */
  for(i=0;i<inter_num;i++)
    calc_dist_of_line(x,y,inter_points[i].x, inter_points[i].y);

  /* Calc new objects based on the two closest intersections */
  object->selected=OBJECT_ERASE;
  redraw_object(doc, object);

  if(int_found1 && int_found2){
    object->data.line.p1.x=int_x1;
    object->data.line.p1.y=int_y1;
    redraw_object(doc, obj_add_line(doc,object->layer, object->name, object->type_colour, object->colour, 
				    object->type_linestyle, object->linestyle,
				    object->type_linewidth, object->linewidth, Ax, Ay ,int_x2, int_y2));
  }
  else if(int_found1 && !int_found2){
    object->data.line.p1.x=int_x1;
    object->data.line.p1.y=int_y1;
  }
  else if(int_found2 && !int_found1){
    object->data.line.p2.x=int_x2;
    object->data.line.p2.y=int_y2;
  }
  else
    print( _("\n  No intersection found."));

  obj_bound_line(object);
  object->selected=OBJECT_NOT_SEL;
  redraw_object(doc, object);
}

/* --- Object to trim CIRCLE --- */
void calc_dist_of_circle(gdouble x, gdouble y, gdouble Px, gdouble Py)
{
  gdouble pangle, angle;
  gdouble Pdist, Tdist;

  angle=get_angle(Ax,Ay,x,y);
  pangle=get_angle(Ax,Ay,Px,Py);

  Pdist=dist_arc(angle,pangle,Bx);
  Tdist=M_PI*Bx*Bx-Pdist;

  if(int_found1==FALSE && int_found2==FALSE)
    {
      int_found1=TRUE;
      int_dist1=Pdist;
      int_x1=Px;
      int_y1=Py;
      int_found2=TRUE;
      int_dist2=Tdist;
      int_x2=Px;
      int_y2=Py;
    }
  else
    {
      if(Pdist<int_dist1)
	{
	  int_dist1=Pdist;
	  int_x1=Px;
	  int_y1=Py;
	}

      if(Tdist<int_dist2)
	{
	  int_dist2=Tdist;
	  int_x2=Px;
	  int_y2=Py;
	}

    }
}

void trim_circle(DOCUMENT *doc, OBJECT *object, gdouble x, gdouble y)
{
  GList *layer_list;
  GList *cutt_list;
  OBJECT *cutt_edge;
  gint i;

  /* Init global variables */
  int_found1=FALSE;
  int_found2=FALSE;

  /* Extract data from circle */
  Ax=object->data.circle.centrum.x;
  Ay=object->data.circle.centrum.y;
  Bx=object->data.circle.radius;

  layer_list=doc->layer_list;

  while(layer_list!=NULL){
    cutt_list=((LAYER *)layer_list->data)->object_list;

    while(cutt_list!=NULL){
      cutt_edge=(OBJECT *)cutt_list->data;

      if(cutt_edge->selected==OBJECT_SEL){
	switch(cutt_edge->type)
	  {
	  case TYPE_LINE:          /* Circle vs line */
	    intersect_line_vs_circle(cutt_edge,object, &inter_num, inter_points);
	    break;
	  case TYPE_CIRCLE:         /* Circle vs Circle */
	    intersect_circle_vs_circle(object, cutt_edge, &inter_num, inter_points);
	    break;
	  case TYPE_ARC:            /* Circle vs arc */
	    intersect_circle_vs_arc(object, cutt_edge, &inter_num, inter_points);
	    break; 
	  }	
      }
      cutt_list=g_list_next(cutt_list);
    }
    layer_list=g_list_next(layer_list);
  }

  /* Calc the two closest intersections */
  for(i=0;i<inter_num;i++)
    calc_dist_of_circle(x,y,inter_points[i].x, inter_points[i].y);

  /* Calc new objects based on the two closest intersections */
  object->selected=OBJECT_ERASE;
  redraw_object(doc, object);

  if(inter_num==0)
    print( _("\n  No intersection found."));      
  else if(inter_num<2)
    print( _("\n  Circle needs at least two intersections."));
  else
    {
      object->type=TYPE_ARC;
      object->data.arc.centrum.x=Ax;
      object->data.arc.centrum.y=Ay;
      object->data.arc.radius=Bx;      
      object->data.arc.start_angle=get_angle(Ax,Ay, int_x1, int_y1);
      object->data.arc.end_angle=get_angle(Ax,Ay, int_x2, int_y2);

    }

  object->selected=OBJECT_NOT_SEL;
  obj_bound_arc(object);    
  redraw_object(doc, object);
}

/* --- Object to trim ARC --- */
void calc_dist_of_arc(gdouble radius, gdouble x, gdouble y, gdouble Px, gdouble Py)
{
  gdouble Pdist, Tdist;

  Pdist=dist_arc(Bx,get_angle(Ax,Ay,Px,Py),radius);
  Tdist=dist_arc(get_angle(Ax,Ay,x,y),get_angle(Ax,Ay,Px,Py),radius);
  if(Tdist<=Pdist){
    if(int_found1){
      if(Pdist<int_dist1){
	int_x1=Px;
	int_y1=Py;
	int_dist1=Pdist;
      }      
    }else{
      int_x1=Px;
      int_y1=Py;
      int_found1=TRUE;
      int_dist1=Pdist;
    }      
  }else{
    Pdist=dist_arc(get_angle(Ax,Ay,Px,Py),By,radius);
    if(int_found2){
      if(Pdist<int_dist2){
	int_x2=Px;
	int_y2=Py;
	int_dist2=Pdist;
      }
      
    }else{
      int_x2=Px;
      int_y2=Py;
      int_found2=TRUE;
      int_dist2=Pdist;
    }
  }
}

void trim_arc(DOCUMENT *doc, OBJECT *object, gdouble x, gdouble y)
{
  GList *layer_list;
  GList *cutt_list;
  OBJECT *cutt_edge;
  gdouble r, L;
  gint i;

  /* Init global variables */
  int_found1=FALSE;
  int_found2=FALSE;

  /* Extract data from line */
  Ax=object->data.arc.centrum.x;
  Ay=object->data.arc.centrum.y;
  Bx=object->data.arc.start_angle;
  By=object->data.arc.end_angle;

  L=object->data.arc.radius;      

  r=180-get_angle(x,y,Ax,Ay);
  x=L*cos(r*M_PI/180)+Ax;
  y=L*sin(r*M_PI/180)+Ay;
  
  layer_list=doc->layer_list;

  while(layer_list!=NULL){
    cutt_list=((LAYER *)layer_list->data)->object_list;
    while(cutt_list!=NULL){
      cutt_edge=(OBJECT *)cutt_list->data;

      if(cutt_edge->selected==OBJECT_SEL){
	switch(cutt_edge->type)
	  {
	  case TYPE_LINE:          /* Arc vs line */
	    intersect_line_vs_arc( cutt_edge,object, &inter_num, inter_points);
	    break;
	  case TYPE_CIRCLE:         /* Arc vs Circle */
	    intersect_circle_vs_arc(cutt_edge,object, &inter_num, inter_points);
	    break;
	  case TYPE_ARC:            /* Arc vs arc */
	    intersect_arc_vs_arc(object, cutt_edge, &inter_num, inter_points);
	    break;
	  }	
      }
      cutt_list=cutt_list->next;
    }
    layer_list=layer_list->next;
  }

  /* Calc the two closest intersections */
  for(i=0;i<inter_num;i++)
    calc_dist_of_arc(L,x,y,inter_points[i].x, inter_points[i].y);

  /* Calc new objects based on the two closest intersections */
  object->selected=OBJECT_ERASE;
  redraw_object(doc,object);
  
  if(int_found1 && int_found2){
    redraw_object(doc, obj_add_arc(doc,object->layer, object->name, object->type_colour, object->colour, 
				   object->type_linestyle, object->linestyle,
				   object->type_linewidth, object->linewidth,
				   Ax, Ay ,L, object->data.arc.start_angle,  get_angle(Ax,Ay,int_x2,int_y2)));
    object->data.arc.start_angle=get_angle(Ax,Ay,int_x1,int_y1);
  }
  else if(int_found1 && !int_found2)
    object->data.arc.start_angle=get_angle(Ax,Ay,int_x1,int_y1);
  else if(int_found2 && !int_found1)
    object->data.arc.end_angle=get_angle(Ax,Ay,int_x2,int_y2);
  else
    print( _("\n  No intersection found."));

  object->selected=OBJECT_NOT_SEL;
  obj_bound_arc(object);
  redraw_object(doc,object);    
}


void calculate(DOCUMENT *doc, OBJECT *object, gdouble x, gdouble y)
{
  if(object==NULL){
      print( _("\n  No object found."));
      return;
  }

  if(object->selected==OBJECT_SEL){
    print( _("\n Cannot trim a cutting edge."));
    return;
  }

  inter_num=0;

  switch(object->type)
    {
    case TYPE_LINE:
      trim_line(doc, object, x,y);
      break;
    case TYPE_CIRCLE:
      trim_circle(doc,object,x,y);
      break;
    case TYPE_ARC:
      trim_arc(doc,object,x,y);
      break;
    default:
      print( _("\n  Not yet supported!"));
      break;
    }
}

void tool_set_prompt_trim(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("Select object to trim: "));
      break;
    }

}

void tool_set_tool_trim(GtkWidget *widget, gpointer data)
{
  remove_old();
 
  if(!command_from_keyboard)
    print( _("trim"));

  if(work_document!=NULL)
    deselect_all_objects(work_document);
  
  if(misc_tool_get_document()==NULL)
    return;

  gl_tool=TOOL_TRIM;
  gl_stage=0;
  work_document=NULL;

  UpdateCoordsLabel();

  spec_set_select( _("Select cutting edge(s): "), _("Selection - Trim"));
  set_prompt();

  work_document=NULL;
}

void tool_button_trim(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      deselect_all_objects(doc);
      tool_idle(doc);
      return;
    }
  
  switch(gl_stage)
    {
    case 0:
      calculate(doc,select_singel_object_not_sel(doc,x,y),x,y);
      doc->saved=FALSE;
      make_window_title(doc);
      set_prompt();
      break;
    }
}

void tool_prompt_trim(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx,ty;

  switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,buffer,
		      FALSE,NULL,0,0,&tx,&ty,NULL,NULL)){
  case PROMPT_POINT2:
    work_document=doc;
    remove_old();
    tool_button_trim(doc,1,tx,ty);
    break;
  }
}









