/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include <math.h>
#include "struct.h"
#include "global.h"
#include "redraw.h"
#include "calc.h"
#include "dia_comm.h"
#include "dia_coords.h"
#include "dia_message.h"
#include "xor.h"
#include "misc.h"
#include "parse.h"
#include "translation.h"


/* --  G L O B A L   V A R I A B L E S  -- */

/* --  L O C A L   V A R I A B L E S  -- */
static gint o_gl_tool, o_gl_stage, o_gl_osnap;
static gdouble o_gl_oldmx, o_gl_oldmy;
static gdouble bound_area[4];
static bound_first;

static gdouble pan_sx,pan_sy;

/* --  S T R U C T U R E  -- */
typedef struct s_zoom_previous{
  gdouble xpan,ypan;
  gdouble zoom;
} ZOOM_PREVIOUS;

/* --  F U N C T I O N S  -- */
static void save_zoom(DOCUMENT *doc)
{
  ZOOM_PREVIOUS *temp;

  temp=(ZOOM_PREVIOUS *)g_malloc(sizeof(ZOOM_PREVIOUS));
  temp->xpan=doc->xpan;
  temp->ypan=doc->ypan;
  temp->zoom=doc->zoom;
  doc->zoom_previous=g_list_append(doc->zoom_previous, (gpointer)temp);
}

static void bound_func(DOCUMENT *doc, LAYER *layer, OBJECT *object)
{
  if(object->type==TYPE_RLINE || object->type==TYPE_CLINE)
    return;

  if(bound_first)
    {
      bound_area[0]=MIN(bound_area[0], object->bounding_box[0]);
      bound_area[1]=MIN(bound_area[1], object->bounding_box[1]);
      bound_area[2]=MAX(bound_area[2], object->bounding_box[2]);
      bound_area[3]=MAX(bound_area[3], object->bounding_box[3]);
    }
  else
    {
      bound_area[0]=object->bounding_box[0];
      bound_area[1]=object->bounding_box[1];
      bound_area[2]=object->bounding_box[2];
      bound_area[3]=object->bounding_box[3];
      bound_first=TRUE;
    }
}

void tool_set_previous_zoom(void)
{
  gl_tool=o_gl_tool;
  gl_stage=o_gl_stage;
  gl_oldmx=o_gl_oldmx;
  gl_oldmy=o_gl_oldmy;
  gl_osnap=o_gl_osnap;
  UpdateCoordsLabel();
  set_prompt();
}

void tool_set_prompt_zoom(void)
{
  switch(gl_stage)
    {
    case 0: 
      set_prompt_text( _("preVious/All/Pan/Relative/<First point>: "));
      break;
    case 1: 
      set_prompt_text( _("Second point: "));
      break;
    case 2:
      set_prompt_text( _("Displacement: "));
      break;
    case 3: 
      set_prompt_text( _("Second Point: "));
      break;
    case 4: 
      set_prompt_text( _("Scale relative: "));
      break;
    }
}

static void zoom_box(DOCUMENT *doc, gdouble x1, gdouble y1, gdouble x2, gdouble y2)
{
  gdouble wh,ww;
  gdouble dw, dh;
  gint nx, ny;
  gdouble sx,sy;
  gdouble scale;

  remove_old();

  /* Save old zoom */
  save_zoom(doc);
  
  /* Calculate new zoom */

  /* Get current window size */
  ww=WLDtoSCR(doc, doc->widgets.draw->allocation.width);
  wh=WLDtoSCR(doc, doc->widgets.draw->allocation.height);
  
  /* Get new window size */
  dw=ABS(x1-x2);
  dh=ABS(y1-y2);

  doc->xpan=-MIN(x1, x2);
  doc->ypan=-MIN(y1, y2);
  

  sx=(SCRtoWLD_x(doc, doc->widgets.draw->allocation.width) - SCRtoWLD_x(doc, 0)) /
    (MAX(x1, x2) - MIN(x1, x2));

  sy=(SCRtoWLD_y(doc, doc->widgets.draw->allocation.height) - SCRtoWLD_y(doc, 0)) /
    (MAX(y1, y2) - MIN(y1, y2));

  if(sx<sy)
    scale=sx*0.95;
  else
    scale=sy*0.95;

  // g_warning("Scale %f", scale);
  //  nx=MIN( x1,x2 )-SCRtoWLD(doc,5);
  //ny=MIN( y1,y2 )-SCRtoWLD(doc,5);

  //if(WLDtoSCR(doc,tw)<1 || WLDtoSCR(doc,th)<1)  /* If this is a too small box */
  //  scale=2;
  //else
  //  {
  //   tw=((gdouble)doc->widgets.draw->allocation.width-1-10)/WLDtoSCR(doc, tw);
  //   th=((gdouble)doc->widgets.draw->allocation.height-1-10)/WLDtoSCR(doc, th);

  /*      if(tw<th)
	scale=tw;
      else
	scale=th;

    }
  */
    doc->zoom*=scale;

  /* doc->xpan=-(nx);
  doc->ypan=-(ny); */
}

void tool_pan(DOCUMENT *doc, gdouble dx, gdouble dy)
{
  remove_old();
  save_zoom(doc);
  doc->xpan-=dx;
  doc->ypan-=dy;
  
  update_rulers(doc);
  redraw_window(doc);
  set_cursor(doc);
}

void tool_zoom_previous(DOCUMENT *doc)
{
  GList *temp;

  if(doc==NULL)
    {
      doc=misc_tool_get_document();
      if(doc==NULL)
	return;
    }

  temp=g_list_last(doc->zoom_previous);
  if(temp!=NULL)
    {
      doc->xpan=((ZOOM_PREVIOUS *)temp->data)->xpan;
      doc->ypan=((ZOOM_PREVIOUS *)temp->data)->ypan;
      doc->zoom=((ZOOM_PREVIOUS *)temp->data)->zoom;
      redraw_window(doc);
      /* Remove old zoom */
      doc->zoom_previous=g_list_remove(doc->zoom_previous,
				       temp->data);
      update_rulers(doc);
      redraw_window(doc);
    }
  else
    message(ICON_INFO, _("No previous zooms for this drawing."));
 
}

void tool_zoom_all(DOCUMENT *doc)
{

  if(doc==NULL)
    {
      doc=misc_tool_get_document();
      if(doc==NULL)
	return;
    }

  bound_first=FALSE;
  /* Create drawing box */
  foreach_object(doc, bound_func);

  if(bound_first)
    {
      zoom_box(doc, bound_area[0],bound_area[1],bound_area[2],bound_area[3]);
      update_rulers(doc);
      redraw_window(doc);
    }
  else
    message(ICON_INFO, _("Cannot zoom since there\nare no valid objects in the drawing."));
}


void tool_zoom_relative(GtkWidget *widget, gint data)
{
  gdouble d[8]={16.0/1.0, 8.0/1.0, 4.0/1.0, 2.0/1.0, 1.0/2.0, 1.0/4.0, 1.0/8.0, 1.0/16.0};
  DOCUMENT *doc;

  doc=misc_tool_get_document();
  if(doc==NULL)
    return;

  save_zoom(doc);

  doc->zoom*=d[data];
  update_rulers(doc);
  redraw_window(doc);
}

void tool_button_zoom(DOCUMENT *doc, gint button, gdouble x, gdouble y)
{
  /* Right mousebutton => cancel/break */
  if(button==3)
    {
      remove_old();
      print( _("*Cancel*"));
      work_document=NULL;
      tool_set_previous_zoom();
      set_cursor(doc);
      return;
    }

  switch(gl_stage)
    {
    case 0:
      gl_oldmx=x;
      gl_oldmy=y;
      gl_stage=1;
      set_prompt();
      break;
    case 1:
      zoom_box(doc, x,y,gl_oldmx, gl_oldmy);
      tool_set_previous_zoom();
      update_rulers(doc);
      redraw_window(doc);
      set_cursor(doc);
      break;
    case 2:
      work_document=doc;
      gl_oldmx=pan_sx=x;
      gl_oldmy=pan_sy=y;
      gl_stage=3;
      set_prompt();
      break;
    case 3:
      tool_set_previous_zoom();
      tool_pan(doc,(pan_sx-x), (pan_sy-y));
      break;
  }
  
}

void tool_prompt_zoom(DOCUMENT *doc, gchar *buffer)
{
  gdouble tx, ty;
  gint mn;

  switch(gl_stage)
    {
    case 0:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2|PROMPT_MENU,
			  buffer,FALSE, _("V/A/P/R/"),0,0,&tx,&ty,NULL,&mn))
	{
	case PROMPT_POINT2:
	  work_document=doc;
	  tool_button_zoom(doc,1,tx,ty);
	  break;
	case PROMPT_MENU:
	  switch(mn)
	    {
	    case 0: /* Previous */
	      tool_zoom_previous(doc);
	      remove_old();
	      tool_set_previous_zoom();
	      set_cursor(doc);
	      break;
	    case 1: /* Fit */
	      tool_zoom_all(doc);
	      remove_old();
	      tool_set_previous_zoom();
	      set_cursor(doc);
	      break;
	    case 2: /* Pan */
	      gl_stage=2;	    
	      set_cursor(doc);
	      set_prompt();
	      break;
	    case 3: /* Relative */
	      gl_stage=4;	    
	      set_cursor(doc);
	      set_prompt();
	      break;
	    }
	  UpdateCoordsLabel();
	  break;
	}
      break;
    case 1:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,
			  buffer,TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL))
	{
	case PROMPT_POINT2:
      	  work_document=doc;
	  tool_button_zoom(doc,1,tx,ty);
	  break;
	}
      break;
    case 2:
       switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,
			   buffer,FALSE,NULL,0,0,&tx,&ty,NULL,NULL))
	 {
	 case PROMPT_POINT2:
      	  work_document=doc;
	   remove_old();
	   tool_button_zoom(doc,1,tx,ty);
	   break;
	 }
       break;
    case 3:
      switch(parse_prompt(PROMPT_OSNAP|PROMPT_POINT2,
			  buffer,TRUE,NULL,gl_oldmx,gl_oldmy,&tx,&ty,NULL,NULL))
	{
	case PROMPT_POINT2:
	  work_document=doc;
	  remove_old();
	  tool_button_zoom(doc,1,tx,ty);
	  break;
	}
      break;
    case 4:
      switch(parse_prompt(PROMPT_POINT1,
			  buffer,FALSE,NULL,0,0,NULL,NULL,&tx,NULL))
	{
	case PROMPT_POINT1:
	  save_zoom(doc);
	  
	  doc->zoom*=tx;
	  update_rulers(doc);
	  redraw_window(doc);
	  tool_set_previous_zoom();
	  set_cursor(doc);
	  break;
	}
      break;
    }
}

void tool_set_tool_zoom(GtkWidget *widget, gpointer data)
{
  DOCUMENT *doc;

  if(gl_tool!=TOOL_IDLE)
    {
      set_prompt();

      if(GPOINTER_TO_INT(data)==2)
	print( _("_pan"));
      else
	print( _("_zoom"));
    }
  else if(!command_from_keyboard)
    {
      if(GPOINTER_TO_INT(data)==2)
	print( _("pan"));
      else
	print( _("zoom"));
    }

  remove_old();

  doc=misc_tool_get_document();
  if(doc==NULL)
    return;
  
  if(gl_tool!=TOOL_ZOOM)
    {
      o_gl_tool=gl_tool;
      o_gl_stage=gl_stage;
      o_gl_oldmx=gl_oldmx;
      o_gl_oldmy=gl_oldmy;
      o_gl_osnap=gl_osnap;
    }


  gl_tool=TOOL_ZOOM;
  gl_stage=GPOINTER_TO_INT(data);
  gl_osnap=OSNAP_NONE;

  set_prompt();
  UpdateCoordsLabel();

}


void tool_xor_zoom(DOCUMENT *doc, gdouble x, gdouble y)
{
  gdouble tw,th;
  
  tw=ABS(x-gl_oldmx);
  th=ABS(y-gl_oldmy);
  
  misc_setup_black_line(doc);

  gdk_draw_rectangle(doc->widgets.draw->window, 
                     doc->xor_gc,FALSE,
                     WLDtoSCR_x(doc,MIN(x,gl_oldmx)),
		     WLDtoSCR_y(doc,MIN(y,gl_oldmy)),
                     WLDtoSCR(doc,tw),WLDtoSCR(doc,th));
}



