/* oCADis
 * Copyright (C) 2001 Jonas Jarvoll
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <gtk/gtk.h>
#include "struct.h"
#include "struct/grdraw.h"
#include "global.h"
#include "misc.h"
#include "calc.h"
#include "drag.h"
#include "obj_line.h"
#include "obj_circle.h"
#include "obj_arc.h"
#include "obj_attribute.h"
#include "obj_polyline.h"
#include "obj_rline.h"
#include "obj_cline.h"
#include "obj_text.h"
#include "obj_ellipse.h"
#include "obj_elarc.h"
#include "obj_rectangle.h"
#include "obj_polygon.h"
#include "obj_point.h"
#include "obj_spline.h"
#include "obj_insert.h"
#include "dim_horiz.h"
#include "dim_vert.h"
#include "dim_aligned.h"
#include "dim_radius.h"
#include "dim_diameter.h"
#include "dim_continue.h"
#include "dim_baseline.h"
#include "dim_leader.h"
#include "dim_angular.h"
#include "dimension.h"
#include "tool_zoom.h"
#include "tool_copy.h"
#include "tool_offset.h"
#include "tool_move.h"
#include "tool_mirror.h"
#include "tool_rotate.h"
#include "tool_stretch.h"
#include "tool_scale.h"
#include "dia_coords.h"

/* --  G L O B A L   V A R I A B L E S  -- */
gint draw_old=FALSE;

/* --  L O C A L   V A R I A B L E S  -- */
static gdouble ox, oy; 
static gint o_gl_tool;
static gint o_gl_stage;
static DOCUMENT *old_src_document;
static DOCUMENT *old_dest_document;

/* --  F U N C T I O N S  -- */
static void save_value(DOCUMENT *src_doc, DOCUMENT *dest_doc, gdouble x, gdouble y)
{
  ox=x;
  oy=y;
  draw_old=TRUE;
  o_gl_tool=gl_tool;
  o_gl_stage=gl_stage;
  old_src_document=src_doc;
  old_dest_document=dest_doc;
}

void remove_old(void)
{
  /* If we havnt drawn anything to the screen don't do an remove_old */
  if(!draw_old)
    return;

  if(old_src_document!=old_dest_document)
    {
      switch(o_gl_tool)
	{
	case TOOL_MOVE:
	  if(o_gl_stage==1)
	    tool_xor_move_drag(old_src_document, old_dest_document, ox, oy);
	  break;
	case TOOL_COPY:
	  if(o_gl_stage==1 || o_gl_stage==3)
	    tool_xor_move_drag(old_src_document, old_dest_document, ox, oy);
	  break;
	}
    }
  else
    switch(o_gl_tool){
    case TOOL_DIST:
      if(o_gl_stage==1)
	misc_xor_line(old_src_document,ox,oy);
      break;
    case TOOL_HORIZ:
      if(o_gl_stage==1)
	dim_xor_horiz_1(old_src_document,ox,oy);
      if(o_gl_stage==2)
	dim_xor_horiz_2(old_src_document,ox,oy);
      break;
    case TOOL_VERT:
      if(o_gl_stage==1)
	dim_xor_vert_1(old_src_document,ox,oy);
      if(o_gl_stage==2)
	dim_xor_vert_2(old_src_document,ox,oy);
      break;
    case TOOL_ALIGNED:
      if(o_gl_stage==1)
	dim_xor_aligned_1(old_src_document,ox,oy);
      if(o_gl_stage==2)
	dim_xor_aligned_2(old_src_document,ox,oy);
      break;
    case TOOL_RADIUS:
      if(o_gl_stage==1)
	dim_xor_radius(old_src_document,ox,oy);
      break;
    case TOOL_DIAMETER:
      if(o_gl_stage==1)
	dim_xor_diameter(old_src_document,ox,oy);
      break;
    case TOOL_CONTINUE:
      if(o_gl_stage==1)
	dim_xor_continue(old_src_document,ox,oy);
      break;
    case TOOL_BASELINE:
      if(o_gl_stage==1)
	dim_xor_baseline(old_src_document,ox,oy);
      break;
    case TOOL_LEADER:
      if(o_gl_stage==1)
	dim_xor_leader(old_src_document,ox,oy);
      break;
    case TOOL_ANGULAR:
      if(o_gl_stage==1 || o_gl_stage==4)
	dim_xor_angular(old_src_document,ox,oy);
      if(o_gl_stage==5)
	dim_xor_angular_first_line(old_src_document,ox,oy);
      if(o_gl_stage==6)
	dim_xor_angular_second_line(old_src_document,ox,oy);
      break;
    case TOOL_LINE:
      if(o_gl_stage==1)
	obj_xor_line(old_src_document,ox,oy);
      break;
    case TOOL_SPLINE:
      if(o_gl_stage==1)
	misc_xor_line(old_src_document,ox,oy);
      if(o_gl_stage>1)
	obj_xor_spline(old_src_document,ox,oy);
      break;
    case TOOL_RLINE:
      if(o_gl_stage==1)
	obj_xor_rline(old_src_document,ox,oy);
      break;
    case TOOL_ELLIPSE:
      if(o_gl_stage==1 || o_gl_stage==4)
	misc_xor_line(old_src_document,ox,oy);
      if(o_gl_stage==2 || o_gl_stage==5)
	obj_xor_ellipse(old_src_document,ox,oy);
      if(o_gl_stage==6)
	obj_xor_ellipse_rot(old_src_document,ox,oy);
      break;
    case TOOL_ELARC:
      if(o_gl_stage==1 || o_gl_stage==4)
	misc_xor_line(old_src_document,ox,oy);
      if(o_gl_stage==2 || o_gl_stage==5)
	obj_xor_ellipse(old_src_document,ox,oy);
      if(o_gl_stage==6)
	obj_xor_ellipse_rot(old_src_document,ox,oy);
      if(o_gl_stage==7)
	obj_xor_elarc_start_angle(old_src_document,ox,oy);
      if(o_gl_stage==8 || o_gl_stage==9)
	obj_xor_elarc_end_angle(old_src_document,ox,oy);
      break;
    case TOOL_TEXT:
      if(o_gl_stage==4)
	obj_xor_text(old_src_document,ox,oy);
      if(o_gl_stage==7 || o_gl_stage==9)
	misc_xor_line(old_src_document,ox,oy);
      break;
    case TOOL_ATTRIBUTE:
      if(o_gl_stage==4)
	obj_xor_attribute(old_src_document,ox,oy);
      if(o_gl_stage==7 || o_gl_stage==9)
	misc_xor_line(old_src_document,ox,oy);
      break;
    case TOOL_CLINE:
      if(o_gl_stage==1 || o_gl_stage==2 || o_gl_stage==3 || o_gl_stage==5)
	obj_xor_cline(old_src_document,ox,oy);
      break;
    case TOOL_POLYLINE:
      if(o_gl_stage==1)
	obj_xor_polyline(old_src_document,ox,oy);
      if(o_gl_stage==4 || gl_stage==13)
	obj_xor_polyline_arc(old_src_document,ox,oy);
      if(o_gl_stage==5)
	obj_xor_polyline_second_point(old_src_document,ox,oy);
      if(o_gl_stage==6)
	obj_xor_polyline_end_point(old_src_document,ox,oy);
      if(o_gl_stage==7)
	obj_xor_polyline_center_point(old_src_document,ox,oy);
      if(o_gl_stage==8)
	obj_xor_polyline_end_point_2(old_src_document,ox,oy);
      if(o_gl_stage==9)
	obj_xor_polyline_included_angle(old_src_document,ox,oy);
      if(o_gl_stage==10)
	obj_xor_polyline_length_of_coord(old_src_document,ox,oy);
      if(o_gl_stage==11)
	obj_xor_polyline_length_of_line(old_src_document,ox,oy);
      if(o_gl_stage==12)
	obj_xor_polyline_direction(old_src_document,ox,oy);
      break;
    case TOOL_RECTANGLE:
      if(o_gl_stage==1)
	obj_xor_rectangle(old_src_document,ox,oy);
      break;
    case TOOL_POLYGON:
      if(o_gl_stage==2)
	obj_xor_polygon(old_src_document,ox,oy);
      if(o_gl_stage==4)
	obj_xor_polygon_edge(old_src_document,ox,oy);
      break;
    case TOOL_INSERT:
      if( o_gl_stage==1 )
	obj_xor_insert_ins(old_src_document,ox,oy);
      if( o_gl_stage==2 )
	obj_xor_insert_scale_x(old_src_document,ox,oy);
      if( o_gl_stage==3 )
	obj_xor_insert_scale_y(old_src_document,ox,oy);
      if( o_gl_stage==4 )
	obj_xor_insert_rot(old_src_document,ox,oy);
      break;
    case TOOL_CIRCLE:
      if( (o_gl_stage==1) || (o_gl_stage==3))
	obj_xor_circle(old_src_document,ox,oy);
      if(o_gl_stage==5)
	misc_xor_line(old_src_document,ox,oy);
      if(o_gl_stage==6)
	obj_xor_circle_3P(old_src_document,ox,oy);
      if(o_gl_stage==9)
	obj_xor_circle_2P(old_src_document,ox,oy);
      if(o_gl_stage==16)
	obj_xor_circle_TTR(old_src_document,ox,oy);
      if(o_gl_stage==20)
	obj_xor_circle_TTT(old_src_document,ox,oy);
      break;
    case TOOL_ARC:
      if(o_gl_stage==1 || o_gl_stage==3 || o_gl_stage==4 || o_gl_stage==8 || o_gl_stage==12)
	misc_xor_line(old_src_document,ox,oy);
      if(o_gl_stage==2)
	obj_xor_arc_3p(old_src_document,ox,oy);
      if(o_gl_stage==5 || o_gl_stage==13)
	obj_xor_arc_CSE(old_src_document,ox,oy);
      if(o_gl_stage==7)
	obj_xor_arc_CSL(old_src_document,ox,oy);
      if(o_gl_stage==9)
	obj_xor_arc_SEC(old_src_document,ox,oy);
      if(o_gl_stage==6)
	obj_xor_arc_CSA(old_src_document,ox,oy);
      if(o_gl_stage==10)
	obj_xor_arc_SED(old_src_document,ox,oy);
      if(o_gl_stage==11)
	obj_xor_arc_SER(old_src_document,ox,oy);
      if(o_gl_stage==14)
	obj_xor_arc_SEA(old_src_document,ox,oy);
      if(o_gl_stage==15)
	obj_xor_arc_cont(old_src_document,ox,oy);
      break;
    case TOOL_ZOOM:
      if(o_gl_stage==1)
	tool_xor_zoom(old_src_document,ox,oy);
      if(o_gl_stage==3)
	misc_xor_line(old_src_document,ox,oy);
      break;
    case TOOL_COPY:
      if(o_gl_stage==1 || o_gl_stage==3)
	tool_xor_move(old_src_document,ox,oy);
      break;
    case TOOL_OFFSET:
      if(o_gl_stage==1)
	tool_xor_offset(old_src_document,ox,oy);
      break;
    case TOOL_ARRAY:
      if(o_gl_stage==4)
	tool_xor_zoom(old_src_document,ox,oy);
      break;
    case TOOL_MOVE:
      if(o_gl_stage==1)
	tool_xor_move(old_src_document,ox,oy);
      break;
    case TOOL_MIRROR:
      if(o_gl_stage==1)
	tool_xor_mirror(old_src_document,ox,oy);
      break;
    case TOOL_ROTATE:
      if(o_gl_stage==1)
	tool_xor_rotate(old_src_document,ox,oy);
      break;
    case TOOL_STRETCH:
      if(o_gl_stage==1)
	tool_xor_stretch(old_src_document,ox,oy);
      break;
    case TOOL_SCALE:
      if(o_gl_stage==1 || o_gl_stage==4)
	tool_xor_scale(old_src_document,ox,oy);
      if(o_gl_stage==3)
	misc_xor_line(old_src_document,ox,oy);
      break;
    case TOOL_SELECT:
      if(o_gl_stage==2 || o_gl_stage==4)
	tool_xor_zoom(old_src_document,ox,oy);
      break;
    }

  draw_old=FALSE;
}

static gboolean dragmouse(DOCUMENT *doc, gdouble x, gdouble y)
{
  remove_old();

  switch(gl_tool)
    {
    case TOOL_COPY:
    case TOOL_MOVE:
      make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
      tool_xor_move_drag(work_document, doc, x, y);
      save_value(work_document, doc, x, y);
      break;
    }

  return TRUE;
}

gboolean mousemove(DOCUMENT *doc, gdouble x, gdouble y)
{
  UpdateCoords(doc,x,y);
  
  if(( (gl_tool==TOOL_MOVE && gl_stage==1) || 
       (gl_tool==TOOL_COPY && (gl_stage==1 || gl_stage==3)))
     && doc!=work_document)
    return dragmouse(doc, x, y);
  else if(doc!=work_document)
    return FALSE;       
  
  remove_old();

  switch(gl_tool)
    {
    case TOOL_DIST:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc, x,y);
	}
      break;
    case TOOL_HORIZ:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_horiz_1(doc,x,y);
	  save_value(doc, doc, x,y);
	}
      else if(gl_stage==2)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  dim_xor_horiz_2(doc,x,y);
	  save_value(doc, doc, x, y);
	}
      break;
    case TOOL_VERT:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_vert_1(doc,x,y);
	  save_value(doc, doc, x, y);
	}
      else if(gl_stage==2)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  dim_xor_vert_2(doc,x,y);
	  save_value(doc, doc, x, y);
	}
      break;
    case TOOL_ALIGNED:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_aligned_1(doc,x,y);
	  save_value(doc, doc, x, y);
	}
      else if(gl_stage==2)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  dim_xor_aligned_2(doc,x,y);
	  save_value(doc, doc, x, y);
	}
      break;
    case TOOL_RADIUS:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_radius(doc,x,y);
	  save_value(doc, doc, x, y);
	}
      break;
    case TOOL_DIAMETER:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_diameter(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_CONTINUE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_continue(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_BASELINE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_baseline(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_LEADER:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_leader(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ANGULAR:
      if(gl_stage==1 || gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_angular(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==5)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_angular_first_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==6)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  dim_xor_angular_second_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_LINE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_SPLINE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      else if(gl_stage>1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_spline(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ELLIPSE:
      if(gl_stage==1 || gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==2 || gl_stage==5)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  obj_xor_ellipse(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==6)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  obj_xor_ellipse_rot(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ELARC:
      if(gl_stage==1 || gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==2 || gl_stage==5)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  obj_xor_ellipse(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==6)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  obj_xor_ellipse_rot(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==7)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  obj_xor_elarc_start_angle(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==8 || gl_stage==9)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  obj_xor_elarc_end_angle(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_RLINE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_rline(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_CLINE:
      if(gl_stage==1 || gl_stage==2 || gl_stage==3 || gl_stage==5) 
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_cline(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_POLYLINE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline(doc,x,y);
	  save_value(doc, doc,x,y);
      }
      if(gl_stage==4 || gl_stage==13)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_arc(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==5)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_second_point(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==6)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_end_point(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==7)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_center_point(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==8)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_end_point_2(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==9)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_included_angle(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==10)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_length_of_coord(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==11)
	{
	  if(doc->gl_snap)
	    make_snap(doc, &x, &y);

	  obj_xor_polyline_length_of_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
     if(gl_stage==12)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polyline_direction(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_TEXT:
      if(gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_text(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==7 || gl_stage==9)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ATTRIBUTE:
      if(gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_attribute(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==7 || gl_stage==9)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_RECTANGLE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_rectangle(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_POLYGON:
      if(gl_stage==2)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polygon(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_polygon_edge(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_CIRCLE:
      if( (gl_stage==1) || (gl_stage==3))
	{
	  if(doc->gl_snap)
	    make_snap(doc,&x,&y);
	  
	  obj_xor_circle(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==5)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==6)
	{
	  if(doc->gl_snap)
	    make_snap(doc,&x,&y);
	  
	  obj_xor_circle_3P(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==9)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_circle_2P(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==16)
	{
	  obj_xor_circle_TTR(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==20)
	{
	  obj_xor_circle_TTT(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ARC:
      if(gl_stage==1 || gl_stage==3 || gl_stage==4 || gl_stage==8 || gl_stage==12 )
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==2)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_3p(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==5 || gl_stage==13)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_CSE(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==7)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_CSL(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==9)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_SEC(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==6)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_CSA(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==10)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_SED(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==11)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_SER(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==14)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_SEA(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==15)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_arc_cont(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_INSERT:
      if(gl_stage==1)
	{
	  if(doc->gl_snap)
	    make_snap(doc,  &x, &y);
	  obj_xor_insert_ins(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==2)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_insert_scale_x(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==3)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_insert_scale_y(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  obj_xor_insert_rot(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ZOOM:
      if(gl_stage==1)
	{
	  tool_xor_zoom(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==3)
	{
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_COPY:
      if(gl_stage==1 || gl_stage==3)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  tool_xor_move(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_OFFSET:
      if(gl_stage==1)
	{
	  tool_xor_offset(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ARRAY:
      if(gl_stage==4)
	{
	  if(doc->gl_snap)
	    make_snap(doc,&x,&y);
	  tool_xor_zoom(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_MOVE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  tool_xor_move(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_STRETCH:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  tool_xor_stretch(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_SCALE:
      if(gl_stage==1 || gl_stage==4)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  tool_xor_scale(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      if(gl_stage==3)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  misc_xor_line(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_MIRROR:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  tool_xor_mirror(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_ROTATE:
      if(gl_stage==1)
	{
	  make_ortho_snap(doc, gl_oldmx, gl_oldmy, &x, &y);
	  tool_xor_rotate(doc,x,y);
	  save_value(doc, doc,x,y);
	}
      break;
    case TOOL_SELECT:
      if(gl_stage==2 || gl_stage==4)
	{
	  tool_xor_zoom(doc,x,y);
	  save_value(doc, doc,x,y);
	}
    break;
    }  
  return TRUE;
}

void xor_objects_transform(DOCUMENT *src_doc, DOCUMENT *dest_doc, TRANSFORM *transform)
     /* Draws all selected objects with the transform matrix transform. The objects
	are picked from src_doc but displayed on the dest_doc. The selected objects
	are picked from selection_list. */
{
  GList *select_list;
  OBJECT *object;
  GdkGC *gc;

  if(src_doc==NULL || src_doc->selection_list==NULL)
    return;

  if(dest_doc==NULL)
    dest_doc=src_doc;

  select_list=g_list_first(src_doc->selection_list);

  while(select_list!=NULL)
    {
      object=(OBJECT *)select_list->data;

      gc=misc_setup_gc_object_xor(src_doc, object);     
	  
      switch(object->type)
	{
	case TYPE_LINE:
	  obj_draw_line_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_CLINE:
	  obj_draw_cline_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_RLINE:
	  obj_draw_rline_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_CIRCLE:
	  obj_draw_circle_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_ARC:
	  obj_draw_arc_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_POLYLINE:
	  obj_draw_polyline_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_POINT:
	  obj_draw_point_block(dest_doc,gc,object, transform);	  
	  break;
	case TYPE_SPLINE:
	  obj_draw_spline_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_TEXT:
	  obj_draw_text_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_ATTRIBUTE:
	  obj_draw_attribute_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_ELLIPSE:
	  obj_draw_ellipse_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_ELARC:
	  obj_draw_elarc_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_BLOCK:
	  obj_draw_insert_block(dest_doc,gc,object, transform);
	  break;
	case TYPE_DIMENSION:
	  dim_draw_dimension_block(dest_doc,gc,object, transform);
	  break;
	}	

      select_list=g_list_next(select_list);
    }
}













