/* HPGL
 * Copyright (C) 2000 Jonas Jarvoll
 * 
 * Program to write a HPGL to a serial line.
 * Default is /dev/ttyS1.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* --  I N C L U D E S  -- */
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <termios.h> /* POSIX terminal control definitions */

#define DEFAULT "/dev/ttyS1" 

#define FALSE   (0)
#define TRUE    (!FALSE)

static int tty_fd;
static FILE *file_fd;
static char tty[255];
static char filename[255];

static int init(void)
{
  struct termios options;

  tty_fd=open(tty, O_RDWR);

  tty_fd=open("dev/ttyS1", O_RDWR);
  
  if(tty_fd<0)
    {
      printf("\nError: Cannot open \"%s\"\n",tty);
      return FALSE;
    }
  else
    printf("Ok to open /dev/ttyS1\n");
#ifdef 0
  /* Setup port: 2400, 8 data, None paritet, stop */
  tcgetattr(tty_fd, &options);
  
  /* Set baud rate */ 
  cfsetispeed(&options, B2400);
  cfsetospeed(&options, B2400);

  options.c_cflag &= ~PARENB;
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CRTSCTS;
  options.c_cflag |= CS8;
  options.c_lflag &= ~(ICANON | ISIG | ECHO);
  options.c_iflag |= (IGNPAR | ICRNL);
  options.c_iflag |= (IXON | IXOFF);

  tcsetattr(tty_fd, TCSANOW, &options);

#endif

  file_fd=fopen(filename,"r");

  if(file_fd<0)
    {
      close(tty_fd);
      printf("\nError: Cannot open \"%s\".\n", filename);
      return FALSE;
    }
  else
    printf("OK to open file \"%s\".\n", filename);
  return TRUE;
}

static void allclose()
{
  /* Close file */
  fclose(file_fd);

  /* Close plotter */
  close(tty_fd);
}

int main(int argc, char *argv[])
{
  char read_buffer[255];

  if(argc==2)
    {
      strcpy(tty, DEFAULT);
      strcpy(filename, argv[1]);
    }
  else if(argc==3)
    {
      strcpy(tty,argv[1]);
      strcpy(filename, argv[2]);
    }
  else
    {
      printf("Usage: %s [tty name] filename\n", argv[0]);
      return 0;
    }

  printf("Write a file to a device");

  if(!init())
    return -1;

  while(!feof(file_fd))
    {
      fgets(read_buffer,255,file_fd);
      printf("%s\n", read_buffer);
      write(tty_fd, read_buffer, strlen(read_buffer));
    }

  printf(" Done writing.\nClosing and exiting.\n");

  allclose();

}





